@@ -2,24 +2,9 @@
 
 (use-modules (ice-9 format))
 
-(if (not (defined? 'sound-property)) ; extensions.scm
-    (define sound-property
-      (make-procedure-with-setter
-       
-       (lambda (key snd)
-	 "(sound-property key snd) returns the value associated with 'key' in the given sound's property list, or #f"
-	 (let ((data (assoc key (sound-properties snd))))
-	   (if data
-	       (cdr data)
-	       #f)))
-
-       (lambda (key snd new-val)
-	 (let ((old-val (assoc key (sound-properties snd))))
-	   (if old-val
-	       (set-cdr! old-val new-val)
-	       (set! (sound-properties snd) (cons (cons key new-val) (sound-properties snd))))
-	   new-val)))))
+(provide 'snd-autosave.scm)
 
+(if (not (provided? 'snd-extensions.scm)) (load-from-path "extensions.scm"))
 
 (define auto-save-interval 60.0) ;seconds between auto-save checks
 (define auto-saving #f)
@@ -75,24 +60,24 @@
   (define (auto-save-func)
     (if auto-saving
 	(begin
-	  (map (lambda (snd)
-		 (if (> (unsaved-edits snd) 0)
-		     (begin
-		       (report-in-minibuffer "auto-saving..." snd)
-		       (in (* 1000 3) (lambda () (report-in-minibuffer "" snd)))
-		       (save-sound-as (auto-save-temp-name snd) snd)
-		       (clear-unsaved-edits snd))))
-	       (sounds))
+	  (for-each (lambda (snd)
+		      (if (> (unsaved-edits snd) 0)
+			  (begin
+			    (report-in-minibuffer "auto-saving..." snd)
+			    (in (* 1000 3) (lambda () (report-in-minibuffer "" snd)))
+			    (save-sound-as (auto-save-temp-name snd) snd)
+			    (clear-unsaved-edits snd))))
+		    (sounds))
 	  (in (* 1000 auto-save-interval) auto-save-func))))
   
   (if (not (member auto-save-done (hook->list close-hook)))
       (begin
 	(if (not (null? (sounds)))
-	    (map auto-save-open-func (sounds)))
+	    (for-each auto-save-open-func (sounds)))
 	(add-hook! after-open-hook auto-save-open-func)
 	(add-hook! close-hook auto-save-done)
 	(add-hook! save-hook (lambda (snd name) (auto-save-done snd)))
-	(add-hook! exit-hook (lambda () (map auto-save-done (sounds))))))
+	(add-hook! exit-hook (lambda () (for-each auto-save-done (sounds))))))
   (set! auto-saving #t)
   (in (* 1000 auto-save-interval) auto-save-func))
 
