@@ -1,14 +1,28 @@
 #ifndef CLM_H
 #define CLM_H
 
-#define MUS_VERSION 2
-#define MUS_REVISION 54
-#define MUS_DATE "21-June-04"
+#define MUS_VERSION 3
+#define MUS_REVISION 9
+#define MUS_DATE "20-Dec-04"
 
 /*
+ * 20-Dec:     changed "jitter" handling if hop < .05 in granulate.
+ * 15-Dec:     mus_generator? for type checks (clm2xen).
+ * 14-Dec:     mus_env_linear.
+ * 11-Sep:     removed buffer generator.
+ * 6-Sep:      removed mus_oscil_bank, mus_bank.
+ * 24-Aug:     removed mus_inspect method -- overlaps mus_describe and is useless given gdb capabilities.
+ * 27-July:    mus_granulate_with_editor and mus_phase_vocoder_with_editors.
+ * 21-July:    edit-func as run-time arg to granulate (for CL/clm compatibility)
+ * 19-July:    clm 3.0!
+ *             deprecated mus_ina|b, mus-outa|b|c|d.
+ *             mus_make_frame_to_file_with_comment, mus_mixer_scale, mus_make_frame|mixer_with_data.
+ *             mus_make_scalar_mixer, mus_mixer_add, mus_continue_frame_to_file.
+ *             changed pv_* to phase_vocoder_*
+ * 28-June:    ssb_am + added fm arg (ssb_am_1 is the previous form).
  * 21-June:    wrapper method.
  * 14-June:    ssb_am generator.
- *             removed mus-a*|b*, replaced by mus-x|ycoeff.
+ *             deprecated mus-a*|b*, replaced by mus-x|ycoeff.
  * 9-June:     mus_edot_product.
  * 7-June:     removed mus-x*|y* generic functions.
  * 24-May:     distribution arg to make-rand, make-rand-interp.
@@ -136,17 +150,6 @@
 
 /* isn't mus_env_interp backwards? */
 
-/* compile time switches:
- *
- * HAVE_GSL(default 0)
- *   if 1, use libgsl for some math functions
- * 
- * Float can be defined to be double or float(default)
- *
- * HAVE_FFTW
- *  if 1, include mus_fftw which calls the fftw real fft
- */
-
 #include "sndlib.h"
 
 #if HAVE_COMPLEX_TRIG
@@ -170,7 +173,6 @@ typedef struct mus_any_class {
   char *name;
   int (*release)(mus_any *ptr);
   char* (*describe)(mus_any *ptr);
-  char* (*inspect)(mus_any *ptr);
   bool (*equalp)(mus_any *gen1, mus_any *gen2);
   Float* (*data)(mus_any *ptr);
   Float* (*set_data)(mus_any *ptr, Float *new_data);
@@ -234,6 +236,8 @@ typedef enum {MUS_RECTANGULAR_WINDOW, MUS_HANN_WINDOW, MUS_WELCH_WINDOW, MUS_PAR
   #define MUS_RUN(GEN, ARG_1, ARG_2) ((*(((GEN)->core)->run))(GEN, ARG_1, ARG_2))
 #endif
 #define MUS_RUN_P(GEN) (((GEN)->core)->run)
+#define MUS_MAX_CLM_SINC_WIDTH 65536
+#define MUS_MAX_CLM_SRC 65536.0
 
 #ifdef __cplusplus
 extern "C" {
@@ -273,7 +277,6 @@ Float mus_array_interp(Float *wave, Float phase, int size);
 
 int mus_free(mus_any *ptr);
 char *mus_describe(mus_any *gen);
-char *mus_inspect(mus_any *gen);
 bool mus_equalp(mus_any *g1, mus_any *g2);
 Float mus_phase(mus_any *gen);
 Float mus_set_phase(mus_any *gen, Float val);
@@ -297,7 +300,6 @@ char *mus_file_name(mus_any *ptr);
 Float mus_oscil(mus_any *o, Float fm, Float pm);
 Float mus_oscil_0(mus_any *ptr);
 Float mus_oscil_1(mus_any *ptr, Float fm);
-Float mus_oscil_bank(Float *amps, mus_any **oscils, Float *inputs, int size);
 bool mus_oscil_p(mus_any *ptr);
 mus_any *mus_make_oscil(Float freq, Float phase);
 
@@ -444,15 +446,6 @@ Float mus_wave_train_1(mus_any *gen);
 mus_any *mus_make_wave_train(Float freq, Float phase, Float *wave, int wsize, mus_interp_t type);
 bool mus_wave_train_p(mus_any *gen);
 
-Float mus_buffer_to_sample(mus_any *ptr);
-Float mus_sample_to_buffer(mus_any *ptr, Float val);
-mus_any *mus_make_buffer(Float *preloaded_buffer, int size, Float current_file_time);
-bool mus_buffer_p(mus_any *ptr);
-bool mus_buffer_empty_p(mus_any *ptr);
-bool mus_buffer_full_p(mus_any *ptr);
-mus_any *mus_buffer_to_frame(mus_any *rb, mus_any *fr);
-mus_any *mus_frame_to_buffer(mus_any *rb, mus_any *fr);
-
 mus_any *mus_make_waveshape(Float frequency, Float phase, Float *table, int size);
 Float mus_waveshape(mus_any *ptr, Float index, Float fm);
 Float mus_waveshape_1(mus_any *ptr, Float index);
@@ -461,8 +454,10 @@ Float *mus_partials_to_waveshape(int npartials, Float *partials, int size, Float
 Float *mus_partials_to_polynomial(int npartials, Float *partials, int kind);
 
 Float mus_env(mus_any *ptr);
+Float mus_env_linear(mus_any *ptr);
 mus_any *mus_make_env(Float *brkpts, int pts, Float scaler, Float offset, Float base, Float duration, off_t start, off_t end, Float *odata);
 bool mus_env_p(mus_any *ptr);
+bool mus_env_linear_p(mus_any *ptr);
 void mus_restart_env(mus_any *ptr);
 Float mus_env_interp(Float x, mus_any *env);
 off_t *mus_env_passes(mus_any *gen); /* for Snd */
@@ -476,6 +471,7 @@ int mus_env_breakpoints(mus_any *gen); /* for Snd */
 bool mus_frame_p(mus_any *ptr);
 mus_any *mus_make_empty_frame(int chans);
 mus_any *mus_make_frame(int chans, ...);
+mus_any *mus_make_frame_with_data(int chans, Float *data);
 mus_any *mus_frame_add(mus_any *f1, mus_any *f2, mus_any *res);
 mus_any *mus_frame_multiply(mus_any *f1, mus_any *f2, mus_any *res);
 Float mus_frame_ref(mus_any *f, int chan);
@@ -486,6 +482,7 @@ bool mus_mixer_p(mus_any *ptr);
 mus_any *mus_make_empty_mixer(int chans);
 mus_any *mus_make_identity_mixer(int chans);
 mus_any *mus_make_mixer(int chans, ...);
+mus_any *mus_make_mixer_with_data(int chans, Float *data);
 Float **mus_mixer_data(mus_any *f);
 Float mus_mixer_ref(mus_any *f, int in, int out);
 Float mus_mixer_set(mus_any *f, int in, int out, Float val);
@@ -493,6 +490,9 @@ mus_any *mus_frame_to_frame(mus_any *f, mus_any *in, mus_any *out);
 mus_any *mus_sample_to_frame(mus_any *f, Float in, mus_any *out);
 Float mus_frame_to_sample(mus_any *f, mus_any *in);
 mus_any *mus_mixer_multiply(mus_any *f1, mus_any *f2, mus_any *res);
+mus_any *mus_mixer_add(mus_any *f1, mus_any *f2, mus_any *res);
+mus_any *mus_mixer_scale(mus_any *uf1, Float scaler, mus_any *ures);
+mus_any *mus_make_scalar_mixer(int chans, Float scalar);
 
 bool mus_file_to_sample_p(mus_any *ptr);
 mus_any *mus_make_file_to_sample(const char *filename);
@@ -510,8 +510,6 @@ int mus_channel(mus_any *rd);
 bool mus_output_p(mus_any *ptr);
 bool mus_input_p(mus_any *ptr);
 Float mus_in_any(off_t frame, int chan, mus_any *IO);
-Float mus_ina(off_t frame, mus_any *inp);
-Float mus_inb(off_t frame, mus_any *inp);
 
 mus_any *mus_make_file_to_frame(const char *filename);
 bool mus_file_to_frame_p(mus_any *ptr);
@@ -525,14 +523,11 @@ mus_any *mus_continue_sample_to_file(const char *filename);
 int mus_close_file(mus_any *ptr);
 
 Float mus_out_any(off_t frame, Float val, int chan, mus_any *IO);
-Float mus_outa(off_t frame, Float val, mus_any *IO);
-Float mus_outb(off_t frame, Float val, mus_any *IO);
-Float mus_outc(off_t frame, Float val, mus_any *IO);
-Float mus_outd(off_t frame, Float val, mus_any *IO);
-
 mus_any *mus_make_frame_to_file(const char *filename, int chans, int out_format, int out_type);
 bool mus_frame_to_file_p(mus_any *ptr);
 mus_any *mus_frame_to_file(mus_any *ptr, off_t samp, mus_any *data);
+mus_any *mus_make_frame_to_file_with_comment(const char *filename, int chans, int out_format, int out_type, const char *comment);
+mus_any *mus_continue_frame_to_file(const char *filename);
 
 mus_any *mus_locsig(mus_any *ptr, off_t loc, Float val);
 mus_any *mus_make_locsig(Float degree, Float distance, Float reverb, int chans, mus_any *output, mus_any *revput, mus_interp_t type);
@@ -545,15 +540,15 @@ Float mus_locsig_reverb_set(mus_any *ptr, int chan, Float val);
 void mus_move_locsig(mus_any *ptr, Float degree, Float distance);
 void mus_fill_locsig(Float *arr, int chans, Float degree, Float scaler, mus_interp_t type);
 
-mus_any *mus_make_src(Float(*input)(void *arg, int direction), Float srate, int width, void *closure);
-Float mus_src(mus_any *srptr, Float sr_change, Float(*input)(void *arg, int direction));
+mus_any *mus_make_src(Float (*input)(void *arg, int direction), Float srate, int width, void *closure);
+Float mus_src(mus_any *srptr, Float sr_change, Float (*input)(void *arg, int direction));
 bool mus_src_p(mus_any *ptr);
 Float mus_src_20(mus_any *srptr, Float (*input)(void *arg, int direction));
 Float mus_src_05(mus_any *srptr, Float (*input)(void *arg, int direction));
 
 bool mus_convolve_p(mus_any *ptr);
-Float mus_convolve(mus_any *ptr, Float(*input)(void *arg, int direction));
-mus_any *mus_make_convolve(Float(*input)(void *arg, int direction), Float *filter, int fftsize, int filtersize, void *closure);
+Float mus_convolve(mus_any *ptr, Float (*input)(void *arg, int direction));
+mus_any *mus_make_convolve(Float (*input)(void *arg, int direction), Float *filter, int fftsize, int filtersize, void *closure);
 Float *mus_spectrum(Float *rdat, Float *idat, Float *window, int n, int type);
 void mus_fft(Float *rl, Float *im, int n, int is);
 #if HAVE_FFTW || HAVE_FFTW3
@@ -565,13 +560,15 @@ Float *mus_convolution(Float* rl1, Float* rl2, int n);
 void mus_convolve_files(const char *file1, const char *file2, Float maxamp, const char *output_file);
 
 bool mus_granulate_p(mus_any *ptr);
-Float mus_granulate(mus_any *ptr, Float(*input)(void *arg, int direction));
-mus_any *mus_make_granulate(Float(*input)(void *arg, int direction), 
+Float mus_granulate(mus_any *ptr, Float (*input)(void *arg, int direction));
+Float mus_granulate_with_editor(mus_any *ptr, Float (*input)(void *arg, int direction), int (*edit)(void *closure));
+mus_any *mus_make_granulate(Float (*input)(void *arg, int direction), 
 			    Float expansion, Float length, Float scaler, 
 			    Float hop, Float ramp, Float jitter, int max_size, 
 			    int (*edit)(void *closure),
 			    void *closure);
 int mus_granulate_grain_max_length(mus_any *ptr);
+void mus_granulate_set_edit_function(mus_any *ptr, int (*edit)(void *closure));
 off_t mus_ramp(mus_any *ptr);
 off_t mus_set_ramp(mus_any *ptr, off_t val);
 off_t mus_hop(mus_any *ptr);
@@ -586,33 +583,37 @@ int mus_file_to_float_array(const char *filename, int chan, off_t start, int sam
 int mus_float_array_to_file(const char *filename, Float *ddata, int len, int srate, int channels);
 
 Float mus_apply(mus_any *gen, ...);
-Float mus_bank(mus_any **gens, Float *scalers, Float *arg1, Float *arg2, int size);
 
 bool mus_phase_vocoder_p(mus_any *ptr);
-mus_any *mus_make_phase_vocoder(Float(*input)(void *arg, int direction), 
-				       int fftsize, int overlap, int interp,
-				       Float pitch,
-				       bool(*analyze)(void *arg, Float(*input)(void *arg1, int direction)),
-				       bool(*edit)(void *arg), 
-				       Float(*synthesize)(void *arg), 
-				       void *closure);
-Float mus_phase_vocoder(mus_any *ptr, Float(*input)(void *arg, int direction));
+mus_any *mus_make_phase_vocoder(Float (*input)(void *arg, int direction), 
+				int fftsize, int overlap, int interp,
+				Float pitch,
+				bool (*analyze)(void *arg, Float (*input)(void *arg1, int direction)),
+				int (*edit)(void *arg), /* return value is ignored (int return type is intended to be consistent with granulate) */
+				Float (*synthesize)(void *arg), 
+				void *closure);
+Float mus_phase_vocoder(mus_any *ptr, Float (*input)(void *arg, int direction));
+Float mus_phase_vocoder_with_editors(mus_any *ptr, 
+				     Float (*input)(void *arg, int direction),
+				     bool (*analyze)(void *arg, Float (*input)(void *arg1, int direction)),
+				     int (*edit)(void *arg), 
+				     Float (*synthesize)(void *arg));
 
 Float *mus_phase_vocoder_amp_increments(mus_any *ptr);
 Float *mus_phase_vocoder_amps(mus_any *ptr);
 Float *mus_phase_vocoder_freqs(mus_any *ptr);
 Float *mus_phase_vocoder_phases(mus_any *ptr);
 Float *mus_phase_vocoder_phase_increments(mus_any *ptr);
-Float *mus_phase_vocoder_previous_phases(mus_any *ptr);
 int mus_phase_vocoder_outctr(mus_any *ptr);
 int mus_phase_vocoder_set_outctr(mus_any *ptr, int val);
 
 mus_any *mus_make_ssb_am(Float freq, int order);
 bool mus_ssb_am_p(mus_any *ptr);
-Float mus_ssb_am(mus_any *ptr, Float insig);
+Float mus_ssb_am_1(mus_any *ptr, Float insig);
+Float mus_ssb_am(mus_any *ptr, Float insig, Float fm);
 
 void mus_clear_sinc_tables(void);
-void *mus_environ(mus_any *rd);
+void *mus_environ(mus_any *gen);
 void *mus_wrapper(mus_any *gen);
 
 /* for internal use */
@@ -630,13 +631,9 @@ void *_mus_wrap_one_vct_wrapped(mus_any *ge);
 #define mus_db2linear(X) mus_db_to_linear(X)
 #define mus_linear2db(X) mus_linear_to_db(X)
 #define mus_rectangular2polar(Real, Imag, Size) mus_rectangular_to_polar(Real, Imag, Size)
-#define mus_polar2rectangular(Real, Imag, Size) mus_polar_to_rectangular(Real, IMag, Size)
+#define mus_polar2rectangular(Real, Imag, Size) mus_polar_to_rectangular(Real, Imag, Size)
 #define mus_partials2wave(Data, Partials, Table, Size, Normalize) mus_partials_to_wave(Data, Partials, Table, Size, Normalize)
 #define mus_phasepartials2wave(Data, Partials, Table, Size, Normalize) mus_phase_partials_to_wave(Data, Partials, Table, Size, Normalize)
-#define mus_buffer2sample(Ptr) mus_buffer_to_sample(Ptr)
-#define mus_sample2buffer(Ptr, Val) mus_sample_to_buffer(Ptr, Val)
-#define mus_buffer2frame(Rb, Fr) mus_buffer_to_frame(Rb, Fr)
-#define mus_frame2buffer(Rb, Fr) mus_frame_to_buffer(Rb, Fr)
 #define mus_partials2waveshape(Npartials, Partials, Size, Table) mus_partials_to_waveshape(Npartials, Partials, Size, Table)
 #define mus_partials2polynomial(Npartials, Partials, Kind) mus_partials_to_polynomial(Npartials, Partials, Kind)
 #define mus_frame2frame(Fr, In, Out) mus_frame_to_frame(Fr, In, Out)
@@ -658,7 +655,7 @@ void *_mus_wrap_one_vct_wrapped(mus_any *ge);
 #define mus_frame2file(Ptr, Samp, Data) mus_frame_to_file(Ptr, Samp, Data)
 #define mus_file2fltarray(Filename, Chan, Start, Samples, Array) mus_file_to_float_array(Filename, Chan, Start, Samples, Array)
 #define mus_fltarray2file(Filename, Ddata, Len, Srate, Channels) mus_float_array_to_file(Filename, Ddata, Len, Srate, Channels)
-Float mus_sin(Float phase);
+#define mus_sin(Phase) sin(Phase)
 #define MUS_LINEAR 1
 #define MUS_SINUSOIDAL 2
 
@@ -673,6 +670,16 @@ Float mus_sin(Float phase);
 #define mus_b2(Gen) mus_ycoeff(Gen, 2)
 #define mus_set_b2(Gen, Val) mus_set_ycoeff(Gen, 2, Val)
 
+#define mus_ina(Samp, Inp) mus_in_any(Samp, 0, Inp)
+#define mus_inb(Samp, Inp) mus_in_any(Samp, 1, Inp)
+
+#define mus_outa(Samp, Val, Outp) mus_out_any(Samp, Val, 0, Outp)
+#define mus_outb(Samp, Val, Outp) mus_out_any(Samp, Val, 1, Outp)
+#define mus_outc(Samp, Val, Outp) mus_out_any(Samp, Val, 2, Outp)
+#define mus_outd(Samp, Val, Outp) mus_out_any(Samp, Val, 3, Outp)
+
+#define mus_inspect(Gen) mus_describe(Gen)
+
 #endif
 
 #ifdef __cplusplus
