@@ -69,16 +69,16 @@
 
 ;;; -------- move sound down by n (a power of 2)
 
-(define (down-oct n)
+(define* (down-oct n #:optional snd chn)
   "(down-n n) moves a sound down by power of 2 n"
   ;; I think this is "stretch" in DSP jargon -- to interpolate in the time domain we're squeezing the frequency domain
   ;;  the power-of-2 limitation is based on the underlying fft function's insistence on power-of-2 data sizes
   ;;  see stretch-sound-via-dft below for a general version
-  (let* ((len (frames))
+  (let* ((len (frames snd chn))
 	 (pow2 (ceiling (/ (log len) (log 2))))
 	 (fftlen (inexact->exact (expt 2 pow2)))
 	 (fftscale (/ 1.0 fftlen))
-	 (rl1 (samples->vct 0 fftlen))
+	 (rl1 (channel->vct 0 fftlen snd chn))
 	 (im1 (make-vct fftlen)))
     (fft rl1 im1 1)
     (vct-scale! rl1 fftscale)
@@ -94,14 +94,14 @@
 	(vct-set! im2 i (vct-ref im1 i))
 	(vct-set! im2 j (vct-ref im1 k)))
       (fft rl2 im2 -1)
-      (vct->samples 0 (* n len) rl2))))
+      (vct->channel rl2 0 (* n len) snd chn #f (format #f "down-oct ~A" n)))))
 
-(define (stretch-sound-via-dft factor)
+(define* (stretch-sound-via-dft factor #:optional snd chn)
   ;; this is very slow! factor>1.0
-  (let* ((n (frames))
+  (let* ((n (frames snd chn))
 	 (n2 (inexact->exact (floor (/ n 2.0))))
 	 (out-n (inexact->exact (round (* n factor))))
-	 (in-data (channel->vct))
+	 (in-data (channel->vct 0 n snd chn))
 	 (out-data (make-vct out-n))
 	 (fr (make-vector out-n 0.0))
 	 (freq (/ (* 2 pi) n)))
@@ -116,7 +116,7 @@
 	((or (c-g?) (= i out-n)))
       ;; inverse DFT
       (vct-set! out-data i (real-part (/ (edot-product (* freq 0.0+1.0i i) fr) n))))
-    (vct->channel out-data)))
+    (vct->channel out-data 0 out-n snd chn #f (format #f "stretch-sound-via-dft ~A" factor))))
 
 
 
@@ -201,7 +201,7 @@
 	    (vct-set! data i (+ g2 (* k (- g1 g2))))))
 	(let ((curamp (vct-peak data)))
 	  (vct-scale! data (/ amp curamp)))
-	(vct->samples 0 dur data)))))
+	(vct->channel data 0 dur)))))
 
 (define compute-string
   ;; this is the more general form
@@ -235,66 +235,69 @@
 
 ;;; -------- "frequency division" -- an effect from sed_sed@my-dejanews.com
 
-(define (freqdiv n)
+(define* (freqdiv n #:optional snd chn)
   "(freqdiv n) repeats each nth sample n times (clobbering the intermediate samples): (freqdiv 8)"
   (let ((div 0)
 	(curval 0.0))
-    (map-chan (lambda (val)
-		(if (= div 0)
-		    (set! curval val))
-		(set! div (1+ div))
-		(if (= div n) (set! div 0))
-		curval))))
+    (map-channel (lambda (val)
+		   (if (= div 0)
+		       (set! curval val))
+		   (set! div (1+ div))
+		   (if (= div n) (set! div 0))
+		   curval)
+		 0 #f snd chn #f (format #f "freqdiv ~A" n))))
 
 
 
 ;;; -------- "adaptive saturation" -- an effect from sed_sed@my-dejanews.com
 ;;;
 ;;; a more extreme effect is "saturation":
-;;;   (map-chan (lambda (val) (if (< (abs val) .1) val (if (>= val 0.0) 0.25 -0.25))))
+;;;   (map-channel (lambda (val) (if (< (abs val) .1) val (if (>= val 0.0) 0.25 -0.25))))
 
-(define (adsat size)
+(define* (adsat size #:optional beg dur snd chn)
   "(adsat size) is an 'adaptive saturation' sound effect"
   (let ((mn 0.0)
 	(mx 0.0)
 	(n 0)
 	(vals (make-vct size)))
-    (map-chan (lambda (val)
-		(if (= n size)
-		    (begin
-		      (do ((i 0 (1+ i)))
-			  ((= i size))
-			(if (>= (vct-ref vals i) 0.0)
-			    (vct-set! vals i mx)
-			    (vct-set! vals i mn)))
-		      (set! n 0)
-		      (set! mx 0.0)
-		      (set! mn 0.0)
-		      vals)
-		    (begin
-		      (vct-set! vals n val)
-		      (if (> val mx) (set! mx val))
-		      (if (< val mn) (set! mn val))
-		      (set! n (1+ n))
-		      #f))))))
+    (map-channel (lambda (val)
+		   (if (= n size)
+		       (begin
+			 (do ((i 0 (1+ i)))
+			     ((= i size))
+			   (if (>= (vct-ref vals i) 0.0)
+			       (vct-set! vals i mx)
+			       (vct-set! vals i mn)))
+			 (set! n 0)
+			 (set! mx 0.0)
+			 (set! mn 0.0)
+			 vals)
+		       (begin
+			 (vct-set! vals n val)
+			 (if (> val mx) (set! mx val))
+			 (if (< val mn) (set! mn val))
+			 (set! n (1+ n))
+			 #f)))
+		 beg dur snd chn #f (format #f "adsat ~A ~A ~A" size beg dur))))
 
 
 ;;; -------- spike
 ;;;
 ;;; makes sound more spikey -- sometimes a nice effect
 
-(define (spike)
+(define* (spike #:optional snd chn)
   "(spike) multiplies successive samples together to make a sound more spikey"
-  (map-chan (let ((x1 0.0) 
-		  (x2 0.0) 
-		  (amp (maxamp))) ; keep resultant peak at maxamp
-	      (lambda (x0) 
-		(let ((res (* (/ x0 (* amp amp)) 
-			      (abs x2) 
-			      (abs x1)))) 
-		  (set! x2 x1) 
-		  (set! x1 x0) 
-		  res)))))
+  (map-channel (let ((x1 0.0) 
+		     (x2 0.0) 
+		     (amp (maxamp snd chn))) ; keep resultant peak at maxamp
+		 (lambda (x0) 
+		   (let ((res (* (/ x0 (* amp amp)) 
+				 (abs x2) 
+				 (abs x1)))) 
+		     (set! x2 x1) 
+		     (set! x1 x0) 
+		     res)))
+	       0 #f snd chn #f "spike"))
 
 ;;; the more successive samples we include in the product, the more we
 ;;;   limit the output to pulses placed at (just after) wave peaks
@@ -310,7 +313,7 @@
 	   (chn (if (> (length args) 2) (list-ref args 2) #f))
 	   (pow2 (ceiling (/ (log (/ (srate snd) 20.0)) (log 2))))
 	   (fftlen (inexact->exact (expt 2 pow2)))
-	   (data (autocorrelate (samples->vct s0 fftlen snd chn)))
+	   (data (autocorrelate (channel->vct s0 fftlen snd chn)))
 	   (cor-peak (vct-peak data)))
       (call-with-current-continuation
        (lambda (return)
@@ -382,13 +385,13 @@
 ;;; -------- zero-phase, rotate-phase
 ;;; fft games (from the "phazor" package of Scott McNab)
 
-(define (zero-phase)
+(define* (zero-phase #:optional snd chn)
   "(zero-phase) calls fft, sets all phases to 0, and un-ffts"
-  (let* ((len (frames))
+  (let* ((len (frames snd chn))
 	 (pow2 (ceiling (/ (log len) (log 2))))
 	 (fftlen (inexact->exact (expt 2 pow2)))
 	 (fftscale (/ 1.0 fftlen))
-	 (rl (samples->vct 0 fftlen))
+	 (rl (channel->vct 0 fftlen snd chn))
 	 (old-pk (vct-peak rl))
 	 (im (make-vct fftlen)))
     (fft rl im 1)
@@ -397,16 +400,16 @@
     (vct-scale! im 0.0)
     (fft rl im -1)
     (let ((pk (vct-peak rl)))
-      (vct->samples 0 len (vct-scale! rl (/ old-pk pk))))))
+      (vct->channel (vct-scale! rl (/ old-pk pk)) 0 len snd chn #f "zero-phase"))))
 
-(define (rotate-phase func)
+(define* (rotate-phase func #:optional snd chn)
   "(rotate-phase func) calls fft, applies func to each phase, then un-ffts"
-  (let* ((len (frames))
+  (let* ((len (frames snd chn))
 	 (pow2 (ceiling (/ (log len) (log 2))))
 	 (fftlen (inexact->exact (expt 2 pow2)))
 	 (fftlen2 (inexact->exact (floor (/ fftlen 2))))
 	 (fftscale (/ 1.0 fftlen))
-	 (rl (samples->vct 0 fftlen))
+	 (rl (channel->vct 0 fftlen snd chn))
 	 (old-pk (vct-peak rl))
 	 (im (make-vct fftlen)))
     (fft rl im 1)
@@ -422,7 +425,8 @@
     (polar->rectangular rl im)
     (fft rl im -1)
     (let ((pk (vct-peak rl)))
-      (vct->samples 0 len (vct-scale! rl (/ old-pk pk))))))
+      (vct->channel (vct-scale! rl (/ old-pk pk)) 0 len snd chn #f 
+		    (format #f "rotate-phase ~A" (procedure-source func))))))
 
 ;(rotate-phase (lambda (x) 0.0)) is the same as (zero-phase)
 ;(rotate-phase (lambda (x) (random 3.1415))) randomizes phases
@@ -560,8 +564,10 @@
 
 (define (fejer-sum angle n)
   ;; from "Trigonometric Series" Zygmund p88
-  (let ((val (/ (sin (* 0.5 (+ n 1) angle)) (* 2 (sin (* 0.5 angle))))))
-    (* 2 (/ (* val val) (+ n 1)))))
+  (if (= angle 0.0)
+      1.0
+      (let ((val (/ (sin (* 0.5 (+ n 1) angle)) (* 2 (sin (* 0.5 angle))))))
+	(* 2 (/ (* val val) (+ n 1))))))
 
 (define (legendre-sum angle n)
   ;; from Andrews, Askey, Roy "Special Functions" p 314
@@ -604,15 +610,22 @@
 
 ;;; (Gradshteyn and Ryzhik 1.342)
 
+;;; or take advantage of 1/(1-x):
+;;; (map-channel (lambda (y) (set! i (1+ i)) (/ 0.001 (- 1.0 (* .99 (cos (/ (* i 2.0 3.14159) 100.0)))))))
+;;;   here the .99 controls the number of cosines, like an "index", and it can be matched at
+;;;   run time to keep the amplitude constant via the 0.001 (set above to get a maxamp of .1),
+;;;   and the frequency can be swept without problems.
+
 
 ;;; -------- brighten-slightly
 
-(define (brighten-slightly amount)
+(define* (brighten-slightly amount #:optional snd chn)
   "(brighten-slightly amount) is a form of contrast-enhancement ('amount' between ca .1 and 1)"
   (let* ((mx (maxamp))
 	 (brt (/ (* 2 pi amount) mx)))
     (map-channel (lambda (y)
-		   (* mx (sin (* y brt)))))))
+		   (* mx (sin (* y brt))))
+		 0 #f snd chn #f (format #f "brighten-slightly ~A" amount))))
 
 
 ;;; -------- FIR filters
@@ -639,17 +652,17 @@
 
 (define (fltit-1 order spectr)
   "(fltit-1 order spectrum) creates an FIR filter from spectrum and order and returns a closure that calls it: 
-(map-chan (fltit-1 10 (vct 0 1.0 0 0 0 0 0 0 1.0 0)))"
+(map-channel (fltit-1 10 (vct 0 1.0 0 0 0 0 0 0 1.0 0)))"
   (let* ((flt (make-fir-filter order (spectrum->coeffs order spectr))))
     (lambda (x)
       (fir-filter flt x))))
 
-;(map-chan (fltit-1 10 (vct 0 1.0 0 0 0 0 0 0 1.0 0)))
+;(map-channel (fltit-1 10 (vct 0 1.0 0 0 0 0 0 0 1.0 0)))
 ;
 ;(let ((notched-spectr (make-vct 40)))
 ;  (vct-set! notched-spectr 2 1.0)  
 ;  (vct-set! notched-spectr 37 1.0)
-;  (map-chan (fltit-1 40 notched-spectr)))
+;  (map-channel (fltit-1 40 notched-spectr)))
 ;
 
 ;;; -------- Hilbert transform
@@ -682,22 +695,23 @@
 		   (hilbert-transform h y))))
 
 ;;; this comes from R Lyons:
-(define (sound->amp-env)
+(define* (sound->amp-env #:optional snd chn)
   (let ((hlb (make-hilbert-transform 40))
 	(d (make-delay 40)))
     (map-channel
      (lambda (y)
        (let ((hy (hilbert-transform hlb y))
 	     (dy (delay d y)))
-	 (sqrt (+ (* hy hy) (* dy dy))))))))
+	 (sqrt (+ (* hy hy) (* dy dy)))))
+     0 #f snd chn #f "sound->amp-env")))
 
-(define (hilbert-transform-via-fft)
+(define* (hilbert-transform-via-fft #:optional snd chn)
   ;; same as FIR version but use FFT and change phases by hand
-  (let* ((size (frames))
+  (let* ((size (frames snd chn))
 	 (len (inexact->exact (expt 2 (ceiling (/ (log size) (log 2.0))))))
 	 (rl (make-vct len))
 	 (im (make-vct len))
-	 (rd (make-sample-reader 0)))
+	 (rd (make-sample-reader 0 snd chn)))
     (do ((i 0 (1+ i)))
 	((= i size))
       (vct-set! rl i (rd)))
@@ -715,7 +729,7 @@
 	(vct-set! im i (imag-part c))))
     (mus-fft rl im len -1)
     (vct-scale! rl (/ 1.0 len))
-    (vct->channel rl)))
+    (vct->channel rl 0 len snd chn #f "hilbert-transform-via-fft")))
 !#
 
 ;;; -------- highpass filter 
@@ -1012,19 +1026,19 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 		 (vct 0.0 (* -2.0 gamma) (* 2.0 beta)))))
 
 (define* (make-eliminate-hum #:optional (hum-freq 60.0) (hum-harmonics 5) (bandwidth 10))
-  (let ((gen (make-vector hum-harmonics)))
+  (let ((gen (make-vct hum-harmonics)))
     (do ((i 0 (1+ i)))
 	((= i hum-harmonics))
       (let ((center (* (+ i 1.0) hum-freq))
 	    (b2 (* 0.5 bandwidth)))
-	(vector-set! gen i (make-iir-band-stop-2 (- center b2) (+ center b2)))))
+	(vct-set! gen i (make-iir-band-stop-2 (- center b2) (+ center b2)))))
     gen))
 
 (define (eliminate-hum gen x0)
   (let ((val x0))
     (do ((i 0 (1+ i)))
-	((= i (vector-length gen)))
-      (set! val (filter (vector-ref gen i) val))) ; "cascade" n filters
+	((= i (vct-length gen)))
+      (set! val (filter (vct-ref gen i) val))) ; "cascade" n filters
     val))
 
 ;;; (let ((hummer (make-eliminate-hum))) (map-channel (lambda (x) (eliminate-hum hummer x))))
@@ -1212,13 +1226,15 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
   "(notch-channel freqs #:optional (filter-order #f) beg dur (snd #f) (chn #f) edpos (truncate #t) (notch-width 2)) -> notch filter removing freqs"
   (filter-channel (make-notch-frequency-response (exact->inexact (srate snd)) freqs notch-width)
 		  (or filter-order (inexact->exact (expt 2 (ceiling (/ (log (/ (srate snd) notch-width)) (log 2.0))))))
-		  beg dur snd chn edpos truncate))
+		  beg dur snd chn edpos truncate
+		  (format #f "notch-channel '~A ~A ~A ~A" freqs filter-order beg dur)))
 
 (define* (notch-sound freqs #:optional (filter-order #f) (snd #f) (chn #f) (notch-width 2))
   "(notch-sound freqs #:optional (filter-order #f) (snd #f) (chn #f) (notch-width 2)) -> notch filter removing freqs"
   (filter-sound (make-notch-frequency-response (exact->inexact (srate snd)) freqs notch-width)
 		(or filter-order (inexact->exact (expt 2 (ceiling (/ (log (/ (srate snd) notch-width)) (log 2.0))))))
-		snd chn))
+		snd chn #f
+		(format #f "notch-channel '~A ~A 0 #f" freqs filter-order)))
 
 (define* (notch-selection freqs #:optional (filter-order #f) (notch-width 2))
   "(notch-selection freqs #:optional (filter-order #f) (notch-width 2)) -> notch filter removing freqs"
@@ -1371,10 +1387,10 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
       (set! e (cons (* scl (/ a (expt y (+ a 1.0)))) e)))
     (reverse e)))
 
-;(map-chan (lambda (y) (any-random 1.0 '(0 1 1 1)))) ; uniform distribution
-;(map-chan (lambda (y) (any-random 1.0 '(0 0 0.95 0.1 1 1)))) ; mostly toward 1.0
-;(let ((g (gaussian-distribution 1.0))) (map-chan (lambda (y) (any-random 1.0 g))))
-;(let ((g (pareto-distribution 1.0))) (map-chan (lambda (y) (any-random 1.0 g))))
+;(map-channel (lambda (y) (any-random 1.0 '(0 1 1 1)))) ; uniform distribution
+;(map-channel (lambda (y) (any-random 1.0 '(0 0 0.95 0.1 1 1)))) ; mostly toward 1.0
+;(let ((g (gaussian-distribution 1.0))) (map-channel (lambda (y) (any-random 1.0 g))))
+;(let ((g (pareto-distribution 1.0))) (map-channel (lambda (y) (any-random 1.0 g))))
 
 ;;; this is the inverse integration function used by CLM to turn a distribution function into a weighting function
 
@@ -1416,6 +1432,7 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 ;;; (make-rand :envelope (gaussian-envelope 1.0))
 
 
+
 ;;; ---------------- Julius Smith stuff ----------------
 ;;;
 ;;; these are from "Mathematics of the DFT", W3K Pubs
@@ -1483,8 +1500,19 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 	(norm1 (channel-norm s1 c1)))
     (/ inprod (* norm1 norm1))))
 
-;;; the projection is now (scale-by coeff 0 (frames) s1 c1)
-;;; end of JOS stuff
+;;; -------- end of JOS stuff --------
+
+
+(define (channel-distance s1 c1 s2 c2)
+  (let* ((r1 (make-sample-reader 0 s1 c1))
+	 (r2 (make-sample-reader 0 s2 c2))
+	 (sum 0.0)
+	 (N (min (frames s1 c1) (frames s2 c2))))
+    (do ((i 0 (1+ i)))
+	((= i N))
+      (let ((diff (- (r1) (r2))))
+	(set! sum (+ sum (* diff diff)))))
+    (sqrt sum)))
 
 
 (define (periodogram N)
@@ -1512,14 +1540,17 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 
 ;;; -------- ssb-am friends
 
-(define* (map-ssb-am freq #:optional (order 40)) ; higher order = better cancellation
-  ;; TODO: a better name -- perhaps channel-shift-spectrum? shift-channel-spectrum?
+(define* (shift-channel-pitch freq #:optional (order 40) (beg 0) dur snd chn edpos)
+  ;; higher order = better cancellation
   (let* ((gen (make-ssb-am freq order)))
-    (map-channel (lambda (y) (ssb-am gen y)))))
+    (map-channel (lambda (y) 
+		   (ssb-am gen y)) 
+		 beg dur snd chn edpos 
+		 (format #f "shift-channel-pitch ~A ~A ~A ~A" freq order beg dur))))
 
 (define (hz->2pi freq) (/ (* 2 pi freq) (srate))) ; hz->radians follows mus-srate unfortunately
 
-(define* (ssb-bank old-freq new-freq pairs-1 #:optional (order 40) (bw 50.0))
+(define* (ssb-bank old-freq new-freq pairs-1 #:optional (order 40) (bw 50.0) (beg 0) dur snd chn edpos)
   (let* ((pairs pairs-1) ; for run's benefit
 	 (ssbs (make-vector pairs))
 	 (bands (make-vector pairs))
@@ -1545,15 +1576,51 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 					 (bandpass (vector-ref bands i) 
 						   y)))))
 	      (set! nmx (max nmx (abs sum)))
-	      sum)))
-	 (scale-by (/ mx nmx)))))))
+	      sum))
+	  beg dur snd chn edpos)
+	 (scale-channel (/ mx nmx) beg dur snd chn edpos)))
+     (format #f "ssb-bank ~A ~A ~A ~A ~A ~A ~A" old-freq new-freq pairs-1 order bw beg dur))))
 
+(define* (ssb-bank-env old-freq new-freq freq-env pairs-1 #:optional (order 40) (bw 50.0) (beg 0) dur snd chn edpos)
+  ;; this version adds a frequency envelope
+  ;; (ssb-bank-env 557 880 '(0 0 1 100.0) 7)
+  (let* ((pairs pairs-1) ; for run's benefit
+	 (ssbs (make-vector pairs))
+	 (bands (make-vector pairs))
+	 (factor (/ (- new-freq old-freq) old-freq))
+	 (frenvs (make-vector pairs))
+	 (mx (maxamp)))
+    (do ((i 1 (1+ i)))
+	((> i pairs))
+      (let* ((aff (* i old-freq))
+	     (bwf (* bw (+ 1.0 (/ i (* 2 pairs))))))
+	(vector-set! ssbs (1- i) (make-ssb-am (* i factor old-freq)))
+	(vector-set! bands (1- i) (make-bandpass (hz->2pi (- aff bwf)) 
+						 (hz->2pi (+ aff bwf)) 
+						 order))
+	(vector-set! frenvs (1- i) (make-env freq-env 
+					     :scaler (hz->radians (exact->inexact i)) 
+					     :end (1- (frames))))))
+    (as-one-edit
+     (lambda ()
+       (let ((nmx 0.0))
+	 (map-channel
+	  (lambda (y)
+	    (let ((sum 0.0))
+	      (do ((i 0 (1+ i)))
+		  ((= i pairs))
+		(set! sum (+ sum (ssb-am (vector-ref ssbs i) 
+					 (bandpass (vector-ref bands i) 
+						   y)
+					 (env (vector-ref frenvs i))))))
+	      (set! nmx (max nmx (abs sum)))
+	      sum))
+	  beg dur snd chn edpos)
+	 (scale-channel (/ mx nmx) beg dur snd chn edpos)))
+     (format #f "ssb-bank-env ~A ~A '~A ~A ~A ~A ~A ~A" old-freq new-freq freq-env pairs-1 order bw beg dur))))
 
 ;;; TODO: auto-detect main freq so ssb-bank can work semi-automatically (bw/pairs choices also automated)
-;;; TODO: freq env to add (or remove) pitch fluctuations [if pitch follower, this could be automated]
-;;; TODO: hz->radians should be smart (or someone should) about srates
-;;; TODO: a channel (regularized) version of ssb-bank -- repitch-channel? (+ retime or whatever)
-;;; TODO: a realtime interface to this -- a slider for pitch/bw etc
+;;; TODO: if pitch follower, auto-remove gliss/vib (ssb-bank-env could be written to use oscil or triangle wave = add vib)
 
 #!
 (define* (repitch-sound old-freq new-freq)
@@ -1601,3 +1668,74 @@ can be used directly: (filter-sound (make-butter-low-pass 500.0)), or via the 'b
 (define (fm2 gen index)
   (* .25 (+ (oscil (list-ref gen 0) (* index (oscil (list-ref gen 1))))
 	    (oscil (list-ref gen 2) (* index (oscil (list-ref gen 3)))))))
+
+
+#!
+;;; a "bump function" (Stein and Shakarchi)
+(define (bumpy)
+  (let* ((x 0.0) 
+	 (xi (/ 1.0 (frames)))
+	 (start 0)
+	 (end 1)
+	 (scl (exp (/ 4.0 (- end start))))) ; normalize it
+    (map-channel (lambda (y) 
+		   (let ((val (if (and (>= x start)
+				       (<= x end))
+				  (* (exp (/ -1.0 (- x start))) 
+				     (exp (/ -1.0 (- end x))))
+				  0.0)))
+		     (set! x (+ x xi))
+		     (* scl val))))))
+!#
+
+
+;;; vct|channel|spectral-polynomial
+
+(define (vct-polynomial v coeffs)
+  ;; Horner's rule applied to entire vct
+  (let* ((v-len (vct-length v))
+	 (num-coeffs (vct-length coeffs))
+	 (new-v (make-vct v-len (vct-ref coeffs (1- num-coeffs)))))
+    (do ((i (- num-coeffs 2) (1- i)))
+	((< i 0))
+      (vct-offset! (vct-multiply! new-v v) (vct-ref coeffs i)))
+    new-v))
+
+(define* (channel-polynomial coeffs #:optional snd chn)
+  (let ((len (frames snd chn)))
+    (vct->channel (vct-polynomial (channel->vct 0 len snd chn) coeffs) 0 len snd chn #f (format #f "channel-polynomial ~A" (vct->string coeffs)))))
+
+;;; (channel-polynomial (vct 0.0 .5)) = x*.5
+;;; (channel-polynomial (vct 0.0 1.0 1.0 1.0)) = x*x*x + x*x + x
+
+;;; convolution -> * in freq
+
+(define* (spectral-polynomial coeffs #:optional snd chn)
+  (let* ((len (frames snd chn))
+	 (sound (channel->vct 0 len snd chn))
+	 (num-coeffs (vct-length coeffs))
+	 (fft-len (if (< num-coeffs 2) 
+		      len 
+		      (inexact->exact (expt 2 (ceiling (/ (log (* (1- num-coeffs) len)) (log 2)))))))
+	 (rl1 (make-vct fft-len 0.0))
+	 (rl2 (make-vct fft-len 0.0))
+	 (new-sound (make-vct fft-len)))
+    (if (> (vct-ref coeffs 0) 0.0)
+	(let ((dither (vct-ref coeffs 0)))
+	  (do ((i 0 (1+ i)))
+	      ((= i fft-len))
+	    (vct-set! new-sound i (mus-random dither)))))
+    (if (> num-coeffs 1)
+	(begin
+	  (vct-add! new-sound (vct-scale! (vct-copy sound) (vct-ref coeffs 1)))
+	  (if (> num-coeffs 2)
+	      (let ((peak (maxamp snd chn)))
+		(vct-add! (vct-scale! rl1 0.0) sound)
+		(do ((i 2 (1+ i)))
+		    ((= i num-coeffs))
+		  (convolution rl1 (vct-add! (vct-scale! rl2 0.0) sound) fft-len)
+		  (let ((pk (vct-peak rl1)))
+		    (vct-add! new-sound (vct-scale! (vct-copy rl1) (/ (* (vct-ref coeffs i) peak) pk)))))
+		(let ((pk (vct-peak new-sound)))
+		  (vct-scale! new-sound (/ peak pk)))))))
+    (vct->channel new-sound 0 (max len (* len (1- num-coeffs))) snd chn #f (format #f "spectral-polynomial ~A" (vct->string coeffs)))))
