@@ -25,13 +25,10 @@
 
 ;;; -------- selection -> new file
 
-(define selctr 0)
-
 (define (selection->new)
   "(selection-<new) saves the selection in a new file, then opens that file"
   (if (selection?)
-      (let ((new-file-name (format #f "sel-~D.snd" selctr)))
-	(set! selctr (+ selctr 1))
+      (let ((new-file-name (snd-tempnam)))
 	(save-selection new-file-name)
 	(open-sound new-file-name))
       #f))
@@ -44,8 +41,7 @@
 (define (cut-selection->new)
   "(cut-selection->new) saves the selection, deletes it, then opens the saved file"
   (if (selection?)
-      (let ((new-file-name (format #f "sel-~D.snd" selctr)))
-	(set! selctr (+ selctr 1))
+      (let ((new-file-name (snd-tempnam)))
 	(save-selection new-file-name)
 	(delete-selection)
 	(open-sound new-file-name))
@@ -54,11 +50,7 @@
 (add-to-menu edit-menu "Cut selection->new" cut-selection->new 9)
 
 
-;;; -------- append sound (and append selection for lafs)
-
-(define (append-sound name)
-  "(append-sound name) appends file 'name'"
-  (insert-sound name (frames)))
+;;; -------- append selection
 
 (define (append-selection)
   "(append-selection) appends the current selection"
@@ -67,6 +59,7 @@
 
 (add-to-menu edit-menu "Append selection" append-selection 10)
 
+
 ;;; -------- make-stereofile
 (define (make-stereofile)
   (let* ((ofile-name (file-name))
@@ -130,18 +123,19 @@
 
 ;;; -------- crop (trims front and back)
 
+(define* (crop-one-channel #:optional snd chn)
+  (if (< (length (marks snd chn)) 2)
+      (report-in-minibuffer "crop needs start and end marks" snd)
+      (as-one-edit
+       (lambda ()
+	 (delete-samples 0 (mark-sample (car (marks snd chn))) snd chn)
+	 (let ((endpt (mark-sample (car (reverse (marks snd chn))))))
+	   (delete-samples (+ endpt 1) (- (frames snd chn) endpt))))
+       "crop-one-channel")))
+
 (define (crop)
   "(crop) finds the first and last marks in each of the syncd channels and removes all samples outside them"
   (let ((snc (sync)))
-    (define (crop-one-channel snd chn)
-      (if (< (length (marks snd chn)) 2)
-	  (report-in-minibuffer "crop needs start and end marks" snd)
-	  (as-one-edit
-	   (lambda ()
-	     (delete-samples 0 (mark-sample (car (marks snd chn))) snd chn)
-	     (let ((endpt (mark-sample (car (reverse (marks snd chn))))))
-	       (delete-samples (+ endpt 1) (- (frames snd chn) endpt))))
-	   "crop")))
     (if (> snc 0)
 	(apply map
 	       (lambda (snd chn)
