@@ -286,7 +286,6 @@
 
 
 (define (change-prop winat name command)
-  ;; implements change-window-property via xm
   (define (find-window dpy top natom)
     (let ((res (XGetWindowProperty dpy top natom 0 1024 #f XA_STRING)))
       (if (and (= (car res) Success) 
@@ -309,7 +308,9 @@
     (if (Window? window)
 	(begin
 	  (XChangeProperty dpy window (XInternAtom dpy name #f) XA_STRING 8 PropModeReplace command)
-	  (XFlush dpy)))))
+	  (XFlush dpy)
+	  command)
+	#f)))
 
 (define beep-state
   (make-procedure-with-setter
