@@ -49,7 +49,6 @@
 ;;;    C-x b support along the same lines 
 ;;; remove-clicks
 ;;; searching examples (zero+, next-peak, find-pitch)
-;;; sound-data->list
 ;;; file->vct and a sort of cue-list, I think, and region-play-list, region-play-sequence
 ;;; replace-with-selection
 ;;; explode-sf2 -- turn soundfont file into a bunch of files of the form sample-name.aif
@@ -133,7 +132,7 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
 (define* (region-rms #:optional (n 0))
   "(region-rms #:optional (n 0)) -> rms of region n's data (chan 0)"
   (if (region? n)
-      (let* ((data (region-samples->vct 0 0 n)))
+      (let* ((data (region->vct 0 0 n)))
 	(sqrt (/ (dot-product data data) (vct-length data))))
       (throw 'no-such-region (list "region-rms" n))))
 
@@ -142,20 +141,22 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
   "(window-samples #:optional snd chn) -> samples in snd channel chn in current graph window"
   (let ((wl (left-sample snd chn))
 	(wr (right-sample snd chn)))
-    (samples wl (+ 1 (- wr wl)) snd chn)))
+    (channel->vct wl (+ 1 (- wr wl)) snd chn)))
 
 (define (display-energy snd chn)
   ;; in this version, the y-zoom-slider controls the graph amp
   "(display-energy snd chn) is a lisp-graph-hook function to display the time domain data as energy (squared)"
-  (let* ((ls (left-sample))
-	 (rs (right-sample))
+  (let* ((ls (left-sample snd chn))
+	 (rs (right-sample snd chn))
 	 (datal (make-graph-data snd chn))
 	 (data (if (vct? datal) datal (cadr datal)))
 	 (len (vct-length data))
 	 (sr (srate snd))
 	 (y-max (y-zoom-slider snd chn)))
-    (vct-multiply! data data)
-    (graph data "energy" (/ ls sr) (/ rs sr) 0.0 (* y-max y-max) snd chn #f)))
+    (if (and data ls rs)
+	(begin
+	  (vct-multiply! data data)
+	  (graph data "energy" (/ ls sr) (/ rs sr) 0.0 (* y-max y-max) snd chn #f)))))
 
 ;(add-hook! lisp-graph-hook display-energy)
 
@@ -185,7 +186,7 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
   "(window-rms) -> rms of data in currently selected graph window"
   (let* ((ls (left-sample))
 	 (rs (right-sample))
-	 (data (samples->vct ls (+ 1 (- rs ls))))
+	 (data (channel->vct ls (+ 1 (- rs ls))))
 	 (len (vct-length data)))
     (sqrt (/ (dot-product data data) len))))
 
@@ -195,7 +196,7 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
   (if (and (transform-graph?) 
 	   (= (transform-graph-type) graph-once))
       (report-in-minibuffer 
-       (number->string (/ (* 2.0 (vct-peak (transform-samples->vct snd chn))) 
+       (number->string (/ (* 2.0 (vct-peak (transform->vct snd chn))) 
 			  (transform-size)))
        snd)
       #f))
@@ -233,8 +234,8 @@ two sounds open (indices 0 and 1 for example), and the second has two channels,
 	     (fftlen (inexact->exact (expt 2 pow2)))
 	     (fftlen2 (/ fftlen 2))
 	     (fftscale (/ 1.0 fftlen))
-	     (rl1 (samples->vct ls fftlen snd 0))
-	     (rl2 (samples->vct ls fftlen snd 1))
+	     (rl1 (channel->vct ls fftlen snd 0))
+	     (rl2 (channel->vct ls fftlen snd 1))
 	     (im1 (make-vct fftlen))
 	     (im2 (make-vct fftlen)))
 	(fft rl1 im1 1)
@@ -401,7 +402,7 @@ this can be confusing if fft normalization is on (the default)"
 		 (lambda (n)
 		   (if (and (= (sync n) (sync snd))
 			    (> (chans n) chn))
-		       (set! ffts (append ffts (let* ((fdr (samples->vct ls fftlen n chn))
+		       (set! ffts (append ffts (let* ((fdr (channel->vct ls fftlen n chn))
 						      (fdi (make-vct fftlen))
 						      (spectr (make-vct (/ fftlen 2))))
 						 (list (vct-add! spectr (spectrum fdr fdi #f 2))))))))
@@ -430,6 +431,32 @@ this can be confusing if fft normalization is on (the default)"
 	   n)))))
 
 
+;;; -------- do?
+;;; a version of "do" that is interruptible and continuable
+;;;   to continue an interrupted "do?", (do-go-on)
+
+(define do-go-on-continuation #f)
+
+(define (do-go-on)
+  (if (continuation? do-go-on-continuation)
+      (do-go-on-continuation #f)
+      ";sorry! can't continue"))
+
+(defmacro do? (inits ends . body)
+  `(do ,inits
+       ((or (and ,(car ends)
+		 (begin 
+		   (set! do-go-on-continuation #f)
+		   #t))
+	    (and (c-g?)
+		 (call-with-current-continuation
+		  (lambda (go-on)
+		    (set! do-go-on-continuation go-on)))))
+	,(and (not (null? (cdr ends)))
+	      (cadr ends)))
+     ,@body))
+
+
 ;;; -------- make a system call from the listener
 ;;;
 ;;;   (shell "df") for example -- there's probably a more elegant way to do this is in Scheme
@@ -697,7 +724,7 @@ this can be confusing if fft normalization is on (the default)"
   "(do-all-chans func edhist) applies func to all active channels, using edhist as the edit history 
 indication: (do-all-chans (lambda (val) (* 2.0 val)) \"double all samples\")"
   (apply map (lambda (snd chn)
-	       (map-chan func #f #f origin snd chn))
+	       (map-channel func 0 #f snd chn #f origin))
 	 (all-chans)))
 
 (define (update-graphs)
@@ -713,7 +740,7 @@ indication: (do-all-chans (lambda (val) (* 2.0 val)) \"double all samples\")"
 	(apply map
 	       (lambda (snd chn)
 		 (if (= (sync snd) snc)
-		     (map-chan func #f #f origin snd chn)))
+		     (map-channel func 0 #f snd chn #f origin)))
 	       (all-chans))
 	(snd-warning "sync not set"))))
 
@@ -724,13 +751,13 @@ indication: (do-all-chans (lambda (val) (* 2.0 val)) \"double all samples\")"
 	(begin
 	  (do ((chn 0 (1+ chn)))
 	      ((= chn (channels snd)) #f)
-	    (map-chan proc #f #f origin snd chn)))
+	    (map-channel proc 0 #f snd chn #f origin)))
 	(snd-warning "no selected sound"))))
 
 (define (every-sample? proc)
   "(every-sample func) -> #t if func is not #f for all samples in the current channel, 
 otherwise it moves the cursor to the first offending sample"
-  (let ((baddy (scan-chan 
+  (let ((baddy (scan-channel 
 		(lambda (y) 
 		  (not (proc y))))))
     (if baddy (set! (cursor) (cadr baddy)))
@@ -739,7 +766,7 @@ otherwise it moves the cursor to the first offending sample"
 (define (sort-samples nbins)
   "(sort-samples bins) provides a histogram in 'bins' bins"
   (let ((bins (make-vector nbins 0)))
-    (scan-chan
+    (scan-channel
      (lambda (y)
        (let ((bin (inexact->exact (floor (* (abs y) nbins)))))
 	 (vector-set! bins bin (+ (vector-ref bins bin) 1))
@@ -781,16 +808,16 @@ a number, the sound is split such that 0 is all in channel 0 and 90 is all in ch
 ;;; -------- FFT-based editing
 ;;;
 
-(define (fft-edit bottom top)
+(define* (fft-edit bottom top #:optional snd chn)
   "(fft-edit low-Hz high-Hz) ffts an entire sound, removes all energy below low-Hz and all above high-Hz, 
 then inverse ffts."
-  (let* ((sr (srate))
-	 (len (frames))
+  (let* ((sr (srate snd))
+	 (len (frames snd chn))
 	 (fsize (expt 2 (ceiling (/ (log len) (log 2.0)))))
-	 (rdata (samples->vct 0 fsize))
+	 (rdata (channel->vct 0 fsize snd chn))
 	 (idata (make-vct fsize))
-	 (lo (round (/ bottom (/ sr fsize))))
-	 (hi (round (/ top (/ sr fsize)))))
+	 (lo (inexact->exact (round (/ bottom (/ sr fsize)))))
+	 (hi (inexact->exact (round (/ top (/ sr fsize))))))
     (fft rdata idata 1)
     (do ((i 0 (1+ i))
 	 (j (- fsize 1) (1- j)))
@@ -808,14 +835,14 @@ then inverse ffts."
       (vct-set! idata j 0.0))
     (fft rdata idata -1)
     (vct-scale! rdata (/ 1.0 fsize))
-    (vct->channel rdata 0 (1- len))))
+    (vct->channel rdata 0 (1- len) snd chn #f (format #f "fft-edit ~A ~A" bottom top))))
 
-(define (fft-squelch squelch)
+(define* (fft-squelch squelch #:optional snd chn)
   "(fft-squelch squelch) ffts an entire sound, sets all bins to 0.0 whose energy is below squelch, then inverse ffts"
-  (let* ((sr (srate))
-	 (len (frames))
+  (let* ((sr (srate snd))
+	 (len (frames snd chn))
 	 (fsize (expt 2 (ceiling (/ (log len) (log 2.0)))))
-	 (rdata (samples->vct 0 fsize))
+	 (rdata (channel->vct 0 fsize snd chn))
 	 (idata (make-vct fsize))
 	 (fsize2 (/ fsize 2))
 	 (scaler 1.0))
@@ -824,28 +851,28 @@ then inverse ffts."
 	  (vi (vct-copy idata)))
       (rectangular->polar vr vi)
       (set! scaler (vct-peak vr)))
-    (set! squelch (* squelch scaler))
-    (do ((i 0 (1+ i))
-	 (j (- fsize 1) (1- j)))
-	((= i fsize2))
-      (let ((magnitude (sqrt (+ (* (vct-ref rdata i) (vct-ref rdata i)) (* (vct-ref idata i) (vct-ref idata i))))))
-	(if (< magnitude squelch)
-	    (begin
-	      (vct-set! rdata i 0.0)
-	      (vct-set! rdata j 0.0)
-	      (vct-set! idata i 0.0)
-	      (vct-set! idata j 0.0)))))
-    (fft rdata idata -1)
-    (vct-scale! rdata (/ 1.0 fsize))
-    (vct->channel rdata 0 (1- len))
+    (let ((scl-squelch (* squelch scaler)))
+      (do ((i 0 (1+ i))
+	   (j (- fsize 1) (1- j)))
+	  ((= i fsize2))
+	(let ((magnitude (sqrt (+ (* (vct-ref rdata i) (vct-ref rdata i)) (* (vct-ref idata i) (vct-ref idata i))))))
+	  (if (< magnitude scl-squelch)
+	      (begin
+		(vct-set! rdata i 0.0)
+		(vct-set! rdata j 0.0)
+		(vct-set! idata i 0.0)
+		(vct-set! idata j 0.0)))))
+      (fft rdata idata -1)
+      (vct-scale! rdata (/ 1.0 fsize)))
+    (vct->channel rdata 0 (1- len) snd chn #f (format #f "fft-squelch ~A" squelch))
     scaler))
 
-(define (fft-cancel lo-freq hi-freq)
+(define* (fft-cancel lo-freq hi-freq #:optional snd chn)
   "(fft-cancel lo-freq hi-freq) ffts an entire sound, sets the bin(s) representing lo-freq to hi-freq to 0.0, then inverse ffts"
-  (let* ((sr (srate))
-	 (len (frames))
+  (let* ((sr (srate snd))
+	 (len (frames snd chn))
 	 (fsize (expt 2 (ceiling (/ (log len) (log 2.0)))))
-	 (rdata (samples->vct 0 fsize))
+	 (rdata (channel->vct 0 fsize snd chn))
 	 (idata (make-vct fsize))
 	 (fsize2 (/ fsize 2))
 	 (scaler 1.0))
@@ -862,7 +889,7 @@ then inverse ffts."
 	(vct-set! idata j 0.0)))
     (fft rdata idata -1)
     (vct-scale! rdata (/ 1.0 fsize))
-    (vct->channel rdata 0 (1- len))))
+    (vct->channel rdata 0 (1- len) snd chn #f (format #f "fft-cancel ~A ~A" lo-freq hi-freq))))
     
 
 ;;; same idea but used to distinguish vowels (steady-state) from consonants
@@ -885,7 +912,7 @@ then inverse ffts."
   "(make-ramp #:optional (size 128)) returns a ramp generator"
   (list 0 size))
 
-(define (squelch-vowels)
+(define* (squelch-vowels #:optional snd chn)
   "(squelch-vowels) suppresses portions of a sound that look like steady-state"
   (let* ((fft-size 32)
 	 (fft-mid (inexact->exact (floor (/ fft-size 2))))
@@ -893,7 +920,7 @@ then inverse ffts."
 	 (im (make-vct fft-size))
 	 (ramper (make-ramp 256)) ; 512 ok too
 	 (peak (/ (maxamp) fft-mid))
-	 (read-ahead (make-sample-reader))
+	 (read-ahead (make-sample-reader 0 snd chn))
 	 (ctr 0)
 	 (in-vowel #f))
     (do ((i 0 (1+ i)))
@@ -919,15 +946,15 @@ then inverse ffts."
 		     ;(and (> rval 0.0) ; if this is included, the vowel-portions are omitted
 		     (* y rval) ; squelch vowels 
 		     ;(* y (+ (* 2 rval) .1)) ;accentuate consonants
-		     )))))
+		     ))
+		 0 #f snd chn #f "squelch-vowels")))
 
-
-(define (fft-env-data fft-env)
+(define* (fft-env-data fft-env #:optional snd chn)
   "(fft-env-data fft-env) applies fft-env as spectral env to current sound, returning vct of new data"
-  (let* ((sr (srate))
-	 (len (frames))
+  (let* ((sr (srate snd))
+	 (len (frames snd chn))
 	 (fsize (expt 2 (ceiling (/ (log len) (log 2.0)))))
-	 (rdata (samples->vct 0 fsize))
+	 (rdata (channel->vct 0 fsize snd chn))
 	 (idata (make-vct fsize))
 	 (fsize2 (/ fsize 2))
 	 (e (make-env fft-env :end (1- fsize2))))
@@ -943,16 +970,16 @@ then inverse ffts."
     (fft rdata idata -1)
     (vct-scale! rdata (/ 1.0 fsize))))
 
-(define (fft-env-edit fft-env)
+(define* (fft-env-edit fft-env #:optional snd chn)
   "(fft-env-edit fft-env) edits (filters) current chan using fft-env"
-  (vct->channel (fft-env-data fft-env) 0 (1- (frames))))
+  (vct->channel (fft-env-data fft-env snd chn) 0 (1- (frames)) snd chn #f (format "fft-env-edit '~A" fft-env)))
 
-(define (fft-env-interp env1 env2 interp)
+(define* (fft-env-interp env1 env2 interp #:optional snd chn)
   "(fft-env-interp env1 env2 interp) interpolates between two fft-filtered versions (env1 and env2 are the 
 spectral envelopes) following interp (an env between 0 and 1)"
-  (let* ((data1 (fft-env-data env1))
-	 (data2 (fft-env-data env2))
-	 (len (frames))
+  (let* ((data1 (fft-env-data env1 snd chn))
+	 (data2 (fft-env-data env2 snd chn))
+	 (len (frames snd chn))
 	 (new-data (make-vct len))
 	 (e (make-env interp :end (1- len))))
     (do ((i 0 (1+ i)))
@@ -961,16 +988,15 @@ spectral envelopes) following interp (an env between 0 and 1)"
 	(vct-set! new-data i 
 		  (+ (* (- 1.0 pan) (vct-ref data1 i))
 		     (* pan (vct-ref data2 i))))))
-    (vct->channel new-data 0 (1- len))))
+    (vct->channel new-data 0 (1- len) snd chn #f (format #f "fft-env-interp '~A '~A '~A" env1 env2 interp))))
 
-(define (fft-smoother cutoff start samps snd chn)
+(define* (fft-smoother cutoff start samps #:optional snd chn)
   "(fft-smoother cutoff start samps snd chn) uses fft-filtering to smooth a 
-section: (vct->samples (cursor) 400 (fft-smoother .1 (cursor) 400 0 0))"
+section: (vct->channel (fft-smoother .1 (cursor) 400 0 0) (cursor) 400)"
   (let* ((fftpts (inexact->exact (expt 2 (ceiling (/ (log (1+ samps)) (log 2.0))))))
-	 (rl (make-vct fftpts))
+	 (rl (channel->vct start samps snd chn rl))
 	 (im (make-vct fftpts))
 	 (top (inexact->exact (floor (* fftpts cutoff)))))
-    (samples->vct start samps snd chn rl)
     (let* ((old0 (vct-ref rl 0))
 	   (old1 (vct-ref rl (1- samps)))
 	   (oldmax (vct-peak rl)))
@@ -1003,12 +1029,12 @@ section: (vct->samples (cursor) 400 (fft-smoother .1 (cursor) 400 0 0))"
 ;;; -------- comb-filter
 
 (define (comb-filter-1 scaler size)
-  "(comb-filter-1 scaler size) returns a comb-filter ready for map-chan etc: (map-chan (comb-filter-1 .8 32))"
-  (let ((delay-line (make-vector size 0.0))
+  "(comb-filter-1 scaler size) returns a comb-filter ready for map-channel etc: (map-channel (comb-filter-1 .8 32))"
+  (let ((delay-line (make-vct size 0.0))
 	(delay-loc 0))
     (lambda (x)
-      (let ((result (vector-ref delay-line delay-loc)))
-	(vector-set! delay-line delay-loc (+ x (* scaler result)))
+      (let ((result (vct-ref delay-line delay-loc)))
+	(vct-set! delay-line delay-loc (+ x (* scaler result)))
 	(set! delay-loc (1+ delay-loc))
 	(if (= delay-loc size) (set! delay-loc 0))
 	result))))
@@ -1016,7 +1042,7 @@ section: (vct->samples (cursor) 400 (fft-smoother .1 (cursor) 400 0 0))"
 ;;; the same thing using the CLM module is:
 
 (define (comb-filter scaler size)
-  "(comb-filter scaler size) returns a comb-filter ready for map-chan etc: (map-chan (comb-filter .8 32)).  If you're 
+  "(comb-filter scaler size) returns a comb-filter ready for map-channel etc: (map-channel (comb-filter .8 32)).  If you're 
 in a hurry use: (clm-channel (make-comb .8 32)) instead"
   (let ((cmb (make-comb scaler size)))
     (lambda (x) 
@@ -1025,10 +1051,10 @@ in a hurry use: (clm-channel (make-comb .8 32)) instead"
 ;;; by using filters at harmonically related sizes, we can get chords:
 
 (define (comb-chord scaler size amp)
-  "(comb-chord scaler size amp) returns a set of harmonically-related comb filters: (map-chan (comb-chord .95 100 .3))"
-  (let ((c1 (make-comb scaler size))
-	(c2 (make-comb scaler (* size .75)))
-	(c3 (make-comb scaler (* size 1.2))))
+  "(comb-chord scaler size amp) returns a set of harmonically-related comb filters: (map-channel (comb-chord .95 100 .3))"
+  (let ((c1 (make-comb scaler (inexact->exact size)))
+	(c2 (make-comb scaler (inexact->exact (* size .75))))
+	(c3 (make-comb scaler (inexact->exact (* size 1.2)))))
     (lambda (x) 
       (* amp (+ (comb c1 x) (comb c2 x) (comb c3 x))))))
 
@@ -1036,25 +1062,25 @@ in a hurry use: (clm-channel (make-comb .8 32)) instead"
 
 (define (zcomb scaler size pm)
   "(zcomb scaler size pm) returns a comb filter whose length varies according to an 
-envelope: (map-chan (zcomb .8 32 '(0 0 1 10)))"
+envelope: (map-channel (zcomb .8 32 '(0 0 1 10)))"
   (define (max-envelope-1 e mx)
     (if (null? e)
 	mx
 	(max-envelope-1 (cddr e) (max mx (abs (cadr e))))))
 
-  (let ((cmb (make-comb scaler size :max-size (+ size 1 (max-envelope-1 pm 0.0))))
+  (let ((cmb (make-comb scaler size :max-size (inexact->exact (+ size 1 (max-envelope-1 pm 0.0)))))
 	(penv (make-env :envelope pm :end (frames))))
     (lambda (x)
       (comb cmb x (env penv)))))
 
 (define (notch-filter scaler size)
-  "(notch-filter scaler size) returns a notch-filter: (map-chan (notch-filter .8 32))"
+  "(notch-filter scaler size) returns a notch-filter: (map-channel (notch-filter .8 32))"
   (let ((cmb (make-notch scaler size)))
     (lambda (x) 
       (notch cmb x))))
 
 (define (formant-filter radius frequency)
-  "(formant-filter radius frequency) returns a formant generator: (map-chan (formant-filter .99 2400)). Faster 
+  "(formant-filter radius frequency) returns a formant generator: (map-channel (formant-filter .99 2400)). Faster 
 is: (filter-sound (make-formant .99 2400))"
   (let ((frm (make-formant radius frequency)))
     (lambda (x) 
@@ -1063,7 +1089,7 @@ is: (filter-sound (make-formant .99 2400))"
 ;;; to impose several formants, just add them in parallel:
 
 (define (formants r1 f1 r2 f2 r3 f3)
-  "(formants r1 f1 r2 f2 r3 f3) returns 3 formant filters in parallel: (map-chan (formants .99 900 .98 1800 .99 2700))"
+  "(formants r1 f1 r2 f2 r3 f3) returns 3 formant filters in parallel: (map-channel (formants .99 900 .98 1800 .99 2700))"
   (let ((fr1 (make-formant r1 f1))
 	(fr2 (make-formant r2 f2))
 	(fr3 (make-formant r3 f3)))
@@ -1073,7 +1099,7 @@ is: (filter-sound (make-formant .99 2400))"
 	 (formant fr3 x)))))
 
 (define (moving-formant radius move)
-  "(moving-formant radius move) returns a time-varying (in frequency) formant filter: (map-chan (moving-formant .99 '(0 1200 1 2400)))"
+  "(moving-formant radius move) returns a time-varying (in frequency) formant filter: (map-channel (moving-formant .99 '(0 1200 1 2400)))"
   (let ((frm (make-formant radius (cadr move)))
 	(menv (make-env :envelope move :end (frames))))
     (lambda (x)
@@ -1083,7 +1109,7 @@ is: (filter-sound (make-formant .99 2400))"
 
 (define (osc-formants radius bases amounts freqs)
   "(osc-formants radius bases amounts freqs) set up any number of independently oscillating 
-formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))"
+formants: (map-channel (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))"
   (let* ((len (length bases))
 	 (frms (make-vector len))
 	 (oscs (make-vector len)))
@@ -1107,16 +1133,16 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
 ;;; -------- echo
 
 (define (echo scaler secs)
-  "(echo scaler secs) returns an echo maker: (map-chan (echo .5 .5) 0 44100)"
-  (let ((del (make-delay (round (* secs (srate))))))
+  "(echo scaler secs) returns an echo maker: (map-channel (echo .5 .5) 0 44100)"
+  (let ((del (make-delay (inexact->exact (round (* secs (srate)))))))
     (lambda (inval)
       (+ inval (delay del (* scaler (+ (tap del) inval)))))))
 
 (define (zecho scaler secs frq amp)
-  "(zecho scaler secs freq amp) returns a modulated echo maker: (map-chan (zecho .5 .75 6 10.0) 0 65000)"
+  "(zecho scaler secs freq amp) returns a modulated echo maker: (map-channel (zecho .5 .75 6 10.0) 0 65000)"
   (let* ((os (make-oscil frq))
-	 (len (round (* secs (srate))))
-	 (del (make-delay len :max-size (+ len amp 1))))
+	 (len (inexact->exact (round (* secs (srate)))))
+	 (del (make-delay len :max-size (inexact->exact (+ len amp 1)))))
     (lambda (inval)
       (+ inval 
 	 (delay del 
@@ -1124,9 +1150,9 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
 		(* amp (oscil os)))))))
 
 (define (flecho scaler secs)
-  "(flecho scaler secs) returns a low-pass filtered echo maker: (map-chan (flecho .5 .9) 0 75000)"
+  "(flecho scaler secs) returns a low-pass filtered echo maker: (map-channel (flecho .5 .9) 0 75000)"
   (let* ((flt (make-fir-filter :order 4 :xcoeffs (vct .125 .25 .25 .125)))
-	 (del (make-delay  (round (* secs (srate))))))
+	 (del (make-delay  (inexact->exact (round (* secs (srate)))))))
     (lambda (inval)
       (+ inval 
 	 (delay del 
@@ -1138,7 +1164,7 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
 ;;; CLM instrument is ring-modulate.ins
 
 (define (ring-mod freq gliss-env)
-  "(ring-mod freq gliss-env) returns a time-varying ring-modulation filter: (map-chan (ring-mod 10 (list 0 0 1 (hz->radians 100))))"
+  "(ring-mod freq gliss-env) returns a time-varying ring-modulation filter: (map-channel (ring-mod 10 (list 0 0 1 (hz->radians 100))))"
   (let* ((os (make-oscil :frequency freq))
 	 (len (frames))
 	 (genv (make-env :envelope gliss-env :end len)))
@@ -1146,7 +1172,7 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
       (* (oscil os (env genv)) inval))))
 
 (define (am freq)
-  "(am freq)returns an amplitude-modulator: (map-chan (am 440))"
+  "(am freq)returns an amplitude-modulator: (map-channel (am 440))"
   (let ((os (make-oscil freq))) 
     (lambda (inval) 
       (amplitude-modulate 1.0 inval (oscil os)))))
@@ -1164,13 +1190,13 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
 ;;;
 ;;; CLM instrument version is in clm.html
 
-(define (hello-dentist frq amp)
+(define* (hello-dentist frq amp #:optional snd chn)
   "(hello-dentist frq amp) varies the sampling rate randomly, making a voice sound quavery: (hello-dentist 40.0 .1)"
   (let* ((rn (make-rand-interp :frequency frq :amplitude amp))
 	 (i 0)
 	 (j 0)
 	 (len (frames))
-	 (in-data (samples->vct 0 len))
+	 (in-data (channel->vct 0 len snd chn))
 	 (out-len (inexact->exact (round (* len (+ 1.0 (* 2 amp))))))
 	 (out-data (make-vct out-len))
 	 (rd (make-src :srate 1.0 
@@ -1183,33 +1209,35 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
     (vct->channel
      (vct-map! out-data
 	       (lambda ()
-		 (src rd (rand-interp rn)))))))
+		 (src rd (rand-interp rn))))
+     0 len snd chn #f (format #f "hello-dentist ~A ~A" frq amp))))
 
 ;;; a very similar function uses oscil instead of rand-interp, giving
 ;;; various "Forbidden Planet" sound effects:
 
-(define (fp sr osamp osfrq)
+(define* (fp sr osamp osfrq #:optional snd chn)
   "(fp sr osamp osfrq) varies the sampling rate via an oscil: (fp 1.0 .3 20)"
   (let* ((os (make-oscil osfrq))
 	 (s (make-src :srate sr))
-	 (len (frames))
-	 (sf (make-sample-reader))
-	 (out-data (make-vct len)))
+	 (len (frames snd chn))
+	 (sf (make-sample-reader 0 snd chn))
+	 (out-data (make-vct len))
+	 (amp osamp))
     (vct-map! out-data
 	      (lambda () 
-		(src s (* osamp (oscil os))
+		(src s (* amp (oscil os))
 		     (lambda (dir)
 		       (if (> dir 0)
 			   (next-sample sf)
 			   (previous-sample sf))))))
     (free-sample-reader sf)
-    (vct->samples 0 len out-data)))
+    (vct->channel out-data 0 len snd chn #f (format #f "fp ~A ~A ~A" sr osamp osfrq))))
 	    
 
 ;;; -------- compand, compand-channel
 
 (define (compand)
-  "(compand) returns a compander: (map-chan (compand))"
+  "(compand) returns a compander: (map-channel (compand))"
   (let ((tbl (vct -1.000 -0.960 -0.900 -0.820 -0.720 -0.600 -0.450 -0.250 
 		  0.000 0.250 0.450 0.600 0.720 0.820 0.900 0.960 1.000)))
     ;; (we're eye-balling the curve on p55 of Steiglitz's "a DSP Primer")
@@ -1228,7 +1256,8 @@ formants: (map-chan (osc-formants .99 '(400 800 1200) '(400 800 1200) '(4 2 3)))
   (ptree-channel (lambda (inval)
 		   (let ((index (+ 8.0 (* 8.0 inval))))
 		     (array-interp compand-table index 17)))
-		 beg dur snd chn edpos #t))
+		 beg dur snd chn edpos #t #f
+		 (format #f "compand-channel ~A ~A" beg dur)))
 
 
 
@@ -1246,7 +1275,7 @@ to produce a sound at a new pitch but at the original tempo.  It returns a funct
 	 (sr (make-src :srate rate))
 	 (vsize 1024)
 	 (vbeg 0)
-	 (v (samples->vct 0 vsize))
+	 (v (channel->vct 0 vsize))
 	 (inctr 0))
     (lambda (inval)
       (src sr 0.0
@@ -1259,39 +1288,30 @@ to produce a sound at a new pitch but at the original tempo.  It returns a funct
 				(begin
 				  (set! vbeg (+ vbeg inctr))
 				  (set! inctr 0)
-				  (samples->vct vbeg vsize snd chn v)))
+				  (set! v (channel->vct vbeg vsize snd chn))))
 			    val))))))))
 
 ;;; the next (expsnd) changes the tempo according to an envelope; the new duration
 ;;; will depend on the expansion envelope -- we integrate it to get
 ;;; the overall expansion, then use that to decide the new length.
 
-(define (expsnd gr-env)
+(define* (expsnd gr-env #:optional snd chn)
   "(expsnd gr-env) uses the granulate generator to change tempo according to an envelope: (expsnd '(0 .5 2 2.0))"
-
-  (define (integrate-envelope e sum)
-    (if (or (null? e) (null? (cddr e)))
-	sum
-	(integrate-envelope (cddr e) (+ sum (* (+ (cadr e) (cadddr e)) .5 (- (caddr e) (car e)))))))
-  
-  (define (max-x e)
-    (if (null? (cddr e))
-	(car e)
-	(max-x (cddr e))))
-
-  (let* ((dur (/ (* (/ (frames) (srate)) (integrate-envelope gr-env 0.0)) (max-x gr-env)))
+  (let* ((dur (/ (* (/ (frames snd chn) (srate snd)) 
+		    (integrate-envelope gr-env)) ; in env.scm
+		 (envelope-last-x gr-env)))
 	 (gr (make-granulate :expansion (cadr gr-env) :jitter 0))
 	 (ge (make-env :envelope gr-env :duration dur))
-	 (sound-len (round (* (srate) dur)))
-	 (len (max sound-len (frames)))
+	 (sound-len (inexact->exact (round (* (srate snd) dur))))
+	 (len (max sound-len (frames snd chn)))
 	 (out-data (make-vct len))
-	 (sf (make-sample-reader)))
+	 (sf (make-sample-reader 0 snd chn)))
     (vct-map! out-data (lambda ()
 			 (let ((val (granulate gr (lambda (dir) (next-sample sf)))))
 			   (set! (mus-increment gr) (env ge))
 			   val)))
     (free-sample-reader sf)
-    (vct->samples 0 len out-data)))
+    (vct->channel out-data 0 len snd chn #f (format #f "expsnd '~A" gr-env))))
 
 
 ;;; -------- cross-synthesis
@@ -1300,7 +1320,7 @@ to produce a sound at a new pitch but at the original tempo.  It returns a funct
 
 (define (cross-synthesis cross-snd amp fftsize r)
   "(cross-synthesis cross-snd amp fftsize r) does cross-synthesis between 'cross-snd' (a sound index) and the currently 
-selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
+selected sound: (map-channel (cross-synthesis 1 .5 128 6.0))"
   (let* ((freq-inc (/ fftsize 2))
 	 (fdr (make-vct fftsize))
 	 (fdi (make-vct fftsize))
@@ -1317,7 +1337,7 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
       (let ((outval 0.0))
 	(if (= ctr freq-inc)
 	    (begin
-	      (samples->vct inctr fftsize cross-snd 0 fdr)
+	      (set! fdr (channel->vct inctr fftsize cross-snd 0))
 	      (set! inctr (+ inctr freq-inc))
 	      (spectrum fdr fdi #f 2)
 	      (vct-subtract! fdr spectr)
@@ -1329,18 +1349,18 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 
 ;;; similar ideas can be used for spectral cross-fades, etc -- for example:
 
-(define (voiced->unvoiced amp fftsize r tempo)
+(define* (voiced->unvoiced amp fftsize r tempo #:optional snd chn)
   "(voiced->unvoiced amp fftsize r tempo) turns a vocal sound into whispering: (voiced->unvoiced 1.0 256 2.0 2.0)"
   (let* ((freq-inc (/ fftsize 2))
 	 (fdr (make-vct fftsize))
 	 (fdi (make-vct fftsize))
 	 (spectr (make-vct freq-inc))
-	 (noi (make-rand (/ (srate) 3)))
+	 (noi (make-rand (/ (srate snd) 3)))
 	 (inctr 0)
 	 (ctr freq-inc)
 	 (radius (- 1.0 (/ r fftsize)))
-	 (bin (/ (srate) fftsize))
-	 (len (frames))
+	 (bin (/ (srate snd) fftsize))
+	 (len (frames snd chn))
 	 (outlen (inexact->exact (floor (/ len tempo))))
 	 (hop (inexact->exact (floor (* freq-inc tempo))))
 	 (out-data (make-vct (max len outlen)))
@@ -1352,14 +1372,14 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
       (vector-set! formants i (make-formant radius (* i bin))))
     (call-with-current-continuation ; setup non-local exit (for C-g interrupt)
      (lambda (break)                ;   now (break value) will exit the call/cc returning value
-       (do ((k 0 (1+ k)))           ; this is our actual loop 
+       (do ((k 0 (1+ k)))
 	   ((= k outlen))
 	 (let ((outval 0.0))
 	   (if (= ctr freq-inc)
 	       (begin
 		 (if (c-g?)               ; let interface run
 		     (break "interrupted")) ;   if C-g exit the loop returning the string "interrupted"
-		 (samples->vct inctr fftsize (selected-sound) 0 fdr)
+		 (set! fdr (channel->vct inctr fftsize snd chn))
 		 (let ((pk (vct-peak fdr)))
 		   (if (> pk old-peak-amp) (set! old-peak-amp pk)))
 		 (spectrum fdr fdi #f 2)
@@ -1373,10 +1393,8 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 	   (if (> (abs outval) new-peak-amp) (set! new-peak-amp (abs outval)))
 	   (vct-set! out-data k outval)))
        (vct-scale! out-data (* amp (/ old-peak-amp new-peak-amp)))
-       (vct->samples 0 (max len outlen) out-data)
-       (play-and-wait)))))
-
-
+       (vct->channel out-data 0 (max len outlen) snd chn #f 
+		     (format #f "voiced->unvoiced ~A ~A ~A ~A" amp fftsize r tempo))))))
 
 
 ;;; -------- convolution example
@@ -1385,14 +1403,14 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
   "(cnvtest snd0 snd1 amp) convolves snd0 and snd1, scaling by amp, returns new max amp: (cnvtest 0 1 .1)"
   (let* ((flt-len (frames snd0))
 	 (total-len (+ flt-len (frames snd1)))
-	 (cnv (make-convolve :filter (samples->vct 0 flt-len snd0)))
+	 (cnv (make-convolve :filter (channel->vct 0 flt-len snd0)))
 	 (sf (make-sample-reader 0 snd1))
 	 (out-data (make-vct total-len)))
     (vct-map! out-data (lambda () (convolve cnv (lambda (dir) (next-sample sf)))))
     (free-sample-reader sf)
     (vct-scale! out-data amp)
     (let ((max-samp (vct-peak out-data)))
-      (vct->samples 0 total-len out-data snd1)
+      (vct->channel out-data 0 total-len snd1)
       (if (> max-samp 1.0) (set! (y-bounds snd1) (list (- max-samp) max-samp)))
       max-samp)))
 
@@ -1400,7 +1418,7 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 ;;; -------- time varying FIR filter
 
 (define (fltit)
-  "(fltit) returns a time-varying filter: (map-chan (fltit))"
+  "(fltit) returns a time-varying filter: (map-channel (fltit))"
   (let* ((coeffs (list .1 .2 .3 .4 .4 .3 .2 .1))
 	 (flt (make-fir-filter 8 (list->vct coeffs)))
 	 (es (make-vector 8)))
@@ -1422,7 +1440,7 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 ;
 ;;; this is a strong notch filter centered at 550 Hz
 ;
-;(map-chan (lambda (x) (two-zero flt x)))
+;(map-channel (lambda (x) (two-zero flt x)))
 ;
 ;;; similarly make-ppolar/two-pole (or better, make-formant)
 ;;; can be used for resonances.
@@ -1471,7 +1489,7 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 	   (chn (if (> (length rest) 1) (cadr rest) #f))
 	   (bufsize 2048)
 	   (buf4size 128)
-	   (data (samples->vct start bufsize snd chn))
+	   (data (channel->vct start bufsize snd chn))
 	   (curbeg start)
 	   (curend (+ start bufsize)))
       (lambda (loc)
@@ -1480,13 +1498,13 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 	      ;; get previous buffer
 	      (set! curbeg (+ buf4size (- loc bufsize)))
 	      (set! curend (+ curbeg bufsize))
-	      (samples->vct curbeg bufsize snd chn data))
+	      (set! data (channel->vct curbeg bufsize snd chn)))
 	    (if (> loc curend)
 		(begin
 		  ;; get next buffer
 		  (set! curbeg (- loc buf4size))
 		  (set! curend (+ curbeg bufsize))
-		  (samples->vct curbeg bufsize snd chn data))))
+		  (set! data (channel->vct curbeg bufsize snd chn)))))
 	(array-interp data (- loc curbeg) bufsize)))))
 
 (define sound-interp ;make it look like a clm generator
@@ -1501,8 +1519,8 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
     (let ((osc (make-oscil :frequency freq :initial-phase (+ pi (/ pi 2))))
 	  (reader (make-sound-interp 0 0 0)) 
 	  (len (1- (frames 0 0))))
-      (map-chan (lambda (val) 
-		  (sound-interp reader (* len (+ 0.5 (* 0.5 (oscil osc))))))))))
+      (map-channel (lambda (val) 
+		     (sound-interp reader (* len (+ 0.5 (* 0.5 (oscil osc))))))))))
 
 ;;; (test-interp 0.5)
 !#
@@ -1517,37 +1535,34 @@ selected sound: (map-chan (cross-synthesis 1 .5 128 6.0))"
 ;;   envelope could be used for this effect, but it is much more direct to apply the
 ;;   envelope to sound sample positions.
 
-(define env-sound-interp
-  (lambda (envelope . rest)
-    "(env-sound-interp env #:optional (time-scale 1.0) snd chn) reads snd's channel chn according to env and time-scale"
-    ;; since the old/new sounds can be any length, we'll write a temp file rather than trying to use map-chan or vct-map!
-    (let* ((time-scale (if (not (null? rest)) (car rest) 1.0))
-	   (snd (if (> (length rest) 1) (cadr rest) #f))
-	   (chn (if (> (length rest) 2) (caddr rest) #f))
-	   (len (frames snd chn))
-	   (newlen (inexact->exact (floor (* time-scale len))))
-	   (reader (make-sound-interp 0 snd chn))
-	   (read-env (make-env envelope :end newlen :scaler len))
-	   (tempfilename (snd-tempnam))
-	   (fil (mus-sound-open-output tempfilename (srate snd) 1 #f mus-next "env-sound-interp temp file"))
-	   ;; #f as data-format -> format compatible with sndlib (so no data translation is needed)
-	   (bufsize 8192)
-	   (data (make-sound-data 1 bufsize))
-	   (data-ctr 0))
-      (do ((i 0 (1+ i)))
-	  ((= i newlen))
-	(sound-data-set! data 0 data-ctr (reader (env read-env)))
-	(set! data-ctr (1+ data-ctr))
-	(if (= bufsize data-ctr)
-	    (begin
-	      (mus-sound-write fil 0 (1- bufsize) 1 data)
-	      (set! data-ctr 0))))
-      (if (> data-ctr 0)
-	  (mus-sound-write fil 0 (1- data-ctr) 1 data))
-      (mus-sound-close-output fil (* 4 newlen))
-      ;; #t trunc arg to set samples shortens the sound as needed
-      (set-samples 0 newlen tempfilename snd chn #t)
-      (delete-file tempfilename))))
+(define* (env-sound-interp envelope #:optional (time-scale 1.0) snd chn)
+  "(env-sound-interp env (time-scale 1.0) snd chn) reads snd's channel chn according to env and time-scale"
+  ;; since the old/new sounds can be any length, we'll write a temp file rather than trying to use map-channel or vct-map!
+  (let* ((len (frames snd chn))
+	 (newlen (inexact->exact (floor (* time-scale len))))
+	 (reader (make-sound-interp 0 snd chn))
+	 (read-env (make-env envelope :end newlen :scaler len))
+	 (tempfilename (snd-tempnam))
+	 (fil (mus-sound-open-output tempfilename (srate snd) 1 #f mus-next "env-sound-interp temp file"))
+	 ;; #f as data-format -> format compatible with sndlib (so no data translation is needed)
+	 (bufsize 8192)
+	 (data (make-sound-data 1 bufsize))
+	 (data-ctr 0))
+    (do ((i 0 (1+ i)))
+	((= i newlen))
+      (sound-data-set! data 0 data-ctr (reader (env read-env)))
+      (set! data-ctr (1+ data-ctr))
+      (if (= bufsize data-ctr)
+	  (begin
+	    (mus-sound-write fil 0 (1- bufsize) 1 data)
+	    (set! data-ctr 0))))
+    (if (> data-ctr 0)
+	(mus-sound-write fil 0 (1- data-ctr) 1 data))
+    (mus-sound-close-output fil (* 4 newlen))
+    ;; #t trunc arg to set samples shortens the sound as needed
+    (set-samples 0 newlen tempfilename snd chn #t
+		 (format #f "env-sound-interp '~A ~A" envelope time-scale))
+    (delete-file tempfilename)))
 
 
 
@@ -1600,18 +1615,19 @@ this clock, set retitle-time to 0"
 
 ;;; -------- filtered-env 
 
-(define (filtered-env e)
+(define* (filtered-env e #:optional snd chn)
   "(filtered-env env) is a time-varying one-pole filter: when env is at 1.0, no filtering, 
 as env moves to 0.0, low-pass gets more intense; amplitude and low-pass amount move together"
   (let* ((samps (frames))
 	 (flt (make-one-pole 1.0 0.0))
 	 (amp-env (make-env e :end (1- samps))))
-    (map-chan
+    (map-channel
      (lambda (val)
        (let ((env-val (env amp-env)))
 	 (set! (mus-xcoeff flt 0) env-val)
 	 (set! (mus-ycoeff flt 1) (- env-val 1.0))
-	 (one-pole flt (* env-val val)))))))
+	 (one-pole flt (* env-val val))))
+     0 #f snd chn #f (format #f "filtered-env '~A" e))))
 
 
 ;;; -------- multi-colored rxvt printout
@@ -1927,24 +1943,6 @@ In most cases, this will be slightly offset from the true beginning of the note"
 	 rtn))))
 
 
-;;; -------- sound-data->list
-
-(define (sound-data->list sd)
-  "(sound-data->list sd chan) turns a sound-data object's data into a list of lists (one for each channel)"
-
-  (define (sound-data-channel->list sd chan)
-    (let ((ls '()))
-      (do ((i (1- (sound-data-length sd)) (1- i)))
-	  ((< i 0) ls)
-	(set! ls (cons (sound-data-ref sd chan i) ls)))))
-
-  (let ((lst '()))
-    (do ((i (1- (sound-data-chans sd)) (1- i)))
-	((< i 0) lst)
-      (set! lst (cons (sound-data-channel->list sd i) lst)))))
-
-
-
 ;;; -------- file->vct and a sort of cue-list, I think
 
 (define (file->vct file)
@@ -1958,12 +1956,10 @@ In most cases, this will be slightly offset from the true beginning of the note"
     (free-sample-reader reader)
     data))
 
-(define (add-notes notes)
+(define* (add-notes notes #:optional snd chn)
   "(add-notes notes) adds (mixes) 'notes' which is a list of lists of the form: file #:optional (offset 0.0) (amp 1.0) 
-starting at the cursor in the currently selected channel: (add-notes '(("oboe.snd") ("pistol.snd" 1.0 2.0)))"
-  (let* ((snd (or (selected-sound) (car (sounds))))
-	 (chn (or (selected-channel) 0))
-	 (start (cursor snd chn)))
+starting at the cursor in the currently selected channel: (add-notes '((\"oboe.snd\") (\"pistol.snd\" 1.0 2.0)))"
+  (let* ((start (cursor snd chn)))
     (as-one-edit
      (lambda ()
        (for-each 
@@ -1976,7 +1972,8 @@ starting at the cursor in the currently selected channel: (add-notes '(("oboe.sn
 		     (not (= amp 1.0)))
 		(mix-vct (vct-scale! (file->vct file) amp) beg snd chn #f "add-notes")
 		(mix file beg 0 snd chn #f))))
-	notes)))))
+	notes))
+     (format #f "add-notes '~S" notes))))
 
 (define (region-play-list data)
   "(region-play-list data): 'data' is list of lists (list (list time reg)...), time in secs, setting up 
@@ -2244,7 +2241,8 @@ a sort of play list: (region-play-list (list (list 0.0 0) (list 0.5 1) (list 1.0
        (let ((incr (/ pi frag-dur)))
 	 (vct-set! data 1 incr)
 	 (vct-set! data 0 (* frag-beg incr))
-	 data)))))
+	 data))
+     (format #f "smooth-channel-via-ptree ~A ~A" beg dur))))
 
 
 ;;; -------- ring-modulate-channel (ring-mod as virtual op)
@@ -2263,7 +2261,8 @@ a sort of play list: (region-play-list (list (list 0.0 0) (list 0.5 1) (list 1.0
    beg dur snd chn edpos #f
    (lambda (frag-beg frag-dur)
      (let ((incr (/ (* 2 pi freq) (srate))))
-       (vct (fmod (* frag-beg incr) (* 2 pi)) incr)))))
+       (vct (fmod (* frag-beg incr) (* 2 pi)) incr)))
+   (format #f "ring-modulate-channel ~A ~A ~A" freq beg dur)))
 
 ;;; amplitude-modulate-channel could be (lambda (y data forward) (* y 0.5 (+ 1.0 (sin angle))) etc ...)
 
