@@ -1,7 +1,7 @@
 ;;; translated from popup.scm
 ;;;
 ;;; no attempt here to set widget colors, no listener popup menu (gtk predefines
-;;;   a popup for each entry/testview widget and there's no way to get rid of it!
+;;;   a popup for each entry/textview widget and there's no way to get rid of it!
 ;;;   perhaps we could find a pointer to its children and hide all of them?)
 
 (use-modules (ice-9 format))
@@ -40,12 +40,15 @@
 			 (gtk_menu_item_new_with_label (car entry))
 			 (gtk_menu_item_new)))) ; separator
 	 (gtk_menu_shell_append (GTK_MENU_SHELL menu) widget)
-	 (if (cadr entry) ((cadr entry) widget))
+	 (if (and (cadr entry) 
+		  (procedure? (cadr entry)))
+	     ((cadr entry) widget))
 	 (gtk_widget_show widget)
-	 (if (not (null? (cddr entry)))
+	 (if (and (not (null? (cddr entry)))
+		  (procedure? (caddr entry)))
 	     (g_signal_connect widget "activate" (lambda (w data) 
 						   ((caddr entry) w data) 
-						   (gtk_widget_hide menu))))))
+						   (gtk_widget_hide menu)) #f))))
      entries)
     menu))
 
@@ -65,7 +68,7 @@
 		 #f)
 	       (list
 		;; name field-func callback
-		(list "Selection" (lambda x #f) (lambda x #f))
+		(list "Selection" #f)
 		(list #f #f) ; separator
 		(list "Play" every-menu
 		      (lambda (w data)
@@ -82,9 +85,9 @@
 			      (change-label w "Stop")
 			      (set! stop-widget w)
 			      (set! stopping #t)
-			      (play-selection #f -1 (lambda (reason)
-						      (set! stopping #f)
-						      (change-label w "Play")))))))
+			      (play-selection #f (lambda (reason)
+						   (set! stopping #f)
+						   (change-label w "Play")))))))
 		(list "Loop play" every-menu
 		      (lambda (w data) 
 			(define (stop-playing-selection)
@@ -98,7 +101,7 @@
 			  (if (and (not (c-g?))
 				   (= reason 0)
 				   stopping1)
-			      (play-selection #f -1 play-selection-again)
+			      (play-selection #f play-selection-again)
 			      (stop-playing-selection)))
 			(if stopping1
 			    (begin
@@ -108,7 +111,7 @@
 			      (change-label w "Stop!")
 			      (set! stop-widget1 w) ; needs to be separate from Play case since we're stopping/restarting deliberately
 			      (set! stopping1 #t)
-			      (play-selection #f -1 play-selection-again)))))
+			      (play-selection #f play-selection-again)))))
 		(list "Delete"    every-menu (lambda (w data) (delete-selection)))
 		(list "Zero"      every-menu (lambda (w data) (scale-selection-by 0.0)))
 		(list "Crop"      every-menu
@@ -136,20 +139,16 @@
 			   sndlist))))
 		(list "Save as"   every-menu (lambda (w data) (save-selection-dialog)))
 		(list "Copy->New" every-menu 
-		      (let ((selctr 0)) 
-			(lambda (w data) 
-			  (let ((new-file-name (format #f "newf-~D.snd" selctr)))
-			    (set! selctr (+ selctr 1))
-			    (save-selection new-file-name)
-			    (open-sound new-file-name)))))
+		      (lambda (w data) 
+			(let ((new-file-name (snd-tempnam)))
+			  (save-selection new-file-name)
+			  (open-sound new-file-name))))
 		(list "Cut->New"   every-menu 
-		      (let ((selctr 0)) 
-			(lambda (w data) 
-			  (let ((new-file-name (format #f "newf-~D.snd" selctr)))
-			    (set! selctr (+ selctr 1))
-			    (save-selection new-file-name)
-			    (delete-selection)
-			    (open-sound new-file-name)))))
+		      (lambda (w data) 
+			(let ((new-file-name (snd-tempnam)))
+			  (save-selection new-file-name)
+			  (delete-selection)
+			  (open-sound new-file-name))))
 		(list "Snap marks" every-menu 
 		      (lambda (w data)
 			(for-each 
@@ -159,10 +158,12 @@
 			     (apply add-mark pos select)
 			     (apply add-mark (+ pos len) select)))
 			 (selection-members))))
-		(list "Unselect"  every-menu (lambda (w data) (set! (selection-member? #t) #f)))
-		(list "Reverse"   every-menu (lambda (w data) (reverse-selection)))
-		(list "Mix"       every-menu (lambda (w data) (mix-selection (cursor))))
-		(list "Invert"    every-menu (lambda (w data) (scale-selection-by -1)))))))
+		(list "Apply controls" every-menu (lambda (w data) (apply-controls (selected-sound) 2))) ; 2=selection
+		(list "Reset controls" every-menu (lambda (w data) (reset-controls)))
+		(list "Unselect"       every-menu (lambda (w data) (set! (selection-member? #t) #f)))
+		(list "Reverse"        every-menu (lambda (w data) (reverse-selection)))
+		(list "Mix"            every-menu (lambda (w data) (mix-selection (cursor))))
+		(list "Invert"         every-menu (lambda (w data) (scale-selection-by -1)))))))
 	   
 	   ;; -------- time domain popup
 	   
@@ -295,6 +296,12 @@
 		      (lambda (w) (set! unselect-popup-menu w))
 		      (lambda (w data) 
 			(set! (selection-member? #t) #f)))
+		(list "Apply controls"          every-menu 
+		      (lambda (w data) 
+			(apply-controls)))
+		(list "Reset controls"          every-menu 
+		      (lambda (w data) 
+			(reset-controls)))
 		(list "Info"                every-menu 
 		      (lambda (w data)
 			(let ((snd graph-popup-snd))
@@ -374,8 +381,7 @@
 	   (windows (list rectangular-window hann-window welch-window parzen-window bartlett-window hamming-window blackman2-window 
 			  blackman3-window blackman4-window exponential-window riemann-window kaiser-window cauchy-window 
 			  poisson-window gaussian-window tukey-window dolph-chebyshev-window hann-poisson-window connes-window))
-	   (types (list fourier-transform wavelet-transform autocorrelation cepstrum
-			walsh-transform hadamard-transform haar-transform))
+	   (types (list fourier-transform wavelet-transform autocorrelation cepstrum walsh-transform haar-transform))
 	   
 	   (fft-popup-menu #f)
 	   ;; used within graph if pointer is in the fft graph
@@ -468,7 +474,7 @@
 				(list name 
 				      (lambda (w data)
 					(set! (transform-type graph-popup-snd (choose-chan)) val))))
-			      (list "Fourier" "Wavelet" "Autocorrelate" "Cepstrum" "Walsh" "Hadamard" "Haar")
+			      (list "Fourier" "Wavelet" "Autocorrelate" "Cepstrum" "Walsh" "Haar")
 			      types)
 			 'fft-types)))
 		(list "Wavelet type"
@@ -584,3 +590,156 @@
 			   (gtk_menu_popup (GTK_MENU menu) #f #f #f #f (.button e) (.time e))
 			   #t)
 			 #f))))))
+
+
+;;; -------- edit history popup
+
+(define edhist-funcs '())
+(define edhist-widgets '())
+(define edhist-channel-widgets '())
+(define edhist-snd #f)
+(define edhist-chn #f)
+(define edhist-time 0)
+
+(define (edhist-clear-edits)
+  (set! edhist-funcs '())
+  #f)
+
+(define (edhist-save-edits)
+  (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs))
+	 (cur-edits (edits edhist-snd edhist-chn))
+	 (new-func (edit-list->function edhist-snd edhist-chn (1+ (car cur-edits)) (apply + cur-edits))))
+    (if old-val
+	(set-cdr! old-val new-func)
+	;; perhaps this should save the previous function under the current file name?
+	(set! edhist-funcs (cons (cons (cons edhist-snd edhist-chn) new-func) edhist-funcs)))
+    #f))
+
+(define (edhist-reapply-edits)
+  (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs)))
+    (if old-val	((cdr old-val) edhist-snd edhist-chn))))
+
+(define (edhist-apply-edits)
+  ;; the current popdown widgets are in edhist-widgets, current funcs are in edhist-funcs
+  ;; edhist-apply (for new widgets) is car of edhist-widgets
+
+  (define (edhist-apply w d)
+    (let ((index (cadr (g_object_get_data (G_OBJECT w) "popup-index"))))
+      (if (and (> index 0)
+	       (<= index (length edhist-funcs)))
+	  ((cdr (list-ref edhist-funcs (1- index))) edhist-snd edhist-chn))))
+	  
+  (let ((wids (cdr edhist-widgets))
+	(funcs edhist-funcs)
+	(parent (car edhist-widgets))
+	(index 1))
+    (if (not (null? funcs))
+	(for-each
+	 (lambda (func)
+	   (let* ((label (car func))
+		  (button (if (not (null? wids))
+			      (let ((wid (car wids)))
+				(set! wids (cdr wids))
+				wid)
+			      (let ((wid (gtk_menu_item_new_with_label "label")))
+				(gtk_menu_shell_append (GTK_MENU_SHELL parent) wid)
+				(set! edhist-widgets (append edhist-widgets (list wid)))
+				(g_signal_connect wid "activate" edhist-apply)
+				wid))))
+	     (if (pair? label)
+		 (change-label button (format #f "~A[~A]" (short-file-name (car label)) (cdr label)))
+		 (change-label button label))
+	     (g_object_set_data (G_OBJECT button) "popup-index" (GPOINTER index))
+	     (gtk_widget_show button)
+	     (set! index (1+ index))))
+	 edhist-funcs))
+    (if (not (null? wids))
+	(for-each
+	 (lambda (w)
+	   (gtk_widget_hide w)))
+	 wids)
+    (gtk_menu_popup (GTK_MENU parent) #f #f #f #f 3 edhist-time)
+    #f))
+
+(define (edhist-help-edits)
+  (help-dialog "Edit History Functions"
+	       "This popup menu gives access to the edit-list function handlers in Snd. \
+At any time you can backup in the edit list, 'save' the current trailing edits, make some \
+new set of edits, then 'reapply' the saved edits.  The 'apply' choice gives access to all \
+currently saved edit lists -- any such list can be applied to any channel.  'Clear' deletes \
+all saved edit lists."
+	       (list "{edit lists}" "{edit-list->function}")
+	       (list "extsnd.html#editlists" "extsnd.html#editlist_to_function")
+	       ))
+
+(define edhist-save-menu #f)
+(define edhist-apply-menu #f)
+(define edhist-reapply-menu #f)
+(define edhist-clear-menu #f)
+
+;(gc) -- trouble here?
+(define edit-history-menu
+  (let ((every-menu (lambda (w) #f)))
+    (make-popup-menu 
+     (lambda (w) #f)
+     (list
+      (list "Edits"    #f)
+      (list #f #f)
+      (list "Save"
+	    (lambda (w) (set! edhist-save-menu w))
+	    (lambda (w d) (edhist-save-edits)))
+      (list "Reapply"
+	    (lambda (w) (set! edhist-reapply-menu w))
+	    (lambda (w d) (edhist-reapply-edits)))
+      (list "Apply"
+	    (lambda (w)
+	      (set! edhist-apply-menu w)
+	      (let* ((top-cascade (gtk_menu_new)))
+		(gtk_menu_item_set_submenu (GTK_MENU_ITEM w) top-cascade)
+		(set! edhist-widgets (list top-cascade))))
+	    (lambda (w d) (edhist-apply-edits)))
+      (list "Clear"
+	    (lambda (w) (set! edhist-clear-menu w))
+	    (lambda (w d) (edhist-clear-edits)))
+      (list "Help"     #f 
+	    (lambda (w d) (edhist-help-edits)))))))
+
+(define (add-edhist-popup snd)
+  (let ((chns (chans snd)))
+    (do ((i 0 (1+ i)))
+	((= i chns))
+      (let ((edhist (list-ref (channel-widgets snd i) 7)))
+	(if (not (member edhist edhist-channel-widgets))
+	    (begin
+	      (set! edhist-channel-widgets (cons edhist edhist-channel-widgets))
+	      (g_signal_connect edhist "button_press_event" 
+		(lambda (w e d) 
+		  (let ((button (.button (GDK_EVENT_BUTTON e))))
+		    (if (= button 3)
+			(begin
+			  (set! edhist-snd snd)
+			  (set! edhist-chn i)
+			  (gtk_widget_set_sensitive edhist-clear-menu (not (null? edhist-funcs)))
+			  (gtk_widget_set_sensitive edhist-save-menu (> (apply + (edits snd i)) 0))
+			  (gtk_widget_set_sensitive edhist-apply-menu (not (null? edhist-funcs)))
+			  (gtk_widget_set_sensitive edhist-reapply-menu (not (eq? #f (assoc (cons snd i) edhist-funcs))))
+			  (gtk_widget_show edit-history-menu)
+			  (set! edhist-time (.time (GDK_EVENT_BUTTON e)))
+			  (gtk_menu_popup (GTK_MENU edit-history-menu) #f #f #f #f (.button (GDK_EVENT_BUTTON e)) (.time (GDK_EVENT_BUTTON e)))
+			  #t) ; don't select anything in the list (i.e. don't pass event to widget)
+			#f)))
+		#f)
+	      ))))))
+
+(add-hook! after-open-hook add-edhist-popup)
+(add-hook! close-hook (lambda (snd)
+			(let ((chns (chans snd))
+			      (name (short-file-name snd)))
+			  (do ((i 0 (1+ i)))
+			      ((= i chns))
+			    (let* ((old-val (assoc (cons snd i) edhist-funcs)))
+			      (if old-val
+				  (set-car! old-val (format #f "~A[~A]" name i))))))))
+(for-each add-edhist-popup (sounds))
+
+
