@@ -61,7 +61,6 @@ void mus_bint_to_char(unsigned char *j, int x)
   j[0] = ox[3]; j[1] = ox[2]; j[2] = ox[1]; j[3] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 4);
-  /* j[0] = ox[0]; j[1] = ox[1]; j[2] = ox[2]; j[3] = ox[3]; */
 #endif
 }
 
@@ -73,7 +72,6 @@ int mus_char_to_bint (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -85,7 +83,6 @@ void mus_lint_to_char(unsigned char *j, int x)
   j[0] = ox[3]; j[1] = ox[2]; j[2] = ox[1]; j[3] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 4);
-  /* j[0] = ox[0]; j[1] = ox[1]; j[2] = ox[2]; j[3] = ox[3]; */
 #endif
 }
 
@@ -97,7 +94,6 @@ int mus_char_to_lint (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -107,7 +103,6 @@ int mus_char_to_uninterpreted_int (const unsigned char *inp)
   int o;
   unsigned char *outp = (unsigned char *)&o;
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
   return(o);
 }
 
@@ -119,7 +114,6 @@ unsigned int mus_char_to_ubint (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -132,7 +126,6 @@ unsigned int mus_char_to_ulint (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -145,7 +138,6 @@ void mus_bfloat_to_char(unsigned char *j, float x)
   j[0] = ox[3]; j[1] = ox[2]; j[2] = ox[1]; j[3] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 4);
-  /* j[0] = ox[0]; j[1] = ox[1]; j[2] = ox[2]; j[3] = ox[3]; */
 #endif
 }
 
@@ -157,7 +149,6 @@ float mus_char_to_bfloat (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -169,7 +160,6 @@ void mus_lfloat_to_char(unsigned char *j, float x)
   j[0] = ox[3]; j[1] = ox[2]; j[2] = ox[1]; j[3] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 4);
-  /* j[0] = ox[0]; j[1] = ox[1]; j[2] = ox[2]; j[3] = ox[3]; */
 #endif
 }
 
@@ -181,7 +171,6 @@ float mus_char_to_lfloat (const unsigned char *inp)
   outp[0] = inp[3]; outp[1] = inp[2]; outp[2] = inp[1]; outp[3] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 4);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; outp[2] = inp[2]; outp[3] = inp[3]; */
 #endif
   return(o);
 }
@@ -193,7 +182,6 @@ void mus_bshort_to_char(unsigned char *j, short x)
   j[0] = ox[1]; j[1] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 2); /* I wonder if this is faster */
-  /* j[0] = ox[0]; j[1] = ox[1]; */
 #endif
 }
 
@@ -205,7 +193,6 @@ short mus_char_to_bshort (const unsigned char *inp)
   outp[0] = inp[1]; outp[1] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 2);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; */
 #endif
   return(o);
 }
@@ -217,7 +204,6 @@ void mus_lshort_to_char(unsigned char *j, short x)
   j[0] = ox[1]; j[1] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 2);
-  /* j[0] = ox[0]; j[1] = ox[1]; */
 #endif
 }
 
@@ -229,7 +215,6 @@ short mus_char_to_lshort (const unsigned char *inp)
   outp[0] = inp[1]; outp[1] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 2);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; */
 #endif
   return(o);
 }
@@ -241,7 +226,6 @@ void mus_ubshort_to_char(unsigned char *j, unsigned short x)
   j[0] = ox[1]; j[1] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 2);
-  /* j[0] = ox[0]; j[1] = ox[1]; */
 #endif
 }
 
@@ -253,7 +237,6 @@ unsigned short mus_char_to_ubshort (const unsigned char *inp)
   outp[0] = inp[1]; outp[1] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 2);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; */
 #endif
   return(o);
 }
@@ -265,7 +248,6 @@ void mus_ulshort_to_char(unsigned char *j, unsigned short x)
   j[0] = ox[1]; j[1] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 2);
-  /* j[0] = ox[0]; j[1] = ox[1]; */
 #endif
 }
 
@@ -277,7 +259,6 @@ unsigned short mus_char_to_ulshort (const unsigned char *inp)
   outp[0] = inp[1]; outp[1] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 2);
-  /* outp[0] = inp[0]; outp[1] = inp[1]; */
 #endif
   return(o);
 }
@@ -288,7 +269,6 @@ double mus_char_to_ldouble (const unsigned char *inp)
   unsigned char *outp = (unsigned char *)&o;
 #if (MUS_LITTLE_ENDIAN)
   memcpy((void *)outp, (void *)inp, 8);
-  /* for (i = 0; i < 8; i++) outp[i] = inp[i]; */
 #else
   outp[0] = inp[7]; outp[1] = inp[6]; outp[2] = inp[5]; outp[3] = inp[4]; outp[4] = inp[3]; outp[5] = inp[2]; outp[6] = inp[1]; outp[7] = inp[0];
 #endif
@@ -303,7 +283,6 @@ double mus_char_to_bdouble (const unsigned char *inp)
   outp[0] = inp[7]; outp[1] = inp[6]; outp[2] = inp[5]; outp[3] = inp[4]; outp[4] = inp[3]; outp[5] = inp[2]; outp[6] = inp[1]; outp[7] = inp[0];
 #else
   memcpy((void *)outp, (void *)inp, 8);
-  /* for (i = 0; i < 8; i++) outp[i] = inp[i]; */
 #endif
   return(o);
 }
@@ -315,7 +294,6 @@ void mus_bdouble_to_char(unsigned char *j, double x)
   j[0] = ox[7]; j[1] = ox[6]; j[2] = ox[5]; j[3] = ox[4]; j[4] = ox[3]; j[5] = ox[2]; j[6] = ox[1]; j[7] = ox[0];
 #else
   memcpy((void *)j, (void *)ox, 8);
-  /* for (i = 0; i < 8; i++) j[i] = ox[i]; */
 #endif
 }
 
@@ -324,7 +302,6 @@ void mus_ldouble_to_char(unsigned char *j, double x)
   unsigned char *ox = (unsigned char *)&x;
 #if (MUS_LITTLE_ENDIAN)
   memcpy((void *)j, (void *)ox, 8);
-  /* for (i = 0; i < 8; i++) j[i] = ox[i]; */
 #else
   j[0] = ox[7]; j[1] = ox[6]; j[2] = ox[5]; j[3] = ox[4]; j[4] = ox[3]; j[5] = ox[2]; j[6] = ox[1]; j[7] = ox[0];
 #endif
@@ -429,14 +406,6 @@ void mus_ldouble_to_char(unsigned char *j, double x)
 
 /* ---------------- file descriptors ---------------- */
 
-static int rt_ap_out;   /* address of RT audio ports, if any */
-
-/* for CLM */
-void mus_set_rt_audio_p (int rt)
-{
-  rt_ap_out = rt;
-}
-
 typedef struct {
   char *name;
   int data_format, bytes_per_sample, chans, header_type;
@@ -448,6 +417,13 @@ typedef struct {
 static int io_fd_size = 0;
 static io_fd **io_fds = NULL;
 #define IO_FD_ALLOC_SIZE 8
+static bool file_data_clipped_default = false;
+static Float file_prescaler_default = 1.0;
+
+bool mus_file_data_clipped_default(void) {return(file_data_clipped_default);}
+bool mus_file_set_data_clipped_default(bool new_value) {file_data_clipped_default = new_value; return(new_value);}
+Float mus_file_prescaler_default(void) {return(file_prescaler_default);}
+Float mus_file_set_prescaler_default(Float new_value) {file_prescaler_default = new_value; return(new_value);}
 
 int mus_file_open_descriptors(int tfd, const char *name, int format, int size /* datum size */, off_t location, int chans, int type)
 {
@@ -472,8 +448,8 @@ int mus_file_open_descriptors(int tfd, const char *name, int format, int size /*
   fd->data_format = format;
   fd->bytes_per_sample = size;
   fd->data_location = location;
-  fd->data_clipped = false;
-  fd->prescaler = 1.0;
+  fd->data_clipped = file_data_clipped_default;
+  fd->prescaler = file_prescaler_default;
   fd->header_type = type;
   fd->chans = chans;
   if (name)
@@ -626,7 +602,6 @@ int mus_file_close(int fd)
 off_t mus_file_seek_frame(int tfd, off_t frame)
 {
   io_fd *fd;
-  if ((tfd == MUS_DAC_CHANNEL) || (tfd == MUS_DAC_REVERB)) return(0);
   if (io_fds == NULL) 
     return(mus_error(MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, "mus_file_seek_frame: no file descriptors!"));
   if (tfd < 0)
@@ -670,12 +645,12 @@ static int search(int val, short *table, int size)
 static unsigned char to_alaw(int pcm_val)
 {
   int mask, seg;
-  unsigned char	aval;
-  if (pcm_val >= 0) {mask = 0xD5;} else {mask = 0x55; pcm_val = -pcm_val - 8;}
+  if (pcm_val >= 0) mask = 0xD5; else {mask = 0x55; pcm_val = -pcm_val - 8;}
   seg = search(pcm_val, seg_end, 8);
   if (seg >= 8)	return (0x7F ^ mask);
   else 
     {
+      unsigned char	aval;
       aval = seg << SEG_SHIFT;
       if (seg < 2) aval |= (pcm_val >> 4) & QUANT_MASK; else aval |= (pcm_val >> (seg + 3)) & QUANT_MASK;
       return (aval ^ mask);
@@ -707,12 +682,12 @@ static unsigned char to_mulaw(int pcm_val)
 {
   int mask;
   int seg;
-  unsigned char	uval;
   if (pcm_val < 0) {pcm_val = BIAS - pcm_val; mask = 0x7F;} else {pcm_val += BIAS; mask = 0xFF;}
   seg = search(pcm_val, seg_end, 8);
   if (seg >= 8) return (0x7F ^ mask);
   else 
     {
+      unsigned char	uval;
       uval = (seg << 4) | ((pcm_val >> (seg + 3)) & 0xF);
       return (uval ^ mask);
     }
@@ -1006,7 +981,6 @@ int mus_file_read_buffer(int charbuf_data_format, int beg, int chans, int nints,
 int mus_file_read(int tfd, int beg, int end, int chans, mus_sample_t **bufs)
 {
   int num, rtn, i, k;
-  mus_sample_t *buffer;
   num = (end - beg + 1);
   rtn = mus_read_any_1(tfd, beg, chans, num, bufs, NULL, NULL);
   if (rtn == MUS_ERROR) return(MUS_ERROR);
@@ -1014,6 +988,7 @@ int mus_file_read(int tfd, int beg, int end, int chans, mus_sample_t **bufs)
     /* this zeroing can be fooled if the file is chunked and has trailing, non-data chunks */
     for (k = 0; k < chans; k++)
       {
+	mus_sample_t *buffer;
 	buffer = (mus_sample_t *)(bufs[k]);
 	i = rtn + beg;
 	/* this happens routinely in mus_outa + initial write (reads ahead in effect) */
@@ -1026,7 +1001,6 @@ int mus_file_read_chans(int tfd, int beg, int end, int chans, mus_sample_t **buf
 {
   /* an optimization of mus_file_read -- just reads the desired channels */
   int num, rtn, i, k;
-  mus_sample_t *buffer;
   num = (end - beg + 1);
   rtn = mus_read_any_1(tfd, beg, chans, num, bufs, cm, NULL);
   if (rtn == MUS_ERROR) return(MUS_ERROR);
@@ -1034,6 +1008,7 @@ int mus_file_read_chans(int tfd, int beg, int end, int chans, mus_sample_t **buf
     for (k = 0; k < chans; k++)
       if ((cm == NULL) || (cm[k]))
 	{
+	  mus_sample_t *buffer;
 	  buffer = (mus_sample_t *)(bufs[k]);
 	  i = rtn + beg;
 	  memset((void *)(buffer + i), 0, (end - i + 1) * sizeof(mus_sample_t));
@@ -1047,33 +1022,28 @@ int mus_file_read_chans(int tfd, int beg, int end, int chans, mus_sample_t **buf
 static int checked_write(int tfd, char *buf, int chars)
 {
   int bytes;
-  io_fd *fd;
-  if (tfd == MUS_DAC_CHANNEL)
-    mus_audio_write(rt_ap_out, buf, chars);
-  else
+  bytes = write(tfd, buf, chars);
+  if (bytes != chars) 
     {
-      bytes = write(tfd, buf, chars);
-      if (bytes != chars) 
-	{
-	  if ((io_fds == NULL) || (tfd >= io_fd_size) || (tfd < 0) || (io_fds[tfd] == NULL))
-	    return(mus_error(MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, "mus_write: no file descriptors!"));
-	  fd = io_fds[tfd];
-	  if (fd->data_format == MUS_UNKNOWN) 
-	    return(mus_error(MUS_FILE_CLOSED,
-			     "attempt to write closed file %s",
-			     fd->name));
-	  else
+      io_fd *fd;
+      if ((io_fds == NULL) || (tfd >= io_fd_size) || (tfd < 0) || (io_fds[tfd] == NULL))
+	return(mus_error(MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, "mus_write: no file descriptors!"));
+      fd = io_fds[tfd];
+      if (fd->data_format == MUS_UNKNOWN) 
+	return(mus_error(MUS_FILE_CLOSED,
+			 "attempt to write closed file %s",
+			 fd->name));
+      else
 #ifndef MACOS
-	    return(mus_error(MUS_WRITE_ERROR,
-			     "mus_write: write error for %s%s%s: only %d of %d bytes written",
-			     fd->name, (errno) ? ": " : "", (errno) ? strerror(errno) : "",
-			     bytes, chars));
+	return(mus_error(MUS_WRITE_ERROR,
+			 "mus_write: write error for %s%s%s: only %d of %d bytes written",
+			 fd->name, (errno) ? ": " : "", (errno) ? strerror(errno) : "",
+			 bytes, chars));
 #else
-	    return(mus_error(MUS_WRITE_ERROR,
-			     "mus_write: write error for %s: only %d of %d bytes written",
-			     fd->name, bytes, chars));
+      return(mus_error(MUS_WRITE_ERROR,
+		       "mus_write: write error for %s: only %d of %d bytes written",
+		       fd->name, bytes, chars));
 #endif
-	}
     }
   return(MUS_NO_ERROR);
 }
@@ -1082,11 +1052,9 @@ off_t mus_file_write_zeros(int tfd, off_t num)
 {
 #if (!USE_SND)
   off_t lim;
-  int err;
   off_t curnum;
   io_fd *fd;
   char *charbuf = NULL;
-  if (tfd == MUS_DAC_REVERB) return(0);
   if ((io_fds == NULL) || (tfd >= io_fd_size) || (tfd < 0) || (io_fds[tfd] == NULL))
     return(mus_error(MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, "mus_file_write_zeros: no file descriptors!"));
   fd = io_fds[tfd];
@@ -1098,9 +1066,9 @@ off_t mus_file_write_zeros(int tfd, off_t num)
   lim = num * (fd->bytes_per_sample);
   curnum = lim;
   if (curnum > BUFLIM) curnum = BUFLIM;
-  /* for (i = 0; i < curnum; i++) charbuf[i] = 0; */ /* using calloc, so surely this is unnecessary? */
   while (curnum > 0)
     {
+      int err;
       err = checked_write(tfd, charbuf, curnum);
       if (err == MUS_ERROR) return(MUS_ERROR);
       lim -= (BUFLIM);
@@ -1129,7 +1097,6 @@ static int mus_write_1(int tfd, int beg, int end, int chans, mus_sample_t **bufs
   if (inbuf) to_buffer = true;
   if (!to_buffer)
     {
-      if (tfd == MUS_DAC_REVERB) return(0);
       if ((io_fds == NULL) || (tfd >= io_fd_size) || (tfd < 0) || (io_fds[tfd] == NULL))
 	return(mus_error(MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, "mus_write: no file descriptors!"));
       fd = io_fds[tfd];
@@ -1335,6 +1302,8 @@ void mus_reset_io_c(void)
 {
   io_fd_size = 0;
   io_fds = NULL;
+  file_data_clipped_default = false;
+  file_prescaler_default = 1.0;
 }
 
 
@@ -1465,8 +1434,10 @@ char *mus_expand_filename(const char *filename)
 	      if (res) break;
 	    }
 #else
+#if HAVE_GETWD
 	  pwd = (char *)CALLOC(PATH_MAX, sizeof(char));
 	  getwd(pwd);
+#endif
 #endif
 	  file_name_buf = (char *)CALLOC(len + sndlib_strlen(pwd) + 8, sizeof(char));
 	  if (pwd) 
@@ -1572,7 +1543,7 @@ char *mus_format(const char *format, ...)
   sprintf(buf, "%s...[you need vprintf]", format);
 #endif
 #endif
-#if DEBUG_MEMORY
+#if DEBUGGING
   rtn = copy_string(buf);
 #else
   rtn = strdup(buf);
