@@ -1,14 +1,15 @@
 ;;; specialize popup menus
 ;;;
-;;; currently there are four special popup menus:
-;;;   selection fft time-domain lisp-listener
+;;; currently there are five special popup menus:
+;;;   selection fft time-domain lisp-listener edit-history
 ;;;
-;;; (add-selection-popup) creates popup menus specialized for the fft, selection, and time-domain sections of the graph
+;;; (add-popups) creates popup menus specialized for the fft, selection, and time-domain sections of the graph
 ;;;    (change-selection-popup-color new-color) to change selection memu's color
 ;;;    (change-fft-popup-color new-color) to change fft menu's color
 ;;;    (change-graph-popup-color new-color) to change time-domain menu's color
 ;;; (add-listener-popup) posts a special popup menu if the pointer is in the listener
 ;;;    (change-listener-popup-color new-color) to change its color
+;;; a popup menu is also added to each edit history pane to display the "spreadsheet" edit-list->function menu
 
 (use-modules (ice-9 common-list) (ice-9 format))
 (provide 'snd-popup.scm)
@@ -49,9 +50,9 @@
      (lambda (entry)
        ;; entry is list: name type args optional-callback optional-func
        (let ((widget (XtCreateManagedWidget (car entry)
-					     (cadr entry)
-					     menu
-					     (caddr entry))))
+					    (cadr entry)
+					    menu
+					    (caddr entry))))
 	 (if (> (length entry) 3)
 	     (begin
 	       (XtAddCallback widget XmNactivateCallback (list-ref entry 3))
@@ -106,7 +107,7 @@
 		(if (and (not (c-g?))
 			 (= reason 0)
 			 stopping1)
-		    (play-selection #f -1 play-selection-again)
+		    (play-selection #f play-selection-again)
 		    (stop-playing-selection)))
 	      (if stopping1
 		  (begin
@@ -116,7 +117,7 @@
 		    (change-label w "Stop!")
 		    (set! stop-widget1 w) ; needs to be separate from Play case since we're stopping/restarting deliberately
 		    (set! stopping1 #t)
-		    (play-selection #f -1 play-selection-again)))))
+		    (play-selection #f play-selection-again)))))
       (list "Delete"    xmPushButtonWidgetClass every-menu (lambda (w c i) (delete-selection)))
       (list "Zero"      xmPushButtonWidgetClass every-menu (lambda (w c i) (scale-selection-by 0.0)))
       (list "Crop"      xmPushButtonWidgetClass every-menu
@@ -144,20 +145,16 @@
 		 sndlist))))
       (list "Save as"   xmPushButtonWidgetClass every-menu (lambda (w c i) (save-selection-dialog)))
       (list "Copy->New" xmPushButtonWidgetClass every-menu 
-	    (let ((selctr 0)) 
-	      (lambda (w c i) 
-		(let ((new-file-name (format #f "newf-~D.snd" selctr)))
-		  (set! selctr (+ selctr 1))
-		  (save-selection new-file-name)
-		  (open-sound new-file-name)))))
+	    (lambda (w c i) 
+	      (let ((new-file-name (snd-tempnam)))
+		(save-selection new-file-name)
+		(open-sound new-file-name))))
       (list "Cut->New"   xmPushButtonWidgetClass every-menu 
-	    (let ((selctr 0)) 
-	      (lambda (w c i) 
-		(let ((new-file-name (format #f "newf-~D.snd" selctr)))
-		  (set! selctr (+ selctr 1))
-		  (save-selection new-file-name)
-		  (delete-selection)
-		  (open-sound new-file-name)))))
+	    (lambda (w c i) 
+	      (let ((new-file-name (snd-tempnam)))
+		(save-selection new-file-name)
+		(delete-selection)
+		(open-sound new-file-name))))
       (list "Snap marks" xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (for-each 
@@ -186,10 +183,12 @@
 				(sqrt (/ sum len)))
 			     (let ((val (next-sample reader)))
 			       (set! sum (+ sum (* val val)))))))))))
-      (list "Unselect"  xmPushButtonWidgetClass every-menu (lambda (w c i) (set! (selection-member? #t) #f)))
-      (list "Reverse"   xmPushButtonWidgetClass every-menu (lambda (w c i) (reverse-selection)))
-      (list "Mix"       xmPushButtonWidgetClass every-menu (lambda (w c i) (mix-selection (cursor))))
-      (list "Invert"    xmPushButtonWidgetClass every-menu (lambda (w c i) (scale-selection-by -1)))))))
+      (list "Apply controls" xmPushButtonWidgetClass every-menu (lambda (w c i) (apply-controls (selected-sound) 2))) ; 2=selection
+      (list "Reset controls" xmPushButtonWidgetClass every-menu (lambda (w c i) (reset-controls)))
+      (list "Unselect"       xmPushButtonWidgetClass every-menu (lambda (w c i) (set! (selection-member? #t) #f)))
+      (list "Reverse"        xmPushButtonWidgetClass every-menu (lambda (w c i) (reverse-selection)))
+      (list "Mix"            xmPushButtonWidgetClass every-menu (lambda (w c i) (mix-selection (cursor))))
+      (list "Invert"         xmPushButtonWidgetClass every-menu (lambda (w c i) (scale-selection-by -1)))))))
 
 
 ;;; -------- time domain popup
@@ -339,6 +338,12 @@
       (list "Equalize panes"     xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (equalize-panes)))
+      (list "Apply controls"     xmPushButtonWidgetClass every-menu
+	    (lambda (w c i)
+	      (apply-controls)))
+      (list "Reset controls"     xmPushButtonWidgetClass every-menu
+	    (lambda (w c i)
+	      (reset-controls)))
       (list "Info"               xmPushButtonWidgetClass every-menu 
 	    (lambda (w c i)
 	      (let ((snd graph-popup-snd))
@@ -573,15 +578,14 @@
 	  lst windows))
        every-menu))
 
-    (let ((types (list fourier-transform wavelet-transform autocorrelation cepstrum
-		       walsh-transform hadamard-transform haar-transform)))
+    (let ((types (list fourier-transform wavelet-transform autocorrelation cepstrum walsh-transform haar-transform)))
       (make-simple-popdown-menu 
        "Transform type"
        (map (lambda (name val)
 	      (list name 
 		    (lambda (w c i)
 		      (set! (transform-type graph-popup-snd (choose-chan)) val))))
-	    (list "Fourier" "Wavelet" "Autocorrelate" "Cepstrum" "Walsh" "Hadamard" "Haar")
+	    (list "Fourier" "Wavelet" "Autocorrelate" "Cepstrum" "Walsh" "Haar")
 	    types)
        fft-popup 
        (lambda (lst)
@@ -618,9 +622,126 @@
 
     fft-popup))
 
-(define (add-selection-popup)
-  "(add-selection-popup) makes the selection-related popup menu"
+
+;;; -------- edit history popup
+
+(define edhist-funcs '())
+(define edhist-widgets '()) ; the apply popdown list
+(define edhist-snd #f)
+(define edhist-chn #f)
+
+(define (edhist-clear-edits w c i)
+  (set! edhist-funcs '())
+  #f)
+
+(define (edhist-save-edits w c i)
+  (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs))
+	 (cur-edits (edits edhist-snd edhist-chn))
+	 (new-func (edit-list->function edhist-snd edhist-chn (1+ (car cur-edits)) (apply + cur-edits))))
+    (if old-val
+	(set-cdr! old-val new-func)
+	;; perhaps this should save the previous function under the current file name?
+	(set! edhist-funcs (cons (cons (cons edhist-snd edhist-chn) new-func) edhist-funcs)))
+    #f))
+
+(define (edhist-reapply-edits w c i)
+  (let* ((old-val (assoc (cons edhist-snd edhist-chn) edhist-funcs)))
+    (if old-val	((cdr old-val) edhist-snd edhist-chn))))
+
+(define (edhist-apply-edits w c i)
+  ;; the current popdown widgets are in edhist-widgets, current funcs are in edhist-funcs
+  ;; edhist-apply (for new widgets) is car of edhist-widgets
+
+  (define (edhist-apply w c i)
+    (let ((index (cadr (XtVaGetValues w (list XmNuserData 0)))))
+      (if (and (> index 0)
+	       (<= index (length edhist-funcs)))
+	  ((cdr (list-ref edhist-funcs (1- index))) edhist-snd edhist-chn))))
+	  
+  (let ((wids (cdr edhist-widgets))
+	(funcs edhist-funcs)
+	(parent (car edhist-widgets))
+	(index 1))
+    (if (not (null? funcs))
+	(for-each
+	 (lambda (func)
+	   (let* ((label (car func))
+		  (button (if (not (null? wids))
+			      (let ((wid (car wids)))
+				(set! wids (cdr wids))
+				(XtManageChild wid)
+				wid)
+			      (let ((wid (XtCreateManagedWidget "wid" xmPushButtonWidgetClass parent (list XmNbackground (highlight-color)))))
+				(set! edhist-widgets (append edhist-widgets (list wid)))
+				(XtAddCallback wid XmNactivateCallback edhist-apply)
+				wid))))
+	     (if (pair? label)
+		 (change-label button (format #f "~A[~A]" (short-file-name (car label)) (cdr label)))
+		 (change-label button label))
+	     (XtVaSetValues button (list XmNuserData index))
+	     (set! index (1+ index))))
+	 edhist-funcs))
+    (if (not (null? wids))
+	(for-each
+	 (lambda (w)
+	   (XtUnmanageChild w))
+	 wids)))
+  #f)
+
+(define (edhist-help-edits w c i)
+  (help-dialog "Edit History Functions"
+	       "This popup menu gives access to the edit-list function handlers in Snd. \
+At any time you can backup in the edit list, 'save' the current trailing edits, make some \
+new set of edits, then 'reapply' the saved edits.  The 'apply' choice gives access to all \
+currently saved edit lists -- any such list can be applied to any channel.  'Clear' deletes \
+all saved edit lists."
+	       (list "{edit lists}" "{edit-list->function}")
+	       (list "extsnd.html#editlists" "extsnd.html#editlist_to_function")
+	       ))
+
+(define edit-history-menu
+  (let* ((every-menu (list XmNbackground (highlight-color)))
+	 (edhist-popup (XmCreatePopupMenu (caddr (main-widgets)) "edhist-popup"
+					  (append (list XmNpopupEnabled #t) every-menu))))
+    (XtCreateManagedWidget "Edits" xmLabelWidgetClass edhist-popup every-menu)
+    (XtCreateManagedWidget "sep" xmSeparatorWidgetClass edhist-popup every-menu)
+    (let ((edhist-save (XtCreateManagedWidget "Save" xmPushButtonWidgetClass edhist-popup every-menu)))
+      (XtAddCallback edhist-save XmNactivateCallback edhist-save-edits))
+    (let ((edhist-reapply (XtCreateManagedWidget "Reapply" xmPushButtonWidgetClass edhist-popup every-menu)))
+      (XtAddCallback edhist-reapply XmNactivateCallback edhist-reapply-edits))
+    (let* ((edhist-apply (XmCreatePulldownMenu edhist-popup "Apply" every-menu))
+	   (apply-cascade (XtCreateManagedWidget "Apply" xmCascadeButtonWidgetClass edhist-popup
+						 (append (list XmNsubMenuId edhist-apply)
+							 every-menu))))
+      (XtAddCallback apply-cascade XmNcascadingCallback edhist-apply-edits)
+      (set! edhist-widgets (list edhist-apply)))
+    (let ((edhist-clear (XtCreateManagedWidget "Clear" xmPushButtonWidgetClass edhist-popup every-menu)))
+      (XtAddCallback edhist-clear XmNactivateCallback edhist-clear-edits))
+    (let ((edhist-help (XtCreateManagedWidget "Help" xmPushButtonWidgetClass edhist-popup every-menu)))
+      (XtAddCallback edhist-help XmNactivateCallback edhist-help-edits))
+    (add-hook! close-hook (lambda (snd)
+			    (let ((chns (chans snd))
+				  (name (short-file-name snd)))
+			      (do ((i 0 (1+ i)))
+				  ((= i chns))
+				(let* ((old-val (assoc (cons snd i) edhist-funcs)))
+				  (if old-val
+				      (set-car! old-val (format #f "~A[~A]" name i))))))))
+    edhist-popup))
+
+(define (edit-history-popup-menu snd chn)
+  (set! edhist-snd snd)
+  (set! edhist-chn chn)
+  edit-history-menu)
+
+
+
+;;; -------- activate the above menus
+
+(define (add-popups)
+  "(add-popups) adds context-sensitive popup menus to various parts of the interface"
   (let ((popups '()))
+
     (define (find-popup snd chn dats)
       (if (not (null? dats))
 	  (let ((cur (car dats)))
@@ -636,6 +757,28 @@
 	(if (not (find-popup snd chn popups))
 	    (let ((chn-grf (car (channel-widgets snd chn))))
 	      (set! popups (cons (list snd chn) popups))
+
+	      ;; edit history list spreadsheet support
+	      (XtAddCallback  (list-ref (channel-widgets snd chn) 7) XmNpopupHandlerCallback 
+		 (lambda (w data info)
+		   (let ((e (.event info)))
+		     (if (= ButtonPress (.type e))
+			 (begin
+			   (if edit-history-menu
+			       (for-each-child
+				edit-history-menu
+				(lambda (w)
+				  (let ((name (XtName w)))
+				    (if (string=? name "Clear")
+					(XtSetSensitive w (not (null? edhist-funcs)))
+					(if (string=? name "Save")
+					    (XtSetSensitive w (> (apply + (edits snd chn)) 0))
+					    (if (string=? name "Apply")
+						(XtSetSensitive w (not (null? edhist-funcs)))
+						(if (string=? name "Reapply")
+						    (XtSetSensitive w (not (eq? #f (assoc (cons snd chn) edhist-funcs))))))))))))
+			   (set! (.menuToPost info) (edit-history-popup-menu snd chn)))))))
+
 	      (XtAddCallback chn-grf XmNpopupHandlerCallback 
 		 (lambda (w data info)
 		   (let* ((e (.event info))
@@ -691,10 +834,7 @@
 					   (set! (.menuToPost info) graph-popup-menu)))))))))))))))
 
     (add-hook! after-open-hook add-popup)
-    (for-each 
-     (lambda (snd)
-       (add-popup snd))
-     (sounds))))
+    (for-each add-popup (sounds))))
 
 (define (change-menu-color menu new-color)
   "(change-menu-color menu new-color) changes the color of menu to new-color. new-color can be the \
@@ -887,13 +1027,10 @@ color name, an xm Pixel, a snd color, or a list of rgb values (as in Snd's make-
 		(set! (.menuToPost info) listener-popup)))))
       listener-popup)))
 
-(add-selection-popup)
+(add-popups)
 (define listener-menu (add-listener-popup))
 
 (define (change-listener-popup-color new-color)
   "(change-listener-popup-color new-color) changes the listener popup menu's color"
   ;; slightly different from the earlier cases because the menu parent is not explicitly in the list
   (change-menu-color listener-menu new-color))
-
-
-
