@@ -98,7 +98,7 @@
 #define TIME_STR_SIZE 64
 #define PRINT_BUFFER_SIZE 512
 #define LABEL_BUFFER_SIZE 64
-#define REPORTING_SIZE (FILE_BUFFER_SIZE * 20)
+#define REPORTING_SIZE (MAX_BUFFER_SIZE * 8)
 /* progress bar (hourglass icon) is displayed if more than this many samples are being processed */
 #define NUM_GLASSES 15
 #define NUM_BOMBS 15
@@ -148,6 +148,8 @@ typedef enum {GRAPH_LINES, GRAPH_DOTS, GRAPH_FILLED, GRAPH_DOTS_AND_LINES, GRAPH
   *   and I think lollipop is more accurate -- in my experience lollipops have
   *     circular candies whereas popsicles have a sort of squared-off bottom.
   */
+
+typedef enum {APPLY_TO_SOUND, APPLY_TO_CHANNEL, APPLY_TO_SELECTION} snd_apply_t;
 typedef enum {GRAPH_ONCE, GRAPH_AS_SONOGRAM, GRAPH_AS_SPECTROGRAM, GRAPH_AS_WAVOGRAM} graph_type_t;
 typedef enum {ZOOM_FOCUS_LEFT, ZOOM_FOCUS_RIGHT, ZOOM_FOCUS_ACTIVE, ZOOM_FOCUS_MIDDLE} zoom_focus_t;
 typedef enum {DONT_LOCK_MIXES, LOCK_MIXES} lock_mix_t;
@@ -167,6 +169,35 @@ typedef enum {COLOR_DIALOG, ORIENTATION_DIALOG, ENVED_DIALOG, ERROR_DIALOG, YES_
 typedef enum {SOUND_IDLE, SOUND_NORMAL, SOUND_WRAPPER, SOUND_REGION, SOUND_READER} sound_inuse_t;
 typedef enum {NOT_FILING, INPUT_FILING, REGION_FILING, CHANNEL_FILING, TEMP_FILING, CHANGE_FILING, INSERT_FILING, MACRO_FILING} sp_filing_t;
 
+#define FORCE_REFFT true
+#define DONT_FORCE_REFFT false
+#define FORCE_REDISPLAY true
+#define DONT_FORCE_REDISPLAY false
+#define WITH_EVENTS true
+#define WITHOUT_EVENTS false
+#define NOT_IN_AS_ONE_EDIT false
+#define IN_AS_ONE_EDIT true
+
+#if HAVE_RUBY
+  #define TO_PROC_NAME(Str) xen_scheme_procedure_to_ruby(Str)
+  #define TO_VAR_NAME(Str) xen_scheme_constant_to_ruby(Str)
+  #define PROC_OPEN "("
+  #define PROC_SEP ", "
+  #define VECTOR_OPEN "["
+  #define VECTOR_CLOSE "]"
+  #define BPAREN ""
+  #define EPAREN ""
+#else
+  #define TO_PROC_NAME(Str) Str
+  #define TO_VAR_NAME(Str) Str
+  #define PROC_OPEN " "
+  #define PROC_SEP " "
+  #define VECTOR_OPEN "'#("
+  #define VECTOR_CLOSE ")"
+  #define BPAREN "("
+  #define EPAREN ")"
+#endif
+
 #define MAX_MAIN_MENUS 64
 #define NO_REGIONS -2
 #define INVALID_REGION -1
@@ -179,6 +210,10 @@ typedef enum {SEARCH_OK, SEARCH_FAILED} search_result_t;
 typedef enum {IGNORE_CACHE, REMOVE_FROM_CACHE} cache_remove_t;
 typedef enum {FFT_UNCHANGED, FFT_CHANGED, FFT_CHANGE_LOCKED} fft_change_t;
 typedef enum {WITHOUT_GRAPH, WITH_GRAPH, WITHOUT_INITIAL_GRAPH_HOOK} channel_graph_t;
+typedef enum {NOT_PRINTING, PRINTING} printing_t;
+typedef enum {NO_X_AXIS, WITH_X_AXIS} with_x_axis_t;
+typedef enum {NO_GRID, WITH_GRID} with_grid_t;
+typedef enum {WITH_LINEAR_AXES, WITH_LOG_X_AXIS, WITH_LOG_Y_AXIS, WITH_LOG_AXES} log_axis_t;
 
 #if defined(__GNUC__) && (!(defined(__cplusplus)))
   #define IS_PLAYER(snd) ({ snd_info *_snd_0_h_1 = snd; ((_snd_0_h_1) && (_snd_0_h_1->index < 0)); })
@@ -202,18 +237,16 @@ typedef enum {FD_CLOSED, FD_OPEN} fd_open_t;
 typedef enum {PRINT_SND, PRINT_ENV} print_choice_t;
 typedef enum {SND_DATA_NO_DATA, SND_DATA_FILE, SND_DATA_BUFFER} snd_data_file_t;
 typedef enum {SOUNDS_VERTICAL, SOUNDS_HORIZONTAL, SOUNDS_IN_NOTEBOOK, SOUNDS_IN_SEPARATE_WINDOWS} sound_style_t;
-enum {FOURIER, WAVELET, WALSH, AUTOCORRELATION, CEPSTRUM, HADAMARD, HAAR}; /* not typedef'd -- grows as new ones are added */
+enum {FOURIER, WAVELET, WALSH, AUTOCORRELATION, CEPSTRUM, HAAR}; /* not typedef'd -- grows as new ones are added */
 #define NUM_FFT_WINDOWS 19
 #define NUM_WAVELETS 48
 
-typedef enum {APPLY_TO_SOUND, APPLY_TO_CHANNEL, APPLY_TO_SELECTION} snd_apply_t;
 typedef enum {FCP_X_ANGLE, FCP_X_SCALE, FCP_Y_ANGLE, FCP_Y_SCALE, FCP_Z_ANGLE, FCP_Z_SCALE, FCP_CUTOFF, FCP_START, FCP_BETA, FCP_BEATS} fcp_t;
 typedef enum {TIME_AXIS_INFO, TRANSFORM_AXIS_INFO, LISP_AXIS_INFO} axis_info_t;
 typedef enum {COLOR_POSITION, COLOR_ZOOM} slider_choice_t;
 typedef enum {MINI_OFF, MINI_CURSOR, MINI_FIND, MINI_PROMPT, MINI_REPORT, MINI_USER} minibuffer_choice_t;
 
-typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET, 
-	      PLAY_STOP_CALLED, PLAY_C_G, PLAY_NO_CHANNEL, PLAY_DISABLED, 
+typedef enum {PLAY_COMPLETE, PLAY_CLOSE, PLAY_BUTTON_UNSET, PLAY_STOP_CALLED, PLAY_C_G, PLAY_NO_CHANNEL,
 	      PLAY_ERROR, PLAY_APPLY, PLAY_EDIT, PLAY_C_T} play_stop_t;
 
 #define DEFAULT_MIN_DB -60.0
@@ -386,10 +419,6 @@ typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET,
 #define set_dac_combines_channels(a) ss->Dac_Combines_Channels = a
 #define DEFAULT_DAC_COMBINES_CHANNELS true
 
-#define emacs_style_save_as(ss) ss->Emacs_Style_Save_As
-#define set_emacs_style_save_as(a) ss->Emacs_Style_Save_As = a
-#define DEFAULT_EMACS_STYLE_SAVE_AS false
-
 #define max_regions(ss) ss->Max_Regions
 #define in_set_max_regions(a) ss->Max_Regions = a
 #define DEFAULT_MAX_REGIONS 16
@@ -454,6 +483,10 @@ typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET,
 #define set_trap_segfault(a) ss->Trap_Segfault = a
 #define DEFAULT_TRAP_SEGFAULT true
 
+#define log_freq_start(ss) ss->Log_Freq_Start
+#define in_set_log_freq_start(a) ss->Log_Freq_Start = a
+#define DEFAULT_LOG_FREQ_START 32.0
+
 #define optimization(ss) ss->Optimization
 #define set_optimization(a) ss->Optimization = a
 #define DEFAULT_OPTIMIZATION 0
@@ -462,6 +495,10 @@ typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET,
 #define in_set_dot_size(a) ss->Dot_Size = a
 #define DEFAULT_DOT_SIZE 1
 
+#define grid_density(ss) ss->Grid_Density
+#define in_set_grid_density(a) ss->Grid_Density = a
+#define DEFAULT_GRID_DENSITY 1.0
+
 #define minibuffer_history_length(ss) ss->Minibuffer_History_Length
 #define set_minibuffer_history_length(a) ss->Minibuffer_History_Length = a
 #define DEFAULT_MINIBUFFER_HISTORY_LENGTH 8
@@ -717,7 +754,11 @@ typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET,
 
 #define show_grid(ss) ss->Show_Grid
 #define in_set_show_grid(a) ss->Show_Grid = a
-#define DEFAULT_SHOW_GRID false
+#define DEFAULT_SHOW_GRID NO_GRID
+
+#define show_sonogram_cursor(ss) ss->Show_Sonogram_Cursor
+#define in_set_show_sonogram_cursor(a) ss->Show_Sonogram_Cursor = a
+#define DEFAULT_SHOW_SONOGRAM_CURSOR false
 
 #define show_axes(ss) ss->Show_Axes
 #define in_set_show_axes(a) ss->Show_Axes = a
@@ -837,4 +878,12 @@ typedef enum {PLAY_COMPLETE, PLAY_EXIT, PLAY_CLOSE, PLAY_BUTTON_UNSET,
 #define set_mix_tag_height(a) ss->Mix_Tag_Height = a
 #define DEFAULT_MIX_TAG_HEIGHT 14
 
+#define mark_tag_width(ss) ss->Mark_Tag_Width
+#define set_mark_tag_width(a) ss->Mark_Tag_Width = a
+#define DEFAULT_MARK_TAG_WIDTH 10
+
+#define mark_tag_height(ss) ss->Mark_Tag_Height
+#define set_mark_tag_height(a) ss->Mark_Tag_Height = a
+#define DEFAULT_MARK_TAG_HEIGHT 4
+
 #endif
