@@ -101,15 +101,14 @@ typedef struct snd_fd {
   Float fscaler, rscaler;
   int iscaler;
   off_t frag_pos;
-  double incr, curval;
-  void *ptree, *ptree1;
-  XEN xen_code, closure, closure1;
-  int protect, protect1;
-  double incr2, curval2, incr3, curval3, incr4, curval4;
+  void *ptree, *ptree2, *ptree3;
+  XEN closure, closure2, closure3;
+  int protect, protect2, protect3;
+  double incr, curval, incr2, curval2, incr3, curval3, incr4, curval4;
+  bool zero, xramp2;
+  int edit_ctr, dangling_loc, region;
   mus_sample_t (*rev_run)(struct snd_fd *sf);
   Float (*rev_runf)(struct snd_fd *sf);
-  int edit_ctr, dangling_loc, region;
-  bool zero, xramp2;
   Float (*rampf)(struct snd_fd *sf);
   Float (*rev_rampf)(struct snd_fd *sf);
 } snd_fd;
@@ -204,11 +203,11 @@ typedef struct chan_info {
   bool graph_lisp_p;       /* is lisp graph active */
   void *lisp_info;
   bool cursor_on;          /* channel's cursor */
-  bool cursor_visible;     /* for XOR decisions */
+  bool cursor_visible, fft_cursor_visible;     /* for XOR decisions */
   off_t *cursors;          /* sample number (follows edit history) */
   int cursor_size;
   cursor_style_t cursor_style;
-  int cx, cy;               /* graph-relative cursor loc (for XOR) */
+  int cx, cy, fft_cx;      /* graph-relative cursor loc (for XOR) */
   int edit_ctr;            /* channel's edit history */
   int edit_size;           /* current edit list size */
   ed_list **edits;         /* the edit list */
@@ -218,11 +217,11 @@ typedef struct chan_info {
   int ptree_size;          /* ditto for ptrees */
   int ptree_ctr;
   void **ptrees;
-  XEN *ptree_inits, *xens;
-  int *init_locs, *xen_locs;
+  XEN *ptree_inits;
+  int *init_locs;
   void *enved_spectra;
   fft_info *fft;           /* possibly null fft data */
-  struct snd_info *sound; /* containing sound */
+  struct snd_info *sound;  /* containing sound */
   axis_info *axis;         /* time domain axis */
   mark ***marks;           /* current marks, indexed by edit_ctr, then mark_number, then the mark pointer */
   int marks_size;
@@ -233,20 +232,23 @@ typedef struct chan_info {
   env_info **amp_envs;
   void *sonogram_data;
   void *last_sonogram, *temp_sonogram;
+  bool show_sonogram_cursor;
   void *fft_data;          /* parallels sonogram -- try to avoid repeating large ffts needlessly */
-  bool printing;
+  printing_t printing;
   fft_change_t fft_changed;
   Float gsy, gzy;
   void *mix_dragging;
-  int height, mixes;
+  int height;
+  bool have_mixes;
   off_t original_cursor, original_left_sample, original_window_size;   /* for cursor reset after cursor-moving play */
   with_hook_t hookable;
   int selection_transform_size;
   bool squelch_update, previous_squelch_update, waiting_to_make_graph, in_as_one_edit;
   /* moved from global to channel-local 4-Aug-00 */
   Float spectro_x_scale, spectro_y_scale, spectro_z_scale, spectro_z_angle, spectro_x_angle, spectro_y_angle, spectro_cutoff, spectro_start;
-  Float lin_dB, min_dB, fft_window_beta, beats_per_minute;
-  bool show_y_zero, show_marks, verbose_cursor, show_grid;
+  Float lin_dB, min_dB, fft_window_beta, beats_per_minute, grid_density;
+  bool show_y_zero, show_marks, verbose_cursor;
+  with_grid_t show_grid;
   int wavo_hop, wavo_trace, zero_pad, wavelet_type, max_transform_peaks;
   x_axis_style_t x_axis_style;
   show_axes_t show_axes;
@@ -269,7 +271,7 @@ typedef struct chan_info {
   Locus old_x0, old_x1;
   Float *amp_control; /* local amp controls in snd-dac; should it be extended to other controls? */
   search_result_t last_search_result;
-  bool just_zero, new_peaks;
+  bool just_zero, new_peaks, editable;
   track_info **tracks;
 #if HAVE_GL
   int gl_fft_list;
@@ -320,7 +322,8 @@ typedef struct snd_info {
   off_t marking;
   int searching, amping;
   sp_filing_t filing;
-  bool prompting, loading, finding_mark, printing, selectioning;
+  bool prompting, loading, finding_mark, selectioning;
+  printing_t printing;
   off_t macroing;
   minibuffer_choice_t minibuffer_on;
   bool read_only;
@@ -374,7 +377,8 @@ typedef struct snd_state {
   Latus init_window_width, init_window_height;
   Locus init_window_x, init_window_y;
   bool graph_hook_active, lisp_graph_hook_active;
-  bool Show_Transform_Peaks, Show_Y_Zero, Show_Marks, Show_Grid;
+  bool Show_Transform_Peaks, Show_Y_Zero, Show_Marks;
+  with_grid_t Show_Grid;
   bool Fft_Log_Frequency, Fft_Log_Magnitude;
   channel_style_t Channel_Style;
   sound_style_t Sound_Style;
@@ -383,7 +387,7 @@ typedef struct snd_state {
   char *Listener_Font, *Axis_Label_Font, *Axis_Numbers_Font, *Tiny_Font, *Peaks_Font, *Bold_Peaks_Font;
   bool Verbose_Cursor, Trap_Segfault;
   int Enved_Filter_Order;
-  Float Vu_Size, Vu_Font_Size, Eps_Left_Margin, Eps_Bottom_Margin, Eps_Size;
+  Float Vu_Size, Vu_Font_Size, Eps_Left_Margin, Eps_Bottom_Margin, Eps_Size, Log_Freq_Start;
   char *Vu_Font;
   Float Spectro_X_Scale, Spectro_Y_Scale, Spectro_Z_Scale, Spectro_Z_Angle, Spectro_X_Angle, Spectro_Y_Angle, Spectro_Cutoff, Spectro_Start;
   int Default_Output_Type, Default_Output_Format, Default_Output_Chans, Default_Output_Srate;
@@ -393,7 +397,7 @@ typedef struct snd_state {
   mus_fft_window_t Fft_Window;
   graph_type_t Transform_Graph_Type, Time_Graph_Type;
   bool Ask_Before_Overwrite;
-  Float Fft_Window_Beta;
+  Float Fft_Window_Beta, Grid_Density;
   Float Color_Scale, Color_Cutoff, Beats_Per_Minute;
   bool Color_Inverted, Show_Mix_Waveforms;
   int Mix_Waveform_Height;
@@ -405,19 +409,19 @@ typedef struct snd_state {
   bool Auto_Resize, Auto_Update;
   int Max_Regions, Max_Transform_Peaks;
   int Audio_Output_Device, Audio_Input_Device;
-  bool Show_Backtrace, Emacs_Style_Save_As, With_GL, With_Relative_Panes;
+  bool Show_Backtrace, With_GL, With_Relative_Panes;
   int Print_Length, Dac_Size, Previous_Files_Sort;
   bool Dac_Combines_Channels, Show_Selection_Transform, With_Mix_Tags, Selection_Creates_Region;
   char *Save_State_File, *Listener_Prompt;
   Float Enved_Base, Enved_Power, Auto_Update_Interval;
   bool Enved_Wave_p, Graphs_Horizontal, With_Background_Processes;
   env_type_t Enved_Style;
-  int Graph_Cursor, Mix_Tag_Width, Mix_Tag_Height, Minibuffer_History_Length;
+  int Graph_Cursor, Mix_Tag_Width, Mix_Tag_Height, Mark_Tag_Height, Mark_Tag_Width, Minibuffer_History_Length;
   enved_target_t Enved_Target;
   bool Data_Clipped, Show_Indices;
   int Cursor_Size;
   cursor_style_t Cursor_Style;
-  bool Filter_Control_In_Db, Filter_Control_In_Hz;
+  bool Filter_Control_In_Db, Filter_Control_In_Hz, Show_Sonogram_Cursor;
   int Speed_Control_Tones;
   speed_style_t Speed_Control_Style;
   Float Expand_Control_Length, Expand_Control_Ramp, Expand_Control_Hop, Expand_Control_Jitter;
@@ -478,7 +482,7 @@ int snd_open_read(const char *arg);
 int snd_reopen_write(const char *arg);
 int snd_write_header(const char *name, int type, int srate, int chans, off_t loc, off_t samples, int format, const char *comment, int len, int *loops);
 bool snd_overwrite_ok(const char *ofile);
-snd_io *make_file_state(int fd, file_info *hdr, int chan, int suggested_bufsize);
+snd_io *make_file_state(int fd, file_info *hdr, int chan, off_t beg, int suggested_bufsize);
 void file_buffers_forward(off_t ind0, off_t ind1, off_t indx, snd_fd *sf, snd_data *cur_snd);
 void file_buffers_back(off_t ind0, off_t ind1, off_t indx, snd_fd *sf, snd_data *cur_snd);
 int snd_remove(const char *name, cache_remove_t forget);
@@ -487,7 +491,7 @@ int snd_fclose(FILE *fd, const char *name);
 void remember_temp(const char *filename, int chans);
 void forget_temps(void);
 snd_data *make_snd_data_file(const char *name, snd_io *io, file_info *hdr, file_delete_t temp, int ctr, int temp_chan);
-snd_data *copy_snd_data(snd_data *sd, int bufsize);
+snd_data *copy_snd_data(snd_data *sd, off_t beg, int bufsize);
 snd_data *free_snd_data(snd_data *sf);
 snd_data *make_snd_data_buffer(mus_sample_t *data, int len, int ctr);
 snd_data *make_snd_data_buffer_for_simple_channel(int len);
@@ -719,6 +723,8 @@ bool map_over_sounds (bool (*func)(snd_info *, void *), void *userptr);
 bool map_over_chans (bool (*func)(chan_info *, void *), void *userptr);
 void for_each_chan(void (*func)(chan_info *));
 void for_each_chan_1(void (*func)(chan_info *, void *), void *userptr);
+void for_each_normal_chan(void (*func)(chan_info *));
+void for_each_normal_chan_1(void (*func)(chan_info *, void *), void *userptr);
 bool map_over_sound_chans (snd_info *sp, bool (*func)(chan_info *, void *), void *userptr);
 void for_each_sound_chan(snd_info *sp, void (*func)(chan_info *));
 void for_each_sound(void (*func)(snd_info *, void *), void *userptr);
@@ -761,16 +767,20 @@ void as_one_edit(chan_info *cp, int one_edit, const char *one_edit_origin);
 void free_sound_list (chan_info *cp);
 void free_ptree_list(chan_info *cp);
 void release_dangling_readers(chan_info *cp, int edit_ctr);
-bool extend_with_zeros(chan_info *cp, off_t beg, off_t num, const char *origin, int edpos);
-bool extend_edit_list(chan_info *cp, const char *origin, int edpos);
+void after_edit(chan_info *cp);
+bool extend_with_zeros(chan_info *cp, off_t beg, off_t num, int edpos);
+bool extend_edit_list(chan_info *cp, int edpos);
 bool file_insert_samples(off_t beg, off_t num, char *tempfile, chan_info *cp, int chan, file_delete_t auto_delete, const char *origin, int edpos);
-bool delete_samples(off_t beg, off_t num, chan_info *cp, const char *origin, int edpos);
+bool delete_samples(off_t beg, off_t num, chan_info *cp, int edpos);
 bool change_samples(off_t beg, off_t num, mus_sample_t *vals, chan_info *cp, lock_mix_t lock, const char *origin, int edpos);
 bool file_change_samples(off_t beg, off_t num, char *tempfile, chan_info *cp, int chan, 
 			 file_delete_t auto_delete, lock_mix_t lock, const char *origin, int edpos);
+bool file_mix_change_samples(off_t beg, off_t num, char *tempfile, chan_info *cp, int chan, 
+			     file_delete_t auto_delete, lock_mix_t lock, const char *origin, int edpos, bool with_mix);
 bool file_override_samples(off_t num, char *tempfile, chan_info *cp, int chan, 
 			   file_delete_t auto_delete, lock_mix_t lock, const char *origin);
 Float chn_sample(off_t samp, chan_info *cp, int pos);
+void check_saved_temp_file(const char *type, XEN filename, XEN date_and_length);
 bool editable_p(chan_info *cp);
 snd_fd *free_snd_fd(snd_fd *sf);
 char *sf_to_string(snd_fd *fd);
@@ -779,28 +789,29 @@ bool sf_p(XEN obj);
 snd_fd *get_sf(XEN obj);
 snd_fd *free_snd_fd_almost(snd_fd *sf);
 bool scale_channel(chan_info *cp, Float scaler, off_t beg, off_t num, int pos, bool in_as_one_edit);
+bool scale_channel_with_origin(chan_info *cp, Float scl, off_t beg, off_t num, int pos, bool in_as_one_edit, const char *origin);
 bool ramp_channel(chan_info *cp, Float rmp0, Float rmp1, off_t beg, off_t num, int pos, bool in_as_one_edit);
 bool xramp_channel(chan_info *cp, Float rmp0, Float rmp1, Float scaler, Float offset, 
 		   off_t beg, off_t num, int pos, bool in_as_one_edit, mus_any *e, int e_pos);
-bool ptree_channel(chan_info *cp, void *ptree, off_t beg, off_t num, int pos, bool env_it, XEN init_func, bool is_xen, XEN code);
+bool ptree_channel(chan_info *cp, void *ptree, off_t beg, off_t num, int pos, bool env_it, XEN init_func, const char *origin);
 snd_fd *init_sample_read(off_t samp, chan_info *cp, read_direction_t direction);
 snd_fd *init_sample_read_any(off_t samp, chan_info *cp, read_direction_t direction, int edit_position);
 void read_sample_change_direction(snd_fd *sf, read_direction_t dir);
-bool ramp_or_ptree_fragments_in_use(chan_info *cp, off_t beg, off_t dur, int pos, Float base);
+bool ramp_or_ptree_fragments_in_use(chan_info *cp, off_t beg, off_t dur, int pos, bool is_xramp);
 bool ptree_or_sound_fragments_in_use(chan_info *cp, int pos);
-bool ptree_fragments_in_use(chan_info *cp, off_t beg, off_t dur, int pos, bool is_xen);
+bool ptree_fragments_in_use(chan_info *cp, off_t beg, off_t dur, int pos);
 #define read_sample(Sf) (*((Sf)->run))(Sf)
 #define read_sample_to_float(Sf) (*((Sf)->runf))(Sf)
 Float protected_next_sample_to_float(snd_fd *sf);
 Float protected_previous_sample_to_float(snd_fd *sf);
-Float local_maxamp(chan_info *cp, off_t beg, off_t num, int edpos);
+Float channel_local_maxamp(chan_info *cp, off_t beg, off_t num, int edpos);
 void undo_edit_with_sync(chan_info *cp, int count);
 void redo_edit_with_sync(chan_info *cp, int count);
 void undo_edit(chan_info *cp, int count);
 void redo_edit(chan_info *cp, int count);
-int save_channel_edits(chan_info *cp, char *ofile, XEN edpos, const char *caller, int arg_pos);
+int save_channel_edits(chan_info *cp, char *ofile, int pos);
 void save_edits(snd_info *sp, void *ptr);
-int save_edits_without_display(snd_info *sp, char *new_name, int type, int format, int srate, char *comment, XEN edpos, const char *caller, int arg_pos);
+int save_edits_without_display(snd_info *sp, char *new_name, int type, int format, int srate, char *comment, int pos);
 void revert_edits(chan_info *cp, void *ptr);
 off_t current_location(snd_fd *sf);
 void g_init_edits(void);
@@ -830,12 +841,11 @@ void sono_update(chan_info *cp);
 void set_spectro_cutoff_and_redisplay(Float val);
 void c_convolve(char *fname, Float amp, int filec, off_t filehdr, int filterc, off_t filterhdr, int filtersize,
 		 int fftsize, int filter_chans, int filter_chan, int data_size, snd_info *gsp, enved_progress_t from_enved, int ip, int total_chans);
-void *make_sonogram_state(chan_info *cp);
-void single_fft(chan_info *cp, bool dpy);
+void *make_sonogram_state(chan_info *cp, bool force_recalc);
+void single_fft(chan_info *cp, bool update_display, bool force_recalc);
 Cessate sonogram_in_slices(void *sono);
 char *added_transform_name(int type);
 void clear_transform_edit_ctrs(chan_info *cp);
-void make_fft_graph(chan_info *cp, axis_info *fap, axis_context *ax, with_hook_t with_hook);
 void g_init_fft(void);
 Float fft_beta_max(mus_fft_window_t win);
 void cp_free_fft_state(chan_info *cp);
@@ -870,7 +880,12 @@ XEN g_call_any(XEN proc, XEN arglist, const char *caller);
 char *procedure_ok(XEN proc, int args, const char *caller, const char *arg_name, int argn);
 bool procedure_ok_with_error(XEN proc, int req_args, const char *caller, const char *arg_name, int argn);
 bool procedure_arity_ok(XEN proc, int args);
-int snd_protect(XEN obj);
+#if DEBUGGING
+  int snd_protect_1(XEN obj, const char *caller);
+  #define snd_protect(Obj) snd_protect_1(Obj, c__FUNCTION__)
+#else
+  int snd_protect(XEN obj);
+#endif
 void snd_unprotect(XEN obj);
 void snd_unprotect_at(int loc);
 XEN run_or_hook (XEN hook, XEN args, const char *caller);
@@ -898,6 +913,7 @@ void clear_stdin(void);
 #if HAVE_RUBY
   void snd_rb_raise(XEN type, XEN info);
 #endif
+void set_log_freq_start(Float base);
 
 
 /* -------- snd-select.c -------- */
@@ -919,16 +935,16 @@ void start_selection_creation(chan_info *cp, off_t samp);
 void update_possible_selection_in_progress(off_t samp);
 int make_region_from_selection(void);
 void display_selection(chan_info *cp);
-bool delete_selection(const char *origin, cut_selection_regraph_t regraph);
+bool delete_selection(cut_selection_regraph_t regraph);
 void move_selection(chan_info *cp, int x);
 void finish_selection_creation(void);
 int select_all(chan_info *cp);
 int save_selection(char *ofile, int type, int format, int srate, const char *comment, int chan);
 bool selection_creation_in_progress(void);
 void cancel_selection_watch(void);
-void add_selection_or_region(int reg, chan_info *cp, const char *origin);
+void add_selection_or_region(int reg, chan_info *cp);
 void insert_selection_from_menu(void);
-void insert_selection_or_region(int reg, chan_info *cp, const char *origin);
+void insert_selection_or_region(int reg, chan_info *cp);
 
 void g_init_selection(void);
   
@@ -948,8 +964,8 @@ region_state *region_report(void);
 void free_region_state (region_state *r);
 int remove_region_from_list(int pos);
 int save_region(int n, char *ofile, int data_format);
-void paste_region(int n, chan_info *cp, const char *origin);
-void add_region(int n, chan_info *cp, const char *origin);
+void paste_region(int n, chan_info *cp);
+void add_region(int n, chan_info *cp);
 int define_region(sync_info *si, off_t *ends);
 snd_fd *init_region_read (off_t beg, int n, int chan, read_direction_t direction);
 void cleanup_region_temp_files(void);
@@ -971,8 +987,8 @@ env *copy_env(env *e);
 bool envs_equal(env *e1, env *e2);
 env *free_env(env *e);
 char *env_to_string(env *e);
-int find_env(const char *name);
 env *make_envelope(Float *env_buffer, int len);
+env *make_envelope_with_offset_and_scaler(Float *env_buffer, int len, Float offset, Float scaler);
 XEN envelope_base(XEN obj);
 env *window_env(env *e, off_t local_beg, off_t local_dur, off_t e_beg, off_t e_dur, Float maxx);
 env *multiply_envs(env *e1, env *e2, Float maxx);
@@ -984,10 +1000,10 @@ bool env_editor_button_press(env_editor *edp, int evx, int evy, Tempus time, env
 void env_editor_button_release(env_editor *edp, env *e);
 double env_editor_ungrf_y_dB(env_editor *edp, int y);
 void init_env_axes(axis_info *ap, const char *name, int x_offset, int ey0, int width, int height, 
-		   Float xmin, Float xmax, Float ymin, Float ymax, bool printing);
+		   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing);
 void env_editor_display_env(env_editor *edp, env *e, axis_context *ax, const char *name, 
-			    int x0, int y0, int width, int height, bool printing);
-void view_envs(int env_window_width, int env_window_height, bool printing);
+			    int x0, int y0, int width, int height, printing_t printing);
+void view_envs(int env_window_width, int env_window_height, printing_t printing);
 int hit_env(int xe, int ye, int env_window_width, int env_window_height);
 void prepare_enved_edit(env *new_env);
 void redo_env_edit(void);
@@ -998,7 +1014,7 @@ char *enved_all_names(int n);
 void set_enved_env_list_top(int n);
 env *enved_all_envs(int pos);
 void alert_envelope_editor(char *name, env *val);
-void enved_show_background_waveform(axis_info *ap, axis_info *gray_ap, bool apply_to_selection, bool show_fft, bool printing);
+void enved_show_background_waveform(axis_info *ap, axis_info *gray_ap, bool apply_to_selection, bool show_fft, printing_t printing);
 void save_envelope_editor_state(FILE *fd);
 char *env_name_completer(char *text);
 env *enved_next_env(void);
@@ -1023,16 +1039,14 @@ void cleanup_dac(void);
 void stop_playing_sound(snd_info *sp, play_stop_t reason);
 void stop_playing_sound_without_hook(snd_info *sp, play_stop_t reason);
 void stop_playing_sound_no_toggle(snd_info *sp, play_stop_t reason);
-void stop_playing_all_sounds_without_hook(play_stop_t reason);
 void stop_playing_all_sounds(play_stop_t reason);
 void stop_playing_region(int n, play_stop_t reason);
 void play_region(int n, play_process_t background);
 void play_region_1(int region, play_process_t background, XEN stop_proc);
-void play_channel(chan_info *cp, off_t start, off_t end, play_process_t background, XEN edpos, const char *caller, int arg_pos);
-void play_sound(snd_info *sp, off_t start, off_t end, play_process_t background, XEN edpos, const char *caller, int arg_pos);
-void play_channels(chan_info **cps, int chans, off_t *starts, off_t *ends, play_process_t background, 
-		   XEN edpos, const char *caller, int arg_pos, bool selection);
-void play_selection(play_process_t background, XEN edpos, const char *caller, int arg_pos);
+void play_channel(chan_info *cp, off_t start, off_t end, play_process_t background, int pos);
+void play_sound(snd_info *sp, off_t start, off_t end, play_process_t background, int pos);
+void play_channels(chan_info **cps, int chans, off_t *starts, off_t *ends, play_process_t background, int pos, bool selection);
+void play_selection(play_process_t background, int pos);
 void toggle_dac_pausing(void); /* snd-dac.c */
 bool play_in_progress(void);
 void initialize_apply(snd_info *sp, int chans, off_t beg, off_t frames);
@@ -1071,9 +1085,11 @@ void set_wavo_trace(int uval);
 void set_dot_size(int val);
 chan_info *virtual_selected_channel(chan_info *cp);
 void handle_cursor(chan_info *cp, kbd_cursor_t redisplay);
+void handle_cursor_with_sync(chan_info *cp, kbd_cursor_t redisplay);
 void chans_field(fcp_t field, Float val);
 void in_set_transform_graph_type(graph_type_t val);
 void in_set_fft_window(mus_fft_window_t val);
+void set_max_transform_peaks(int val);
 void combine_sound(snd_info *sp);
 void separate_sound(snd_info *sp);
 void superimpose_sound(snd_info *sp);
@@ -1125,6 +1141,7 @@ axis_context *selection_context (chan_info *cp);
 axis_context *mark_context (chan_info *cp);
 axis_context *mix_waveform_context (chan_info *cp);
 void calculate_fft(chan_info *cp);
+void set_min_db(Float db);
 
 
 /* -------- snd-axis.c -------- */
@@ -1134,7 +1151,8 @@ axis_context *free_axis_context(axis_context *ax);
 Locus grf_x(double val, axis_info *ap);
 Locus grf_y(Float val, axis_info *ap);
 void init_axis_scales(axis_info *ap);
-void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t axes, bool printing, bool show_x_axis);
+void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t axes, printing_t printing, 
+		 with_x_axis_t show_x_axis, with_grid_t grid, log_axis_t log_axes, Float grid_scale);
 
 #define ungrf_x(AP, X) (((X) - (AP)->x_base) / (AP)->x_scale)
 #define ungrf_y(AP, Y) (((Y) - (AP)->y_base) / (AP)->y_scale)
@@ -1175,24 +1193,22 @@ void save_controls(snd_info *sp);
 void restore_controls(snd_info *sp);
 void reset_controls(snd_info *sp);
 void stop_applying(snd_info *sp);
-void remove_apply(snd_info *sp);
-Cessate apply_controls(Indicium xp);
-void *make_apply_state_with_implied_beg_and_dur(void *xp);
+void menu_apply_controls(snd_info *sp);
+void menu_reset_controls(snd_info *sp);
 env_info *env_on_env(env *e, chan_info *cp);
 void amp_env_env(chan_info *cp, Float *brkpts, int npts, int pos, Float base, Float scaler, Float offset);
 void amp_env_env_selection_by(chan_info *cp, mus_any *e, off_t beg, off_t num, int pos);
-void amp_env_ptree(chan_info *cp, void *pt, int pos, XEN init_func, bool is_xen, XEN code);
-void amp_env_ptree_selection(chan_info *cp, void *pt, off_t beg, off_t num, int pos, XEN init_func, bool is_xen, XEN code);
+void amp_env_ptree(chan_info *cp, void *pt, int pos, XEN init_func);
+void amp_env_ptree_selection(chan_info *cp, void *pt, off_t beg, off_t num, int pos, XEN init_func);
 void amp_env_insert_zeros(chan_info *cp, off_t beg, off_t num, int pos);
 snd_info *snd_new_file(char *newname, int header_type, int data_format, int srate, int chans, char *new_comment, off_t samples);
-#if HAVE_SCM_MAKE_RATIO
+#if (HAVE_SCM_MAKE_RATIO || HAVE_SCM_C_MAKE_RECTANGULAR)
   void snd_rationalize(Float a, int *num, int *den);
 #endif
 
 void g_init_snd(void);
 XEN snd_no_such_sound_error(const char *caller, XEN n);
 
-void set_speed_style(speed_style_t val);
 void amp_env_scale_by(chan_info *cp, Float scl, int pos);
 void amp_env_scale_selection_by(chan_info *cp, Float scl, off_t beg, off_t num, int pos);
 env_info *amp_env_copy(chan_info *cp, bool reversed, int edpos);
@@ -1272,6 +1288,8 @@ void set_fallback_srate(int sr);
 void set_fallback_chans(int ch);
 void set_fallback_format(int fr);
 
+void run_after_save_as_hook(snd_info *sp, const char *already_saved_as_name, bool from_save_as_dialog);
+
 void g_init_file(void);
 void initialize_format_lists(void);
 
@@ -1296,7 +1314,7 @@ char *shorter_tempnam(const char *dir, const char *prefix);
 char *snd_tempnam(void);
 void snd_exit(int val);
 void g_init_utils(void);
-#ifdef DEBUG_MEMORY
+#ifdef DEBUGGING
   void set_encloser(char *name);
 #endif
 #if DEBUGGING && HAVE_CLOCK
@@ -1327,7 +1345,7 @@ mix_context *make_mix_context(chan_info *cp);
 mix_context *free_mix_context(mix_context *ms);
 void free_mix_list(chan_info *cp);
 void free_mixes(chan_info *cp);
-void mix_complete_file_at_cursor(snd_info *sp, char *str, const char *origin, bool with_tag, int track_id);
+void mix_complete_file_at_cursor(snd_info *sp, char *str, bool with_tag, int track_id);
 int mix_file(off_t beg, off_t num, int chans, chan_info **cps, char *mixinfile, file_delete_t temp, const char *origin, bool with_tag, int track_id);
 void backup_mix_list(chan_info *cp, int edit_ctr);
 bool active_mix_p(chan_info *cp);
@@ -1411,12 +1429,14 @@ void *get_mf(XEN obj);
 char *run_mix_reader_to_string(void *ptr);
 void run_free_mix_fd(void *ptr);
 void *run_make_mix_sample_reader(int id, off_t beg);
+bool mix_sample_reader_at_end_p(void *md);
 Float track_read_sample_to_float(void *ptr);
 bool tf_p(XEN obj);
 void *get_tf(XEN obj);
 char *run_track_reader_to_string(void *ptr);
 void run_free_track_fd(void *ptr);
 void *run_make_track_sample_reader(int id, int chan, off_t beg);
+bool track_sample_reader_at_end_p(void *rd);
 bool mix_dialog_mix_inverted(int id);
 void mix_dialog_set_mix_inverted(int id, bool on);
 
@@ -1431,6 +1451,10 @@ XEN g_track_sample_reader_at_end_p(XEN obj);
 XEN g_track_sample_reader_position(XEN obj);
 XEN g_free_track_sample_reader(XEN obj);
 
+#if HAVE_GUILE
+char *edit_list_mix_and_track_init(chan_info *cp);
+#endif
+
 
 
 /* -------- snd-find.c -------- */
@@ -1490,21 +1514,21 @@ void g_init_kbd(void);
 
 void scale_by(chan_info *cp, Float *scalers, int len, bool selection);
 bool scale_to(snd_info *sp, chan_info *cp, Float *scalers, int len, bool selection);
-Float get_maxamp(snd_info *sp, chan_info *cp, int edpos);
+Float channel_maxamp(chan_info *cp, int edpos);
 src_state *make_src(Float srate, snd_fd *sf, Float initial_srate);
 Float src_input_as_needed(void *arg, int dir);
 src_state *free_src(src_state *sr);
 void src_env_or_num(chan_info *cp, env *e, Float ratio, bool just_num, 
 		    enved_progress_t from_enved, const char *origin, bool over_selection, mus_any *gen, XEN edpos, int arg_pos);
-void apply_filter(chan_info *ncp, int order, env *e, enved_progress_t from_enved, const char *origin, 
+void apply_filter(chan_info *ncp, int order, env *e, enved_progress_t from_enved, const char *caller, const char *origin, 
 		  bool over_selection, Float *ur_a, mus_any *gen, XEN edpos, int arg_pos, bool truncate);
 void apply_env(chan_info *cp, env *e, off_t beg, off_t dur, bool over_selection, 
 	       enved_progress_t from_enved, const char *origin, mus_any *gen, XEN edpos, int arg_pos);
-void cos_smooth(chan_info *cp, off_t beg, off_t num, bool over_selection, const char *origin);
+void cos_smooth(chan_info *cp, off_t beg, off_t num, bool over_selection);
 void display_frequency_response(env *e, axis_info *ap, axis_context *gax, int order, bool dBing);
-void cursor_delete(chan_info *cp, off_t count, const char *origin);
+void cursor_delete(chan_info *cp, off_t count);
 void cursor_zeros(chan_info *cp, off_t count, bool over_selection);
-void cursor_insert(chan_info *cp, off_t beg, off_t count, const char *origin);
+void cursor_insert(chan_info *cp, off_t beg, off_t count);
 
 void g_init_sig(void);
 int to_c_edit_position(chan_info *cp, XEN edpos, const char *caller, int arg_pos);
