@@ -14,6 +14,7 @@ typedef struct {
   Locus min_label_x, max_label_x;
   Latus maj_tick_len, min_tick_len;
   char *min_label, *max_label;
+  Float grid_scale;
 } tick_descriptor;
 
 static tick_descriptor *free_tick_descriptor (tick_descriptor *td)
@@ -27,9 +28,9 @@ static tick_descriptor *free_tick_descriptor (tick_descriptor *td)
   return(NULL);
 }
 
-static tick_descriptor *describe_ticks(tick_descriptor *gd_td, double lo, double hi, int max_ticks)
+static tick_descriptor *describe_ticks(tick_descriptor *gd_td, double lo, double hi, int max_ticks, Float grid_scale)
 {
-  /* given absolute (unchangeable) axis bounds lo and hi, and abolute maximum number of ticks to use, find a "pretty" tick placement */
+  /* given absolute (unchangeable) axis bounds lo and hi, and maximum number of ticks to use, find a "pretty" tick placement */
   /* much of the work here involves floating point rounding problems.  We assume the tick labeller will round as well */
   tick_descriptor *td;
   int ten, hib, lob, offset = 0;
@@ -44,11 +45,13 @@ static tick_descriptor *describe_ticks(tick_descriptor *gd_td, double lo, double
       td = gd_td;
       if ((td->hi == hi) && 
 	  (td->lo == lo) && 
-	  (td->max_ticks == max_ticks)) 
+	  (td->max_ticks == max_ticks) &&
+	  ((fabs(td->grid_scale - grid_scale)) < .01))
 	return(td);
     }
   td->hi = hi;
   td->lo = lo;
+  td->grid_scale = grid_scale;
   hilo_diff = hi - lo;
   if (hilo_diff < .001) 
     {
@@ -94,7 +97,7 @@ static tick_descriptor *describe_ticks(tick_descriptor *gd_td, double lo, double
   flog10 = floor(log10(mfdiv));
   plog10 = pow(10, flog10);
   td->tens = (int)fabs(flog10);
-  mten = (double)(floor(4.0 * (.00001 + (mfdiv / plog10)))) / 4.0;
+  mten = grid_scale * (double)(floor(4.0 * (.00001 + (mfdiv / plog10)))) / 4.0;
   if (mten < 1.0) mten = 1.0;
   if ((mten == 1.0) || (mten == 2.0) || (mten == 2.5) || (mten == 5.0)) ften = mten;
   else if (mten < 2.0) ften = 2.0;
@@ -214,13 +217,8 @@ static void activate_gl_fonts(void)
     {
       label_base = glGenLists(128);
       number_base = glGenLists(128);
-#if HAVE_GDK_GL_FONT_USE_GDK_FONT
-      gdk_gl_font_use_gdk_font(gdk_font_from_description(AXIS_LABEL_FONT(ss)), 32, 96, label_base + 32);
-      gdk_gl_font_use_gdk_font(gdk_font_from_description(AXIS_NUMBERS_FONT(ss)), 32, 96, number_base + 32);
-#else
       gdk_gl_font_use_pango_font(AXIS_LABEL_FONT(ss), 32, 96, label_base + 32);
       gdk_gl_font_use_pango_font(AXIS_NUMBERS_FONT(ss), 32, 96, number_base + 32);
-#endif
       gl_fonts_activated = true;
     }
 #endif
@@ -242,11 +240,7 @@ void reload_label_font(void)
     {
       glDeleteLists(label_base, 128);
       label_base = glGenLists(128);
-#if HAVE_GDK_GL_FONT_USE_GDK_FONT
-      gdk_gl_font_use_gdk_font(gdk_font_from_description(AXIS_LABEL_FONT(ss)), 32, 96, label_base + 32);
-#else
       gdk_gl_font_use_pango_font(AXIS_LABEL_FONT(ss), 32, 96, label_base + 32);
-#endif
     }
 #endif
 }
@@ -267,11 +261,7 @@ void reload_number_font(void)
     {
       glDeleteLists(number_base, 128);
       number_base = glGenLists(128);
-#if HAVE_GDK_GL_FONT_USE_GDK_FONT
-      gdk_gl_font_use_gdk_font(gdk_font_from_description(AXIS_NUMBERS_FONT(ss)), 32, 96, number_base + 32);
-#else
       gdk_gl_font_use_pango_font(AXIS_NUMBERS_FONT(ss), 32, 96, number_base + 32);
-#endif
     }
 #endif
 }
@@ -307,19 +297,83 @@ static void draw_vertical_grid_line(int x, axis_info *ap, axis_context *ax)
   set_foreground_color(ap->cp, ax, old_color);
 }
 
-void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t axes, bool printing, bool show_x_axis)
+static void draw_label(char *label, int x, int y, axis_info *ap, axis_context *ax, printing_t printing)
+{
+  draw_string(ax, x, y, label, snd_strlen(label));
+  if (printing) 
+    ps_draw_string(ap, x, y, label);
+}
+
+static void draw_vertical_tick(int x, int y0, int y1, axis_info *ap, axis_context *ax, printing_t printing, bool include_grid)
+{
+  draw_line(ax, x, y1, x, y0);
+  if (printing) ps_draw_line(ap, x, y1, x, y0);
+  if (include_grid) draw_vertical_grid_line(x, ap, ax);
+}
+
+static void draw_horizontal_tick(int x0, int x1, int y, axis_info *ap, axis_context *ax, printing_t printing, bool include_grid)
+{
+  draw_line(ax, x0, y, x1, y);
+  if (printing) ps_draw_line(ap, x0, y, x1, y);
+  if (include_grid) draw_horizontal_grid_line(y, ap, ax);
+}
+
+static void draw_log_tick_label(char *label, int logx, int x_label_width, int right_border_width, axis_info *ap, axis_context *ax, printing_t printing)
+{
+  /* is there room for a label? */
+  /* the main label is at ap->x_label_x to that plus x_label_width */
+  int lx0, lx1, tx0, tx1, label_width;
+  lx0 = ap->x_label_x;
+  lx1 = lx0 + x_label_width;
+  label_width = number_width(label);
+  tx0 = (int)(logx - .45 * label_width);
+  if ((tx0 + label_width) > ap->x_axis_x1)
+    tx0 = (int)(logx - label_width + .75 * right_border_width);
+  tx1 = tx0 + label_width;
+  if ((lx0 > tx1) || (lx1 < tx0))
+    draw_label(label, tx0, ap->x_label_y + X_NUMBERS_OFFSET, ap, ax, printing);
+}
+
+static void set_numbers_font(axis_context *ax, printing_t printing)
+{
+#if USE_MOTIF
+  ax->current_font = ((XFontStruct *)(AXIS_NUMBERS_FONT(ss)))->fid;
+  XSetFont(ax->dp, ax->gc, ((XFontStruct *)(AXIS_NUMBERS_FONT(ss)))->fid);
+#else
+#if USE_GTK
+  ax->current_font = AXIS_NUMBERS_FONT(ss);
+#endif
+#endif
+  if (printing) ps_set_number_font();
+}
+
+static void set_labels_font(axis_context *ax, printing_t printing)
+{
+#if USE_MOTIF
+  ax->current_font = ((XFontStruct *)(AXIS_LABEL_FONT(ss)))->fid;
+  XSetFont(ax->dp, ax->gc, ((XFontStruct *)(AXIS_LABEL_FONT(ss)))->fid);
+#else
+  #if USE_GTK
+  ax->current_font = AXIS_LABEL_FONT(ss);
+  #endif
+#endif
+  if (printing)
+    ps_set_label_font();
+}
+
+
+void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t axes, printing_t printing, 
+		 with_x_axis_t show_x_axis, with_grid_t with_grid, log_axis_t log_axes, Float grid_scale)
 {
   Latus width, height;
-  double x_range, y_range, tens;
-  Latus axis_thickness, left_border_width, bottom_border_width, top_border_width, right_border_width;
-  Latus inner_border_width, tick_label_width;
+  Latus axis_thickness, left_border_width, bottom_border_width, top_border_width, right_border_width, inner_border_width;
   Latus major_tick_length, minor_tick_length, x_tick_spacing, y_tick_spacing;
   bool include_x_label, include_x_ticks, include_x_tick_labels, include_y_ticks, include_y_tick_labels, include_grid;
+  bool y_axis_linear = true, x_axis_linear = true;
   Latus x_label_width, x_label_height, x_number_height;
-  int num_ticks, majy, miny, majx, minx, x, y, tx, ty, x0, y0;
-  double fy, fx;
+  int num_ticks;
   tick_descriptor *tdx = NULL, *tdy = NULL;
-  Locus curx, cury, curdy;
+  Locus curx, cury;
   axis_context *ax;
 #if HAVE_GL
   Float xthick, ythick, xmajorlen, xminorlen, ymajorlen, yminorlen;
@@ -329,7 +383,7 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
   width = ap->width;
   height = ap->height;
   ap->graph_active = ((width > 4) || (height > 10));
-  include_grid = ((ap->cp) && (ap->cp->show_grid));
+  include_grid = ((ap->cp) && (with_grid));
 
   if ((axes == SHOW_NO_AXES) || (width < 40) || (height < 40) || (ap->xmax == 0.0))
     {
@@ -400,6 +454,8 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
       include_x_tick_labels = false;
       include_x_ticks = false;
     }
+  if (log_axes == WITH_LOG_X_AXIS) x_axis_linear = false;
+
   if ((axes != SHOW_X_AXIS) && (axes != SHOW_X_AXIS_UNLABELLED))
     {
       include_y_tick_labels = ((width > 100) && (height > 60));
@@ -410,10 +466,10 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
       include_y_tick_labels = false;
       include_y_ticks = false;
     }
+  if (log_axes == WITH_LOG_Y_AXIS) y_axis_linear = false;
 
   curx = left_border_width;
   cury = height - bottom_border_width;
-  
   x_number_height = number_height();
   x_label_height = 0;
   x_label_width = 0;
@@ -433,107 +489,121 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
     if (include_x_ticks) 
       x_label_height = label_height();
 
-  curdy = cury;
-  if (include_y_ticks)
+  if (y_axis_linear)
     {
-      /* figure out how long the longest tick label will be and make room for it */
-      /* values go from ap->y0 to ap->y1 */
-      /* basic tick spacing is tick_spacing pixels */
-      num_ticks = curdy / y_tick_spacing;
-      /* ticks start and stop at 10-based locations (i.e. sloppy axis bounds) */
-      /* so, given the current min..max, find the pretty min..max for ticks */
-      y_range = ap->y1 - ap->y0;
-      if (y_range <= 0.0)
+      if (include_y_ticks)
 	{
-	  if (ap->y0 != 0.0)
+	  double y_range;
+	  /* figure out how long the longest tick label will be and make room for it */
+	  /* values go from ap->y0 to ap->y1 */
+	  /* basic tick spacing is tick_spacing pixels */
+	  num_ticks = cury / y_tick_spacing;
+	  /* ticks start and stop at 10-based locations (i.e. sloppy axis bounds) */
+	  /* so, given the current min..max, find the pretty min..max for ticks */
+	  y_range = ap->y1 - ap->y0;
+	  if (y_range <= 0.0)
 	    {
-	      ap->y1 = ap->y0 * 1.25;
-	      y_range = ap->y0 * .25;
+	      if (ap->y0 != 0.0)
+		{
+		  ap->y1 = ap->y0 * 1.25;
+		  y_range = ap->y0 * .25;
+		}
+	      else
+		{
+		  ap->y1 = 1.0;
+		  y_range = 1.0;
+		}
 	    }
-	  else
+	  tdy = describe_ticks((tick_descriptor *)(ap->y_ticks), ap->y0, ap->y1, num_ticks, grid_scale);
+	  ap->y_ticks = tdy;
+	  if (include_y_tick_labels)
 	    {
-	      ap->y1 = 1.0;
-	      y_range = 1.0;
+	      Latus tick_label_width;
+	      if (tdy->min_label) 
+		{
+		  FREE(tdy->min_label); 
+		  tdy->min_label = NULL;
+		}
+	      tdy->min_label = prettyf(tdy->mlo, tdy->tens);
+	      tdy->min_label_width = number_width(tdy->min_label);
+	      
+	      if (tdy->max_label) 
+		{
+		  FREE(tdy->max_label); 
+		  tdy->max_label = NULL;
+		}
+	      tdy->max_label = prettyf(tdy->mhi, tdy->tens);
+	      tdy->max_label_width = number_width(tdy->max_label);
+	      tick_label_width = tdy->min_label_width;
+	      if (tick_label_width < tdy->max_label_width) 
+		tick_label_width = tdy->max_label_width;
+	      if (((curx + tick_label_width) > (int)(.61 * width)) || 
+		  ((4 * x_number_height) > height))
+		include_y_tick_labels = false;
+	      else curx += tick_label_width;
 	    }
+	  
+	  curx += major_tick_length;
+	  tdy->maj_tick_len = major_tick_length;
+	  tdy->min_tick_len = minor_tick_length;
+	  ap->y_axis_y1 = top_border_width;
 	}
-
-      tdy = describe_ticks((tick_descriptor *)(ap->y_ticks), ap->y0, ap->y1, num_ticks);
-      ap->y_ticks = tdy;
-      if (include_y_tick_labels)
-	{
-	  if (tdy->min_label) 
-	    {
-	      FREE(tdy->min_label); 
-	      tdy->min_label = NULL;
-	    }
-	  tdy->min_label = prettyf(tdy->mlo, tdy->tens);
-	  tdy->min_label_width = number_width(tdy->min_label);
-
-	  if (tdy->max_label) 
-	    {
-	      FREE(tdy->max_label); 
-	      tdy->max_label = NULL;
-	    }
-	  tdy->max_label = prettyf(tdy->mhi, tdy->tens);
-	  tdy->max_label_width = number_width(tdy->max_label);
-	  tick_label_width = tdy->min_label_width;
-	  if (tick_label_width < tdy->max_label_width) 
-	    tick_label_width = tdy->max_label_width;
-	  if (((curx + tick_label_width) > (int)(.61 * width)) || 
-	      ((4 * x_number_height) > height))
-	    include_y_tick_labels = false;
-	  else curx += tick_label_width;
-	}
-      
-      curx += major_tick_length;
-      tdy->maj_tick_len = major_tick_length;
-      tdy->min_tick_len = minor_tick_length;
-      ap->y_axis_y1 = top_border_width;
+      else ap->y_axis_y1 = 0;
     }
-  else ap->y_axis_y1 = 0;
-
+  else
+    {
+      /* log case */
+      if (include_y_tick_labels)
+	curx += number_width("10000");
+      curx += major_tick_length;
+    }
+  
   ap->x_axis_x1 = width - right_border_width;
   ap->x_axis_x0 = curx;
-  x_range = ap->x1 - ap->x0;
-  if (x_range <= 0)
-    {
-      if (ap->x0 != 0.0)
-	{
-	  ap->x1 = ap->x0 * 1.25;
-	  x_range = ap->x0 * .25;
-	}
-      else
-	{
-	  ap->x1 = .1;
-	  x_range = .1;
-	}
-    }
-  if (include_x_ticks) 
+  {
+    double x_range;
+    x_range = ap->x1 - ap->x0;
+    if (x_range <= 0)
+      {
+	if (ap->x0 != 0.0)
+	  {
+	    ap->x1 = ap->x0 * 1.25;
+	    x_range = ap->x0 * .25;
+	  }
+	else
+	  {
+	    ap->x1 = .1;
+	    x_range = .1;
+	  }
+      }
+  }
+  if ((x_axis_linear) && (include_x_ticks))
     {
       num_ticks = (ap->x_axis_x1 - curx) / x_tick_spacing;
       switch (x_style)
 	{
 	case X_AXIS_IN_SECONDS: 
-	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0, ap->x1, num_ticks); 
+	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0, ap->x1, num_ticks, grid_scale); 
 	  break;
 	case X_AXIS_IN_SAMPLES: 
-	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0 * srate, ap->x1 * srate, num_ticks); 
+	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0 * srate, ap->x1 * srate, num_ticks, grid_scale); 
 	  break;
 	case X_AXIS_IN_BEATS: 
 	  if (ap->cp)
 	    tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), 
 				 (ap->x0 * ap->cp->beats_per_minute / 60.0), 
 				 (ap->x1 * ap->cp->beats_per_minute / 60.0), 
-				 num_ticks); 
-	  else tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0, ap->x1, num_ticks); 
+				 num_ticks, grid_scale); 
+	  else tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0, ap->x1, num_ticks, grid_scale); 
 	  break;
 	case X_AXIS_AS_PERCENTAGE: 
-	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0 / ap->xmax, ap->x1 / ap->xmax, num_ticks); 
+	  tdx = describe_ticks((tick_descriptor *)(ap->x_ticks), ap->x0 / ap->xmax, ap->x1 / ap->xmax, num_ticks, grid_scale); 
 	  break;
 	}
       ap->x_ticks = tdx;
       if (include_x_tick_labels)
 	{
+	  Latus tick_label_width;
 	  if (tdx->min_label) 
 	    {
 	      FREE(tdx->min_label); 
@@ -588,16 +658,11 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
   if (ap->use_gl) activate_gl_fonts();
   ap->used_gl = ap->use_gl;
 #endif
+
+  /* x axis label */
   if (include_x_label)
     {
-#if USE_MOTIF
-      ax->current_font = ((XFontStruct *)(AXIS_LABEL_FONT(ss)))->fid;
-      XSetFont(ax->dp, ax->gc, ((XFontStruct *)(AXIS_LABEL_FONT(ss)))->fid);
-#else
-  #if USE_GTK
-      ax->current_font = AXIS_LABEL_FONT(ss);
-  #endif
-#endif
+      set_labels_font(ax, printing);
 #if HAVE_GL
       if (ap->use_gl)
 	{
@@ -612,12 +677,10 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 #endif
       draw_string(ax, ap->x_label_x, ap->x_label_y + X_LABEL_Y_OFFSET, ap->xlabel, snd_strlen(ap->xlabel));
       if (printing) 
-	{
-	  ps_set_label_font();
-	  ps_draw_string(ap, ap->x_label_x, ap->x_label_y + X_LABEL_Y_OFFSET, ap->xlabel);
-	}
+	ps_draw_string(ap, ap->x_label_x, ap->x_label_y + X_LABEL_Y_OFFSET, ap->xlabel);
     }
 
+  /* x axis */
   if (show_x_axis)
     {
 #if HAVE_GL
@@ -635,6 +698,8 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 #endif
       fill_rectangle(ax, ap->x_axis_x0, ap->x_axis_y0, (unsigned int)(ap->x_axis_x1 - ap->x_axis_x0), axis_thickness);
     }
+
+  /* y axis */
   if ((axes != SHOW_X_AXIS) && (axes != SHOW_X_AXIS_UNLABELLED))
     {
 #if HAVE_GL
@@ -652,30 +717,21 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 #endif
       fill_rectangle(ax, ap->y_axis_x0, ap->y_axis_y1, axis_thickness, (unsigned int)(ap->y_axis_y0 - ap->y_axis_y1));
     }
-  if ((include_y_tick_labels) || 
-      (include_x_tick_labels))
-#if USE_MOTIF
-    {
-      ax->current_font = ((XFontStruct *)(AXIS_NUMBERS_FONT(ss)))->fid;
-      XSetFont(ax->dp, ax->gc, ((XFontStruct *)(AXIS_NUMBERS_FONT(ss)))->fid);
-    }
-#else
-  #if USE_GTK
-    ax->current_font = AXIS_NUMBERS_FONT(ss);
-  #endif
-#endif
+
   if (printing) 
     {
       if (show_x_axis)
 	ps_fill_rectangle(ap, ap->x_axis_x0, ap->x_axis_y0, ap->x_axis_x1 - ap->x_axis_x0, axis_thickness);
       if ((axes != SHOW_X_AXIS) && (axes != SHOW_X_AXIS_UNLABELLED))
 	ps_fill_rectangle(ap, ap->y_axis_x0, ap->y_axis_y1, axis_thickness, ap->y_axis_y0 - ap->y_axis_y1);
-      if ((include_y_tick_labels) || 
-	  (include_x_tick_labels)) 
-	ps_set_number_font();
     }
 
-  if (include_y_tick_labels)
+  /* linear axis ticks/labels */
+  if ((include_y_tick_labels) || 
+      (include_x_tick_labels))
+    set_numbers_font(ax, printing);
+  
+  if ((y_axis_linear) && (include_y_tick_labels))
     {
 #if HAVE_GL
       if (ap->use_gl)
@@ -703,6 +759,7 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 #endif
 		  tdy->min_label,
 		  strlen(tdy->min_label));
+
       draw_string(ax,
 		  ap->y_axis_x0 - tdy->maj_tick_len - tdy->max_label_width - inner_border_width,
 #if USE_GTK
@@ -721,13 +778,14 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 		      ap->y_axis_x0 - tdy->maj_tick_len - tdy->min_label_width - inner_border_width,
 		      (int)(grf_y(tdy->mlo, ap) + .25 * x_number_height),
 		      tdy->min_label);
+
 	  ps_draw_string(ap,
 		      ap->y_axis_x0 - tdy->maj_tick_len - tdy->max_label_width - inner_border_width,
 		      (int)(grf_y(tdy->mhi, ap) + .5 * x_number_height),
 		      tdy->max_label);
 	}
     }
-  if (include_x_tick_labels)
+  if ((x_axis_linear) && (include_x_tick_labels))
     {
       int lx0, lx1, tx0, tx1;
       /* the label is at ap->x_label_x to that plus x_label_width */
@@ -749,16 +807,7 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_string(ax,
-		      tx0,
-		      ap->x_label_y + X_NUMBERS_OFFSET,
-		      tdx->min_label,
-		      strlen(tdx->min_label));
-	  if (printing) 
-	    ps_draw_string(ap,
-			   tx0,
-			   ap->x_label_y + X_NUMBERS_OFFSET,
-			   tdx->min_label);
+          draw_label(tdx->min_label, tx0, ap->x_label_y + X_NUMBERS_OFFSET, ap, ax, printing);
 	}
       tx0 = (int)(tick_grf_x(tdx->mhi, ap, x_style, srate) - (.45 * tdx->max_label_width)); /* try centered label first */
       if ((tx0 + tdx->max_label_width) > ap->x_axis_x1)
@@ -777,20 +826,13 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_string(ax,
-		      tx0,
-		      ap->x_label_y + X_NUMBERS_OFFSET,
-		      tdx->max_label,
-		      strlen(tdx->max_label));
-	  if (printing) 
-	    ps_draw_string(ap,
-			   tx0,
-			   ap->x_label_y + X_NUMBERS_OFFSET,
-			   tdx->max_label);
+          draw_label(tdx->max_label, tx0, ap->x_label_y + X_NUMBERS_OFFSET, ap, ax, printing);
 	}
     }
-  if (include_y_ticks)
+  if ((y_axis_linear) && (include_y_ticks))
     {
+      double fy, tens;
+      int ty, x0, majx, minx, x;
       /* start ticks at flo, go to fhi by step, major ticks at mlo mhi and intervals of tenstep surrounding */
       x0 = ap->y_axis_x0;
       majx = x0 - tdy->maj_tick_len;
@@ -809,9 +851,8 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	}
       else
 #endif
-      draw_line(ax, majx, ty, x0, ty);
-      if (printing) ps_draw_line(ap, majx, ty, x0, ty);
-      if (include_grid) draw_horizontal_grid_line(ty, ap, ax);
+
+      draw_horizontal_tick(majx, x0, ty, ap, ax, printing, include_grid);
       tens = 0.0;
       fy -= tdy->step;
       while (fy >= tdy->flo)
@@ -836,9 +877,7 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_line(ax, x, ty, x0, ty);
-	  if (printing) ps_draw_line(ap, x, ty, x0, ty);
-	  if (include_grid) draw_horizontal_grid_line(ty, ap, ax);
+          draw_horizontal_tick(x, x0, ty, ap, ax, printing, include_grid);
 	  fy -= tdy->step;
 	}
       tens = 0.0;
@@ -866,14 +905,14 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_line(ax, x, ty, x0, ty);
-	  if (printing) ps_draw_line(ap, x, ty, x0, ty);
-	  if (include_grid) draw_horizontal_grid_line(ty, ap, ax);
+          draw_horizontal_tick(x, x0, ty, ap, ax, printing, include_grid);
 	  fy += tdy->step;
 	}
     }
-  if (include_x_ticks)
+  if ((x_axis_linear) && (include_x_ticks))
     {
+      double fx, tens;
+      int tx, y0, majy, miny, y;
       /* start ticks at flo, go to fhi by step, major ticks at mlo mhi and intervals of tenstep surrounding */
       y0 = ap->x_axis_y0;
       majy = y0 + tdx->maj_tick_len;
@@ -892,9 +931,8 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	}
       else
 #endif
-      draw_line(ax, tx, majy, tx, y0);
-      if (printing) ps_draw_line(ap, tx, majy, tx, y0);
-      if (include_grid) draw_vertical_grid_line(tx, ap, ax);
+
+      draw_vertical_tick(tx, y0, majy, ap, ax, printing, include_grid);
       tens = 0.0;
       fx -= tdx->step;
       while (fx >= tdx->flo)
@@ -919,9 +957,7 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_line(ax, tx, y, tx, y0);
-	  if (printing) ps_draw_line(ap, tx, y, tx, y0);
-	  if (include_grid) draw_vertical_grid_line(tx, ap, ax);
+          draw_vertical_tick(tx, y0, y, ap, ax, printing, include_grid);
 	  fx -= tdx->step;
 	}
       tens = 0.0;
@@ -949,12 +985,157 @@ void make_axes_1(axis_info *ap, x_axis_style_t x_style, int srate, show_axes_t a
 	    }
 	  else
 #endif
-	  draw_line(ax, tx, y, tx, y0);
-	  if (printing) ps_draw_line(ap, tx, y, tx, y0);
-	  if (include_grid) draw_vertical_grid_line(tx, ap, ax);
+          draw_vertical_tick(tx, y0, y, ap, ax, printing, include_grid);
 	  fx += tdx->step;
 	}
     }
+
+  /* all linear axis stuff has been taken care of. check for log axes (assume fft context, not spectrogram so no GL) */
+  /* x axis overall label has already gone out */
+
+  if ((!x_axis_linear) && (show_x_axis))
+    {
+      double min_freq, max_freq;
+      Float minlx = 0.0, maxlx, fap_range, log_range, lscale = 1.0, curlx;
+      Locus logx;
+      int y0, majy, miny;
+      if (include_x_ticks)
+	{
+	  int i;
+	  char *label = NULL;
+	  Float freq = 0.0, freq10 = 0.0;
+	  /* get min (log-freq or spectro-start), max, add major ticks and brief labels, then if room add log-style minor ticks (100's, 1000's) */
+	  y0 = ap->x_axis_y0;
+	  majy = y0 + major_tick_length;
+	  miny = y0 + minor_tick_length;
+	  min_freq = ap->x0;
+	  max_freq = ap->x1;
+	  if (include_x_tick_labels)
+	    set_numbers_font(ax, printing);
+	  fap_range = max_freq - min_freq;
+	  if (min_freq > 1.0) minlx = log(min_freq); else minlx = 0.0;
+	  maxlx = log(max_freq);
+	  log_range = (maxlx - minlx);
+	  lscale = fap_range / log_range;
+	  for (i = 0; i < 3; i++)
+	    {
+	      switch (i)
+		{
+		case 0:
+		  label = "100";
+		  freq = 100.0;
+		  break;
+		case 1:
+		  label = "1000";
+		  freq = 1000.0;
+		  break;
+		case 2:
+		  label = "10000";
+		  freq = 10000.0;
+		  break;
+		}
+	      if ((min_freq <= freq) && (max_freq >= freq))
+		{
+		  /* draw major tick at freq */
+		  freq10 = freq / 10.0;
+		  logx = grf_x(min_freq + lscale * (log(freq) - minlx), ap);
+		  draw_vertical_tick(logx, y0, majy, ap, ax, printing, include_grid);	      
+		  draw_log_tick_label(label, logx, x_label_width, right_border_width, ap, ax, printing);
+		  if (width > 200)
+		    {
+		      curlx = snd_round(min_freq / freq10) * freq10;
+		      for (; curlx < freq; curlx += freq10)
+			{
+			  logx = grf_x(min_freq + lscale * (log(curlx) - minlx), ap);
+			  draw_vertical_tick(logx, y0, miny, ap, ax, printing, include_grid);	      
+			}
+		    }
+		}
+	    }
+	}
+    }
+
+  if ((!y_axis_linear) && (axes != SHOW_X_AXIS) && (axes != SHOW_X_AXIS_UNLABELLED))
+    {
+      double min_freq, max_freq;
+      Float minlx = 0.0, maxlx, fap_range, log_range, lscale = 1.0, curly;
+      Locus logy;
+      int x0, majx, minx;
+      if (include_y_ticks)
+	{
+	  int i;
+	  char *label = NULL;
+	  Float freq = 0.0, freq10 = 0.0;
+	  /* get min (log-freq or spectro-start), max, add major ticks and brief labels, then if room add log-style minor ticks (100's, 1000's) */
+	  x0 = ap->y_axis_x0;
+	  majx = x0 - major_tick_length;
+	  minx = x0 - minor_tick_length;
+	  min_freq = ap->y0;
+	  max_freq = ap->y1;
+	  if (include_y_tick_labels)
+	    set_numbers_font(ax, printing);
+	  fap_range = max_freq - min_freq;
+	  if (min_freq > 1.0) minlx = log(min_freq); else minlx = 0.0;
+	  maxlx = log(max_freq);
+	  log_range = (maxlx - minlx);
+	  lscale = fap_range / log_range;
+	  for (i = 0; i < 3; i++)
+	    {
+	      switch (i)
+		{
+		case 0:
+		  label = "100";
+		  freq = 100.0;
+		  break;
+		case 1:
+		  label = "1000";
+		  freq = 1000.0;
+		  break;
+		case 2:
+		  label = "10000";
+		  freq = 10000.0;
+		  break;
+		}
+	      if ((min_freq <= freq) && (max_freq >= freq))
+		{
+		  /* draw major tick at freq */
+		  freq10 = freq / 10.0;
+		  logy = grf_y(min_freq + lscale * (log(freq) - minlx), ap);
+		  draw_horizontal_tick(majx, x0, logy, ap, ax, printing, include_grid);	      
+		  if (include_y_tick_labels)
+		    {
+		      int label_width;
+		      label_width = number_width(label);
+		      draw_string(ax,
+				  ap->y_axis_x0 - major_tick_length - label_width - inner_border_width,
+#if USE_GTK
+				  (int)(logy - .5 * x_number_height),
+#else
+				  (int)(logy + .25 * x_number_height),
+#endif
+				  label,
+				  snd_strlen(label));
+		      
+		      if (printing) 
+			ps_draw_string(ap,
+				       ap->y_axis_x0 - major_tick_length - label_width - inner_border_width,
+				       (int)(logy + .25 * x_number_height),
+				       label);
+		    }
+		  if (height > 200)
+		    {
+		      curly = snd_round(min_freq / freq10) * freq10;
+		      for (; curly < freq; curly += freq10)
+			{
+			  logy = grf_y(min_freq + lscale * (log(curly) - minlx), ap);
+			  draw_horizontal_tick(minx, x0, logy, ap, ax, printing, include_grid);	      
+			}
+		    }
+		}
+	    }
+	}
+      
+    }
 }
 
 axis_info *make_axis_info (chan_info *cp, double xmin, double xmax, Float ymin, Float ymax, 
@@ -1103,7 +1284,6 @@ Returns actual (pixel) axis bounds -- a list (x0 y0 x1 y1)."
   GdkGC *gc;
 #endif
   XEN ref;
-  char *xlabel; 
   double x0 = 0.0, x1 = 1.0; 
   Float y0 = -1.0, y1 = 1.0; 
   x_axis_style_t x_style = X_AXIS_IN_SECONDS;
@@ -1127,7 +1307,6 @@ Returns actual (pixel) axis bounds -- a list (x0 y0 x1 y1)."
 #endif
   ref = XEN_LIST_REF(args, 2);
   XEN_ASSERT_TYPE(XEN_STRING_P(ref), ref, XEN_ARG_3, S_draw_axes, "a string");
-  xlabel = XEN_TO_C_STRING(XEN_LIST_REF(args, 2));
   if (len > 3) 
     {
       xx0 = XEN_LIST_REF(args, 3);
@@ -1172,6 +1351,9 @@ Returns actual (pixel) axis bounds -- a list (x0 y0 x1 y1)."
 #if USE_MOTIF
   ax->dp = XtDisplay(w);
   ax->wn = XtWindow(w);
+#if DEBUGGING
+  if (!(ax->wn)) {fprintf(stderr, "widget %p's window is null?", w); abort();}
+#endif
 #else
   ax->wn = w->window;
   ax->w = w;
@@ -1182,7 +1364,7 @@ Returns actual (pixel) axis bounds -- a list (x0 y0 x1 y1)."
   ap->ymax = y1;
   ap->y_ambit = y1 - y0;
   ap->x_ambit = x1 - x0;
-  ap->xlabel = copy_string(xlabel);
+  ap->xlabel = copy_string(XEN_TO_C_STRING(XEN_LIST_REF(args, 2)));
   ap->x0 = x0;
   ap->x1 = x1;
   ap->y0 = y0;
@@ -1194,7 +1376,7 @@ Returns actual (pixel) axis bounds -- a list (x0 y0 x1 y1)."
   ap->graph_x0 = 0;
   clear_window(ax);
   ax->gc = gc;
-  make_axes_1(ap, x_style, 1, axes, false, true);
+  make_axes_1(ap, x_style, 1, axes, NOT_PRINTING, WITH_X_AXIS, NO_GRID, WITH_LINEAR_AXES, grid_density(ss));
   val = XEN_CONS(C_TO_XEN_INT(ap->x_axis_x0),
 	 XEN_CONS(C_TO_XEN_INT(ap->y_axis_y0),
           XEN_CONS(C_TO_XEN_INT(ap->x_axis_x1),
@@ -1259,7 +1441,9 @@ XEN_ARGIFY_4(g_grf_y_w, g_grf_y)
 XEN_ARGIFY_4(g_ungrf_x_w, g_ungrf_x)
 XEN_ARGIFY_4(g_ungrf_y_w, g_ungrf_y)
 XEN_ARGIFY_3(g_axis_info_w, g_axis_info)
+#if (!USE_NO_GUI)
 XEN_VARGIFY(g_draw_axes_w, g_draw_axes)
+#endif
 XEN_ARGIFY_3(g_x_axis_label_w, g_x_axis_label)
 XEN_ARGIFY_4(g_set_x_axis_label_w, g_set_x_axis_label)
 #else
