@@ -18,7 +18,7 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
       cp = (chan_info *)CALLOC(1, sizeof(chan_info)); 
       cp->cgx = (chan_context *)CALLOC(1, sizeof(chan_context));
       (cp->cgx)->ax = (axis_context *)CALLOC(1, sizeof(axis_context));
-      cp->mixes = 0;
+      cp->have_mixes = false;
       cp->last_sonogram = NULL;
       cp->temp_sonogram = NULL;
 #if HAVE_GL
@@ -43,7 +43,10 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->tracks = NULL;
   cp->cursor_on = false;
   cp->cursor_visible = false;
+  cp->fft_cursor_visible = false;
+  cp->show_sonogram_cursor = show_sonogram_cursor(ss);
   cp->selection_visible = false;
+  cp->editable = true; /* must be unset (region display, variable display, etc) */
   cp->cursor_style = cursor_style(ss);
   cp->cursor_size = cursor_size(ss);
   cp->cursor_proc = XEN_UNDEFINED;
@@ -86,12 +89,13 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->transform_graph_style = graph_style(ss);
   cp->graphs_horizontal = graphs_horizontal(ss);
   cp->dot_size = dot_size(ss);
+  cp->grid_density = grid_density(ss);
   cp->x_axis_style = x_axis_style(ss);
   cp->beats_per_minute = beats_per_minute(ss);
   cp->show_axes = show_axes(ss);
   cp->graph_time_p = true; /* the default state (button is set when we start) */
   cp->graph_transform_p = false;
-  cp->printing = false;
+  cp->printing = NOT_PRINTING;
   cp->waiting_to_make_graph = false;
   cp->new_peaks = false;
   cp->amp_envs = NULL;
@@ -103,6 +107,7 @@ chan_info *make_chan_info(chan_info *cip, int chan, snd_info *sound)
   cp->gsy = 1.0;
   cp->cx = 0;
   cp->cy = 0;
+  cp->fft_cx = 0;
   cp->selection_transform_size = 0;
   cp->last_search_result = SEARCH_OK;
   if (cp->last_sonogram) 
@@ -132,7 +137,7 @@ static chan_info *free_chan_info(chan_info *cp)
   if (cp->fft) cp->fft = free_fft_info(cp->fft);
   cp_free_fft_state(cp);
   cp->graph_transform_p = false;
-  cp->printing = false;
+  cp->printing = NOT_PRINTING;
   cp->graph_time_p = true;
   release_dangling_readers(cp, -1);
   if (cp->samples) {FREE(cp->samples); cp->samples = NULL;}
@@ -147,6 +152,8 @@ static chan_info *free_chan_info(chan_info *cp)
   cp->sound = NULL;  /* a backpointer */
   cp->cursor_on = false;
   cp->cursor_visible = false;
+  cp->fft_cursor_visible = false;
+  cp->show_sonogram_cursor = false;
   cp->selection_visible = false;
   if (cp->amp_control)
     {
@@ -345,7 +352,6 @@ void free_snd_info(snd_info *sp)
   if (sp->sgx)
     {
       env_editor *edp;
-      if ((sp->sgx)->apply_in_progress) remove_apply(sp);
       edp = (env_editor *)(sp->sgx->flt);
       if (edp)
 	{
@@ -440,19 +446,20 @@ snd_info *completely_free_snd_info(snd_info *sp)
 bool map_over_chans(bool (*func)(chan_info *, void *), void *userptr)
 {
   /* argument to func is chan_info pointer+void pointer of user spec, return non-zero = abort map, skips inactive sounds */
+  /* currently used only in cases where only normal sounds are expected */
   int i, j;
-  bool val;
-  snd_info *sp;
-  chan_info *cp;
-  val = false;
+  bool val = false;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
+      chan_info *cp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	for (j = 0; j < sp->nchans; j++)
 	  if ((cp = ((chan_info *)(sp->chans[j]))))
 	    {
-	      val = (*func)(cp, userptr);
+	      if (cp)
+		val = (*func)(cp, userptr);
 	      if (val) return(val);
 	    }
       }
@@ -462,12 +469,15 @@ bool map_over_chans(bool (*func)(chan_info *, void *), void *userptr)
 void for_each_chan_1(void (*func)(chan_info *, void *), void *userptr)
 {
   int i, j;
-  snd_info *sp;
-  chan_info *cp;
+#if DEBUGGING
+  if (ss->max_sounds > 2000) abort();
+#endif
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
+      chan_info *cp;
       sp = ss->sounds[i];
-      if ((sp) && (sp->inuse == SOUND_NORMAL))
+      if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
 	for (j = 0; j < sp->nchans; j++)
 	  if ((cp = ((chan_info *)(sp->chans[j]))))
 	    (*func)(cp, userptr);
@@ -477,10 +487,43 @@ void for_each_chan_1(void (*func)(chan_info *, void *), void *userptr)
 void for_each_chan(void (*func)(chan_info *))
 {
   int i, j;
-  snd_info *sp;
-  chan_info *cp;
+#if DEBUGGING
+  if (ss->max_sounds > 2000) abort();
+#endif
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && ((sp->inuse == SOUND_NORMAL) || (sp->inuse == SOUND_WRAPPER)))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp);
+    }
+}
+
+void for_each_normal_chan_1(void (*func)(chan_info *, void *), void *userptr)
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
+      sp = ss->sounds[i];
+      if ((sp) && (sp->inuse == SOUND_NORMAL))
+	for (j = 0; j < sp->nchans; j++)
+	  if ((cp = ((chan_info *)(sp->chans[j]))))
+	    (*func)(cp, userptr);
+    }
+}
+
+void for_each_normal_chan(void (*func)(chan_info *))
+{
+  int i, j;
+  for (i = 0; i < ss->max_sounds; i++)
+    {
+      snd_info *sp;
+      chan_info *cp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	for (j = 0; j < sp->nchans; j++)
@@ -493,9 +536,8 @@ bool map_over_sound_chans(snd_info *sp, bool (*func)(chan_info *, void *), void
 {
   /* argument to func is chan_info pointer+void pointer of user spec, return non-zero = abort map, skips inactive sounds */
   int j;
-  bool val;
+  bool val = false;
   chan_info *cp;
-  val = false;
   for (j = 0; j < sp->nchans; j++)
     if ((cp = sp->chans[j]))
       {
@@ -518,11 +560,10 @@ bool map_over_sounds(bool (*func)(snd_info *, void *), void *userptr)
 {
   /* argument to func is snd_info pointer, return true = abort map, skips inactive sounds */
   int i;
-  bool val;
-  snd_info *sp;
-  val = false;
+  bool val = false;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	{
@@ -536,9 +577,9 @@ bool map_over_sounds(bool (*func)(snd_info *, void *), void *userptr)
 void for_each_sound(void (*func)(snd_info *, void *), void *userptr)
 {
   int i;
-  snd_info *sp;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	(*func)(sp, userptr);
@@ -547,12 +588,12 @@ void for_each_sound(void (*func)(snd_info *, void *), void *userptr)
 
 bool map_over_separate_chans(bool (*func)(chan_info *, void *), void *userptr)
 {
+  /* used only to lock/unlock panes */
   int i;
-  bool val;
-  snd_info *sp;
-  val = false;
+  bool val =false;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	{
@@ -574,11 +615,10 @@ bool snd_ok(snd_info *sp)
 
 int active_channels(virtual_channels_t count_virtual_channels)
 {
-  int chans, i;
-  snd_info *sp;
-  chans = 0;
+  int chans = 0, i;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if (snd_ok(sp) && 
 	  (sp->inuse != SOUND_WRAPPER))
@@ -644,10 +684,10 @@ int find_free_sound_slot_for_channel_display(void)
 
 static snd_info *any_active_sound(void)
 {
-  snd_info *sp;
   int i;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	return(sp);
@@ -679,9 +719,9 @@ chan_info *any_selected_channel(snd_info *sp)
 
 chan_info *selected_channel(void)
 {
-  snd_info *sp;
   if (ss->selected_sound != NO_SELECTION)
     {
+      snd_info *sp;
       sp = ss->sounds[ss->selected_sound];
       if ((sp->inuse == SOUND_NORMAL) && (sp->selected_channel != NO_SELECTION))
 	return(sp->chans[sp->selected_channel]);
@@ -701,67 +741,7 @@ static void select_sound(snd_info *sp)
 	     S_select_sound_hook);
   if (ss->selected_sound != sp->index)
     {
-#if USE_MOTIF
-      if (!(ss->using_schemes))
-	{
-	  snd_info *osp = NULL;
-	  if (ss->selected_sound != NO_SELECTION) osp = ss->sounds[ss->selected_sound];
-	  if ((osp) && (sp != osp) && (osp->inuse == SOUND_NORMAL)) 
-	    {
-	      XmChangeColor(w_snd_name(osp), (ss->sgx)->highlight_color);
-#if (XmVERSION > 1)
-	      if (sound_style(ss) == SOUNDS_IN_NOTEBOOK) 
-		XmChangeColor((osp->sgx)->tab, (ss->sgx)->graph_color);
-#endif
-	    }
-	  if (sp->selected_channel != NO_SELECTION) 
-	    {
-	      XmChangeColor(w_snd_name(sp), (ss->sgx)->white);
-#if (XmVERSION > 1)
-	      if (sound_style(ss) == SOUNDS_IN_NOTEBOOK) 
-		{
-		  int page, current_page;
-		  XmNotebookPageStatus status;
-		  XmNotebookPageInfo info;
-		  XmChangeColor((sp->sgx)->tab, (ss->sgx)->selected_graph_color);
-		  XtVaGetValues(SOUND_PANE(ss), XmNcurrentPageNumber, &current_page, NULL);
-		  XtVaGetValues(sp->sgx->tab, XmNpageNumber, &page, NULL);
-		  if (page != current_page)
-		    {
-		      status = XmNotebookGetPageInfo(SOUND_PANE(ss), page, &info);
-		      if (status == XmPAGE_FOUND)
-			{
-			  XtVaSetValues(SOUND_PANE(ss), XmNcurrentPageNumber, page, NULL);
-			  if (sp->nchans > 1)
-			    equalize_sound_panes(sp, sp->chans[0], false);
-			}
-		    }
-		}
-#endif
-	    }
-	}
-#endif
-#if USE_GTK
-      {
-	snd_info *osp = NULL;
-	if (ss->selected_sound != NO_SELECTION) osp = ss->sounds[ss->selected_sound];
-	if ((osp) && (sp != osp) && (osp->inuse == SOUND_NORMAL)) 
-	  gtk_widget_modify_fg(w_snd_name(osp), GTK_STATE_NORMAL, ss->sgx->black);
-	if (sp->selected_channel != NO_SELECTION) 
-	  gtk_widget_modify_fg(w_snd_name(sp), GTK_STATE_NORMAL, ss->sgx->red);
-	if (sound_style(ss) == SOUNDS_IN_NOTEBOOK) 
-	  {
-	    int page, current_page;
-	    current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(SOUND_PANE_BOX(ss)));
-	    page = sp->sgx->page;
-	    if ((page != current_page) && (current_page >= 0))
-	      {
-		ss->selected_sound = sp->index; /* break infinite recursion here */
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(SOUND_PANE_BOX(ss)), page);
-	      }
-	  }
-      }
-#endif
+      reflect_sound_selection(sp);
       ss->selected_sound = sp->index;
       highlight_selected_sound();
       reflect_undo_or_redo_in_menu(any_selected_channel(sp));
@@ -834,7 +814,6 @@ sync_info *snd_sync(int sync)
 {
   int i, j, k, chans = 0;
   snd_info *sp;
-  sync_info *si;
   for (i = 0; i < ss->max_sounds; i++)
     {
       sp = ss->sounds[i];
@@ -843,6 +822,7 @@ sync_info *snd_sync(int sync)
     }
   if (chans > 0)
     {
+      sync_info *si;
       si = (sync_info *)CALLOC(1, sizeof(sync_info));
       si->begs = (off_t *)CALLOC(chans, sizeof(off_t));
       si->cps = (chan_info **)CALLOC(chans, sizeof(chan_info *));
@@ -883,13 +863,13 @@ sync_info *sync_to_chan(chan_info *cp)
 
 snd_info *find_sound(const char *name, int nth)
 {
-  snd_info *sp;
   char *sname;
   int i, which = 0;
   if (name == NULL) return(NULL);
   sname = filename_without_home_directory(name);
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	if ((strcmp(name, sp->short_filename) == 0) || 
@@ -930,14 +910,14 @@ static char timestr[TIME_STR_SIZE];
 
 void display_info(snd_info *sp)
 {
-  char *buffer = NULL;
-  file_info *hdr;
-  char *comment = NULL, *ampstr = NULL;
   if (sp)
     {
+      file_info *hdr;
       hdr = sp->hdr;
       if (hdr)
 	{
+	  char *comment = NULL, *ampstr = NULL;
+	  char *buffer = NULL;
 	  comment = hdr->comment;
 	  while ((comment) && (*comment) && 
 		 (((*comment) == '\n') || 
