@@ -12,10 +12,10 @@ static axis_context *get_ax(chan_info *cp, int ax_id, const char *caller)
   XEN_ERROR(NO_SUCH_AXIS_CONTEXT,
 	    XEN_LIST_3(C_TO_XEN_STRING(caller),
 		       C_TO_XEN_STRING("axis: ~A, sound index: ~A (~A), chan: ~A"),
-		       XEN_LIST_4(C_TO_SMALL_XEN_INT(ax_id),
-				  C_TO_SMALL_XEN_INT(cp->sound->index),
+		       XEN_LIST_4(C_TO_XEN_INT(ax_id),
+				  C_TO_XEN_INT(cp->sound->index),
 				  C_TO_XEN_STRING(cp->sound->short_filename),
-				  C_TO_SMALL_XEN_INT(cp->chan))));
+				  C_TO_XEN_INT(cp->chan))));
   return(NULL);
 }
 
@@ -29,17 +29,19 @@ axis_info *get_ap(chan_info *cp, axis_info_t ap_id, const char *caller)
   if ((cp) && (AXIS_INFO_ID_OK(ap_id)))
     switch (ap_id)
       {
-      case TIME_AXIS_INFO:      return(cp->axis);                               break;
-      case TRANSFORM_AXIS_INFO: if (cp->fft) return(cp->fft->axis);             break;
+      case TIME_AXIS_INFO:      return(cp->axis);                              break;
+      case TRANSFORM_AXIS_INFO: if (cp->fft) return(cp->fft->axis);            break;
       case LISP_AXIS_INFO:      if (cp->lisp_info) return(lisp_info_axis(cp)); break;
       }
   XEN_ERROR(NO_SUCH_AXIS_INFO,
 	    XEN_LIST_3(C_TO_XEN_STRING(caller),
-		       C_TO_XEN_STRING("axis: ~A, sound index: ~A (~A), chan: ~A (axis should be one of " S_time_graph ", " S_lisp_graph ", or " S_transform_graph ")"),
-		       XEN_LIST_4(C_TO_SMALL_XEN_INT((int)(ap_id)),
-				  C_TO_SMALL_XEN_INT(cp->sound->index),
+		       ((!(cp->squelch_update)) || (!(AXIS_INFO_ID_OK(ap_id)))) ?
+		       C_TO_XEN_STRING("axis: ~A of sound ~A (~A), chan: ~A (axis should be " S_time_graph ", " S_lisp_graph ", or " S_transform_graph ")") :
+		       C_TO_XEN_STRING("axis: ~A of sound ~A (~A), chan: ~A does not exist, probably because output is squelched"),
+		       XEN_LIST_4(C_TO_XEN_INT((int)(ap_id)),
+				  C_TO_XEN_INT(cp->sound->index),
 				  C_TO_XEN_STRING(cp->sound->short_filename),
-				  C_TO_SMALL_XEN_INT(cp->chan))));
+				  C_TO_XEN_INT(cp->chan))));
   return(NULL);
 }
 
@@ -96,11 +98,13 @@ static XEN g_fill_rectangle(XEN x0, XEN y0, XEN width, XEN height, XEN snd, XEN
 static XEN g_draw_string(XEN text, XEN x0, XEN y0, XEN snd, XEN chn, XEN ax)
 {
   #define H_draw_string "(" S_draw_string " text x0 y0 (snd #f) (chn #f) (ax #f)): draw a string"
-
+  
+  char *tmp = NULL;
   ASSERT_CHANNEL(S_draw_string, snd, chn, 4);
   XEN_ASSERT_TYPE(XEN_STRING_P(text), text, XEN_ARG_1, S_draw_string, "a string");
   XEN_ASSERT_TYPE(XEN_NUMBER_P(x0), x0, XEN_ARG_2, S_draw_string, "a number");
   XEN_ASSERT_TYPE(XEN_NUMBER_P(y0), y0, XEN_ARG_3, S_draw_string, "a number");
+  tmp = XEN_TO_C_STRING(text);
 #if USE_MOTIF
   /* snd-xdraw to make motif draw-string act in the same way (coordinate-wise) as gtk */
   /*   despite the name, this is not a gtk function */
@@ -110,21 +114,21 @@ static XEN g_draw_string(XEN text, XEN x0, XEN y0, XEN snd, XEN chn, XEN ax)
 #endif
 	      XEN_TO_C_INT(x0),
 	      XEN_TO_C_INT(y0),
-	      XEN_TO_C_STRING(text),
-	      snd_strlen(XEN_TO_C_STRING(text)));
+	      tmp,
+	      snd_strlen(tmp));
   return(text);
 }
 
 #if USE_MOTIF
-  #define POINT XPoint
+  #define point_t XPoint
 #else
-  #define POINT GdkPoint
+  #define point_t GdkPoint
 #endif
 
-static POINT *TO_C_POINTS(XEN pts, const char *caller)
+static point_t *TO_C_POINTS(XEN pts, const char *caller)
 {
   int i, j, len;
-  POINT *pack_pts;
+  point_t *pack_pts;
   XEN *data;
   len = XEN_VECTOR_LENGTH(pts) / 2;
   if (len <= 0) 
@@ -133,7 +137,7 @@ static POINT *TO_C_POINTS(XEN pts, const char *caller)
 			 C_TO_XEN_STRING("empty vector?"), 
 			 pts));
   data = XEN_VECTOR_ELEMENTS(pts);
-  pack_pts = (POINT *)CALLOC(len, sizeof(POINT));
+  pack_pts = (point_t *)CALLOC(len, sizeof(point_t));
   for (i = 0, j = 0; i < len; i++, j += 2)
     {
       pack_pts[i].x = XEN_TO_C_INT_OR_ELSE(data[j], 0);
@@ -147,7 +151,7 @@ static XEN g_draw_lines(XEN pts, XEN snd, XEN chn, XEN ax)
   /* pts should be a vector of integers as (x y) pairs */
   #define H_draw_lines "(" S_draw_lines " lines (snd #f) (chn #f) (ax #f)): draw a vector of lines"
 
-  POINT *pack_pts;
+  point_t *pack_pts;
   axis_context *ax1;
   ASSERT_CHANNEL(S_draw_lines, snd, chn, 2);
   XEN_ASSERT_TYPE(XEN_VECTOR_P(pts), pts, XEN_ARG_1, S_draw_lines, "a vector");
@@ -165,7 +169,7 @@ static XEN g_draw_dots(XEN pts, XEN size, XEN snd, XEN chn, XEN ax)
   /* pts should be a vector of integers as (x y) pairs */
   #define H_draw_dots "(" S_draw_dots " positions dot-size (snd #f) (chn #f) (ax #f)): draw a vector of dots"
  
-  POINT *pack_pts;
+  point_t *pack_pts;
   axis_context *ax1;
   ASSERT_CHANNEL(S_draw_dots, snd, chn, 3);
   XEN_ASSERT_TYPE(XEN_VECTOR_P(pts), pts, XEN_ARG_1, S_draw_dots, "a vector");
@@ -183,7 +187,7 @@ static XEN g_fill_polygon(XEN pts, XEN snd, XEN chn, XEN ax_id)
 { 
   #define H_fill_polygon "(" S_fill_polygon " points (snd #f) (chn #f) (ax #f)): draw a filled polygon"
 
-  POINT *pack_pts;
+  point_t *pack_pts;
   axis_context *ax;
   ASSERT_CHANNEL(S_fill_polygon, snd, chn, 2);
   XEN_ASSERT_TYPE(XEN_VECTOR_P(pts), pts, XEN_ARG_1, S_fill_polygon, "a vector");
@@ -227,7 +231,7 @@ defined by the 4 controlling points x0..y3; 'n' is how many points to return"
   ax = x[3] - (x[0] + cx + bx);
   ay = y[3] - (y[0] + cy + by);
   incr = 1.0 / (float)n;
-  pts = XEN_MAKE_VECTOR(2 * (n + 1), C_TO_SMALL_XEN_INT(0));
+  pts = XEN_MAKE_VECTOR(2 * (n + 1), C_TO_XEN_INT(0));
   data = XEN_VECTOR_ELEMENTS(pts);
   /* VECTOR_SET here */
   data[0] = C_TO_XEN_INT(x[0]);
