@@ -31,9 +31,9 @@ env *free_env(env *e)
 
 env *copy_env(env *e)
 {
-  env *ne;
   if (e)
     {
+      env *ne;
       ne = (env *)CALLOC(1, sizeof(env));
       ne->pts = e->pts;
       ne->data_size = e->pts * 2;
@@ -60,12 +60,12 @@ bool envs_equal(env *e1, env *e2)
 
 char *env_to_string(env *e)
 {
-  int i, j, len;
-  bool first = true;
-  char *expr_buf;
   char *news = NULL;
   if (e)
     {
+      int i, j, len;
+      bool first = true;
+      char *expr_buf;
       len = 4 + (e->pts * 2 * 16);
       news = (char *)CALLOC(len, sizeof(char));
 #if HAVE_RUBY
@@ -103,7 +103,7 @@ char *env_to_string(env *e)
   return(news);
 }
 
-env *make_envelope(Float *env_buffer, int len)
+env *make_envelope_with_offset_and_scaler(Float *env_buffer, int len, Float offset, Float scaler)
 {
   env *e;
   int i, flen;
@@ -112,7 +112,11 @@ env *make_envelope(Float *env_buffer, int len)
   e->data = (Float *)CALLOC(flen, sizeof(Float));
   e->data_size = flen;
   e->pts = flen / 2;
-  for (i = 0; i < len; i++) e->data[i] = env_buffer[i];
+  for (i = 0; i < len; i += 2) 
+    {
+      e->data[i] = env_buffer[i];
+      e->data[i + 1] = offset + scaler * env_buffer[i + 1];
+    }
   if ((flen == 4) && (len == 2))  /* fixup degenerate envelope */
     {
       e->data[2] = e->data[0] + 1.0; 
@@ -122,6 +126,11 @@ env *make_envelope(Float *env_buffer, int len)
   return(e);
 }
 
+env *make_envelope(Float *env_buffer, int len)
+{
+  return(make_envelope_with_offset_and_scaler(env_buffer, len, 0.0, 1.0));
+}
+
 static env *normalize_x_axis(env *e)
 {
   /* make sure env goes from 0 to 1 x-wise; e is changed */
@@ -496,13 +505,12 @@ static void local_mus_error(int type, char *msg)
 }
 
 void env_editor_display_env(env_editor *edp, env *e, axis_context *ax, const char *name, 
-			    int x0, int y0, int width, int height, bool printing)
+			    int x0, int y0, int width, int height, printing_t printing)
 {
   int i, j, k;
   Float ex0, ey0, ex1, ey1, val;
   int ix0, ix1, iy0, iy1, size = 0, lx0, lx1, ly0, ly1, index = 0;
   Float env_val, curx, xincr;
-  mus_any *ce;
   int dur;
   axis_info *ap;
   if (e)
@@ -536,6 +544,7 @@ void env_editor_display_env(env_editor *edp, env *e, axis_context *ax, const cha
     }
   if (edp == ss->enved)
     {
+      /* don't free edp->axis! */
       ap = enved_make_axis(name, ax, x0, y0, width, height, ex0, ex1, ey0, ey1, printing); /* ax used only for GC here */
       edp->axis = ap;
     }
@@ -544,7 +553,7 @@ void env_editor_display_env(env_editor *edp, env *e, axis_context *ax, const cha
       ap = edp->axis;
       ap->ax = ax;
       if (edp->with_dots)
-	init_env_axes(ap, name, x0, y0, width, height, ex0, ex1, ey0, ey1, false);
+	init_env_axes(ap, name, x0, y0, width, height, ex0, ex1, ey0, ey1, NOT_PRINTING);
     }
   if (e)
     {
@@ -647,6 +656,7 @@ void env_editor_display_env(env_editor *edp, env *e, axis_context *ax, const cha
 	    }
 	  else
 	    {
+	      mus_any *ce;
 	      if (edp->with_dots)
 		for (j = 1, i = 2; i < e->pts * 2; i += 2, j++)
 		  env_editor_set_current_point(edp, j, grf_x(e->data[i], ap), grf_y(e->data[i + 1], ap));
@@ -805,7 +815,7 @@ static int all_envs_size = 0;    /* size of this array */
 static int all_envs_top = 0;     /* one past pointer to last entry in this array */
 
 void init_env_axes(axis_info *ap, const char *name, int x_offset, int ey0, int width, int height, 
-		   Float xmin, Float xmax, Float ymin, Float ymax, bool printing)
+		   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing)
 {
   if (ap->xlabel) FREE(ap->xlabel);
   ap->xmin = xmin;
@@ -824,11 +834,11 @@ void init_env_axes(axis_info *ap, const char *name, int x_offset, int ey0, int w
   ap->y_offset = ey0;
   ap->height = height;
   ap->graph_x0 = x_offset;
-  make_axes_1(ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, printing, true);
+  make_axes_1(ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, printing, WITH_X_AXIS, NO_GRID, WITH_LINEAR_AXES, grid_density(ss));
   /* if this is too small for an axis, it still sets up the fields needed for grf_x|y, so tiny envelope graphs will work */
 }
 
-void view_envs(int env_window_width, int env_window_height, bool printing)
+void view_envs(int env_window_width, int env_window_height, printing_t printing)
 {
   /* divide space available into a grid (if needed) that shows all currently defined envelopes */
   /* I suppose if there were several hundred envelopes, we'd need a scrollable viewer... */
@@ -858,7 +868,6 @@ void view_envs(int env_window_width, int env_window_height, bool printing)
 
 int hit_env(int xe, int ye, int env_window_width, int env_window_height)
 {
-  int cols, rows, i, j, width, height, x, y, k;
   if (all_envs_top == 0)
     return(-1);
   else
@@ -867,6 +876,7 @@ int hit_env(int xe, int ye, int env_window_width, int env_window_height)
 	return(0);
       else
 	{
+	  int cols, rows, i, j, width, height, x, y, k;
 	  cols = snd_round(sqrt((Float)(all_envs_top * env_window_width) / (Float)env_window_height));
 	  rows = snd_round((Float)all_envs_top / (Float)cols);
 	  if ((rows * cols) < all_envs_top) rows++;
@@ -960,7 +970,7 @@ void revert_env_edit(void)
     }
 }
 
-int find_env(const char *name)
+static int find_env(const char *name)
 { /* -1 upon failure */
   int i;
   if ((all_envs) && (name))
@@ -978,12 +988,12 @@ env *enved_all_envs(int pos) {return(all_envs[pos]);}
 
 static void add_envelope(char *name, env *val)
 {
-  int i;
   if (all_envs_top == all_envs_size)
     {
       all_envs_size += 16;
       if (all_envs)
 	{
+	  int i;
 	  all_envs = (env **)REALLOC(all_envs, all_envs_size * sizeof(env *));
 	  all_names = (char **)REALLOC(all_names, all_envs_size * sizeof(char *));
 	  for (i = all_envs_size - 16; i < all_envs_size; i++) {all_names[i] = NULL; all_envs[i] = NULL;}
@@ -1007,10 +1017,11 @@ static void add_envelope(char *name, env *val)
 
 void delete_envelope(char *name)
 {
-  int i, pos;
+  int pos;
   pos = find_env(name);
   if (pos != -1)
     {
+      int i;
       if (all_names[pos]) FREE(all_names[pos]);
       for (i = pos; i < all_envs_size - 1; i++)
 	{
@@ -1201,11 +1212,12 @@ static void display_enved_spectrum(chan_info *cp, enved_fft *ef, axis_info *ap)
 }
 
 
-void enved_show_background_waveform(axis_info *ap, axis_info *gray_ap, bool apply_to_selection, bool show_fft, bool printing)
+void enved_show_background_waveform(axis_info *ap, axis_info *gray_ap, bool apply_to_selection, bool show_fft, printing_t printing)
 {
   int srate, pts = 0;
   graph_type_t old_time_graph_type = GRAPH_ONCE;
-  off_t samps, old_printing;
+  off_t samps;
+  printing_t old_printing;
   bool two_sided = false;
   axis_info *active_ap = NULL;
   chan_info *active_channel = NULL;
@@ -1287,10 +1299,11 @@ env *enved_next_env(void)
 
 char *env_name_completer(char *text)
 {
-  int i, j, len, curlen, matches = 0;
+  int matches = 0;
   char *current_match = NULL;
   if ((all_envs) && (text) && (*text))
     {
+      int i, j, len, curlen;
       len = strlen(text);
       for (i = 0; i < all_envs_top; i++)
 	if (strncmp(text, all_names[i], len) == 0)
@@ -1319,10 +1332,10 @@ char *env_name_completer(char *text)
 
 void save_envelope_editor_state(FILE *fd)
 {
-  char *estr;
   int i;
   for (i = 0; i < all_envs_top; i++)
     {
+      char *estr;
       estr = env_to_string(all_envs[i]);
       if (estr)
 	{
@@ -1339,18 +1352,20 @@ void save_envelope_editor_state(FILE *fd)
 
 env *xen_to_env(XEN res)
 {
-  XEN el, lst;
-  int i, len = 0;
-  Float *data;
   env *rtn = NULL;
   if (XEN_LIST_P(res))
     {
+      int len = 0;
       len = XEN_LIST_LENGTH(res);
       if (len > 0)
 	{
+	  int i;
+	  Float *data;
+	  XEN lst;
 	  data = (Float *)CALLOC(len, sizeof(Float));
 	  for (i = 0, lst = XEN_COPY_ARG(res); i < len; i++, lst = XEN_CDR(lst))
 	    {
+	      XEN el;
 	      el = XEN_CAR(lst);
 	      if (XEN_NUMBER_P(el))
 		data[i] = XEN_TO_C_DOUBLE(el);
@@ -1369,11 +1384,12 @@ static bool x_increases(XEN res)
 {
   int i, len;
   XEN lst;
-  Float x, nx;
+  Float x;
   len = XEN_LIST_LENGTH(res);
   x = XEN_TO_C_DOUBLE(XEN_CAR(res));
   for (i = 2, lst = XEN_CDDR(XEN_COPY_ARG(res)); i < len; i += 2, lst = XEN_CDDR(lst))
     {
+      Float nx;
       nx = XEN_TO_C_DOUBLE(XEN_CAR(lst));
       if (x >= nx) return(false);
       x = nx;
@@ -1481,16 +1497,18 @@ static XEN g_define_envelope(XEN name, XEN data, XEN base)
   /* defines 'name' and attaches various 'envelope-* properties */
   XEN temp;
   env *e;
+  char *ename;
   #define H_define_envelope "(" S_define_envelope " name data (base 1.0)): load 'name' with associated 'data', a list of breakpoints \
 and 'base' into the envelope editor."
   XEN_ASSERT_TYPE(XEN_STRING_P(name), name, XEN_ARG_1, S_define_envelope, "a string");
   XEN_ASSERT_TYPE(XEN_LIST_P(data), data, XEN_ARG_2, S_define_envelope, "a list of breakpoints");
   XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(base) || XEN_FALSE_P(base), base, XEN_ARG_3, S_define_envelope, "a float or #f");
+  ename = XEN_TO_C_STRING(name);
   e = xen_to_env(data);
   if ((e) && (XEN_NUMBER_P(base)))
     e->base = XEN_TO_C_DOUBLE(base);
-  alert_envelope_editor(XEN_TO_C_STRING(name), e);
-  XEN_DEFINE_VARIABLE(XEN_TO_C_STRING(name), temp, data); /* already gc protected */
+  alert_envelope_editor(ename, e);
+  XEN_DEFINE_VARIABLE(ename, temp, data); /* already gc protected */
 #if HAVE_GUILE
   /* add property */
   XEN_SET_VARIABLE_PROPERTY(temp, envelope_base_sym, C_TO_XEN_DOUBLE(e->base));
@@ -1503,7 +1521,7 @@ XEN envelope_base(XEN obj) {return(XEN_VARIABLE_PROPERTY(obj, envelope_base_sym)
 static XEN set_envelope_base(XEN obj, XEN base) {XEN_SET_VARIABLE_PROPERTY(obj, envelope_base_sym, base); return(base);}
 #else
 XEN envelope_base(XEN obj) {return(XEN_FALSE);}
-static XEN set_envelope_base(XEN obj, XEN base) {return(XEN_FALSE);}
+/* static XEN set_envelope_base(XEN obj, XEN base) {return(XEN_FALSE);} */
 #endif
 
 XEN env_to_xen(env *e)
@@ -1594,12 +1612,12 @@ static XEN enved_hook;
 
 static bool check_enved_hook(env *e, int pos, Float x, Float y, enved_point_t reason)
 {
-  XEN result = XEN_FALSE;
-  XEN procs, env_list;
   bool env_changed = false;
-  int len = 0;
   if (XEN_HOOKED(enved_hook))
     {
+      int len = 0;
+      XEN result = XEN_FALSE;
+      XEN procs, env_list;
       /* if hook procedure returns a list, that is the new contents of the
        * envelope -- if its length doesn't match current, we need to remake
        * current. Otherwise return 0, and assume the caller will handle default
@@ -1610,10 +1628,10 @@ static bool check_enved_hook(env *e, int pos, Float x, Float y, enved_point_t re
 	{
 	  result = XEN_APPLY(XEN_CAR(procs), 
 			     XEN_LIST_5(env_list,
-					C_TO_SMALL_XEN_INT(pos),
+					C_TO_XEN_INT(pos),
 					C_TO_XEN_DOUBLE(x),
 					C_TO_XEN_DOUBLE(y),
-					C_TO_SMALL_XEN_INT((int)reason)),
+					C_TO_XEN_INT((int)reason)),
 			     S_enved_hook);
 	  procs = XEN_CDR (procs);
 	  if ((XEN_NOT_FALSE_P(result)) && 
