@@ -17,17 +17,22 @@
 
 #if HAVE_SYS_STATFS_H
   #include <sys/statfs.h>
+#else
+  #if HAVE_SYS_VFS_H
+    #include <sys/vfs.h>
+  #endif
 #endif
 #if HAVE_SYS_STATVFS_H && SUN
   #include <sys/statvfs.h>
 #endif
-#if HAVE_SYS_VFS_H
-  #include <sys/vfs.h>
-#endif
 #if (__bsdi__ || HAVE_SYS_PARAM_H)
   #include <sys/param.h>
 #endif
-#if (defined(HAVE_SYS_MOUNT_H) || defined(__APPLE__) || defined(__bsdi__))
+/* to handle mount.h correctly in autoconf I'd need to specialize the header checker for
+ *   whatever headers mount.h needs, which I assume depends on the OS -- not worth the
+ *   trouble!  Perhaps there's a better way to handle disk-kspace?
+ */
+#if MAC_OSX || defined(__bsdi__)
   #include <sys/mount.h>
 #endif
 
@@ -170,9 +175,9 @@ file_info *copy_header(const char *fullname, file_info *ohdr)
 
 static file_info *translate_file(const char *filename, int type)
 {
-  char *newname, *tempname;
-  int *loops = NULL;
   file_info *hdr = NULL;
+  char *newname;
+  int *loops = NULL;
   int err, len, fd;
   len = strlen(filename);
   loops = mus_sound_loop_info(filename);
@@ -187,6 +192,7 @@ static file_info *translate_file(const char *filename, int type)
   fd = CREAT(newname, 0666);
   if (fd == -1)
     {
+      char *tempname;
       tempname = snd_tempnam();
       fd = CREAT(tempname, 0666);
       if (fd == -1)
@@ -195,6 +201,7 @@ static file_info *translate_file(const char *filename, int type)
 		    newname, tempname);
 	  FREE(newname);
 	  FREE(tempname);
+	  if (loops) FREE(loops);
 	  return(NULL);
 	}
       FREE(newname);
@@ -228,9 +235,9 @@ static char *raw_data_explanation(const char *filename, file_info *hdr);
 file_info *make_file_info(const char *fullname)
 {
   file_info *hdr = NULL;
-  int type = MUS_UNSUPPORTED, format = MUS_UNKNOWN;
   if (mus_file_probe(fullname))
     {
+      int type = MUS_UNSUPPORTED, format = MUS_UNKNOWN;
       type = mus_sound_header_type(fullname);
       if (type == MUS_ERROR) 
 	type = mus_header_type();
@@ -416,10 +423,10 @@ static dir *make_dir(const char *name)
 
 dir *free_dir (dir *dp)
 {
-  int i;
   if (dp->name) FREE(dp->name);
   if (dp->files)
     {
+      int i;
       for (i = 0; i < dp->len; i++) 
 	if (dp->files[i]) 
 	  FREE(dp->files[i]);
@@ -431,11 +438,11 @@ dir *free_dir (dir *dp)
   
 static void add_snd_file_to_dir_list(dir *dp, const char *name)
 {
-  int i;
   dp->files[dp->len] = copy_string(name);
   dp->len++;
   if (dp->len == dp->size) 
     {
+      int i;
       dp->size += 32;
       dp->files = (char **)REALLOC(dp->files, dp->size * sizeof(char *));
       for (i = dp->size - 32; i < dp->size; i++) dp->files[i] = NULL;
@@ -506,17 +513,17 @@ dir *find_sound_files_in_dir(const char *name)
 #if (!HAVE_OPENDIR)
   return(NULL);
 #else
-  struct dirent *dirp;
   DIR *dpos;
-  char *dot, *sp;
   dir *dp = NULL;
-  int i;
   if ((dpos = opendir(name)) != NULL)
     {
+      struct dirent *dirp;
       dp = make_dir(name);
       while ((dirp = readdir(dpos)) != NULL)
 	if (dirp->d_name[0] != '.')
 	  {
+	    int i;
+	    char *dot, *sp;
 	    dot = NULL;
 	    for (sp = dirp->d_name; (*sp) != '\0'; sp++) 
 	      if ((*sp) == '.') 
@@ -609,7 +616,7 @@ void reflect_file_change_in_title(void)
   alist->sounds = (int *)CALLOC(ss->max_sounds, sizeof(int));
   alist->names = (char **)CALLOC(ss->max_sounds, sizeof(char *));
   for_each_sound(add_sound_to_active_list, alist);
-  len = snd_strlen(ss->startup_title) + 16;
+  len = snd_strlen(ss->startup_title) + 32;
   if (alist->active_sounds > 0)
     {
       if (alist->active_sounds < 4) 
@@ -658,7 +665,23 @@ static void read_memo_file(snd_info *sp)
   char *newname;
   newname = memo_file_name(sp);
   if (file_write_date(newname) >= sp->write_date)
-    snd_load_file(newname);
+    {
+#if HAVE_GUILE
+      /* this file shouldn't be left in the load list -- it will confuse the save-state process 
+       *   (memo-sound is defined here but not at the saved state file reload point)
+       * snd-loaded-files is the variable name (snd-xen.c), so we save and restore its value if possible 
+       */
+      XEN var = XEN_FALSE, val = XEN_FALSE;
+      var = XEN_NAME_AS_C_STRING_TO_VARIABLE("snd-loaded-files");
+      if (!(XEN_FALSE_P(var)))
+	val = XEN_VARIABLE_REF(var);
+      snd_load_file(newname);
+      if ((!(XEN_FALSE_P(var))) && (XEN_LIST_P(val)))
+	XEN_VARIABLE_SET(var, val);
+#else
+      snd_load_file(newname);
+#endif
+    }
   FREE(newname);
 }
 
@@ -699,10 +722,11 @@ static snd_info *snd_open_file_1 (const char *filename, bool select, bool read_o
   snd_info *sp;
   char *mcf = NULL;
   int files;
-  XEN res = XEN_FALSE, fstr;
+  XEN res = XEN_FALSE;
   mcf = mus_expand_filename(filename);
   if (XEN_HOOKED(open_hook))
     {
+      XEN fstr;
       fstr = C_TO_XEN_STRING(mcf);
       res = run_or_hook(open_hook,
 			XEN_LIST_1(fstr),
@@ -742,9 +766,9 @@ static snd_info *snd_open_file_1 (const char *filename, bool select, bool read_o
   if (sp)
     {
 #if HAVE_RUBY
-      XEN_VARIABLE_SET(S_memo_sound, C_TO_SMALL_XEN_INT(sp->index));
+      XEN_VARIABLE_SET(S_memo_sound, C_TO_XEN_INT(sp->index));
 #else
-      XEN_VARIABLE_SET(memo_sound, C_TO_SMALL_XEN_INT(sp->index));
+      XEN_VARIABLE_SET(memo_sound, C_TO_XEN_INT(sp->index));
 #endif
       sp->write_date = file_write_date(sp->filename);
       sp->need_update = false;
@@ -789,7 +813,7 @@ void snd_close_file(snd_info *sp)
   XEN res = XEN_FALSE;
   if (XEN_HOOKED(close_hook))
     res = run_or_hook(close_hook,
-		      XEN_LIST_1(C_TO_SMALL_XEN_INT(sp->index)),
+		      XEN_LIST_1(C_TO_XEN_INT(sp->index)),
 		      S_close_hook);
   if (XEN_TRUE_P(res)) return;
   /* exit does not go through this function to clean up temps -- see snd_exit_cleanly in snd-main.c */
@@ -874,8 +898,7 @@ int copy_file(const char *oldname, const char *newname)
 
 int move_file(const char *oldfile, const char *newfile)
 {
-  int err;
-  err = 0;
+  int err = 0;
   if ((err = (rename(oldfile, newfile))))
     {
       if (errno == EXDEV)
@@ -900,7 +923,6 @@ snd_info *make_sound_readable(const char *filename, bool post_close)
   chan_info *cp;
   file_info *hdr = NULL;
   snd_data *sd;
-  snd_io *io;
   int i, fd;
   off_t len;
   /* we've already checked that filename exists */
@@ -918,6 +940,7 @@ snd_info *make_sound_readable(const char *filename, bool post_close)
   for (i = 0; i < sp->nchans; i++)
     {
       cp = make_chan_info(NULL, i, sp);
+      cp->editable = false;
       FREE((cp->cgx)->ax);
       FREE(cp->cgx);
       cp->cgx = NULL;
@@ -929,6 +952,7 @@ snd_info *make_sound_readable(const char *filename, bool post_close)
       fd = snd_open_read(filename); /* sends the error if any */
       if (fd != -1)
 	{
+	  snd_io *io;
 	  mus_file_open_descriptors(fd,
 				    filename,
 				    hdr->format,
@@ -936,7 +960,7 @@ snd_info *make_sound_readable(const char *filename, bool post_close)
 				    hdr->data_location,
 				    hdr->chans,
 				    hdr->type);
-	  io = make_file_state(fd, hdr, i, 
+	  io = make_file_state(fd, hdr, i, 0,
 			       (post_close) ? MAX_BUFFER_SIZE : MIX_FILE_BUFFER_SIZE);
 	  cp->sounds[0] = make_snd_data_file(filename, io,
 					     copy_header(hdr->name, hdr),
@@ -982,9 +1006,7 @@ enum {SA_X0, SA_X1, SA_Y0, SA_Y1, SA_XMIN, SA_XMAX, SA_YMIN, SA_YMAX};
 void *make_axes_data(snd_info *sp)
 {
   axes_data *sa;
-  chan_info *cp;
-  axis_info *ap;
-  int i, loc;
+  int i;
   sa = (axes_data *)CALLOC(1, sizeof(axes_data));
   sa->chans = sp->nchans;
   sa->fields = 8;
@@ -993,6 +1015,9 @@ void *make_axes_data(snd_info *sp)
   sa->wavep = (bool *)CALLOC(sa->chans, sizeof(bool));
   for (i = 0; i < sa->chans; i++)
     {
+      chan_info *cp;
+      axis_info *ap;
+      int loc;
       cp = sp->chans[i];
       ap = cp->axis;
       loc = i * sa->fields;
@@ -1013,13 +1038,14 @@ void *make_axes_data(snd_info *sp)
 bool restore_axes_data(snd_info *sp, void *usa, Float new_duration, bool need_edit_history_update)
 {
   axes_data *sa = (axes_data *)usa;
-  int i, j, loc;
+  int i, j;
   bool need_update = false;
-  Float old_duration;
-  chan_info *cp;
-  axis_info *ap;
   for (i = 0, j = 0; i < sp->nchans; i++)
     {
+      Float old_duration;
+      chan_info *cp;
+      axis_info *ap;
+      int loc;
       cp = sp->chans[i];
       loc = j * sa->fields;
       old_duration = sa->axis_data[loc + SA_X1] - sa->axis_data[loc + SA_X0];  /* old duration is x1 - x0 */
@@ -1079,6 +1105,8 @@ static void copy_chan_info(chan_info *ncp, chan_info *ocp)
   ncp->beats_per_minute = ocp->beats_per_minute;
   ncp->show_y_zero = ocp->show_y_zero;
   ncp->show_grid = ocp->show_grid;
+  ncp->grid_density = ocp->grid_density;
+  ncp->show_sonogram_cursor = ocp->show_sonogram_cursor;
   ncp->show_marks = ocp->show_marks;
   ncp->wavo_hop = ocp->wavo_hop;
   ncp->wavo_trace = ocp->wavo_trace;
@@ -1218,7 +1246,7 @@ static XEN update_hook;
 static snd_info *snd_update_1(snd_info *sp, const char *ur_filename)
 {
   /* we can't be real smart here because the channel number may have changed and so on */
-  int i, old_srate, old_chans, old_format, sp_chans, old_index;
+  int i, old_srate, old_chans, old_format, sp_chans, old_index, gc_loc = -1;
   channel_style_t old_channel_style;
   bool read_only, old_raw;
   void *sa;
@@ -1239,8 +1267,8 @@ static snd_info *snd_update_1(snd_info *sp, const char *ur_filename)
       if (XEN_TRUE_P(update_hook_result)) return(sp);
       if (XEN_PROCEDURE_P(update_hook_result))
 	{
-	  if ((XEN_REQUIRED_ARGS(update_hook_result)) == 1)
-	    snd_protect(update_hook_result);
+	  if (XEN_REQUIRED_ARGS_OK(update_hook_result, 1))
+	    gc_loc = snd_protect(update_hook_result);
 	  else XEN_BAD_ARITY_ERROR(S_update_hook, 0, update_hook_result, S_update_hook " function result should require 1 arg");
 	}
     }
@@ -1312,7 +1340,8 @@ static snd_info *snd_update_1(snd_info *sp, const char *ur_filename)
       XEN_CALL_1(update_hook_result,
 		 (nsp) ? C_TO_XEN_INT(nsp->index) : XEN_FALSE,
 		 "procedure returned by " S_update_hook);
-      snd_unprotect(update_hook_result);
+      if (gc_loc != -1) snd_unprotect_at(gc_loc);
+      gc_loc = -1;
     }
   if (saved_sp)
     {
@@ -1346,6 +1375,25 @@ snd_info *snd_update(snd_info *sp)
   return(sp);
 }
 
+static XEN after_save_as_hook;
+
+void run_after_save_as_hook(snd_info *sp, const char *already_saved_as_name, bool from_save_as_dialog)
+{
+  if (XEN_HOOKED(after_save_as_hook))
+    {
+      char *fullname;
+      fullname = mus_expand_filename(already_saved_as_name);
+      run_progn_hook(after_save_as_hook,
+		     XEN_LIST_3(C_TO_XEN_INT(sp->index),
+				C_TO_XEN_STRING(fullname),
+				C_TO_XEN_BOOLEAN(from_save_as_dialog)),
+		     S_after_save_as_hook);
+      FREE(fullname);
+    }
+}
+
+
+
 /* View:Files lists */
 
 static int prevfile_size = 0;
@@ -1392,7 +1440,7 @@ void view_curfiles_play(int pos, bool play)
     {
       if (sp->playing) stop_playing_sound(sp, PLAY_BUTTON_UNSET);
       if (play)
-	play_sound(sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "current files play", 0);
+	play_sound(sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
       else set_play_button(sp, false);
     }
 }
@@ -1417,7 +1465,6 @@ static XEN previous_files_select_hook;
 
 void view_prevfiles_select(int pos)
 {
-  snd_info *sp;
   XEN res = XEN_FALSE;
   if (XEN_HOOKED(previous_files_select_hook))
     res = run_or_hook(previous_files_select_hook,
@@ -1425,6 +1472,7 @@ void view_prevfiles_select(int pos)
 		      S_previous_files_select_hook);
   if (XEN_NOT_TRUE_P(res))
     {
+      snd_info *sp;
       sp = snd_open_file(prevfullnames[pos], false);
       if (sp) select_channel(sp, 0); 
     }
@@ -1454,7 +1502,7 @@ bool view_prevfiles_play(int pos, bool play)
 	{
 	  play_sp->short_filename = prevnames[pos];
 	  play_sp->filename = NULL;
-	  play_sound(play_sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "previous files play", 0);
+	  play_sound(play_sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 	}
       else return(true); /* can't find or setup file */
     }
@@ -1468,10 +1516,11 @@ bool view_prevfiles_play(int pos, bool play)
 
 static void file_unprevlist(const char *filename)
 {
-  int i, j;
+  int i;
   i = find_prevfile_regrow(filename);
   if (i != -1)
     {
+      int j;
       FREE(prevnames[i]);
       prevnames[i] = NULL;
       FREE(prevfullnames[i]);
@@ -1490,11 +1539,12 @@ static void file_unprevlist(const char *filename)
 
 static void file_prevlist(const char *filename, const char *fullname)
 {
-  int k, i, new_size;
+  int k;
   if (find_prevfile_regrow(filename) != -1) return;
   prevfile_end++;
   if (prevfile_end == prevfile_size)
     {
+      int new_size;
       new_size = prevfile_size + 32;
       if (prevfile_size == 0)
 	{
@@ -1504,6 +1554,7 @@ static void file_prevlist(const char *filename, const char *fullname)
 	}
       else
 	{
+	  int i;
 	  prevnames = (char **)REALLOC(prevnames, new_size * sizeof(char *));
 	  prevfullnames = (char **)REALLOC(prevfullnames, new_size * sizeof(char *));
 	  prevtimes = (int *)REALLOC(prevtimes, new_size * sizeof(int));
@@ -1533,12 +1584,12 @@ static void file_prevlist(const char *filename, const char *fullname)
 void add_directory_to_prevlist(const char *dirname)
 {
   dir *sound_files = NULL;
-  char *fullpathname = NULL;
-  char **fullnames;
-  int i, end;
   sound_files = find_sound_files_in_dir(dirname);
   if ((sound_files) && (sound_files->len > 0))
     {
+      char *fullpathname = NULL;
+      char **fullnames;
+      int i, end;
       fullpathname = (char *)CALLOC(FILENAME_MAX, sizeof(char));
       strcpy(fullpathname, dirname);
       if (dirname[strlen(dirname) - 1] != '/') 
@@ -1565,11 +1616,12 @@ void add_directory_to_prevlist(const char *dirname)
 
 static void add_to_previous_files(const char *shortname, const char *fullname)
 {
-  int i, j;
+  int i;
   file_prevlist(shortname, fullname);
   i = find_curfile_regrow(shortname);
   if (i != -1)
     {
+      int j;
       if (curnames[i]) FREE(curnames[i]);
       curnames[i] = NULL;
       for (j = i; j < curfile_end - 1; j++)
@@ -1595,9 +1647,9 @@ void init_curfiles(int size)
 
 static void add_to_current_files(const char *shortname)
 {
-  int i, new_size;
   if (curfile_end == curfile_size)
     {
+      int i, new_size;
       new_size = curfile_size + 32;
       if (curfile_size == 0)
 	curnames = (char **)CALLOC(new_size, sizeof(char *));
@@ -1689,12 +1741,13 @@ void clear_prevlist(void)
 void update_prevlist(void)
 {
   /* here we need the file's full name */
-  int i, j, fd;
   if (prevnames)
     {
+      int i, j;
       for (i = 0; i <= prevfile_end; i++)
 	if (prevnames[i]) 
 	  {
+	    int fd;
 	    fd = OPEN(prevfullnames[i], O_RDONLY, 0);
 	    if (fd == -1) 
 	      {
@@ -1768,11 +1821,11 @@ static int less_compare(const void *a, const void *b)
 
 void make_prevfiles_list_1(void)
 {
-  heapdata **data;
-  int i, len;
   update_prevlist();
   if (prevfile_end >= 0)
     {
+      heapdata **data;
+      int i, len;
       len = prevfile_end + 1;
       data = (heapdata **)CALLOC(len, sizeof(heapdata *));
       for (i = 0; i < len; i++)
@@ -1858,7 +1911,6 @@ static XEN g_previous_files_sort_procedure(void)
 static XEN g_set_previous_files_sort_procedure(XEN proc)
 {
   char *error = NULL;
-  XEN errstr;
   if (XEN_PROCEDURE_P(ss->file_sort_proc))
     snd_unprotect(ss->file_sort_proc);
   ss->file_sort_proc = XEN_UNDEFINED;
@@ -1875,6 +1927,7 @@ static XEN g_set_previous_files_sort_procedure(XEN proc)
     }
   else 
     {
+      XEN errstr;
       set_file_sort_sensitive(false);
       errstr = C_TO_XEN_STRING(error);
       FREE(error);
@@ -2107,14 +2160,14 @@ typedef struct {
 
 static bool check_for_same_name(snd_info *sp1, void *ur_info)
 {
-  int i;
-  chan_info *cp;
   same_name_info *info = (same_name_info *)ur_info;
   if ((sp1) && (strcmp(sp1->filename, info->filename) == 0))
     {
+      int i;
       info->sp = sp1;
       for (i = 0; i < sp1->nchans; i++) 
 	{
+	  chan_info *cp;
 	  cp = sp1->chans[i];
 	  if (info->edits < cp->edit_ctr) 
 	    info->edits = cp->edit_ctr;
@@ -2129,7 +2182,7 @@ bool saved_file_needs_update(snd_info *sp, char *str, save_dialog_t save_type, i
   /* only from save-as dialog where type/format are known to be writable */
   /* returns true if new file not yet opened, false if opened (same name as old) or if cancelled by error of some sort */
   same_name_info *collision = NULL;
-  char *fullname, *ofile;
+  char *fullname;
   int result = 0;
   bool needs_update = true;
   if (sp) clear_minibuffer(sp);
@@ -2144,6 +2197,7 @@ bool saved_file_needs_update(snd_info *sp, char *str, save_dialog_t save_type, i
     }
   if (strcmp(fullname, sp->filename) == 0)
     {
+      char *ofile;
       /* normally save-as saves the current edit tree, merely saving the current state
        * in a separate, presumably inactive file; here we're being asked to overwrite
        * the current file in save-as; we can't have it both ways -- we'll save the edits 
@@ -2159,7 +2213,7 @@ bool saved_file_needs_update(snd_info *sp, char *str, save_dialog_t save_type, i
       /* also what if a sound is write-protected in one window, and not in another? */
       ofile = snd_tempnam(); 
       if (save_type == FILE_SAVE_AS)
-	result = save_edits_without_display(sp, ofile, type, format, srate, comment, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "file save as", 0);
+	result = save_edits_without_display(sp, ofile, type, format, srate, comment, AT_CURRENT_EDIT_POSITION);
       else result = save_selection(ofile, type, format, srate, comment, SAVE_ALL_CHANS);
       if (result != MUS_NO_ERROR)
 	report_in_minibuffer(sp, _("save as temp %s hit error: %s"), ofile, strerror(errno));
@@ -2194,7 +2248,7 @@ bool saved_file_needs_update(snd_info *sp, char *str, save_dialog_t save_type, i
 	}
       mus_sound_forget(fullname);
       if (save_type == FILE_SAVE_AS)
-	result = save_edits_without_display(sp, str, type, format, srate, comment, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "file save as", 0);
+	result = save_edits_without_display(sp, str, type, format, srate, comment, AT_CURRENT_EDIT_POSITION);
       else result = save_selection(str, type, format, srate, comment, SAVE_ALL_CHANS);
       if (result != MUS_NO_ERROR)
 	{
@@ -2219,10 +2273,7 @@ bool saved_file_needs_update(snd_info *sp, char *str, save_dialog_t save_type, i
 void edit_header_callback(snd_info *sp, file_data *edit_header_data)
 {
   /* this blindly changes the header info -- it does not actually reformat the data or whatever */
-  int err, chans, srate, type, format;
-  off_t loc, samples;
-  char *comment, *original_comment = NULL;
-  file_info *hdr;
+  int err;
   if (sp->read_only)
     {
       snd_error(_("%s is write-protected"), sp->filename);
@@ -2235,6 +2286,10 @@ void edit_header_callback(snd_info *sp, file_data *edit_header_data)
 #endif
   if (err == 0)
     {
+      off_t loc, samples;
+      char *comment, *original_comment = NULL;
+      file_info *hdr;
+      int chans, srate, type, format;
       hdr = sp->hdr;
       original_comment = mus_sound_comment(sp->filename);
       if ((hdr->type == MUS_AIFF) || (hdr->type == MUS_AIFC)) mus_header_set_full_aiff_loop_info(mus_sound_loop_info(sp->filename));
@@ -2434,7 +2489,7 @@ static XEN g_set_sound_loop_info(XEN snd, XEN vals)
   snd_info *sp;
   char *tmp_file;
   file_info *hdr;
-  int type, len = 0;
+  int type, len = 0, err = MUS_NO_ERROR;
   XEN start0 = XEN_UNDEFINED, end0 = XEN_UNDEFINED; 
   XEN start1 = XEN_UNDEFINED, end1 = XEN_UNDEFINED; 
   XEN mode0 = XEN_UNDEFINED, mode1 = XEN_UNDEFINED;
@@ -2521,16 +2576,20 @@ static XEN g_set_sound_loop_info(XEN snd, XEN vals)
    *   it would not be so hard).
    */
   tmp_file = snd_tempnam();
-  save_edits_without_display(sp, tmp_file, type, 
-			     hdr->format, 
-			     hdr->srate, 
-			     hdr->comment,
-			     C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION),
-			     S_sound_loop_info, 0);
+  err = save_edits_without_display(sp, tmp_file, type, 
+				   hdr->format, 
+				   hdr->srate, 
+				   hdr->comment,
+				   AT_CURRENT_EDIT_POSITION);
+  if (err != MUS_NO_ERROR)
+    XEN_ERROR(CANNOT_SAVE,
+	      XEN_LIST_3(C_TO_XEN_STRING(S_setB S_sound_loop_info),
+			 C_TO_XEN_STRING(tmp_file),
+			 C_TO_XEN_STRING(strerror(errno))));
   move_file(tmp_file, sp->filename);
-  FREE(tmp_file);
   snd_update(sp);
-  return(xen_return_first(XEN_TRUE, snd, vals));
+  FREE(tmp_file);
+  return(xen_return_first((err == MUS_NO_ERROR) ? XEN_TRUE : C_TO_XEN_INT(err), snd, vals));
 }
 
 static XEN g_soundfont_info(XEN snd)
@@ -2540,7 +2599,6 @@ static XEN g_soundfont_info(XEN snd)
 each inner list has the form: (name start loopstart loopend)"
 
   XEN inlist = XEN_EMPTY_LIST, outlist = XEN_EMPTY_LIST;
-  int i, lim;
   snd_info *sp;
   ASSERT_SOUND(S_soundfont_info, snd, 1);
   sp = get_sp(snd, NO_PLAYERS);
@@ -2549,6 +2607,7 @@ each inner list has the form: (name start loopstart loopend)"
   mus_header_read(sp->filename);
   if (mus_header_type() == MUS_SOUNDFONT)
     {
+      int i, lim;
       lim = mus_header_sf2_entries();
       if (lim > 0)
 	for (i = lim - 1; i >= 0; i--)
@@ -2586,9 +2645,7 @@ static XEN g_preload_file(XEN file)
 static XEN g_sound_files_in_directory(XEN dirname)
 {
   #define H_sound_files_in_directory "(" S_sound_files_in_directory " (directory \".\")): return a list of the sound files in 'directory'"
-  dir *dp = NULL;
   char *name = NULL;
-  int i;
   XEN res = XEN_EMPTY_LIST;
   XEN_ASSERT_TYPE(XEN_STRING_IF_BOUND_P(dirname), dirname, XEN_ONLY_ARG, S_sound_files_in_directory, "a string");
   if (XEN_STRING_P(dirname))
@@ -2596,9 +2653,11 @@ static XEN g_sound_files_in_directory(XEN dirname)
   else name = ".";
   if (name)
     {
+      dir *dp = NULL;
       dp = find_sound_files_in_dir(name);
       if (dp)
 	{
+	  int i;
 	  for (i = dp->len - 1; i >= 0; i--)
 	    res = XEN_CONS(C_TO_XEN_STRING(dp->files[i]), res);
 	  free_dir(dp);
@@ -2716,10 +2775,14 @@ the " S_just_sounds " button is set. Return #f to filter out filename. "
   #define H_bad_header_hook S_bad_header_hook " (filename): called if a file has some bogus-looking header. \
 Return #t to give up on that file."
 
-  XEN_DEFINE_HOOK(open_hook, S_open_hook, 1, H_open_hook);                        /* arg = filename */
-  XEN_DEFINE_HOOK(close_hook, S_close_hook, 1, H_close_hook);                     /* arg = sound index */
-  XEN_DEFINE_HOOK(just_sounds_hook, S_just_sounds_hook, 1, H_just_sounds_hook);   /* arg = filename */
-  XEN_DEFINE_HOOK(bad_header_hook, S_bad_header_hook, 1, H_bad_header_hook);      /* arg = filename */
+  #define H_after_save_as_hook " (saved-sound-index save-as-full-filename from-save-as-dialog): called \
+upon File:Save as completion."
+
+  XEN_DEFINE_HOOK(open_hook, S_open_hook, 1, H_open_hook);                            /* arg = filename */
+  XEN_DEFINE_HOOK(close_hook, S_close_hook, 1, H_close_hook);                         /* arg = sound index */
+  XEN_DEFINE_HOOK(just_sounds_hook, S_just_sounds_hook, 1, H_just_sounds_hook);       /* arg = filename */
+  XEN_DEFINE_HOOK(bad_header_hook, S_bad_header_hook, 1, H_bad_header_hook);          /* arg = filename */
+  XEN_DEFINE_HOOK(after_save_as_hook, S_after_save_as_hook, 3, H_after_save_as_hook); /* args: index filename from-dialog */
 
   #define H_open_raw_sound_hook S_open_raw_sound_hook " (filename current-choices): called when a headerless sound file is opened. \
 Its result can be a list describing the raw file's attributes (thereby bypassing the Raw File Dialog and so on). \
