@@ -35,6 +35,8 @@
                                   (strcmp("GdkColor_", XEN_SYMBOL_TO_C_STRING(XEN_CAR(Value))) == 0))
 #define XEN_WRAP_EVENT(Value)    ((Value) ? XEN_LIST_2(C_STRING_TO_XEN_SYMBOL("GdkEvent_"), C_TO_XEN_ULONG((unsigned long)Value)) : XEN_FALSE)
 
+#define SG_SIGNAL_CONNECT(Widget, Signal, Function, Data) g_signal_connect(G_OBJECT(Widget), Signal, G_CALLBACK(Function), (gpointer)Data)
+
 #define Cessator gint
 #define Cessate gint
 #define Indicium gpointer
@@ -62,12 +64,10 @@ typedef struct {
 
 typedef struct {
   GdkPixmap *file_pix;
-  Cessator apply_in_progress;
   GtkWidget **snd_widgets;
   GtkObject **snd_adjs;
   void *flt;
   GtkWidget *dialog;
-  bool controls_fixed;
   int page;
   bool mini_active;
 } snd_context;
