@@ -110,6 +110,7 @@ void connect_mouse_to_text(GtkWidget *text);
 
 color_t get_in_between_color(color_t fg, color_t bg);
 void snd_doit(int argc, char **argv);
+void auto_update_restart(void);
 #ifdef SND_AS_WIDGET
   GtkWidget *snd_as_widget(int argc, char **argv, GtkWidget *parent, void (*error_func)(const char *));
 #endif
@@ -202,6 +203,9 @@ void set_fft_log_frequency(bool val);
 void set_transform_normalization(fft_normalize_t val);
 void set_show_selection_transform(bool show);
 void set_transform_graph_type(graph_type_t val);
+void reflect_peaks_in_transform_dialog(void);
+void reflect_log_freq_start_in_transform_dialog(void);
+void reflect_min_db_in_transform_dialog(void);
 
 
 
@@ -389,6 +393,9 @@ void sg_make_resizable(GtkWidget *w);
 Cessator add_work_proc(GtkFunction func, gpointer data);
 GtkWidget *snd_gtk_dialog_new(void);
 GtkWidget *snd_gtk_label_new(const char *label, GdkColor *color);
+void widget_int_to_text(GtkWidget *w, int val);
+void widget_float_to_text(GtkWidget *w, Float val);
+void widget_off_t_to_text(GtkWidget *w, off_t val);
 
 
 /* -------- snd-gsnd.c -------- */
@@ -430,8 +437,6 @@ void set_filter_in_dB(snd_info *sp, bool val);
 void set_filter_in_hz(snd_info *sp, bool val);
 void filter_env_changed(snd_info *sp, env *e);
 void color_filter_waveform(GdkColor *color);
-void lock_apply(snd_info *sp);
-void unlock_apply(snd_info *sp);
 void reflect_amp_env_completion(snd_info *sp);
 void reflect_amp_env_in_progress(snd_info *sp);
 snd_info *add_sound_window (char *filename, bool read_only);
@@ -445,8 +450,8 @@ void hide_controls(void);
 void start_progress_report(snd_info *sp, enved_progress_t from_enved);
 void finish_progress_report(snd_info *sp, enved_progress_t from_enved);
 void progress_report(snd_info *sp, const char *funcname, int curchan, int chans, Float pct, enved_progress_t from_enved);
-void set_apply_button(snd_info *sp, bool val);
 void g_init_gxsnd(void);
+void reflect_sound_selection(snd_info *sp);
 
 
 /* -------- snd-gmix.c -------- */
@@ -468,8 +473,8 @@ void show_track_background_wave(int pts, bool two_sided);
 /* -------- snd-genv.c -------- */
 
 axis_info *enved_make_axis(const char *name, axis_context *ax, int ex0, int ey0, int width, int height, 
-			   Float xmin, Float xmax, Float ymin, Float ymax, bool printing);
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing);
+			   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing);
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing);
 void set_enved_redo_sensitive(bool val);
 void set_enved_revert_sensitive(bool val);
 void set_enved_undo_sensitive(bool val);
