@@ -55,9 +55,13 @@ bool channel_graph_is_visible(chan_info *cp)
 	 (channel_graph(cp)) &&
 	 (GTK_WIDGET_VISIBLE(channel_graph(cp))) &&
 	 (cp->sound) &&
-	 (cp->sound->sgx) &&
-	 (w_snd_pane(cp->sound)) &&
-	 (GTK_WIDGET_VISIBLE(w_snd_pane(cp->sound))));
+	 /* here we may have a sound wrapper for variable display in which case the sound widgets are not implemented */
+	 (((cp->sound->inuse == SOUND_WRAPPER) || (cp->sound->inuse == SOUND_REGION)) ||
+	  ((cp->sound->inuse == SOUND_NORMAL) &&
+	   /* other choice: SOUND_IDLE -> no display */
+	   (cp->sound->sgx) &&
+	   (w_snd_pane(cp->sound)) &&
+	   (GTK_WIDGET_VISIBLE(w_snd_pane(cp->sound))))));
 }
 
 bool channel_open_pane(chan_info *cp, void *ptr)
@@ -182,10 +186,10 @@ void resize_sy(chan_info *cp)
 {
   /* something changed the y axis view, so the scale scroller needs to reflect that change (in size and position) */
   axis_info *ap;
-  Float size;
   ap = cp->axis;
   if (ap->y_ambit != 0.0)
     {
+      Float size;
       size = (ap->y1 - ap->y0) / ap->y_ambit;
       set_scrollbar(sy_adj(cp), 
 		    1.0 - ((ap->y0 - ap->ymin) / ap->y_ambit + size), 
@@ -359,8 +363,8 @@ static gboolean graph_mouse_enter(GtkWidget *w, GdkEventCrossing *ev, gpointer d
   pdata = get_user_int_data(G_OBJECT(w));
   if (XEN_HOOKED(mouse_enter_graph_hook))
     run_hook(mouse_enter_graph_hook,
-	     XEN_LIST_2(C_TO_SMALL_XEN_INT(UNPACK_SOUND(pdata)),
-			C_TO_SMALL_XEN_INT(UNPACK_CHANNEL(pdata))),
+	     XEN_LIST_2(C_TO_XEN_INT(UNPACK_SOUND(pdata)),
+			C_TO_XEN_INT(UNPACK_CHANNEL(pdata))),
 	     S_mouse_enter_graph_hook);
   gdk_window_set_cursor(w->window, (ss->sgx)->graph_cursor);
   return(false);
@@ -372,8 +376,8 @@ static gboolean graph_mouse_leave(GtkWidget *w, GdkEventCrossing *ev, gpointer d
   pdata = get_user_int_data(G_OBJECT(w));
   if (XEN_HOOKED(mouse_leave_graph_hook))
     run_hook(mouse_leave_graph_hook,
-	     XEN_LIST_2(C_TO_SMALL_XEN_INT(UNPACK_SOUND(pdata)),
-			C_TO_SMALL_XEN_INT(UNPACK_CHANNEL(pdata))),
+	     XEN_LIST_2(C_TO_XEN_INT(UNPACK_SOUND(pdata)),
+			C_TO_XEN_INT(UNPACK_CHANNEL(pdata))),
 	     S_mouse_leave_graph_hook);
   gdk_window_set_cursor(w->window, (ss->sgx)->arrow_cursor);
   return(false);
@@ -404,7 +408,7 @@ static void show_gz_scrollbars(snd_info *sp)
 
 /* edit history support */
 
-static bool flush_select = false, first_history_select = true;
+static bool flush_select = false;
 
 static void history_select_callback(GtkTreeSelection *selection, gpointer *gp)
 {
@@ -416,17 +420,7 @@ static void history_select_callback(GtkTreeSelection *selection, gpointer *gp)
   if (!(gtk_tree_selection_get_selected(selection, &model, &iter))) return;
   path = gtk_tree_model_get_path(model, &iter);
   indices = gtk_tree_path_get_indices(path);
-  if (first_history_select) /* goddamn gtk sends a bogus initial select of row 0 when the list is first clicked
-			     *   but that causes the entire mix panel to close because it thinks we've reverted
-			     */
-    {
-      first_history_select = false;
-      if (indices[0] == 0)
-	{
-	  gtk_tree_path_free(path);
-	  return;
-	}
-    }
+  if (indices[0] == 0)
   flush_select = true;
   edit_history_select((chan_info *)gp, indices[0]);
   flush_select = false;
@@ -436,21 +430,28 @@ static void history_select_callback(GtkTreeSelection *selection, gpointer *gp)
 static void remake_edit_history(GtkWidget *lst, chan_info *cp)
 {
   snd_info *sp;
-  chan_info *ncp;
   int i, eds;
   char *str;
   gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lst))));
   sp = cp->sound;
+#if DEBUGGING
+  if ((!sp) || (!(sp->active)) || (sp->inuse != SOUND_NORMAL))
+    fprintf(stderr, "trouble in remake_edit_history: %p %d %d\n", sp, (sp) ? sp->active : -1, (sp) ? sp->inuse : -1);
+#endif
   if (sp->channel_style != CHANNELS_SEPARATE)
     {
       int k, all_eds = 0, ed, filelen;
       char *title;
+      chan_info *ncp;
       for (k = 0; k < sp->nchans; k++)
 	{
 	  ncp = sp->chans[k];
-	  eds = ncp->edit_ctr;
-	  while ((eds < (ncp->edit_size - 1)) && (ncp->edits[eds + 1])) eds++;
-	  all_eds += eds;
+	  if ((ncp) && (ncp->sound))
+	    {
+	      eds = ncp->edit_ctr;
+	      while ((eds < (ncp->edit_size - 1)) && (ncp->edits[eds + 1])) eds++;
+	      all_eds += eds;
+	    }
 	}
       all_eds += 3 * sp->nchans;
       filelen = 16 + strlen(sp->filename);
@@ -458,21 +459,24 @@ static void remake_edit_history(GtkWidget *lst, chan_info *cp)
       for (k = 0, ed = 0; k < sp->nchans; k++)
 	{
 	  ncp = sp->chans[k];
-	  ncp->edhist_base = ed++;
-	  sprintf(title, "chan %d: %s", k + 1, sp->filename);
-	  sg_list_append(lst, title);
-	  eds = ncp->edit_ctr;
-	  while ((eds < (ncp->edit_size - 1)) && (ncp->edits[eds + 1])) eds++;
-	  for (i = 1; i <= eds; i++, ed++) 
+	  if ((ncp) && (ncp->sound))
 	    {
-	      str = edit_to_string(ncp, i);
-	      sg_list_append(lst, str);
+	      ncp->edhist_base = ed++;
+	      sprintf(title, "chan %d: %s", k + 1, sp->filename);
+	      sg_list_append(lst, title);
+	      eds = ncp->edit_ctr;
+	      while ((eds < (ncp->edit_size - 1)) && (ncp->edits[eds + 1])) eds++;
+	      for (i = 1; i <= eds; i++, ed++) 
+		{
+		  str = edit_to_string(ncp, i);
+		  sg_list_append(lst, str);
+		}
+	      if (k < sp->nchans - 1)
+		{
+		  sg_list_append(lst, "______________________________"); 
+		  ed++; 
+		} 
 	    }
-	  if (k < sp->nchans - 1)
-	    {
-	      sg_list_append(lst, "______________________________"); 
-	      ed++; 
-	    } 
 	}
       FREE(title);
     }
@@ -499,14 +503,17 @@ void reflect_edit_history_change(chan_info *cp)
   /* new edit so it is added, and any trailing lines removed */
   GtkWidget *lst;
   snd_info *sp;
-  chan_info *ncp;
   if ((cp->in_as_one_edit) || (cp->cgx == NULL)) return;
   sp = cp->sound;
   if ((cp->chan > 0) && (sp->channel_style != CHANNELS_SEPARATE))
     {
+      chan_info *ncp;
       ncp = sp->chans[0];
-      lst = EDIT_HISTORY_LIST(ncp);
-      if (lst) remake_edit_history(lst, ncp);
+      if ((ncp) && (ncp->sound))
+	{
+	  lst = EDIT_HISTORY_LIST(ncp);
+	  if (lst) remake_edit_history(lst, ncp);
+	}
     }
   else
     {
@@ -519,11 +526,11 @@ void reflect_edit_counter_change(chan_info *cp)
 {
   /* undo/redo/revert -- change which line is highlighted */
   snd_info *sp;
-  chan_info *ncp;
   if (cp->cgx == NULL) return;
   sp = cp->sound;
   if ((cp->chan > 0) && (sp->channel_style != CHANNELS_SEPARATE))
     {
+      chan_info *ncp;
       ncp = sp->chans[0];
       flush_select = true;
       sg_list_select(EDIT_HISTORY_LIST(ncp), cp->edit_ctr + cp->edhist_base);
@@ -578,7 +585,6 @@ gboolean graph_key_press(GtkWidget *w, GdkEventKey *ev, gpointer data)
  
 static gboolean graph_button_press(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
-  chan_info *cp = (chan_info *)data;
   if ((ev->state == 0) && (ev->type == GDK_BUTTON_PRESS) && (ev->button == POPUP_BUTTON))
     {
       int pdata;
@@ -588,9 +594,11 @@ static gboolean graph_button_press(GtkWidget *w, GdkEventButton *ev, gpointer da
     }
   else
     {
+      chan_info *cp = (chan_info *)data;
       (ss->sgx)->graph_is_active = true;
       gtk_widget_grab_focus(w);
-      ((cp->sound)->sgx)->mini_active = false;
+      if ((cp->sound) && (cp->sound->sgx))
+	((cp->sound)->sgx)->mini_active = false;
       graph_button_press_callback(cp, (int)(ev->x), (int)(ev->y), ev->state, ev->button, ev->time);
     }
   return(false);
@@ -610,10 +618,10 @@ static gboolean graph_scroll(GtkWidget *w, GdkEventScroll *ev, gpointer data)
 
 static gboolean graph_button_motion(GtkWidget *w, GdkEventMotion *ev, gpointer data)
 { 
-  int x, y;
-  GdkModifierType state;
   if (ev->state & GDK_BUTTON1_MASK)
     {
+      int x, y;
+      GdkModifierType state;
       if (ev->is_hint)
 	gdk_window_get_pointer(ev->window, &x, &y, &state);
       else
@@ -691,15 +699,11 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 
       cw[W_graph] = gtk_drawing_area_new();
 #if HAVE_GL
-#if HAVE_GTK_GL_EXT_0_1
-      gtk_widget_set_gl_capability(GTK_WIDGET(cw[W_graph]), config_attributes, GDK_GL_RGBA_TYPE, NULL, true);
-#else
   #if HAVE_GDK_GL_CONTEXT_COPY
       gtk_widget_set_gl_capability(GTK_WIDGET(cw[W_graph]), gdk_gl_config_new(&config_attributes[0]), NULL, true, GDK_GL_RGBA_TYPE);
   #else
       gtk_widget_set_gl_capability(GTK_WIDGET(cw[W_graph]), gdk_gl_config_new(&config_attributes[0]), GDK_GL_RGBA_TYPE, NULL, true);
   #endif
-#endif
 #endif
       add_drop(cw[W_graph]);
       set_user_int_data(G_OBJECT(cw[W_graph]), PACK_SOUND_AND_CHANNEL(sp->index, cp->chan));
@@ -712,54 +716,18 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
       gtk_widget_show(cw[W_graph]);
       if (with_events)
 	{
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("expose_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(channel_expose_callback), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("configure_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(channel_resize_callback), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(cw[W_graph], "expose_event", channel_expose_callback, cp);
+	  SG_SIGNAL_CONNECT(cw[W_graph], "configure_event", channel_resize_callback, cp);
 	}
+      SG_SIGNAL_CONNECT(cw[W_graph], "button_press_event", graph_button_press, cp);
+      SG_SIGNAL_CONNECT(cw[W_graph], "button_release_event", graph_button_release, cp);
+      SG_SIGNAL_CONNECT(cw[W_graph], "motion_notify_event", graph_button_motion, cp);
       if (main == NULL)
 	{
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("enter_notify_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_mouse_enter), NULL, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("leave_notify_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_mouse_leave), NULL, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("key_press_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(real_graph_key_press), (gpointer)cp, 0),
-					 0);
- 	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("scroll_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_scroll), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("button_press_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_button_press), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("button_release_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_button_release), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_graph]),
-					 g_signal_lookup("motion_notify_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_graph]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(graph_button_motion), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(cw[W_graph], "enter_notify_event", graph_mouse_enter, NULL);
+	  SG_SIGNAL_CONNECT(cw[W_graph], "leave_notify_event", graph_mouse_leave, NULL);
+	  SG_SIGNAL_CONNECT(cw[W_graph], "key_press_event", real_graph_key_press, cp);
+ 	  SG_SIGNAL_CONNECT(cw[W_graph], "scroll_event", graph_scroll, cp);
 	}
 
       cw[W_bottom_scrollers] = gtk_vbox_new(true, 0);
@@ -774,11 +742,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
       cw[W_sx] = gtk_hscrollbar_new(GTK_ADJUSTMENT(adjs[W_sx_adj]));
       gtk_box_pack_start(GTK_BOX(cw[W_bottom_scrollers]), cw[W_sx], true, true, 0);
       set_user_data(G_OBJECT(adjs[W_sx_adj]), (gpointer)cp);
-      g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_sx_adj]),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_sx_adj]))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sx_valuechanged_callback), (gpointer)cp, 0),
-				     0);
+      SG_SIGNAL_CONNECT(adjs[W_sx_adj], "value_changed", sx_valuechanged_callback, cp);
       gtk_widget_show(cw[W_sx]);
       gtk_widget_set_name(cw[W_sx], "sx_slider");
 
@@ -786,11 +750,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
       cw[W_zx] = gtk_hscrollbar_new(GTK_ADJUSTMENT(adjs[W_zx_adj]));
       gtk_box_pack_start(GTK_BOX(cw[W_bottom_scrollers]), cw[W_zx], true, true, 0);
       set_user_data(G_OBJECT(adjs[W_zx_adj]), (gpointer)cp);
-      g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_zx_adj]),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_zx_adj]))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(zx_valuechanged_callback), (gpointer)cp, 0),
-				     0);
+      SG_SIGNAL_CONNECT(adjs[W_zx_adj], "value_changed", zx_valuechanged_callback, cp);
       gtk_widget_set_name(cw[W_zx], "zx_slider");
       gtk_widget_show(cw[W_zx]);
 
@@ -805,31 +765,15 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 	  gtk_box_pack_start(GTK_BOX(cw[W_wf_buttons]), cw[W_f], true, true, 0);
 	  gtk_widget_show(cw[W_f]);
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cw[W_f]), false);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_f]),
-					 g_signal_lookup("button_press_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_f]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(f_toggle_callback), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_f]),
-					 g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(cw[W_f]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(f_toggle_click_callback), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(cw[W_f], "button_press_event", f_toggle_callback, cp);
+	  SG_SIGNAL_CONNECT(cw[W_f], "toggled", f_toggle_click_callback, cp);
   
 	  cw[W_w] = gtk_check_button_new_with_label(_("w"));
 	  gtk_box_pack_start(GTK_BOX(cw[W_wf_buttons]), cw[W_w], true, true, 0);
 	  gtk_widget_show(cw[W_w]);
 	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cw[W_w]), true);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_w]),
-					 g_signal_lookup("button_press_event", G_OBJECT_TYPE(GTK_OBJECT(cw[W_w]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(w_toggle_callback), (gpointer)cp, 0),
-					 0);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(cw[W_w]),
-					 g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(cw[W_w]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(w_toggle_click_callback), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(cw[W_w], "button_press_event", w_toggle_callback, cp);
+	  SG_SIGNAL_CONNECT(cw[W_w], "toggled", w_toggle_click_callback, cp);
 
 	}
       else
@@ -858,11 +802,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), 
 		       0, 0);
       set_user_data(G_OBJECT(adjs[W_zy_adj]), (gpointer)cp);
-      g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_zy_adj]),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_zy_adj]))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(zy_valuechanged_callback), (gpointer)cp, 0),
-				     0);
+      SG_SIGNAL_CONNECT(adjs[W_zy_adj], "value_changed", zy_valuechanged_callback, cp);
       gtk_widget_show(cw[W_zy]);
       gtk_widget_set_name(cw[W_zy], "zy_slider");
 
@@ -873,11 +813,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), 
 		       0, 0);
       set_user_data(G_OBJECT(adjs[W_sy_adj]), (gpointer)cp);
-      g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_sy_adj]),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_sy_adj]))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sy_valuechanged_callback), (gpointer)cp, 0),
-				     0);
+      SG_SIGNAL_CONNECT(adjs[W_sy_adj], "value_changed", sy_valuechanged_callback, cp);
       gtk_widget_show(cw[W_sy]);
       gtk_widget_set_name(cw[W_sy], "sy_slider");
 
@@ -890,11 +826,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 			   (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), 
 			   0, 0);
 	  set_user_data(G_OBJECT(adjs[W_gsy_adj]), (gpointer)cp);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_gsy_adj]),
-					 g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_gsy_adj]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(gsy_valuechanged_callback), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(adjs[W_gsy_adj], "value_changed", gsy_valuechanged_callback, cp);
 	  gtk_widget_show(cw[W_gsy]);
 	  gtk_widget_set_name(cw[W_gsy], "gsy_slider");
 
@@ -905,11 +837,7 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
 			   (GtkAttachOptions)(GTK_EXPAND | GTK_FILL | GTK_SHRINK), 
 			   0, 0);
 	  set_user_data(G_OBJECT(adjs[W_gzy_adj]), (gpointer)cp);
-	  g_signal_connect_closure_by_id(GTK_OBJECT(adjs[W_gzy_adj]),
-					 g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(adjs[W_gzy_adj]))),
-					 0,
-					 g_cclosure_new(GTK_SIGNAL_FUNC(gzy_valuechanged_callback), (gpointer)cp, 0),
-					 0);
+	  SG_SIGNAL_CONNECT(adjs[W_gzy_adj], "value_changed", gzy_valuechanged_callback, cp);
 	  gtk_widget_show(cw[W_gzy]);
 	  gtk_widget_set_name(cw[W_gzy], "gzy_slider");
 	  
@@ -933,6 +861,8 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Gtk
   if ((need_extra_scrollbars) && (sp->channel_style != CHANNELS_COMBINED))
     hide_gz_scrollbars(sp); /* default is on in this case */  
 
+  reflect_edit_history_change(cp);
+
   cax = cx->ax;
   cax->gc = sx->basic_gc;
   /* cax->wn has to wait until update_graph */
@@ -984,10 +914,10 @@ GdkGC *erase_GC(chan_info *cp)
 
 void cleanup_cw(chan_info *cp)
 {
-  chan_context *cx;
-  GtkWidget **cw;
   if ((cp) && (cp->cgx))
     {
+      chan_context *cx;
+      GtkWidget **cw;
       cx = cp->cgx;
       cx->selected = false;
       cw = cx->chan_widgets;
@@ -1005,21 +935,15 @@ void cleanup_cw(chan_info *cp)
 
 void change_channel_style(snd_info *sp, channel_style_t new_style)
 {
-  int i;
-  channel_style_t old_style;
-  chan_info *ncp, *cp, *pcp;
-  int height[1];
-  chan_context *mcgx;
-  GtkWidget **cw;
-  axis_info *ap;
-  chan_context *cx;
   if ((sp) && (sp->nchans > 1))
     {
+      channel_style_t old_style;
       old_style = sp->channel_style;
       if (new_style != old_style)
 	{
+	  int i;
+	  int height[1];
 	  sp->channel_style = new_style;
-
 	  if ((new_style == CHANNELS_SEPARATE) || (old_style == CHANNELS_SEPARATE))
 	    {
 	      GtkWidget* lst;
@@ -1060,6 +984,8 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 	  height[0] = widget_height(w_snd_pane(sp)) - control_panel_height(sp) - 16;
 	  if (old_style == CHANNELS_SEPARATE)
 	    {
+	      chan_context *mcgx;
+	      chan_info *ncp;
 	      ncp = sp->chans[0];
 	      mcgx = ncp->cgx;
 	      for (i = 1; i < sp->nchans; i++) 
@@ -1076,6 +1002,10 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 	    {
 	      if (new_style == CHANNELS_SEPARATE)
 		{
+		  axis_info *ap;
+		  chan_info* pcp;
+		  GtkWidget **cw;
+		  chan_context *cx;
 		  /* height[0] = total space available */
 		  height[0] /= sp->nchans;
 		  map_over_sound_chans(sp, channel_open_pane, NULL);
@@ -1084,6 +1014,7 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 		  ap = pcp->axis;
 		  for (i = 1; i < sp->nchans; i++)
 		    {
+		      chan_info *cp;
 		      cp = sp->chans[i];
 		      cp->tcgx = NULL;
 		      cx = cp->cgx;
