@@ -71,17 +71,20 @@ void draw_polygon(axis_context *ax, int points, ...)
 void draw_lines (axis_context *ax, GdkPoint *points, int num)
 {
   if (num == 0) return;
+#if DEBUGGING
+  if (!GDK_IS_DRAWABLE(ax->wn)) abort();
+#endif
   gdk_draw_lines(ax->wn, ax->gc, points, num);
 }
 
 void draw_points (axis_context *ax, GdkPoint *points, int num, int size)
 {
-  int i, size2;
   if (num == 0) return;
   if (size == 1)
     gdk_draw_points(ax->wn, ax->gc, points, num);
   else
     {
+      int i, size2;
       size2 = size / 2;
       for (i = 0; i < num; i++) 
 	gdk_draw_arc(ax->wn, ax->gc, true, points[i].x - size2, points[i].y - size2, size, size, 0, 360 * 64);
@@ -156,7 +159,7 @@ void set_grf_point(Locus xi, int j, Locus yi)
 
 void draw_both_grf_points(int dot_size, axis_context *ax, int j, graph_style_t graph_style)
 {
-  int i, size8, size4;
+  int i;
   switch (graph_style)
     {
     case GRAPH_LINES:
@@ -187,6 +190,7 @@ void draw_both_grf_points(int dot_size, axis_context *ax, int j, graph_style_t g
 	}
       else
 	{
+	  int size8, size4;
 	  size8 = dot_size / 8;
 	  size4 = dot_size / 4;
 	  if (size4 < 1) size4 = 1;
@@ -200,7 +204,7 @@ void draw_both_grf_points(int dot_size, axis_context *ax, int j, graph_style_t g
 
 void draw_grf_points(int dot_size, axis_context *ax, int j, axis_info *ap, Float y0, graph_style_t graph_style)
 {
-  int i, gy0, size8, size4;
+  int i, gy0;
   switch (graph_style)
     {
     case GRAPH_LINES: draw_lines(ax, points, j); break;
@@ -219,6 +223,7 @@ void draw_grf_points(int dot_size, axis_context *ax, int j, axis_info *ap, Float
 	}
       else
 	{
+	  int size8, size4;
 	  size8 = dot_size / 8;
 	  size4 = dot_size / 4;
 	  if (size4 < 1) size4 = 1;
@@ -488,9 +493,9 @@ void set_sono_rectangle(int j, int color, Locus x, Locus y, Latus width, Latus h
 
 void allocate_sono_rects(int size)
 {
-  int i;
   if (size != sono_bins)
     {
+      int i;
       for (i = 0; i < sono_colors; i++)
 	{
 	  if ((sono_bins > 0) && (sono_data[i])) 
@@ -503,11 +508,11 @@ void allocate_sono_rects(int size)
 
 void allocate_color_map(int colormap)
 {
-  int i;
-  GdkColormap *cmap;
-  GdkColor tmp_color;
   if (grays_allocated != colormap)
     {
+      int i;
+      GdkColormap *cmap;
+      GdkColor tmp_color;
       cmap = gdk_colormap_get_system();
       if (grays_allocated != -1) 
 	for (i = 0; i < grays_size; i++) 
@@ -604,14 +609,12 @@ static void list_color_callback(GtkTreeSelection *selection, gpointer *gp)
 {
   GtkTreeIter iter;
   gchar *value = NULL;
-  int i, size;
-
   GtkTreeModel *model;
   if (!(gtk_tree_selection_get_selected(selection, &model, &iter))) return;
   gtk_tree_model_get(model, &iter, 0, &value, -1);
-
   if (value)
     {
+      int i, size;
       size = num_colormaps();
       for (i = 0; i < size; i++)
 	if ((colormap_name(i)) &&
@@ -661,18 +664,19 @@ static void help_color_callback(GtkWidget *w, gpointer context)
   color_dialog_help();
 }
 
-static void delete_color_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint delete_color_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(ccd->dialog);
+  return(true);
 }
 
 void reflect_color_list(bool setup_time)
 {
-  int i, size;
-  GtkTreeIter iter;
-  GtkListStore *model;
   if ((ccd) && (ccd->list))
     {
+      int i, size;
+      GtkTreeIter iter;
+      GtkListStore *model;
       size = num_colormaps();
       model = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ccd->list)));
       gtk_list_store_clear(model);
@@ -686,19 +690,15 @@ void reflect_color_list(bool setup_time)
 
 static void start_view_color_dialog(bool managed)
 {
-  GtkWidget *light_label, *dark_label, *help_button, *dismiss_button;
-  GtkWidget *outer_table, *scale_box, *cutoff_box, *cutoff_label, *colormap_box;
-
   if (!ccd)
     {
+      GtkWidget *light_label, *dark_label, *help_button, *dismiss_button;
+      GtkWidget *outer_table, *scale_box, *cutoff_box, *cutoff_label, *colormap_box;
+
       /* create color chooser dialog window */
       ccd = (color_chooser_info *)CALLOC(1, sizeof(color_chooser_info));
       ccd->dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(ccd->dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(ccd->dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(delete_color_dialog), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(ccd->dialog, "delete_event", delete_color_dialog, NULL);
       gtk_window_set_title(GTK_WINDOW(ccd->dialog), _("Color Editor"));
       sg_make_resizable(ccd->dialog);
       gtk_container_set_border_width (GTK_CONTAINER(ccd->dialog), 4);
@@ -711,16 +711,8 @@ static void start_view_color_dialog(bool managed)
       gtk_widget_set_name(dismiss_button, "quit_button");
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ccd->dialog)->action_area), dismiss_button, false, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(ccd->dialog)->action_area), help_button, false, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(dismiss_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(dismiss_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(dismiss_color_callback), (gpointer)ccd, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(help_color_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(dismiss_button, "clicked", dismiss_color_callback, ccd);
+      SG_SIGNAL_CONNECT(help_button, "clicked", help_color_callback, NULL);
       gtk_widget_show(dismiss_button);
       gtk_widget_show(help_button);
 
@@ -739,11 +731,7 @@ static void start_view_color_dialog(bool managed)
       gtk_scale_set_value_pos(GTK_SCALE(ccd->scale), GTK_POS_TOP);
       gtk_scale_set_draw_value(GTK_SCALE(ccd->scale), true);
       gtk_table_attach_defaults(GTK_TABLE(scale_box), ccd->scale, 0, 2, 0, 1);
-      g_signal_connect_closure_by_id(GTK_OBJECT(ccd->scale_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(ccd->scale_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(scale_color_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(ccd->scale_adj, "value_changed", scale_color_callback, NULL);
 
       light_label = gtk_label_new(_("light"));
       gtk_misc_set_alignment(GTK_MISC (light_label), 0.05, 0.0);
@@ -768,11 +756,7 @@ static void start_view_color_dialog(bool managed)
       gtk_scale_set_value_pos(GTK_SCALE(ccd->cutoff), GTK_POS_TOP);
       gtk_scale_set_draw_value(GTK_SCALE(ccd->cutoff), true);
       gtk_table_attach_defaults(GTK_TABLE(cutoff_box), ccd->cutoff, 0, 2, 0, 1);
-      g_signal_connect_closure_by_id(GTK_OBJECT(ccd->cutoff_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(ccd->cutoff_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(cutoff_color_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(ccd->cutoff_adj, "value_changed", cutoff_color_callback, NULL);
 
       cutoff_label = gtk_label_new(_("data cutoff"));
       gtk_misc_set_alignment(GTK_MISC (cutoff_label), 0.1, 0.0);
@@ -784,11 +768,7 @@ static void start_view_color_dialog(bool managed)
 
       ccd->invert = gtk_check_button_new_with_label(_("invert"));
       gtk_table_attach(GTK_TABLE(outer_table), ccd->invert, 0, 1, 2, 3, (GtkAttachOptions)GTK_FILL, (GtkAttachOptions)0, 0, 4);
-      g_signal_connect_closure_by_id(GTK_OBJECT(ccd->invert),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(ccd->invert))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(invert_color_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(ccd->invert, "toggled", invert_color_callback, NULL);
       gtk_widget_show(ccd->invert);
       set_toggle_button(ccd->invert, color_inverted(ss), false, NULL);
 
@@ -1013,9 +993,10 @@ static void dismiss_orientation_callback(GtkAdjustment *adj, gpointer context)
   gtk_widget_hide(od->dialog);
 }
 
-static void delete_orientation_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint delete_orientation_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(oid->dialog);
+  return(true);
 }
 
 static int fixup_angle(Float ang)
@@ -1075,20 +1056,17 @@ static void start_view_orientation_dialog(bool managed)
   #define XX_PAD 0
   #define YY_PAD 0
 
-  GtkWidget *outer_table, *dismiss_button, *help_button, *reset_button;
-  GtkWidget *ax_box, *ay_box, *az_box, *sx_box, *sy_box, *sz_box, *hop_box, *cut_box;
-  GtkWidget *ax_label, *ay_label, *az_label, *sx_label, *sy_label, *sz_label, *hop_label, *cut_label;
   /* set up dialog with table 8 by 2 */
   if (!oid)
     {
+      GtkWidget *outer_table, *dismiss_button, *help_button, *reset_button;
+      GtkWidget *ax_box, *ay_box, *az_box, *sx_box, *sy_box, *sz_box, *hop_box, *cut_box;
+      GtkWidget *ax_label, *ay_label, *az_label, *sx_label, *sy_label, *sz_label, *hop_label, *cut_label;
+
       /* create orientation window */
       oid = (orientation_info *)CALLOC(1, sizeof(orientation_info));
       oid->dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(oid->dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(delete_orientation_dialog), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->dialog, "delete_event", delete_orientation_dialog, NULL);
       gtk_window_set_title(GTK_WINDOW(oid->dialog), _("Spectrogram Orientation"));
       sg_make_resizable(oid->dialog);
       gtk_container_set_border_width(GTK_CONTAINER(oid->dialog), 4);
@@ -1104,21 +1082,9 @@ static void start_view_orientation_dialog(bool managed)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(oid->dialog)->action_area), reset_button, false, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(oid->dialog)->action_area), dismiss_button, false, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(oid->dialog)->action_area), help_button, false, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(reset_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(reset_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(reset_orientation_callback), (gpointer)oid, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(dismiss_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(dismiss_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(dismiss_orientation_callback), (gpointer)oid, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(help_orientation_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(reset_button, "clicked", reset_orientation_callback, oid);
+      SG_SIGNAL_CONNECT(dismiss_button, "clicked", dismiss_orientation_callback, oid);
+      SG_SIGNAL_CONNECT(help_button, "clicked", help_orientation_callback, NULL);
       gtk_widget_show(reset_button);
       gtk_widget_show(dismiss_button);
       gtk_widget_show(help_button);
@@ -1145,11 +1111,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_table_attach(GTK_TABLE(ax_box), oid->ax, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
 
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->ax_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->ax_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(ax_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->ax_adj, "value_changed", ax_orientation_callback, oid);
 
       ax_label = gtk_label_new(_("x angle"));
       gtk_misc_set_alignment(GTK_MISC (ax_label), 0.1, 0.0);
@@ -1174,12 +1136,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->ay), true);
       gtk_table_attach(GTK_TABLE(ay_box), oid->ay, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->ay_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->ay_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(ay_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->ay_adj, "value_changed", ay_orientation_callback, oid);
 
       ay_label = gtk_label_new(_("y angle"));
       gtk_misc_set_alignment(GTK_MISC(ay_label), 0.1, 0.0);
@@ -1204,12 +1161,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->az), true);
       gtk_table_attach(GTK_TABLE(az_box), oid->az, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->az_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->az_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(az_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->az_adj, "value_changed", az_orientation_callback, oid);
 
       az_label = gtk_label_new(_("z angle"));
       gtk_misc_set_alignment(GTK_MISC (az_label), 0.1, 0.0);
@@ -1234,12 +1186,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->hop), true);
       gtk_table_attach(GTK_TABLE(hop_box), oid->hop, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->hop_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->hop_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(hop_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->hop_adj, "value_changed", hop_orientation_callback, oid);
 
       hop_label = gtk_label_new(_("hop"));
       gtk_misc_set_alignment(GTK_MISC (hop_label), 0.1, 0.0);
@@ -1264,12 +1211,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->sx), true);
       gtk_table_attach(GTK_TABLE(sx_box), oid->sx, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->sx_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->sx_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sx_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->sx_adj, "value_changed", sx_orientation_callback, oid);
 
       sx_label = gtk_label_new(_("x scale"));
       gtk_misc_set_alignment(GTK_MISC (sx_label), 0.1, 0.0);
@@ -1294,11 +1236,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->sy), true);
       gtk_table_attach(GTK_TABLE(sy_box), oid->sy, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->sy_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->sy_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sy_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->sy_adj, "value_changed", sy_orientation_callback, oid);
 
       sy_label = gtk_label_new(_("y scale"));
       gtk_misc_set_alignment(GTK_MISC (sy_label), 0.1, 0.0);
@@ -1323,11 +1261,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->sz), true);
       gtk_table_attach(GTK_TABLE(sz_box), oid->sz, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->sz_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->sz_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sz_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->sz_adj, "value_changed", sz_orientation_callback, oid);
 
       sz_label = gtk_label_new(_("z scale"));
       gtk_misc_set_alignment(GTK_MISC (sz_label), 0.1, 0.0);
@@ -1351,11 +1285,7 @@ static void start_view_orientation_dialog(bool managed)
       gtk_scale_set_draw_value(GTK_SCALE(oid->cut), true);
       gtk_table_attach(GTK_TABLE(cut_box), oid->cut, 0, 1, 0, 1,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XX_PAD, YY_PAD);
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->cut_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(oid->cut_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(cut_orientation_callback), (gpointer)oid, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->cut_adj, "value_changed", cut_orientation_callback, oid);
 
       cut_label = gtk_label_new(_("portion of spectrum"));
       gtk_misc_set_alignment(GTK_MISC (cut_label), 0.1, 0.0);
@@ -1370,11 +1300,7 @@ static void start_view_orientation_dialog(bool managed)
       oid->glbutton = gtk_check_button_new_with_label(_("use OpenGL"));
       gtk_table_attach(GTK_TABLE(outer_table), oid->glbutton, 0, 1, 4, 5,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), X_PAD, Y_PAD);
-      g_signal_connect_closure_by_id(GTK_OBJECT(oid->glbutton),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(oid->glbutton))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(glbutton_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(oid->glbutton, "toggled", glbutton_callback, NULL);
       gtk_widget_show(oid->glbutton);
       set_toggle_button(oid->glbutton, with_gl(ss), false, NULL);
 #endif
@@ -1411,8 +1337,9 @@ bool set_with_gl(bool val)
   in_set_with_gl(val);
 #if HAVE_GL
   sgl_set_currents();
-  set_toggle_button(oid->glbutton, val, false, NULL);
-  for_each_chan(update_graph);
+  if (oid)
+    set_toggle_button(oid->glbutton, val, false, NULL);
+  /* for_each_chan(update_graph); */
 #endif
   return(with_gl(ss));
 }
