@@ -34,7 +34,7 @@ static bool ignore_button_release = false;
 axis_info *enved_make_axis(const char *name, axis_context *ax, 
 			   int ex0, int ey0, int width, int height, 
 			   Float xmin, Float xmax, Float ymin, Float ymax,
-			   bool printing)
+			   printing_t printing)
 {
   /* conjure up minimal context for axis drawer in snd-axis.c */
   if (!axis) 
@@ -54,7 +54,7 @@ axis_info *enved_make_axis(const char *name, axis_context *ax,
   return(axis);
 }
 
-static void display_env(env *e, char *name, GdkGC *cur_gc, int x0, int y0, int width, int height, bool dots, bool printing)
+static void display_env(env *e, char *name, GdkGC *cur_gc, int x0, int y0, int width, int height, bool dots, printing_t printing)
 {
   axis_context *ax = NULL;  
   ax = (axis_context *)CALLOC(1, sizeof(axis_context));
@@ -66,7 +66,7 @@ static void display_env(env *e, char *name, GdkGC *cur_gc, int x0, int y0, int w
   ax = free_axis_context(ax);
 }
 
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing)
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing)
 {
   display_env(e, name, (selected_env == e) ? rgc : gc, x0, y0, width, height, dots, printing);
 }
@@ -95,11 +95,11 @@ void set_enved_show_sensitive(bool val) {set_sensitive(showB, val);}
 void make_scrolled_env_list (void)
 {
   int n, size;
-  char *str;
   size = enved_all_envs_top();
   gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(env_list))));
   for (n = 0; n < size; n++) 
     {
+      char *str;
       str = enved_all_names(n);
       sg_list_append(env_list, str);
     }
@@ -129,10 +129,10 @@ static void dismiss_enved_callback(GtkWidget *w, gpointer context)
   gtk_widget_hide(enved_dialog);
 }
 
-static gboolean delete_enved_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint delete_enved_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(enved_dialog);
-  return(false);
+  return(true);
 }
 
 static void help_enved_callback(GtkWidget *w, gpointer context)
@@ -144,8 +144,7 @@ static bool within_selection_src = false;
 
 static void apply_enved(void)
 {
-  int i, j;
-  env *max_env = NULL;
+  char *origin = NULL, *estr = NULL;
   if (active_env)
     {
       active_channel = current_channel();
@@ -158,33 +157,49 @@ static void apply_enved(void)
 	  switch (enved_target(ss))
 	    {
 	    case ENVED_AMPLITUDE:
+	      origin = mus_format("%s %s%s", 
+				  (apply_to_selection) ? S_env_selection : S_env_channel,
+				  estr = env_to_string(active_env),
+				  (apply_to_selection) ? "" : " 0 #f");
 	      apply_env(active_channel, active_env, 0, 
 			CURRENT_SAMPLES(active_channel),
 			apply_to_selection, FROM_ENVED, 
-			"Enved: amp", NULL,
+			origin, NULL,
 			C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0);
 	      /* calls update_graph, I think, but in short files that doesn't update the amp-env */
 	      if (enved_wave_p(ss)) env_redisplay();
+	      if (estr) FREE(estr);
+	      if (origin) FREE(origin);
 	      break;
 	    case ENVED_SPECTRUM: 
+	      origin = mus_format("%s %s %d%s",
+				  (apply_to_selection) ? S_filter_selection : S_filter_channel,
+				  estr = env_to_string(active_env), enved_filter_order(ss),
+				  (apply_to_selection) ? "" : " 0 #f");
 	      apply_filter(active_channel, 
 			   (FIR_p) ? enved_filter_order(ss) : 0,
 			   active_env, 
-			   FROM_ENVED, "Enved: flt", 
+			   FROM_ENVED, origin, NULL,
 			   apply_to_selection, NULL, NULL,
 			   C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0, false);
+	      if (estr) FREE(estr);
+	      if (origin) FREE(origin);
 	      break;
 	    case ENVED_SRATE:
-	      max_env = copy_env(active_env);
-	      for (i = 0, j = 1; i < max_env->pts; i++, j += 2)
-		if (max_env->data[j] < .01) max_env->data[j] = .01;
-	      within_selection_src = true;
-	      src_env_or_num(active_channel, max_env, 0.0, 
-			     false, FROM_ENVED, "Enved: src", apply_to_selection, NULL,
-			     C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0);
-	      within_selection_src = false;
-	      max_env = free_env(max_env);
-	      if (enved_wave_p(ss)) env_redisplay();
+	      {
+		int i, j;
+		env *max_env = NULL;
+		max_env = copy_env(active_env);
+		for (i = 0, j = 1; i < max_env->pts; i++, j += 2)
+		  if (max_env->data[j] < .01) max_env->data[j] = .01;
+		within_selection_src = true;
+		src_env_or_num(active_channel, max_env, 0.0, 
+			       false, FROM_ENVED, "Enved: src", apply_to_selection, NULL,
+			       C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0);
+		within_selection_src = false;
+		max_env = free_env(max_env);
+		if (enved_wave_p(ss)) env_redisplay();
+	      }
 	      break;
 	    }
 	  set_sensitive(applyB, true);
@@ -194,9 +209,8 @@ static void apply_enved(void)
     }
 }
 
-static void env_redisplay_1(bool printing)
+static void env_redisplay_1(printing_t printing)
 {
-  char *name = NULL;
   if (enved_dialog_is_active())
     {
       gdk_window_clear(drawer->window);
@@ -204,13 +218,14 @@ static void env_redisplay_1(bool printing)
 	view_envs(env_window_width, env_window_height, printing);
       else 
 	{
+	  char *name = NULL;
 	  name = (char *)gtk_entry_get_text(GTK_ENTRY(textL));
 	  if (!name) name = _("noname");
 	  display_env(active_env, name, gc, 0, 0, env_window_width, env_window_height, true, printing);
 	  name = NULL;
 	  if (enved_wave_p(ss))
 	    {
-	      if ((enved_target(ss) == ENVED_SPECTRUM) && (active_env) && (FIR_p) && (!printing))
+	      if ((enved_target(ss) == ENVED_SPECTRUM) && (active_env) && (FIR_p) && (printing == NOT_PRINTING))
 		display_frequency_response(active_env, axis, gray_ap->ax, enved_filter_order(ss), enved_in_dB(ss));
 	      enved_show_background_waveform(axis, gray_ap, apply_to_selection, (enved_target(ss) == ENVED_SPECTRUM), printing);
 	    }
@@ -218,8 +233,8 @@ static void env_redisplay_1(bool printing)
     }
 }
 
-void env_redisplay(void) {env_redisplay_1(false);}
-void env_redisplay_with_print(void) {env_redisplay_1(true);}
+void env_redisplay(void) {env_redisplay_1(NOT_PRINTING);}
+void env_redisplay_with_print(void) {env_redisplay_1(PRINTING);}
 
 static void enved_filter_order_callback(GtkWidget *w, gpointer data)
 {
@@ -229,10 +244,10 @@ static void enved_filter_order_callback(GtkWidget *w, gpointer data)
 static void text_field_activated(GtkWidget *w, gpointer context)
 { /* might be breakpoints to load or an envelope name (<cr> in enved text field) */
   char *str = NULL;
-  env *e = NULL;
   str = (char *)gtk_entry_get_text(GTK_ENTRY(w));
   if ((str) && (*str))
     {
+      env *e = NULL;
       while (isspace((int)(*str))) str++;
       e = name_to_env(str);
       if (!e)
@@ -386,11 +401,11 @@ static gboolean drawer_button_motion(GtkWidget *w, GdkEventMotion *ev, gpointer
 
 static gboolean drawer_button_press(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
-  int pos;
   ss->enved->down_time = ev->time;
   ss->enved->env_dragged = false;
   if (showing_all_envs)
     {
+      int pos;
       pos = hit_env((int)(ev->x), (int)(ev->y), env_window_width, env_window_height);
       sg_list_select(env_list, pos);
       if ((pos >= 0) && 
@@ -470,9 +485,9 @@ static void selection_button_pressed(GtkWidget *w, gpointer context)
 
 static void delete_button_pressed(GtkWidget *w, gpointer context)
 {
-  int i, len;
   if (selected_env)
     {
+      int i, len;
       len = enved_all_envs_top();
       for (i = 0; i < len; i++)
 	if (selected_env == enved_all_envs(i))
@@ -591,13 +606,13 @@ static void env_browse_callback(GtkTreeSelection *selection, gpointer *gp)
   GtkTreeIter iter;
   gchar *value = NULL;
   int size, n;
-  char *str;
   GtkTreeModel *model;
   if (!(gtk_tree_selection_get_selected(selection, &model, &iter))) return;
   gtk_tree_model_get(model, &iter, 0, &value, -1);
   size = enved_all_envs_top();
   for (n = 0; n < size; n++) 
     {
+      char *str;
       str = enved_all_names(n);
       if (strcmp(str, value) == 0)
 	{
@@ -735,15 +750,12 @@ static void fir_button_pressed(GtkWidget *w, gpointer context)
 
 GtkWidget *create_envelope_editor (void)
 {
-  GtkWidget *mainform, *helpB, *leftbox, *bottombox, *leftframe, *toprow, *bottomrow;
   if (!enved_dialog)
     {
+      GtkWidget *mainform, *helpB, *leftbox, *bottombox, *leftframe, *toprow, *bottomrow;
+
       enved_dialog = gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(enved_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(enved_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(delete_enved_dialog), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(enved_dialog, "delete_event", delete_enved_dialog, NULL);
       gtk_window_set_title(GTK_WINDOW(enved_dialog), _("Edit Envelope"));
       sg_make_resizable(enved_dialog);
       gtk_container_set_border_width(GTK_CONTAINER(enved_dialog), 4);
@@ -778,31 +790,11 @@ GtkWidget *create_envelope_editor (void)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(enved_dialog)->action_area), apply2B, false, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(enved_dialog)->action_area), resetB, false, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(enved_dialog)->action_area), helpB, false, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(cancelB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(cancelB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(dismiss_enved_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(applyB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(applyB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(apply_enved_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(apply2B),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(apply2B))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(undo_and_apply_enved_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(resetB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(resetB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(reset_button_pressed), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(helpB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(helpB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(help_enved_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(cancelB, "clicked", dismiss_enved_callback, NULL);
+      SG_SIGNAL_CONNECT(applyB, "clicked", apply_enved_callback, NULL);
+      SG_SIGNAL_CONNECT(apply2B, "clicked", undo_and_apply_enved_callback, NULL);
+      SG_SIGNAL_CONNECT(resetB, "clicked", reset_button_pressed, NULL);
+      SG_SIGNAL_CONNECT(helpB, "clicked", help_enved_callback, NULL);
       gtk_widget_show(cancelB);
       gtk_widget_show(applyB);
       gtk_widget_show(apply2B);
@@ -835,11 +827,7 @@ GtkWidget *create_envelope_editor (void)
 
       showB = gtk_button_new_with_label(_("view envs"));
       gtk_box_pack_start(GTK_BOX(leftbox), showB, false, false, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(showB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(showB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(show_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(showB, "clicked", show_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(showB), -1, BUTTON_HEIGHT);
       gtk_widget_show(showB);
 
@@ -849,21 +837,13 @@ GtkWidget *create_envelope_editor (void)
 
       saveB = gtk_button_new_with_label(_(" save "));
       gtk_box_pack_start(GTK_BOX(saverow), saveB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(saveB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(saveB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(save_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(saveB, "clicked", save_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(saveB), -1, BUTTON_HEIGHT);
       gtk_widget_show(saveB);
 
       printB = gtk_button_new_with_label(_(" print  "));
       gtk_box_pack_start(GTK_BOX(saverow), printB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(printB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(printB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(print_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(printB, "clicked", print_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(printB), -1, BUTTON_HEIGHT);
       gtk_widget_show(printB);
 
@@ -873,21 +853,13 @@ GtkWidget *create_envelope_editor (void)
 
       revertB = gtk_button_new_with_label(_("revert "));
       gtk_box_pack_start(GTK_BOX(revrow), revertB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(revertB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(revertB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(revert_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(revertB, "clicked", revert_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(revertB), -1, BUTTON_HEIGHT);
       gtk_widget_show(revertB);
 
       deleteB = gtk_button_new_with_label(_("delete"));
       gtk_box_pack_start(GTK_BOX(revrow), deleteB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(deleteB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(deleteB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(delete_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(deleteB, "clicked", delete_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(deleteB), -1, BUTTON_HEIGHT);
       gtk_widget_show(deleteB);
 
@@ -897,21 +869,13 @@ GtkWidget *create_envelope_editor (void)
 
       undoB = gtk_button_new_with_label(_(" undo "));
       gtk_box_pack_start(GTK_BOX(unrow), undoB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(undoB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(undoB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(undo_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(undoB, "clicked", undo_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(undoB), -1, BUTTON_HEIGHT);
       gtk_widget_show(undoB);
 
       redoB = gtk_button_new_with_label(_(" redo "));
       gtk_box_pack_start(GTK_BOX(unrow), redoB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(redoB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(redoB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(redo_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(redoB, "clicked", redo_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(redoB), -1, BUTTON_HEIGHT);
       gtk_widget_show(redoB);
 
@@ -921,31 +885,19 @@ GtkWidget *create_envelope_editor (void)
 
       ampB = gtk_button_new_with_label(_("amp"));
       gtk_box_pack_start(GTK_BOX(rbrow), ampB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(ampB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(ampB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(amp_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(ampB, "clicked", amp_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(ampB), -1, BUTTON_HEIGHT);
       gtk_widget_show(ampB);
 
       fltB = gtk_button_new_with_label(_("flt"));
       gtk_box_pack_start(GTK_BOX(rbrow), fltB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(fltB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(fltB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(flt_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(fltB, "clicked", flt_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(fltB), -1, BUTTON_HEIGHT);
       gtk_widget_show(fltB);
 
       srcB = gtk_button_new_with_label(_("src"));
       gtk_box_pack_start(GTK_BOX(rbrow), srcB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(srcB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(srcB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(src_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(srcB, "clicked", src_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(srcB), -1, BUTTON_HEIGHT);
       gtk_widget_show(srcB);
 
@@ -955,21 +907,13 @@ GtkWidget *create_envelope_editor (void)
 
       linB = gtk_button_new_with_label(_("lin"));
       gtk_box_pack_start(GTK_BOX(lerow), linB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(linB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(linB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(lin_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(linB, "clicked", lin_button_pressed, NULL);
       gtk_widget_show(linB);
       gtk_widget_set_size_request(GTK_WIDGET(linB), -1, BUTTON_HEIGHT);
 
       expB = gtk_button_new_with_label(_("exp"));
       gtk_box_pack_start(GTK_BOX(lerow), expB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(expB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(expB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(exp_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(expB, "clicked", exp_button_pressed, NULL);
       gtk_widget_show(expB);
       gtk_widget_set_size_request(GTK_WIDGET(expB), -1, BUTTON_HEIGHT);
 
@@ -979,11 +923,7 @@ GtkWidget *create_envelope_editor (void)
 
       selectionB = gtk_button_new_with_label(_("selection"));
       gtk_box_pack_start(GTK_BOX(selrow), selectionB, true, true, BB_MARGIN);
-      g_signal_connect_closure_by_id(GTK_OBJECT(selectionB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(selectionB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(selection_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(selectionB, "clicked", selection_button_pressed, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(selectionB), -1, BUTTON_HEIGHT);
       gtk_widget_show(selectionB);
 
@@ -1004,11 +944,7 @@ GtkWidget *create_envelope_editor (void)
       gtk_widget_show(nameL);
 
       textL = snd_entry_new(toprow, true);
-      g_signal_connect_closure_by_id(GTK_OBJECT(textL),
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(textL))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(text_field_activated), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(textL, "activate", text_field_activated, NULL);
 
       blank = gdk_pixmap_create_from_xpm_d(MAIN_WINDOW(ss), NULL, NULL, blank_bits());
       brkpixL = gtk_drawing_area_new();
@@ -1016,40 +952,24 @@ GtkWidget *create_envelope_editor (void)
       gtk_widget_set_size_request(brkpixL, 16, 16);
       gtk_box_pack_start(GTK_BOX(toprow), brkpixL, false, false, 0);
       gtk_widget_show(brkpixL);
-      g_signal_connect_closure_by_id(GTK_OBJECT(brkpixL),
-				     g_signal_lookup("expose_event", G_OBJECT_TYPE(GTK_OBJECT(brkpixL))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(brkpixL_expose), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(brkpixL, "expose_event", brkpixL_expose, NULL);
       
       brktxtL = gtk_label_new(NULL);
       gtk_box_pack_start(GTK_BOX(toprow), brktxtL, false, false, 0);
       gtk_widget_show(brktxtL);
 
       clipB = gtk_check_button_new_with_label(_("clip"));
-      g_signal_connect_closure_by_id(GTK_OBJECT(clipB),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(clipB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(clip_button_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(clipB, "toggled", clip_button_callback, NULL);
       gtk_box_pack_start(GTK_BOX(toprow), clipB, false, false, 0);
       gtk_widget_show(clipB);
 
       graphB = gtk_check_button_new_with_label(_("wave"));
-      g_signal_connect_closure_by_id(GTK_OBJECT(graphB),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(graphB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(graph_button_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(graphB, "toggled", graph_button_callback, NULL);
       gtk_box_pack_start(GTK_BOX(toprow), graphB, false, false, 0);
       gtk_widget_show(graphB);
 
       dBB = gtk_check_button_new_with_label(_("dB"));
-      g_signal_connect_closure_by_id(GTK_OBJECT(dBB),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(dBB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(dB_button_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(dBB, "toggled", dB_button_callback, NULL);
       gtk_box_pack_start(GTK_BOX(toprow), dBB, false, false, 0);
       gtk_widget_show(dBB);
 
@@ -1065,11 +985,7 @@ GtkWidget *create_envelope_editor (void)
       baseAdj = gtk_adjustment_new(0.5, 0.0, 1.0, 0.001, 0.01, .1);
       baseScale = gtk_hscrollbar_new(GTK_ADJUSTMENT(baseAdj));
       gtk_widget_modify_bg(baseScale, GTK_STATE_NORMAL, (ss->sgx)->position_color);
-      g_signal_connect_closure_by_id(GTK_OBJECT(baseAdj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(baseAdj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(base_changed_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(baseAdj, "value_changed", base_changed_callback, NULL);
       gtk_box_pack_start(GTK_BOX(bottomrow), baseScale, true, true, 4);
       gtk_widget_show(baseScale);
 
@@ -1077,50 +993,22 @@ GtkWidget *create_envelope_editor (void)
       orderL = gtk_spin_button_new(GTK_ADJUSTMENT(orderAdj), 0.0, 0);
       gtk_box_pack_end(GTK_BOX(bottomrow), orderL, false, false, 4);
       gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(orderL), true);
-      g_signal_connect_closure_by_id(GTK_OBJECT(orderAdj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(orderAdj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(enved_filter_order_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(orderAdj, "value_changed", enved_filter_order_callback, NULL);
       gtk_widget_show(orderL);
 
       firB = gtk_button_new_with_label((FIR_p) ? "fir" : "fft");
-      g_signal_connect_closure_by_id(GTK_OBJECT(firB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(firB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(fir_button_pressed), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(firB, "clicked", fir_button_pressed, NULL);
       gtk_box_pack_end(GTK_BOX(bottomrow), firB, false, false, 0);
       gtk_widget_show(firB);
 
       gtk_widget_show(mainform);
       gtk_widget_show(enved_dialog);
 
-      g_signal_connect_closure_by_id(GTK_OBJECT(drawer),
-				     g_signal_lookup("expose_event", G_OBJECT_TYPE(GTK_OBJECT(drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(drawer_expose), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(drawer),
-				     g_signal_lookup("configure_event", G_OBJECT_TYPE(GTK_OBJECT(drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(drawer_resize), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(drawer),
-				     g_signal_lookup("button_press_event", G_OBJECT_TYPE(GTK_OBJECT(drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(drawer_button_press), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(drawer),
-				     g_signal_lookup("button_release_event", G_OBJECT_TYPE(GTK_OBJECT(drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(drawer_button_release), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(drawer),
-				     g_signal_lookup("motion_notify_event", G_OBJECT_TYPE(GTK_OBJECT(drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(drawer_button_motion), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(drawer, "expose_event", drawer_expose, NULL);
+      SG_SIGNAL_CONNECT(drawer, "configure_event", drawer_resize, NULL);
+      SG_SIGNAL_CONNECT(drawer, "button_press_event", drawer_button_press, NULL);
+      SG_SIGNAL_CONNECT(drawer, "button_release_event", drawer_button_release, NULL);
+      SG_SIGNAL_CONNECT(drawer, "motion_notify_event", drawer_button_motion, NULL);
 
       if (enved_all_envs_top() == 0)
 	{
@@ -1188,7 +1076,6 @@ bool enved_dialog_is_active(void)
 
 void set_enved_filter_order(int order)
 {
-  char str[LABEL_BUFFER_SIZE];
   if ((order > 0) && (order < 2000))
     {
       if (order & 1) 
@@ -1196,8 +1083,7 @@ void set_enved_filter_order(int order)
       else in_set_enved_filter_order(order);
       if (enved_dialog)
 	{
-	  mus_snprintf(str, LABEL_BUFFER_SIZE, "%d", enved_filter_order(ss));
-	  gtk_entry_set_text(GTK_ENTRY(orderL), str);
+	  widget_int_to_text(orderL, enved_filter_order(ss));
 	  if ((enved_target(ss) == ENVED_SPECTRUM) && 
 	      (enved_wave_p(ss)) && 
 	      (!showing_all_envs)) 
@@ -1306,9 +1192,9 @@ static XEN g_enved_dialog_widgets(void)
 
 static XEN g_enved_axis_info(void)
 {
-  axis_info *ap;
   if (enved_dialog)
     {
+      axis_info *ap;
       if (axis == NULL)
 	enved_reset();
       ap = axis;
