@@ -31,12 +31,12 @@ static char *FFT_WINDOWS[NUM_FFT_WINDOWS] =
   {"Rectangular", "Hann", "Welch", "Parzen", "Bartlett", "Hamming", "Blackman2", "Blackman3", "Blackman4",
    "Exponential", "Riemann", "Kaiser", "Cauchy", "Poisson", "Gaussian", "Tukey", "Dolph-Chebyshev", "Hann-Poisson", "Connes"};
 
-#define NUM_TRANSFORM_TYPES 7
-static char *TRANSFORM_TYPES[NUM_TRANSFORM_TYPES] = {"Fourier", "Wavelet", "Walsh", "Autocorrelate", "Cepstrum", "Hadamard", "Haar"};
+#define NUM_TRANSFORM_TYPES 6
+static char *TRANSFORM_TYPES[NUM_TRANSFORM_TYPES] = {"Fourier", "Wavelet", "Walsh", "Autocorrelate", "Cepstrum", "Haar"};
 static int num_transform_types = NUM_TRANSFORM_TYPES;
 
 static char *TRANSFORM_TYPE_CONSTANTS[NUM_TRANSFORM_TYPES] = {
-  S_fourier_transform, S_wavelet_transform, S_walsh_transform, S_autocorrelation, S_cepstrum, S_hadamard_transform, S_haar_transform};
+  S_fourier_transform, S_wavelet_transform, S_walsh_transform, S_autocorrelation, S_cepstrum, S_haar_transform};
 
 char *transform_type_name(int choice)
 {
@@ -49,7 +49,7 @@ int max_transform_type(void) {return(num_transform_types - 1);}
 
 static Float fp_dB(Float py)
 {
-  return((py <= ss->lin_dB) ? 0.0 : (1.0 - ((20.0 * (log10(py))) / ss->min_dB)));
+  return((py <= ss->lin_dB) ? 0.0 : (1.0 - (20.0 * log10(py) / ss->min_dB)));
 }
 
 static axis_info *axis_ap = NULL;
@@ -96,7 +96,7 @@ static void graph_redisplay(void)
   axis_ap->height = widget_height(graph_drawer);
   axis_ap->graph_x0 = 0;
   gdk_window_clear(ax->wn);
-  make_axes_1(axis_ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, false, true);
+  make_axes_1(axis_ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, NOT_PRINTING, WITH_X_AXIS, NO_GRID, WITH_LINEAR_AXES, grid_density(ss));
   ax->gc = gc;
   ix1 = grf_x(0.0, axis_ap);
   iy1 = grf_y(current_graph_data[0], axis_ap);
@@ -139,12 +139,12 @@ static void get_fft_window_data(void)
 
 static void chans_transform_size(chan_info *cp, void *ptr) 
 {
-  fft_info *fp;
   int size;
   size = (*((int *)ptr)); 
   cp->transform_size = size;
   if (cp->fft) 
     {
+      fft_info *fp;
       fp = cp->fft;
       if (fp->size < size) fp->ok = false; /* "dirty" flag for fft data array = needs reallocation */
       fp->size = size;
@@ -367,14 +367,16 @@ static void beta_callback(GtkAdjustment *adj, gpointer context)
     }
 } 
 
-static void graph_configure_callback(GtkWidget *w, GdkEventConfigure *ev, gpointer data)
+static gboolean graph_configure_callback(GtkWidget *w, GdkEventConfigure *ev, gpointer data)
 {
   graph_redisplay();
+  return(false);
 }
 
-static void graph_expose_callback(GtkWidget *w, GdkEventExpose *ev, gpointer data)
+static gboolean graph_expose_callback(GtkWidget *w, GdkEventExpose *ev, gpointer data)
 {
   graph_redisplay();
+  return(false);
 }
 
 static void dismiss_transform_callback(GtkWidget *w, gpointer context)
@@ -392,9 +394,10 @@ static void color_transform_callback(GtkWidget *w, gpointer context)
   start_color_dialog(true);
 }
 
-static void delete_transform_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint delete_transform_dialog(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(transform_dialog);
+  return(true);
 }
 
 static void help_transform_callback(GtkWidget *w, gpointer context)
@@ -402,26 +405,65 @@ static void help_transform_callback(GtkWidget *w, gpointer context)
   transform_dialog_help();
 }
 
+static GtkWidget *db_txt, *peaks_txt, *lf_txt;
+
+static void max_peaks_callback(GtkWidget *w, gpointer data)
+{
+  int new_peaks;
+  new_peaks = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(data));
+  set_max_transform_peaks(new_peaks);
+  for_each_chan(calculate_fft);
+}
+
+static void min_db_callback(GtkWidget *w, gpointer data)
+{
+  Float new_db;
+  new_db = gtk_spin_button_get_value(GTK_SPIN_BUTTON(data));
+  set_min_db(-new_db);
+}
+
+static void log_freq_callback(GtkWidget *w, gpointer data)
+{
+  Float new_lfb;
+  new_lfb = gtk_spin_button_get_value(GTK_SPIN_BUTTON(data));
+  set_log_freq_start(new_lfb);
+}
+
+void reflect_peaks_in_transform_dialog(void) 
+{
+  if (transform_dialog)
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(peaks_txt), max_transform_peaks(ss));
+}
+
+void reflect_log_freq_start_in_transform_dialog(void) 
+{
+  if (transform_dialog)
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(lf_txt), log_freq_start(ss));
+}
+
+void reflect_min_db_in_transform_dialog(void) 
+{
+  if (transform_dialog)
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(db_txt), (gfloat)(-(ss->min_dB)));
+}
+
+
 #define BUTTON_HEIGHT 20
 #define BUTTON_WIDTH 40
 /* for some reason gtk puts a mile and a half of padding around buttons */
 
 GtkWidget *fire_up_transform_dialog(bool managed)
 {
-  GtkWidget *outer_table, *buttons;
-  int i;
   bool need_callback = false;
-  GtkWidget *display_frame, *help_button, *dismiss_button;
-  GtkWidget *window_box, *orient_button, *color_button;
-
   if (!transform_dialog)
     {
+      GtkWidget *outer_table, *buttons;
+      int i;
+      GtkWidget *display_frame, *help_button, *dismiss_button;
+      GtkWidget *window_box, *orient_button, *color_button;
+
       transform_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(transform_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(transform_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(delete_transform_dialog), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(transform_dialog, "delete_event", delete_transform_dialog, NULL);
       gtk_window_set_title(GTK_WINDOW(transform_dialog), _("Transform Options"));
       sg_make_resizable(transform_dialog);
       gtk_container_set_border_width(GTK_CONTAINER(transform_dialog), 4);
@@ -440,32 +482,16 @@ GtkWidget *fire_up_transform_dialog(bool managed)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(transform_dialog)->action_area), color_button, false, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(transform_dialog)->action_area), orient_button, false, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(transform_dialog)->action_area), help_button, false, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(dismiss_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(dismiss_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(dismiss_transform_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(color_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(color_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(color_transform_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(orient_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(orient_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(orient_transform_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(help_transform_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(dismiss_button, "clicked", dismiss_transform_callback, NULL);
+      SG_SIGNAL_CONNECT(color_button, "clicked", color_transform_callback, NULL);
+      SG_SIGNAL_CONNECT(orient_button, "clicked", orient_transform_callback, NULL);
+      SG_SIGNAL_CONNECT(help_button, "clicked", help_transform_callback, NULL);
       gtk_widget_show(dismiss_button);
       gtk_widget_show(color_button);
       gtk_widget_show(orient_button);
       gtk_widget_show(help_button);
 
-      outer_table = gtk_table_new(2, 3, false);
+      outer_table = gtk_table_new(6, 3, false);
       gtk_container_add(GTK_CONTAINER(GTK_DIALOG(transform_dialog)->vbox), outer_table);
       gtk_table_set_row_spacings(GTK_TABLE(outer_table), 4);
       gtk_table_set_col_spacings(GTK_TABLE(outer_table), 4);
@@ -486,23 +512,19 @@ GtkWidget *fire_up_transform_dialog(bool managed)
 	    else transform_names[i] = added_transform_name(i);
 	  }
 	transform_list = sg_make_list(_("type"), outer_table, TABLE_ATTACH, NULL, num_transform_types, transform_names, 
-				      GTK_SIGNAL_FUNC(transform_browse_callback), 0, 1, 0, 1);
+				      GTK_SIGNAL_FUNC(transform_browse_callback), 0, 1, 0, 3);
 	gtk_widget_show(transform_list);
 	FREE(transform_names);
       }
 
       /* SIZE */
       size_list = sg_make_list(_("size"), outer_table, TABLE_ATTACH, NULL, NUM_TRANSFORM_SIZES, TRANSFORM_SIZES, 
-			       GTK_SIGNAL_FUNC(size_browse_callback), 1, 2, 0, 1);
+			       GTK_SIGNAL_FUNC(size_browse_callback), 1, 2, 0, 3);
       gtk_widget_show(size_list);
 
       /* DISPLAY */
       display_frame = gtk_frame_new(_("display"));
-      /* gtk_table_attach_defaults(GTK_TABLE(outer_table), display_frame, 2, 3, 0, 1); */
-      gtk_table_attach(GTK_TABLE(outer_table), display_frame, 2, 3, 0, 1,
-		       (GtkAttachOptions)(GTK_FILL | GTK_SHRINK), 
-		       (GtkAttachOptions)(GTK_FILL | GTK_SHRINK),
-		       0, 0);
+      gtk_table_attach_defaults(GTK_TABLE(outer_table), display_frame, 2, 3, 0, 4);
       gtk_frame_set_label_align(GTK_FRAME(display_frame), 0.5, 0.0);
       gtk_frame_set_shadow_type(GTK_FRAME(display_frame), GTK_SHADOW_ETCHED_IN);
 
@@ -512,94 +534,100 @@ GtkWidget *fire_up_transform_dialog(bool managed)
       normal_fft_button = gtk_radio_button_new_with_label(NULL, _("single transform"));
       gtk_box_pack_start(GTK_BOX(buttons), normal_fft_button, false, false, 0);
       gtk_widget_show(normal_fft_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(normal_fft_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(normal_fft_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(normal_fft_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(normal_fft_button, "clicked", normal_fft_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(normal_fft_button), BUTTON_WIDTH, BUTTON_HEIGHT);
 
       sono_button = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(normal_fft_button)), _("sonogram"));
       gtk_box_pack_start(GTK_BOX(buttons), sono_button, false, false, 0);
       gtk_widget_show(sono_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(sono_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(sono_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(sonogram_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(sono_button, "clicked", sonogram_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(sono_button), BUTTON_WIDTH, BUTTON_HEIGHT);
 
       spectro_button = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(normal_fft_button)), _("spectrogram"));
       gtk_box_pack_start(GTK_BOX(buttons), spectro_button, false, false, 0);
       gtk_widget_show(spectro_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(spectro_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(spectro_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(spectrogram_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(spectro_button, "clicked", spectrogram_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(spectro_button), BUTTON_WIDTH, BUTTON_HEIGHT);
       
       peaks_button = gtk_check_button_new_with_label(_("peaks"));
       gtk_box_pack_start(GTK_BOX(buttons), peaks_button, false, false, 0);
       gtk_widget_show(peaks_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(peaks_button),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(peaks_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(peaks_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(peaks_button, "toggled", peaks_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(peaks_button), BUTTON_WIDTH, BUTTON_HEIGHT);
  
       db_button = gtk_check_button_new_with_label(_("dB"));
       gtk_box_pack_start(GTK_BOX(buttons), db_button, false, false, 0);
       gtk_widget_show(db_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(db_button),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(db_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(db_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(db_button, "toggled", db_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(db_button), BUTTON_WIDTH, BUTTON_HEIGHT);
  
       logfreq_button = gtk_check_button_new_with_label(_("log freq"));
       gtk_box_pack_start(GTK_BOX(buttons), logfreq_button, false, false, 0);
       gtk_widget_show(logfreq_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(logfreq_button),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(logfreq_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(logfreq_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(logfreq_button, "toggled", logfreq_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(logfreq_button), BUTTON_WIDTH, BUTTON_HEIGHT);
 
       normalize_button = gtk_check_button_new_with_label(_("normalize"));
       gtk_box_pack_start(GTK_BOX(buttons), normalize_button, false, false, 0);
       gtk_widget_show(normalize_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(normalize_button),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(normalize_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(normalize_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(normalize_button, "toggled", normalize_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(normalize_button), BUTTON_WIDTH, BUTTON_HEIGHT);
 
       selection_button = gtk_check_button_new_with_label(_("selection"));
       gtk_box_pack_start(GTK_BOX(buttons), selection_button, false, false, 0);
       gtk_widget_show(selection_button);
-      g_signal_connect_closure_by_id(GTK_OBJECT(selection_button),
-				     g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(selection_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(selection_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(selection_button, "toggled", selection_callback, NULL);
       gtk_widget_set_size_request(GTK_WIDGET(selection_button), BUTTON_WIDTH, BUTTON_HEIGHT);
 
+      {
+	GtkWidget *pk_lab, *db_lab, *lf_lab;
+	GtkObject *pk_vals, *db_vals, *lf_vals;
+
+	pk_lab = gtk_label_new(_("max peaks:"));
+	gtk_box_pack_start(GTK_BOX(buttons), pk_lab, false, false, 0);
+	gtk_widget_show(pk_lab);
+      
+	pk_vals = gtk_adjustment_new(max_transform_peaks(ss), 2, 1000, 2, 10, 0);
+	peaks_txt = gtk_spin_button_new(GTK_ADJUSTMENT(pk_vals), 0.0, 0);
+	gtk_box_pack_start(GTK_BOX(buttons), peaks_txt, false, false, 0);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(peaks_txt), true);
+	SG_SIGNAL_CONNECT(pk_vals, "value_changed", max_peaks_callback, (gpointer)peaks_txt);
+	gtk_widget_show(peaks_txt);
+
+	db_lab = gtk_label_new(_("min dB:"));
+	gtk_box_pack_start(GTK_BOX(buttons), db_lab, false, false, 0);
+	gtk_widget_show(db_lab);
+      
+	db_vals = gtk_adjustment_new((int)(-(ss->min_dB)), 2, 1000, 2, 10, 0); /* can't be negative!! */
+	db_txt = gtk_spin_button_new(GTK_ADJUSTMENT(db_vals), 0.0, 0);
+	gtk_box_pack_start(GTK_BOX(buttons), db_txt, false, false, 0);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(db_txt), true);
+	SG_SIGNAL_CONNECT(db_vals, "value_changed", min_db_callback, (gpointer)db_txt);
+	gtk_widget_show(db_txt);
+
+	lf_lab = gtk_label_new(_("log freq start:"));
+	gtk_box_pack_start(GTK_BOX(buttons), lf_lab, false, false, 0);
+	gtk_widget_show(lf_lab);
+      
+	lf_vals = gtk_adjustment_new((int)(log_freq_start(ss)), 1, 1000, 1, 10, 0);
+	lf_txt = gtk_spin_button_new(GTK_ADJUSTMENT(lf_vals), 0.0, 0);
+	gtk_box_pack_start(GTK_BOX(buttons), lf_txt, false, false, 0);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(lf_txt), true);
+	SG_SIGNAL_CONNECT(lf_vals, "value_changed", log_freq_callback, (gpointer)lf_txt);
+	gtk_widget_show(lf_txt);
+      }
+
       gtk_widget_show(buttons);
       gtk_widget_show(display_frame);
 
       /* WAVELET */
       wavelet_list = sg_make_list(_("wavelet"), outer_table, TABLE_ATTACH, NULL, NUM_WAVELETS, wavelet_names(),
-				  GTK_SIGNAL_FUNC(wavelet_browse_callback), 0, 1, 1, 2);
+				  GTK_SIGNAL_FUNC(wavelet_browse_callback), 0, 1, 3, 6);
       gtk_widget_show(wavelet_list);
 
       /* WINDOW */
       window_box = gtk_table_new(2, 2, false);
-      gtk_table_attach_defaults(GTK_TABLE(outer_table), window_box, 1, 2, 1, 2);
+      gtk_table_attach_defaults(GTK_TABLE(outer_table), window_box, 1, 2, 3, 6);
       window_list = sg_make_list(_("window"), window_box, TABLE_ATTACH, NULL, GUI_NUM_FFT_WINDOWS, FFT_WINDOWS, 
 				 GTK_SIGNAL_FUNC(window_browse_callback), 0, 1, 0, 1);
 
@@ -610,11 +638,7 @@ GtkWidget *fire_up_transform_dialog(bool managed)
       gtk_scale_set_digits(GTK_SCALE(window_beta_scale), 2);
       gtk_scale_set_value_pos(GTK_SCALE(window_beta_scale), GTK_POS_TOP);
       gtk_scale_set_draw_value(GTK_SCALE(window_beta_scale), true);
-      g_signal_connect_closure_by_id(GTK_OBJECT(beta_adj),
-				     g_signal_lookup("value_changed", G_OBJECT_TYPE(GTK_OBJECT(beta_adj))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(beta_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(beta_adj, "value_changed", beta_callback, NULL);
       gtk_table_attach(GTK_TABLE(window_box), window_beta_scale, 0, 1, 1, 2,
 		       (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 
 		       (GtkAttachOptions)(GTK_FILL), 
@@ -627,7 +651,7 @@ GtkWidget *fire_up_transform_dialog(bool managed)
 
       /* GRAPH */
       graph_frame = gtk_frame_new(FFT_WINDOWS[(int)fft_window(ss)]);
-      gtk_table_attach_defaults(GTK_TABLE(outer_table), graph_frame, 2, 3, 1, 2);
+      gtk_table_attach_defaults(GTK_TABLE(outer_table), graph_frame, 2, 3, 4, 6);
       gtk_frame_set_label_align(GTK_FRAME(graph_frame), 0.5, 0.0);
       gtk_frame_set_shadow_type(GTK_FRAME(graph_frame), GTK_SHADOW_ETCHED_IN);
 
@@ -680,16 +704,8 @@ GtkWidget *fire_up_transform_dialog(bool managed)
   if (need_callback)
     {
       get_fft_window_data();
-      g_signal_connect_closure_by_id(GTK_OBJECT(graph_drawer),
-				     g_signal_lookup("expose_event", G_OBJECT_TYPE(GTK_OBJECT(graph_drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(graph_expose_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(graph_drawer),
-				     g_signal_lookup("configure_event", G_OBJECT_TYPE(GTK_OBJECT(graph_drawer))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(graph_configure_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(graph_drawer, "expose_event", graph_expose_callback, NULL);
+      SG_SIGNAL_CONNECT(graph_drawer, "configure_event", graph_configure_callback, NULL);
       need_callback = false;
     }
   return(transform_dialog);
@@ -716,12 +732,12 @@ void set_fft_window_beta(Float val)
 
 void set_transform_size(int val)
 {
-  int i;
   for_each_chan(force_fft_clear);
   in_set_transform_size(val);
   for_each_chan_1(chans_transform_size, (void *)(&val));
   if (transform_dialog)
     {
+      int i;
       for (i = 0; i < NUM_TRANSFORM_SIZES; i++)
 	if (transform_sizes[i] == val)
 	  {
