@@ -103,19 +103,9 @@ static void load_header_and_data_lists(file_data *fdat, int type, int format, in
   sg_list_select(fdat->format_list, fdat->format_pos);
   g_signal_handlers_unblock_matched(GTK_OBJECT(fdat->format_list), G_SIGNAL_MATCH_DATA, 0, 0, NULL, 0, (gpointer)fdat);
   if ((srate > 0) && (fdat->srate_text))
-    {
-      str = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
-      mus_snprintf(str, LABEL_BUFFER_SIZE, "%d", srate);
-      gtk_entry_set_text(GTK_ENTRY(fdat->srate_text), str);
-      FREE(str);
-    }
+    widget_int_to_text(fdat->srate_text, srate);
   if ((chans > 0) && (fdat->chans_text))
-    {
-      str = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
-      mus_snprintf(str, LABEL_BUFFER_SIZE, "%d", chans);
-      gtk_entry_set_text(GTK_ENTRY(fdat->chans_text), str);
-      FREE(str);
-    }
+    widget_int_to_text(fdat->chans_text, chans);
   if (fdat->comment_text) 
     {
       if (GTK_IS_TEXT_VIEW(fdat->comment_text))
@@ -126,19 +116,9 @@ static void load_header_and_data_lists(file_data *fdat, int type, int format, in
       else gtk_entry_set_text(GTK_ENTRY(fdat->comment_text), comment);
     }
   if ((location >= 0) && (fdat->location_text))
-    {
-      str = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
-      mus_snprintf(str, LABEL_BUFFER_SIZE, OFF_TD, location);
-      gtk_entry_set_text(GTK_ENTRY(fdat->location_text), str);
-      FREE(str);
-    }
+    widget_off_t_to_text(fdat->location_text, location);
   if ((samples >= 0) && (fdat->samples_text))
-    {
-      str = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
-      mus_snprintf(str, LABEL_BUFFER_SIZE, OFF_TD, samples);
-      gtk_entry_set_text(GTK_ENTRY(fdat->samples_text), str);
-      FREE(str);
-    }
+    widget_off_t_to_text(fdat->samples_text, samples);
 }
 
 #if HAVE_GFCDN
@@ -198,15 +178,10 @@ static GtkWidget *snd_filer_new(char *title, bool saving, GtkSignalFunc gdelete,
   gtk_window_set_default_size(GTK_WINDOW(new_dialog), 600, 400);
 
   set_user_data(G_OBJECT(new_dialog), (gpointer)wrap_filer_callbacks(ok, cancel, help));
-  g_signal_connect(GTK_OBJECT(new_dialog), "response", G_CALLBACK(chooser_response_callback), NULL);
+  SG_SIGNAL_CONNECT(new_dialog, "response", chooser_response_callback, NULL);
 
   /* this has to be separate (not handled as a "response" because the latter deletes the goddamn widget!! */
-  g_signal_connect_closure_by_id(GTK_OBJECT(new_dialog),
-				 g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(new_dialog))),
-				 0,
-				 g_cclosure_new(gdelete, NULL, 0),
-				 0);
-
+  SG_SIGNAL_CONNECT(new_dialog, "delete_event", gdelete, NULL);
   return(new_dialog);
 }
 
@@ -251,21 +226,9 @@ static GtkWidget *snd_filer_new(char *title, bool saving, GtkSignalFunc gdelete,
   gtk_widget_modify_bg(entry, GTK_STATE_NORMAL, ss->sgx->white);
   connect_mouse_to_text(entry);
 
-  g_signal_connect_closure_by_id(GTK_OBJECT(new_dialog),
-				 g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(new_dialog))),
-				 0,
-				 g_cclosure_new(gdelete, NULL, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(filer->ok_button),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(filer->ok_button))),
-				 0,
-				 g_cclosure_new(ok, NULL, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(filer->cancel_button), 
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(filer->cancel_button))),
-				 0,
-				 g_cclosure_new_swap(cancel, NULL, 0),
-				 0);
+  SG_SIGNAL_CONNECT(new_dialog, "delete_event", gdelete, NULL);
+  SG_SIGNAL_CONNECT(filer->ok_button, "clicked", ok, NULL);
+  SG_SIGNAL_CONNECT(filer->cancel_button, "clicked", cancel, NULL);
 
   gtk_widget_set_name(filer->ok_button, "doit_button");
   gtk_widget_set_name(filer->cancel_button, "quit_button");
@@ -301,15 +264,15 @@ void alert_new_file(void) {}
 static void update_preview_callback(GtkFileChooser *chooser)
 {
   char *filename;
-  char *buf;
-  char timestr[64];
-  time_t date;
   gboolean have_preview = false;
-  file_dialog_info *fd;
   filename = (char *)gtk_file_chooser_get_preview_filename(chooser);
-  fd = (file_dialog_info *)g_object_get_data(G_OBJECT(chooser), "snd-dialog");
   if ((filename) && (sound_file_p(filename)))
     {
+      char *buf;
+      char timestr[64];
+      time_t date;
+      file_dialog_info *fd;
+      fd = (file_dialog_info *)g_object_get_data(G_OBJECT(chooser), "snd-dialog");
       have_preview = true;
       buf = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
       mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s: %d chan%s, %d Hz, %.3f secs",
@@ -344,13 +307,13 @@ static void update_preview_callback(GtkFileChooser *chooser)
 static void dialog_select_callback(GtkTreeSelection *selection, gpointer context)
 {
   char *filename;
-  char *buf;
-  char timestr[64];
-  time_t date;
   file_dialog_info *fd = (file_dialog_info *)context;
   filename = snd_filer_get_filename(fd->dialog);
   if ((filename) && (sound_file_p(filename)))
     {
+      char *buf;
+      char timestr[64];
+      time_t date;
       buf = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
       mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s: %d chan%s, %d Hz, %.3f secs",
 		   filename_without_home_directory(filename),
@@ -398,9 +361,9 @@ void clear_deleted_snd_info(void *data)
 static void play_selected_callback(GtkWidget *w, gpointer data)
 {
   file_dialog_info *fd = (file_dialog_info *)data;
-  char *filename;
   if (GTK_TOGGLE_BUTTON(w)->active)
     {
+      char *filename;
       if ((fd->file_play_sp) && (fd->file_play_sp->playing)) 
 	stop_playing_sound(fd->file_play_sp, PLAY_BUTTON_UNSET);
       filename = snd_filer_get_filename(fd->dialog);
@@ -409,10 +372,7 @@ static void play_selected_callback(GtkWidget *w, gpointer data)
 	  fd->file_play_sp = make_sound_readable(filename, false);
 	  fd->file_play_sp->delete_me = (void *)fd;
 	  if (fd->file_play_sp)
-	    play_sound(fd->file_play_sp, 0, 
-		       NO_END_SPECIFIED, IN_BACKGROUND, 
-		       C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 
-		       "selected file play", 0);
+	    play_sound(fd->file_play_sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 	}
     }
   else file_dialog_stop_playing(fd);
@@ -452,13 +412,8 @@ static file_dialog_info *make_file_dialog(int read_only, char *title, snd_dialog
   gtk_box_pack_start(GTK_BOX(fd->dialog_vbox), fd->dialog_info2, false, true, 2);
   fd->playb = gtk_check_button_new_with_label(_("play selected sound"));
   gtk_box_pack_start(GTK_BOX(fd->dialog_vbox), fd->playb, false, true, 2);
-  g_signal_connect_closure_by_id(GTK_OBJECT(fd->playb),
-				 g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(fd->playb))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(play_selected_callback), (gpointer)fd, 0),
-				 0);
-
-  g_signal_connect(fd->dialog, "update-preview", G_CALLBACK(update_preview_callback), NULL);
+  SG_SIGNAL_CONNECT(fd->playb, "toggled", play_selected_callback, fd);
+  SG_SIGNAL_CONNECT(fd->dialog, "update-preview", update_preview_callback, NULL);
 
   all_files_filter = gtk_file_filter_new();
   gtk_file_filter_set_name(all_files_filter, "All Files");
@@ -502,16 +457,11 @@ static file_dialog_info *make_file_dialog(int read_only, char *title, snd_dialog
 	
   fd->dialog_info2 = gtk_label_new(NULL);
   gtk_box_pack_start(GTK_BOX(fd->dialog_vbox), fd->dialog_info2, true, true, 2);
-  g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(SND_FILER(fd->dialog)->file_list)), "changed",
-		   G_CALLBACK (dialog_select_callback), fd);
+  SG_SIGNAL_CONNECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(SND_FILER(fd->dialog)->file_list)), "changed", dialog_select_callback, fd);
 
   fd->playb = gtk_check_button_new_with_label(_("play selected sound"));
   gtk_box_pack_start(GTK_BOX(fd->dialog_vbox), fd->playb, true, true, 2);
-  g_signal_connect_closure_by_id(GTK_OBJECT(fd->playb),
-				 g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(fd->playb))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(play_selected_callback), (gpointer)fd, 0),
-				 0);
+  SG_SIGNAL_CONNECT(fd->playb, "toggled", play_selected_callback, fd);
   set_dialog_widget(which_dialog, fd->dialog);
   return(fd);
 }
@@ -534,7 +484,10 @@ static void file_open_dialog_ok(GtkWidget *w, gpointer data)
   snd_info *sp;
   char *filename;
   filename = snd_filer_get_filename(open_dialog->dialog);
-  gtk_widget_hide(open_dialog->dialog);
+  int hide_me = 0;
+  hide_me = (int)g_object_get_data(G_OBJECT(w), "hide-me");
+  if (hide_me == 0)
+    gtk_widget_hide(open_dialog->dialog);
   file_dialog_stop_playing(open_dialog);
   if (!(directory_p(filename)))
     {
@@ -606,19 +559,23 @@ static void file_mix_help_callback(GtkWidget *w, gpointer context)
 }
 #endif
 
-static void file_mix_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint file_mix_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   file_dialog_stop_playing(mix_dialog);
   gtk_widget_hide(mix_dialog->dialog);
+  return(true);
 }
 
 static void file_mix_ok_callback(GtkWidget *w, gpointer context)
 {
-  gtk_widget_hide(mix_dialog->dialog);
+  int hide_me = 0;
+  hide_me = (int)g_object_get_data(G_OBJECT(w), "hide-me");
+  if (hide_me == 0)
+    gtk_widget_hide(mix_dialog->dialog);
   file_dialog_stop_playing(mix_dialog);
   mix_complete_file_at_cursor(any_selected_sound(),
 			      snd_filer_get_filename(mix_dialog->dialog),
-			      "File: mix", with_mix_tags(ss), 0);
+			      with_mix_tags(ss), 0);
 }
 
 widget_t make_mix_file_dialog(bool managed)
@@ -703,7 +660,7 @@ static void save_as_data_format_callback(GtkTreeSelection *selection, gpointer *
 file_data *make_file_data_panel(GtkWidget *parent, char *name, 
 				bool with_chan, int header_type, int data_format, bool with_loc, bool comment_as_entry, bool with_samples)
 {
-  GtkWidget *form, *slab, *clab, *comment_label, *loclab, *scbox, *combox, *samplab;
+  GtkWidget *form, *slab, *comment_label, *scbox, *combox;
   file_data *fdat;
   int dformats = 0;
   char **formats = NULL;
@@ -739,6 +696,7 @@ file_data *make_file_data_panel(GtkWidget *parent, char *name,
 
   if (with_chan)
     {
+      GtkWidget *clab, *loclab;
       clab = snd_gtk_label_new(_("chans:"), ss->sgx->highlight_color);
       gtk_box_pack_start(GTK_BOX(scbox), clab, false, false, 0);
       gtk_widget_show(clab);
@@ -757,6 +715,7 @@ file_data *make_file_data_panel(GtkWidget *parent, char *name,
 
   if (with_samples)
     {
+      GtkWidget *samplab;
       samplab = snd_gtk_label_new(_("samples:"), ss->sgx->highlight_color);
       gtk_box_pack_start(GTK_BOX(scbox), samplab, false, false, 0);
       gtk_widget_show(samplab);
@@ -809,10 +768,11 @@ static char *last_save_as_filename = NULL;
 
 static void save_as_ok_callback(GtkWidget *w, gpointer data)
 {
-  char *comment = NULL, *fullname = NULL;
-  int i, type, format, srate, chans;
+  char *comment = NULL;
+  int type, format, srate, chans;
   bool need_update = false;
   off_t location, samples;
+  int hide_me = 0;
   snd_info *sp;
   comment = read_file_data_choices(save_as_file_data, &srate, &chans, &type, &format, &location, &samples);
   last_save_as_filename = snd_filer_get_filename(save_as_dialog);
@@ -823,18 +783,13 @@ static void save_as_ok_callback(GtkWidget *w, gpointer data)
     if (sp) 
       report_in_minibuffer(sp, _("not saved (no name given)"));
   if (comment) FREE(comment);
-  gtk_widget_hide(save_as_dialog);
-  if ((sp) && (last_save_as_filename) && (emacs_style_save_as(ss)) &&
+  hide_me = (int)g_object_get_data(G_OBJECT(w), "hide-me");
+  if (hide_me == 0)
+    gtk_widget_hide(save_as_dialog);
+  if ((sp) && (last_save_as_filename) &&
       (save_as_dialog_type == FILE_SAVE_AS) && 
       (need_update))
-    {
-      for (i = 0; i < sp->nchans; i++) 
-	sp->chans[i]->edit_ctr = 0; /* don't trigger close-hook unsaved-edit checks */
-      snd_close_file(sp);
-      fullname = mus_expand_filename(last_save_as_filename);
-      snd_open_file(fullname, false); /* false = not read_only */
-      FREE(fullname);
-    }
+    run_after_save_as_hook(sp, last_save_as_filename, true); /* true => from dialog */
 } 
 
 static void save_as_cancel_callback(GtkWidget *w, gpointer data)
@@ -849,17 +804,18 @@ static void save_as_help_callback(GtkWidget *w, gpointer data)
 }
 #endif
 
-static void save_as_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint save_as_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(save_as_dialog);
+  return(true);
 }
 
 static void make_save_as_dialog(char *sound_name, int header_type, int format_type)
 {
   /* save old as new, close old, open new */
-  GtkWidget *fbox;
   if (!save_as_dialog)
     {
+      GtkWidget *fbox;
 #if HAVE_GFCDN
       save_as_dialog = snd_filer_new(_("save as:"), true,
 				     (GtkSignalFunc)save_as_delete_callback,
@@ -936,13 +892,13 @@ void set_file_sort_sensitive(bool sensitive)
 
 ww_info *make_title_row(GtkWidget *formw, char *top_str, char *main_str, dialog_pad_t pad, dialog_sort_t with_sort, dialog_paned_t with_pane)
 {
-  GtkWidget *rlw, *sep1, *cww, *phbox;
-  GtkWidget *smenu, *sbar, *sitem;
+  GtkWidget *sep1, *cww;
   ww_info *wwi;
   wwi = (ww_info *)CALLOC(1, sizeof(ww_info));
   /* assuming "formw" is a vbox */
   if (main_str)
     {
+      GtkWidget *rlw;
       rlw = snd_gtk_label_new(main_str, ss->sgx->highlight_color);
       gtk_box_pack_start(GTK_BOX(formw), rlw, false, false, 0);
       gtk_widget_show(rlw);
@@ -955,6 +911,7 @@ ww_info *make_title_row(GtkWidget *formw, char *top_str, char *main_str, dialog_
   
   if (with_pane == WITH_PANED_WINDOW)
     {
+      GtkWidget *phbox;
       wwi->panes = gtk_vpaned_new();
       gtk_box_pack_start(GTK_BOX(formw), wwi->panes, true, true, 0);
       gtk_widget_show(wwi->panes);
@@ -992,6 +949,7 @@ ww_info *make_title_row(GtkWidget *formw, char *top_str, char *main_str, dialog_
 
   if (with_sort == WITH_SORT_BUTTON)
     {
+      GtkWidget *sbar, *sitem, *smenu;
       sbar = gtk_menu_bar_new();
       gtk_box_pack_end(GTK_BOX(wwi->tophbox), sbar, false, false, 0);
       gtk_widget_show(sbar);
@@ -1044,15 +1002,15 @@ ww_info *make_title_row(GtkWidget *formw, char *top_str, char *main_str, dialog_
 static XEN mouse_enter_label_hook;
 static XEN mouse_leave_label_hook;
 
-static gint mouse_name(XEN hook, GtkWidget *w, const char *caller)
+static gboolean mouse_name(XEN hook, GtkWidget *w, const char *caller)
 {
-  char *label = NULL;
-  regrow *r;
   if (XEN_HOOKED(hook))
     {
+      regrow *r;
       r = (regrow *)get_user_data(G_OBJECT(w));
       if (r)
 	{
+	  char *label = NULL;
 	  if (r->parent == CURRENT_FILE_VIEWER)
 	    label = get_curfullname(r->pos);
 	  else
@@ -1064,21 +1022,21 @@ static gint mouse_name(XEN hook, GtkWidget *w, const char *caller)
 	    }
 	  if (label)
 	    run_hook(hook,
-		     XEN_LIST_3(C_TO_SMALL_XEN_INT(r->parent),
-				C_TO_SMALL_XEN_INT(r->pos),
+		     XEN_LIST_3(C_TO_XEN_INT(r->parent),
+				C_TO_XEN_INT(r->pos),
 				C_TO_XEN_STRING(label)),
 		     caller);
 	}
     }
-  return(0);
+  return(false);
 }
 
-static gint label_enter_callback(GtkWidget *w, GdkEventCrossing *ev)
+static gboolean label_enter_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer *gp)
 {
   return(mouse_name(mouse_enter_label_hook, w, S_mouse_enter_label_hook));
 }
 
-static gint label_leave_callback(GtkWidget *w, GdkEventCrossing *ev)
+static gboolean label_leave_callback(GtkWidget *w, GdkEventCrossing *ev, gpointer *gp)
 {
   return(mouse_name(mouse_leave_label_hook, w, S_mouse_leave_label_hook));
 }
@@ -1096,31 +1054,15 @@ regrow *make_regrow(GtkWidget *ww, GtkSignalFunc play_callback, GtkSignalFunc na
 
   r->pl = gtk_check_button_new();
   gtk_box_pack_start(GTK_BOX(r->rw), r->pl, false, false, 2);
-  g_signal_connect_closure_by_id(GTK_OBJECT(r->pl),
-				 g_signal_lookup("toggled", G_OBJECT_TYPE(GTK_OBJECT(r->pl))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(play_callback), (gpointer)r, 0),
-				 0);
+  SG_SIGNAL_CONNECT(r->pl, "toggled", play_callback, r);
   gtk_widget_show(r->pl);
 
   r->nm = gtk_button_new_with_label("");
   sg_left_justify_button(r->nm);
   gtk_box_pack_start(GTK_BOX(r->rw), r->nm, true, true, 2);
-  g_signal_connect_closure_by_id(GTK_OBJECT(r->nm),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(r->nm))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(name_callback), (gpointer)r, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(r->nm),
-				 g_signal_lookup("enter_notify_event", G_OBJECT_TYPE(GTK_OBJECT(r->nm))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(label_enter_callback), (gpointer)r, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(r->nm),
-				 g_signal_lookup("leave_notify_event", G_OBJECT_TYPE(GTK_OBJECT(r->nm))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(label_leave_callback), (gpointer)r, 0),
-				 0);
+  SG_SIGNAL_CONNECT(r->nm, "clicked", name_callback, r);
+  SG_SIGNAL_CONNECT(r->nm, "enter_notify_event", label_enter_callback, r);
+  SG_SIGNAL_CONNECT(r->nm, "leave_notify_event", label_leave_callback, r);
   set_user_data(G_OBJECT(r->nm), (gpointer)r);
   gtk_widget_show(r->nm);
 
@@ -1138,11 +1080,11 @@ static regrow **prev_name_row = NULL;
 
 void make_cur_name_row(int old_size, int new_size)
 {
-  int i;
   if (cur_name_row == NULL)
     cur_name_row = (regrow **)CALLOC(new_size, sizeof(regrow *));
   else 
     {
+      int i;
       cur_name_row = (regrow **)REALLOC(cur_name_row, new_size * sizeof(regrow *));
       for (i = old_size; i < new_size; i++) cur_name_row[i] = NULL;
     }
@@ -1150,11 +1092,11 @@ void make_cur_name_row(int old_size, int new_size)
 
 void make_prev_name_row(int old_size, int new_size)
 {
-  int i;
   if (prev_name_row == NULL)
     prev_name_row = (regrow **)CALLOC(new_size, sizeof(regrow *));
   else 
     {
+      int i;
       prev_name_row = (regrow **)REALLOC(prev_name_row, new_size * sizeof(regrow *));
       for (i = old_size; i < new_size; i++) prev_name_row[i] = NULL;
     }
@@ -1177,9 +1119,10 @@ static void view_files_dismiss_callback(GtkWidget *w, gpointer context)
   gtk_widget_hide(view_files_dialog);
 }
 
-static void view_files_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint view_files_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(view_files_dialog);
+  return(true);
 }
 
 static void view_files_clear_callback(GtkWidget *w, gpointer context) 
@@ -1197,15 +1140,14 @@ static void view_files_update_callback(GtkWidget *w, gpointer context)
 
 void set_file_browser_play_button(char *name, int state)
 {
-  int i, list;
-  regrow *r;
-  list = 0;
   if (view_files_dialog_is_active())
     {
+      int i, list = 0;
       i = find_curfile_regrow(name); 
       if (i != -1) list = 1; else i = find_prevfile_regrow(name);
       if (i != -1)
 	{
+	  regrow *r;
 	  if (list) r = cur_name_row[i]; else r = prev_name_row[i];
 	  set_toggle_button(r->pl, state, false, (void *)r);
 	}
@@ -1220,11 +1162,11 @@ static void view_curfiles_play_callback(GtkWidget *w, gpointer context)
 
 static void curfile_unhighlight(void)
 {
-  regrow *r;
   if (view_files_dialog_is_active())
     {
       if (vf_selected_file != -1)
 	{
+	  regrow *r;
 	  r = cur_name_row[vf_selected_file];
 	  gtk_widget_modify_bg(r->nm, GTK_STATE_NORMAL, ss->sgx->basic_color);
 	  gtk_widget_modify_base(r->nm, GTK_STATE_NORMAL, ss->sgx->basic_color);
@@ -1237,9 +1179,9 @@ static void curfile_unhighlight(void)
 
 void curfile_highlight(int i)
 {
-  regrow *r;
   if (view_files_dialog_is_active())
     {
+      regrow *r;
       if (vf_selected_file != -1) curfile_unhighlight();
       r = cur_name_row[i];
       gtk_widget_modify_bg(r->nm, GTK_STATE_NORMAL, ss->sgx->zoom_color);
@@ -1274,10 +1216,10 @@ static void view_prevfiles_select_callback(GtkWidget *w, gpointer context)
 void highlight_selected_sound(void)
 {
   snd_info *sp;
-  int i;
   sp = selected_sound();
   if (sp)
     {
+      int i;
       i = find_curfile_regrow(sp->short_filename);
       if (i != -1) 
 	curfile_highlight(i); 
@@ -1390,17 +1332,13 @@ static void start_view_files_dialog(bool managed)
    * The previous files list requires that we keep such a list as we go along, on the
    * off-chance this browser will be fired up.  (Such files may be subsequently moved or deleted).
    */
-  ww_info *wwl;
-  GtkWidget *mainform, *curform, *prevform, *updateB, *helpB, *dismissB, *clearB, *sep;
   if (!view_files_dialog)
     {
+      ww_info *wwl;
+      GtkWidget *mainform, *curform, *prevform, *updateB, *helpB, *dismissB, *clearB, *sep;
       vf_selected_file = -1;
       view_files_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(view_files_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(view_files_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(view_files_delete_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(view_files_dialog, "delete_event", view_files_delete_callback, NULL);
       gtk_window_set_title(GTK_WINDOW(view_files_dialog), _("Files"));
       sg_make_resizable(view_files_dialog);
       gtk_container_set_border_width (GTK_CONTAINER(view_files_dialog), 10);
@@ -1419,26 +1357,10 @@ static void start_view_files_dialog(bool managed)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(view_files_dialog)->action_area), updateB, true, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(view_files_dialog)->action_area), clearB, true, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(view_files_dialog)->action_area), helpB, true, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(dismissB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(dismissB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(view_files_dismiss_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(helpB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(helpB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(view_files_help_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(updateB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(updateB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(view_files_update_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(clearB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(clearB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(view_files_clear_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(dismissB, "clicked", view_files_dismiss_callback, NULL);
+      SG_SIGNAL_CONNECT(helpB, "clicked", view_files_help_callback, NULL);
+      SG_SIGNAL_CONNECT(updateB, "clicked", view_files_update_callback, NULL);
+      SG_SIGNAL_CONNECT(clearB, "clicked", view_files_clear_callback, NULL);
       gtk_widget_show(dismissB);
       gtk_widget_show(updateB);
       gtk_widget_show(helpB);
@@ -1473,31 +1395,11 @@ static void start_view_files_dialog(bool managed)
       wwl = make_title_row(prevform, _("play"), _("previous files"), PAD_TITLE_ON_LEFT, WITH_SORT_BUTTON, WITHOUT_PANED_WINDOW);
       fs3 = wwl->tophbox;
 
-      g_signal_connect_closure_by_id(GTK_OBJECT(wwl->byname), 
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(wwl->byname))), 
-				     0,
-				     g_cclosure_new_swap(GTK_SIGNAL_FUNC(sort_prevfiles_by_name), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(wwl->bydate), 
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(wwl->bydate))), 
-				     0,
-				     g_cclosure_new_swap(GTK_SIGNAL_FUNC(sort_prevfiles_by_date), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(wwl->bysize), 
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(wwl->bysize))),
-				     0,
-				     g_cclosure_new_swap(GTK_SIGNAL_FUNC(sort_prevfiles_by_size), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(wwl->byentry), 
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(wwl->byentry))), 
-				     0,
-				     g_cclosure_new_swap(GTK_SIGNAL_FUNC(sort_prevfiles_by_entry), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(wwl->byproc), 
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(wwl->byproc))), 
-				     0,
-				     g_cclosure_new_swap(GTK_SIGNAL_FUNC(sort_prevfiles_by_user_procedure), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(wwl->byname,  "activate", sort_prevfiles_by_name, NULL);
+      SG_SIGNAL_CONNECT(wwl->bydate,  "activate", sort_prevfiles_by_date, NULL);
+      SG_SIGNAL_CONNECT(wwl->bysize,  "activate", sort_prevfiles_by_size, NULL);
+      SG_SIGNAL_CONNECT(wwl->byentry,  "activate", sort_prevfiles_by_entry, NULL);
+      SG_SIGNAL_CONNECT(wwl->byproc,  "activate", sort_prevfiles_by_user_procedure, NULL);
       vf_prevww = wwl->list;
       vf_prevlst = wwl->list;
       FREE(wwl); 
@@ -1536,7 +1438,13 @@ static bool raw_cancelled = false, raw_done = false;
 
 static void raw_data_ok_callback(GtkWidget *w, gpointer context) {raw_cancelled = false; raw_done = true;}
 static void raw_data_cancel_callback(GtkWidget *w, gpointer context) {raw_cancelled = true; raw_done = true;}
-static void raw_data_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context) {raw_cancelled = true; raw_done = true;}
+
+static gint raw_data_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context) 
+{
+  raw_cancelled = true; 
+  raw_done = true;
+  return(true);
+}
 
 static void raw_data_default_callback(GtkWidget *w, gpointer context) 
 {
@@ -1547,9 +1455,9 @@ static void raw_data_default_callback(GtkWidget *w, gpointer context)
 static char **data_formats = NULL;
 static char **data_format_names(void)
 {
-  int i;
   if (data_formats == NULL)
     {
+      int i;
       data_formats = (char **)CALLOC(MUS_LAST_DATA_FORMAT, sizeof(char *));
       for (i = 0; i < MUS_LAST_DATA_FORMAT; i++)
 	data_formats[i] = (char *)mus_data_format_name(i + 1);
@@ -1587,17 +1495,12 @@ static void raw_data_help_callback(GtkWidget *w, gpointer context)
 
 static void make_raw_data_dialog(void)
 {
-  char dfs_str[LABEL_BUFFER_SIZE];
   int sr, oc, fr;
   GtkWidget *lst, *dls, *dloclab, *chnlab;
   GtkWidget *defaultB, *helpB, *cancelB, *okB, *sratehbox, *lochbox;
   mus_header_raw_defaults(&sr, &oc, &fr);
   raw_data_dialog = snd_gtk_dialog_new();
-  g_signal_connect_closure_by_id(GTK_OBJECT(raw_data_dialog),
-				 g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(raw_data_dialog))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(raw_data_delete_callback), NULL, 0),
-				 0);
+  SG_SIGNAL_CONNECT(raw_data_dialog, "delete_event", raw_data_delete_callback, NULL);
   gtk_window_set_title(GTK_WINDOW(raw_data_dialog), _("No Header on File"));
   sg_make_resizable(raw_data_dialog);
   gtk_container_set_border_width(GTK_CONTAINER(raw_data_dialog), 10);
@@ -1616,26 +1519,10 @@ static void make_raw_data_dialog(void)
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(raw_data_dialog)->action_area), defaultB, true, true, 10);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(raw_data_dialog)->action_area), cancelB, true, true, 10);
   gtk_box_pack_end(GTK_BOX(GTK_DIALOG(raw_data_dialog)->action_area), helpB, true, true, 10);
-  g_signal_connect_closure_by_id(GTK_OBJECT(okB),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(okB))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(raw_data_ok_callback), NULL, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(helpB),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(helpB))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(raw_data_help_callback), NULL, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(defaultB),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(defaultB))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(raw_data_default_callback), NULL, 0),
-				 0);
-  g_signal_connect_closure_by_id(GTK_OBJECT(cancelB),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(cancelB))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(raw_data_cancel_callback), NULL, 0),
-				 0);
+  SG_SIGNAL_CONNECT(okB, "clicked", raw_data_ok_callback, NULL);
+  SG_SIGNAL_CONNECT(helpB, "clicked", raw_data_help_callback, NULL);
+  SG_SIGNAL_CONNECT(defaultB, "clicked", raw_data_default_callback, NULL);
+  SG_SIGNAL_CONNECT(cancelB, "clicked", raw_data_cancel_callback, NULL);
   gtk_widget_show(okB);
   gtk_widget_show(cancelB);
   gtk_widget_show(helpB);
@@ -1654,16 +1541,14 @@ static void make_raw_data_dialog(void)
   gtk_widget_show(dls);
 
   raw_srate_text = snd_entry_new(sratehbox, true);
-  mus_snprintf(dfs_str, LABEL_BUFFER_SIZE, "%d", sr);
-  gtk_entry_set_text(GTK_ENTRY(raw_srate_text), dfs_str);
+  widget_int_to_text(raw_srate_text, sr);
 
   chnlab = gtk_label_new(_("chans:"));
   gtk_box_pack_start(GTK_BOX(sratehbox), chnlab, false, false, 4);
   gtk_widget_show(chnlab);
 
   raw_chans_text = snd_entry_new(sratehbox, true);
-  mus_snprintf(dfs_str, LABEL_BUFFER_SIZE, "%d", oc);
-  gtk_entry_set_text(GTK_ENTRY(raw_chans_text), dfs_str);
+  widget_int_to_text(raw_chans_text, oc);
 
   lochbox = gtk_hbox_new(false, 2);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(raw_data_dialog)->vbox), lochbox, false, false, 6);
@@ -1730,7 +1615,6 @@ file_info *raw_data_dialog_to_file_info(const char *filename, const char *title)
 }
 
 
-
 /* -------------------------------- New File -------------------------------- */
 
 static bool new_file_cancelled = false, new_file_done = false;
@@ -1739,7 +1623,13 @@ static file_data *new_dialog_data = NULL;
 
 static void new_file_ok_callback(GtkWidget *w, gpointer context) {new_file_cancelled = false; new_file_done = true;}
 static void new_file_cancel_callback(GtkWidget *w, gpointer context) {new_file_cancelled = true; new_file_done = true;}
-static void new_file_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context) {new_file_cancelled = true; new_file_done = true;}
+
+static gint new_file_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context) 
+{
+  new_file_cancelled = true; 
+  new_file_done = true;
+  return(true);
+}
 
 static void new_file_help_callback(GtkWidget *w, gpointer context) 
 {
@@ -1750,22 +1640,16 @@ static off_t initial_samples = 1;
 
 snd_info *make_new_file_dialog(char *newname, int header_type, int data_format, int srate, int chans, char *comment)
 {
-  off_t loc;
-  char *tmpstr, *title, *newer_name = NULL;
+  char *title;
   snd_info *sp = NULL;
-  GtkWidget *name_label, *hform;
-  GtkWidget *help_button, *cancel_button, *ok_button;
   new_file_cancelled = false;
   title = (char *)CALLOC(snd_strlen(newname) + 32, sizeof(char));
   sprintf(title, _("create new sound: %s"), newname);
   if (!new_dialog)
     {
+      GtkWidget *name_label, *hform, *help_button, *cancel_button, *ok_button;
       new_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(new_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(new_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(new_file_delete_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(new_dialog, "delete_event", new_file_delete_callback, NULL);
       gtk_window_set_title(GTK_WINDOW(new_dialog), title);
       sg_make_resizable(new_dialog);
       gtk_container_set_border_width (GTK_CONTAINER(new_dialog), 10);
@@ -1781,21 +1665,9 @@ snd_info *make_new_file_dialog(char *newname, int header_type, int data_format,
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_dialog)->action_area), ok_button, true, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_dialog)->action_area), cancel_button, true, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(new_dialog)->action_area), help_button, true, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(cancel_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(cancel_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(new_file_cancel_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(new_file_help_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(ok_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(ok_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(new_file_ok_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(cancel_button, "clicked", new_file_cancel_callback, NULL);
+      SG_SIGNAL_CONNECT(help_button, "clicked", new_file_help_callback, NULL);
+      SG_SIGNAL_CONNECT(ok_button, "clicked", new_file_ok_callback, NULL);
       gtk_widget_show(cancel_button);
       gtk_widget_show(ok_button);
       gtk_widget_show(help_button);
@@ -1829,6 +1701,8 @@ snd_info *make_new_file_dialog(char *newname, int header_type, int data_format,
     return(NULL);
   else
     {
+      off_t loc;
+      char *tmpstr, *newer_name = NULL;
       newer_name = (char *)gtk_entry_get_text(GTK_ENTRY(new_file_name));
       if (newer_name == NULL) return(NULL);
       tmpstr = read_file_data_choices(new_dialog_data, &srate, &chans, &header_type, &data_format, &loc, &initial_samples);
@@ -1856,9 +1730,10 @@ static void edit_header_cancel_callback(GtkWidget *w, gpointer context)
   gtk_widget_hide(edit_header_dialog);
 }
 
-static void edit_header_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint edit_header_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(edit_header_dialog);
+  return(true);
 }
 
 static void edit_header_ok_callback(GtkWidget *w, gpointer context) 
@@ -1876,7 +1751,6 @@ GtkWidget *edit_header(snd_info *sp)
    * if any are changed, need save button, cancel button, dismiss (leave unsaved but pending), reflect (change Snd display, not file)
    * this means the Snd-effective header is separate from the in-file header even across saves??
    */
-  GtkWidget *help_button, *cancel_button, *save_button;
   char *str;
   file_info *hdr;
   if (!sp) return(NULL);
@@ -1884,12 +1758,9 @@ GtkWidget *edit_header(snd_info *sp)
   hdr = sp->hdr;
   if (!edit_header_dialog)
     {
+      GtkWidget *help_button, *cancel_button, *save_button;
       edit_header_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(edit_header_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(edit_header_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_header_delete_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(edit_header_dialog, "delete_event", edit_header_delete_callback, NULL);
       /* gtk_window_set_title(GTK_WINDOW(edit_header_dialog), _("Edit Header")); */
       sg_make_resizable(edit_header_dialog);
       gtk_container_set_border_width (GTK_CONTAINER(edit_header_dialog), 10);
@@ -1905,21 +1776,9 @@ GtkWidget *edit_header(snd_info *sp)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_header_dialog)->action_area), cancel_button, true, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_header_dialog)->action_area), save_button, true, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(edit_header_dialog)->action_area), help_button, true, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(cancel_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(cancel_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_header_cancel_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_header_help_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(save_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(save_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_header_ok_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(cancel_button, "clicked", edit_header_cancel_callback, NULL);
+      SG_SIGNAL_CONNECT(help_button, "clicked", edit_header_help_callback, NULL);
+      SG_SIGNAL_CONNECT(save_button, "clicked", edit_header_ok_callback, NULL);
       gtk_widget_show(cancel_button);
       gtk_widget_show(save_button);
       gtk_widget_show(help_button);
@@ -1948,18 +1807,19 @@ GtkWidget *edit_header(snd_info *sp)
 static GtkWidget *post_it_text = NULL, *post_it_dialog = NULL;
 
 static void dismiss_post_it(GtkWidget *w, gpointer context) {gtk_widget_hide(post_it_dialog);}
-static void delete_post_it(GtkWidget *w, GdkEvent *event, gpointer context) {gtk_widget_hide(post_it_dialog);}
+
+static gint delete_post_it(GtkWidget *w, GdkEvent *event, gpointer context) 
+{
+  gtk_widget_hide(post_it_dialog);
+  return(true);
+}
 
 static void create_post_it_monolog(void)
 {
   /* create scrollable but not editable text window */
   GtkWidget *ok_button;
   post_it_dialog = gtk_dialog_new();
-  g_signal_connect_closure_by_id(GTK_OBJECT(post_it_dialog),
-				 g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(post_it_dialog))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(delete_post_it), NULL, 0),
-				 0);
+  SG_SIGNAL_CONNECT(post_it_dialog, "delete_event", delete_post_it, NULL);
 
   gtk_window_set_title(GTK_WINDOW(post_it_dialog), _("Info"));
   sg_make_resizable(post_it_dialog);
@@ -1970,11 +1830,7 @@ static void create_post_it_monolog(void)
   ok_button = gtk_button_new_with_label(_("Ok"));
   gtk_widget_set_name(ok_button, "quit_button");
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(post_it_dialog)->action_area), ok_button, false, true, 20);
-  g_signal_connect_closure_by_id(GTK_OBJECT(ok_button),
-				 g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(ok_button))),
-				 0,
-				 g_cclosure_new(GTK_SIGNAL_FUNC(dismiss_post_it), NULL, 0),
-				 0);
+  SG_SIGNAL_CONNECT(ok_button, "clicked", dismiss_post_it, NULL);
   gtk_widget_show(ok_button);
 
   post_it_text = make_scrolled_text(GTK_DIALOG(post_it_dialog)->vbox, false, NULL, NULL);
