@@ -11,9 +11,10 @@ static void edit_find_dismiss(GtkWidget *w, gpointer context)
   else gtk_widget_hide(edit_find_dialog);
 } 
 
-static void edit_find_delete(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint edit_find_delete(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(edit_find_dialog);
+  return(true);
 }
 
 static void edit_find_help(GtkWidget *w, gpointer context) 
@@ -24,10 +25,10 @@ static void edit_find_help(GtkWidget *w, gpointer context)
 static void edit_find_find(read_direction_t direction, GtkWidget *w, gpointer context) 
 { /* "Find" is the label here */
   char *str, *buf = NULL;
-  XEN proc;
   str = (char *)gtk_entry_get_text(GTK_ENTRY(edit_find_text));
   if ((str) && (*str))
     {
+      XEN proc;
       ss->search_expr = copy_string(str);
       if (XEN_PROCEDURE_P(ss->search_proc)) snd_unprotect(ss->search_proc);
       ss->search_proc = XEN_UNDEFINED;
@@ -74,16 +75,12 @@ static void edit_find_previous(GtkWidget *w, gpointer context) {edit_find_find(R
 
 static void make_edit_find_dialog(bool managed)
 {
-  GtkWidget *dl, *rc;
-  GtkWidget *help_button;
   if (!edit_find_dialog)
     {
+      GtkWidget *dl, *rc;
+      GtkWidget *help_button;
       edit_find_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(edit_find_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(edit_find_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_delete), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(edit_find_dialog, "delete_event", edit_find_delete, NULL);
       gtk_window_set_title(GTK_WINDOW(edit_find_dialog), _("Find"));
       sg_make_resizable(edit_find_dialog);
       gtk_container_set_border_width (GTK_CONTAINER(edit_find_dialog), 10);
@@ -102,26 +99,10 @@ static void make_edit_find_dialog(bool managed)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_find_dialog)->action_area), next_button, true, true, 10);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_find_dialog)->action_area), previous_button, true, true, 10);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(edit_find_dialog)->action_area), help_button, true, true, 10);
-      g_signal_connect_closure_by_id(GTK_OBJECT(cancelB),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(cancelB))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_dismiss), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_help), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(next_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(next_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_next), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(previous_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(previous_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_previous), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(cancelB, "clicked", edit_find_dismiss, NULL);
+      SG_SIGNAL_CONNECT(help_button, "clicked", edit_find_help, NULL);
+      SG_SIGNAL_CONNECT(next_button, "clicked", edit_find_next, NULL);
+      SG_SIGNAL_CONNECT(previous_button, "clicked", edit_find_previous, NULL);
       gtk_widget_show(cancelB);
       gtk_widget_show(next_button);
       gtk_widget_show(previous_button);
@@ -136,11 +117,7 @@ static void make_edit_find_dialog(bool managed)
       gtk_widget_show(dl);
 
       edit_find_text = snd_entry_new(rc, true);
-      g_signal_connect_closure_by_id(GTK_OBJECT(edit_find_text),
-				     g_signal_lookup("activate", G_OBJECT_TYPE(GTK_OBJECT(edit_find_text))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(edit_find_next), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(edit_find_text, "activate", edit_find_next, NULL);
       
       edit_find_label = gtk_label_new(_("global search"));
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(edit_find_dialog)->vbox), edit_find_label, false, false, 4);
@@ -163,11 +140,10 @@ static XEN g_find_dialog(XEN managed)
 {
   #define H_find_dialog "(" S_find_dialog "): create and activate the Edit:Find dialog, return the dialog widget"
   XEN_ASSERT_TYPE(XEN_BOOLEAN_IF_BOUND_P(managed), managed, XEN_ONLY_ARG, S_find_dialog, "a boolean");
-  make_edit_find_dialog(XEN_TO_C_BOOLEAN_OR_TRUE(managed));
+  make_edit_find_dialog(XEN_TO_C_BOOLEAN(managed));
   return(XEN_WRAP_WIDGET(edit_find_dialog));
 }
 
-#if DEBUGGING
 static XEN g_find_dialog_widgets(void)
 {
   if (edit_find_dialog)
@@ -179,25 +155,18 @@ static XEN g_find_dialog_widgets(void)
 		     XEN_EMPTY_LIST))))));
   return(XEN_EMPTY_LIST);
 }
-#endif
 
 #ifdef XEN_ARGIFY_1
 XEN_ARGIFY_1(g_find_dialog_w, g_find_dialog)
-#if DEBUGGING
-  XEN_NARGIFY_0(g_find_dialog_widgets_w, g_find_dialog_widgets)
-#endif
+XEN_NARGIFY_0(g_find_dialog_widgets_w, g_find_dialog_widgets)
 #else
 #define g_find_dialog_w g_find_dialog
-#if DEBUGGING
-  #define g_find_dialog_widgets_w g_find_dialog_widgets
-#endif
+#define g_find_dialog_widgets_w g_find_dialog_widgets
 #endif
 
 void g_init_gxfind(void)
 {
-   XEN_DEFINE_PROCEDURE(S_find_dialog, g_find_dialog_w, 0, 1, 0, H_find_dialog);
-#if DEBUGGING
-  XEN_DEFINE_PROCEDURE("find-dialog-widgets", g_find_dialog_widgets_w, 0, 0, 0, "");
-#endif
+  XEN_DEFINE_PROCEDURE(S_find_dialog, g_find_dialog_w, 0, 1, 0, H_find_dialog);
+  XEN_DEFINE_PROCEDURE("find-dialog-widgets", g_find_dialog_widgets_w, 0, 0, 0, "internal auto-test function");
 }
 
