@@ -60,6 +60,8 @@ static gint window_close(GtkWidget *w, GdkEvent *event, gpointer context)
 }
 #endif
 
+static guint auto_update_proc = 0;
+
 static gint auto_update_check(gpointer context)
 {
   if (auto_update_interval(ss) > 0.0)
@@ -67,25 +69,32 @@ static gint auto_update_check(gpointer context)
       if ((!(play_in_progress())) && 
 	  (!(record_in_progress())))
 	for_each_sound(sound_not_current, NULL);
-      g_timeout_add_full(0, (guint32)(auto_update_interval(ss) * 1000), auto_update_check, context, NULL);
+      auto_update_proc = g_timeout_add_full(0, (guint32)(auto_update_interval(ss) * 1000), auto_update_check, context, NULL);
     }
+  else auto_update_proc = 0;
   return(0);
 }
 
+void auto_update_restart(void)
+{
+  if (auto_update_proc == 0)
+    g_timeout_add_full(0, (guint32)(auto_update_interval(ss) * 1000), auto_update_check, NULL, NULL);
+}
+
 static GdkAtom snd_v, snd_c;
 
 #if HAVE_EXTENSION_LANGUAGE
 static XEN window_property_changed_hook;
 
-static void who_called(GtkWidget *w, GdkEvent *event, gpointer context) 
+static gboolean who_called(GtkWidget *w, GdkEvent *event, gpointer context) 
 {
   /* watch for communication from some other program via the SND_COMMAND property */
   GdkEventProperty *ev = (GdkEventProperty *)event;
-  GdkAtom type;
-  gint format, nitems;
-  guchar *version[1];
   if (ev->atom == snd_c)
     {
+      GdkAtom type;
+      gint format, nitems;
+      guchar *version[1];
       if (gdk_property_get(MAIN_WINDOW(ss), snd_c, 
 			   GDK_TARGET_STRING, 0L, (long)BUFSIZ, false,
 			   &type, &format, &nitems, (guchar **)version))
@@ -101,6 +110,7 @@ static void who_called(GtkWidget *w, GdkEvent *event, gpointer context)
 	    }
 	}
     }
+  return(false);
 }
 #endif
 
@@ -249,7 +259,6 @@ static int tm_slice = 0;
 
 static Cessate startup_funcs(gpointer context)
 {
-  snd_info *sp;
   static int auto_open_ctr = 0;
   switch (tm_slice)
     {
@@ -267,18 +276,10 @@ static Cessate startup_funcs(gpointer context)
 			  strlen(SND_DATE) + 1);
 #if HAVE_EXTENSION_LANGUAGE
       gtk_widget_add_events(MAIN_SHELL(ss), GDK_PROPERTY_CHANGE_MASK);
-      g_signal_connect_closure_by_id(GTK_OBJECT(MAIN_SHELL(ss)),
-				     g_signal_lookup("property_notify_event", G_OBJECT_TYPE(GTK_OBJECT(MAIN_SHELL(ss)))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(who_called), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(MAIN_SHELL(ss), "property_notify_event", who_called, NULL);
 #endif
       /* trap outer-level Close for cleanup check */
-      g_signal_connect_closure_by_id(GTK_OBJECT(MAIN_SHELL(ss)),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(MAIN_SHELL(ss)))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(window_close), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(MAIN_SHELL(ss), "delete_event", window_close, NULL);
 #endif
       (ss->sgx)->graph_cursor = gdk_cursor_new((GdkCursorType)in_graph_cursor(ss));
       (ss->sgx)->wait_cursor = gdk_cursor_new(GDK_WATCH);
@@ -321,7 +322,8 @@ static Cessate startup_funcs(gpointer context)
       if ((ss->init_window_x != DEFAULT_INIT_WINDOW_X) && (ss->init_window_y != DEFAULT_INIT_WINDOW_Y))
 	set_widget_position(GTK_WIDGET(MAIN_SHELL(ss)), ss->init_window_x, ss->init_window_y);
 #endif
-      g_timeout_add_full(0, (guint32)(auto_update_interval(ss) * 1000), auto_update_check, NULL, NULL);
+      if (auto_update_interval(ss) > 0.0)
+	g_timeout_add_full(0, (guint32)(auto_update_interval(ss) * 1000), auto_update_check, NULL, NULL);
       break;
     case 4: 
 #if TRAP_SEGFAULT
@@ -330,6 +332,7 @@ static Cessate startup_funcs(gpointer context)
       if ((ss->sounds) &&
 	  (ss->selected_sound == NO_SELECTION))
 	{
+	  snd_info *sp;
 	  sp = ss->sounds[0];
 	  if ((sp) && 
 	      (sp->inuse == SOUND_NORMAL) &&
@@ -395,16 +398,16 @@ static void notebook_switch_page(GtkNotebook *w, GtkNotebookPage *page_widget, g
   /* as far as I can tell there's nothing that a user can do with the idiotic page_widget argument --
    *   the GtkNotebookPage structure is hidden, and not one public function gives any access to it.
    */
-  int index = 0;
-  snd_info *sp;
   GtkWidget *pw;
   pw = gtk_notebook_get_nth_page(w, page_num);
   if (pw)
     {
+      int index = 0;
       index = get_user_int_data(G_OBJECT(pw));
       if ((index < ss->max_sounds) && 
 	  (snd_ok(ss->sounds[index])))
 	{
+	  snd_info *sp;
 	  sp = ss->sounds[index];
 	  if (sp->selected_channel == NO_SELECTION)
 	    select_channel(ss->sounds[index], 0);
@@ -413,10 +416,6 @@ static void notebook_switch_page(GtkNotebook *w, GtkNotebookPage *page_widget, g
     }
 }
 
-#if HAVE_PWD_H
-  #include <pwd.h>
-#endif
-
 #ifdef SND_AS_WIDGET
 GtkWidget *snd_as_widget(int argc, char **argv, GtkWidget *parent, void (*error_func)(const char *))
 {
@@ -566,6 +565,7 @@ void snd_doit(int argc, char **argv)
 	  str = mus_expand_filename("~/Snd.gtkrc");
 	  if (mus_file_probe(str))
 	    gtk_rc_parse(str);
+#if WITH_BUILTIN_GTKRC
 	  else 
 	    {
 	      gtk_rc_parse_string("\n\
@@ -710,7 +710,10 @@ style \"reset\" = \"default_button\"\n\
 \n\
 widget \"*.reset_button\" style \"reset\"\n\
 ");
-      }}}
+      }
+#endif
+    }
+  }
   if (str) FREE(str);
 
   MAIN_PANE(ss) = gtk_vbox_new(false, 0); /* not homogenous, spacing 0 */
@@ -732,11 +735,7 @@ widget \"*.reset_button\" style \"reset\"\n\
 	{
 	  SOUND_PANE_BOX(ss) = gtk_notebook_new();
 	  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(SOUND_PANE_BOX(ss)), GTK_POS_TOP);
-          g_signal_connect_closure_by_id(GTK_OBJECT(SOUND_PANE_BOX(ss)),
-				         g_signal_lookup("switch_page", G_OBJECT_TYPE(GTK_OBJECT(SOUND_PANE_BOX(ss)))),
-				         0,
-				         g_cclosure_new(GTK_SIGNAL_FUNC(notebook_switch_page), NULL, 0),
-				         0);
+          SG_SIGNAL_CONNECT(SOUND_PANE_BOX(ss), "switch_page", notebook_switch_page, NULL);
 	}
       else 
 	{
