@@ -19,9 +19,10 @@ static void file_print_cancel_callback(GtkWidget *w, gpointer context)
   gtk_widget_hide(file_print_dialog);
 }
 
-static void file_print_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
+static gint file_print_delete_callback(GtkWidget *w, GdkEvent *event, gpointer context)
 {
   gtk_widget_hide(file_print_dialog);
+  return(true);
 }
 
 static int lpr (char *name)
@@ -31,18 +32,17 @@ static int lpr (char *name)
   return(system(print_string));
 }
 
-static bool printing = false;
+static printing_t printing = NOT_PRINTING;
 
 static void file_print_ok_callback(GtkWidget *w, gpointer context)
 {
-  bool print_it, quit = false;
-  int err = 0;
-  char *name;
+  bool quit = false;
   snd_info *nsp = NULL;
   if (printing) 
     ss->stopped_explicitly = true;
   else
     {
+      bool print_it;
       if (ss->print_choice == PRINT_SND)
 	{
 	  set_button_label(file_print_ok_button, _("Stop"));
@@ -50,11 +50,13 @@ static void file_print_ok_callback(GtkWidget *w, gpointer context)
 	  mus_snprintf(print_string, PRINT_BUFFER_SIZE, _("printing %s"), nsp->short_filename);
 	  set_label(file_print_message, print_string);
 	}
-      printing = true;
+      printing = PRINTING;
       print_it = (bool)GTK_TOGGLE_BUTTON(file_print_eps_or_lpr)->active;
       quit = (ss->print_choice == PRINT_ENV);
       if (print_it)
 	{
+	  char *name;
+	  int err = 0;
 	  name = snd_tempnam();
 	  switch (ss->print_choice)
 	    {
@@ -84,7 +86,7 @@ static void file_print_ok_callback(GtkWidget *w, gpointer context)
 	    }
 	}
     }
-  printing = false;
+  printing = NOT_PRINTING;
   if (ss->print_choice == PRINT_SND)
     {
       set_button_label(file_print_ok_button, _("Print"));
@@ -97,15 +99,12 @@ static void file_print_ok_callback(GtkWidget *w, gpointer context)
 
 static void start_file_print_dialog(void)
 {
-  GtkWidget *print_button, *help_button, *dismiss_button, *epsbox, *epslabel;
   if (!file_print_dialog)
     {
+      GtkWidget *print_button, *help_button, *dismiss_button, *epsbox, *epslabel;
+
       file_print_dialog = snd_gtk_dialog_new();
-      g_signal_connect_closure_by_id(GTK_OBJECT(file_print_dialog),
-				     g_signal_lookup("delete_event", G_OBJECT_TYPE(GTK_OBJECT(file_print_dialog))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(file_print_delete_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(file_print_dialog, "delete_event", file_print_delete_callback, NULL);
       gtk_window_set_title(GTK_WINDOW(file_print_dialog), _("Print"));
       sg_make_resizable(file_print_dialog);
       gtk_container_set_border_width (GTK_CONTAINER(file_print_dialog), 10);
@@ -121,21 +120,9 @@ static void start_file_print_dialog(void)
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(file_print_dialog)->action_area), print_button, true, true, 4);
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(file_print_dialog)->action_area), dismiss_button, true, true, 4);
       gtk_box_pack_end(GTK_BOX(GTK_DIALOG(file_print_dialog)->action_area), help_button, true, true, 4);
-      g_signal_connect_closure_by_id(GTK_OBJECT(dismiss_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(dismiss_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(file_print_cancel_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(help_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(help_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(file_print_help_callback), NULL, 0),
-				     0);
-      g_signal_connect_closure_by_id(GTK_OBJECT(print_button),
-				     g_signal_lookup("clicked", G_OBJECT_TYPE(GTK_OBJECT(print_button))),
-				     0,
-				     g_cclosure_new(GTK_SIGNAL_FUNC(file_print_ok_callback), NULL, 0),
-				     0);
+      SG_SIGNAL_CONNECT(dismiss_button, "clicked", file_print_cancel_callback, NULL);
+      SG_SIGNAL_CONNECT(help_button, "clicked", file_print_help_callback, NULL);
+      SG_SIGNAL_CONNECT(print_button, "clicked", file_print_ok_callback, NULL);
       gtk_widget_show(dismiss_button);
       gtk_widget_show(print_button);
       gtk_widget_show(help_button);
@@ -165,10 +152,10 @@ static void start_file_print_dialog(void)
 
 void file_print_callback(GtkWidget *w, gpointer context)
 {
-  snd_info *nsp;
   start_file_print_dialog();
   if (ss->print_choice == PRINT_SND)
     {
+      snd_info *nsp;
       nsp = any_selected_sound();
       mus_snprintf(print_string, PRINT_BUFFER_SIZE, _("print %s"), nsp->short_filename);
     }
