@@ -32,6 +32,13 @@
 (if (not (provided? 'snd-extensions.scm)) (load-from-path "extensions.scm"))
 (if (not (provided? 'snd-play.scm)) (load-from-path "play.scm"))
 
+(if (not (defined? 'g_signal_connect))
+    (define* (g_signal_connect obj name func #:optional data)
+      (g_signal_connect_closure_by_id (GPOINTER obj)
+				      (g_signal_lookup name (G_OBJECT_TYPE (GTK_OBJECT obj)))
+				      0
+				      (g_cclosure_new func data #f)
+				      #f)))
 
 (define (g-list-foreach glist func)
   (let ((len (g_list_length glist)))
@@ -257,10 +264,7 @@
 	 (gtk_widget_show scale)
 	 (gtk_box_pack_start (GTK_BOX scan-row) label #t #t 0)
 	 (gtk_widget_show label)
-	 (g_signal_connect_closure_by_id (GPOINTER adj)
-					 (g_signal_lookup "value_changed" (G_OBJECT_TYPE (GTK_OBJECT adj)))
-					 0
-					 (g_cclosure_new (lambda (w d) (func (.value (GTK_ADJUSTMENT adj)))) #f #f) #f)))
+	 (g_signal_connect adj "value_changed" (lambda (w d) (func (.value (GTK_ADJUSTMENT adj)))) #f)))
      (list (list "mass" 1 200 100 2 (lambda (val) (set! mass (/ val 100.0))))
 	   (list "spring" 1 100 10 2 (lambda (val) (set! xspring (/ val 100.0))))
 	   (list "damping" 0 100 0 4 (lambda (val) (set! damp (/ val 10000.0))))))
@@ -275,44 +279,37 @@
 	  (gtk_box_pack_start (GTK_BOX scan-size) scan-text #t #t 0)
 	  (gtk_widget_show scan-text)
 	  (gtk_entry_set_text (GTK_ENTRY scan-text) (number->string size))
-	  (g_signal_connect_closure_by_id (GPOINTER scan-text)
-					  (g_signal_lookup "activate" 
-							   (G_OBJECT_TYPE (GTK_OBJECT scan-text)))
-					  0
-					  (g_cclosure_new (lambda (w d) 
-							    (stop-synthesis)
-							    (set! size (string->number (gtk_entry_get_text (GTK_ENTRY scan-text))))
-							    (set! tbl (make-table-lookup :size size))
-							    (set! gx0 (mus-data tbl))
-							    (set! gx1 (make-vct size))	   
-							    (set! gx2 (make-vct size))
-							    (set! vect (make-vector (* size 2))))
-							  #f #f) #f))))
+	  (g_signal_connect scan-text "activate"
+			    (lambda (w d) 
+			      (stop-synthesis)
+			      (set! size (string->number (gtk_entry_get_text (GTK_ENTRY scan-text))))
+			      (set! tbl (make-table-lookup :size size))
+			      (set! gx0 (mus-data tbl))
+			      (set! gx1 (make-vct size))	   
+			      (set! gx2 (make-vct size))
+			      (set! vect (make-vector (* size 2))))
+			    #f))))
     (set! play-button (gtk_check_button_new_with_label "play"))
     (gtk_box_pack_start (GTK_BOX scan-row) play-button #f #f 4)
     (gtk_widget_show play-button)
-    (g_signal_connect_closure_by_id (GPOINTER play-button)
-				    (g_signal_lookup "toggled" 
-						     (G_OBJECT_TYPE (GTK_OBJECT play-button)))
-				    0
-				    (g_cclosure_new (lambda (w d) 
-						      (if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON play-button))
-							  (let* ((audio-info (open-play-output 1 22050 #f 128))
-								 (audio-fd (car audio-info))
-								 (outchans (cadr audio-info))
-								 (frames (caddr audio-info))
-								 (data (make-sound-data outchans frames)))
-							    (if (not (= audio-fd -1))
-								(do ()
-								    ((or (c-g?) (not (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON play-button))))
-								     (mus-audio-close audio-fd))
-								  (tick-synthesis work-proc)
-								  (do ((k 0 (1+ k)))
-								      ((= k frames))
-								    (sound-data-set! data 0 k (* amplitude (table-lookup tbl))))
-								  (mus-audio-write audio-fd data frames))))))
-						    #f #f) #f)
-    
+    (g_signal_connect play-button "toggled"
+		      (lambda (w d) 
+			(if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON play-button))
+			    (let* ((audio-info (open-play-output 1 22050 #f 128))
+				   (audio-fd (car audio-info))
+				   (outchans (cadr audio-info))
+				   (frames (caddr audio-info))
+				   (data (make-sound-data outchans frames)))
+			      (if (not (= audio-fd -1))
+				  (do ()
+				      ((or (c-g?) (not (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON play-button))))
+				       (mus-audio-close audio-fd))
+				    (tick-synthesis work-proc)
+				    (do ((k 0 (1+ k)))
+					((= k frames))
+				      (sound-data-set! data 0 k (* amplitude (table-lookup tbl))))
+				    (mus-audio-write audio-fd data frames))))))
+		      #f)
     (let* ((freq-adj (gtk_adjustment_new 440.0 20.0 1000.0 1.0 10.0 1.0)) ; step incr, page incr page size
 	   (amp-adj (gtk_adjustment_new 0.02 0.0 0.1 .001 .01 .001))
 	   (freq-scale (gtk_hscale_new (GTK_ADJUSTMENT freq-adj)))
@@ -335,46 +332,24 @@
       (gtk_widget_show amp-scale)
       (gtk_widget_show freq-label)
       (gtk_widget_show amp-label)
-      (g_signal_connect_closure_by_id (GPOINTER freq-adj)
-				      (g_signal_lookup "value_changed" (G_OBJECT_TYPE (GTK_OBJECT freq-adj)))
-				      0
-				      (g_cclosure_new (lambda (w d) (set! (mus-frequency tbl) (.value (GTK_ADJUSTMENT freq-adj)))) 
-						      #f #f) 
-				      #f)
-      (g_signal_connect_closure_by_id (GPOINTER amp-adj)
-				      (g_signal_lookup "value_changed" (G_OBJECT_TYPE (GTK_OBJECT amp-adj)))
-				      0
-				      (g_cclosure_new (lambda (w d) (set! amplitude (.value (GTK_ADJUSTMENT amp-adj)))) 
-						      #f #f) 
-				      #f)
+      (g_signal_connect freq-adj "value_changed" (lambda (w d) (set! (mus-frequency tbl) (.value (GTK_ADJUSTMENT freq-adj)))) #f)
+      (g_signal_connect amp-adj "value_changed" (lambda (w d) (set! amplitude (.value (GTK_ADJUSTMENT amp-adj)))) #f)
       )
     
-    (g_signal_connect_closure_by_id (GPOINTER scan-pane)
-				    (g_signal_lookup "expose_event" (G_OBJECT_TYPE (GTK_OBJECT scan-pane)))
-				    0 (g_cclosure_new (lambda (w e d) (redraw-graph)) #f #f) #f)
-    (g_signal_connect_closure_by_id (GPOINTER scan-pane)
-				    (g_signal_lookup "configure_event" (G_OBJECT_TYPE (GTK_OBJECT scan-pane)))
-				    0 (g_cclosure_new (lambda (w e d) (redraw-graph)) #f #f) #f)
-    (g_signal_connect_closure_by_id (GPOINTER scan-pane)
-				    (g_signal_lookup "button_press_event" (G_OBJECT_TYPE (GTK_OBJECT scan-pane)))
-				    0
-				    (g_cclosure_new (lambda (w e d) 
-						      (let ((button (.button (GDK_EVENT_BUTTON e))))
-							(if (not work-proc)
-							    (if (= button 2)
-								(continue-synthesis)
-								(start-synthesis))
-							    (stop-synthesis))))
-						    #f #f) #f)
-    (g_signal_connect_closure_by_id (GPOINTER scan-start)
-				    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT scan-start))) 0
-				    (g_cclosure_new (lambda (w d) (start-synthesis)) #f #f) #f)
-    (g_signal_connect_closure_by_id (GPOINTER scan-continue)
-				    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT scan-continue))) 0
-				    (g_cclosure_new (lambda (w d) (continue-synthesis)) #f #f) #f)
-    (g_signal_connect_closure_by_id (GPOINTER scan-stop)
-				    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT scan-stop))) 0
-				    (g_cclosure_new (lambda (w d) (stop-synthesis)) #f #f) #f)
+    (g_signal_connect scan-pane "expose_event" (lambda (w e d) (redraw-graph)) #f)
+    (g_signal_connect scan-pane "configure_event" (lambda (w e d) (redraw-graph)) #f)
+    (g_signal_connect scan-pane "button_press_event" 
+		      (lambda (w e d) 
+			(let ((button (.button (GDK_EVENT_BUTTON e))))
+			  (if (not work-proc)
+			      (if (= button 2)
+				  (continue-synthesis)
+				  (start-synthesis))
+			      (stop-synthesis))))
+		      #f)
+    (g_signal_connect scan-start "clicked" (lambda (w d) (start-synthesis)) #f)
+    (g_signal_connect scan-continue "clicked" (lambda (w d) (continue-synthesis)) #f)
+    (g_signal_connect scan-stop "clicked" (lambda (w d) (stop-synthesis)) #f)
     #t))
 
 
@@ -637,24 +612,16 @@ Reverb-feedback sets the scaler on the feedback.
 	(gtk_window_set_default_size (GTK_WINDOW hidden-controls-dialog) -1 -1)
 	(gtk_window_set_resizable (GTK_WINDOW hidden-controls-dialog) #t)
 	(gtk_widget_realize hidden-controls-dialog)
-	(g_signal_connect_closure_by_id (GPOINTER hidden-controls-dialog)
-					(g_signal_lookup "delete_event" (G_OBJECT_TYPE (GTK_OBJECT hidden-controls-dialog)))
-					0 (g_cclosure_new (lambda (w ev data) (gtk_widget_hide hidden-controls-dialog)) #f #f) #f)
+	(g_signal_connect hidden-controls-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide hidden-controls-dialog) #t) #f)
 
 	(gtk_box_pack_start (GTK_BOX (.action_area (GTK_DIALOG hidden-controls-dialog))) dismiss-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER dismiss-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT dismiss-button)))
-					0 (g_cclosure_new (lambda (w data) (gtk_widget_hide hidden-controls-dialog)) #f #f) #f)
+	(g_signal_connect dismiss-button "clicked" (lambda (w data) (gtk_widget_hide hidden-controls-dialog)) #f)
 	(gtk_widget_show dismiss-button)
 	(gtk_box_pack_start (GTK_BOX (.action_area (GTK_DIALOG hidden-controls-dialog))) reset-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER reset-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT reset-button)))
-					0 (g_cclosure_new (lambda (w data) (reset-all-sliders)) #f #f) #f)
+	(g_signal_connect reset-button "clicked" (lambda (w data) (reset-all-sliders)) #f)
 	(gtk_widget_show reset-button)
 	(gtk_box_pack_end (GTK_BOX (.action_area (GTK_DIALOG hidden-controls-dialog))) help-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER help-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT help-button)))
-					0 (g_cclosure_new (lambda (w data) (help-dialog "More Controls" hidden-controls-help)) #f #f) #f)
+	(g_signal_connect help-button "clicked" (lambda (w data) (help-dialog "More Controls" hidden-controls-help)) #f)
 	(gtk_widget_show help-button)
 	
 	(let ((mainform (.vbox (GTK_DIALOG hidden-controls-dialog))))
@@ -675,19 +642,14 @@ Reverb-feedback sets the scaler on the feedback.
 	       (gtk_box_pack_start (GTK_BOX mainform) slider #t #t 4)
 	       (gtk_box_pack_start (GTK_BOX mainform) label #t #t 4)
 	       (set! hidden-controls (cons (list adj initial func) hidden-controls))	       
-	       (g_signal_connect_closure_by_id (GPOINTER adj)
-					       (g_signal_lookup "value_changed" (G_OBJECT_TYPE (GTK_OBJECT adj)))
-					       0
-					       (g_cclosure_new (lambda (adj data)
-								 (set! (func) (.value (GTK_ADJUSTMENT adj))))
-							       #f #f)
-					       #f)
+	       (g_signal_connect adj "value_changed" (lambda (adj data) (set! (func) (.value (GTK_ADJUSTMENT adj)))) #f)
 	       (gtk_widget_show slider)
 	       (gtk_widget_show label)))
-	   (list (list "expand-hop" 0.01 1.0 0.05  expand-control-hop)
+
+	   (list (list "expand-hop" 0.001 0.3 0.05  expand-control-hop)
 		 (list "expand-length" 0.01 .5 0.15 expand-control-length)
 		 (list "expand-ramp" 0.01 .5 0.4 expand-control-ramp)
-		 (list "expand-jitter" 0.0 .5 0.1 expand-control-jitter)
+		 (list "expand-jitter" 0.0 2.0 1.0 expand-control-jitter)
 		 (list "contrast-amp" 0.0 2.0 1.0 contrast-control-amp)
 		 (list "reverb-lowpass" 0.0 1.0 0.7 reverb-control-lowpass)
 		 (list "reverb-feedback" 0.0 1.25 1.09 reverb-control-feedback))))
@@ -753,14 +715,28 @@ Reverb-feedback sets the scaler on the feedback.
 ;;;   (This order means we can't use stop_emission to disable the built-in callback)
 
 (define (keep-file-dialog-open-upon-ok)
-  (let* ((dialog (open-file-dialog #f))
-	 (ok-button (.ok_button (GTK_FILE_SELECTION dialog))))
-    (g_signal_connect_closure_by_id 
-     (GPOINTER ok-button)
-     (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT ok-button)))
-     0
-     (g_cclosure_new (lambda (w d) (gtk_widget_show dialog)) #f #f) #f)
-    'ok))
+  (let ((dialog (open-file-dialog #f)))
+    (g_object_set_data (G_OBJECT dialog) "hide-me" (GPOINTER 1)))) ; anything not 0 means don't hide (this is a stupid kludge forced on me by goddamn gtk)
+	
+#!
+;;; The new Gtk FileChooserDialog "response" callback has no "context"
+;;;   argument (which I use on all other callbacks to pass in the Scheme/Ruby function
+;;;   to be called), so there's no way to connect a local function to that event.
+;;;   So this code can't work:
+	(if (GTK_IS_FILE_CHOOSER dialog)
+	    (g_signal_connect dialog "response" (lambda (w id)
+						  (if (= id GTK_RESPONSE_OK)
+						      (gtk_widget_show dialog)))
+			      #f)
+	    ...)
+;;; I'll try to trap the problem in xg.c and throw an error (rather than segfault)
+;;;   [added later: that failed, so now a bad context simply returns silently!]
+;;;
+;;; so use g_object_set_data with "hide-me"
+;;;
+;;; Another problem: how many more of these broken callbacks are there in Gtk?
+!#
+
 
 
 ;;; -------- snd-clock-icon --------
@@ -768,6 +744,7 @@ Reverb-feedback sets the scaler on the feedback.
 ;;; a clock icon to replace Snd's hourglass
 ;;;   call from a work proc or whatever with hour going from 0 to 12 then #f
 
+(if (defined? 'gdk_pixmap_new)
 (define snd-clock-icon
   (let* ((shell (cadr (main-widgets)))
 	 (win (car (main-widgets)))
@@ -775,7 +752,7 @@ Reverb-feedback sets the scaler on the feedback.
 	 (dgc (car (snd-gcs))))
     (do ((i 0 (1+ i)))
 	((= i 12))
-      (let* ((pix (gdk_pixmap_new win 16 16 -1))
+      (let* ((pix (gdk_pixmap_new (GDK_DRAWABLE win) 16 16 -1))
 	     (pixwin (GDK_DRAWABLE pix)))
 	(vector-set! clock-pixmaps i pix)
 	(gdk_gc_set_foreground dgc (basic-color))
@@ -791,7 +768,7 @@ Reverb-feedback sets the scaler on the feedback.
     (lambda (snd hour)
       (gdk_draw_drawable (GDK_DRAWABLE (.window (list-ref (sound-widgets snd) 8))) dgc 
 			 (GDK_DRAWABLE (vector-ref clock-pixmaps hour)) 0 0 0 4 16 16)
-      #f)))
+      #f))))
 
 
 
@@ -848,31 +825,26 @@ Reverb-feedback sets the scaler on the feedback.
 	     (filter (if (> (length args) 3) (list-ref args 3) "*"))
 	     (dialog (or (find-free-dialog file-selector-dialogs)
 		 	 (let ((new-dialog (gtk_file_selection_new title)))
-			   (g_signal_connect_closure_by_id 
-			    (GPOINTER new-dialog)
-			    (g_signal_lookup "delete_event" (G_OBJECT_TYPE (GTK_OBJECT new-dialog))) 0
-			    (g_cclosure_new (lambda (w e d) 
-					      (let ((lst (find-dialog new-dialog file-selector-dialogs)))
-						(list-set! lst 1 #f)
-						(gtk_widget_hide new-dialog)))
-					    #f #f) #f)
-			   (g_signal_connect_closure_by_id 
-			    (GPOINTER (.ok_button (GTK_FILE_SELECTION new-dialog)))
-			    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT (.ok_button (GTK_FILE_SELECTION new-dialog))))) 0
-			    (g_cclosure_new (lambda (w d)
-					      (let ((lst (find-dialog new-dialog file-selector-dialogs)))
-						((list-ref lst 2) (gtk_file_selection_get_filename (GTK_FILE_SELECTION new-dialog)))
-						(list-set! lst 1 #f)
-						(gtk_widget_hide new-dialog)))
-					    #f #f) #f)
-			   (g_signal_connect_closure_by_id 
-			    (GPOINTER (.cancel_button (GTK_FILE_SELECTION new-dialog)))
-			    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT (.cancel_button (GTK_FILE_SELECTION new-dialog))))) 0
-			    (g_cclosure_new (lambda (w d) 
-					      (let ((lst (find-dialog new-dialog file-selector-dialogs)))
-						(list-set! lst 1 #f)
-						(gtk_widget_hide new-dialog)))
-					    #f #f) #f)
+			   (g_signal_connect new-dialog "delete_event" 
+					     (lambda (w e d) 
+					       (let ((lst (find-dialog new-dialog file-selector-dialogs)))
+						 (list-set! lst 1 #f)
+						 (gtk_widget_hide new-dialog)
+						 #t))
+					     #f)
+			   (g_signal_connect  (.ok_button (GTK_FILE_SELECTION new-dialog)) "clicked"
+					      (lambda (w d)
+						(let ((lst (find-dialog new-dialog file-selector-dialogs)))
+						  ((list-ref lst 2) (gtk_file_selection_get_filename (GTK_FILE_SELECTION new-dialog)))
+						  (list-set! lst 1 #f)
+						  (gtk_widget_hide new-dialog)))
+					      #f)
+			   (g_signal_connect (.cancel_button (GTK_FILE_SELECTION new-dialog)) "clicked"
+					     (lambda (w d) 
+					       (let ((lst (find-dialog new-dialog file-selector-dialogs)))
+						 (list-set! lst 1 #f)
+						 (gtk_widget_hide new-dialog)))
+					     #f)
 			   (set! file-selector-dialogs (cons (list new-dialog #t func title) file-selector-dialogs))
 			   new-dialog))))
 	(gtk_widget_show dialog)))))
@@ -892,21 +864,14 @@ Reverb-feedback sets the scaler on the feedback.
       (gtk_container_add (GTK_CONTAINER frame) meter)
       (gtk_widget_show meter)
       (let ((context (list meter 0.0 1.0 0.0 0.0 width height)))
-	(g_signal_connect_closure_by_id 
-	 (GPOINTER meter)
-	 (g_signal_lookup "expose_event" (G_OBJECT_TYPE (GTK_OBJECT meter))) 0
-	 (g_cclosure_new (lambda (w e d) 
-			   (display-level d)) 
-			 context #f) #f)
-	(g_signal_connect_closure_by_id 
-	 (GPOINTER meter)
-	 (g_signal_lookup "configure_event" (G_OBJECT_TYPE (GTK_OBJECT meter))) 0
-	 (g_cclosure_new (lambda (w e d)
-			   (let ((xy (gdk_drawable_get_size (GDK_DRAWABLE (.window w)))))
-			     (list-set! d 5 (car xy))
-			     (list-set! d 6 (cadr xy))
-			     (display-level d)))
-			 context #f) #f)
+	(g_signal_connect meter "expose_event" (lambda (w e d) (display-level d)) context)
+	(g_signal_connect meter "configure_event" 
+			  (lambda (w e d)
+			    (let ((xy (gdk_drawable_get_size (GDK_DRAWABLE (.window w)))))
+			      (list-set! d 5 (car xy))
+			      (list-set! d 6 (cadr xy))
+			      (display-level d)))
+			  context)
 	context))))
 
 (define (display-level meter-data)
@@ -1025,9 +990,11 @@ Reverb-feedback sets the scaler on the feedback.
 
 (define (add-delete-and-rename-options)
   (let ((dialog (open-file-dialog #f)))
-    (gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION dialog))
+    (if (GTK_IS_FILE_SELECTION dialog) ; in newer gtk's this is a file_chooser, not a file_selection widget
+	(gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION dialog)))
     (set! dialog (mix-file-dialog #f))
-    (gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION dialog))))
+    (if (GTK_IS_FILE_SELECTION dialog)
+	(gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION dialog)))))
 
   
 
@@ -1055,7 +1022,7 @@ Reverb-feedback sets the scaler on the feedback.
 "-------X----------"))
 
 (define (make-pixmap strs) ; strs is list of strings as in arrow-strs above
-  (let ((win (car (main-widgets))))
+  (let ((win (GDK_DRAWABLE (car (main-widgets)))))
     (gdk_pixmap_create_from_xpm_d win #f (basic-color) (list->c-array strs "gchar**"))))
 
 
@@ -1074,14 +1041,9 @@ Reverb-feedback sets the scaler on the feedback.
     (gtk_window_set_default_size (GTK_WINDOW variables-dialog) -1 -1)
     (gtk_window_set_resizable (GTK_WINDOW variables-dialog) #t)
     (gtk_widget_realize variables-dialog)
-    (g_signal_connect_closure_by_id (GPOINTER variables-dialog)
-				    (g_signal_lookup "delete_event" (G_OBJECT_TYPE (GTK_OBJECT variables-dialog)))
-				    0 (g_cclosure_new (lambda (w ev data) (gtk_widget_hide variables-dialog)) #f #f) #f)
-
+    (g_signal_connect_closure variables-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide variables-dialog) #t) #f)
     (gtk_box_pack_start (GTK_BOX (.action_area (GTK_DIALOG variables-dialog))) dismiss-button #t #t 20)
-    (g_signal_connect_closure_by_id (GPOINTER dismiss-button)
-				    (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT dismiss-button)))
-				    0 (g_cclosure_new (lambda (w data) (gtk_widget_hide variables-dialog)) #f #f) #f)
+    (g_signal_connect dismiss-button "clicked" (lambda (w data) (gtk_widget_hide variables-dialog)) #f)
     (gtk_widget_show dismiss-button)
     (set! variables-notebook (gtk_notebook_new))
     (gtk_box_pack_start (GTK_BOX (.vbox (GTK_DIALOG variables-dialog))) variables-notebook #t #t 4)
@@ -1214,62 +1176,51 @@ Reverb-feedback sets the scaler on the feedback.
 	(gtk_window_set_default_size (GTK_WINDOW font-selector-dialog) -1 -1)
 	(gtk_window_set_resizable (GTK_WINDOW font-selector-dialog) #t)
 	(gtk_widget_realize font-selector-dialog)
-	(g_signal_connect_closure_by_id (GPOINTER font-selector-dialog)
-					(g_signal_lookup "delete_event" (G_OBJECT_TYPE (GTK_OBJECT font-selector-dialog)))
-					0 (g_cclosure_new (lambda (w ev data) (gtk_widget_hide font-selector-dialog)) #f #f) #f)
+	(g_signal_connect font-selector-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide font-selector-dialog) #t) #f)
 	(gtk_box_pack_start (GTK_BOX (.action_area (GTK_DIALOG font-selector-dialog))) dismiss-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER dismiss-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT dismiss-button)))
-					0 (g_cclosure_new (lambda (w data) (gtk_widget_hide font-selector-dialog)) #f #f) #f)
+	(g_signal_connect dismiss-button "clicked" (lambda (w data) (gtk_widget_hide font-selector-dialog)) #f)
 	(gtk_widget_show dismiss-button)
 	(gtk_box_pack_end (GTK_BOX (.action_area (GTK_DIALOG font-selector-dialog))) help-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER help-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT help-button)))
-					0 (g_cclosure_new (lambda (w data) 
-							    (help-dialog "Choose a Font" 
-									 "choose a font, set which fields you want to use that font, and click 'ok'"))
-							  #f #f) #f)
+	(g_signal_connect help-button "clicked" 
+			  (lambda (w data) 
+			    (help-dialog "Choose a Font" 
+					 "choose a font, set which fields you want to use that font, and click 'ok'"))
+			  #f)
 	(gtk_widget_show help-button)
 	(let ((ok-button (.ok_button (GTK_FONT_SELECTION_DIALOG font-selector-dialog))))
 	  (gtk_widget_set_name ok-button "doit_button")
-	  (g_signal_connect_closure_by_id 
-	   (GPOINTER ok-button)
-	   (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT ok-button)))
-	   0
-	   (g_cclosure_new (lambda (w d) 
-			     (let ((new-font (gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG font-selector-dialog))))
-			       (for-each 
-				(lambda (fd)
-				  (let ((button (car fd))
-					(func (cadr fd)))
-				    (if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON button))
-					(set! (func) new-font))))
-				font-selectors)
-			       (if (> (length (sounds)) 0)
-				   (begin
-				     (if (time-graph?) (update-time-graph))
-				     (if (transform-graph?) (update-transform-graph))
-				     (if (lisp-graph?) (update-lisp-graph))))))
-			   #f #f) #f))
+	  (g_signal_connect ok-button "clicked"
+			    (lambda (w d) 
+			      (let ((new-font (gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG font-selector-dialog))))
+				(for-each 
+				 (lambda (fd)
+				   (let ((button (car fd))
+					 (func (cadr fd)))
+				     (if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON button))
+					 (set! (func) new-font))))
+				 font-selectors)
+				(if (> (length (sounds)) 0)
+				    (begin
+				      (if (time-graph?) (update-time-graph))
+				      (if (transform-graph?) (update-transform-graph))
+				      (if (lisp-graph?) (update-lisp-graph))))))
+			    #f))
 	(let ((cancel (.cancel_button (GTK_FONT_SELECTION_DIALOG font-selector-dialog))))
 	  (gtk_widget_set_name cancel "reset_button")
-	  (g_signal_connect_closure_by_id 
-	   (GPOINTER cancel)
-	   (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT cancel)))
-	   0
-	   (g_cclosure_new (lambda (w d) 
-			     (for-each 
-			      (lambda (fd)
-				(let ((func (cadr fd))
-				      (old-value (caddr fd)))
-				  (set! (func) old-value)))
-				font-selectors)
-			       (if (> (length (sounds)) 0)
-				   (begin
-				     (if (time-graph?) (update-time-graph))
-				     (if (transform-graph?) (update-transform-graph))
-				     (if (lisp-graph?) (update-lisp-graph)))))
-			   #f #f) #f))
+	  (g_signal_connect cancel "clicked"
+			    (lambda (w d) 
+			      (for-each 
+			       (lambda (fd)
+				 (let ((func (cadr fd))
+				       (old-value (caddr fd)))
+				   (set! (func) old-value)))
+			       font-selectors)
+			      (if (> (length (sounds)) 0)
+				  (begin
+				    (if (time-graph?) (update-time-graph))
+				    (if (transform-graph?) (update-transform-graph))
+				    (if (lisp-graph?) (update-lisp-graph)))))
+			    #f))
 	(let ((mainform (.vbox (GTK_DIALOG font-selector-dialog))))
 	  (let ((label (gtk_label_new "Apply font to:")))
 	    (gtk_box_pack_start (GTK_BOX mainform) label #f #f 4)
@@ -1305,68 +1256,57 @@ Reverb-feedback sets the scaler on the feedback.
 	(gtk_color_selection_set_has_palette (GTK_COLOR_SELECTION (.colorsel (GTK_COLOR_SELECTION_DIALOG color-selector-dialog))) #t)
 	(gtk_window_set_resizable (GTK_WINDOW color-selector-dialog) #t)
 	(gtk_widget_realize color-selector-dialog)
-	(g_signal_connect_closure_by_id (GPOINTER color-selector-dialog)
-					(g_signal_lookup "delete_event" (G_OBJECT_TYPE (GTK_OBJECT color-selector-dialog)))
-					0 (g_cclosure_new (lambda (w ev data) (gtk_widget_hide color-selector-dialog)) #f #f) #f)
+	(g_signal_connect color-selector-dialog "delete_event" (lambda (w ev data) (gtk_widget_hide color-selector-dialog) #t) #f)
 	(gtk_box_pack_start (GTK_BOX (.action_area (GTK_DIALOG color-selector-dialog))) dismiss-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER dismiss-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT dismiss-button)))
-					0 (g_cclosure_new (lambda (w data) (gtk_widget_hide color-selector-dialog)) #f #f) #f)
+	(g_signal_connect dismiss-button "clicked" (lambda (w data) (gtk_widget_hide color-selector-dialog)) #f)
 	(gtk_widget_show dismiss-button)
 	(gtk_box_pack_end (GTK_BOX (.action_area (GTK_DIALOG color-selector-dialog))) help-button #t #t 20)
-	(g_signal_connect_closure_by_id (GPOINTER help-button)
-					(g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT help-button)))
-					0 (g_cclosure_new (lambda (w data) 
-							    (help-dialog "Choose a Color" 
-									 "choose a color, set which fields you want to use that color, and click 'ok'"))
-							  #f #f) #f)
+	(g_signal_connect help-button "clicked" 
+			  (lambda (w data) 
+			    (help-dialog "Choose a Color" 
+					 "choose a color, set which fields you want to use that color, and click 'ok'"))
+			  #f)
 	(gtk_widget_show help-button)
 	(let ((ok-button (.ok_button (GTK_COLOR_SELECTION_DIALOG color-selector-dialog))))
 	  (gtk_widget_set_name ok-button "doit_button")
-	  (g_signal_connect_closure_by_id 
-	   (GPOINTER ok-button)
-	   (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT ok-button)))
-	   0
-	   (g_cclosure_new (lambda (w d) 
-			     (let ((color (GdkColor))
-				   (new-color #f))
-			       (gtk_color_selection_get_current_color 
-				(GTK_COLOR_SELECTION (.colorsel (GTK_COLOR_SELECTION_DIALOG color-selector-dialog)))
-				color)
-			       (set! new-color (gdk_color_copy color))
-			       (gdk_rgb_find_color (gdk_colormap_get_system) new-color)
-			       (for-each 
-				(lambda (fd)
-				  (let ((button (car fd))
-					(func (cadr fd)))
-				    (if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON button))
-					(set! (func) new-color))))
-				color-selectors)
-			       (if (> (length (sounds)) 0)
-				   (begin
-				     (if (time-graph?) (update-time-graph))
-				     (if (transform-graph?) (update-transform-graph))
-				     (if (lisp-graph?) (update-lisp-graph))))))
-			   #f #f) #f))
+	  (g_signal_connect ok-button "clicked"
+			    (lambda (w d) 
+			      (let ((color (GdkColor))
+				    (new-color #f))
+				(gtk_color_selection_get_current_color 
+				 (GTK_COLOR_SELECTION (.colorsel (GTK_COLOR_SELECTION_DIALOG color-selector-dialog)))
+				 color)
+				(set! new-color (gdk_color_copy color))
+				(gdk_rgb_find_color (gdk_colormap_get_system) new-color)
+				(for-each 
+				 (lambda (fd)
+				   (let ((button (car fd))
+					 (func (cadr fd)))
+				     (if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON button))
+					 (set! (func) new-color))))
+				 color-selectors)
+				(if (> (length (sounds)) 0)
+				    (begin
+				      (if (time-graph?) (update-time-graph))
+				      (if (transform-graph?) (update-transform-graph))
+				      (if (lisp-graph?) (update-lisp-graph))))))
+			    #f))
 	(let ((cancel (.cancel_button (GTK_COLOR_SELECTION_DIALOG color-selector-dialog))))
 	  (gtk_widget_set_name cancel "reset_button")
-	  (g_signal_connect_closure_by_id 
-	   (GPOINTER cancel)
-	   (g_signal_lookup "clicked" (G_OBJECT_TYPE (GTK_OBJECT cancel)))
-	   0
-	   (g_cclosure_new (lambda (w d) 
-			     (for-each 
-			      (lambda (fd)
-				(let ((func (cadr fd))
-				      (old-value (caddr fd)))
-				  (set! (func) old-value)))
-				color-selectors)
-			       (if (> (length (sounds)) 0)
-				   (begin
-				     (if (time-graph?) (update-time-graph))
-				     (if (transform-graph?) (update-transform-graph))
-				     (if (lisp-graph?) (update-lisp-graph)))))
-			   #f #f) #f))
+	  (g_signal_connect cancel "clicked"
+			    (lambda (w d) 
+			      (for-each 
+			       (lambda (fd)
+				 (let ((func (cadr fd))
+				       (old-value (caddr fd)))
+				   (set! (func) old-value)))
+			       color-selectors)
+			      (if (> (length (sounds)) 0)
+				  (begin
+				    (if (time-graph?) (update-time-graph))
+				    (if (transform-graph?) (update-transform-graph))
+				    (if (lisp-graph?) (update-lisp-graph)))))
+			    #f))
 	(let ((mainform (.vbox (GTK_DIALOG color-selector-dialog))))
 	  (let ((label (gtk_label_new "Apply color to:")))
 	    (gtk_box_pack_start (GTK_BOX mainform) label #f #f 4)
