@@ -115,8 +115,8 @@ bool set_axis_numbers_font(char *font)
 
 int sg_text_width(char *txt, PangoFontDescription *font)
 {
-  PangoLayout *layout = NULL;
   int wid = 0;
+  PangoLayout *layout = NULL;
   if (txt == NULL) return(0);
   if (strlen(txt) == 0) return(0);
   layout = pango_layout_new(gdk_pango_context_get());
@@ -153,8 +153,8 @@ int number_width(char *num)
 
 static int sg_font2width(PangoFontDescription *font)
 {
-  PangoLayout *layout = NULL;
   int wid = 0;
+  PangoLayout *layout = NULL;
   layout = pango_layout_new(gdk_pango_context_get());
   if (layout)
     {
@@ -168,8 +168,8 @@ static int sg_font2width(PangoFontDescription *font)
 
 static int sg_font2height(PangoFontDescription *font)
 {
-  PangoLayout *layout = NULL;
   int wid = 0;
+  PangoLayout *layout = NULL;
   layout = pango_layout_new(gdk_pango_context_get());
   if (layout)
     {
@@ -279,7 +279,6 @@ void gc_set_foreground_xor(GdkGC *gc, GdkColor *col1, GdkColor *col2)
   gdk_gc_set_foreground(gc, gdk_color_copy(&newcol)); /* memleak? */
 }
 
-
 void color_cursor(GdkColor *color)
 {
   state_context *sx;
@@ -503,10 +502,10 @@ void sg_text_delete(GtkWidget *w, int start, int end)
 
 void sg_text_insert(GtkWidget *w, char *text)
 {
-  GtkTextIter pos;
-  GtkTextBuffer *buf;
   if (text)
     {
+      GtkTextIter pos;
+      GtkTextBuffer *buf;
       buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w));
       gtk_text_buffer_get_end_iter(buf, &pos);
       gtk_text_buffer_insert(buf, &pos, text, strlen(text));
@@ -598,7 +597,7 @@ GtkWidget *sg_make_list(const char *title, GtkWidget *parent, widget_add_t paned
   gtk_container_add(GTK_CONTAINER(scrolled_win), list);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_set_border_width(GTK_CONTAINER(scrolled_win), 0);
-  g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)), "changed", G_CALLBACK(callback), gp);
+  SG_SIGNAL_CONNECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)), "changed", callback, gp);
 
   switch (paned)
     {
@@ -723,3 +722,31 @@ GtkWidget *snd_gtk_label_new(const char *label, GdkColor *color)
   gtk_widget_modify_base(rlw, GTK_STATE_ACTIVE, color);
   return(rlw);
 }
+
+
+void widget_int_to_text(GtkWidget *w, int val)
+{
+  char *str;
+  str = (char *)CALLOC(8, sizeof(char));
+  mus_snprintf(str, 8, "%d", val);
+  gtk_entry_set_text(GTK_ENTRY(w), str);
+  FREE(str);
+}
+
+void widget_float_to_text(GtkWidget *w, Float val)
+{
+  char *str;
+  str = (char *)CALLOC(8, sizeof(char));
+  mus_snprintf(str, 8, "%.2f", val);
+  gtk_entry_set_text(GTK_ENTRY(w), str);
+  FREE(str);
+}
+
+void widget_off_t_to_text(GtkWidget *w, off_t val)
+{
+  char *str;
+  str = (char *)CALLOC(8, sizeof(char));
+  mus_snprintf(str, 8, OFF_TD, val);
+  gtk_entry_set_text(GTK_ENTRY(w), str);
+  FREE(str);
+}
