@@ -98,12 +98,15 @@ string 'cmd'.  cmd should be a URL."
   return(C_TO_XEN_BOOLEAN(send_netscape(html_program(ss), XEN_TO_C_STRING(cmd))));
 }
 
+/* TODO: change_window_property is flakey in gtk if not run with --sync (and is X dependent) */
+
 static void change_window_property(char *winat, char *name, char *command)
 {
   Window window;
   Display *dpy;
   dpy = MAIN_DISPLAY(ss);
-  if ((window = find_window(dpy, DefaultRootWindow(dpy), winat, compare_window)))
+  window = find_window(dpy, DefaultRootWindow(dpy), winat, compare_window);
+  if (window)
     {
       XChangeProperty(dpy, 
 		      window, 
@@ -138,10 +141,10 @@ static XEN g_window_property(XEN winat, XEN name)
       (type != None) &&
       (len > 0))
     {
-#if HAVE_GUILE && HAVE_SCM_MEM2STRING
+#if HAVE_GUILE && (HAVE_SCM_MEM2STRING || HAVE_SCM_C_MAKE_RECTANGULAR)
       if (type == XA_STRING)
 	result = C_TO_XEN_STRING((char *)data[0]);
-      else result = scm_mem2string((char *)data[0], len * format / 8);
+      else result = C_TO_XEN_STRINGN((char *)data[0], len * format / 8);
 #else
       result = C_TO_XEN_STRING((char *)data);
 #endif
