@@ -137,7 +137,6 @@ static void main_snd_help(const char *subject, ...)
 
 static char *xm_version(void)
 {
-  char *version = NULL;
   XEN xm_val = XEN_FALSE;
 #if HAVE_GUILE
   xm_val = XEN_EVAL_C_STRING("(and (or (provided? 'xm) (provided? 'xg)) xm-version)");
@@ -149,6 +148,7 @@ static char *xm_version(void)
 #endif
   if (XEN_STRING_P(xm_val))
     {
+      char *version = NULL;
       version = (char *)CALLOC(32, sizeof(char));
       mus_snprintf(version, 32, "\n    %s: %s", 
 #if USE_MOTIF
@@ -166,7 +166,6 @@ static char *xm_version(void)
 #if HAVE_GL
 static char *gl_version(void)
 {
-  char *version = NULL;
   XEN gl_val = XEN_FALSE;
 #if HAVE_GUILE
   gl_val = XEN_EVAL_C_STRING("(and (provided? 'gl) gl-version)");
@@ -178,6 +177,7 @@ static char *gl_version(void)
 #endif
   if (XEN_STRING_P(gl_val))
     {
+      char *version = NULL;
       version = (char *)CALLOC(32, sizeof(char));
       mus_snprintf(version, 32, "\n    gl: %s", XEN_TO_C_STRING(gl_val));
       if (snd_itoa_ctr < snd_itoa_size) snd_itoa_strs[snd_itoa_ctr++] = version;
@@ -274,6 +274,9 @@ char *version_info(void)
 	  ", Glib ",     snd_itoa(GLIB_MAJOR_VERSION), ".", 
                          snd_itoa(GLIB_MINOR_VERSION), ".", 
                          snd_itoa(GLIB_MICRO_VERSION),
+  #ifdef PANGO_VERSION
+	  ", Pango ", PANGO_VERSION,
+  #endif
 #endif
 	  xm_version(), /* omitted if --version/--help because the init procs haven't run at that point */
 #if WITH_GTK_AND_X11
@@ -288,20 +291,6 @@ char *version_info(void)
 	  snd_itoa(GTKGLEXT_MAJOR_VERSION), ".",
 	  snd_itoa(GTKGLEXT_MINOR_VERSION), ".",
 	  snd_itoa(GTKGLEXT_MICRO_VERSION),
-    #else
-      #if HAVE_GTK_GL_EXT_0_1
-	  "0.1",
-      #else
-        #if HAVE_GDK_GL_FONT_USE_GDK_FONT
-	  "0.2",
-        #else
-          #if HAVE_GDK_GL_CONTEXT_COPY
-	  "0.4",
-          #else
-	  "0.3",
-          #endif
-        #endif
-      #endif
     #endif
   #endif
 #endif
@@ -366,32 +355,27 @@ char *version_info(void)
 
 void about_snd_help(void)
 {
-  char *info = NULL, *features = NULL, *files = NULL;
-  info = version_info();
+  char *info = NULL, *features = NULL;
 #if HAVE_GUILE
+  char *files = NULL;
   features = word_wrap(XEN_AS_STRING(XEN_EVAL_C_STRING("*features*")), 400);
   files = word_wrap(XEN_AS_STRING(XEN_EVAL_C_STRING("snd-loaded-files")), 400);
 #endif
 #if HAVE_RUBY
   features = word_wrap(XEN_AS_STRING(XEN_EVAL_C_STRING("$\".join(' ')")), 400);
 #endif
+  info = version_info();
   main_snd_help("Snd is a sound editor.",
 	    info,
 	    "\nRecent changes include:\n\
 \n\
-23-Jun:  new configure variables GUILE_CONFIG_name and GUILE_name (for OSX/fink)\n\
-         cursor-location-offset\n\
-18-Jun:  ssb-fm in dsp.scm.\n\
-10-Jun:  gtk support in fft-menu.scm thanks to Kjetil\n\
-         ssb-bank in dsp.scm.\n\
-9-Jun:   ssb-am in CLM.\n\
-4-Jun:   added add-colormap, delete-colormap, colormap-size, colormap-name, changed colormap-ref\n\
-         removed 2 previously built-in colormaps, added rainbow colormap, default colormap is 0\n\
-1-Jun:   read-ascii in examp.scm.\n\
-27-May:  cursor-update-interval.\n\
-25-May:  removed stop-playing-region-hook, stop-playing-channel-hook, before-apply-hook\n\
-         added stop-function arg to play, play-and-wait, play-selection, play-region, add-player\n\
-21-May:  --with-doc-dir configuration switch.\n\
+3-Jan:   removed next-to-last (xen fallback) arg to ptree-channel,\n\
+           and associated xen-channel stuff.\n\
+29-Dec:  eval-c.scm (Kjetil Matheussen).\n\
+7-Dec:   after-save-as-hook, replacing emacs-style-save-as (see snd7.scm).\n\
+3-Dec:   mark-tag-width|height.\n\
+17-Nov:  snd 7.9\n\
+16-Nov:  moved the 'Apply' button and friends from the Controls pane to a popup menu.\n\
 ",
 #if HAVE_GUILE
 	    "\n    *features*: \n'", features, "\n\n",
@@ -407,6 +391,9 @@ void about_snd_help(void)
 NULL);
   if (info) FREE(info);
   if (features) FREE(features);
+#if HAVE_GUILE
+  if (files) FREE(files);
+#endif
 }
 
 
@@ -559,13 +546,15 @@ variable " S_transform_normalization " to " S_dont_normalize ".",
 		      snd_xref_urls("FFT"));
 }
 
-static char *control_xrefs[8] = {
+static char *control_xrefs[10] = {
   "various control panel variables: {Control panel}",
   "amplitude: {scale-by}",
   "speed or srate: {src-sound}",
   "expand: {granulate}",
   "contrast: {contrast-enhancement}",
   "filter: {filter-sound}",
+  "{apply-controls}",
+  "(controls->channel}",
   NULL};
 
 void controls_help(void) 
@@ -602,14 +591,6 @@ envelope and filter order in the text windows provided. \
 The envelope X axis goes from 0 to half the sampling rate. The actual frequency response (given the current filter order) \
 is displayed in blue.  The filter is on only if the filter button is set. \
 \n\n\
-The 'Remember' button saves the current control panel state for a subsequent 'Restore'. \
-The 'Reset' button returns the control panel to a clean (no-change) condition. \
-To take the current panel settings and turn them into an edit of the sound, click the 'Apply' button.  Apply may change \
-the length of the file; for example, if reverb is on, the reverb decay length is added onto \
-the end.  Once Apply has taken effect, the controls section is reset to its clean state (so \
-a subsequent 'play' plays the unmodified newly edited version).  To use 'Apply' \
-over the current selection, or over the current channel (rather than the entire sound), use control-click. \
-\n\n\
 The keyboard commands associated with the control panel are: \
 \n\n\
   C-x C-o   show control panel\n\
@@ -841,10 +822,10 @@ static char *key_xrefs[4] = {
 
 static void show_key_help(int key, int state, bool cx, char *help)
 {
-  char buf[1024];
-  char cbuf[256];
   if (help)
     {
+      char buf[1024];
+      char cbuf[256];
       make_key_name(cbuf, 256, key, state, cx);
       mus_snprintf(buf, 1024, "\n%s: %s", cbuf, help);
       snd_help_append(buf);
@@ -1553,7 +1534,7 @@ static char **xref_tables[NUM_XREFS] = {
   Deletions_xrefs, Envelopes_xrefs, Filters_xrefs, Searching_xrefs, Insertions_xrefs, Maxamps_xrefs,
   Playing_xrefs, Reversing_xrefs, Saving_xrefs, Smoothing_xrefs, Resampling_xrefs, FFTs_xrefs, Reverb_xrefs,
   Resampling_xrefs, Searching_xrefs, Undo_and_Redo_xrefs, Undo_and_Redo_xrefs, 
-  sync_xrefs, control_xrefs, record_xrefs, header_and_data_xrefs, key_xrefs, Tracks_xrefs, Vcts_xrefs,
+  sync_xrefs, control_xrefs, record_xrefs, header_and_data_xrefs, key_xrefs, Tracks_xrefs, Copying_xrefs,
   Noise_Reduction_xrefs, Window_size_and_position_xrefs, Colors_xrefs, control_xrefs, Random_Numbers_xrefs
 };
 
@@ -1562,7 +1543,7 @@ static char **xref_url_tables[NUM_XREFS] = {
   Deletions_urls, Envelopes_urls, Filters_urls, Searching_urls, Insertions_urls, Maxamps_urls,
   Playing_urls, Reversing_urls, Saving_urls, Smoothing_urls, Resampling_urls, FFTs_urls, Reverb_urls,
   Resampling_urls, Searching_urls, Undo_and_Redo_urls, Undo_and_Redo_urls, 
-  NULL, NULL, NULL, NULL, Tracks_urls, Vcts_urls, 
+  NULL, NULL, NULL, NULL, Tracks_urls, Copying_urls, 
   Noise_Reduction_urls, Window_size_and_position_urls, Colors_urls, NULL, Random_Numbers_urls,
 };
 
@@ -1714,11 +1695,12 @@ static bool strings_might_match(const char *a, const char *b, int len)
 char **help_name_to_xrefs(const char *name)
 {
   char **xrefs = NULL;
-  int i, xref_ctr = 0, xrefs_size = 0, name_len, cur_len;
+  int i, xref_ctr = 0, xrefs_size = 0, name_len;
   name_len = strlen(name);
   for (i = 0; i < HELP_NAMES_SIZE; i++)
     if (name[0] == help_names[i][0])
       {
+	int cur_len;
 	cur_len = strlen(help_names[i]);
 	if (strings_might_match(name, help_names[i], (name_len < cur_len) ? name_len : cur_len))
 	  {
@@ -1881,7 +1863,6 @@ static char *doc_files[DOC_DIRECTORIES] = {
   
 static char *html_directory(void)
 {
-  char *hd = NULL;
   int i;
   if (mus_file_probe("snd.html"))
     {
@@ -1893,6 +1874,7 @@ static char *html_directory(void)
   if (html_dir(ss))
     {
       bool happy;
+      char *hd = NULL;
       hd = (char *)CALLOC(snd_strlen(html_dir(ss)) + 16, sizeof(char));
       sprintf(hd, html_dir(ss), "/snd.html");
       happy = mus_file_probe(hd);
@@ -1910,7 +1892,7 @@ static char *html_directory(void)
 
 void url_to_html_viewer(char *url)
 {
-  char *path, *dir_path;
+  char *dir_path;
   dir_path = html_directory();
   if (dir_path)
     {
@@ -1918,6 +1900,7 @@ void url_to_html_viewer(char *url)
       program = html_program(ss);
       if (program)
 	{
+	  char *path;
 	  path = (char *)CALLOC(strlen(dir_path) + strlen(url) + 256, sizeof(char));
 	  if ((strcmp(program, "netscape") == 0) ||
 	      (strcmp(program, "mozilla") == 0))
@@ -2254,5 +2237,3 @@ If more than one hook function, each function gets the previous function's outpu
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_html_dir, g_html_dir_w, H_html_dir, S_setB S_html_dir, g_set_html_dir_w,  0, 0, 1, 0);
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_html_program, g_html_program_w, H_html_program, S_setB S_html_program, g_set_html_program_w,  0, 0, 1, 0);
 }
-
-
