@@ -5,10 +5,10 @@
 static void c_io_bufclr (snd_io *io, int beg)
 {
   int k, end;
-  mus_sample_t *j;
   end = io->bufsize;
   for (k = 0; k < io->chans; k++)
     {
+      mus_sample_t *j;
       j = io->arrays[k];
       if (j)
 	memset((void *)(j + beg), 0, (end - beg) * sizeof(mus_sample_t));
@@ -44,10 +44,10 @@ static void reposition_file_buffers(snd_data *sd, off_t index)
 {
   int fd = 0;
   bool reclose = false;
-  file_info *hdr;
   if (index < 0) index = 0; /* if reading in reverse, don't fall off the start of the buffer */
   if (sd->open == FD_CLOSED)
     {
+      file_info *hdr;
       /* try to open it with sndlib descriptors */
       fd = mus_file_open_read(sd->filename); 
       if (fd == -1) 
@@ -80,7 +80,7 @@ static void reposition_file_buffers(snd_data *sd, off_t index)
     }
 }
 
-snd_io *make_file_state(int fd, file_info *hdr, int chan, int suggested_bufsize)
+snd_io *make_file_state(int fd, file_info *hdr, int chan, off_t beg, int suggested_bufsize)
 {
   snd_io *io;
   int bufsize;
@@ -103,7 +103,7 @@ snd_io *make_file_state(int fd, file_info *hdr, int chan, int suggested_bufsize)
   if (bufsize <= 0) {fprintf(stderr,"make_file_state, bufsize: %d\n", bufsize); abort();}
 #endif
   io->arrays[chan] = (mus_sample_t *)CALLOC(bufsize, sizeof(mus_sample_t));
-  reposition_file_buffers_1(0, io); /* get ready to read -- we're assuming mus_file_read_chans here */
+  reposition_file_buffers_1(beg, io); /* get ready to read -- we're assuming mus_file_read_chans here */
   return(io);
 }
 
@@ -149,29 +149,26 @@ void file_buffers_back(off_t ind0, off_t ind1, off_t indx, snd_fd *sf, snd_data
 
 static void close_temp_files(chan_info *cp, void *closed)
 {
-  int i, rtn;
-  snd_data *sd;
-  if (cp)
+  if ((cp) && (cp->sounds))
     {
-      if (cp->sounds)
+      int i, rtn;
+      rtn = (*((int *)closed));
+      for (i = 0; i < cp->sound_size; i++)
 	{
-	  rtn = (*((int *)closed));
-	  for (i = 0; i < cp->sound_size; i++)
+	  snd_data *sd;
+	  sd = cp->sounds[i];
+	  if ((sd) && 
+	      (sd->type == SND_DATA_FILE) && 
+	      (sd->io) && 
+	      (sd->open == FD_OPEN))
 	    {
-	      sd = cp->sounds[i];
-	      if ((sd) && 
-		  (sd->type == SND_DATA_FILE) && 
-		  (sd->io) && 
-		  (sd->open == FD_OPEN))
-		{
-		  mus_file_close(sd->io->fd);
-		  sd->open = FD_CLOSED;
-		  sd->io->fd = -1;
-		  rtn++;
-		}
+	      mus_file_close(sd->io->fd);
+	      sd->open = FD_CLOSED;
+	      sd->io->fd = -1;
+	      rtn++;
 	    }
-	  (*((int *)closed)) = rtn;
 	}
+      (*((int *)closed)) = rtn;
     }
 }
 
@@ -182,7 +179,7 @@ static int too_many_files_cleanup(void)
   rtn = -1;
   closed = (int *)MALLOC(sizeof(int));
   (*closed) = 0;
-  for_each_chan_1(close_temp_files, (void *)closed);
+  for_each_normal_chan_1(close_temp_files, (void *)closed);
   if ((*closed) == 0) 
     for_each_region_chan(close_temp_files, (void *)closed);
   if ((*closed) == 0)
@@ -196,7 +193,7 @@ int snd_open_read(const char *arg)
 {
   int fd;
   fd = OPEN(arg, O_RDONLY, 0);
-  if ((fd == -1) && (errno == EMFILE))
+  if ((fd == -1) && (errno == EMFILE)) /* there's also ENFILE = file table overflow (/usr/include/asm/errno.h) */
     {
       fd = too_many_files_cleanup();
       if (fd != -1) 
@@ -209,10 +206,10 @@ int snd_open_read(const char *arg)
 
 bool snd_overwrite_ok(const char *ofile)
 {
-  int fil;
   bool rtn = true;
   if (ask_before_overwrite(ss))
     {
+      int fil;
       fil = OPEN(ofile, O_RDONLY, O_NONBLOCK);
       if (fil != -1) 
 	{
@@ -347,10 +344,10 @@ void remember_temp(const char *filename, int chans)
 static void forget_temp(const char *filename, int chan)
 {
   int i, j;
-  bool happy = false;
-  tempfile_ctr *tmp;
   for (i = 0; i < tempfiles_size; i++)
     {
+      bool happy = false;
+      tempfile_ctr *tmp;
       tmp = tempfiles[i];
       if ((tmp) && (strcmp(filename, tmp->name) == 0))
 	{
@@ -377,9 +374,9 @@ static void forget_temp(const char *filename, int chan)
 static void tick_temp(const char *filename, int chan)
 {
   int i;
-  tempfile_ctr *tmp;
   for (i = 0; i < tempfiles_size; i++)
     {
+      tempfile_ctr *tmp;
       tmp = tempfiles[i];
       if ((tmp) && (strcmp(filename, tmp->name) == 0))
 	{
@@ -421,7 +418,7 @@ snd_data *make_snd_data_file(const char *name, snd_io *io, file_info *hdr, file_
   return(sd);
 }
 
-snd_data *copy_snd_data(snd_data *sd, int bufsize)
+snd_data *copy_snd_data(snd_data *sd, off_t beg, int bufsize)
 {
   snd_data *sf;
   snd_io *io;
@@ -439,7 +436,7 @@ snd_data *copy_snd_data(snd_data *sd, int bufsize)
 			    hdr->chans,
 			    hdr->type);
   during_open(fd, sd->filename, SND_COPY_READER);
-  io = make_file_state(fd, hdr, sd->chan, bufsize);
+  io = make_file_state(fd, hdr, sd->chan, beg, bufsize);
   sf = (snd_data *)CALLOC(1, sizeof(snd_data));
   sf->type = sd->type;
   sf->buffered_data = io->arrays[sd->chan];
