@@ -70,14 +70,15 @@ static void continue_macro (int keysym, int state)
 static named_macro *name_macro(char *name)
 {
   named_macro *nm;
-  int i, old_size;
   if (named_macro_ctr == named_macro_size)
     {
+      int old_size;
       old_size = named_macro_size;
       named_macro_size += 16;
       if (!named_macros) named_macros = (named_macro **)CALLOC(named_macro_size, sizeof(named_macro *));
       else 
 	{
+	  int i;
 	  named_macros = (named_macro **)REALLOC(named_macros, named_macro_size * sizeof(named_macro *));
 	  for (i = old_size; i < named_macro_size; i++) named_macros[i] = NULL;
 	}
@@ -93,13 +94,13 @@ static named_macro *name_macro(char *name)
 static void name_last_macro (char *name)
 {
   named_macro *nm;
-  macro_cmd *mc;
   int i;
   nm = name_macro(name);
   nm->macro_size = macro_size;
   nm->cmds = (macro_cmd **)CALLOC(macro_size, sizeof(macro_cmd *));
   for (i = 0; i < macro_size; i++)
     {
+      macro_cmd *mc;
       nm->cmds[i] = (macro_cmd *)CALLOC(1, sizeof(macro_cmd));
       mc = nm->cmds[i];
       mc->keysym = macro_cmds[i]->keysym;
@@ -136,21 +137,22 @@ static void save_macro_1(named_macro *nm, FILE *fd)
 #endif
 }
 
-static int execute_named_macro_1(chan_info *cp, char *name, off_t count)
+static bool execute_named_macro_1(chan_info *cp, char *name, off_t count)
 {
-  int i, k;
-  off_t j;
-  named_macro *nm;
-  macro_cmd *mc;
+  int k;
   for (k = 0; k < named_macro_ctr; k++)
     {
       if ((named_macros[k]->name) && 
 	  (strcmp(name, named_macros[k]->name) == 0))
 	{
+	  int i;
+	  off_t j;
+	  named_macro *nm;
 	  nm = named_macros[k];
 	  for (j = 0; j < count; j++)
 	    for (i = 0; i < nm->macro_size; i++) 
 	      {
+		macro_cmd *mc;
 		mc = nm->cmds[i];
 		if (mc->keysym != 0)
 		  {
@@ -159,20 +161,20 @@ static int execute_named_macro_1(chan_info *cp, char *name, off_t count)
 		    keyboard_command(cp, mc->keysym, mc->state);
 		  }
 	      }
-	  return(1);
+	  return(true);
 	}
     }
-  return(0);
+  return(false);
 }
 
 static void execute_named_macro(chan_info *cp, char *name, off_t count)
 {
-  int one_edit, i, loc, form_loc;
-  XEN form, result = XEN_UNDEFINED;
   if (!(execute_named_macro_1(cp, name, count)))
     /* not a macro...*/
     {
 #if HAVE_EXTENSION_LANGUAGE
+      int one_edit, i, loc, form_loc;
+      XEN form, result = XEN_UNDEFINED;
       one_edit = cp->edit_ctr + 1;
       form = string_to_form(name);
       form_loc = snd_protect(form);
@@ -187,7 +189,7 @@ static void execute_named_macro(chan_info *cp, char *name, off_t count)
 	  if (ss->deferred_regions > 0) 
 	    sequester_deferred_regions(cp, one_edit - 1);
 	  while (cp->edit_ctr > one_edit) backup_edit_list(cp);
-	  if (cp->mixes) backup_mix_list(cp, one_edit);
+	  if (cp->have_mixes) backup_mix_list(cp, one_edit);
 	}
 #else
       /* not sure it's possible to get here at all -- execute undefined named macro?? */
@@ -203,6 +205,7 @@ typedef struct {
   XEN func; 
   bool cx_extended; /* Sun/Forte C defines "extended" somewhere */
   char *origin;
+  int gc_loc;
 } key_entry; 
 
 static key_entry *user_keymap = NULL;
@@ -237,66 +240,65 @@ bool map_over_key_bindings(bool (*func)(int, int, bool, XEN, char *))
 
 #define NUM_BUILT_IN_KEY_BINDINGS 77
 static key_entry built_in_key_bindings[NUM_BUILT_IN_KEY_BINDINGS] = {
-  {snd_K_Down, 0, 0, XEN_FALSE, false, "zoom out"},
-  {snd_K_Up, 0, 0, XEN_FALSE, false, "zoom in"},
-  {snd_K_Left, 0, 0, XEN_FALSE, false, "move window left"},
-  {snd_K_Right, 0, 0, XEN_FALSE, false, "move window right"},
-  {snd_K_less, 0, 0, XEN_FALSE, false, "move cursor to sample 0"},
+  {snd_K_Down,    0, 0, XEN_FALSE, false, "zoom out"},
+  {snd_K_Up,      0, 0, XEN_FALSE, false, "zoom in"},
+  {snd_K_Left,    0, 0, XEN_FALSE, false, "move window left"},
+  {snd_K_Right,   0, 0, XEN_FALSE, false, "move window right"},
+  {snd_K_less,    0, 0, XEN_FALSE, false, "move cursor to sample 0"},
   {snd_K_greater, 0, 0, XEN_FALSE, false, "move cursor to last sample"},
 
-  {snd_K_less, snd_ControlMask, 0, XEN_FALSE, false, "move cursor to sample 0"},
-  {snd_K_greater, snd_ControlMask, 0, XEN_FALSE, false, "move cursor to last sample"},
-  {snd_K_a, snd_ControlMask, 0, XEN_FALSE, false, "move cursor to window start"},
-  {snd_K_b, snd_ControlMask, 0, XEN_FALSE, false, "move cursor back one sample"},
-  {snd_K_d, snd_ControlMask, 0, XEN_FALSE, false, "delete sample at cursor"},
-  {snd_K_e, snd_ControlMask, 0, XEN_FALSE, false, "move cursor to window end"},
-  {snd_K_f, snd_ControlMask, 0, XEN_FALSE, false, "move cursor ahead one sample"},
-  {snd_K_g, snd_ControlMask, 0, XEN_FALSE, false, "abort current command"},
-  {snd_K_h, snd_ControlMask, 0, XEN_FALSE, false, "delete previous sample"},
-  {snd_K_i, snd_ControlMask, 0, XEN_FALSE, false, "display cursor info"},
-  {snd_K_j, snd_ControlMask, 0, XEN_FALSE, false, "goto mark"},
-  {snd_K_k, snd_ControlMask, 0, XEN_FALSE, false, "delete one line's worth of samples"},
-  {snd_K_l, snd_ControlMask, 0, XEN_FALSE, false, "position window so cursor is in the middle"},
-  {snd_K_m, snd_ControlMask, 0, XEN_FALSE, false, "place (or remove) mark at cursor location"},
-  {snd_K_n, snd_ControlMask, 0, XEN_FALSE, false, "move cursor ahead one 'line'"},
-  {snd_K_o, snd_ControlMask, 0, XEN_FALSE, false, "insert one zero sample at cursor"},
-  {snd_K_p, snd_ControlMask, 0, XEN_FALSE, false, "move cursor back one 'line'"},
-  {snd_K_q, snd_ControlMask, 0, XEN_FALSE, false, "play current channel starting at cursor"},
-  {snd_K_r, snd_ControlMask, 0, XEN_FALSE, false, "search backwards"},
-  {snd_K_s, snd_ControlMask, 0, XEN_FALSE, false, "search forwards"},
-  {snd_K_t, snd_ControlMask, 0, XEN_FALSE, false, "stop playing"},
-  {snd_K_u, snd_ControlMask, 0, XEN_FALSE, false, "start arg count definition."},
-  {snd_K_v, snd_ControlMask, 0, XEN_FALSE, false, "move cursor to mid-window"},
-  {snd_K_w, snd_ControlMask, 0, XEN_FALSE, false, "delete selection"},
-  {snd_K_y, snd_ControlMask, 0, XEN_FALSE, false, "insert selection."},
-  {snd_K_z, snd_ControlMask, 0, XEN_FALSE, false, "set sample at cursor to 0.0"},
+  {snd_K_less,       snd_ControlMask, 0, XEN_FALSE, false, "move cursor to sample 0"},
+  {snd_K_greater,    snd_ControlMask, 0, XEN_FALSE, false, "move cursor to last sample"},
+  {snd_K_a,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor to window start"},
+  {snd_K_b,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor back one sample"},
+  {snd_K_d,          snd_ControlMask, 0, XEN_FALSE, false, "delete sample at cursor"},
+  {snd_K_e,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor to window end"},
+  {snd_K_f,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor ahead one sample"},
+  {snd_K_g,          snd_ControlMask, 0, XEN_FALSE, false, "abort current command"},
+  {snd_K_h,          snd_ControlMask, 0, XEN_FALSE, false, "delete previous sample"},
+  {snd_K_i,          snd_ControlMask, 0, XEN_FALSE, false, "display cursor info"},
+  {snd_K_j,          snd_ControlMask, 0, XEN_FALSE, false, "goto mark"},
+  {snd_K_k,          snd_ControlMask, 0, XEN_FALSE, false, "delete one line's worth of samples"},
+  {snd_K_l,          snd_ControlMask, 0, XEN_FALSE, false, "position window so cursor is in the middle"},
+  {snd_K_m,          snd_ControlMask, 0, XEN_FALSE, false, "place (or remove) mark at cursor location"},
+  {snd_K_n,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor ahead one 'line'"},
+  {snd_K_o,          snd_ControlMask, 0, XEN_FALSE, false, "insert one zero sample at cursor"},
+  {snd_K_p,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor back one 'line'"},
+  {snd_K_q,          snd_ControlMask, 0, XEN_FALSE, false, "play current channel starting at cursor"},
+  {snd_K_r,          snd_ControlMask, 0, XEN_FALSE, false, "search backwards"},
+  {snd_K_s,          snd_ControlMask, 0, XEN_FALSE, false, "search forwards"},
+  {snd_K_t,          snd_ControlMask, 0, XEN_FALSE, false, "stop playing"},
+  {snd_K_u,          snd_ControlMask, 0, XEN_FALSE, false, "start arg count definition."},
+  {snd_K_v,          snd_ControlMask, 0, XEN_FALSE, false, "move cursor to mid-window"},
+  {snd_K_w,          snd_ControlMask, 0, XEN_FALSE, false, "delete selection"},
+  {snd_K_y,          snd_ControlMask, 0, XEN_FALSE, false, "insert selection."},
+  {snd_K_z,          snd_ControlMask, 0, XEN_FALSE, false, "set sample at cursor to 0.0"},
   {snd_K_underscore, snd_ControlMask, 0, XEN_FALSE, false, "undo"},
-  {snd_K_space, snd_ControlMask, 0, XEN_FALSE, false, "start selection definition"},
+  {snd_K_space,      snd_ControlMask, 0, XEN_FALSE, false, "start selection definition"},
+  {snd_K_g,          snd_ControlMask | snd_MetaMask, 0, XEN_FALSE, false, "clear listener"},
+  {snd_K_less,       snd_MetaMask,    0, XEN_FALSE, false, "move cursor to sample 0"},
+  {snd_K_greater,    snd_MetaMask,    0, XEN_FALSE, false, "move cursor to last sample"},
 
-  {snd_K_g, snd_ControlMask | snd_MetaMask, 0, XEN_FALSE, false, "clear listener"},
-  {snd_K_less, snd_MetaMask, 0, XEN_FALSE, false, "move cursor to sample 0"},
-  {snd_K_greater, snd_MetaMask, 0, XEN_FALSE, false, "move cursor to last sample"},
-
-  {snd_K_a, 0, 0, XEN_FALSE, true, "apply envelope to selection"},
-  {snd_K_b, 0, 0, XEN_FALSE, true, "position window so cursor is on left margin"},
-  {snd_K_c, 0, 0, XEN_FALSE, true, "define selection from cursor to nth mark"},
-  {snd_K_d, 0, 0, XEN_FALSE, true, "set temp dir name"},
-  {snd_K_e, 0, 0, XEN_FALSE, true, "execute keyboard macro"},
-  {snd_K_f, 0, 0, XEN_FALSE, true, "position window so cursor is on right margin"},
-  {snd_K_i, 0, 0, XEN_FALSE, true, "insert region"},
-  {snd_K_j, 0, 0, XEN_FALSE, true, "goto named mark"},
-  {snd_K_k, 0, 0, XEN_FALSE, true, "close file"},
-  {snd_K_l, 0, 0, XEN_FALSE, true, "position selection in mid-view"},
-  {snd_K_o, 0, 0, XEN_FALSE, true, "move to next or previous graph"},
-  {snd_K_p, 0, 0, XEN_FALSE, true, "play selection or region n"},
-  {snd_K_q, 0, 0, XEN_FALSE, true, "mix in selection"},
-  {snd_K_r, 0, 0, XEN_FALSE, true, "redo"},
-  {snd_K_u, 0, 0, XEN_FALSE, true, "undo"},
-  {snd_K_v, 0, 0, XEN_FALSE, true, "position window over selection"},
-  {snd_K_w, 0, 0, XEN_FALSE, true, "save selection as file"},
-  {snd_K_z, 0, 0, XEN_FALSE, true, "smooth selection"},
-  {snd_K_slash, 0, 0, XEN_FALSE, true, "place named mark"},
-  {snd_K_openparen, 0, 0, XEN_FALSE, true, "begin keyboard macro definition"},
+  {snd_K_a,          0, 0, XEN_FALSE, true, "apply envelope to selection"},
+  {snd_K_b,          0, 0, XEN_FALSE, true, "position window so cursor is on left margin"},
+  {snd_K_c,          0, 0, XEN_FALSE, true, "define selection from cursor to nth mark"},
+  {snd_K_d,          0, 0, XEN_FALSE, true, "set temp dir name"},
+  {snd_K_e,          0, 0, XEN_FALSE, true, "execute keyboard macro"},
+  {snd_K_f,          0, 0, XEN_FALSE, true, "position window so cursor is on right margin"},
+  {snd_K_i,          0, 0, XEN_FALSE, true, "insert region"},
+  {snd_K_j,          0, 0, XEN_FALSE, true, "goto named mark"},
+  {snd_K_k,          0, 0, XEN_FALSE, true, "close file"},
+  {snd_K_l,          0, 0, XEN_FALSE, true, "position selection in mid-view"},
+  {snd_K_o,          0, 0, XEN_FALSE, true, "move to next or previous graph"},
+  {snd_K_p,          0, 0, XEN_FALSE, true, "play selection or region n"},
+  {snd_K_q,          0, 0, XEN_FALSE, true, "mix in selection"},
+  {snd_K_r,          0, 0, XEN_FALSE, true, "redo"},
+  {snd_K_u,          0, 0, XEN_FALSE, true, "undo"},
+  {snd_K_v,          0, 0, XEN_FALSE, true, "position window over selection"},
+  {snd_K_w,          0, 0, XEN_FALSE, true, "save selection as file"},
+  {snd_K_z,          0, 0, XEN_FALSE, true, "smooth selection"},
+  {snd_K_slash,      0, 0, XEN_FALSE, true, "place named mark"},
+  {snd_K_openparen,  0, 0, XEN_FALSE, true, "begin keyboard macro definition"},
   {snd_K_closeparen, 0, 0, XEN_FALSE, true, "end keyboard macro definition"},
 
   {snd_K_a, snd_ControlMask, 0, XEN_FALSE, true, "apply envelope"},
@@ -321,9 +323,6 @@ static key_entry built_in_key_bindings[NUM_BUILT_IN_KEY_BINDINGS] = {
 
 char *key_binding_description(int key, int state, bool cx_extended)
 {
-#if HAVE_GUILE
-  XEN value, help_text = XEN_FALSE;
-#endif
   int pos;
   if ((key < MIN_KEY_CODE) || (key > MAX_KEY_CODE) ||
       (state < MIN_KEY_STATE) || (state > MAX_KEY_STATE))
@@ -333,6 +332,7 @@ char *key_binding_description(int key, int state, bool cx_extended)
   if (pos >= 0)
     {
 #if HAVE_GUILE
+      XEN value, help_text = XEN_FALSE;
       value = user_keymap[pos].func;
       help_text = XEN_PROCEDURE_HELP(value);            /* (procedure-property ...) */
       if (XEN_FALSE_P(help_text))
@@ -344,7 +344,7 @@ char *key_binding_description(int key, int state, bool cx_extended)
       if (!(XEN_FALSE_P(help_text))) 
 	{
 	  if (XEN_STRING_P(help_text))
-	    return(XEN_TO_C_STRING(help_text));
+	    return(XEN_TO_C_STRING(help_text)); /* this is dangerous -- these are temp strings, but they're used as temps in snd-help */
 	  return(XEN_AS_STRING(help_text));
 	}
 #endif
@@ -367,7 +367,7 @@ static XEN g_key_binding(XEN key, XEN state, XEN cx_extended)
   XEN_ASSERT_TYPE(XEN_INTEGER_P(key), key, XEN_ARG_1, S_key_binding, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(state), state, XEN_ARG_2, S_key_binding, "an integer");
   XEN_ASSERT_TYPE(XEN_BOOLEAN_IF_BOUND_P(cx_extended), cx_extended, XEN_ARG_3, S_key_binding, "a boolean");
-  k = XEN_TO_SMALL_C_INT(key);
+  k = XEN_TO_C_INT(key);
   s = XEN_TO_C_INT_OR_ELSE(state, 0);
   if ((k < MIN_KEY_CODE) || (k > MAX_KEY_CODE) ||
       (s < MIN_KEY_STATE) || (s > MAX_KEY_STATE))
@@ -406,6 +406,7 @@ static void set_keymap_entry(int key, int state, int args, XEN func, bool cx_ext
 		  user_keymap[i].func = XEN_UNDEFINED;
 		  user_keymap[i].cx_extended = false;
 		  user_keymap[i].origin = NULL;
+		  user_keymap[i].gc_loc = -1;
 		}
 	    }
 	}
@@ -418,8 +419,12 @@ static void set_keymap_entry(int key, int state, int args, XEN func, bool cx_ext
     }
   else
     {
-      if (XEN_PROCEDURE_P(user_keymap[i].func))
-	snd_unprotect(user_keymap[i].func);
+      if ((XEN_PROCEDURE_P(user_keymap[i].func)) &&
+	  (user_keymap[i].gc_loc >= 0))
+	{
+	  snd_unprotect_at(user_keymap[i].gc_loc);
+	  user_keymap[i].gc_loc = -1;
+	}
       if (user_keymap[i].origin)
 	{
 	  FREE(user_keymap[i].origin);
@@ -429,7 +434,8 @@ static void set_keymap_entry(int key, int state, int args, XEN func, bool cx_ext
   user_keymap[i].origin = copy_string(origin);
   user_keymap[i].args = args;
   user_keymap[i].func = func;
-  if (XEN_PROCEDURE_P(func)) snd_protect(func);
+  if (XEN_PROCEDURE_P(func)) 
+    user_keymap[i].gc_loc = snd_protect(func);
 }
 
 static void call_user_keymap(int hashedsym, int count)
@@ -486,7 +492,7 @@ void report_in_minibuffer(snd_info *sp, const char *format, ...)
 {
   char *buf;
   va_list ap;
-  if ((!sp) || (!(sp->active)) || (!(sp->sgx))) return;
+  if ((!sp) || (!(sp->active)) || (!(sp->sgx)) || (sp->inuse != SOUND_NORMAL)) return;
   va_start(ap, format);
   buf = vstr(format, ap);
   va_end(ap);
@@ -519,7 +525,7 @@ void clear_minibuffer(snd_info *sp)
   sp->searching = 0;
   sp->marking = 0;
   sp->filing = NOT_FILING;
-  sp->printing = false;
+  sp->printing = NOT_PRINTING;
   sp->minibuffer_on = MINI_OFF;
   sp->loading = false;
   sp->amping = 0;
@@ -698,19 +704,10 @@ static chan_info *goto_next_graph (chan_info *cp, int count)
 
 void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
 {
-  snd_info *nsp;
   bool s_or_r = false;
-  int nc, i, j, err;
-  off_t len;
+  int err;
   chan_info *active_chan;
-  char *str = NULL, *mcf = NULL;
-  char *newdir, *str1;
-  env *e;
-  mark *m;
-#if HAVE_OPENDIR
-  DIR *dp;
-#endif
-  XEN proc;
+  char *str = NULL;
   if ((keysym == snd_K_s) || (keysym == snd_K_r)) s_or_r = true;
   if (sp != selected_sound()) select_channel(sp, 0);
   active_chan = any_selected_channel(sp);
@@ -742,9 +739,9 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
       /* if not nil, replace previous */
       if (!s_or_r)
 	{
-	  /* str = get_minibuffer_string(sp); */
 	  if ((str) && (*str))
 	    {
+	      XEN proc;
 	      /* check for procedure as arg, or lambda form:
 	       * (lambda (y) (> y .1)) 
 	       * if returns #t, search stops
@@ -770,11 +767,11 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
     }
 #endif
   
-  /* str = get_minibuffer_string(sp); */
   if ((sp->marking) || (sp->finding_mark))
     {
       if (sp->marking) 
 	{
+	  mark *m;
 	  m = add_mark(sp->marking - 1, str, active_chan);
 	  if (m)
 	    {
@@ -797,7 +794,7 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
       if (sp->printing)
 	{
 	  snd_print(str);
-	  sp->printing = false;
+	  sp->printing = NOT_PRINTING;
 	  clear_minibuffer(sp);
 	  free(str);
 	  return;
@@ -816,86 +813,113 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
 	    {
 	      /* don't free(str) locally in this switch statement without setting it to null as well */
 	    case INPUT_FILING:
-	      nsp = snd_open_file(str, false); /* will post error if any */
-	      if (nsp) 
-		{
-		  select_channel(nsp, 0);
-		  clear_minibuffer(sp);
-		}
+	      {
+		snd_info *nsp;
+		nsp = snd_open_file(str, false); /* will post error if any */
+		if (nsp) 
+		  {
+		    select_channel(nsp, 0);
+		    clear_minibuffer(sp);
+		  }
+	      }
 	      break;
 	    case REGION_FILING:
-	      str1 = mus_expand_filename(str);
-	      if (!(snd_overwrite_ok(str1))) 
-		{
-		  free(str); 
-		  FREE(str1); 
-		  return;
-		}
-	      if ((region_count == 0) && (selection_is_active()))
-		err = save_selection(str1, default_output_type(ss), default_output_format(ss), SND_SRATE(sp), NULL, SAVE_ALL_CHANS);
-	      else err = save_region(region_count, str1, default_output_format(ss));
-	      if (err == MUS_NO_ERROR)
-		{
-		  clear_minibuffer(sp); /* get rid of prompt */
-		  report_in_minibuffer(sp, _("%s saved as %s"), 
-				       ((region_count == 0) && (selection_is_active())) ? "selection" : "region",
-				       str1);
-		}
-	      FREE(str1);
+	      {
+		char *filename;
+		filename = mus_expand_filename(str);
+		if (!(snd_overwrite_ok(filename))) 
+		  {
+		    free(str); 
+		    FREE(filename); 
+		    return;
+		  }
+		if ((region_count == 0) && (selection_is_active()))
+		  err = save_selection(filename, default_output_type(ss), default_output_format(ss), SND_SRATE(sp), NULL, SAVE_ALL_CHANS);
+		else err = save_region(region_count, filename, default_output_format(ss));
+		if (err == MUS_NO_ERROR)
+		  {
+		    clear_minibuffer(sp); /* get rid of prompt */
+		    report_in_minibuffer(sp, _("%s saved as %s"), 
+					 ((region_count == 0) && (selection_is_active())) ? "selection" : "region",
+					 filename);
+		  }
+		FREE(filename);
+	      }
 	      break;
 	    case CHANNEL_FILING:
-	      err = save_channel_edits(active_chan, mcf = mus_expand_filename(str), C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "C-x C-w", 0);
-	      if (err == MUS_NO_ERROR)
-		{
-		  clear_minibuffer(sp);
-		  report_in_minibuffer(sp, _("channel %d saved as %s"), 
-				       active_chan->chan,
-				       mcf);
-		}
-	      if (mcf) FREE(mcf);
+	      {
+		char *filename;
+		filename = mus_expand_filename(str);
+		err = save_channel_edits(active_chan, filename, AT_CURRENT_EDIT_POSITION);
+		if (err == MUS_NO_ERROR)
+		  {
+		    clear_minibuffer(sp);
+		    report_in_minibuffer(sp, _("channel %d saved as %s"), 
+					 active_chan->chan,
+					 filename);
+		  }
+		if (filename) FREE(filename);
+	      }
 	      break;
 #if HAVE_OPENDIR
 	    case TEMP_FILING:
-	      newdir = copy_string(str);
-	      clear_minibuffer(sp);
-	      dp = opendir(newdir);
-	      if (dp) 
-		{
-		  closedir(dp);
-		  set_temp_dir(newdir);
-		}
-	      else 
-		{
-		  report_in_minibuffer_and_save(sp, _("can't access %s! temp dir is unchanged"), newdir);
-		  if (newdir) FREE(newdir);
-		}
+	      {
+		DIR *dp;
+		char *newdir;
+		newdir = copy_string(str);
+		clear_minibuffer(sp);
+		dp = opendir(newdir);
+		if (dp) 
+		  {
+		    closedir(dp);
+		    set_temp_dir(newdir);
+		  }
+		else 
+		  {
+		    report_in_minibuffer_and_save(sp, _("can't access %s! temp dir is unchanged"), newdir);
+		    if (newdir) FREE(newdir);
+		  }
+	      }
 	      break;
 #endif
 	    case CHANGE_FILING:
 	      clear_minibuffer(sp);
-	      mix_complete_file_at_cursor(sp, str, "C-x C-q", with_mix_tags(ss), 0);
+	      mix_complete_file_at_cursor(sp, str, with_mix_tags(ss), 0);
 	      break;
 	    case INSERT_FILING:
-	      str1 = mus_expand_filename(str);
-	      nc = mus_sound_chans(str1);
-	      if (nc > 0)
-		{
-		  len = mus_sound_frames(str1);
-		  if (len == 0)
-		    report_in_minibuffer(sp, _("file %s has no data"), str);
-		  else
-		    {
-		      if (!active_chan) active_chan = sp->chans[0];
-		      for (i = active_chan->chan, j = 0; (j < nc) && (i < sp->nchans); i++, j++)
-			{
-			  if (file_insert_samples(CURSOR(active_chan), len, str1, sp->chans[i], j, DONT_DELETE_ME, "C-x C-i", sp->chans[i]->edit_ctr))
-			    update_graph(sp->chans[i]);
-			}
-		      clear_minibuffer(sp);
-		    }
-		}
-	      else report_in_minibuffer_and_save(sp, _("can't read %s's header"), str);
-	      FREE(str1);
+	      {
+		int nc;
+		char *filename;
+		filename = mus_expand_filename(str);
+		nc = mus_sound_chans(filename);
+		if (nc > 0)
+		  {
+		    off_t len;
+		    len = mus_sound_frames(filename);
+		    if (len == 0)
+		      report_in_minibuffer(sp, _("file %s has no data"), str);
+		    else
+		      {
+			int i, j;
+			off_t chan_beg;
+			char *origin;
+			chan_info *ncp;
+			if (!active_chan) active_chan = sp->chans[0];
+			for (i = active_chan->chan, j = 0; (j < nc) && (i < sp->nchans); i++, j++)
+			  {
+			    ncp = sp->chans[i];
+			    chan_beg = CURSOR(active_chan);
+			    origin = mus_format("%s \"%s\" " OFF_TD " %d", S_insert_sound, filename, chan_beg, j);
+			    if (file_insert_samples(chan_beg, len, filename, ncp, j, DONT_DELETE_ME, origin, ncp->edit_ctr))
+			      update_graph(ncp);
+			    FREE(origin);
+			  }
+			clear_minibuffer(sp);
+		      }
+		  }
+		else report_in_minibuffer_and_save(sp, _("can't read %s's header"), str);
+		FREE(filename);
+	      }
 	      break;
 	    case MACRO_FILING: 
 	      if ((macro_cmds) && (macro_size > 0))
@@ -914,6 +938,7 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
 	}
       if (sp->amping)
 	{
+	  env *e;
 	  if (!active_chan) active_chan = sp->chans[0];
 	  e = string_to_env(str);
 	  if (e)
@@ -938,6 +963,7 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
 #if HAVE_EXTENSION_LANGUAGE
       if (sp->macroing)
 	{
+	  off_t len;
 	  len = CURSOR(active_chan);
 	  execute_named_macro(active_chan, str, sp->macroing);
 	  /* if this is a close command from the current minibuffer, the sound may not exist when we return */
@@ -956,6 +982,7 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
       int loc;
       if (snd_strlen(str) > 0)
 	{
+	  XEN proc;
 	  if (sp->raw_prompt)
 	    proc = C_TO_XEN_STRING(str);
 	  else proc = snd_catch_any(eval_str_wrapper, str, str);
@@ -982,9 +1009,9 @@ void snd_minibuffer_activate(snd_info *sp, int keysym, bool with_meta)
 
 static void stop_fft_in_progress(chan_info *cp)
 {
-  chan_context *cx;
   if ((cp) && (cp->cgx))
     {
+      chan_context *cx;
       cx = cp->cgx;
       if (cx->fft_in_progress) 
 	{
@@ -1004,10 +1031,10 @@ static void set_window_bounds(chan_info *cp, int count)
 {
   /* count = sample number to start at */
   axis_info *ap;
-  double sx;
   ap = cp->axis;
   if (ap->x_ambit != 0.0)
     {
+      double sx;
       sx = (((double)count / (double)SND_SRATE(cp->sound)) - ap->xmin) / ap->x_ambit;
       reset_x_display(cp, sx, ap->zx);
     }
@@ -1017,10 +1044,10 @@ static void set_window_size(chan_info *cp, int count)
 {
   /* set samples within window */
   axis_info *ap;
-  double zx;
   ap = cp->axis;
   if (ap->x_ambit != 0.0)
     {
+      double zx;
       zx = ((double)count / (((double)SND_SRATE(cp->sound)) * ap->x_ambit));
       reset_x_display(cp, ap->sx, zx);
     }
@@ -1040,12 +1067,12 @@ static void window_frames_selection(chan_info *cp)
 {
   double x0, x1;
   int i;
-  snd_info *sp;
   x0 = (((double)(selection_beg(cp))) / ((double)SND_SRATE(cp->sound)));
   x1 = x0 + ((double)(selection_len())) / ((double)(SND_SRATE(cp->sound)));
   set_x_axis_x0x1(cp, x0, x1);
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && 
 	  (sp->inuse == SOUND_NORMAL) && 
@@ -1060,7 +1087,6 @@ static void window_frames_selection(chan_info *cp)
 static off_t get_count_1(char *number_buffer, int number_ctr, bool dot_seen, chan_info *cp)
 {
   /* allow floats here = secs */
-  float f;
   int i;
   if (number_ctr == 0) return(1); /* c-u followed by nothing = 1 */
   number_buffer[number_ctr] = '\0';
@@ -1072,6 +1098,7 @@ static off_t get_count_1(char *number_buffer, int number_ctr, bool dot_seen, cha
     }
   if (dot_seen)
     {
+      float f;
       sscanf(number_buffer, "%f", &f);
       return((off_t)(f * SND_SRATE(cp->sound)));
     }
@@ -1160,17 +1187,16 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
   static bool got_count = false;
   static bool m = false;
   bool searching = false, cursor_searching = false, clear_search = true;
-  int hashloc, sync_num, i, state;
+  int hashloc, i, state;
   off_t loc;
   static off_t ext_count = 1;
   static bool got_ext_count = false;
   snd_info *sp;
   axis_info *ap;
-  sync_info *si;
-  mark *mk = NULL;
   /* fprintf(stderr, "(%s %d%s) ", KEY_TO_NAME(keysym), unmasked_state, (extended_mode) ? " (c-x)" : ""); */
   if ((!cp) || (!(cp->sound)) || (!(cp->active))) return;
   sp = cp->sound;
+  if ((!sp) || (sp->inuse != SOUND_NORMAL)) return;
   ap = cp->axis;
   if (keysym >= snd_K_Shift_L) return;
   /* this happens when the user presses Control or Shift etc prior to hitting the actual (modified) key */
@@ -1243,7 +1269,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_D: case snd_K_d: 
 	      cp->cursor_on = true; 
-	      cursor_delete(cp, count, "C-d"); 
+	      cursor_delete(cp, count); 
 	      break;
 	    case snd_K_E: case snd_K_e:
 	      cp->cursor_on = true; 
@@ -1262,7 +1288,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_H: case snd_K_h: 
 	      cp->cursor_on = true; 
-	      cursor_delete(cp, -count, "C-h"); 
+	      cursor_delete(cp, -count); 
 	      break; 
 	    case snd_K_I: case snd_K_i: 
 	      show_cursor_info(cp); 
@@ -1274,43 +1300,48 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_K: case snd_K_k: 
 	      cp->cursor_on = true; 
-	      cursor_delete(cp, count * 128, "C-k");
+	      cursor_delete(cp, count * 128);
 	      break;
 	    case snd_K_L: case snd_K_l: 
 	      cp->cursor_on = true; 
-	      handle_cursor(cp, CURSOR_IN_MIDDLE);
+	      handle_cursor_with_sync(cp, CURSOR_IN_MIDDLE);
 	      break;
 	    case snd_K_M: case snd_K_m:
-	      if (count > 0) 
-		{
-		  cp->cursor_on = true;
-		  set_show_marks(true);
-		  mk = add_mark(CURSOR(cp), NULL, cp);
-		  display_channel_marks(cp);
-		}
-	      else delete_mark_samp(CURSOR(cp), cp);
-	      if ((keysym == snd_K_M) && 
-		  ((cp->sound)->sync != 0))
-		{
-		  sync_num = mark_sync_max() + 1; 
-		  if (mk) set_mark_sync(mk, sync_num);
-		  si = snd_sync((cp->sound)->sync);
-		  for (i = 0; i < si->chans; i++) 
-		    if (cp != si->cps[i])
-		      {
-			if (count > 0)
-			  {
-			    mk = add_mark(CURSOR(cp), NULL, si->cps[i]);
-			    if (mk)
-			      {
-				set_mark_sync(mk, sync_num);
-				display_channel_marks(si->cps[i]);
-			      }
-			  }
-			else delete_mark_samp(CURSOR(cp), si->cps[i]);
-		      }
-		  si = free_sync_info(si);
-		}
+	      {
+		mark *mk = NULL;
+		if (count > 0) 
+		  {
+		    cp->cursor_on = true;
+		    set_show_marks(true);
+		    mk = add_mark(CURSOR(cp), NULL, cp);
+		    display_channel_marks(cp);
+		  }
+		else delete_mark_samp(CURSOR(cp), cp);
+		if ((keysym == snd_K_M) && 
+		    ((cp->sound)->sync != 0))
+		  {
+		    sync_info *si;
+		    int sync_num;
+		    sync_num = mark_sync_max() + 1; 
+		    if (mk) set_mark_sync(mk, sync_num);
+		    si = snd_sync((cp->sound)->sync);
+		    for (i = 0; i < si->chans; i++) 
+		      if (cp != si->cps[i])
+			{
+			  if (count > 0)
+			    {
+			      mk = add_mark(CURSOR(cp), NULL, si->cps[i]);
+			      if (mk)
+				{
+				  set_mark_sync(mk, sync_num);
+				  display_channel_marks(si->cps[i]);
+				}
+			    }
+			  else delete_mark_samp(CURSOR(cp), si->cps[i]);
+			}
+		    si = free_sync_info(si);
+		  }
+	      }
 	      break;
 	    case snd_K_N: case snd_K_n: 
 	      cp->cursor_on = true; 
@@ -1318,14 +1349,14 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_O: case snd_K_o: 
 	      cp->cursor_on = true; 
-	      cursor_insert(cp, CURSOR(cp), count, "C-o"); 
+	      cursor_insert(cp, CURSOR(cp), count); 
 	      break;
 	    case snd_K_P: case snd_K_p: 
 	      cp->cursor_on = true; 
 	      cursor_move(cp, -count * 128); 
 	      break;
 	    case snd_K_Q: case snd_K_q: 
-	      play_channel(cp, CURSOR(cp), NO_END_SPECIFIED, IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "C-q", 0);
+	      play_channel(cp, CURSOR(cp), NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 	      break;
 #if HAVE_EXTENSION_LANGUAGE
 	    case snd_K_R: case snd_K_r: 
@@ -1357,7 +1388,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      cursor_moveto(cp, (off_t)((ap->losamp + ap->hisamp) / 2));
 	      break;
 	    case snd_K_W: case snd_K_w: 
-	      delete_selection("C-w", UPDATE_DISPLAY); 
+	      delete_selection(UPDATE_DISPLAY); 
 	      break;
 	    case snd_K_X: case snd_K_x: 
 	      extended_mode = true; 
@@ -1369,7 +1400,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 		}
 	      break;
 	    case snd_K_Y: case snd_K_y: 
-	      paste_region(region_list_position_to_id(0), cp, "C-y");
+	      paste_region(region_list_position_to_id(0), cp);
 	      break;
 	    case snd_K_Z: case snd_K_z: 
 	      cp->cursor_on = true; 
@@ -1474,7 +1505,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_D: case snd_K_d: 
 	      prompt(sp, _("eps file:"), NULL); 
-	      sp->printing = (ext_count != 0);
+	      sp->printing = ((ext_count != 0) ? PRINTING : NOT_PRINTING);
 	      searching = true; 
 	      break;
 	    case snd_K_E: case snd_K_e: 
@@ -1549,7 +1580,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 	      break;
 	    case snd_K_Z: case snd_K_z: 
 	      cp->cursor_on = true; 
-	      cos_smooth(cp, CURSOR(cp), ext_count, OVER_SOUND, "C-x C-z"); 
+	      cos_smooth(cp, CURSOR(cp), ext_count, OVER_SOUND); 
 	      break;
 	    case snd_K_Right: 
 	      sx_incremented(cp, state_amount(state)); 
@@ -1780,7 +1811,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 		  handle_cursor(cp, CURSOR_ON_RIGHT); 
 		  break;
 		case snd_K_I: case snd_K_i: 
-		  insert_selection_or_region((!got_ext_count) ? 0 : ext_count, cp, "C-x i");
+		  insert_selection_or_region((!got_ext_count) ? 0 : ext_count, cp);
 		  break;
 		case snd_K_J: case snd_K_j: 
 		  prompt(sp, _("mark:"), NULL); 
@@ -1795,7 +1826,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 		  if (selection_is_active_in_channel(cp))
 		    cursor_moveto(cp, (off_t)(selection_beg(cp) + 0.5 * selection_len()));
 		  else no_selection_error(sp); 
-		  handle_cursor(cp, CURSOR_IN_MIDDLE);
+		  handle_cursor_with_sync(cp, CURSOR_IN_MIDDLE);
 		  break;
 		case snd_K_O: case snd_K_o: 
 		  if (ext_count > 0) 
@@ -1804,11 +1835,11 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 		  break;
 		case snd_K_P: case snd_K_p: 
 		  if (!got_ext_count)
-		    play_selection(IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "C-x p", 0);
+		    play_selection(IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 		  else play_region(ext_count, IN_BACKGROUND);
 		  break;
 		case snd_K_Q: case snd_K_q: 
-		  add_selection_or_region((!got_ext_count) ? 0 : ext_count, cp, "C-x q"); 
+		  add_selection_or_region((!got_ext_count) ? 0 : ext_count, cp); 
 		  break;
 		case snd_K_R: case snd_K_r: 
 		  redo_edit_with_sync(cp, (!got_ext_count) ? 1 : ext_count); 
@@ -1829,7 +1860,7 @@ void keyboard_command(chan_info *cp, int keysym, int unmasked_state)
 		  break;
 		case snd_K_Z: case snd_K_z: 
 		  if (selection_is_active_in_channel(cp))
-		    cos_smooth(cp, CURSOR(cp), (!got_ext_count) ? 1 : ext_count, OVER_SELECTION, "C-x z"); 
+		    cos_smooth(cp, CURSOR(cp), (!got_ext_count) ? 1 : ext_count, OVER_SELECTION); 
 		  else no_selection_error(sp); 
 		  break;
 		case snd_K_Right:   
@@ -1916,8 +1947,6 @@ static XEN g_bind_key_1(XEN key, XEN state, XEN code, XEN cx_extended, XEN origi
 {
   int args, k, s;
   bool e;
-  char *errstr;
-  XEN errmsg;
   XEN_ASSERT_TYPE(XEN_INTEGER_P(key), key, XEN_ARG_1, caller, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(state), state, XEN_ARG_2, caller, "an integer");
   XEN_ASSERT_TYPE((XEN_FALSE_P(code) || XEN_PROCEDURE_P(code)), code, XEN_ARG_3, caller, "#f or a procedure");
@@ -1941,6 +1970,8 @@ static XEN g_bind_key_1(XEN key, XEN state, XEN code, XEN cx_extended, XEN origi
       args = XEN_REQUIRED_ARGS(code);
       if (args > 1)
 	{
+	  XEN errmsg;
+	  char *errstr;
 	  errstr = mus_format(_(S_bind_key " function arg should take either zero or one args, not %d"), args);
 	  errmsg = C_TO_XEN_STRING(errstr);
 	  FREE(errstr);
@@ -2015,8 +2046,6 @@ then when the user eventually responds, invoke the function callback, if any, wi
 returned as a string; otherwise it is evaluated first as Scheme code"
 
   snd_info *sp;
-  char *errstr;
-  XEN errmsg;
   XEN_ASSERT_TYPE(XEN_STRING_P(msg), msg, XEN_ARG_1, S_prompt_in_minibuffer, "a string");
   XEN_ASSERT_TYPE((XEN_NOT_BOUND_P(callback)) || (XEN_BOOLEAN_P(callback)) || XEN_PROCEDURE_P(callback), 
 		  callback, XEN_ARG_2, S_prompt_in_minibuffer, "#f or a procedure");
@@ -2031,6 +2060,8 @@ returned as a string; otherwise it is evaluated first as Scheme code"
   if (XEN_BOUND_P(raw)) sp->raw_prompt = XEN_TO_C_BOOLEAN(raw); else sp->raw_prompt = false;
   if (XEN_PROCEDURE_P(callback))
     {
+      char *errstr;
+      XEN errmsg;
       errstr = procedure_ok(callback, 1, S_prompt_in_minibuffer, "callback", 2);
       if (errstr)
 	{
@@ -2107,10 +2138,10 @@ XEN_NARGIFY_4(g_snd_simulate_keystroke_w, g_snd_simulate_keystroke)
 
 void g_init_kbd(void)
 {
-  #define H_cursor_in_view "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is in the view"
-  #define H_cursor_on_left "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is at the left edge"
-  #define H_cursor_on_right "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is at the right edge"
-  #define H_cursor_in_middle "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is in the middle"
+  #define H_cursor_in_view     "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is in the view"
+  #define H_cursor_on_left     "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is at the left edge"
+  #define H_cursor_on_right    "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is at the right edge"
+  #define H_cursor_in_middle   "The value for a " S_bind_key " function that causes it to shift the window so that the cursor is in the middle"
   #define H_keyboard_no_action "The value for a " S_bind_key " function that causes it do nothing upon return"
 
   XEN_DEFINE_CONSTANT(S_cursor_in_view,          CURSOR_IN_VIEW,                      H_cursor_in_view);
