@@ -1,21 +1,13 @@
 #include "snd.h"
 #include "clm2xen.h"
 
-#if HAVE_RUBY
-  #define TO_VAR_NAME(Str) xen_scheme_constant_to_ruby(Str)
-  #define TO_PROC_NAME(Str) xen_scheme_procedure_to_ruby(Str)
-#else
-  #define TO_VAR_NAME(Str) Str
-  #define TO_PROC_NAME(Str) Str
-#endif
-
 static void remove_temp_files(chan_info *cp)
 {
   free_sound_list(cp);
   free_mix_list(cp);
 }
 
-#ifdef DEBUG_MEMORY
+#if DEBUGGING
   void mem_report(void);
 #endif
 
@@ -31,11 +23,11 @@ int snd_exit_cleanly(bool force_exit)
 		      S_exit_hook);
   if ((XEN_TRUE_P(res)) && (!force_exit)) return(0);
   cleanup_dac();
-  for_each_chan(remove_temp_files);
+  for_each_normal_chan(remove_temp_files);
   cleanup_region_temp_files();
   cleanup_recording();
   forget_temps();
-#ifdef DEBUG_MEMORY
+#if DEBUGGING
   mem_report();
 #endif
   return(1);
@@ -57,6 +49,22 @@ void sound_not_current(snd_info *sp, void *ignore)
 
 /* ---------------- save sound state (options, or entire state) ---------------- */
 
+#if HAVE_GUILE
+static void save_loaded_files_list(FILE *fd)
+{
+  /* make sure all previously loaded code is available */
+  char *files;
+  files = XEN_AS_STRING(XEN_LIST_REVERSE(XEN_EVAL_C_STRING("snd-loaded-files"))); 
+  if (files)
+    {
+      fprintf(fd, ";;; reload any missing files\n");
+      fprintf(fd, "(for-each\n  (lambda (file)\n");
+      fprintf(fd, "    (if (and (not (member file snd-loaded-files))\n             (file-exists? file))\n        (load file)))\n");
+      fprintf(fd, "  '%s)\n\n", files);
+    }
+}
+#endif
+
 static bool fneq(Float a, Float b)
 {
   /* floating point != replacement */
@@ -293,7 +301,6 @@ static void save_snd_state_options (FILE *fd)
   if (zero_pad(ss) != DEFAULT_ZERO_PAD) pss_sd(fd, S_zero_pad, zero_pad(ss));
   if (ask_before_overwrite(ss) != DEFAULT_ASK_BEFORE_OVERWRITE) pss_ss(fd, S_ask_before_overwrite, b2s(ask_before_overwrite(ss)));
   if (dac_combines_channels(ss) != DEFAULT_DAC_COMBINES_CHANNELS) pss_ss(fd, S_dac_combines_channels, b2s(dac_combines_channels(ss)));
-  if (emacs_style_save_as(ss) != DEFAULT_EMACS_STYLE_SAVE_AS) pss_ss(fd, S_emacs_style_save_as, b2s(emacs_style_save_as(ss)));
   if (wavo_hop(ss) != DEFAULT_WAVO_HOP) pss_sd(fd, S_wavo_hop, wavo_hop(ss));
   if (wavo_trace(ss) != DEFAULT_WAVO_TRACE) pss_sd(fd, S_wavo_trace, wavo_trace(ss));
   if (spectro_hop(ss) != DEFAULT_SPECTRO_HOP) pss_sd(fd, S_spectro_hop, spectro_hop(ss));
@@ -316,7 +323,9 @@ static void save_snd_state_options (FILE *fd)
   if (show_backtrace(ss) != DEFAULT_SHOW_BACKTRACE) pss_ss(fd, S_show_backtrace, b2s(show_backtrace(ss)));
   if (show_transform_peaks(ss) != DEFAULT_SHOW_TRANSFORM_PEAKS) pss_ss(fd, S_show_transform_peaks, b2s(show_transform_peaks(ss)));
   if (show_y_zero(ss) != DEFAULT_SHOW_Y_ZERO) pss_ss(fd, S_show_y_zero, b2s(show_y_zero(ss)));
-  if (show_grid(ss) != DEFAULT_SHOW_GRID) pss_ss(fd, S_show_grid, b2s(show_grid(ss)));
+  if (show_grid(ss) != DEFAULT_SHOW_GRID) pss_ss(fd, S_show_grid, b2s((bool)show_grid(ss)));
+  if (fneq(grid_density(ss), DEFAULT_GRID_DENSITY)) pss_sf(fd, S_grid_density, grid_density(ss));
+  if (show_sonogram_cursor(ss) != DEFAULT_SHOW_SONOGRAM_CURSOR) pss_ss(fd, S_show_sonogram_cursor, b2s(show_sonogram_cursor(ss)));
   if (show_axes(ss) != DEFAULT_SHOW_AXES) pss_ss(fd, S_show_axes, show_axes2string(show_axes(ss)));
   if (show_marks(ss) != DEFAULT_SHOW_MARKS) pss_ss(fd, S_show_marks, b2s(show_marks(ss)));
   if (data_clipped(ss) != DEFAULT_DATA_CLIPPED) pss_ss(fd, S_data_clipped, b2s(data_clipped(ss)));
@@ -328,6 +337,8 @@ static void save_snd_state_options (FILE *fd)
   if (mix_waveform_height(ss) != DEFAULT_MIX_WAVEFORM_HEIGHT) pss_sd(fd, S_mix_waveform_height, mix_waveform_height(ss));
   if (mix_tag_height(ss) != DEFAULT_MIX_TAG_HEIGHT) pss_sd(fd, S_mix_tag_height, mix_tag_height(ss));
   if (mix_tag_width(ss) != DEFAULT_MIX_TAG_WIDTH) pss_sd(fd, S_mix_tag_width, mix_tag_width(ss));
+  if (mark_tag_height(ss) != DEFAULT_MARK_TAG_HEIGHT) pss_sd(fd, S_mark_tag_height, mark_tag_height(ss));
+  if (mark_tag_width(ss) != DEFAULT_MARK_TAG_WIDTH) pss_sd(fd, S_mark_tag_width, mark_tag_width(ss));
   if (enved_wave_p(ss) != DEFAULT_ENVED_WAVE_P) pss_ss(fd, S_enved_wave_p, b2s(enved_wave_p(ss)));
   if (enved_in_dB(ss) != DEFAULT_ENVED_IN_DB) pss_ss(fd, S_enved_in_dB, b2s(enved_in_dB(ss)));
   if (enved_clip_p(ss) != DEFAULT_ENVED_CLIP_P) pss_ss(fd, S_enved_clip_p, b2s(enved_clip_p(ss)));
@@ -352,6 +363,7 @@ static void save_snd_state_options (FILE *fd)
 
   if (fneq(fft_window_beta(ss), DEFAULT_FFT_WINDOW_BETA)) pss_sf(fd, S_fft_window_beta, fft_window_beta(ss));
   if (fneq(ss->min_dB, DEFAULT_MIN_DB)) pss_sf(fd, S_min_dB, ss->min_dB);
+  if (fneq(ss->Log_Freq_Start, DEFAULT_LOG_FREQ_START)) pss_sf(fd, S_log_freq_start, ss->Log_Freq_Start);
   if (fneq(color_cutoff(ss), DEFAULT_COLOR_CUTOFF)) pss_sf(fd, S_color_cutoff, color_cutoff(ss));
   if (fneq(color_scale(ss), DEFAULT_COLOR_SCALE)) pss_sf(fd, S_color_scale, color_scale(ss));
   if (fneq(spectro_x_scale(ss), DEFAULT_SPECTRO_X_SCALE)) pss_sf(fd, S_spectro_x_scale, spectro_x_scale(ss));
@@ -544,12 +556,25 @@ static void save_property_list(FILE *fd, XEN property_list, int chan)
 }
 #endif
 
+static void check_selection(FILE *fd, chan_info *cp)
+{
+  if (selection_is_active_in_channel(cp))
+    {
+      off_t beg, end;
+      beg = selection_beg(cp);
+      end = selection_end(cp);
+      pcp_ss(fd, S_selection_member, b2s(true), cp->chan);
+      pcp_sod(fd, S_selection_position, beg, cp->chan);
+      pcp_sod(fd, S_selection_frames, end - beg + 1, cp->chan);     
+    }
+}
+
 static int find_sound_nth(snd_info *nsp)
 {
-  snd_info *sp;
   int i, which = 0;
   for (i = 0; i < nsp->index; i++)
     {
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	if ((strcmp(nsp->short_filename, sp->short_filename) == 0) || 
@@ -634,7 +659,7 @@ static void save_sound_state (snd_info *sp, void *ptr)
   if (sp->speed_control_style != DEFAULT_SPEED_CONTROL_STYLE) psp_ss(fd, S_speed_control_style, speed_control_style_name(sp->speed_control_style));
   if (fneq(sp->speed_control, DEFAULT_SPEED_CONTROL)) 
     {
-#if HAVE_SCM_MAKE_RATIO
+#if (HAVE_SCM_MAKE_RATIO || HAVE_SCM_C_MAKE_RECTANGULAR)
       if (sp->speed_control == SPEED_CONTROL_AS_RATIO)
 	{
 	  /* this is only Guile, so we don't need to handle the Ruby syntax */
@@ -703,7 +728,8 @@ static void save_sound_state (snd_info *sp, void *ptr)
       if (cp->cursor_style != DEFAULT_CURSOR_STYLE) pcp_ss(fd, S_cursor_style, cursor_style_name(cp->cursor_style), chan);
       if (cp->show_marks != show_marks(ss)) pcp_ss(fd, S_show_marks, b2s(cp->show_marks), chan);
       if (cp->show_y_zero != show_y_zero(ss)) pcp_ss(fd, S_show_y_zero, b2s(cp->show_y_zero), chan);
-      if (cp->show_grid != show_grid(ss)) pcp_ss(fd, S_show_grid, b2s(cp->show_grid), chan);
+      if (cp->show_grid != show_grid(ss)) pcp_ss(fd, S_show_grid, b2s((bool)(cp->show_grid)), chan);
+      if (cp->show_sonogram_cursor != show_sonogram_cursor(ss)) pcp_ss(fd, S_show_sonogram_cursor, b2s(cp->show_sonogram_cursor), chan);
       if (cp->wavo_hop != wavo_hop(ss)) pcp_sd(fd, S_wavo_hop, cp->wavo_hop, chan);
       if (cp->wavo_trace != wavo_trace(ss)) pcp_sd(fd, S_wavo_trace, cp->wavo_trace, chan);
       if (cp->max_transform_peaks != max_transform_peaks(ss)) pcp_sd(fd, S_max_transform_peaks, cp->max_transform_peaks, chan);
@@ -736,6 +762,7 @@ static void save_sound_state (snd_info *sp, void *ptr)
       if (cp->transform_graph_style != graph_style(ss)) pcp_ss(fd, S_transform_graph_style, graph_style_name(cp->transform_graph_style), chan);
       if (cp->show_mix_waveforms != show_mix_waveforms(ss)) pcp_ss(fd, S_show_mix_waveforms, b2s(cp->show_mix_waveforms), chan);
       if (cp->dot_size != dot_size(ss)) pcp_sd(fd, S_dot_size, cp->dot_size, chan);
+      if (fneq(cp->grid_density, grid_density(ss))) pcp_sf(fd, S_grid_density, cp->grid_density, chan);
       if (cp->x_axis_style != x_axis_style(ss)) pcp_ss(fd, S_x_axis_style, x_axis_style_name(cp->x_axis_style), chan);
       if (cp->beats_per_minute != beats_per_minute(ss)) pcp_sf(fd, S_beats_per_minute, cp->beats_per_minute, chan);
       if (cp->show_axes != show_axes(ss)) pcp_ss(fd, S_show_axes, show_axes2string(cp->show_axes), chan);
@@ -763,6 +790,17 @@ static void save_sound_state (snd_info *sp, void *ptr)
 #endif
 	}
       edit_history_to_file(fd, cp);
+      check_selection(fd, cp);
+      if (selected_channel() == cp)
+	{
+#if HAVE_GUILE
+	  fprintf(fd, "%s(set! _saved_snd_selected_sound_ sfile)\n", white_space);
+	  fprintf(fd, "%s(set! _saved_snd_selected_channel_ %d)\n", white_space, cp->chan);
+#else
+	  fprintf(fd, "%ssaved_snd_selected_sound = sfile\n", white_space);
+	  fprintf(fd, "%ssaved_snd_selected_channel = %d\n", white_space, cp->chan);
+#endif
+	}
     }
   close_save_sound_block(fd);
 }
@@ -771,13 +809,6 @@ static XEN after_save_state_hook;
 
 static char *save_state_or_error (char *save_state_name)
 {
-#if HAVE_RUBY
-  #define BPAREN ""
-  #define EPAREN ""
-#else
-  #define BPAREN "("
-  #define EPAREN ")"
-#endif
   FILE *save_fd;
   char *locale = NULL;
   save_fd = open_restart_file(save_state_name, false);
@@ -787,16 +818,38 @@ static char *save_state_or_error (char *save_state_name)
 		      strerror(errno)));
   else
     {
+#if HAVE_GUILE
+      /* try to make sure all previously loaded files are now loaded */
+      save_loaded_files_list(save_fd);
+#endif
 #if HAVE_SETLOCALE
       locale = copy_string(setlocale(LC_NUMERIC, "C")); /* must use decimal point in floats since Scheme assumes that format */
 #endif
       save_prevlist(save_fd);                                 /* list of previous files (View: Files option) */
-      save_snd_state_options(save_fd);                    /* options = user-settable global state variables */
+      save_snd_state_options(save_fd);                        /* options = user-settable global state variables */
       /* the global settings need to precede possible local settings */
-      for_each_sound(save_sound_state, (void *)save_fd);  /* current sound state -- will traverse chans */
+#if HAVE_GUILE
+      fprintf(save_fd, "\n(define _saved_snd_selected_sound_ #f)\n");
+      fprintf(save_fd, "(define _saved_snd_selected_channel_ #f)\n");
+#else
+      fprintf(save_fd, "\nsaved_snd_selected_sound = -1\n");
+      fprintf(save_fd, "saved_snd_selected_channel = -1\n");
+#endif
+      for_each_sound(save_sound_state, (void *)save_fd);      /* current sound state -- will traverse chans */
+#if HAVE_GUILE
+      fprintf(save_fd, "(if _saved_snd_selected_sound_\n");
+      fprintf(save_fd, "  (begin\n");
+      fprintf(save_fd, "    (%s _saved_snd_selected_sound_)\n", S_select_sound);
+      fprintf(save_fd, "    (%s _saved_snd_selected_channel_)))\n\n", S_select_channel);
+#else
+      fprintf(save_fd, "if saved_snd_selected_sound != -1\n");
+      fprintf(save_fd, "  select_sound(saved_snd_selected_sound)\n");
+      fprintf(save_fd, "  select_channel(saved_snd_selected_channel)\n");
+      fprintf(save_fd, "end\n\n");
+#endif
       save_macro_state(save_fd);                              /* current unsaved keyboard macros (snd-chn.c) */
       save_envelope_editor_state(save_fd);                    /* current envelope editor window state */
-      save_regions(save_fd);                              /* regions */
+      save_regions(save_fd);                                  /* regions */
       if (transform_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_transform_dialog));
       if (enved_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_enved_dialog));
       if (color_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_color_dialog));
@@ -804,6 +857,7 @@ static char *save_state_or_error (char *save_state_name)
       if (view_files_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_view_files_dialog));
       if (region_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_view_regions_dialog));
       if (record_dialog_is_active()) fprintf(save_fd, BPAREN "%s" EPAREN "\n", TO_PROC_NAME(S_recorder_dialog));
+
       /* the problem here (with saving hooks) is that it is not straightforward to save the function source
        *   (with the current print-set! source option, or with an earlier procedure->string function using
        *   procedure_environment etc); many types print in this case in ways that are not readable.
@@ -824,7 +878,6 @@ static char *save_state_or_error (char *save_state_name)
 	  FREE(locale);
 	}
       snd_fclose(save_fd, save_state_name);
-
       if (XEN_HOOKED(after_save_state_hook))
 	run_hook(after_save_state_hook, 
 		 XEN_LIST_1(C_TO_XEN_STRING(save_state_name)),
@@ -1015,11 +1068,11 @@ static XEN g_save_state(XEN filename)
 {
   #define H_save_state "(" S_save_state " filename): save the current Snd state in filename; (load filename) restores it)"
   char *error;
-  XEN result;
   XEN_ASSERT_TYPE(XEN_STRING_P(filename), filename, XEN_ONLY_ARG, S_save_state, "a string");
   error = save_state_or_error(XEN_TO_C_STRING(filename));
   if (error)
     {
+      XEN result;
       result = C_TO_XEN_STRING(error);
       FREE(error);
       XEN_ERROR(CANNOT_SAVE,
@@ -1058,21 +1111,10 @@ static XEN g_exit(XEN val)
   return(XEN_FALSE);
 }
 
-#if DEBUG_MEMORY
-static XEN g_mem_report(void) 
-{
-  mem_report(); 
-  return(XEN_FALSE);
-}
-#endif
-
 #ifdef XEN_ARGIFY_1
 XEN_NARGIFY_1(g_save_options_w, g_save_options)
 XEN_NARGIFY_1(g_save_state_w, g_save_state)
 XEN_ARGIFY_1(g_exit_w, g_exit)
-#if DEBUG_MEMORY
-  XEN_NARGIFY_0(g_mem_report_w, g_mem_report)
-#endif
 XEN_NARGIFY_0(g_script_arg_w, g_script_arg)
 XEN_NARGIFY_1(g_set_script_arg_w, g_set_script_arg)
 XEN_NARGIFY_0(g_script_args_w, g_script_args)
@@ -1080,9 +1122,6 @@ XEN_NARGIFY_0(g_script_args_w, g_script_args)
 #define g_save_options_w g_save_options
 #define g_save_state_w g_save_state
 #define g_exit_w g_exit
-#if DEBUG_MEMORY
-  #define g_mem_report_w g_mem_report
-#endif
 #define g_script_arg_w g_script_arg
 #define g_set_script_arg_w g_set_script_arg
 #define g_script_args_w g_script_args
@@ -1092,11 +1131,10 @@ void g_init_main(void)
 {
   XEN_DEFINE_PROCEDURE(S_save_options, g_save_options_w, 1, 0, 0, H_save_options);
   XEN_DEFINE_PROCEDURE(S_save_state,   g_save_state_w,   1, 0, 0, H_save_state);
-  XEN_DEFINE_PROCEDURE(S_exit,         g_exit_w,         0, 1, 0, H_exit);
-
-#if DEBUG_MEMORY
-  XEN_DEFINE_PROCEDURE("mem-report",   g_mem_report_w, 0, 0, 0, "(mem-report) writes memory usage stats to memlog");
+#if HAVE_GUILE
+  XEN_EVAL_C_STRING("(define %exit exit)");
 #endif
+  XEN_DEFINE_PROCEDURE(S_exit,         g_exit_w,         0, 1, 0, H_exit);
 
   #define H_start_hook S_start_hook " (filename): called upon start-up. If it returns #t, snd exits immediately."
   XEN_DEFINE_HOOK(start_hook, S_start_hook, 1, H_start_hook);                   /* arg = argv filename if any */
