@@ -54,6 +54,12 @@ void set_spectro_cutoff(Float val) {}
 bool color_dialog_is_active(void) {return(false);}
 bool orientation_dialog_is_active(void) {return(false);}
 void reflect_spectro(void) {}
+void reflect_peaks_in_transform_dialog(void) {}
+void reflect_log_freq_start_in_transform_dialog(void) {}
+void reflect_min_db_in_transform_dialog(void) {}
+void widget_int_to_text(widget_t w, int val) {}
+void widget_float_to_text(widget_t w, Float val) {}
+void widget_off_t_to_text(widget_t w, off_t val) {}
 void listener_append_and_prompt(char *msg) {fprintf(stderr, "%s", msg);}
 void goto_listener(void) {}
 void save_listener_text(FILE *fp) {}
@@ -205,14 +211,12 @@ void filter_env_changed(snd_info *sp, env *e) {}
 void set_play_button(snd_info *sp, bool val) {}
 void play_button_pause(bool pausing) {}
 void syncb(snd_info *sp, int on) {sp->sync = on;}
-void lock_apply(snd_info *sp) {}
-void unlock_apply(snd_info *sp) {}
-void set_apply_button(snd_info *sp, bool val) {}
 void snd_file_lock_icon(snd_info *sp, bool on) {}
 void snd_file_bomb_icon(snd_info *sp, bool on) {}
 void x_bomb(snd_info *sp, bool on) {}
 void set_sound_pane_file_label(snd_info *sp, char *str) {}
 void reflect_amp_env_completion(snd_info *sp) {}
+void reflect_sound_selection(snd_info *sp) {}
 void sound_show_ctrls(snd_info *sp) {}
 void sound_hide_ctrls(snd_info *sp) {}
 bool control_panel_open(snd_info *sp) {return(false);}
@@ -238,8 +242,8 @@ widget_t make_edit_save_as_dialog(bool managed) {return(0);}
 widget_t make_file_save_as_dialog(bool managed) {return(0);}
 widget_t make_file_print_dialog(bool managed) {return(0);}
 axis_info *enved_make_axis(const char *name, axis_context *ax, int ex0, int ey0, int width, int height, 
-			   Float xmin, Float xmax, Float ymin, Float ymax, bool printing) {return(NULL);}
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing) {}
+			   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing) {return(NULL);}
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing) {}
 void set_enved_redo_sensitive(bool val) {}
 void set_enved_revert_sensitive(bool val) {}
 void set_enved_undo_sensitive(bool val) {}
@@ -274,10 +278,10 @@ void clear_listener(void) {}
 int menu_widget(int which_menu) {return(0);}
 void get_current_color(int colormap, int j, unsigned short *r, unsigned short *g, unsigned short *b) {}
 
-#define NUM_TRANSFORM_TYPES 7
+#define NUM_TRANSFORM_TYPES 6
 static char *TRANSFORM_TYPE_CONSTANTS[NUM_TRANSFORM_TYPES] =
   {S_fourier_transform, S_wavelet_transform, S_walsh_transform, 
-   S_autocorrelation, S_cepstrum, S_hadamard_transform, S_haar_transform};
+   S_autocorrelation, S_cepstrum, S_haar_transform};
 char *transform_type_name(int choice) {return(TRANSFORM_TYPE_CONSTANTS[choice]);}
 
 int add_transform_to_list(char *name) {return(0);}
@@ -318,6 +322,8 @@ void reflect_recorder_mixer_gain(int ind, Float val) {}
 void reflect_recorder_out_amp(int ind, Float val) {}
 void reflect_recorder_in_amp(int in, int out, Float val) {}
 
+void auto_update_restart(void) {}
+
 snd_info *add_sound_window (char *filename, bool read_only)
 {
   snd_info *sp;
@@ -422,6 +428,7 @@ void snd_doit(int argc, char **argv)
                (define " S_just_sounds " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
                (define " S_copy_context " 0)\
                (define " S_cursor_context " 3)\
+               (define " S_mark_context " 4)\
                (define " S_selection_context " 2)\
                (define " S_time_graph " 0)\
                (define " S_transform_graph " 1)\
@@ -458,6 +465,8 @@ void snd_doit(int argc, char **argv)
                (define " S_text_focus_color " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
                (define " S_sash_color " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
                (define " S_graph_cursor " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
+               (define " S_colormap " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
+               (define " S_colormap_size " (make-procedure-with-setter (lambda () #f) (lambda (val) val)))\
                (define " S_mix_color " (make-procedure-with-setter (lambda args #f) (lambda args #f)))");
 #endif
 #if HAVE_RUBY
@@ -492,6 +501,7 @@ void snd_doit(int argc, char **argv)
 
   XEN_EVAL_C_STRING("Copy_context = 0");
   XEN_EVAL_C_STRING("Cursor_context = 3");
+  XEN_EVAL_C_STRING("Mark_context = 4");
   XEN_EVAL_C_STRING("Selection_context = 2");
 #endif
   set_peaks_font(FALLBACK_FONT);
@@ -501,6 +511,7 @@ void snd_doit(int argc, char **argv)
   set_axis_numbers_font(FALLBACK_FONT);
   set_listener_font(FALLBACK_FONT);
   set_html_dir(copy_string(DEFAULT_HTML_DIR));
+  ss->startup_title = copy_string("snd");
 
   for (i = 1; i < argc; i++)
     {
