@@ -154,6 +154,7 @@ int menu_widget(int which_menu);
 
 color_t get_in_between_color(color_t fg, color_t bg);
 void snd_doit(int argc, char **argv);
+void auto_update_restart(void);
 #ifdef SND_AS_WIDGET
   void snd_as_widget(int argc, char **argv, int app, int parent, int *caller_args, int caller_argn);
 #endif
@@ -180,6 +181,12 @@ void set_transform_normalization(fft_normalize_t val);
 void set_show_selection_transform(bool show);
 void set_transform_graph_type(graph_type_t val);
 
+void reflect_peaks_in_transform_dialog(void);
+void reflect_log_freq_start_in_transform_dialog(void);
+void reflect_min_db_in_transform_dialog(void);
+void widget_int_to_text(widget_t w, int val);
+void widget_float_to_text(widget_t w, Float val);
+void widget_off_t_to_text(widget_t w, off_t val);
 
 
 /* -------- snd-xregion.c -------- */
@@ -287,9 +294,6 @@ void filter_env_changed(snd_info *sp, env *e);
 void set_play_button(snd_info *sp, bool val);
 void play_button_pause(bool pausing);
 void syncb(snd_info *sp, int on);
-void lock_apply(snd_info *sp);
-void unlock_apply(snd_info *sp);
-void set_apply_button(snd_info *sp, bool val);
 void snd_file_lock_icon(snd_info *sp, bool on);
 void snd_file_bomb_icon(snd_info *sp, bool on);
 void x_bomb(snd_info *sp, bool on);
@@ -303,6 +307,7 @@ void start_progress_report(snd_info *sp, enved_progress_t from_enved);
 void finish_progress_report(snd_info *sp, enved_progress_t from_enved);
 void progress_report(snd_info *sp, const char *funcname, int curchan, int chans, Float pct, enved_progress_t from_enved);
 void reflect_amp_env_in_progress(snd_info *sp);
+void reflect_sound_selection(snd_info *sp);
 
 
 /* -------- snd-xfile.c -------- */
@@ -335,8 +340,8 @@ void reflect_just_sounds_state(void);
 /* -------- snd-xenv.c -------- */
 
 axis_info *enved_make_axis(const char *name, axis_context *ax, int ex0, int ey0, int width, int height, 
-			   Float xmin, Float xmax, Float ymin, Float ymax, bool printing);
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing);
+			   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing);
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing);
 void set_enved_redo_sensitive(bool val);
 void set_enved_revert_sensitive(bool val);
 void set_enved_undo_sensitive(bool val);
