@@ -55,14 +55,13 @@ static void deferred_region_to_temp_file(region *r);
 
 static void free_region(region *r, int complete)
 {
-  snd_info *sp;
-  int i;
   /* if not complete, just clear out old data (edited region being saved) */
   if (r)
     {
       release_region_readers(r->id);
       if ((complete == COMPLETE_DELETION) && (r->editor_copy))
 	{
+	  snd_info *sp;
 	  sp = r->editor_copy; 
 	  sp->edited_region = NULL;
 	  r->editor_copy = NULL;
@@ -76,6 +75,7 @@ static void free_region(region *r, int complete)
 	  if (r->lens) FREE(r->lens);
 	  if (r->amp_envs)
 	    {
+	      int i;
 	      for (i = 0; i < r->chans; i++)
 		if (r->amp_envs[i]) 
 		  r->amp_envs[i] = free_env_info(r->amp_envs[i]);
@@ -220,14 +220,14 @@ Float region_maxamp(int n)
 static Float region_sample(int reg, int chn, off_t samp)
 {
   region *r;
-  snd_fd *sf;
-  Float val;
-  deferred_region *drp;
   r = id_to_region(reg);
   if (r)
     {
       if ((samp < r->frames) && (chn < r->chans)) 
 	{
+	  snd_fd *sf;
+	  Float val;
+	  deferred_region *drp;
 	  switch (r->use_temp_file)
 	    {
 	    case REGION_FILE:
@@ -264,14 +264,14 @@ off_t region_current_location(snd_fd *fd)
 static void region_samples(int reg, int chn, off_t beg, off_t num, Float *data)
 {
   region *r;
-  snd_fd *sf;
-  off_t i, j = 0;
-  deferred_region *drp;
   r = id_to_region(reg);
   if (r)
     {
       if ((beg < r->frames) && (chn < r->chans))
 	{
+	  snd_fd *sf;
+	  off_t i, j = 0;
+	  deferred_region *drp;
 	  switch (r->use_temp_file)
 	    {
 	    case REGION_FILE:
@@ -321,10 +321,8 @@ static int check_regions(void)
 static void make_region_readable(region *r)
 {
   snd_info *regsp;
-  chan_info *cp;
   file_info *hdr;
-  snd_io *io;
-  int i, fd;
+  int i;
   if (r->use_temp_file == REGION_DEFERRED) 
     deferred_region_to_temp_file(r);
   if (r->rsp) return;
@@ -341,7 +339,9 @@ static void make_region_readable(region *r)
   hdr->comment = NULL;
   for (i = 0; i < r->chans; i++)
     {
+      chan_info *cp;
       cp = make_chan_info(NULL, i, regsp);
+      cp->editable = false;
       regsp->chans[i] = cp;
       add_channel_data_1(cp, r->srate, r->frames, WITHOUT_GRAPH);
       cp->edits[0] = initial_ed_list(0, r->frames - 1);
@@ -352,6 +352,8 @@ static void make_region_readable(region *r)
       hdr = make_file_info(r->filename);
       if (hdr)
 	{
+	  snd_io *io;
+	  int fd;
 	  fd = snd_open_read(r->filename);
 	  mus_file_open_descriptors(fd,
 				    r->filename,
@@ -360,7 +362,7 @@ static void make_region_readable(region *r)
 				    hdr->data_location,
 				    hdr->chans,
 				    hdr->type);
-	  io = make_file_state(fd, hdr, i, FILE_BUFFER_SIZE);
+	  io = make_file_state(fd, hdr, i, 0, FILE_BUFFER_SIZE);
 	  cp->sounds[0] = make_snd_data_file(r->filename, io, hdr, DONT_DELETE_ME, cp->edit_ctr, i); /* don't auto-delete! */
 	}
       else
@@ -376,16 +378,16 @@ static void make_region_readable(region *r)
 file_info *fixup_region_data(chan_info *cp, int chan, int pos)
 {
   /* for region browser labels */
-  region *r;
-  snd_info *nsp;
-  chan_info *ncp;
   if ((pos >= 0) && 
       (pos < regions_size) &&
       (regions[pos]))
     {
+      region *r;
       r = regions[pos];
       if (chan < r->chans)
 	{
+	  snd_info *nsp;
+	  chan_info *ncp;
 	  make_region_readable(r);
 	  nsp = r->rsp;
 	  ncp = nsp->chans[chan];
@@ -417,15 +419,16 @@ file_info *fixup_region_data(chan_info *cp, int chan, int pos)
 void for_each_region_chan(void (*func)(chan_info *, void *), void *userptr)
 {
   /* used only in snd-io.c to remove dangling temp files (probably can't actually happen) */
-  int i, chn;
-  region *r;
-  chan_info *cp;
+  int i;
   for (i = 0; i < regions_size; i++)
     {
+      int chn;
+      region *r;
       r = regions[i];
       if ((r) && (r->rsp) && (r->use_temp_file == REGION_FILE))
 	for (chn = 0; chn < r->chans; chn++)
 	  {
+	    chan_info *cp;
 	    cp = r->rsp->chans[chn];
 	    (*func)(cp, userptr);
 	  }
@@ -436,8 +439,6 @@ region_state *region_report(void)
 {
   region_state *rs;
   int i, len;
-  char *reg_buf;
-  region *r;
   rs = (region_state *)CALLOC(1, sizeof(region_state));
   len = regions_size;
   for (i = 0; i < regions_size; i++) 
@@ -451,6 +452,8 @@ region_state *region_report(void)
   rs->name = (char **)CALLOC(len, sizeof(char *));
   for (i = 0; i < len; i++)
     {
+      region *r;
+      char *reg_buf;
       r = regions[i];
       reg_buf = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
       mus_snprintf(reg_buf, LABEL_BUFFER_SIZE, "%d: %s (%s:%s)", r->id, r->name, r->start, r->end);
@@ -461,9 +464,9 @@ region_state *region_report(void)
 
 void free_region_state (region_state *r)
 {
-  int i;
   if (r)
     {
+      int i;
       for (i = 0; i < r->len; i++)
 	if (r->name[i]) 
 	  FREE(r->name[i]);
@@ -511,16 +514,16 @@ static void add_to_region_list(region *r)
 
 static int save_region_1(char *ofile, int type, int format, int srate, int reg, char *comment)
 {
-  int ofd, ifd, chans, i, comlen, err = 0;
-  off_t oloc, iloc, ioff, frames, cursamples;
-  mus_sample_t **bufs;
   region *r;
-  comlen = snd_strlen(comment);
   r = id_to_region(reg);
   if (r)
     {
+      off_t oloc, iloc, ioff, frames, cursamples;
+      int ofd, ifd, chans, i, comlen, err = 0;
+      mus_sample_t **bufs;
       if (r->use_temp_file == REGION_DEFERRED) 
 	deferred_region_to_temp_file(r);
+      comlen = snd_strlen(comment);
       if ((snd_write_header(ofile, type, srate, r->chans, 28, r->chans * r->frames, format, comment, comlen, NULL)) == -1)
 	return(MUS_HEADER_WRITE_FAILED);
       oloc = mus_header_data_location();
@@ -600,14 +603,12 @@ int save_region(int n, char *ofile, int data_format)
 
 #define NOT_EDITABLE -2
 
-static int paste_region_1(int n, chan_info *cp, bool add, off_t beg, const char *origin, int trk)
+static int paste_region_1(int n, chan_info *cp, bool add, off_t beg, int trk)
 {
   region *r;
+  char *origin = NULL;
   int i, err = MUS_NO_ERROR, id = -1;
-  sync_info *si;
-  chan_info *ncp;
-  char *tempfile = NULL;
-  si = NULL;
+  sync_info *si = NULL;
   r = id_to_region(n);
   if ((r == NULL) || (r->frames == 0)) return(INVALID_REGION);
   if (!(editable_p(cp))) return(NOT_EDITABLE);
@@ -624,11 +625,17 @@ static int paste_region_1(int n, chan_info *cp, bool add, off_t beg, const char
 	  cp->edit_hook_checked = false;
 	  snd_error(_("can't save mix temp file (%s: %s)"), newname, strerror(errno));
 	}
-      else id = mix_file(beg, r->frames, si->chans, si->cps, newname, DELETE_ME, origin, with_mix_tags(ss), trk);
+      else 
+	{
+	  origin = mus_format("%s " OFF_TD " %d ", S_mix_region, beg, n);
+	  id = mix_file(beg, r->frames, si->chans, si->cps, newname, DELETE_ME, origin, with_mix_tags(ss), trk);
+	  FREE(origin);
+	}
       if (newname) FREE(newname);
     }
   else
     {
+      char *tempfile = NULL;
       if (r->use_temp_file == REGION_FILE)
 	{
 	  tempfile = snd_tempnam();
@@ -644,14 +651,17 @@ static int paste_region_1(int n, chan_info *cp, bool add, off_t beg, const char
 	    if (r->chans > 1) 
 	      remember_temp(tempfile, r->chans);
 	}
+      origin = mus_format("%s " OFF_TD " %d", S_insert_region, beg, n);
       for (i = 0; ((i < r->chans) && (i < si->chans)); i++)
 	{
+	  chan_info *ncp;
 	  ncp = si->cps[i];                       /* currently syncd chan that we might paste to */
 	  if (file_insert_samples(beg, r->frames, tempfile, ncp, i,
 				  (r->chans > 1) ? MULTICHANNEL_DELETION : DELETE_ME,
 				  origin, ncp->edit_ctr))
 	    update_graph(si->cps[i]);
 	}
+      FREE(origin);
       if ((r->use_temp_file == REGION_FILE) && (tempfile)) FREE(tempfile);
     }
   if (si) si = free_sync_info(si);
@@ -659,8 +669,8 @@ static int paste_region_1(int n, chan_info *cp, bool add, off_t beg, const char
   return(id);
 }
 
-void paste_region(int n, chan_info *cp, const char *origin) {paste_region_1(n, cp, false, CURSOR(cp), origin, 0);}
-void add_region(int n, chan_info *cp, const char *origin) {paste_region_1(n, cp, true, CURSOR(cp), origin, 0);}
+void paste_region(int n, chan_info *cp) {paste_region_1(n, cp, false, CURSOR(cp), 0);}
+void add_region(int n, chan_info *cp) {paste_region_1(n, cp, true, CURSOR(cp), 0);}
 
 int define_region(sync_info *si, off_t *ends)
 {
@@ -671,7 +681,6 @@ int define_region(sync_info *si, off_t *ends)
   chan_info *cp0;
   snd_info *sp0;
   region *r;
-  env_info *ep;
   deferred_region *drp;
   len = 0;
   for (i = 0; i < si->chans; i++)
@@ -714,6 +723,7 @@ int define_region(sync_info *si, off_t *ends)
       if ((r->lens[i] > AMP_ENV_CUTOFF) &&
 	  (drp->cps[i]->amp_envs))
 	{
+	  env_info *ep;
 	  ep = drp->cps[i]->amp_envs[drp->edpos[i]];
 	  if ((ep) && (ep->completed))
 	    {
@@ -813,7 +823,17 @@ static void deferred_region_to_temp_file(region *r)
     }
   else
     {
+#if DEBUGGING
+      char *regstr;
+      regstr = mus_format("region %d (%s: %s %s), from %s[%d]",
+			  r->id, r->name, r->start, r->end,
+			  (r->rsp) ? (r->rsp->filename) : "unknown sound",
+			  (r->rsp) ? (r->rsp->index) : 0);
+      hdr = make_temp_header(r->filename, r->srate, r->chans, 0, regstr);
+      FREE(regstr);
+#else
       hdr = make_temp_header(r->filename, r->srate, r->chans, 0, (char *)c__FUNCTION__);
+#endif
       ofd = open_temp_file(r->filename, r->chans, hdr);
       if (ofd == -1)
 	snd_error(_("can't write region temp file %s: %s"), r->filename, strerror(errno));
@@ -866,20 +886,23 @@ void sequester_deferred_regions(chan_info *cp, int edit_top)
 {
   region *r;
   deferred_region *drp;
-  int i, j;
+  int i;
   for (i = 0; i < regions_size; i++)
     {
       r = regions[i];
       if ((r) && (r->use_temp_file == REGION_DEFERRED))
 	{
+	  int j;
 	  drp = r->dr;
 	  for (j = 0; j < drp->chans; j++)
 	    if ((drp->cps[j] == cp) &&
 		(drp->edpos[j] > edit_top))
 	      {
-		report_in_minibuffer(cp->sound, _("sequestering region %d..."), r->id);
+		if (r->lens[j] > 1000000)
+		  report_in_minibuffer(cp->sound, _("sequestering region %d..."), r->id);
 		deferred_region_to_temp_file(r);
-		clear_minibuffer(cp->sound);
+		if (r->lens[j] > 1000000)
+		  clear_minibuffer(cp->sound);
 		break;
 	      }
 	}
@@ -890,7 +913,6 @@ snd_fd *init_region_read (off_t beg, int n, int chan, read_direction_t direction
 {
   /* conjure up a reasonable looking ed list and sound list */
   region *r;
-  deferred_region *drp;
   r = id_to_region(n);
   if ((r) && (chan < r->chans))
     {
@@ -899,6 +921,7 @@ snd_fd *init_region_read (off_t beg, int n, int chan, read_direction_t direction
 	beg = r->frames - 1;
       if (r->use_temp_file == REGION_DEFERRED)
 	{
+	  deferred_region *drp;
 	  drp = r->dr;
 	  return(init_sample_read_any(r->begs[chan] + beg, drp->cps[chan], direction, drp->edpos[chan]));
 	}
@@ -914,9 +937,9 @@ snd_fd *init_region_read (off_t beg, int n, int chan, read_direction_t direction
 void cleanup_region_temp_files(void)
 { /* called upon exit to get rid of lingering region-related temp files */
   int i;
-  region *r;
   for (i = 0; i < regions_size; i++)
     {
+      region *r;
       r = regions[i];
       if ((r) && 
 	  (r->use_temp_file == REGION_FILE) && 
@@ -938,33 +961,37 @@ int snd_regions(void)
   return(num);
 }
 
-/* (restore-region n chans len srate maxamp name start end filename) */
+/* (restore-region n chans len srate maxamp name start end filename [date-and-length]) */
 
 void save_regions(FILE *fd)
 {
   int i;
-  region *r;
-  char *newname;
   for (i = 0; i < regions_size; i++)
     {
+      region *r;
       r = regions[i];
       if (r)
 	{
+	  char *newname;
 	  char *ofile = NULL;
-#if HAVE_RUBY
-	  fprintf(fd, "%s(%d, %d, " OFF_TD ", %d, %.4f, \"%s\", \"%s\", \"%s\", ",
-	          "restore_region", i, r->chans, r->frames, r->srate, r->maxamp, r->name, r->start, r->end);
-#else
-	  fprintf(fd, "(%s %d %d " OFF_TD " %d %.4f \"%s\" \"%s\" \"%s\"",
-	          S_restore_region, i, r->chans, r->frames, r->srate, r->maxamp, r->name, r->start, r->end);
-#endif
 	  if (r->use_temp_file == REGION_DEFERRED) 
 	    deferred_region_to_temp_file(r);
 	  ofile = shorter_tempnam(save_dir(ss), "snd_save_");
 	  newname = run_save_state_hook(ofile);
 	  FREE(ofile);
 	  copy_file(r->filename, newname);
+#if HAVE_RUBY
+	  fprintf(fd, "%s(%d, %d, " OFF_TD ", %d, %.4f, \"%s\", \"%s\", \"%s\", ",
+	          "restore_region", i, r->chans, r->frames, r->srate, r->maxamp, r->name, r->start, r->end);
 	  fprintf(fd, " \"%s\")\n", newname);
+#else
+	  fprintf(fd, "(%s %d %d " OFF_TD " %d %.4f \"%s\" \"%s\" \"%s\"",
+	          S_restore_region, i, r->chans, r->frames, r->srate, r->maxamp, r->name, r->start, r->end);
+	  fprintf(fd, " \"%s\" (list %d " OFF_TD "))\n",
+		  newname,
+		  (int)mus_sound_write_date(newname),
+		  mus_sound_length(newname));
+#endif
 	  FREE(newname);
 	}
     }
@@ -977,9 +1004,6 @@ void region_edit(int pos)
    *   if 'save', save temp file and update region (browser also) (cancelling active display if any)
    *   while editing, if delete in browser, cut backpointer in editor and signal it
    */
-  char *temp_region_name;
-  snd_info *sp;
-  int err;
   region *r = NULL;
   if ((pos >= 0) && 
       (pos < regions_size) &&
@@ -991,12 +1015,15 @@ void region_edit(int pos)
 	snd_error(_("region %d already being edited"), r->id);
       else
 	{
+	  int err;
+	  char *temp_region_name;
 	  if (r->use_temp_file == REGION_DEFERRED) 
 	    deferred_region_to_temp_file(r);
 	  temp_region_name = shorter_tempnam(temp_dir(ss), "region-");
 	  err = copy_file(r->filename, temp_region_name);
 	  if (err == MUS_NO_ERROR)
 	    {
+	      snd_info *sp;
 	      sp = snd_open_file(temp_region_name, false);
 	      if (sp)
 		{
@@ -1020,9 +1047,9 @@ void region_edit(int pos)
 
 void clear_region_backpointer(snd_info *sp)
 {
-  region *r;
   if (sp->edited_region)
     {
+      region *r;
       r = (region *)(sp->edited_region);
       if (r)
 	{
@@ -1040,7 +1067,6 @@ void save_region_backpointer(snd_info *sp)
   /* region being edited, user chose 'save' */
   region *r;
   int i, err;
-  Float val;
   r = (region *)(sp->edited_region);
   /* update r's data in file, deleting old, redisplay if browser active etc */
   if (r == regions[0]) deactivate_selection();
@@ -1050,7 +1076,8 @@ void save_region_backpointer(snd_info *sp)
   r->frames = CURRENT_SAMPLES(sp->chans[0]);
   for (i = 0; i < sp->nchans; i++)
     {
-      val = get_maxamp(sp, sp->chans[i], AT_CURRENT_EDIT_POSITION);
+      Float val;
+      val = channel_maxamp(sp->chans[i], AT_CURRENT_EDIT_POSITION);
       if (val > r->maxamp) r->maxamp = val;
     }
   /* make new region temp file */
@@ -1082,7 +1109,7 @@ static XEN snd_no_such_region_error(const char *caller, XEN n)
   return(XEN_FALSE);
 }
 
-static XEN g_restore_region(XEN pos, XEN chans, XEN len, XEN srate, XEN maxamp, XEN name, XEN start, XEN end, XEN filename)
+static XEN g_restore_region(XEN pos, XEN chans, XEN len, XEN srate, XEN maxamp, XEN name, XEN start, XEN end, XEN filename, XEN date)
 {
   /* internal function used by save-state mechanism -- not intended for external use */
   region *r;
@@ -1096,13 +1123,14 @@ static XEN g_restore_region(XEN pos, XEN chans, XEN len, XEN srate, XEN maxamp,
   XEN_ASSERT_TYPE(XEN_STRING_P(start), start, XEN_ARG_7, S_restore_region, "a string");
   XEN_ASSERT_TYPE(XEN_STRING_P(end), end, XEN_ARG_8, S_restore_region, "a string");
   XEN_ASSERT_TYPE(XEN_STRING_P(filename), filename, XEN_ARG_9, S_restore_region, "a string");
+  check_saved_temp_file("region", filename, date);
   r = (region *)CALLOC(1, sizeof(region));
-  regn = XEN_TO_SMALL_C_INT(pos);
+  regn = XEN_TO_C_INT(pos);
   if (regions[regn]) free_region(regions[regn], COMPLETE_DELETION);
   regions[regn] = r;
   r->id = region_id_ctr++;
   r->maxamp = XEN_TO_C_DOUBLE(maxamp);
-  r->chans = XEN_TO_SMALL_C_INT(chans);
+  r->chans = XEN_TO_C_INT(chans);
   r->rsp = NULL;
   r->editor_copy = NULL;
   r->editor_name = NULL;
@@ -1125,7 +1153,6 @@ insert region data into snd's channel chn starting at start-samp"
 
   chan_info *cp;
   int rg;
-  char *buf;
   off_t samp;
   XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(samp_n), samp_n, XEN_ARG_1, S_insert_region, "a number");
   XEN_ASSERT_TYPE(XEN_REGION_IF_BOUND_P(reg_n), reg_n, XEN_ARG_2, S_insert_region, "a region id");
@@ -1135,9 +1162,7 @@ insert region data into snd's channel chn starting at start-samp"
   if (!(region_ok(rg)))
     return(snd_no_such_region_error(S_insert_region, reg_n));
   samp = beg_to_sample(samp_n, S_insert_region);
-  buf = mus_format("%s %d at " OFF_TD, S_insert_region, rg, samp);
-  paste_region_1(rg, cp, false, samp, buf, 0);
-  FREE(buf);
+  paste_region_1(rg, cp, false, samp, 0);
   update_graph(cp);
   return(reg_n);
 }
@@ -1287,16 +1312,17 @@ static XEN g_make_region(XEN beg, XEN end, XEN snd_n, XEN chn_n)
   #define H_make_region "(" S_make_region " (beg) (end) (snd #f) (chn #f)): make a new region between beg and end in snd, returning its id. \
 If chn is #t, all chans are included, taking the snd sync field into account if it's not 0.  If no args are passed, the current \
 selection is used."
-  chan_info *cp;
-  sync_info *si = NULL;
-  snd_info *sp;
-  off_t *ends = NULL;
-  off_t ibeg, iend;
   int id = INVALID_REGION, new_sync, old_sync, i;
+  if (max_regions(ss) <= 0) return(XEN_FALSE);
   if (XEN_NOT_BOUND_P(beg))
     id = make_region_from_selection();
   else
     {
+      chan_info *cp;
+      sync_info *si = NULL;
+      snd_info *sp;
+      off_t ibeg, iend;
+      off_t *ends = NULL;
       XEN_ASSERT_TYPE(XEN_NUMBER_P(beg), beg, XEN_ARG_1, S_make_region, "a number");
       XEN_ASSERT_TYPE(XEN_NUMBER_P(end), end, XEN_ARG_2, S_make_region, "a number");
       ibeg = beg_to_sample(beg, S_make_region);
@@ -1425,7 +1451,6 @@ mix region into snd's channel chn starting at chn-samp; return new mix id."
 
   chan_info *cp;
   off_t samp;
-  char *buf;
   int rg, id = -1, track_id = 0;
   XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(chn_samp_n), chn_samp_n, XEN_ARG_1, S_mix_region, "a number");
   XEN_ASSERT_TYPE(XEN_REGION_IF_BOUND_P(reg_n), reg_n, XEN_ARG_2, S_mix_region, "a region id");
@@ -1444,9 +1469,7 @@ mix region into snd's channel chn starting at chn-samp; return new mix id."
 	      XEN_LIST_2(C_TO_XEN_STRING(S_mix_region),
 			 tid));
   ss->catch_message = NULL;
-  buf = mus_format("%s %d at " OFF_TD, S_mix_region, rg, samp);
-  id = paste_region_1(rg, cp, true, samp, buf, track_id);
-  FREE(buf);
+  id = paste_region_1(rg, cp, true, samp, track_id);
   /* id might legitmately be invalid mix id if with_mix_tags is #f */
   return(C_TO_XEN_INT(id));
 }
@@ -1470,35 +1493,46 @@ static XEN g_region_sample(XEN samp_n, XEN reg_n, XEN chn_n)
   else return(snd_no_such_channel_error(S_region_sample, XEN_LIST_1(reg_n), chn_n));
 }
 
-static XEN g_region_samples_to_vct(XEN beg_n, XEN num, XEN reg_n, XEN chn_n, XEN v)
+static XEN g_region_to_vct(XEN beg_n, XEN num, XEN reg_n, XEN chn_n, XEN v)
 {
-  #define H_region_samples_to_vct "(" S_region_samples_to_vct " (beg 0) (samps reglen) (region 0) (chan 0) (v #f)): \
+  #define H_region_to_vct "(" S_region_to_vct " (beg 0) (samps reglen) (region 0) (chan 0) (v #f)): \
 write region's samples starting at beg for samps in channel chan to vct v; return v (or create a new one)"
 
   Float *data;
   int reg, chn;
-  off_t len, beg;
+  off_t len;
   vct *v1 = get_vct(v);
-  XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(beg_n), beg_n, XEN_ARG_1, S_region_samples_to_vct, "a number");
-  XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(num), num, XEN_ARG_2, S_region_samples_to_vct, "a number");
-  XEN_ASSERT_TYPE(XEN_REGION_IF_BOUND_P(reg_n), reg_n, XEN_ARG_3, S_region_samples_to_vct, "a region id");
-  XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(chn_n), chn_n, XEN_ARG_4, S_region_samples_to_vct, "an integer");
+  XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(beg_n), beg_n, XEN_ARG_1, S_region_to_vct, "a number");
+  XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(num), num, XEN_ARG_2, S_region_to_vct, "a number");
+  XEN_ASSERT_TYPE(XEN_REGION_IF_BOUND_P(reg_n), reg_n, XEN_ARG_3, S_region_to_vct, "a region id");
+  XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(chn_n), chn_n, XEN_ARG_4, S_region_to_vct, "an integer");
   reg = XEN_REGION_TO_C_INT(reg_n);
   if (!(region_ok(reg)))
-    return(snd_no_such_region_error(S_region_samples_to_vct, reg_n));
+    return(snd_no_such_region_error(S_region_to_vct, reg_n));
   chn = XEN_TO_C_INT_OR_ELSE(chn_n, 0);
   if ((chn < 0) || (chn >= region_chans(reg)))
-    return(snd_no_such_channel_error(S_region_samples_to_vct, XEN_LIST_1(reg_n), chn_n));
+    return(snd_no_such_channel_error(S_region_to_vct, XEN_LIST_1(reg_n), chn_n));
   len = XEN_TO_C_OFF_T_OR_ELSE(num, 0);
+  if (len < 0)
+    XEN_OUT_OF_RANGE_ERROR(S_region_to_vct, 2, num, "length ~A < 0?");
   if (len == 0) len = region_len(reg);
   if (len > 0)
     {
-      beg = beg_to_sample(beg_n, S_region_samples_to_vct);
+      off_t beg;
+      beg = beg_to_sample(beg_n, S_region_to_vct);
       if (beg >= region_len(reg)) 
 	return(XEN_FALSE);
       if (v1)
-	data = v1->data;
-      else data = (Float *)CALLOC(len, sizeof(Float));
+	{
+	  data = v1->data;
+	  if (len > v1->length) len = v1->length;
+	}
+      else 
+	{
+	  if ((beg + len) > region_len(reg))
+	    len = region_len(reg) - beg;
+	  data = (Float *)CALLOC(len, sizeof(Float));
+	}
       region_samples(reg, chn, beg, len, data);
       if (v1)
 	return(v);
@@ -1508,7 +1542,7 @@ write region's samples starting at beg for samps in channel chan to vct v; retur
 }
 
 #ifdef XEN_ARGIFY_1
-XEN_NARGIFY_9(g_restore_region_w, g_restore_region)
+XEN_ARGIFY_10(g_restore_region_w, g_restore_region)
 XEN_ARGIFY_4(g_insert_region_w, g_insert_region)
 XEN_NARGIFY_0(g_regions_w, g_regions)
 XEN_ARGIFY_2(g_region_frames_w, g_region_frames)
@@ -1522,7 +1556,7 @@ XEN_ARGIFY_3(g_play_region_w, g_play_region)
 XEN_ARGIFY_4(g_make_region_w, g_make_region)
 XEN_ARGIFY_5(g_mix_region_w, g_mix_region)
 XEN_ARGIFY_3(g_region_sample_w, g_region_sample)
-XEN_ARGIFY_5(g_region_samples_to_vct_w, g_region_samples_to_vct)
+XEN_ARGIFY_5(g_region_to_vct_w, g_region_to_vct)
 XEN_NARGIFY_1(g_region_p_w, g_region_p)
 XEN_NARGIFY_0(g_max_regions_w, g_max_regions)
 XEN_NARGIFY_1(g_set_max_regions_w, g_set_max_regions)
@@ -1541,7 +1575,7 @@ XEN_NARGIFY_1(g_set_max_regions_w, g_set_max_regions)
 #define g_make_region_w g_make_region
 #define g_mix_region_w g_mix_region
 #define g_region_sample_w g_region_sample
-#define g_region_samples_to_vct_w g_region_samples_to_vct
+#define g_region_to_vct_w g_region_to_vct
 #define g_region_p_w g_region_p
 #define g_max_regions_w g_max_regions
 #define g_set_max_regions_w g_set_max_regions
@@ -1551,7 +1585,7 @@ void g_init_regions(void)
 {
   init_region_keywords();
 
-  XEN_DEFINE_PROCEDURE(S_restore_region,     g_restore_region_w,     9, 0, 0, "internal func used in save-state, restores a region");
+  XEN_DEFINE_PROCEDURE(S_restore_region,     g_restore_region_w,     9, 1, 0, "internal func used in save-state, restores a region");
   XEN_DEFINE_PROCEDURE(S_insert_region,      g_insert_region_w,      0, 4, 0, H_insert_region);
   XEN_DEFINE_PROCEDURE(S_regions,            g_regions_w,            0, 0, 0, H_regions);
   XEN_DEFINE_PROCEDURE(S_region_frames,      g_region_frames_w,      0, 2, 0, H_region_frames);
@@ -1565,7 +1599,7 @@ void g_init_regions(void)
   XEN_DEFINE_PROCEDURE(S_make_region,        g_make_region_w,        0, 4, 0, H_make_region);
   XEN_DEFINE_PROCEDURE(S_mix_region,         g_mix_region_w,         0, 5, 0, H_mix_region);
   XEN_DEFINE_PROCEDURE(S_region_sample,      g_region_sample_w,      0, 3, 0, H_region_sample);
-  XEN_DEFINE_PROCEDURE(S_region_samples_to_vct, g_region_samples_to_vct_w, 0, 5, 0, H_region_samples_to_vct);
+  XEN_DEFINE_PROCEDURE(S_region_to_vct,      g_region_to_vct_w,      0, 5, 0, H_region_to_vct);
   XEN_DEFINE_PROCEDURE(S_region_p,           g_region_p_w,           1, 0, 0, H_region_p);
 
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_max_regions, g_max_regions_w, H_max_regions, S_setB S_max_regions, g_set_max_regions_w, 0, 0, 1, 0);
