@@ -42,12 +42,11 @@ bool selection_is_visible_in_channel (chan_info *cp)
 static off_t off_t_map_over_chans(off_t (*func)(chan_info *, off_t *), off_t *userptr)
 {
   int i, j;
-  off_t val;
-  snd_info *sp;
-  chan_info *cp;
-  val = 0;
+  off_t val = 0;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      snd_info *sp;
+      chan_info *cp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse == SOUND_NORMAL))
 	for (j = 0; j < sp->nchans; j++)
@@ -134,7 +133,7 @@ int selection_chans(void)
 {
   int count[1];
   count[0] = 0;
-  for_each_chan_1(selection_chans_1, (void *)count);
+  for_each_normal_chan_1(selection_chans_1, (void *)count);
   return(count[0]);
 }
 
@@ -159,33 +158,33 @@ Float selection_maxamp(chan_info *cp)
     {
       val = ed_selection_maxamp(cp);
       if (val >= 0.0) return(val);
-      val = local_maxamp(cp, 
-			 selection_beg(cp), 
-			 selection_end(cp) - selection_beg(cp) + 1,
-			 cp->edit_ctr);
+      val = channel_local_maxamp(cp, 
+				 selection_beg(cp), 
+				 selection_end(cp) - selection_beg(cp) + 1,
+				 cp->edit_ctr);
       set_ed_selection_maxamp(cp, val);
     }
   return(val);
 }
 
-static void cp_delete_selection(chan_info *cp, void *origin)
+static void cp_delete_selection(chan_info *cp)
 {
   ed_list *ed;
   ed = cp->edits[cp->edit_ctr];
   if ((ed) && (ed->selection_beg != NO_SELECTION))
     {
-      delete_samples(ed->selection_beg, cp_selection_len(cp, NULL), cp, (const char *)origin, cp->edit_ctr);
+      delete_samples(ed->selection_beg, cp_selection_len(cp, NULL), cp, cp->edit_ctr);
       ed = cp->edits[cp->edit_ctr];
       ed->selection_beg = NO_SELECTION;
     }
 }
 
-bool delete_selection(const char *origin, cut_selection_regraph_t regraph)
+bool delete_selection(cut_selection_regraph_t regraph)
 {
   if (selection_is_active())
     {
-      for_each_chan_1(cp_delete_selection, (void *)origin);
-      if (regraph == UPDATE_DISPLAY) for_each_chan(update_graph);
+      for_each_normal_chan(cp_delete_selection);
+      if (regraph == UPDATE_DISPLAY) for_each_normal_chan(update_graph);
       reflect_edit_without_selection_in_menu();
       if (XEN_HOOKED(selection_changed_hook)) run_hook(selection_changed_hook, XEN_EMPTY_LIST, S_selection_changed_hook);
       return(true);
@@ -204,8 +203,8 @@ static sync_info *selection_creation_chans = NULL;
 
 void deactivate_selection(void)
 {
-  for_each_chan(cp_deactivate_selection);
-  for_each_chan(update_graph);
+  for_each_normal_chan(cp_deactivate_selection);
+  for_each_normal_chan(update_graph);
   reflect_edit_without_selection_in_menu();
   if (XEN_HOOKED(selection_changed_hook)) run_hook(selection_changed_hook, XEN_EMPTY_LIST, S_selection_changed_hook);
   if (selection_creation_chans) 
@@ -234,7 +233,6 @@ void reactivate_selection(chan_info *cp, off_t beg, off_t end)
 static void update_selection(chan_info *cp, off_t newend)
 {
   ed_list *ed;
-  off_t samps;
   ed = cp->edits[cp->edit_ctr];
   ed->selection_maxamp = -1.0;
   if ((newend != ed->selection_beg) && (newend != ed->selection_end)) /* redundant call from somewhere */
@@ -247,6 +245,7 @@ static void update_selection(chan_info *cp, off_t newend)
 	}
       else 
 	{
+	  off_t samps;
 	  samps = CURRENT_SAMPLES(cp);
 	  if (newend < samps)
 	    ed->selection_end = newend;
@@ -281,10 +280,10 @@ void ripple_selection(ed_list *ed, off_t beg, off_t num)
 
 static void next_selection_chan(chan_info *cp, void *sidata)
 {
-  sync_info *si = (sync_info *)sidata;
-  int chan;
   if (cp_has_selection(cp, NULL))
     {
+      sync_info *si = (sync_info *)sidata;
+      int chan;
       chan = si->chans;
       si->chans++;
       si->begs[chan] = selection_beg(cp);
@@ -301,24 +300,26 @@ sync_info *selection_sync(void)
   si->cps = (chan_info **)CALLOC(si->chans, sizeof(chan_info *));
   si->begs = (off_t *)CALLOC(si->chans, sizeof(off_t));
   si->chans = 0;
-  for_each_chan_1(next_selection_chan, (void *)si);
+  for_each_normal_chan_1(next_selection_chan, (void *)si);
   return(si);
 }
 
-static int mix_selection(chan_info *cp, off_t beg, const char *origin, int trk)
+static int mix_selection(chan_info *cp, off_t beg)
 {
-  char *tempfile = NULL;
-  sync_info *si_out;
+  char *tempfile = NULL, *origin = NULL;
   int err, id = INVALID_MIX_ID;
   if (!(editable_p(cp))) return(id);
   tempfile = snd_tempnam();
   err = save_selection(tempfile, MUS_NEXT, MUS_OUT_FORMAT, SND_SRATE(cp->sound), NULL, SAVE_ALL_CHANS);
   if (err == MUS_NO_ERROR)
     {
+      sync_info *si_out;
       si_out = sync_to_chan(cp);
+      origin = mus_format("%s " OFF_TD, S_mix_selection, beg);
       id = mix_file(beg, selection_len(), si_out->chans, si_out->cps, tempfile, 
 		    (si_out->chans > 1) ? MULTICHANNEL_DELETION : DELETE_ME, 
-		    origin, with_mix_tags(ss), trk);
+		    origin, with_mix_tags(ss), 0);
+      FREE(origin);
       free_sync_info(si_out);	      
     }
   if (tempfile) FREE(tempfile);
@@ -326,23 +327,20 @@ static int mix_selection(chan_info *cp, off_t beg, const char *origin, int trk)
   return(id);
 }
 
-void add_selection_or_region(int reg, chan_info *cp, const char *origin)
+void add_selection_or_region(int reg, chan_info *cp)
 {
   if (cp) 
     {
       if ((reg == 0) && (selection_is_active()))
-	mix_selection(cp, CURSOR(cp), origin, 0);
-      else add_region(reg, cp, origin);
+	mix_selection(cp, CURSOR(cp));
+      else add_region(reg, cp);
     }
 }
 
-static int insert_selection(chan_info *cp, off_t beg, const char *origin)
+static int insert_selection(chan_info *cp, off_t beg)
 {
-  char *tempfile = NULL;
-  sync_info *si_out, *si_in;
-  chan_info *cp_in, *cp_out;
+  char *tempfile = NULL, *origin = NULL;
   int i, err = MUS_NO_ERROR, out_format = MUS_OUT_FORMAT;
-  off_t len;
   if (!(editable_p(cp))) return(MUS_NO_ERROR);
   if (mus_header_writable(MUS_NEXT, cp->sound->hdr->format))
     out_format = cp->sound->hdr->format;
@@ -350,26 +348,29 @@ static int insert_selection(chan_info *cp, off_t beg, const char *origin)
   err = save_selection(tempfile, MUS_NEXT, out_format, SND_SRATE(cp->sound), NULL, SAVE_ALL_CHANS);
   if (err == MUS_NO_ERROR)
     {
+      sync_info *si_out, *si_in;
       si_out = sync_to_chan(cp);
       si_in = selection_sync();
       if (si_in == NULL) /* C-g just after save? */
-	{
-	  free_sync_info(si_out);
-	}
+	free_sync_info(si_out);
       else
 	{
 	  if (si_in->chans > 1) 
 	    remember_temp(tempfile, si_in->chans);
 	  for (i = 0; ((i < si_in->chans) && (i < si_out->chans)); i++)
 	    {
+	      chan_info *cp_in, *cp_out;
+	      off_t len;
 	      cp_out = si_out->cps[i]; /* currently syncd chan that we might paste to */
 	      cp_in = si_in->cps[i];   /* selection chan to paste in (no wrap-around here) */
 	      len = cp_selection_len(cp_in, NULL);
+	      origin = mus_format("%s " OFF_TD, S_insert_selection, beg);
 	      if (file_insert_samples(beg, len,
 				      tempfile, cp_out, i,
 				      (si_in->chans > 1) ? MULTICHANNEL_DELETION : DELETE_ME,
 				      origin, cp_out->edit_ctr))
 		update_graph(cp_out);
+	      FREE(origin);
 	    }
 	  free_sync_info(si_in);
 	  free_sync_info(si_out);
@@ -380,19 +381,19 @@ static int insert_selection(chan_info *cp, off_t beg, const char *origin)
   return(err);
 }
 
-void insert_selection_or_region(int reg, chan_info *cp, const char *origin)
+void insert_selection_or_region(int reg, chan_info *cp)
 {
   if (cp) 
     {
       if ((reg == 0) && (selection_is_active()))
-	insert_selection(cp, CURSOR(cp), origin);
-      else paste_region(reg, cp, origin);
+	insert_selection(cp, CURSOR(cp));
+      else paste_region(reg, cp);
     }
 }
 
 void insert_selection_from_menu(void)
 {
-  insert_selection_or_region(0, selected_channel(), "Edit: Insert selection");
+  insert_selection_or_region(0, selected_channel());
 }
 
 
@@ -439,12 +440,12 @@ void finish_selection_creation(void)
 
 static void cp_redraw_selection(chan_info *cp, void *with_fft)
 {
-  Locus x0, x1;
-  off_t beg, end;
-  axis_info *ap;
-  double sp_srate;
   if (selection_is_visible(cp))
     {
+      Locus x0, x1;
+      off_t beg, end;
+      axis_info *ap;
+      double sp_srate;
       ap = cp->axis;
       beg = selection_beg(cp);
       end = selection_end(cp);
@@ -480,7 +481,7 @@ static void cp_redraw_selection(chan_info *cp, void *with_fft)
 static void redraw_selection(void)
 {
   /* selection transform while synced redraw */
-  for_each_chan_1(cp_redraw_selection, (void *)1);
+  for_each_normal_chan_1(cp_redraw_selection, (void *)1);
 }
 
 void display_selection(chan_info *cp)
@@ -511,10 +512,10 @@ int make_region_from_selection(void)
 
 int select_all(chan_info *cp)
 {
-  sync_info *si;
-  int i;
   if (cp) 
     {
+      sync_info *si;
+      int i;
       deactivate_selection();
       si = sync_to_chan(cp);
       for (i = 0; i < si->chans; i++)
@@ -751,7 +752,7 @@ static XEN g_delete_selection(void)
   #define H_delete_selection "(" S_delete_selection "): delete the currently selected portion"
   if (selection_is_active())
     {
-      delete_selection(S_delete_selection, UPDATE_DISPLAY);
+      delete_selection(UPDATE_DISPLAY);
       return(XEN_TRUE);
     }
   return(snd_no_active_selection_error(S_delete_selection));
@@ -760,49 +761,34 @@ static XEN g_delete_selection(void)
 static XEN g_insert_selection(XEN beg, XEN snd, XEN chn)
 {
   #define H_insert_selection "(" S_insert_selection " (beg 0) (snd #f) (chn #f)): insert the currently selected portion starting at beg"
-  chan_info *cp;
-  off_t samp;
-  int err = MUS_NO_ERROR;
   if (selection_is_active())
     {
-      char *buf;
+      chan_info *cp;
+      off_t samp;
+      int err = MUS_NO_ERROR;
       ASSERT_CHANNEL(S_insert_selection, snd, chn, 2);
       XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(beg), beg, XEN_ARG_1, S_insert_selection, "a number");
       cp = get_cp(snd, chn, S_insert_selection);
       samp = beg_to_sample(beg, S_insert_selection);
-      buf = mus_format("%s at " OFF_TD, S_insert_selection, samp);
-      err = insert_selection(cp, samp, buf);
-      FREE(buf);
+      err = insert_selection(cp, samp);
       return(C_TO_XEN_BOOLEAN((err == MUS_NO_ERROR)));
     }
   return(snd_no_active_selection_error(S_insert_selection));
 }
 
-static XEN g_mix_selection(XEN beg, XEN snd, XEN chn, XEN id)
+static XEN g_mix_selection(XEN beg, XEN snd, XEN chn)
 {
-  #define H_mix_selection "(" S_mix_selection " (beg 0) (snd #f) (chn #f) (track 0)): mix the currently selected portion starting at beg"
-  chan_info *cp;
-  off_t obeg;
-  int track_id = 0;
+  #define H_mix_selection "(" S_mix_selection " (beg 0) (snd #f) (chn #f)): mix the currently selected portion starting at beg"
   if (selection_is_active())
     {
+      chan_info *cp;
+      off_t obeg;
       XEN res;
-      char *buf;
       ASSERT_CHANNEL(S_mix_selection, snd, chn, 2);
       XEN_ASSERT_TYPE(XEN_NUMBER_IF_BOUND_P(beg), beg, XEN_ARG_1, S_mix_selection, "a number");
-      XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(id), id, XEN_ARG_4, S_mix_selection, "an integer");
       cp = get_cp(snd, chn, S_mix_selection);
       obeg = beg_to_sample(beg, S_mix_selection);
-      track_id = XEN_TO_C_INT_OR_ELSE(id, 0);
-      if ((track_id > 0) && (!(track_p(track_id))))
-	XEN_ERROR(NO_SUCH_TRACK,
-		  XEN_LIST_2(C_TO_XEN_STRING(S_mix_selection),
-			     id));
-      if (track_id == 0)
-	buf = mus_format("%s at " OFF_TD, S_mix_selection, obeg);
-      else buf = mus_format("%s at " OFF_TD " in track %d", S_mix_selection, obeg, track_id);
-      res = C_TO_XEN_INT(mix_selection(cp, obeg, S_mix_selection, track_id));
-      FREE(buf);
+      res = C_TO_XEN_INT(mix_selection(cp, obeg));
       return(res);
     }
   return(snd_no_active_selection_error(S_mix_selection));
@@ -817,13 +803,13 @@ static XEN g_selection_p(void)
 static XEN g_selection_position(XEN snd, XEN chn)
 {
   #define H_selection_position "(" S_selection_position " (snd #f) (chn #f)): selection start samp"
-  chan_info *cp;
   if (selection_is_active())
     {
       if (XEN_NOT_BOUND_P(snd))
 	return(C_TO_XEN_OFF_T(selection_beg(NULL)));
       else
 	{
+	  chan_info *cp;
 	  ASSERT_CHANNEL(S_selection_position, snd, chn, 1);
 	  cp = get_cp(snd, chn, S_selection_position);
 	  return(C_TO_XEN_OFF_T(selection_beg(cp)));
@@ -835,14 +821,13 @@ static XEN g_selection_position(XEN snd, XEN chn)
 static XEN g_set_selection_position(XEN pos, XEN snd, XEN chn)
 {
   chan_info *cp;
-  sync_info *si = NULL;
-  int i;
   off_t beg;
   ASSERT_CHANNEL(S_setB S_selection_position, snd, chn, 2);
   XEN_ASSERT_TYPE(XEN_NUMBER_P(pos), pos, XEN_ARG_1, S_selection_position, "a number");
   beg = beg_to_sample(pos, S_setB S_selection_position);
   if (XEN_NOT_BOUND_P(snd))
     {
+      sync_info *si = NULL;
       if (selection_is_active())
 	si = selection_sync();
       else 
@@ -852,6 +837,7 @@ static XEN g_set_selection_position(XEN pos, XEN snd, XEN chn)
 	}
       if (si)
 	{
+	  int i;
 	  for (i = 0; i < si->chans; i++) 
 	    cp_set_selection_beg(si->cps[i], beg);
 	  si = free_sync_info(si);
@@ -871,13 +857,13 @@ WITH_REVERSED_CHANNEL_ARGS(g_set_selection_position_reversed, g_set_selection_po
 static XEN g_selection_frames(XEN snd, XEN chn)
 {
   #define H_selection_frames "(" S_selection_frames " (snd #f) (chn #f)): selection length"
-  chan_info *cp;
   if (selection_is_active())
     {
       if (XEN_NOT_BOUND_P(snd))
 	return(C_TO_XEN_OFF_T(selection_len()));
       else
 	{
+	  chan_info *cp;
 	  ASSERT_CHANNEL(S_selection_frames, snd, chn, 1);
 	  cp = get_cp(snd, chn, S_selection_frames);
 	  return(C_TO_XEN_OFF_T(cp_selection_len(cp, NULL)));
@@ -889,8 +875,6 @@ static XEN g_selection_frames(XEN snd, XEN chn)
 static XEN g_set_selection_frames(XEN samps, XEN snd, XEN chn)
 {
   chan_info *cp;
-  sync_info *si = NULL;
-  int i;
   off_t len;
   XEN_ASSERT_TYPE(XEN_NUMBER_P(samps), samps, XEN_ARG_1, S_setB S_selection_frames, "a number");
   len = XEN_TO_C_OFF_T_OR_ELSE(samps, 0);
@@ -898,6 +882,7 @@ static XEN g_set_selection_frames(XEN samps, XEN snd, XEN chn)
     XEN_WRONG_TYPE_ARG_ERROR(S_setB S_selection_frames, XEN_ARG_1, samps, "a positive integer");
   if (XEN_NOT_BOUND_P(snd))
     {
+      sync_info *si = NULL;
       if (selection_is_active())
 	si = selection_sync();
       else 
@@ -907,6 +892,7 @@ static XEN g_set_selection_frames(XEN samps, XEN snd, XEN chn)
 	}
       if (si)
 	{
+	  int i;
 	  for (i = 0; i < si->chans; i++)
 	    cp_set_selection_len(si->cps[i], len);
 	  si = free_sync_info(si);
@@ -935,12 +921,12 @@ static XEN g_selection_member(XEN snd, XEN chn)
 
 static XEN g_set_selection_member(XEN on, XEN snd, XEN chn)
 {
-  chan_info *cp;
   XEN_ASSERT_TYPE(XEN_BOOLEAN_P(on), on, XEN_ARG_1, S_setB S_selection_member, "a boolean");
   if ((XEN_TRUE_P(snd)) && (XEN_FALSE_P(on)))
     deactivate_selection();
   else
     {
+      chan_info *cp;
       ASSERT_CHANNEL(S_setB S_selection_member, snd, chn, 2);
       cp = get_cp(snd, chn, S_setB S_selection_member);
       if (XEN_TRUE_P(on))
@@ -1036,6 +1022,11 @@ save the current selection in file using the indicated file attributes.  If chan
 			 C_TO_XEN_STRING(_("can't write this combination of header type and data format:")),
 			 C_TO_XEN_STRING(mus_header_type_name(type)),
 			 C_TO_XEN_STRING(mus_data_format_name(format))));
+  if (sr <= 0)
+    XEN_ERROR(CANNOT_SAVE,
+	      XEN_LIST_3(C_TO_XEN_STRING(S_save_selection),
+			 C_TO_XEN_STRING(_("srate can't be <= 0")),
+			 C_TO_XEN_INT(sr)));
   fname = mus_expand_filename(file);
   ss->catch_message = NULL;
   err = save_selection(fname, type, format, sr, com, chn);
@@ -1081,7 +1072,7 @@ XEN_NARGIFY_0(g_selection_srate_w, g_selection_srate)
 XEN_ARGIFY_2(g_selection_maxamp_w, g_selection_maxamp)
 XEN_NARGIFY_0(g_delete_selection_w, g_delete_selection)
 XEN_ARGIFY_3(g_insert_selection_w, g_insert_selection)
-XEN_ARGIFY_4(g_mix_selection_w, g_mix_selection)
+XEN_ARGIFY_3(g_mix_selection_w, g_mix_selection)
 XEN_ARGIFY_2(g_select_all_w, g_select_all)
 XEN_VARGIFY(g_save_selection_w, g_save_selection)
 #else
@@ -1124,7 +1115,7 @@ void g_init_selection(void)
   XEN_DEFINE_PROCEDURE(S_selection_maxamp, g_selection_maxamp_w, 0, 2, 0, H_selection_maxamp);
   XEN_DEFINE_PROCEDURE(S_delete_selection, g_delete_selection_w, 0, 0, 0, H_delete_selection);
   XEN_DEFINE_PROCEDURE(S_insert_selection, g_insert_selection_w, 0, 3, 0, H_insert_selection);
-  XEN_DEFINE_PROCEDURE(S_mix_selection,    g_mix_selection_w,    0, 4, 0, H_mix_selection);
+  XEN_DEFINE_PROCEDURE(S_mix_selection,    g_mix_selection_w,    0, 3, 0, H_mix_selection);
   XEN_DEFINE_PROCEDURE(S_select_all,       g_select_all_w,       0, 2, 0, H_select_all);
   XEN_DEFINE_PROCEDURE(S_save_selection,   g_save_selection_w,   0, 0, 1, H_save_selection);
 
