@@ -13,6 +13,7 @@
 #define S_after_edit_hook               "after-edit-hook"
 #define S_after_graph_hook              "after-graph-hook"
 #define S_after_open_hook               "after-open-hook"
+#define S_after_save_as_hook            "after-save-as-hook"
 #define S_after_save_state_hook         "after-save-state-hook"
 #define S_amp_control                   "amp-control"
 #define S_amp_control_bounds            "amp-control-bounds"
@@ -71,6 +72,7 @@
 #define S_contrast_control_amp          "contrast-control-amp"
 #define S_contrast_control_bounds       "contrast-control-bounds"
 #define S_contrast_control_p            "contrast-control?"
+#define S_controls_to_channel           "controls->channel"
 #define S_convolve_selection_with       "convolve-selection-with"
 #define S_convolve_with                 "convolve-with"
 #define S_copy_context                  "copy-context"
@@ -135,10 +137,10 @@
 #define S_edit_fragment                 "edit-fragment"
 #define S_edit_header_dialog            "edit-header-dialog"
 #define S_edit_hook                     "edit-hook"
+#define S_edit_list_to_function         "edit-list->function"
 #define S_edit_position                 "edit-position"
 #define S_edit_tree                     "edit-tree"
 #define S_edits                         "edits"
-#define S_emacs_style_save_as           "emacs-style-save-as"
 #define S_env_channel                   "env-channel"
 #define S_env_selection                 "env-selection"
 #define S_env_sound                     "env-sound"
@@ -226,8 +228,8 @@
 #define S_graph_as_sonogram             "graph-as-sonogram"
 #define S_graph_as_spectrogram          "graph-as-spectrogram"
 #define S_graphs_horizontal             "graphs-horizontal"
+#define S_grid_density                  "grid-density"
 #define S_haar_transform                "haar-transform"
-#define S_hadamard_transform            "hadamard-transform"
 #define S_header_type                   "header-type"
 #define S_help_button_color             "help-button-color"
 #define S_help_dialog                   "help-dialog"
@@ -269,6 +271,7 @@
 #define S_listener_text_color           "listener-text-color"
 #define S_load_font                     "load-font"
 #define S_lock_track                    "lock-track"
+#define S_log_freq_start                "log-freq-start"
 #define S_main_menu                     "main-menu"
 #define S_main_widgets                  "main-widgets"
 #define S_make_color                    "make-color"
@@ -297,6 +300,8 @@
 #define S_mark_sample                   "mark-sample"
 #define S_mark_sync                     "mark-sync"
 #define S_mark_sync_max                 "mark-sync-max"
+#define S_mark_tag_height               "mark-tag-height"
+#define S_mark_tag_width                "mark-tag-width"
 #define S_marks                         "marks"
 #define S_max_regions                   "max-regions"
 #define S_max_transform_peaks           "max-transform-peaks"
@@ -352,6 +357,7 @@
 #define S_next_sample                   "next-sample"
 #define S_normalize_by_channel          "normalize-by-channel"
 #define S_normalize_by_sound            "normalize-by-sound"
+#define S_normalize_channel             "normalize-channel"
 #define S_normalize_globally            "normalize-globally"
 #define S_open_file_dialog              "open-file-dialog"
 #define S_open_hook                     "open-hook"
@@ -378,6 +384,7 @@
 #define S_play_selection                "play-selection"
 #define S_play_track                    "play-track"
 #define S_player_home                   "player-home"
+#define S_players                       "players"
 #define S_player_p                      "player?"
 #define S_position_to_x                 "position->x"
 #define S_position_to_y                 "position->y"
@@ -404,22 +411,8 @@
 #define S_read_mix_sample               "read-mix-sample"
 #define S_read_region_sample            "read-region-sample"
 #define S_read_track_sample             "read-track-sample"
-#define S_recorder_autoload             "recorder-autoload"
-#define S_recorder_buffer_size          "recorder-buffer-size"
 #define S_recorder_dialog               "recorder-dialog"
-#define S_recorder_file                 "recorder-file"
 #define S_recorder_file_hook            "recorder-file-hook"
-#define S_recorder_gain                 "recorder-gain"
-#define S_recorder_in_amp               "recorder-in-amp"
-#define S_recorder_in_format            "recorder-in-format"
-#define S_recorder_in_device            "recorder-in-device"
-#define S_recorder_max_duration         "recorder-max-duration"
-#define S_recorder_out_amp              "recorder-out-amp"
-#define S_recorder_out_chans            "recorder-out-chans"
-#define S_recorder_out_format           "recorder-out-format"
-#define S_recorder_out_type             "recorder-out-type"
-#define S_recorder_srate                "recorder-srate"
-#define S_recorder_trigger              "recorder-trigger"
 #if HAVE_RUBY
   #define S_redo                        "redo-edit"
 #else
@@ -433,7 +426,7 @@
 #define S_region_position               "region-position"
 #define S_region_sample                 "region-sample"
 #define S_region_sample_reader_p        "region-sample-reader?"
-#define S_region_samples_to_vct         "region-samples->vct"
+#define S_region_to_vct                 "region->vct"
 #define S_region_srate                  "region-srate"
 #define S_regions                       "regions"
 #define S_remove_from_menu              "remove-from-menu"
@@ -465,7 +458,6 @@
 #define S_sample_reader_p               "sample-reader?"
 #define S_samples                       "samples"
 #define S_samples_to_sound_data         "samples->sound-data"
-#define S_samples_to_vct                "samples->vct"
 #define S_sash_color                    "sash-color"
 #define S_save_controls                 "save-controls"
 #define S_save_dir                      "save-dir"
@@ -489,8 +481,6 @@
 #define S_scale_channel                 "scale-channel"
 #define S_scale_selection_by            "scale-selection-by"
 #define S_scale_selection_to            "scale-selection-to"
-#define S_scale_sound_by                "scale-sound-by"
-#define S_scale_sound_to                "scale-sound-to"
 #define S_scale_to                      "scale-to" 
 #define S_scan_chan                     "scan-chan"
 #define S_scan_channel                  "scan-channel"
@@ -531,6 +521,7 @@
 #define S_show_mix_waveforms            "show-mix-waveforms"
 #define S_show_no_axes                  "show-no-axes"
 #define S_show_selection_transform      "show-selection-transform"
+#define S_show_sonogram_cursor          "show-sonogram-cursor"
 #define S_show_transform_peaks          "show-transform-peaks"
 #define S_show_widget                   "show-widget"
 #define S_show_x_axis                   "show-x-axis"
@@ -626,8 +617,8 @@
 #define S_transform_hook                "transform-hook"
 #define S_transform_normalization       "transform-normalization"
 #define S_transform_sample              "transform-sample"
-#define S_transform_samples_size        "transform-samples-size"
-#define S_transform_samples_to_vct      "transform-samples->vct"
+#define S_transform_frames              "transform-frames"
+#define S_transform_to_vct              "transform->vct"
 #define S_transform_size                "transform-size"
 #define S_transform_type                "transform-type"
 #define S_trap_segfault                 "trap-segfault"
@@ -639,10 +630,8 @@
 #define S_update_sound                  "update-sound"
 #define S_update_time_graph             "update-time-graph"
 #define S_update_transform_graph        "update-transform-graph"
-#define S_vct_convolve                  "vct-convolve!"
 #define S_vct_map                       "vct-map"
 #define S_vct_to_channel                "vct->channel"
-#define S_vct_to_samples                "vct->samples"
 #define S_vct_to_sound_file             "vct->sound-file"
 #define S_verbose_cursor                "verbose-cursor"
 #define S_view_files_dialog             "view-files-dialog"
