@@ -1,18 +1,17 @@
 /* translate various special case sound files to something we can edit 
  * 
- * I'm ignoring propietary or licensed schemes even where the code is publicly available (Rockwell ADPCM, shorten, etc)
+ * I'm ignoring proprietary or licensed schemes even where the code is publicly available (Rockwell ADPCM, shorten, etc)
  *
  * currently supported:
  *   IEEE text
  *   Mus10 16-bit SAM (mode 4)
- *   IBM CVSD RIFF
  *   HCOM (from Sox)
  *   shortpack NIST
  *   Dvi-Intel (IMA) ADPCM RIFF (comes in 3 and 4 bit flavors, but just 4-bit here) (MS and Apple are variations of this)
  *   MIDI sample dump
  *   Oki (Dialogic) ADPCM (RIFF)
+ *   IBM ADPCM (as per Perry Cook)
  *   Yamaha TX-16 12-bit
- *   IFF Fibonacci and Exponential (untested)
  *   NeXT/Sun G721, G723 3 and 5 bit versions (also RIFF) (AIFC cases could be handled if they exist)
  */
 
@@ -22,23 +21,24 @@
 
 static char write_error_buffer[PRINT_BUFFER_SIZE];
 
-static int snd_checked_write(int fd, unsigned char *buf, int bytes, char *filename)
+static int snd_checked_write(int fd, unsigned char *buf, int bytes, const char *filename)
 {
   /* io.c checked_write assumes its file descriptors are around */
   /* can't call mus_error here because we need to clean up first in case of error */
-  int bytes_written, kfree;
+  int bytes_written;
+  off_t kfree;
   kfree = disk_kspace(filename);
   if (kfree < 0) 
     {
       mus_snprintf(write_error_buffer, PRINT_BUFFER_SIZE,
-		   "no space left on device: %s",
+		   _("no space left on device: %s"),
 		   strerror(errno)); 
       return(MUS_ERROR);
     }
   if (kfree < (bytes >> 10))
     { 
       mus_snprintf(write_error_buffer, PRINT_BUFFER_SIZE,
-		   "only %d bytes left on device (we need %d bytes)",
+		   _("only " PRId64 " bytes left on device (we need %d bytes)"),
 		   kfree << 10, bytes);
       return(MUS_ERROR);
     }
@@ -46,14 +46,14 @@ static int snd_checked_write(int fd, unsigned char *buf, int bytes, char *filena
   if (bytes_written != bytes)
     {
       mus_snprintf(write_error_buffer, PRINT_BUFFER_SIZE,
-		   "write error (wrote %d of requested %d bytes): %s",
+		   _("write error (wrote %d of requested %d bytes): %s"),
 		   bytes_written, bytes, strerror(errno));
       return(MUS_ERROR);
     }
   return(bytes_written);
 }
 
-static int be_snd_checked_write(int fd, unsigned char *buf, int bytes, char *filename)
+static int be_snd_checked_write(int fd, unsigned char *buf, int bytes, const char *filename)
 {
   /* handle little-endian swap if necessary */
 #if MUS_LITTLE_ENDIAN
@@ -69,49 +69,33 @@ static int be_snd_checked_write(int fd, unsigned char *buf, int bytes, char *fil
   return(snd_checked_write(fd, buf, bytes, filename));
 }
 
-#define RETURN_MUS_IO_ERROR(IO_Func, IO_Name) \
-  do { \
-      mus_error(MUS_CANT_OPEN_FILE, "translator: %s(%s): %s\n  [%s[%d] %s]", \
-                IO_Func, IO_Name, strerror(errno), \
-                __FILE__, __LINE__, __FUNCTION__); \
-      return(MUS_ERROR); \
-      } \
-  while (0)
+#define RETURN_MUS_IO_ERROR(IO_Func, IO_Name) return(mus_error(MUS_CANT_OPEN_FILE, "translator: %s(%s) %s", IO_Func, IO_Name, strerror(errno)))
 
 #define RETURN_MUS_WRITE_ERROR(OldName, NewName) \
   do { \
-      mus_error(MUS_WRITE_ERROR, "can't translate %s to %s:\n  %s\n [snd-trans.c[%d] %s]", \
-		OldName, NewName, write_error_buffer, \
-		__LINE__, __FUNCTION__); \
+      mus_error(MUS_WRITE_ERROR, "can't translate %s to %s:\n  %s", OldName, NewName, write_error_buffer); \
       write_error_buffer[0] = '\0'; \
       return(MUS_ERROR); \
     } \
-  while (0)
+  while (false)
 
 #define RETURN_MUS_ALLOC_ERROR(OldName, Bytes, VariableName) \
-  do { \
-      mus_error(MUS_MEMORY_ALLOCATION_FAILED, "translate %s: can't allocate %d bytes for %s:\n  [snd-trans.c[%d] %s]", \
-		OldName, Bytes, VariableName, \
-		__LINE__, __FUNCTION__); \
-      return(MUS_ERROR); \
-    } \
-  while (0)
-
+  return(mus_error(MUS_MEMORY_ALLOCATION_FAILED, "translate %s: can't allocate %d bytes for %s", OldName, Bytes, VariableName))
 
 /* I'm using the same variable names in most cases below, so these two macros save lots of repetition */
 #define CLEANUP() \
   do { \
-      if (fs != -1) close(fs); \
-      if (fd != -1) close(fd); \
+      if (fs != -1) CLOSE(fs); \
+      if (fd != -1) CLOSE(fd); \
       if (buf) FREE(buf); \
      } \
-  while (0)
+  while (false)
 
 #define STARTUP(OldName, NewName, BufSize, BufType) \
   do { \
-    fs = creat(NewName, 0666); \
+    fs = CREAT(NewName, 0666); \
     if (fs == -1) RETURN_MUS_IO_ERROR("create", NewName); \
-    fd = open(OldName, O_RDONLY, 0); \
+    fd = OPEN(OldName, O_RDONLY, 0); \
     if (fd == -1) \
       { \
         CLEANUP(); \
@@ -124,7 +108,7 @@ static int be_snd_checked_write(int fd, unsigned char *buf, int bytes, char *fil
         RETURN_MUS_ALLOC_ERROR(OldName, BufSize, "buf"); \
       } \
     } \
-  while (0)
+  while (false)
 
 
 
@@ -137,9 +121,10 @@ static int be_snd_checked_write(int fd, unsigned char *buf, int bytes, char *fil
  * 0000020       0000 007f f7f0 7e00 0200 4000 003f 7140
  */
 
-static int read_midi_sample_dump(char *oldname, char *newname, char *hdr)
+static int read_midi_sample_dump(const char *oldname, const char *newname, char *hdr)
 {
-  int fs = -1, fd = -1, err = MUS_NO_ERROR, totalin, happy, chans, srate, inp, outp;
+  int fs = -1, fd = -1, err = MUS_NO_ERROR, totalin, chans, srate, inp, outp;
+  bool happy;
   int val = 0, bits, block_count, header_count, state, samples, shift1, shift2, offset;
   int osp;
   unsigned char *buf = NULL;
@@ -159,7 +144,7 @@ static int read_midi_sample_dump(char *oldname, char *newname, char *hdr)
       CLEANUP();
       RETURN_MUS_WRITE_ERROR(oldname, newname);
     }
-  happy = 1;
+  happy = true;
   inp = 21;
   block_count = 120;
   state = 2;
@@ -184,7 +169,7 @@ static int read_midi_sample_dump(char *oldname, char *newname, char *hdr)
       if (inp >= totalin)
 	{
 	  if (totalin < TRANS_BUF_SIZE) 
-	    happy = 0;
+	    happy = false;
 	  else 
 	    {
 	      totalin = read(fd, buf, TRANS_BUF_SIZE); 
@@ -240,7 +225,7 @@ static int read_midi_sample_dump(char *oldname, char *newname, char *hdr)
 	      /* val |= (buf[inp] << shift1);  */
 	      val |= (buf[inp] >> shift2);
 	      state = 0; 
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), val - offset);
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)(val - offset));
 	      osp += 2;
 	      outp += 2; 
 	      break;
@@ -258,32 +243,25 @@ static int read_midi_sample_dump(char *oldname, char *newname, char *hdr)
 
 /* -------------------------------- IEEE TEXT -------------------------------- */
 
-static int read_ieee_text(char *oldname, char *newname, char *hdr)
+static int read_ieee_text(const char *oldname, const char *newname, char *hdr)
 {
   /* from untext.c */
   /* look for "%sampling rate: nn.nn KHz\n", also get end of to comment (i.e. data location) */
   char str[32];
   char *buf = NULL;
   int fd = -1, fs = -1, totalin;
-  int commenting, inp, outp, op, happy, i, j, s0, srate, err = MUS_NO_ERROR;
+  int inp, outp, op, i, j, s0, srate, err = MUS_NO_ERROR;
+  bool commenting, happy;
   float fsrate;
   int osp;
   STARTUP(oldname, newname, TRANS_BUF_SIZE, char);
   totalin = read(fd, buf, TRANS_BUF_SIZE);      
-  commenting = 1;
+  commenting = true;
   inp = 0;
   outp = 24;
   srate = 0;
   op = 0;
-  if (buf[0] != '%') 
-    {
-      CLEANUP();
-      mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
-		"can't translate IEEE text file %s:\n  Expected initial \"%%\" but found \"%c\"\n  [snd-trans.c[%d] %s]",
-		oldname, buf[0], __LINE__, __FUNCTION__);
-      return(MUS_ERROR);
-    }
-  while (commenting)
+  while ((commenting) && (inp < totalin))
     {
       if (buf[inp] == '%') {op = inp; inp++;}
       else
@@ -312,7 +290,7 @@ static int read_ieee_text(char *oldname, char *newname, char *hdr)
 		    }
 		}
 	      inp++;
-	      if (buf[inp] != '%') commenting = 0;
+	      if (buf[inp] != '%') commenting = false;
 	      else
 		{
 		  hdr[outp] = '\n';
@@ -327,16 +305,21 @@ static int read_ieee_text(char *oldname, char *newname, char *hdr)
 	    }
 	}
     }
+  if (srate == 0) 
+    {
+      CLEANUP();
+      return(MUS_ERROR);
+    }
   i = (outp % 4);
   outp += i;
   mus_bint_to_char((unsigned char *)(hdr + 4), outp);
-  if (srate != 0) mus_bint_to_char((unsigned char *)(hdr + 16), srate);
+  mus_bint_to_char((unsigned char *)(hdr + 16), srate);
   if (snd_checked_write(fs, (unsigned char *)hdr, outp, newname) == MUS_ERROR) 
     {
       CLEANUP();
       RETURN_MUS_WRITE_ERROR(oldname, newname);
     }
-  happy = 1;
+  happy = true;
   s0 = 0;
   outp = 0;
   osp = 0;
@@ -345,7 +328,7 @@ static int read_ieee_text(char *oldname, char *newname, char *hdr)
       if (inp >= totalin)
 	{
 	  if (totalin < TRANS_BUF_SIZE) 
-	    happy = 0;
+	    happy = false;
 	  else 
 	    {
 	      totalin = read(fd, buf, TRANS_BUF_SIZE); 
@@ -368,7 +351,7 @@ static int read_ieee_text(char *oldname, char *newname, char *hdr)
 	    {
 	      str[s0] = '\0';
 	      sscanf(str, "%d", &j);
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), j);
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)j);
 	      osp += 2;
 	      outp += 2;
 	      inp++;
@@ -394,13 +377,14 @@ static int read_ieee_text(char *oldname, char *newname, char *hdr)
 
 #define PDP_BUF_SIZE (9*1024)
 
-static int read_mus10(char *oldname, char *newname, char *hdr)
+static int read_mus10(const char *oldname, const char *newname, char *hdr)
 {
   /* from trans.lisp */
   /* nostalgic code -- 36 bit words, two 16-bit samples, right justified */
   /* or (even more archaeological) 12 bits packed 3 to a 36-bit word */
   unsigned char *buf = NULL;
-  int fd = -1, fs = -1, totalin, inp, outp, happy, val, err = MUS_NO_ERROR;
+  int fd = -1, fs = -1, totalin, inp, outp, val, err = MUS_NO_ERROR;
+  bool happy;
   int osp;
   float fsrate, fraction;
   int srateH, srateL, sign, exponent, chans, mode;
@@ -429,10 +413,9 @@ static int read_mus10(char *oldname, char *newname, char *hdr)
   if ((mode != 4) && (mode != 0)) 
     {
       CLEANUP();
-      mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
-		"can't translate Mus10 file %s:\n  mode = %d\n  [snd-trans.c[%d] %s]",
-		oldname, mode, __LINE__, __FUNCTION__);
-      return(MUS_ERROR);
+      return(mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
+		       _("read_mus10: can't translate Mus10 file %s:\n  mode = %d\n"),
+		       oldname, mode));
     }
   /* 4 = SAM 16-bit packing mode, 0 = 12 bit 3 to a word */
   /* now jump to data start */
@@ -442,7 +425,7 @@ static int read_mus10(char *oldname, char *newname, char *hdr)
       CLEANUP();
       RETURN_MUS_WRITE_ERROR(oldname, newname);
     }
-  happy = 1;
+  happy = true;
   outp = 0;
   osp = 0;
   while (happy)
@@ -450,7 +433,7 @@ static int read_mus10(char *oldname, char *newname, char *hdr)
       if (inp >= totalin)
 	{
 	  if (totalin < PDP_BUF_SIZE) 
-	    happy = 0;
+	    happy = false;
 	  else 
 	    {
 	      totalin = read(fd, buf, PDP_BUF_SIZE); 
@@ -475,20 +458,20 @@ static int read_mus10(char *oldname, char *newname, char *hdr)
 	      /* so we grab four at a time here to keep the pointers aligned */
 	      /* we've chosen an input buffer size that is a multiple of 9 so that this code need not constantly check bounds */
 	      val = ((buf[inp] & 0xF) << 12) | (buf[inp + 1] << 4) | (buf[inp + 2] >> 4);
-	      mus_bshort_to_char((unsigned char *)(hdr+osp), val); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)val); osp += 2;
 	      val = ((buf[inp + 2] & 0xF) << 12) | (buf[inp + 3] << 4) | (buf[inp + 4] >> 4);
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), val); osp += 2;
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), ((buf[inp + 5] << 8) | buf[inp + 6])); osp += 2;
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), ((buf[inp + 7] << 8) | buf[inp + 8])); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)val); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)((buf[inp + 5] << 8) | buf[inp + 6])); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)((buf[inp + 7] << 8) | buf[inp + 8])); osp += 2;
 	      outp += 8;
 	      inp += 9;
 	    }
 	  else
 	    {
 	      val = (buf[inp] << 8) | (buf[inp + 1] & 0xF0);
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), val); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)val); osp += 2;
 	      val = ((buf[inp + 1] & 0xF) << 12) | (buf[inp + 2] << 4);
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), val); osp += 2;
+	      mus_bshort_to_char((unsigned char *)(hdr + osp), (short)val); osp += 2;
 	      outp += 4;
 	      inp += 3;
 	    }
@@ -501,94 +484,14 @@ static int read_mus10(char *oldname, char *newname, char *hdr)
 }
 
 
-/* -------------------------------- IBM CVSD --------------------------------
- *
- * sox11 cvsd.c claims there's a spec for some form of this silliness:
- *      The CVSD format is described in the MIL Std 188 113, which is
- *      available from http://bbs.itsi.disa.mil:5580/T3564
- *
- * it also pushes the bits through a filter, and counts down from bit 7 to 0,
- * but it's definitely different from the CVSD as intended in a wav file.
- */
-
-static int read_ibm_cvsd(char *oldname, char *newname, char *hdr)
-{
-  /* assumed to be in a RIFF file, and that we just read the header via c_read_header */
-  /* avg rate gives srate/8 (8 bits per byte) -- can be ignored, can be stereo */
-  int fs = -1, fd = -1, loc, totalin, happy, chans, srate, inp, outp, i, chn, byte, err = MUS_NO_ERROR;
-  int *curvals;
-  int osp;
-  unsigned char *buf = NULL;
-  STARTUP(oldname, newname, TRANS_BUF_SIZE, unsigned char);
-  loc = mus_sound_data_location(oldname);
-  chans = mus_sound_chans(oldname);
-  curvals = (int *)CALLOC(chans, sizeof(int));
-  srate = mus_sound_srate(oldname);
-  mus_bint_to_char((unsigned char *)(hdr + 16), srate);
-  mus_bint_to_char((unsigned char *)(hdr + 20), chans);
-  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR)
-    {
-      CLEANUP();
-      RETURN_MUS_WRITE_ERROR(oldname, newname);
-    }
-  lseek(fd, loc, SEEK_SET);
-  totalin = read(fd, buf, TRANS_BUF_SIZE);
-  happy = 1;
-  inp = 0;
-  outp = 0;
-  osp = 0;
-  while (happy)
-    {
-      if (inp >= totalin)
-	{
-	  if (totalin < TRANS_BUF_SIZE) 
-	    happy = 0;
-	  else 
-	    {
-	      totalin = read(fd, buf, TRANS_BUF_SIZE); 
-	      inp = 0;
-	    }
-	}
-      if (outp >= TRANS_BUF_SIZE) 
-	{
-	  if (snd_checked_write(fs, (unsigned char *)hdr, TRANS_BUF_SIZE, newname) == MUS_ERROR) 
-	    {
-	      CLEANUP();
-	      RETURN_MUS_WRITE_ERROR(oldname, newname);
-	    }
-	  osp = 0;
-	  outp = 0;
-	}
-      if (happy)
-	{
-	  /* each byte becomes 8 samples */
-	  chn = 0;
-	  byte = buf[inp]; inp++;
-	  for (i = 0; i < 8; i++)
-	    {
-	      /* are the bits consumed low to high or high to low? assume low to high for now (count i down from 7 to 0 if high to low) */
-	      if (byte & (1 << i)) curvals[chn]++; else curvals[chn]--;
-	      mus_bshort_to_char((unsigned char *)(hdr + osp), curvals[chn]); osp += 2; chn++;
-	      if (chn == chans) chn = 0;
-	    }
-	  outp += 16;
-	}
-    }
-  err = snd_checked_write(fs, (unsigned char *)hdr, outp, newname);
-  FREE(curvals);
-  CLEANUP();
-  if (err == MUS_ERROR) RETURN_MUS_WRITE_ERROR(oldname, newname);
-  return(MUS_NO_ERROR);
-}
-
-
 /* -------------------------------- HCOM (from Sox) -------------------------------- */
 
-static int read_hcom(char *oldname, char *newname, char *hdr)
+static int read_hcom(const char *oldname, const char *newname, char *hdr)
 {
   short **d;
   int osp, isp;
-  int dc, di, bits, outp, happy, totalin;
+  int dc, di, bits, outp, totalin;
+  bool happy;
   unsigned int curval = 0;
   int i, sample, size, datum, count, err = MUS_NO_ERROR;
   unsigned char *buf = NULL;
@@ -618,16 +521,16 @@ static int read_hcom(char *oldname, char *newname, char *hdr)
   totalin = read(fd, buf, TRANS_BUF_SIZE);
   osp = 0;
   isp = 0;
-  happy = 1;
+  happy = true;
   outp = 2;
-  mus_bshort_to_char((unsigned char *)(hdr + osp), (sample - 128) * 0x100); osp += 2;
+  mus_bshort_to_char((unsigned char *)(hdr + osp), (short)((sample - 128) * 0x100)); osp += 2;
   bits = 0;
   while ((happy) && (count > 0))
     {
       if (isp >= totalin)
 	{
 	  if (totalin < TRANS_BUF_SIZE) 
-	    happy = 0;
+	    happy = false;
 	  else 
 	    {
 	      totalin = read(fd, buf, TRANS_BUF_SIZE); 
@@ -663,8 +566,8 @@ static int read_hcom(char *oldname, char *newname, char *hdr)
 	      if (!dc) sample = 0;
 	      sample = (sample + datum) & 0xff;
 	      count--;
-	      if (sample == 0) mus_bshort_to_char((unsigned char *)(hdr + osp), (-127 * 0x100));
-	      else mus_bshort_to_char((unsigned char *)(hdr + osp), ((sample - 128) * 0x100));
+	      if (sample == 0) mus_bshort_to_char((unsigned char *)(hdr + osp), (short)(-127 * 0x100));
+	      else mus_bshort_to_char((unsigned char *)(hdr + osp), (short)((sample - 128) * 0x100));
 	      osp += 2;
 	      outp += 2;
 	      di = 0;
@@ -684,14 +587,14 @@ static int read_hcom(char *oldname, char *newname, char *hdr)
 
 static unsigned short log2s[] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768};
 
-static int read_nist_shortpack(char *oldname, char *newname, char *hdr)
+static int read_nist_shortpack(const char *oldname, const char *newname, char *hdr)
 {
-  /* assume all relevant header stuff is ready via c_read_header */
-  int fs = -1, fd = -1, err = MUS_NO_ERROR, totalin, happy, chans, srate, outp, i = 0, k, num, bits = 0, out, els = 0;
+  int fs = -1, fd = -1, err = MUS_NO_ERROR, totalin, chans, srate, outp, i = 0, k, num, bits = 0, out, els = 0;
+  bool happy;
   int isp, osp;
   unsigned short *ptr = NULL, *stop, *start, *kptr;
   short temp = 0;
-  unsigned char negative;
+  bool negative;
   unsigned char *buf = NULL;
   chans = mus_sound_chans(oldname);
   srate = mus_sound_srate(oldname);
@@ -705,7 +608,7 @@ static int read_nist_shortpack(char *oldname, char *newname, char *hdr)
     }
   lseek(fd, 1024, SEEK_SET);                       /* NIST header always 1024 bytes */
   totalin = read(fd, buf, TRANS_BUF_SIZE);
-  happy = 1;
+  happy = true;
   outp = 0;
   num = 0;
   osp = 0; /* hdr */
@@ -782,10 +685,10 @@ static int read_nist_shortpack(char *oldname, char *newname, char *hdr)
 	  if (negative)
 	    {
 	      if (out != 0) 
-		mus_bshort_to_char((unsigned char *)(hdr + osp), -out);
-	      else mus_bshort_to_char((unsigned char *)(hdr + osp), 32767);
+		mus_bshort_to_char((unsigned char *)(hdr + osp), (short)(-out));
+	      else mus_bshort_to_char((unsigned char *)(hdr + osp), (short)32767);
 	    }
-	  else mus_bshort_to_char((unsigned char *)(hdr + osp), out);
+	  else mus_bshort_to_char((unsigned char *)(hdr + osp), (short)out);
 	  osp += 2; 
 	  outp += 2;
 	  i++;
@@ -794,7 +697,7 @@ static int read_nist_shortpack(char *oldname, char *newname, char *hdr)
       if (isp >= totalin)
 	{
 	  if (totalin < TRANS_BUF_SIZE) 
-	    happy = 0;
+	    happy = false;
 	  else 
 	    {
 	      totalin = read(fd, buf, TRANS_BUF_SIZE); 
@@ -819,6 +722,68 @@ static int read_nist_shortpack(char *oldname, char *newname, char *hdr)
 }
 
 
+/* -------------------------------- IBM ADPCM --------------------------------
+ *
+ * taken from Perry Cook's adpcmdec.c
+ */
+
+static int read_ibm_adpcm(const char *oldname, const char *newname, char *hdr)
+{
+  short MAX_STEP = 2048, MIN_STEP = 16;
+  int totalin, i, j, k, fs = -1, fd = -1;
+  unsigned char *buf = NULL;
+  short *buf1;
+  off_t loc;
+  short XHAT1 = 0, delndec, del1dec = 8, ln1dec, temp;
+  unsigned short data_in;
+  float M[16] = {0.909, 0.909, 0.909, 0.909, 1.21, 1.4641, 1.771561, 2.143589, 0.909, 0.909, 0.909, 0.909, 1.21, 1.4641, 1.771561, 2.143589};
+  float del_table[16] = {0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 0.0, -0.25, -0.5, -0.75, -1.0, -1.25, -1.5, -1.75};
+  loc = mus_sound_data_location(oldname);
+  mus_bint_to_char((unsigned char *)(hdr + 16), mus_sound_srate(oldname));
+  mus_bint_to_char((unsigned char *)(hdr + 20), 1);
+  STARTUP(oldname, newname, TRANS_BUF_SIZE, unsigned char);
+  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR)
+    {
+      CLEANUP();
+      RETURN_MUS_WRITE_ERROR(oldname, newname);
+    }
+  lseek(fd, loc, SEEK_SET);
+  buf1 = (short *)CALLOC(TRANS_BUF_SIZE, sizeof(short));
+  while (true)
+    {
+      totalin = read(fd, buf, TRANS_BUF_SIZE / 4);
+      if (totalin <= 0) break;
+      for (i = 0, j = 0; i < totalin; i += 2, j += 4)
+	{
+	  data_in = (*((short *)(buf + i)));
+	  for (k = 0; k < 4; k++)
+	    {
+	      ln1dec = data_in >> 12;
+	      data_in = data_in << 4;
+	      delndec = (short)(del1dec * M[ln1dec]);
+	      temp = delndec - MIN_STEP;
+	      if (temp < 0) delndec = MIN_STEP; 
+	      temp = MAX_STEP - delndec;
+	      if (temp < 0) delndec = MAX_STEP;
+	      del1dec = delndec;
+	      XHAT1 += (short)(delndec * del_table[ln1dec]);
+	      if (XHAT1 > 32000 || XHAT1 < -32000) XHAT1 = (short)(XHAT1 * 0.95);
+	      buf1[j + k] = XHAT1;
+	    }
+	}
+      if (be_snd_checked_write(fs, (unsigned char *)buf1, j * 2, newname) == MUS_ERROR) 
+	{
+	  FREE(buf1);
+	  CLEANUP();
+	  RETURN_MUS_WRITE_ERROR(oldname, newname);
+	}
+    }
+  FREE(buf1);
+  CLEANUP();
+  return(MUS_NO_ERROR);
+}
+
+
 /* -------------------------------- Intel ADPCM --------------------------------
  *
  * described in detail Microsoft RIFF docs.  This code assumes bits = 4.
@@ -838,9 +803,10 @@ static int stepsizeTable[89] = {7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23,
 static int adpcm_decoder(unsigned char *indata, short *outdata, int totalbytes, int type)
 {
   unsigned int delta, inputbuffer = 0;
-  int step, valpred, vpdiff, index, bufferstep, i, j, happy;
-  bufferstep = 0;
-  happy = 1;
+  int step, valpred, vpdiff, index, i, j;
+  bool happy;
+  bool bufferstep = false;
+  happy = true;
   if (type == 0)
     {
       j = 4;
@@ -860,7 +826,7 @@ static int adpcm_decoder(unsigned char *indata, short *outdata, int totalbytes,
       if (bufferstep) 
 	{
 	  delta = inputbuffer & 0xf;
-	  if (j == totalbytes) happy = 0;
+	  if (j == totalbytes) happy = false;
 	} 
       else 
 	{
@@ -883,22 +849,19 @@ static int adpcm_decoder(unsigned char *indata, short *outdata, int totalbytes,
 }
 
 
-static int read_dvi_adpcm(char *oldname, char *newname, char *hdr, int type)
+static int read_dvi_adpcm(const char *oldname, const char *newname, char *hdr, int type)
 {
-  int fs = -1, fd = -1, loc, totalin, chans, srate, blksiz, samps, samps_read;
+  int fs = -1, fd = -1, totalin, chans, srate, blksiz, samps, samps_read;
   unsigned char *buf = NULL;
+  off_t loc;
   loc = mus_sound_data_location(oldname);
   chans = mus_sound_chans(oldname);
-  blksiz = mus_sound_align(oldname);
+  blksiz = mus_sound_block_align(oldname);
   samps = mus_sound_fact_samples(oldname);
   if ((chans != 1) || (mus_sound_bits_per_sample(oldname) != 4))
-    {
-      mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
-		"can't translate DVI ADPCM file %s: chans: %d and bits: %d\n  [snd-trans.c[%d] %s]",
-		oldname, chans, mus_sound_bits_per_sample(oldname),
-		__LINE__, __FUNCTION__);
-      return(MUS_ERROR);
-    }
+    return(mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
+		     _("read_dvi_adpcm: can't translate DVI ADPCM file %s: chans: %d and bits: %d\n"),
+		     oldname, chans, mus_sound_bits_per_sample(oldname)));
   srate = mus_sound_srate(oldname);
   mus_bint_to_char((unsigned char *)(hdr + 16), srate);
   mus_bint_to_char((unsigned char *)(hdr + 20), chans);
@@ -942,7 +905,7 @@ static short oki_step_size[49] = { 16, 17, 19, 21, 23, 25, 28, 31, 34, 37, 41,
      190, 209, 230, 253, 279, 307, 337, 371, 408, 449, 494, 544, 598, 658,
      724, 796, 876, 963, 1060, 1166, 1282, 1408, 1552 };
 
-static short oki_adjust[8] ={-1, -1, -1, -1, 2, 4, 6, 8};
+static short oki_adjust[8] = {-1, -1, -1, -1, 2, 4, 6, 8};
 
 static short oki_adpcm_decode(char code, struct oki_adpcm_status *stat) 
 {
@@ -965,33 +928,29 @@ static short oki_adpcm_decode(char code, struct oki_adpcm_status *stat)
   return(samp << 4);
 }
 
-static int read_oki_adpcm(char *oldname, char *newname, char *hdr)
+static int read_oki_adpcm(const char *oldname, const char *newname, char *hdr)
 {
-  int fs = -1, fd = -1, loc, i, j, totalin, chans, srate, blksiz, samps, samps_read;
+  int fs = -1, fd = -1, i, j, totalin, chans, srate, blksiz, samps, samps_read;
   unsigned char *buf = NULL;
+  off_t loc;
   short *buf1;
   struct oki_adpcm_status stat;
   chans = mus_sound_chans(oldname);
   if (chans != 1)
-    {
-      mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
-		"can't translate Oki ADPCM file %s: chans: %d\n  [snd-trans.c[%d] %s]",
-		oldname, chans,
-		__LINE__, __FUNCTION__);
-      return(MUS_ERROR);
-    }
+    return(mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
+		     _("read_oki_adpcm: can't translate Oki ADPCM file %s: chans: %d\n"),
+		     oldname, chans));
   loc = mus_sound_data_location(oldname);
-  blksiz = mus_sound_align(oldname);
+  blksiz = mus_sound_block_align(oldname);
   if (blksiz == 0) blksiz = 256;
   STARTUP(oldname, newname, blksiz, unsigned char);
   buf1 = (short *)CALLOC(blksiz * 2, sizeof(short));
   samps = mus_sound_fact_samples(oldname);
-  if (samps == 0) samps = mus_sound_frames(oldname);
+  if (samps == 0) samps = (int)mus_sound_frames(oldname);
   srate = mus_sound_srate(oldname);
   mus_bint_to_char((unsigned char *)(hdr + 16), srate);
   mus_bint_to_char((unsigned char *)(hdr + 20), chans);
-  fd = open(oldname, O_RDONLY, 0);
-  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR) 
+  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR)
     {
       FREE(buf1);
       CLEANUP();
@@ -1028,11 +987,12 @@ static int read_oki_adpcm(char *oldname, char *newname, char *hdr)
 /* -------------------------------- 12 bit cases --------------------------------
  */
 
-static int read_12bit(char *oldname, char *newname, char *hdr)
+static int read_12bit(const char *oldname, const char *newname, char *hdr)
 {
-  int loc, chans, samps, totalin, i, j, fs = -1, fd = -1;
+  int chans, totalin, i, j, fs = -1, fd = -1;
   unsigned char *buf = NULL;
   short *buf1;
+  off_t loc, samps;
   loc = mus_sound_data_location(oldname);
   chans = mus_sound_chans(oldname);
   samps = mus_sound_samples(oldname);
@@ -1069,134 +1029,6 @@ static int read_12bit(char *oldname, char *newname, char *hdr)
 }
 
 
-/* -------------------------------- IFF Fibonacci and Exponential --------------------------------
- */
-
-static int fb[] = {-34, -21, -13, -8, -5, -3, -2, -1, 0, 1, 2, 3, 5, 8, 13, 21};
-static int ex[] = {-128, -64, -32, -16, -8, -4, -2, -1, 0, 1, 2, 4, 8, 16, 32, 64};
-
-static int read_iff(char *oldname, char *newname, int orig, char *hdr)
-{
-  int loc, chans, samps, totalin, i, j, fs = -1, fd = -1, f1, f2, val;
-  short *buf = NULL;
-  loc = mus_sound_data_location(oldname);
-  chans = mus_sound_chans(oldname);
-  samps = mus_sound_samples(oldname);
-  mus_bint_to_char((unsigned char *)(hdr + 16), mus_sound_srate(oldname));
-  mus_bint_to_char((unsigned char *)(hdr + 20), chans);
-  STARTUP(oldname, newname, TRANS_BUF_SIZE * 2, short);
-  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR)
-    {
-      CLEANUP();
-      RETURN_MUS_WRITE_ERROR(oldname, newname);
-    }
-  lseek(fd, loc, SEEK_SET);
-  val = 0;
-  while (samps > 0)
-    {
-      totalin = read(fd, hdr, TRANS_BUF_SIZE);
-      if (totalin <= 0) break;
-      for (i = 0, j = 0; i < totalin; i++, j += 2)
-	{
-	  f1 = ((unsigned char)hdr[i]) & 0xf;
-	  f2 = (((unsigned char)hdr[i]) >> 4) & 0xf;
-	  if (orig == 1)
-	    {
-	      val += fb[f1]; /* might want a shift << 8 here or something */
-	      buf[j] = val;
-	      val += fb[f2];
-	      buf[j + 1] = val;
-	    }
-	  else
-	    {
-	      val += ex[f1];
-	      buf[j] = val;
-	      val += ex[f2];
-	      buf[j + 1] = val;
-	    }
-	}
-      if (be_snd_checked_write(fs, (unsigned char *)buf, j * 2, newname) == MUS_ERROR)
-	{
-	  CLEANUP();
-	  RETURN_MUS_WRITE_ERROR(oldname, newname);
-	}
-      samps -= j;
-    }
-  CLEANUP();
-  return(MUS_NO_ERROR);
-}
-
-
-/*  -------------------------------- AVI --------------------------------
- *
- * data is squirreled away in ##wb data blocks somewhere within a LIST chunk
- * we have to mimic the header reader to find these guys one by one.
- * we assume we've got 16-bit linear little endian data here all in a single sequence of 'wb' blocks.
- * (in the 'rec' case, audio and video data can be interleaved, but that's too bad)
- */
-
-static int read_avi(char *oldname, char *newname, char *hdr)
-{
-  int totalin, fs = -1, fd = -1, cksize, num, happy;
-#if (!MUS_LITTLE_ENDIAN)
-  int i;
-  unsigned char *bb;
-#endif
-  short *buf = NULL;
-  unsigned char *hdrbuf;
-  mus_bint_to_char((unsigned char *)(hdr + 16), mus_sound_srate(oldname));
-  mus_bint_to_char((unsigned char *)(hdr + 20), mus_sound_chans(oldname));
-  STARTUP(oldname, newname, TRANS_BUF_SIZE, short);
-  if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR)
-    {
-      CLEANUP();
-      RETURN_MUS_WRITE_ERROR(oldname, newname);
-    }
-  hdrbuf = (unsigned char *)CALLOC(8, sizeof(unsigned char));
-  lseek(fd, mus_sound_data_location(oldname), SEEK_SET);
-  happy = 1;
-  while (happy)
-    {
-      totalin = read(fd, hdrbuf, 8);
-      if (totalin < 0) break;
-      cksize = mus_char_to_lint((unsigned char *)(hdrbuf + 4));
-      if ((hdrbuf[2] == 'w') && (hdrbuf[3] == 'b'))
-	{
-	  while (cksize > 0)
-	    {
-	      if (TRANS_BUF_SIZE * 2 > cksize)
-		num = cksize;
-	      else num = TRANS_BUF_SIZE * 2;
-	      totalin = read(fd, (unsigned char *)buf, num);
-	      if (totalin < 0) 
-		{
-		  happy = 0; 
-		  break;
-		}
-	      else 
-		{
-#if (!MUS_LITTLE_ENDIAN)
-		  bb = (unsigned char *)buf;
-		  for (i = 0; i < totalin / 2; i++, bb += 2) buf[i] = mus_char_to_lshort(bb);
-#endif		  
-		  if (be_snd_checked_write(fs, (unsigned char *)buf, totalin, newname) == MUS_ERROR) 
-		    {
-		      FREE(hdrbuf);
-		      CLEANUP();
-		      RETURN_MUS_WRITE_ERROR(oldname, newname);
-		    }
-		}
-	      cksize -= num;
-	    }
-	}
-      else break;
-    }
-  FREE(hdrbuf);
-  CLEANUP();
-  return(MUS_NO_ERROR);
-}
-
-
 /*  -------------------------------- G721 and G723 from Sun --------------------------------
  * code boiled down considerably here since I have no love of compression schemes.
  */
@@ -1306,14 +1138,10 @@ static void update(int	code_size, int y, int wi, int fi, int dq, int sr, int dqs
   if (state_ptr->yu < 544) state_ptr->yu = 544; else if (state_ptr->yu > 5120) state_ptr->yu = 5120;
   state_ptr->yl += state_ptr->yu + ((-state_ptr->yl) >> 6);
   if (tr == 1) {
+    int i;
     state_ptr->a[0] = 0;
     state_ptr->a[1] = 0;
-    state_ptr->b[0] = 0;
-    state_ptr->b[1] = 0;
-    state_ptr->b[2] = 0;
-    state_ptr->b[3] = 0;
-    state_ptr->b[4] = 0;
-    state_ptr->b[5] = 0;
+    for (i = 0; i < 6; i++) state_ptr->b[i] = 0;
   } else { 
     pks1 = pk0 ^ state_ptr->pk[0];		/* UPA2 */
     a2p = state_ptr->a[1] - (state_ptr->a[1] >> 7);
@@ -1371,7 +1199,7 @@ static void update(int	code_size, int y, int wi, int fi, int dq, int sr, int dqs
 	  state_ptr->sr[0] =  (exp << 6) + ((mag << 6) >> exp) - 0x400;
 	} 
       else
-	state_ptr->sr[0] = 0xFC20;
+	state_ptr->sr[0] = 0xFC20; /* it's declared short?? */
   state_ptr->pk[1] = state_ptr->pk[0];
   state_ptr->pk[0] = pk0;
   if (tr == 1) state_ptr->td = 0; else if (a2p < -11776) state_ptr->td = 1; else state_ptr->td = 0;
@@ -1449,7 +1277,7 @@ static int g723_40_decoder(int i, struct g72x_state *state_ptr)
   return (sr << 2);
 }
 
-static int unpack_input(FILE *fin, unsigned char *code, int bits)
+static bool unpack_input(FILE *fin, unsigned char *code, int bits)
 {
   static unsigned int in_buffer = 0;
   static int in_bits = 0;
@@ -1459,7 +1287,7 @@ static int unpack_input(FILE *fin, unsigned char *code, int bits)
       if (fread(&in_byte, sizeof(char), 1, fin) != 1) 
 	{
 	  *code = 0;
-	  return (-1);
+	  return(false);
 	}
       in_buffer |= (in_byte << in_bits);
       in_bits += 8;
@@ -1467,49 +1295,46 @@ static int unpack_input(FILE *fin, unsigned char *code, int bits)
   *code = in_buffer & ((1 << bits) - 1);
   in_buffer >>= bits;
   in_bits -= bits;
-  return (in_bits > 0);
+  return(true);
 }
 
-static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
+static int read_g72x_adpcm(const char *oldname, const char *newname, char *hdr, int which_g)
 {
-  int fs = -1, loc, j, chans, srate, dec_bits = 0, err = MUS_NO_ERROR;
+  int fs = -1, j, chans, srate, dec_bits = 0, err = MUS_NO_ERROR;
   FILE *fd;
+  off_t loc;
   unsigned char code;
   short *buf = NULL;
   struct g72x_state state;
   g72x_init_state(&state);
   chans = mus_sound_chans(oldname);
   if (chans != 1)
-    {
-      mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
-		"can't translate G72x file %s: chans: %d\n  [snd-trans.c[%d] %s]",
-		oldname, chans,
-		__LINE__, __FUNCTION__);
-      return(MUS_ERROR);
-    }
-  fs = creat(newname, 0666);
+    return(mus_error(MUS_UNSUPPORTED_DATA_FORMAT,
+		     _("read_g72x_adpcm: can't translate G72x file %s: chans: %d\n"),
+		     oldname, chans));
+  fs = CREAT(newname, 0666);
   if (fs == -1) RETURN_MUS_IO_ERROR("create", newname);
   loc = mus_sound_data_location(oldname);
   srate = mus_sound_srate(oldname);
   mus_bint_to_char((unsigned char *)(hdr + 16), srate);
   mus_bint_to_char((unsigned char *)(hdr + 20), chans);
-  fd = fopen(oldname, "rb");
+  fd = FOPEN(oldname, "rb");
   if (fd == NULL) 
     {
-      close(fs); 
+      snd_close(fs, newname); 
       RETURN_MUS_IO_ERROR("fopen", oldname);
     }
   if (snd_checked_write(fs, (unsigned char *)hdr, 28, newname) == MUS_ERROR) 
     {
-      close(fs); 
-      fclose(fd);
+      snd_close(fs, newname); 
+      snd_fclose(fd, oldname);
       RETURN_MUS_WRITE_ERROR(oldname, newname);
     }
   buf = (short *)CALLOC(TRANS_BUF_SIZE, sizeof(short));
   if (buf == NULL) 
     {
-      close(fs); 
-      fclose(fd); 
+      snd_close(fs, newname); 
+      snd_fclose(fd, oldname); 
       RETURN_MUS_ALLOC_ERROR(oldname, TRANS_BUF_SIZE, "buf");
     }
   fread(buf, 1, loc, fd);
@@ -1520,7 +1345,7 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
     case 2: /* G723_40 */ dec_bits = 5; break;
     }
   j = 0;
-  while (unpack_input(fd, &code, dec_bits) >= 0)
+  while (unpack_input(fd, &code, dec_bits))
     {
       switch (which_g)
 	{
@@ -1532,8 +1357,8 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
 	{
 	  if (be_snd_checked_write(fs, (unsigned char *)buf, j * 2, newname) == MUS_ERROR) 
 	    {
-	      close(fs); 
-	      fclose(fd); 
+	      snd_close(fs, newname); 
+	      snd_fclose(fd, oldname); 
 	      FREE(buf); 
 	      RETURN_MUS_WRITE_ERROR(oldname, newname);
 	    }
@@ -1541,8 +1366,8 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
 	}
     }
   if (j > 0) err = be_snd_checked_write(fs, (unsigned char *)buf, j * 2, newname);
-  close(fs);
-  fclose(fd);
+  snd_close(fs, newname);
+  snd_fclose(fd, oldname);
   FREE(buf);
   if (err == MUS_ERROR) RETURN_MUS_WRITE_ERROR(oldname, newname);
   return(MUS_NO_ERROR);
@@ -1552,7 +1377,6 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
 
 /* -------------------------------- TRANSLATE -------------------------------- */
 
-#define RIFF_IBM_CVSD 5
 #define RIFF_Intel_ADPCM 0x11
 #define RIFF_Oki_ADPCM 0x10
 #define RIFF_G721 0x40
@@ -1564,6 +1388,7 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
 #define RIFF_G721 0x40
 #define RIFF_MPEG 0x50
 #define RIFF_MS_ADPCM 2
+#define RIFF_IBM_ADPCM 0x103
 #define NeXT_G721 23
 #define NeXT_G722 24
 #define NeXT_G723 25
@@ -1571,23 +1396,23 @@ static int read_g72x_adpcm(char *oldname, char *newname, char *hdr, int which_g)
 
 static int MUS_CANT_TRANSLATE = 0;
 
-static const char *any_format_name(char *name)
+static const char *any_format_name(const char *name)
 {
   int format;
   format = mus_sound_data_format(name);
-  if (format != MUS_UNSUPPORTED)
+  if (format != MUS_UNKNOWN)
     return(mus_data_format_name(format));
   else return(mus_header_original_format_name(mus_sound_original_format(name),
 					      mus_sound_header_type(name)));
 }
 
-int snd_translate(char *oldname, char *newname, int type)
+int snd_translate(const char *oldname, const char *newname, int type)
 {
   /* read oldname, translate to newname as 16-bit linear NeXT file */
   /* called from snd-file.c */
-  int orig, err;
+  int err;
   char *hdr = NULL;
-  if (MUS_CANT_TRANSLATE == 0) MUS_CANT_TRANSLATE = mus_make_error("mus_cant_translate");
+  if (MUS_CANT_TRANSLATE == 0) MUS_CANT_TRANSLATE = mus_make_error("can't translate");
   err = MUS_CANT_TRANSLATE;
   hdr = (char *)CALLOC(TRANS_BUF_SIZE, sizeof(char));
   /* set up default output header */
@@ -1604,12 +1429,12 @@ int snd_translate(char *oldname, char *newname, int type)
     case MUS_MUS10: err = read_mus10(oldname, newname, hdr); break;
     case MUS_HCOM: err = read_hcom(oldname, newname, hdr); break;
     case MUS_YAMAHA_TX16W: err = read_12bit(oldname, newname, hdr); break;
-    case MUS_AVI: err = read_avi(oldname, newname, hdr); break;
+    case MUS_NVF: err = read_g72x_adpcm(oldname, newname, hdr, 0); break;
     case MUS_RIFF:
       switch (mus_sound_original_format(oldname))
 	{
-	case RIFF_IBM_CVSD: err = read_ibm_cvsd(oldname, newname, hdr); break;
 	case RIFF_MS_ADPCM: case RIFF_Intel_ADPCM: err = read_dvi_adpcm(oldname, newname, hdr, 0); break;
+	case RIFF_IBM_ADPCM: err = read_ibm_adpcm(oldname, newname, hdr); break;
 	case RIFF_Oki_ADPCM: err = read_oki_adpcm(oldname, newname, hdr); break;
 	case RIFF_G721: err = read_g72x_adpcm(oldname, newname, hdr, 0); break; /* untested */
 	case RIFF_G723: case RIFF_MS_G723: case RIFF_Lucent_G723: case RIFF_Vivo_G723: /* untested */
@@ -1625,11 +1450,6 @@ int snd_translate(char *oldname, char *newname, int type)
       if (mus_sound_original_format(oldname) == MUS_NIST_SHORTPACK) 
 	err = read_nist_shortpack(oldname, newname, hdr); 
       break;
-    case MUS_SVX:
-      orig = mus_sound_original_format(oldname);
-      if ((orig == 1) || (orig == 2)) 
-	err = read_iff(oldname, newname, orig, hdr);
-      break;
     case MUS_NEXT:
       switch (mus_sound_original_format(oldname))
 	{
@@ -1648,15 +1468,20 @@ int snd_translate(char *oldname, char *newname, int type)
     }
   FREE(hdr);
   if (err == MUS_CANT_TRANSLATE) /* i.e a case we don't even try to handle */
-    mus_error(MUS_CANT_TRANSLATE,
-	      "can't translate %s\n  (%s header: %s (0x%x) data format)\n  [snd-trans.c[%d] %s]",
-	      oldname,
-	      mus_header_type_name(type),
-	      any_format_name(oldname),
-	      mus_sound_original_format(oldname),
-	      __LINE__, __FUNCTION__);
+    {
+      if (ss->catch_exists)
+	return(mus_error(MUS_CANT_TRANSLATE,
+			 _("can't translate %s\n  (%s header: %s (0x%x) data format)\n"),
+			 oldname,
+			 mus_header_type_name(type),
+			 any_format_name(oldname),
+			 mus_sound_original_format(oldname)));
+      else 
+	snd_error(_("can't translate %s\n  (%s header: %s (0x%x) data format)\n"),
+		     oldname,
+		     mus_header_type_name(type),
+		     any_format_name(oldname),
+		     mus_sound_original_format(oldname));
+    }
   return(err);
 }
-
-
-
