@@ -44,12 +44,12 @@ int snd_2pow2(int n)
 
 Float in_dB(Float min_dB, Float lin_dB, Float py)
 {
-  return((py <= lin_dB) ? min_dB : (20.0 * (log10(py))));
+  return((py <= lin_dB) ? min_dB : (20.0 * log10(py)));
 }
 
 char *copy_string(const char *str)
 {
-#if DEBUG_MEMORY
+#if DEBUGGING
   char *newstr = NULL;
   if (str)
     {
@@ -296,13 +296,7 @@ void snd_exit(int val)
 #endif
 }
 
-#if __GNUC__
-#if WITH_EFENCE && (!DEBUG_MEMORY)
-  #warning compile switches are inconsistent (with_efence needs debug_memory)
-#endif
-#endif
-
-#ifdef DEBUG_MEMORY
+#if DEBUGGING
 
 /* mtrace-style malloc hooks are not very useful here since I don't care
  * about X allocations (of which there are millions), and I need readable
@@ -330,8 +324,9 @@ static char *kmg (int num)
   return(str);
 }
 
+#define MEM_PAD_SIZE 32
 static int mem_size = 0, mem_top = -1;
-static int *pointers = NULL, *sizes = NULL, *locations = NULL;
+static int *pointers = NULL, *true_pointers = NULL, *sizes = NULL, *locations = NULL;
 static char **functions = NULL, **files = NULL;
 static int *lines = NULL;
 static int mem_location = -1;
@@ -401,35 +396,101 @@ static int find_mem_location(const char *func, const char *file, int line)
   return(mem_location);
 }
 
+static void describe_pointer(void *p)
+{
+  int i, loc;
+  for (i = mem_top; i >= 0; i--)
+    if (pointers[i] == (int)p)
+      {
+	loc = locations[i];
+	fprintf(stderr, "%s[%d]:%s, len:%d", files[loc], lines[loc], functions[loc], sizes[loc]);
+	return;
+      }
+  fprintf(stderr, "pointer lost??");
+}
+
+static char pad[MEM_PAD_SIZE] = {'W','I','L','L','I','A','M',' ','G','A','R','D','N','E','R',' ','S','C','H','O','T','T','S','T','A','E','D','T',' ','D','M','A'};
+static void set_padding(void *p1, void *p2, int len)
+{
+  char *ip2;
+  ip2 = (char *)p2;
+  memcpy((void *)ip2, (void *)pad, MEM_PAD_SIZE);
+  memcpy((void *)(ip2 + len + MEM_PAD_SIZE), (void *)pad, MEM_PAD_SIZE);
+}
+
+static void check_padding(void *p1, void *p2, int len, bool refill)
+{
+  int i, j;
+  char *ip2;
+  ip2 = (char *)p2;
+  for (i = 0; i < MEM_PAD_SIZE; i++) 
+    if (ip2[i] != pad[i])
+      {
+	fprintf(stderr, "memory clobbered %d bytes before %p (", MEM_PAD_SIZE - i, p1);
+	describe_pointer(p1);
+	fprintf(stderr,")\n");
+	abort();
+      }
+  if (refill)
+    memset((void *)(ip2 + MEM_PAD_SIZE), 'X', len);
+  for (i = 0, j = len + MEM_PAD_SIZE; i < MEM_PAD_SIZE; i++, j++) 
+    if (ip2[j] != pad[i])
+      {
+	fprintf(stderr, "memory clobbered %d bytes after %p (", i, p1);
+	describe_pointer(p1);
+	fprintf(stderr,")\n");
+	abort();
+      }
+}
+
+#define PTR_HISTORY_SIZE 128
+static int ptr_history[PTR_HISTORY_SIZE], ptr_history_loc[PTR_HISTORY_SIZE];
+static int ptr_history_ptr = 0;
+
 void mem_report(void);
 static int last_forgotten = -1;
-static int last_remembered = -1, last_remembered_ptr = -1;
-static void forget_pointer(void *ptr, const char *func, const char *file, int line)
+#define FREED_POINTER 0x99999999
+
+static void *forget_pointer(void *ptr, const char *func, const char *file, int line, bool refill)
 {
   int i;
+  void *rtp;
   if (ptr == NULL) {fprintf(stderr, "attempt to free NULL"); mem_report(); abort();}
-  if (last_remembered_ptr == (int)ptr)
-    {
-      pointers[last_remembered] = 0;
-      stacks[last_remembered] = NULL;
-      last_forgotten = last_remembered;
-      last_remembered_ptr = -1;
-      return;
-    }
+  if (ptr == (void *)FREED_POINTER) {fprintf(stderr," attempt to free pointer twice"); abort();}
+  for (i = 0; i < PTR_HISTORY_SIZE; i++)
+    if (ptr_history[i] == (int)ptr)
+      {
+	int last_remembered_ptr, last_remembered;
+	last_remembered_ptr = ptr_history[i];
+	last_remembered = ptr_history_loc[i];
+	ptr_history[i] = 0;
+	ptr_history_loc[i] = -1;
+	rtp = (void *)true_pointers[last_remembered];
+	check_padding(ptr, rtp, sizes[last_remembered], refill);
+	pointers[last_remembered] = 0;
+	true_pointers[last_remembered] = 0;
+	stacks[last_remembered] = NULL;
+	last_forgotten = last_remembered;
+	return(rtp);
+      }
   for (i = mem_top; i >= 0; i--)
     if (pointers[i] == (int)ptr)
       {
+	rtp = (void *)true_pointers[i];
+	check_padding(ptr, rtp, sizes[i], refill);
 	pointers[i] = 0;
+	true_pointers[i] = 0;
 	stacks[i] = NULL;
 	last_forgotten = i;
-	if (i == mem_top)
-	  while ((mem_top >= 0) && (pointers[mem_top] == 0)) mem_top--;
-	return;
+	while ((mem_top >= 0) && (pointers[mem_top] == 0)) mem_top--;
+	if (mem_top < last_forgotten) last_forgotten = mem_top + 1;
+	return(rtp);
       }
   fprintf(stderr, "forget %p ", ptr); mem_report(); abort();
+  return(NULL);
 }
 
-static void remember_pointer(void *ptr, size_t len, const char *func, const char *file, int line)
+static void remember_pointer(void *ptr, void *true_ptr, size_t len, const char *func, const char *file, int line)
 {
   int i, loc = 0;
   if (last_forgotten == -1)
@@ -438,19 +499,13 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
 	{
 	  mem_size = 8192;
 	  pointers = (int *)calloc(mem_size, sizeof(int));
+	  true_pointers = (int *)calloc(mem_size, sizeof(int));
 	  sizes = (int *)calloc(mem_size, sizeof(int));
 	  locations = (int *)calloc(mem_size, sizeof(int));
 	  stacks = (char **)calloc(mem_size, sizeof(char *));
 	  loc = 0;
 	  goto GOT_ONE;
 	}
-      if (last_remembered != -1)
-	for (i = last_remembered + 1; i < mem_size; i++)
-	  if (pointers[i] == 0) 
-	    {
-	      loc = i;
-	      goto GOT_ONE;
-	    }
       for (i = 0; i < mem_size; i++)
 	if (pointers[i] == 0) 
 	  {
@@ -460,12 +515,14 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
       loc = mem_size;
       mem_size += 4096;
       pointers = (int *)realloc(pointers, mem_size * sizeof(int));
+      true_pointers = (int *)realloc(true_pointers, mem_size * sizeof(int));
       sizes = (int *)realloc(sizes, mem_size * sizeof(int));
       locations = (int *)realloc(locations, mem_size * sizeof(int));
       stacks = (char **)realloc(stacks, mem_size * sizeof(char *));
       for (i = loc; i < mem_size; i++)
 	{
 	  pointers[i] = 0;
+	  true_pointers[i] = 0;
 	  sizes[i] = 0;
 	  locations[i] = 0;
 	  stacks[i] = NULL;
@@ -478,87 +535,73 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
     }
  GOT_ONE:
   pointers[loc] = (int)ptr;
+  true_pointers[loc] = (int)true_ptr;
+  set_padding(ptr, true_ptr, (int)len);
   sizes[loc] = (int)len;
   locations[loc] = find_mem_location(func, file, line);
   if (encloser) stacks[loc] = encloser;
-  last_remembered = loc;
-  last_remembered_ptr = (int)ptr;
+  ptr_history[ptr_history_ptr] = (int)ptr;
+  ptr_history_loc[ptr_history_ptr++] = loc;
+  if (ptr_history_ptr >= PTR_HISTORY_SIZE) ptr_history_ptr = 0;
   if (mem_top < loc) mem_top = loc;
 }
 
 #define MAX_MALLOC (1 << 28)
 
-#if WITH_EFENCE
-void *ef_calloc(size_t a, size_t b);
-void *ef_malloc(size_t a);
-void *ef_free(void *a);
-void *ef_realloc(void *a, size_t b);
-#endif
-
-void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line)
+void *mem_calloc(int len, int size, const char *func, const char *file, int line)
 {
-  void *ptr;
-  if ((len == 0) || (len > MAX_MALLOC))
+  char *ptr, *true_ptr;
+  if ((len <= 0) || ((len * size) > MAX_MALLOC) || (size <= 0))
     {
       fprintf(stderr, "%s:%s[%d] attempt to calloc %d bytes", func, file, line, len * size);
       mem_report(); abort();
     }
-#if WITH_EFENCE
-  ptr = ef_calloc(len, size);
-#else
-  ptr = calloc(len, size);
-#endif
+  true_ptr = (char *)malloc(len * size + 2 * MEM_PAD_SIZE);
+  memset(true_ptr, 0, len * size + 2 * MEM_PAD_SIZE);
+  ptr = (char *)(true_ptr + MEM_PAD_SIZE);
   if (ptr == NULL) {fprintf(stderr,"calloc->null"); abort();}
-  remember_pointer(ptr, len * size, func, file, line);
-  return(ptr);
+  remember_pointer((void *)ptr, (void *)true_ptr, len * size, func, file, line);
+  return((void *)ptr);
 }
 
-void *mem_malloc(size_t len, const char *func, const char *file, int line)
+void *mem_malloc(int len, const char *func, const char *file, int line)
 {
-  void *ptr;
-  if ((len == 0) || (len > MAX_MALLOC))
+  char *ptr, *true_ptr;
+  if ((len <= 0) || (len > MAX_MALLOC))
     {
       fprintf(stderr, "%s:%s[%d] attempt to malloc %d bytes", func, file, line, len);
       mem_report(); abort();
     }
-#if WITH_EFENCE
-  ptr = ef_malloc(len);
-#else
-  ptr = malloc(len);
-#endif
+  true_ptr = (char *)malloc(len + 2 * MEM_PAD_SIZE);
+  ptr = (char *)(true_ptr + MEM_PAD_SIZE);
   if (ptr == NULL) {fprintf(stderr,"malloc->null"); abort();}
-  remember_pointer(ptr, len, func, file, line);
-  return(ptr);
+  remember_pointer((void *)ptr, (void *)true_ptr, len, func, file, line);
+  return((void *)ptr);
 }
 
 void mem_free(void *ptr, const char *func, const char *file, int line)
 {
+  void *true_ptr;
   /* fprintf(stderr,"free %s %s[%d]: %p\n", func, file, line, ptr); */
-  forget_pointer(ptr, func, file, line);
-#if WITH_EFENCE
-  ef_free(ptr);
-#else
-  free(ptr);
-#endif
+  true_ptr = forget_pointer(ptr, func, file, line, true);
+  free(true_ptr);
+  ptr = (void *)FREED_POINTER;
 }
 
-void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line)
+void *mem_realloc(void *ptr, int size, const char *func, const char *file, int line)
 {
-  void *new_ptr;
-  if ((size == 0) || (size > MAX_MALLOC))
+  char *new_ptr, *true_ptr, *new_true_ptr;
+  if ((size <= 0) || (size > MAX_MALLOC))
     {
       fprintf(stderr, "%s:%s[%d] attempt to realloc %d bytes", func, file, line, size);
       mem_report(); abort();
     }
-  forget_pointer(ptr, func, file, line);
-#if WITH_EFENCE
-  new_ptr = ef_realloc(ptr, size);
-#else
-  new_ptr = realloc(ptr, size);
-#endif
+  true_ptr = (char *)forget_pointer(ptr, func, file, line, false);
+  new_true_ptr = (char *)realloc((void *)true_ptr, size + 2 * MEM_PAD_SIZE);
+  new_ptr = (char *)(new_true_ptr + MEM_PAD_SIZE);
   if (new_ptr == NULL) {fprintf(stderr,"realloc->null"); abort();}
-  remember_pointer(new_ptr, size, func, file, line);
-  return(new_ptr);
+  remember_pointer((void *)new_ptr, (void *)new_true_ptr, size, func, file, line);
+  return((void *)new_ptr);
 }
 
 static char *mem_stats(int ub)
@@ -771,45 +814,7 @@ void mem_report(void)
 	  }
       if (sum > 0)
 	{
-	  fprintf(Fp, "%s[%d]:%s:  %d (%d)", files[ptr], lines[ptr], functions[ptr], sums[ptr], ptrs[ptr]);
-	  if (0)
-	    {
-	      int fd, k, line, bytes;
-	      bool happy = true;
-	      char buf[8192];
-	      fd = open(files[ptr], O_RDONLY, 0);
-	      line = 1;
-	      while (happy)
-		{
-		  bytes = read(fd, buf, 8192);
-		  if (bytes <= 0) 
-		    {
-		      fprintf(Fp, "where is %s[%d]?\n", files[ptr], lines[ptr]);
-		      happy = false;
-		      break;
-		    }
-		  for (k = 0; k < bytes; k++)
-		    {
-		      if (buf[k] == '\n')
-			{
-			  line++;
-			  if (line == lines[ptr]) fprintf(Fp, ":   ");
-			  if (line > lines[ptr]) 
-			    {
-			      fprintf(Fp, "\n");
-			      happy = false;
-			      break;
-			    }
-			}
-		      else
-			{
-			  if (line == lines[ptr]) fprintf(Fp, "%c", buf[k]);
-			}
-		    }
-		}
-	      close(fd);
-	    }
-	  else fprintf(Fp, "\n");
+	  fprintf(Fp, "%s[%d]:%s:  %d (%d)\n", files[ptr], lines[ptr], functions[ptr], sums[ptr], ptrs[ptr]);
 	  sums[ptr] = 0;
 	  if (have_stacks)
 	    for (j = 0; j < mem_size; j++)
@@ -818,6 +823,8 @@ void mem_report(void)
 	  if ((strcmp("mus_format", functions[ptr]) == 0) ||
 	      (strcmp("copy_string", functions[ptr]) == 0))
 	    {
+	      if (have_stacks)
+		fprintf(Fp, "                          ");
 	      for (j = 0; j < mem_size; j++)
 		if ((locations[j] == ptr) && (pointers[j]))
 		  fprintf(Fp, "[%s] ", (char *)(pointers[j]));
@@ -884,10 +891,30 @@ static XEN g_file_to_string(XEN name)
 }
 #endif
 
+#if DEBUGGING
+static XEN g_mem_report(void) 
+{
+  mem_report(); 
+  return(XEN_FALSE);
+}
+#endif
+
+#if DEBUGGING
+#ifdef XEN_ARGIFY_1
+  XEN_NARGIFY_0(g_mem_report_w, g_mem_report)
+#else
+  #define g_mem_report_w g_mem_report
+#endif
+#endif
+
 void g_init_utils(void)
 {
   decimal_pt = local_decimal_point();
 #if HAVE_GUILE
   XEN_DEFINE_PROCEDURE(S_file2string, g_file_to_string, 1, 0, 0, "file contents as string");
 #endif
+
+#if DEBUGGING
+  XEN_DEFINE_PROCEDURE("mem-report",   g_mem_report_w, 0, 0, 0, "(mem-report) writes memory usage stats to memlog");
+#endif
 }
