@@ -186,6 +186,7 @@ void check_menu_labels(int key, int state, bool extended);
 
 void snd_doit(int argc, char **argv);
 color_t get_in_between_color(color_t fg, color_t bg);
+void auto_update_restart(void);
 
 #ifdef SND_AS_WIDGET
   void snd_as_widget(int argc, char **argv, XtAppContext app, Widget parent, Arg *caller_args, int caller_argn);
@@ -215,6 +216,9 @@ void set_fft_log_frequency(bool val);
 void set_transform_normalization(fft_normalize_t val);
 void set_show_selection_transform(bool show);
 void set_transform_graph_type(graph_type_t val);
+void reflect_peaks_in_transform_dialog(void);
+void reflect_log_freq_start_in_transform_dialog(void);
+void reflect_min_db_in_transform_dialog(void);
 
 
 /* -------- snd-xdrop.c -------- */
@@ -329,6 +333,9 @@ void fixup_axis_context(axis_context *ax, Widget w, GC gc);
 Pixmap make_pixmap(unsigned char *bits, int width, int height, int depth, GC gc);
 Cessator add_work_proc(XtWorkProc func, XtPointer data);
 int attach_all_sides(Arg *args, int n);
+void widget_int_to_text(Widget w, int val);
+void widget_float_to_text(Widget w, Float val);
+void widget_off_t_to_text(Widget w, off_t val);
 
 
 /* -------- snd-xchn.c -------- */
@@ -410,9 +417,6 @@ void filter_env_changed(snd_info *sp, env *e);
 void set_play_button(snd_info *sp, bool val);
 void play_button_pause(bool pausing);
 void syncb(snd_info *sp, int on);
-void lock_apply(snd_info *sp);
-void unlock_apply(snd_info *sp);
-void set_apply_button(snd_info *sp, bool val);
 void snd_file_lock_icon(snd_info *sp, bool on);
 void snd_file_bomb_icon(snd_info *sp, bool on);
 void x_bomb(snd_info *sp, bool on);
@@ -435,6 +439,7 @@ void progress_report(snd_info *sp, const char *funcname, int curchan, int chans,
 XmString initial_speed_label(speed_style_t style);
 void g_init_gxsnd(void);
 void make_sound_icons_transparent_again(Pixel old_color, Pixel new_color);
+void reflect_sound_selection(snd_info *sp);
 
 
 /* -------- snd-xfile.c -------- */
@@ -476,8 +481,8 @@ void reflect_just_sounds_state(void);
 /* -------- snd-xenv.c -------- */
 
 axis_info *enved_make_axis(const char *name, axis_context *ax, int ex0, int ey0, int width, int height, 
-			   Float xmin, Float xmax, Float ymin, Float ymax, bool printing);
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing);
+			   Float xmin, Float xmax, Float ymin, Float ymax, printing_t printing);
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing);
 void set_enved_redo_sensitive(bool val);
 void set_enved_revert_sensitive(bool val);
 void set_enved_undo_sensitive(bool val);
