@@ -35,9 +35,13 @@ bool channel_graph_is_visible(chan_info *cp)
 	 (channel_graph(cp)) &&
 	 (XtIsManaged(channel_graph(cp))) &&
 	 (cp->sound) &&
-	 (cp->sound->sgx) &&
-	 (w_snd_pane(cp->sound)) &&
-	 (XtIsManaged(w_snd_pane(cp->sound))));
+	 /* here we may have a sound wrapper for variable display in which case the sound widgets are not implemented */
+	 (((cp->sound->inuse == SOUND_WRAPPER) || (cp->sound->inuse == SOUND_REGION)) ||
+	  ((cp->sound->inuse == SOUND_NORMAL) &&
+	   /* other choice: SOUND_IDLE -> no display */
+	   (cp->sound->sgx) &&
+	   (w_snd_pane(cp->sound)) &&
+	   (XtIsManaged(w_snd_pane(cp->sound))))));
 }
 
 #define EDIT_HISTORY_LIST(Cp) (Cp->cgx)->chan_widgets[W_edhist]
@@ -157,8 +161,8 @@ static void gsy_changed(int value, chan_info *cp)
 void fixup_gsy(chan_info *cp, Float low, Float high)
 {
   Widget wcp;
-  int ival;
   Float val, size;
+  int ival = 0;
   wcp = channel_gsy(cp);
   XtVaGetValues(wcp, XmNvalue, &ival, NULL);
   val = (Float)ival / (Float)(SCROLLBAR_MAX);
@@ -476,8 +480,8 @@ static void graph_mouse_enter(Widget w, XtPointer context, XEvent *event, Boolea
   XtVaGetValues(w, XmNuserData, &data, NULL);
   if (XEN_HOOKED(mouse_enter_graph_hook))
     run_hook(mouse_enter_graph_hook,
-	     XEN_LIST_2(C_TO_SMALL_XEN_INT(UNPACK_SOUND(data)),
-			C_TO_SMALL_XEN_INT(UNPACK_CHANNEL(data))),
+	     XEN_LIST_2(C_TO_XEN_INT(UNPACK_SOUND(data)),
+			C_TO_XEN_INT(UNPACK_CHANNEL(data))),
 	     S_mouse_enter_graph_hook);
   XDefineCursor(XtDisplay(w), XtWindow(w), (ss->sgx)->graph_cursor);
 }
@@ -488,8 +492,8 @@ static void graph_mouse_leave(Widget w, XtPointer context, XEvent *event, Boolea
   XtVaGetValues(w, XmNuserData, &data, NULL);
   if (XEN_HOOKED(mouse_leave_graph_hook))
     run_hook(mouse_leave_graph_hook,
-	     XEN_LIST_2(C_TO_SMALL_XEN_INT(UNPACK_SOUND(data)),
-			C_TO_SMALL_XEN_INT(UNPACK_CHANNEL(data))),
+	     XEN_LIST_2(C_TO_XEN_INT(UNPACK_SOUND(data)),
+			C_TO_XEN_INT(UNPACK_CHANNEL(data))),
 	     S_mouse_leave_graph_hook);
   XUndefineCursor(XtDisplay(w), XtWindow(w));
 }
@@ -568,13 +572,13 @@ static void history_select_callback(Widget w, XtPointer context, XtPointer info)
 static void remake_edit_history(Widget lst, chan_info *cp, int from_graph)
 {
   snd_info *sp;
-  chan_info *ncp;
-  int i, eds, items = 0;
+  int i, eds;
   XmString *edits;
   XmListDeleteAllItems(lst);
   sp = cp->sound;
   if (sp->channel_style != CHANNELS_SEPARATE)
     {
+      chan_info *ncp;
       int k, all_eds = 0, ed, filelen;
       char *title;
       for (k = 0; k < sp->nchans; k++)
@@ -614,6 +618,7 @@ static void remake_edit_history(Widget lst, chan_info *cp, int from_graph)
     }
   else
     {
+      int items = 0;
       eds = cp->edit_ctr;
       while ((eds < (cp->edit_size - 1)) && (cp->edits[eds + 1])) eds++;
       edits = (XmString *)CALLOC(eds + 1, sizeof(XmString));
@@ -665,9 +670,6 @@ void reflect_edit_history_change(chan_info *cp)
   if ((cp->in_as_one_edit) || (cp->cgx == NULL)) return;
   sp = cp->sound;
   lst = EDIT_HISTORY_LIST(cp);
-  /*
-  fprintf(stderr,"%s[%d:%d]: %s\n", cp->sound->short_filename, cp->chan, cp->edit_ctr, cp->edits[cp->edit_ctr]->origin);
-  */
 #if WITH_RELATIVE_PANES && (XmVERSION > 1)
   if ((lst) && (widget_width(lst) > 1))
     remake_edit_history(lst, cp, true);
@@ -732,11 +734,11 @@ void reflect_edit_counter_change(chan_info *cp)
   /* undo/redo/revert -- change which line is highlighted */
 #if (XmVERSION > 1)
   Widget lst;
-  int len, top;
   if (cp->cgx == NULL) return;
   lst = EDIT_HISTORY_LIST(cp);
   if ((lst) && (widget_width(lst) > 1))
     {
+      int len, top;
       XmListSelectPos(lst, cp->edit_ctr + 1, false);
       XtVaGetValues(lst, 
 		    XmNvisibleItemCount, &len, 
@@ -800,19 +802,17 @@ static void cp_graph_key_press(Widget w, XtPointer context, XEvent *event, Boole
 
 int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Widget main, fw_button_t button_style, bool with_events)
 {
+  int i;
+  bool need_extra_scrollbars;
   Widget *cw;
-  XtCallbackList n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14;
   chan_info *cp;
   chan_context *cx;
   axis_context *cax;
   state_context *sx;
   bool make_widgets;
-  int i, n;
-  bool need_colors, need_extra_scrollbars;
-  Arg args[32];
   make_widgets = ((sp->chans[channel]) == NULL);
   sp->chans[channel] = make_chan_info(sp->chans[channel], channel, sp);
-  if ((main) && (!(XmIsForm(main)))) return(-1);
+  if ((main) && ((!(XmIsForm(main))) || (!(XtWindow(main))))) return(-1);
   cp = sp->chans[channel];
   cx = cp->cgx;
   if (cx->chan_widgets == NULL) 
@@ -823,6 +823,11 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Wid
 
   if (make_widgets)
     {
+      XtCallbackList n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14;
+      int n;
+      bool need_colors;
+      Arg args[32];
+
       /* allocate the entire widget apparatus for this channel of this sound */
       need_colors = (!(ss->using_schemes));
 
@@ -1043,19 +1048,22 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Wid
 
       if (with_events)
 	{
+	  /* region regraph sets up its own callbacks */
 	  XtAddCallback(cw[W_graph], XmNresizeCallback, channel_resize_callback, (XtPointer)cp);
 	  XtAddCallback(cw[W_graph], XmNexposeCallback, channel_expose_callback, (XtPointer)cp);
 	}
+      /* allow cursor in all cases (zoom to cursor in region window for example, or fft axis drag in variable display) */
+      XtAddEventHandler(cw[W_graph], ButtonPressMask, false, graph_button_press, (XtPointer)cp);
+      XtAddEventHandler(cw[W_graph], ButtonReleaseMask, false, graph_button_release, (XtPointer)cp);
+      XtAddEventHandler(cw[W_graph], ButtonMotionMask, false, graph_button_motion, (XtPointer)cp);
       if (main == NULL)
 	{
 	  XtAddEventHandler(cw[W_graph], EnterWindowMask, false, graph_mouse_enter, NULL);
 	  XtAddEventHandler(cw[W_graph], LeaveWindowMask, false, graph_mouse_leave, (XtPointer)cp);
-	  XtAddEventHandler(cw[W_graph], ButtonPressMask, false, graph_button_press, (XtPointer)cp);
-	  XtAddEventHandler(cw[W_graph], ButtonMotionMask, false, graph_button_motion, (XtPointer)cp);
-	  XtAddEventHandler(cw[W_graph], ButtonReleaseMask, false, graph_button_release, (XtPointer)cp);
 	  XtAddEventHandler(cw[W_graph], KeyPressMask, false, cp_graph_key_press, (XtPointer)cp);
 	  add_drop(cw[W_graph]);
 	}
+
       FREE(n1);
       FREE(n2);
       FREE(n3);
@@ -1143,6 +1151,9 @@ int add_channel_window(snd_info *sp, int channel, int chan_y, int insertion, Wid
     hide_gz_scrollbars(sp); /* default is on in this case */  
   cax = cx->ax;
   cax->wn = XtWindow(cw[W_graph]);
+#if DEBUGGING
+  if (!(cax->wn)) {fprintf(stderr, "graph widget %p's window is null?", cw[W_graph]); abort();}
+#endif
   cax->dp = XtDisplay(cw[W_graph]);
   cax->gc = sx->basic_gc;
   return(0);
@@ -1205,10 +1216,10 @@ GC erase_GC(chan_info *cp)
 
 void cleanup_cw(chan_info *cp)
 {
-  chan_context *cx;
-  Widget *cw;
   if ((cp) && (cp->cgx))
     {
+      chan_context *cx;
+      Widget *cw;
       cx = cp->cgx;
       cx->selected = false;
       cw = cx->chan_widgets;
@@ -1227,19 +1238,14 @@ void cleanup_cw(chan_info *cp)
 
 void change_channel_style(snd_info *sp, channel_style_t new_style)
 {
-  int i, j;
-  channel_style_t old_style;
-  chan_info *ncp, *cp, *pcp;
-  int height[1];
-  chan_context *mcgx;
-  Widget *cw;
-  axis_info *ap;
-  chan_context *cx;
   if ((sp) && (sp->nchans > 1))
     {
+      int i;
+      channel_style_t old_style;
       old_style = sp->channel_style;
       if (new_style != old_style)
 	{
+	  int height[1];
 	  sp->channel_style = new_style;
 #if WITH_RELATIVE_PANES && (XmVERSION > 1)
 	  if ((new_style == CHANNELS_SEPARATE) || (old_style == CHANNELS_SEPARATE))
@@ -1282,6 +1288,8 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 	  height[0] = widget_height(w_snd_pane(sp)) - control_panel_height(sp) - 16;
 	  if (old_style == CHANNELS_SEPARATE)
 	    {
+	      chan_context *mcgx;
+	      chan_info *ncp;
 	      ncp = sp->chans[0];
 	      sound_lock_control_panel(sp, NULL);
 	      channel_lock_pane(ncp, height);
@@ -1300,6 +1308,8 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 	    }
 	  else
 	    {
+	      axis_info *ap;
+	      chan_info *pcp;
 	      if (new_style == CHANNELS_SEPARATE)
 		{
 		  /* height[0] = total space available */
@@ -1315,6 +1325,10 @@ void change_channel_style(snd_info *sp, channel_style_t new_style)
 		  ap = pcp->axis;
 		  for (i = 1; i < sp->nchans; i++)
 		    {
+		      Widget *cw;
+		      chan_context *cx;
+		      chan_info *cp;
+		      int j;
 		      cp = sp->chans[i];
 		      cp->tcgx = NULL;
 		      cx = cp->cgx;
