@@ -31,7 +31,7 @@ static bool ignore_button_release = false;
 axis_info *enved_make_axis(const char *name, axis_context *ax, 
 			   int ex0, int ey0, int width, int height, 
 			   Float xmin, Float xmax, Float ymin, Float ymax,
-			   bool printing)
+			   printing_t printing)
 {
   /* conjure up minimal context for axis drawer in snd-axis.c */
   if (!axis) 
@@ -51,7 +51,7 @@ axis_info *enved_make_axis(const char *name, axis_context *ax,
   return(axis);
 }
 
-static void display_env(env *e, char *name, GC cur_gc, int x0, int y0, int width, int height, bool dots, bool printing)
+static void display_env(env *e, char *name, GC cur_gc, int x0, int y0, int width, int height, bool dots, printing_t printing)
 {
   axis_context *ax = NULL;  
   ax = (axis_context *)CALLOC(1, sizeof(axis_context));
@@ -63,7 +63,7 @@ static void display_env(env *e, char *name, GC cur_gc, int x0, int y0, int width
   ax = free_axis_context(ax);
 }
 
-void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, bool printing)
+void display_enved_env_with_selection(env *e, char *name, int x0, int y0, int width, int height, bool dots, printing_t printing)
 {
   display_env(e, name, (selected_env == e) ? rgc : gc, x0, y0, width, height, dots, printing);
 }
@@ -153,10 +153,11 @@ static bool within_selection_src = false;
 
 static void apply_enved(void)
 {
-  int i, j;
-  env *max_env = NULL;
+  char *origin = NULL, *estr = NULL;
   if (active_env)
     {
+      int i, j;
+      env *max_env = NULL;
       active_channel = current_channel();
       if (active_channel)
 	{
@@ -167,20 +168,32 @@ static void apply_enved(void)
 	  switch (enved_target(ss))
 	    {
 	    case ENVED_AMPLITUDE:
+	      origin = mus_format("%s %s%s", 
+				  (apply_to_selection) ? S_env_selection : S_env_channel,
+				  estr = env_to_string(active_env),
+				  (apply_to_selection) ? "" : " 0 #f");
 	      apply_env(active_channel, active_env, 0,
 			CURRENT_SAMPLES(active_channel), 
 			apply_to_selection, FROM_ENVED, 
-			"Enved: amp", NULL,
+			origin, NULL,
 			C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0);
 	      /* calls update_graph, I think, but in short files that doesn't update the amp-env */
+	      if (estr) FREE(estr);
+	      if (origin) FREE(origin);
 	      break;
 	    case ENVED_SPECTRUM: 
+	      origin = mus_format("%s %s %d%s",
+				  (apply_to_selection) ? S_filter_selection : S_filter_channel,
+				  estr = env_to_string(active_env), enved_filter_order(ss),
+				  (apply_to_selection) ? "" : " 0 #f");
 	      apply_filter(active_channel,
 			   (FIR_p) ? enved_filter_order(ss) : 0,
 			   active_env, FROM_ENVED, 
-			   "Enved: flt", apply_to_selection, 
+			   origin, NULL, apply_to_selection,
 			   NULL, NULL,
 			   C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 0, false);
+	      if (estr) FREE(estr);
+	      if (origin) FREE(origin);
 	      break;
 	    case ENVED_SRATE:
 	      /* mus_src no longer protects against 0 srate */
@@ -205,9 +218,8 @@ static void apply_enved(void)
     }
 }
 
-static void env_redisplay_1(bool printing)
+static void env_redisplay_1(printing_t printing)
 {
-  char *name = NULL;
   if (enved_dialog_is_active())
     {
       XClearWindow(XtDisplay(drawer), XtWindow(drawer));
@@ -215,6 +227,7 @@ static void env_redisplay_1(bool printing)
 	view_envs(env_window_width, env_window_height, printing);
       else 
 	{
+	  char *name = NULL;
 	  name = XmTextGetString(textL);
 	  if (!name) name = copy_string(_("noname"));
 	  /* active_env can be null here if just showing axes (empty initial graph) */
@@ -222,7 +235,7 @@ static void env_redisplay_1(bool printing)
 	  if (name) XtFree(name);
 	  if (enved_wave_p(ss))
 	    {
-	      if ((enved_target(ss) == ENVED_SPECTRUM) && (active_env) && (FIR_p) && (!printing))
+	      if ((enved_target(ss) == ENVED_SPECTRUM) && (active_env) && (FIR_p) && (printing == NOT_PRINTING))
 		display_frequency_response(active_env, axis, gray_ap->ax, enved_filter_order(ss), enved_in_dB(ss));
 	      enved_show_background_waveform(axis, gray_ap, apply_to_selection, (enved_target(ss) == ENVED_SPECTRUM), printing);
 	    }
@@ -230,8 +243,8 @@ static void env_redisplay_1(bool printing)
     }
 }
 
-void env_redisplay(void) {env_redisplay_1(false);}
-void env_redisplay_with_print(void) {env_redisplay_1(true);}
+void env_redisplay(void) {env_redisplay_1(NOT_PRINTING);}
+void env_redisplay_with_print(void) {env_redisplay_1(PRINTING);}
 
 static void enved_reset(void)
 {
@@ -261,10 +274,10 @@ static void order_field_activated(void)
 {
   /* return in order text field */
   char *str = NULL;
-  int order;
   str = XmTextGetString(orderL);
   if ((str) && (*str))
     {
+      int order;
       order = string_to_int(str);
       if (order & 1) order++;
       if ((order > 0) && 
@@ -277,10 +290,10 @@ static void order_field_activated(void)
 static void text_field_activated(void)
 { /* might be breakpoints to load or an envelope name (<cr> in enved text field) */
   char *name = NULL, *str;
-  env *e = NULL;
   name = XmTextGetString(textL);
   if ((name) && (*name))
     {
+      env *e = NULL;
       str = name;
       while (isspace((int)(*str))) str++;
       e = name_to_env(str);
@@ -513,9 +526,9 @@ static void selection_button_pressed(Widget s, XtPointer context, XtPointer info
 
 static void delete_button_pressed(Widget w, XtPointer context, XtPointer info) 
 {
-  int i, len;
   if (selected_env)
     {
+      int i, len;
       len = enved_all_envs_top();
       for (i = 0; i < len; i++)
 	if (selected_env == enved_all_envs(i))
@@ -779,17 +792,16 @@ static void FIR_click_callback(Widget w, XtPointer context, XtPointer info)
 
 Widget create_envelope_editor(void)
 {
-  int n;
-  Arg args[32];
-  Widget colE, colD, colB, colF;
-  Widget spacer, spacer1, aform, mainform, screnvname, baseSep, baseLabel;
-  XmString xhelp, xdismiss, xapply, titlestr, s1;
-  XGCValues gv;
-  XtCallbackList n1, n2;
-  char str[LABEL_BUFFER_SIZE];
-
   if (!enved_dialog)
     {
+      int n;
+      Arg args[32];
+      Widget colE, colD, colB, colF;
+      Widget spacer, spacer1, aform, mainform, screnvname, baseSep, baseLabel;
+      XmString xhelp, xdismiss, xapply, titlestr, s1;
+      XGCValues gv;
+      XtCallbackList n1, n2;
+      char str[LABEL_BUFFER_SIZE];
 
       /* -------- DIALOG -------- */
       xdismiss = XmStringCreate(_("Dismiss"), XmFONTLIST_DEFAULT_TAG);
@@ -1475,7 +1487,6 @@ bool enved_dialog_is_active(void)
 
 void set_enved_filter_order(int order)
 {
-  char str[LABEL_BUFFER_SIZE];
   if ((order > 0) && (order < 2000))
     {
       if (order & 1) 
@@ -1483,8 +1494,7 @@ void set_enved_filter_order(int order)
       else in_set_enved_filter_order(order);
       if (enved_dialog)
 	{
-	  mus_snprintf(str, LABEL_BUFFER_SIZE, "%d", enved_filter_order(ss));
-	  XmTextFieldSetString(orderL, str);
+	  widget_int_to_text(orderL, enved_filter_order(ss));
 	  if ((enved_dialog) && 
 	      (enved_target(ss) == ENVED_SPECTRUM) && 
 	      (enved_wave_p(ss)) && (!showing_all_envs)) 
