@@ -41,13 +41,17 @@ static void create_snd_error_dialog(bool popup)
   XmStringFree(titlestr);
   if (!(ss->using_schemes))
     {
-      XtVaSetValues(XtNameToWidget(snd_error_dialog, _("OK")), XmNarmColor, (ss->sgx)->pushed_button_color, NULL);
-      XtVaSetValues(XtNameToWidget(snd_error_dialog, _("OK")), XmNbackground, (ss->sgx)->quit_button_color, NULL);
+      /* the name "OK" is from Motif, so shouldn't be translated */
+      XtVaSetValues(XtNameToWidget(snd_error_dialog, "OK"), XmNarmColor, (ss->sgx)->pushed_button_color, NULL);
+      XtVaSetValues(XtNameToWidget(snd_error_dialog, "OK"), XmNbackground, (ss->sgx)->quit_button_color, NULL);
       XtVaSetValues(snd_error_history, XmNbackground, (ss->sgx)->white, XmNforeground, (ss->sgx)->black, NULL);
     }
   set_dialog_widget(ERROR_DIALOG, snd_error_dialog);
 }
 
+#define MAX_ERROR_HISTORY_LENGTH 32768
+/* I think XmTextPosition = Position = short */
+
 void add_to_error_history(char *msg, bool popup)
 {
 #if HAVE_STRFTIME
@@ -76,16 +80,19 @@ void add_to_error_history(char *msg, bool popup)
   FREE(tim);
 #endif
   pos = XmTextGetLastPosition(snd_error_history);
-  if (pos == 0) 
+  if ((pos == 0) || (pos > MAX_ERROR_HISTORY_LENGTH))
     XmTextSetString(snd_error_history, msg);
   else 
     {
       XmTextInsert(snd_error_history, pos, msg);
+#if 0
+      /* major memory leak here in Motif */
       if (XmGetVisibility(snd_error_history) != XmVISIBILITY_FULLY_OBSCURED)
 	{
 	  pos = XmTextGetLastPosition(snd_error_history);
 	  XmTextShowPosition(snd_error_history, pos - 1); /* if pos here, stupid thing segfaults! */
 	}
+#endif
     }
 }
 
@@ -121,9 +128,7 @@ static void no_callback(Widget w, XtPointer context, XtPointer info) {yes_or_no
 bool snd_yes_or_no_p(const char *format, ...)
 {
   static Widget yes_or_no_dialog = NULL;
-  Arg args[20];
-  int n;
-  XmString titlestr, error_msg, xmstr1, xmstr2;
+  XmString error_msg;
   char *yes_buf;
 #if HAVE_VPRINTF
   va_list ap;
@@ -146,6 +151,10 @@ bool snd_yes_or_no_p(const char *format, ...)
   yes_or_no = false;
   if (!yes_or_no_dialog)
     {
+      Arg args[20];
+      int n;
+      XmString titlestr, xmstr1, xmstr2;
+
       titlestr = XmStringCreate(_("Yow!"), XmFONTLIST_DEFAULT_TAG);
       xmstr1 = XmStringCreate(_("Yes"), XmFONTLIST_DEFAULT_TAG);
       xmstr2 = XmStringCreate(_("No"), XmFONTLIST_DEFAULT_TAG);
