@@ -3,9 +3,11 @@
 #include "snd.h"
 
 static Widget transform_dialog = NULL; /* main dialog shell */
-static Widget type_list, size_list, wavelet_list, window_list, window_beta_scale,
-              db_button, peaks_button, logfreq_button, sono_button, spectro_button, normo_button, normalize_button, selection_button,
-              graph_label, graph_drawer;
+static Widget type_list, size_list, wavelet_list, window_list, window_beta_scale;
+static Widget db_button, peaks_button, logfreq_button, sono_button, spectro_button, normo_button, normalize_button, selection_button;
+static Widget graph_label, graph_drawer;
+static Widget peak_txt, db_txt, freq_base_txt;
+
 static GC gc, fgc;
 
 #define GRAPH_SIZE 128
@@ -29,12 +31,12 @@ static char *FFT_WINDOWS[NUM_FFT_WINDOWS] =
   {"Rectangular", "Hann", "Welch", "Parzen", "Bartlett", "Hamming", "Blackman2", "Blackman3", "Blackman4",
    "Exponential", "Riemann", "Kaiser", "Cauchy", "Poisson", "Gaussian", "Tukey", "Dolph-Chebyshev", "Hann-Poisson", "Connes"};
 
-#define NUM_TRANSFORM_TYPES 7
-static char *TRANSFORM_TYPES[NUM_TRANSFORM_TYPES] = {"Fourier", "Wavelet", "Walsh", "Autocorrelate", "Cepstrum", "Hadamard", "Haar"};
+#define NUM_TRANSFORM_TYPES 6
+static char *TRANSFORM_TYPES[NUM_TRANSFORM_TYPES] = {"Fourier", "Wavelet", "Walsh", "Autocorrelate", "Cepstrum", "Haar"};
 static int num_transform_types = NUM_TRANSFORM_TYPES;
 
 static char *TRANSFORM_TYPE_CONSTANTS[NUM_TRANSFORM_TYPES] = {
-  S_fourier_transform, S_wavelet_transform, S_walsh_transform, S_autocorrelation, S_cepstrum, S_hadamard_transform, S_haar_transform};
+  S_fourier_transform, S_wavelet_transform, S_walsh_transform, S_autocorrelation, S_cepstrum, S_haar_transform};
 
 char *transform_type_name(int choice) 
 {
@@ -47,7 +49,7 @@ int max_transform_type(void) {return(num_transform_types - 1);}
 
 static Float fp_dB(Float py)
 {
-  return((py <= ss->lin_dB) ? 0.0 : (1.0 - ((20.0 * (log10(py))) / ss->min_dB)));
+  return((py <= ss->lin_dB) ? 0.0 : (1.0 - (20.0 * log10(py) / ss->min_dB)));
 }
 
 static Locus local_grf_x(double val, axis_info *ap)
@@ -102,7 +104,7 @@ static void graph_redisplay(void)
   axis_ap->graph_x0 = 0;
   clear_window(ax);
   ax->gc = gc;
-  make_axes_1(axis_ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, false, true);
+  make_axes_1(axis_ap, X_AXIS_IN_SECONDS, 1, SHOW_ALL_AXES, NOT_PRINTING, WITH_X_AXIS, NO_GRID, WITH_LINEAR_AXES, grid_density(ss));
   ix1 = local_grf_x(0.0, axis_ap);
   iy1 = local_grf_y(current_graph_data[0], axis_ap);
   xincr = 1.0 / (Float)GRAPH_SIZE;
@@ -147,12 +149,12 @@ static void get_fft_window_data(void)
 
 static void chans_transform_size(chan_info *cp, void *ptr) 
 {
-  fft_info *fp;
   int size;
   size = (*((int *)ptr));
   cp->transform_size = size;
   if (cp->fft) 
     {
+      fft_info *fp;
       fp = cp->fft;
       if (fp->size < size) fp->ok = false; /* "dirty" flag for fft data array = needs reallocation */
       fp->size = size;
@@ -224,56 +226,41 @@ static void transform_type_browse_callback(Widget w, XtPointer context, XtPointe
 
 static void graph_transform_once_callback(Widget w, XtPointer context, XtPointer info)
 {
-  XmToggleButtonCallbackStruct *cb = (XmToggleButtonCallbackStruct *)info;
+  graph_type_t old_type;
   ASSERT_WIDGET_TYPE(XmIsToggleButton(w), w);
-  if (cb->set)
-    {
-      XmToggleButtonSetState(sono_button, false, false);
-      XmToggleButtonSetState(spectro_button, false, false);
-      in_set_transform_graph_type(GRAPH_ONCE);
-    }
-  else
-    {
-      XmToggleButtonSetState(sono_button, true, false);
-      in_set_transform_graph_type(GRAPH_AS_SONOGRAM);
-    }
-  for_each_chan(calculate_fft);
+  old_type = transform_graph_type(ss);
+  XmToggleButtonSetState(normo_button, true, false);
+  XmToggleButtonSetState(sono_button, false, false);
+  XmToggleButtonSetState(spectro_button, false, false);
+  in_set_transform_graph_type(GRAPH_ONCE);
+  if (old_type != GRAPH_ONCE)
+    for_each_chan(calculate_fft);
 }
 
 static void sonogram_callback(Widget w, XtPointer context, XtPointer info)
 {
-  XmToggleButtonCallbackStruct *cb = (XmToggleButtonCallbackStruct *)info;
+  graph_type_t old_type;
   ASSERT_WIDGET_TYPE(XmIsToggleButton(w), w);
-  if (cb->set)
-    {
-      XmToggleButtonSetState(normo_button, false, false);
-      XmToggleButtonSetState(spectro_button, false, false);
-      in_set_transform_graph_type(GRAPH_AS_SONOGRAM);
-    }
-  else
-    {
-      XmToggleButtonSetState(normo_button, true, false);
-      in_set_transform_graph_type(GRAPH_ONCE);
-    }
-  for_each_chan(calculate_fft);
+  old_type = transform_graph_type(ss);
+  XmToggleButtonSetState(sono_button, true, false);
+  XmToggleButtonSetState(normo_button, false, false);
+  XmToggleButtonSetState(spectro_button, false, false);
+  in_set_transform_graph_type(GRAPH_AS_SONOGRAM);
+  if (old_type != GRAPH_AS_SONOGRAM)
+    for_each_chan(calculate_fft);
 }
 
 static void spectrogram_callback(Widget w, XtPointer context, XtPointer info)
 {
-  XmToggleButtonCallbackStruct *cb = (XmToggleButtonCallbackStruct *)info;
+  graph_type_t old_type;
   ASSERT_WIDGET_TYPE(XmIsToggleButton(w), w);
-  if (cb->set)
-    {
-      XmToggleButtonSetState(normo_button, false, false);
-      XmToggleButtonSetState(sono_button, false, false);
-      in_set_transform_graph_type(GRAPH_AS_SPECTROGRAM);
-    }
-  else
-    {
-      XmToggleButtonSetState(normo_button, true, false);
-      in_set_transform_graph_type(GRAPH_ONCE);
-    }
-  for_each_chan(calculate_fft);
+  old_type = transform_graph_type(ss);
+  XmToggleButtonSetState(spectro_button, true, false);
+  XmToggleButtonSetState(normo_button, false, false);
+  XmToggleButtonSetState(sono_button, false, false);
+  in_set_transform_graph_type(GRAPH_AS_SPECTROGRAM);
+  if (old_type != GRAPH_AS_SPECTROGRAM)
+    for_each_chan(calculate_fft);
 }
 
 static void map_show_transform_peaks(chan_info *cp, void *ptr) {cp->show_transform_peaks = (*((bool *)ptr));}
@@ -372,7 +359,10 @@ static void graph_resize_callback(Widget w, XtPointer context, XtPointer info)
 
 static void dismiss_transform_callback(Widget w, XtPointer context, XtPointer info)
 {
-  XtUnmanageChild(transform_dialog);
+  Widget active_widget;
+  active_widget = XmGetFocusWidget(transform_dialog);
+  if (active_widget == XmMessageBoxGetChild(transform_dialog, XmDIALOG_OK_BUTTON))
+    XtUnmanageChild(transform_dialog);
 }
 
 static void orient_transform_callback(Widget w, XtPointer context, XtPointer info)
@@ -390,28 +380,109 @@ static void help_transform_callback(Widget w, XtPointer context, XtPointer info)
   transform_dialog_help();
 }
 
+static void blue_textfield_unfocus_callback(Widget w, XtPointer context, XtPointer info)
+{
+  if (!(ss->using_schemes)) 
+    XtVaSetValues(w, XmNbackground, (ss->sgx)->lighter_blue, NULL);
+  XtVaSetValues(w, XmNcursorPositionVisible, false, NULL);
+}
+
+static void blue_mouse_leave_text_callback(Widget w, XtPointer context, XEvent *event, Boolean *flag)
+{
+  if (!(ss->using_schemes)) 
+    XtVaSetValues(w, XmNbackground, (ss->sgx)->lighter_blue, NULL);
+  XtVaSetValues(w, XmNcursorPositionVisible, false, NULL);
+}
+
+static void white_mouse_enter_text_callback(Widget w, XtPointer context, XEvent *event, Boolean *flag)
+{
+  if (!(ss->using_schemes)) 
+    XtVaSetValues(w, XmNbackground, (ss->sgx)->text_focus_color, NULL);
+  XtVaSetValues(w, XmNcursorPositionVisible, true, NULL);
+}
+
+void reflect_peaks_in_transform_dialog(void)
+{
+  if (transform_dialog)
+    widget_int_to_text(peak_txt, max_transform_peaks(ss));
+}
+
+static void peaks_activate_callback(Widget w, XtPointer context, XtPointer info)
+{
+  char *str;
+  str = XmTextFieldGetString(w);
+  if ((str) && (*str))
+    {
+      int new_peaks;
+      new_peaks = string_to_int(str);
+      if (new_peaks >= 1)
+	{
+	  set_max_transform_peaks(new_peaks);
+	  for_each_chan(calculate_fft);
+	}
+      else widget_int_to_text(w, max_transform_peaks(ss));
+    }
+}
+
+void reflect_log_freq_start_in_transform_dialog(void)
+{
+  if (transform_dialog)
+    widget_float_to_text(freq_base_txt, log_freq_start(ss));
+}
+
+static void log_freq_start_activate_callback(Widget w, XtPointer context, XtPointer info)
+{
+  char *str;
+  str = XmTextFieldGetString(w);
+  if ((str) && (*str))
+    {
+      Float new_lfb;
+      new_lfb = string_to_Float(str);
+      if (new_lfb > 0.0)
+	set_log_freq_start(new_lfb);
+      else widget_float_to_text(w, log_freq_start(ss));
+    }
+}
+
+void reflect_min_db_in_transform_dialog(void)
+{
+  if (transform_dialog)
+    widget_float_to_text(db_txt, ss->min_dB);
+}
+
+static void min_db_activate_callback(Widget w, XtPointer context, XtPointer info)
+{
+  char *str;
+  str = XmTextFieldGetString(w);
+  if ((str) && (*str))
+    {
+      Float new_db;
+      new_db = string_to_Float(str);
+      if (new_db < 0.0)
+	set_min_db(new_db);
+      else widget_float_to_text(w, ss->min_dB);
+    }
+}
+
 static bool need_callback = true;
 Widget fire_up_transform_dialog(bool managed)
 {
-  XmString xhelp, xdismiss, xtitle, bstr, xorient;
-  Arg args[32];
-  XmString sizes[NUM_TRANSFORM_SIZES];
-  XmString *types;
-  XmString wavelets[NUM_WAVELETS];
-  XmString windows[GUI_NUM_FFT_WINDOWS];
-  XGCValues gv;
-  XtCallbackList n1, n2;
-  int size_pos = 1;
-  int n, i;
-  Widget mainform, type_frame, type_form, type_label,
-                  size_frame, size_form, size_label,
-                  display_frame, display_form, display_label,
-                  window_frame, window_form, window_label,
-                  wavelet_frame, wavelet_form, wavelet_label,
-                  graph_frame, graph_form, color_button;
+  Widget mainform, type_frame, type_form, type_label, size_frame, size_form, size_label, display_frame, display_form, display_label;
+  Widget window_frame, window_form, window_label, wavelet_frame, wavelet_form, wavelet_label, graph_frame, graph_form, color_button;
     
   if (!transform_dialog)
     {
+      XmString xhelp, xdismiss, xtitle, bstr, xorient;
+      Arg args[32];
+      XmString sizes[NUM_TRANSFORM_SIZES];
+      XmString *types;
+      XmString wavelets[NUM_WAVELETS];
+      XmString windows[GUI_NUM_FFT_WINDOWS];
+      XGCValues gv;
+      XtCallbackList n1, n2;
+      int size_pos = 1;
+      int n, i;
+
       types = (XmString *)CALLOC(num_transform_types, sizeof(XmString));
       for (i = 0; i < NUM_TRANSFORM_SIZES; i++)
 	if (transform_sizes[i] == transform_size(ss))
@@ -483,7 +554,8 @@ Widget fire_up_transform_dialog(bool managed)
       n = 0;
       if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->basic_color); n++;}
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-      XtSetArg(args[n], XmNrightAttachment, XmATTACH_NONE); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+      XtSetArg(args[n], XmNrightPosition, 30); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_POSITION); n++;
       XtSetArg(args[n], XmNbottomPosition, 50); n++;
@@ -537,7 +609,8 @@ Widget fire_up_transform_dialog(bool managed)
       if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->basic_color); n++;}
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
       XtSetArg(args[n], XmNleftWidget, type_frame); n++;
-      XtSetArg(args[n], XmNrightAttachment, XmATTACH_NONE); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+      XtSetArg(args[n], XmNrightPosition, 60); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
       XtSetArg(args[n], XmNbottomWidget, type_frame); n++;
@@ -624,7 +697,7 @@ Widget fire_up_transform_dialog(bool managed)
       XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
       XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
       normo_button = make_togglebutton_widget("normo-button", display_form, args, n);
-      XtAddCallback(normo_button, XmNvalueChangedCallback, graph_transform_once_callback, NULL);
+      XtAddCallback(normo_button, XmNdisarmCallback, graph_transform_once_callback, NULL);
       XmStringFree(bstr);
 
       n = 0;
@@ -643,7 +716,7 @@ Widget fire_up_transform_dialog(bool managed)
       XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
       XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
       sono_button = make_togglebutton_widget("sono-button", display_form, args, n);
-      XtAddCallback(sono_button, XmNvalueChangedCallback, sonogram_callback, NULL);
+      XtAddCallback(sono_button, XmNdisarmCallback, sonogram_callback, NULL);
       XmStringFree(bstr);
 
       n = 0;
@@ -662,7 +735,7 @@ Widget fire_up_transform_dialog(bool managed)
       XtSetArg(args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
       XtSetArg(args[n], XmNindicatorType, XmONE_OF_MANY); n++;
       spectro_button = make_togglebutton_widget("spectro-button", display_form, args, n);
-      XtAddCallback(spectro_button, XmNvalueChangedCallback, spectrogram_callback, NULL);
+      XtAddCallback(spectro_button, XmNdisarmCallback, spectrogram_callback, NULL);
       XmStringFree(bstr);
 
       n = 0;
@@ -673,7 +746,8 @@ Widget fire_up_transform_dialog(bool managed)
 	}
       bstr = XmStringCreate(_("peaks"), XmFONTLIST_DEFAULT_TAG);
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+      XtSetArg(args[n], XmNrightPosition, 67); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
       XtSetArg(args[n], XmNtopWidget, spectro_button); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
@@ -683,6 +757,29 @@ Widget fire_up_transform_dialog(bool managed)
       XtAddCallback(peaks_button, XmNvalueChangedCallback, peaks_callback, NULL);
       XmStringFree(bstr);
 
+      n = 0;
+      if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->lighter_blue); n++;}
+      XtSetArg(args[n], XmNresizeWidth, false); n++;
+      XtSetArg(args[n], XmNcolumns, 6); n++;
+      XtSetArg(args[n], XmNrecomputeSize, false); n++;
+      XtSetArg(args[n], XmNmarginHeight, 1); n++;
+      XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+      XtSetArg(args[n], XmNleftWidget, peaks_button); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNtopWidget, peaks_button); n++;
+      XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNbottomWidget, peaks_button); n++;
+      XtSetArg(args[n], XmNborderWidth, 0); n++;
+      XtSetArg(args[n], XmNshadowThickness, 0); n++;
+      peak_txt = make_textfield_widget("max-peaks", display_form, args, n, ACTIVATABLE, NO_COMPLETER);
+      XtRemoveCallback(peak_txt, XmNlosingFocusCallback, textfield_unfocus_callback, NULL);
+      XtAddCallback(peak_txt, XmNlosingFocusCallback, blue_textfield_unfocus_callback, NULL);
+      XtAddEventHandler(peak_txt, LeaveWindowMask, false, blue_mouse_leave_text_callback, NULL);
+      XtAddEventHandler(peak_txt, EnterWindowMask, false, white_mouse_enter_text_callback, NULL);
+      widget_int_to_text(peak_txt, max_transform_peaks(ss));
+      XtAddCallback(peak_txt, XmNactivateCallback, peaks_activate_callback, NULL);
+
       n = 0;
       if (!(ss->using_schemes)) 
 	{
@@ -691,7 +788,8 @@ Widget fire_up_transform_dialog(bool managed)
 	}
       bstr = XmStringCreate(_("dB"), XmFONTLIST_DEFAULT_TAG);
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+      XtSetArg(args[n], XmNrightPosition, 67); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
       XtSetArg(args[n], XmNtopWidget, peaks_button); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
@@ -701,6 +799,29 @@ Widget fire_up_transform_dialog(bool managed)
       XtAddCallback(db_button, XmNvalueChangedCallback, db_callback, NULL);
       XmStringFree(bstr);
 
+      n = 0;
+      if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->lighter_blue); n++;}
+      XtSetArg(args[n], XmNresizeWidth, false); n++;
+      XtSetArg(args[n], XmNcolumns, 6); n++;
+      XtSetArg(args[n], XmNrecomputeSize, false); n++;
+      XtSetArg(args[n], XmNmarginHeight, 1); n++;
+      XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+      XtSetArg(args[n], XmNleftWidget, db_button); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNtopWidget, db_button); n++;
+      XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNbottomWidget, db_button); n++;
+      XtSetArg(args[n], XmNborderWidth, 0); n++;
+      XtSetArg(args[n], XmNshadowThickness, 0); n++;
+      db_txt = make_textfield_widget("db", display_form, args, n, ACTIVATABLE, NO_COMPLETER);
+      XtRemoveCallback(db_txt, XmNlosingFocusCallback, textfield_unfocus_callback, NULL);
+      XtAddCallback(db_txt, XmNlosingFocusCallback, blue_textfield_unfocus_callback, NULL);
+      XtAddEventHandler(db_txt, LeaveWindowMask, false, blue_mouse_leave_text_callback, NULL);
+      XtAddEventHandler(db_txt, EnterWindowMask, false, white_mouse_enter_text_callback, NULL);
+      widget_float_to_text(db_txt, ss->min_dB);
+      XtAddCallback(db_txt, XmNactivateCallback, min_db_activate_callback, NULL);
+
       n = 0;
       if (!(ss->using_schemes)) 
 	{
@@ -709,7 +830,8 @@ Widget fire_up_transform_dialog(bool managed)
 	}
       bstr = XmStringCreate(_("log freq"), XmFONTLIST_DEFAULT_TAG);
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
+      XtSetArg(args[n], XmNrightPosition, 67); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
       XtSetArg(args[n], XmNtopWidget, db_button); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
@@ -719,6 +841,29 @@ Widget fire_up_transform_dialog(bool managed)
       XtAddCallback(logfreq_button, XmNvalueChangedCallback, logfreq_callback, NULL);
       XmStringFree(bstr);
 
+      n = 0;
+      if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->lighter_blue); n++;}
+      XtSetArg(args[n], XmNresizeWidth, false); n++;
+      XtSetArg(args[n], XmNcolumns, 6); n++;
+      XtSetArg(args[n], XmNrecomputeSize, false); n++;
+      XtSetArg(args[n], XmNmarginHeight, 1); n++;
+      XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+      XtSetArg(args[n], XmNleftWidget, logfreq_button); n++;
+      XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+      XtSetArg(args[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNtopWidget, logfreq_button); n++;
+      XtSetArg(args[n], XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
+      XtSetArg(args[n], XmNbottomWidget, logfreq_button); n++;
+      XtSetArg(args[n], XmNborderWidth, 0); n++;
+      XtSetArg(args[n], XmNshadowThickness, 0); n++;
+      freq_base_txt = make_textfield_widget("lfb", display_form, args, n, ACTIVATABLE, NO_COMPLETER);
+      XtRemoveCallback(freq_base_txt, XmNlosingFocusCallback, textfield_unfocus_callback, NULL);
+      XtAddCallback(freq_base_txt, XmNlosingFocusCallback, blue_textfield_unfocus_callback, NULL);
+      XtAddEventHandler(freq_base_txt, LeaveWindowMask, false, blue_mouse_leave_text_callback, NULL);
+      XtAddEventHandler(freq_base_txt, EnterWindowMask, false, white_mouse_enter_text_callback, NULL);
+      widget_float_to_text(freq_base_txt, log_freq_start(ss));
+      XtAddCallback(freq_base_txt, XmNactivateCallback, log_freq_start_activate_callback, NULL);
+
       n = 0;
       if (!(ss->using_schemes)) 
 	{
@@ -1014,12 +1159,12 @@ void set_transform_graph_type(graph_type_t val)
 
 void set_transform_size(int val)
 {
-  int i;
   for_each_chan(force_fft_clear);
   in_set_transform_size(val);
   for_each_chan_1(chans_transform_size, (void *)(&val));
   if (transform_dialog)
     {
+      int i;
       for (i = 0; i < NUM_TRANSFORM_SIZES; i++)
 	if (transform_sizes[i] == val)
 	  {
@@ -1119,9 +1264,9 @@ void set_show_selection_transform(bool show)
 int add_transform_to_list(char *name)
 {
   /* put at end of list and return associated browse callback row */
-  XmString str;
   if (transform_dialog)
     {
+      XmString str;
       str = XmStringCreate(name, XmFONTLIST_DEFAULT_TAG);
       XmListAddItem(type_list, str, 0);
       XmStringFree(str);
