@@ -90,7 +90,6 @@ static void load_header_and_data_lists(file_data *fdat, int type, int format, in
 {
   int i;
   char **fl = NULL;
-  char *str;
   XmString *strs;
   fdat->current_type = type;
   fdat->current_format = format;
@@ -110,45 +109,25 @@ static void load_header_and_data_lists(file_data *fdat, int type, int format, in
   XmListSelectPos(fdat->format_list, fdat->format_pos + 1, false);
   if ((srate > 0) && 
       (fdat->srate_text))
-    {
-      str = (char *)CALLOC(32, sizeof(char));
-      sprintf(str, "%d", srate);
-      XmTextSetString(fdat->srate_text, str);
-      FREE(str);
-    }
+    widget_int_to_text(fdat->srate_text, srate);
   if ((chans > 0) && 
       (fdat->chans_text))
-    {
-      str = (char *)CALLOC(8, sizeof(char));
-      sprintf(str, "%d", chans);
-      XmTextSetString(fdat->chans_text, str);
-      FREE(str);
-    }
+    widget_int_to_text(fdat->chans_text, chans);
   if (fdat->comment_text) 
     XmTextSetString(fdat->comment_text, comment);
   if ((location >= 0) && 
       (fdat->location_text))
-    {
-      str = (char *)CALLOC(32, sizeof(char));
-      sprintf(str, OFF_TD, location);
-      XmTextSetString(fdat->location_text, str);
-      FREE(str);
-    }
+    widget_off_t_to_text(fdat->location_text, location);
   if ((samples >= 0) && 
       (fdat->samples_text))
-    {
-      str = (char *)CALLOC(32, sizeof(char));
-      sprintf(str, OFF_TD, samples);
-      XmTextSetString(fdat->samples_text, str);
-      FREE(str);
-    }
+    widget_off_t_to_text(fdat->samples_text, samples);
 }
 
 static void color_file_selection_box(Widget w)
 {
-  Widget wtmp = NULL, ftmp = NULL, ltmp = NULL;
   if (!(ss->using_schemes)) 	
     {
+      Widget wtmp = NULL, ftmp = NULL, ltmp = NULL;
       map_over_children(w, set_main_color_of_widget, NULL);
       XtVaSetValues(XmFileSelectionBoxGetChild(w, XmDIALOG_DIR_LIST), 
 		    XmNbackground, (ss->sgx)->white, 
@@ -258,13 +237,13 @@ void clear_deleted_snd_info(void *data)
 
 static void play_selected_callback(Widget w, XtPointer context, XtPointer info) 
 {
-  Widget wtmp;
-  char *filename;
-  file_dialog_info *fd = (file_dialog_info *)context;
   XmToggleButtonCallbackStruct *cb = (XmToggleButtonCallbackStruct *)info;
+  file_dialog_info *fd = (file_dialog_info *)context;
   ASSERT_WIDGET_TYPE(XmIsToggleButton(w), w);
   if (cb->set)
     {
+      Widget wtmp;
+      char *filename;
       if ((fd->file_play_sp) && (fd->file_play_sp->playing)) 
 	stop_playing_sound(fd->file_play_sp, PLAY_BUTTON_UNSET);
       wtmp = XtNameToWidget(fd->dialog, "Text");
@@ -276,10 +255,7 @@ static void play_selected_callback(Widget w, XtPointer context, XtPointer info)
 	  fd->file_play_sp = make_sound_readable(filename, false);
 	  fd->file_play_sp->delete_me = (void *)fd;
 	  if (fd->file_play_sp)
-	    play_sound(fd->file_play_sp, 0, 
-		       NO_END_SPECIFIED, IN_BACKGROUND, 
-		       C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), 
-		       "selected file play", 0);
+	    play_sound(fd->file_play_sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 	}
     }
   else file_dialog_stop_playing(fd);
@@ -315,11 +291,10 @@ static void sound_file_search(Widget FSB_w, XmFileSelectionBoxCallbackStruct *in
    * the pattern (file name mask) only matters if the filter button is hit, 
    * it appears to be "*" until the filter is invoked.
    */
-  char *pattern, *our_dir, *sp, *sn;
+  char *pattern, *our_dir;
   dir *cdp;
   file_dialog_info *fd;
   XmFileSelectionBoxCallbackStruct *data = (XmFileSelectionBoxCallbackStruct *)info;
-  XmString *names = NULL;
   int i, which_dialog;
   bool filter_callback;
   ASSERT_WIDGET_TYPE(XmIsFileSelectionBox(FSB_w), FSB_w);
@@ -384,6 +359,7 @@ static void sound_file_search(Widget FSB_w, XmFileSelectionBoxCallbackStruct *in
   fd->new_file_written = false;
   if (fd->need_update)
     {
+      XmString *names = NULL;
       if ((cdp) && (cdp->len > 0))
 	{
 	  qsort((void *)(cdp->files), cdp->len, sizeof(char *), string_compare);
@@ -401,6 +377,7 @@ static void sound_file_search(Widget FSB_w, XmFileSelectionBoxCallbackStruct *in
 
 	  for (i = 0; i < cdp->len; i++) 
 	    {
+	      char *sp, *sn;
 	      for (sp = fd->save_dir, sn = cdp->files[i]; ((*sp) = (*sn)) != '\0'; sp++, sn++);
 	      /* save_dir is a pointer into fullpathname after the directory portion */
 	      /*   this is unreadable code! -- it's basically sprintf(fullpathname,"%s%s",our_dir,cdp->files[i]) I think */
@@ -451,46 +428,49 @@ static void just_sounds_callback(Widget w, XtPointer context, XtPointer info)
 #if (XmVERSION > 1)
 static void file_dialog_select_callback(Widget w, XtPointer context, XtPointer info)
 {
+  file_dialog_info *fd = (file_dialog_info *)context;
   XmString *strs;
   char *filename = NULL;
-  XmString label;
-  char *buf;
-  char timestr[64];
-  time_t date;
-  file_dialog_info *fd = (file_dialog_info *)context;
   ASSERT_WIDGET_TYPE(XmIsList(w), w);
   XtVaGetValues(w, XmNselectedItems, &strs, NULL);
   filename = (char *)XmStringUnparse(strs[0], NULL, XmCHARSET_TEXT, XmCHARSET_TEXT, NULL, 0, XmOUTPUT_ALL);
-  if ((filename) && (sound_file_p(filename)))
+  if (filename)
     {
-      XtManageChild(fd->play_selected_button);
-      buf = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
-      mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s: %d chan%s, %d Hz, %.3f secs",
-		   filename_without_home_directory(filename),
-		   mus_sound_chans(filename),
-		   (mus_sound_chans(filename) > 1) ? "s" : "",
-		   mus_sound_srate(filename),
-		   mus_sound_duration(filename));
-      label = XmStringCreateLocalized(buf);
-      XtVaSetValues(fd->dialog_info1, XmNlabelString, label, NULL);
-      XmStringFree(label);
-      date = mus_sound_write_date(filename);
+      if (sound_file_p(filename))
+	{
+	  XmString label;
+	  char *buf;
+	  char timestr[64];
+	  time_t date;
+	  XtManageChild(fd->play_selected_button);
+	  buf = (char *)CALLOC(LABEL_BUFFER_SIZE, sizeof(char));
+	  mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s: %d chan%s, %d Hz, %.3f secs",
+		       filename_without_home_directory(filename),
+		       mus_sound_chans(filename),
+		       (mus_sound_chans(filename) > 1) ? "s" : "",
+		       mus_sound_srate(filename),
+		       mus_sound_duration(filename));
+	  label = XmStringCreateLocalized(buf);
+	  XtVaSetValues(fd->dialog_info1, XmNlabelString, label, NULL);
+	  XmStringFree(label);
+	  date = mus_sound_write_date(filename);
 #if HAVE_STRFTIME
-      strftime(timestr, 64, ", %d-%b-%Y", localtime(&date));
+	  strftime(timestr, 64, ", %d-%b-%Y", localtime(&date));
 #else
-      sprintf(timestr, "");
+	  sprintf(timestr, "");
 #endif
-      mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s %s%s",
-		   mus_header_type_name(mus_sound_header_type(filename)),
-		   mus_short_data_format_name(mus_sound_data_format(filename)),
-		   timestr);
-      label = XmStringCreateLocalized(buf);
-      XtVaSetValues(fd->dialog_info2, XmNlabelString, label, NULL);
-      XmStringFree(label);
-      FREE(buf);
+	  mus_snprintf(buf, LABEL_BUFFER_SIZE, "%s %s%s",
+		       mus_header_type_name(mus_sound_header_type(filename)),
+		       mus_short_data_format_name(mus_sound_data_format(filename)),
+		       timestr);
+	  label = XmStringCreateLocalized(buf);
+	  XtVaSetValues(fd->dialog_info2, XmNlabelString, label, NULL);
+	  XmStringFree(label);
+	  FREE(buf);
+	  if (!(XtIsManaged(fd->dialog_frame))) 
+	    XtManageChild(fd->dialog_frame);
+	}
       XtFree(filename);
-      if (!(XtIsManaged(fd->dialog_frame))) 
-	XtManageChild(fd->dialog_frame);
     }
   else
     {
@@ -588,7 +568,6 @@ static file_dialog_info *make_file_dialog(bool read_only, char *title, char *sel
 
 static void file_open_ok_callback(Widget w, XtPointer context, XtPointer info) 
 {
-  snd_info *sp;
   file_dialog_info *fd = (file_dialog_info *)context;
   XmFileSelectionBoxCallbackStruct *cbs = (XmFileSelectionBoxCallbackStruct *)info;
   char *filename;
@@ -598,6 +577,7 @@ static void file_open_ok_callback(Widget w, XtPointer context, XtPointer info)
   file_dialog_stop_playing(fd);
   if (!(directory_p(filename)))               /* this can be a directory name if the user clicked 'ok' when he meant 'cancel' */
     {
+      snd_info *sp;
       sp = snd_open_file(filename, 
 			 fd->file_dialog_read_only);
       if (sp) select_channel(sp, 0);           /* add_sound_window (snd-xsnd.c) will report reason for error, if any */
@@ -664,10 +644,7 @@ static void file_mix_ok_callback(Widget w, XtPointer context, XtPointer info)
   XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &filename);
   file_dialog_stop_playing(fd);
   if (!(directory_p(filename)))               /* this can be a directory name if the user clicked 'ok' when he meant 'cancel' */
-    mix_complete_file_at_cursor(any_selected_sound(), 
-				filename,
-				"File: mix",
-				with_mix_tags(ss), 0);
+    mix_complete_file_at_cursor(any_selected_sound(), filename, with_mix_tags(ss), 0);
   else snd_error(_("%s is a directory"), filename);
 }
 
@@ -714,6 +691,7 @@ void alert_new_file(void)
  * to the current state/file (different from emacs) -- this keeps mix console intact
  * across backups and so on, and seems more useful to me than switching to the new file.
  * changed again 12-Nov-01 to make this choice settable via emacs-style-save-as.
+ * changed again 6-Dec-04 -- need a hook here since gtk changes constantly
  */
 
 static file_data *save_as_file_data = NULL;
@@ -722,9 +700,9 @@ static save_dialog_t save_as_dialog_type = FILE_SAVE_AS;
 
 static void save_as_ok_callback(Widget w, XtPointer context, XtPointer info)
 { 
-  char *str = NULL, *comment, *fullname;
+  char *str = NULL, *comment;
   snd_info *sp;
-  int i, type, format, srate, chans;
+  int type, format, srate, chans;
   bool need_update = false;
   off_t location, samples;
   comment = read_file_data_choices(save_as_file_data, &srate, &chans, &type, &format, &location, &samples);
@@ -737,17 +715,10 @@ static void save_as_ok_callback(Widget w, XtPointer context, XtPointer info)
       report_in_minibuffer(sp, _("not saved (no file name given)"));
   if (comment) FREE(comment);
   XtUnmanageChild(save_as_dialog);
-  if ((sp) && (str) && (emacs_style_save_as(ss)) &&
+  if ((sp) && (str) && 
       (save_as_dialog_type == FILE_SAVE_AS) && 
       (need_update))
-    {
-      for (i = 0; i < sp->nchans; i++) 
-	sp->chans[i]->edit_ctr = 0; /* don't trigger close-hook unsaved-edit checks */
-      snd_close_file(sp);
-      fullname = mus_expand_filename(str);
-      snd_open_file(fullname, false); /* false = not read_only */
-      FREE(fullname);
-    }
+    run_after_save_as_hook(sp, str, true); /* true => from dialog */
   if (str) XtFree(str);
 } 
 
@@ -790,11 +761,8 @@ static char *header_short_names[NUM_HEADER_TYPES] = {"sun  ", "aifc ", "wave ",
 file_data *make_file_data_panel(Widget parent, char *name, Arg *in_args, int in_n, 
 				bool with_chan, int header_type, int data_format, bool with_loc, bool with_comment, bool with_samples)
 {
-  Widget mainform, form,
-    sep1, hlab, hlist, dlab, dlist,
-    slab, stext, clab, ctext = NULL, sep2, sep3, 
-    comment_label, comment_text, sep4,
-    loclab, loctext = NULL, samplab, samptext;
+  Widget mainform, form, sep1, hlab, hlist, dlab, dlist, slab, stext, clab, ctext = NULL, sep2, sep3;
+  Widget comment_label, comment_text, sep4, loclab, loctext = NULL, samplab, samptext;
   file_data *fdat;
   Arg args[32];
   int i, n;
@@ -1054,12 +1022,13 @@ static void save_as_help_callback(Widget w, XtPointer context, XtPointer info)
 static void make_save_as_dialog(char *sound_name, int header_type, int format_type)
 {
   /* save old as new, close old, open new */
-  Arg args[32];
-  int n;
-  XmString xmstr1, xmstr2, s1;
   char *file_string;
   if (!save_as_dialog)
     {
+      Arg args[32];
+      int n;
+      XmString xmstr1, xmstr2, s1;
+
       n = 0;
       if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->basic_color); n++;}
       s1 = XmStringCreate(_("save as:"), XmFONTLIST_DEFAULT_TAG);
@@ -1102,6 +1071,7 @@ static void make_save_as_dialog(char *sound_name, int header_type, int format_ty
     }
   else
     {
+      XmString xmstr2;
       file_string = (char *)CALLOC(PRINT_BUFFER_SIZE, sizeof(char));
       mus_snprintf(file_string, PRINT_BUFFER_SIZE, _("saving %s"), sound_name);
       xmstr2 = XmStringCreate(file_string, XmFONTLIST_DEFAULT_TAG);
@@ -1357,11 +1327,10 @@ static XEN mouse_leave_label_hook;
 
 static void mouse_leave_label_or_enter(regrow *r, XEN hook, const char *caller)
 {
-  XmString s1 = NULL;
-  char *label = NULL;
   if ((r) &&
       (XEN_HOOKED(hook)))
     {
+      char *label = NULL;
       if (r->parent == CURRENT_FILE_VIEWER)
 	label = get_curfullname(r->pos);
       else
@@ -1370,6 +1339,7 @@ static void mouse_leave_label_or_enter(regrow *r, XEN hook, const char *caller)
 	    label = get_prevfullname(r->pos);
 	  else
 	    {
+	      XmString s1 = NULL;
 	      /* it's a bit tedious to get the current button label... */
 	      XtVaGetValues(r->nm, XmNlabelString, &s1, NULL);
 	      if (XmStringEmpty(s1)) return;
@@ -1379,8 +1349,8 @@ static void mouse_leave_label_or_enter(regrow *r, XEN hook, const char *caller)
 	}
       if (label)
 	run_hook(hook,
-		 XEN_LIST_3(C_TO_SMALL_XEN_INT(r->parent),
-			    C_TO_SMALL_XEN_INT(r->pos),
+		 XEN_LIST_3(C_TO_XEN_INT(r->parent),
+			    C_TO_XEN_INT(r->pos),
 			    C_TO_XEN_STRING(label)),
 		 caller);
     }
@@ -1466,11 +1436,11 @@ static regrow **prev_name_row = NULL;
 
 void make_cur_name_row(int old_size, int new_size)
 {
-  int i;
   if (cur_name_row == NULL)
     cur_name_row = (regrow **)CALLOC(new_size, sizeof(regrow *));
   else 
     {
+      int i;
       cur_name_row = (regrow **)REALLOC(cur_name_row, new_size * sizeof(regrow *));
       for (i = old_size; i < new_size; i++) cur_name_row[i] = NULL;
     }
@@ -1478,11 +1448,11 @@ void make_cur_name_row(int old_size, int new_size)
 
 void make_prev_name_row(int old_size, int new_size)
 {
-  int i;
   if (prev_name_row == NULL)
     prev_name_row = (regrow **)CALLOC(new_size, sizeof(regrow *));
   else 
     {
+      int i;
       prev_name_row = (regrow **)REALLOC(prev_name_row, new_size * sizeof(regrow *));
       for (i = old_size; i < new_size; i++) prev_name_row[i] = NULL;
     }
@@ -1520,15 +1490,14 @@ static void view_files_update_callback(Widget w, XtPointer context, XtPointer in
 
 void set_file_browser_play_button(char *name, int state)
 {
-  int i, list;
-  regrow *r;
-  list = 0;
   if (view_files_dialog_is_active())
     {
+      int i, list = 0;
       i = find_curfile_regrow(name); 
       if (i != -1) list = 1; else i = find_prevfile_regrow(name);
       if (i != -1)
 	{
+	  regrow *r;
 	  if (list) r = cur_name_row[i]; else r = prev_name_row[i];
 	  XmToggleButtonSetState(r->pl, (Boolean)state, false);
 	}
@@ -1545,11 +1514,11 @@ static void view_curfiles_play_callback(Widget w, XtPointer context, XtPointer i
 
 static void curfile_unhighlight(void)
 {
-  regrow *r;
   if ((view_files_dialog_is_active()) && 
       (!(ss->using_schemes)) && 
       (vf_selected_file != -1))
     {
+      regrow *r;
       r = cur_name_row[vf_selected_file];
       XtVaSetValues(r->rw, XmNbackground, (ss->sgx)->highlight_color, NULL);
       XtVaSetValues(r->nm, XmNbackground, (ss->sgx)->highlight_color, NULL);
@@ -1559,10 +1528,10 @@ static void curfile_unhighlight(void)
 
 void curfile_highlight(int i)
 {
-  regrow *r;
   if ((view_files_dialog_is_active()) && 
       (!(ss->using_schemes)))
     {
+      regrow *r;
       if (vf_selected_file != -1) curfile_unhighlight();
       r = cur_name_row[i];
       XtVaSetValues(r->rw, XmNbackground, (ss->sgx)->zoom_color, NULL);
@@ -1597,10 +1566,10 @@ static void view_prevfiles_select_callback(Widget w, XtPointer context, XtPointe
 void highlight_selected_sound(void)
 {
   snd_info *sp;
-  int i;
   sp = selected_sound();
   if (sp)
     {
+      int i;
       i = find_curfile_regrow(sp->short_filename);
       if (i != -1) 
 	curfile_highlight(i); 
@@ -1725,14 +1694,15 @@ static void start_view_files_dialog(bool managed)
    * off-chance this browser will be fired up.  (Such files may be subsequently moved or deleted).
    */
   bool new_dialog = false;
-  int n;
-  Arg args[20];
-  ww_info *wwl;
-  regrow *r;
-  XmString xdismiss, xhelp, xclear, titlestr;
-  Widget mainform, curform, prevform, updateB, sep;
   if (!view_files_dialog)
     {
+      int n;
+      Arg args[20];
+      ww_info *wwl;
+      regrow *r;
+      XmString xdismiss, xhelp, xclear, titlestr;
+      Widget mainform, curform, prevform, updateB, sep;
+
       new_dialog = true;
       vf_selected_file = -1;
       xdismiss = XmStringCreate(_("Dismiss"), XmFONTLIST_DEFAULT_TAG);
@@ -1927,9 +1897,6 @@ static void raw_data_help_callback(Widget w, XtPointer context, XtPointer info)
   raw_data_dialog_help();
 }
 
-static char dfs_str[LABEL_BUFFER_SIZE];
-static char dfc_str[LABEL_BUFFER_SIZE];
-
 static void make_raw_data_dialog(const char *filename)
 {
   XmString *formats;
@@ -1997,10 +1964,7 @@ static void make_raw_data_dialog(const char *filename)
   XtSetArg(args[n], XmNcolumns, 6); n++;
   XtSetArg(args[n], XmNresizeWidth, false); n++;
   raw_srate_text = make_textfield_widget("text", rform, args, n, NOT_ACTIVATABLE, add_completer_func(srate_completer));
-  if (sr < 100000) 
-    mus_snprintf(dfs_str, LABEL_BUFFER_SIZE, " %d", sr);
-  else mus_snprintf(dfs_str, LABEL_BUFFER_SIZE, "%d", sr);
-  XmTextSetString(raw_srate_text, dfs_str);
+  widget_int_to_text(raw_srate_text, sr);
 
   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
@@ -2021,10 +1985,7 @@ static void make_raw_data_dialog(const char *filename)
   XtSetArg(args[n], XmNcolumns, 3); n++;
   XtSetArg(args[n], XmNresizeWidth, false); n++;
   raw_chans_text = make_textfield_widget("text", rform, args, n, NOT_ACTIVATABLE, NO_COMPLETER);
-  if (oc < 10) 
-    mus_snprintf(dfc_str, LABEL_BUFFER_SIZE, "  %d", oc);
-  else mus_snprintf(dfc_str, LABEL_BUFFER_SIZE, " %d", oc);
-  XmTextSetString(raw_chans_text, dfc_str);
+  widget_int_to_text(raw_chans_text, oc);
   
   n = 0;
   XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
@@ -2172,20 +2133,25 @@ static void new_file_help_callback(Widget w, XtPointer context, XtPointer info)
 
 snd_info *make_new_file_dialog(char *newname, int header_type, int data_format, int srate, int chans, char *comment)
 {
-  Arg args[20];
-  int n;
-  off_t loc;
-  XmString titlestr, xok, xcancel, xhelp;
-  char *tmpstr, *title, *newer_name = NULL;
   snd_info *sp = NULL;
-  Widget name_label, form;
-  new_file_cancelled = false;
-  title = (char *)CALLOC(snd_strlen(newname) + 32, sizeof(char));
-  sprintf(title, _("create new sound: %s"), newname);
-  titlestr = XmStringCreate(title, XmFONTLIST_DEFAULT_TAG);
-  FREE(title);
+  XmString titlestr;
+
+  {
+    char *title;
+    new_file_cancelled = false;
+    title = (char *)CALLOC(snd_strlen(newname) + 32, sizeof(char));
+    sprintf(title, _("create new sound: %s"), newname);
+    titlestr = XmStringCreate(title, XmFONTLIST_DEFAULT_TAG);
+    FREE(title);
+  }
+
   if (!new_dialog)
     {
+      Arg args[20];
+      int n;
+      XmString xok, xcancel, xhelp;
+      Widget name_label, form;
+
       xhelp = XmStringCreate(_("Help"), XmFONTLIST_DEFAULT_TAG);
       xcancel = XmStringCreate(_("Cancel"), XmFONTLIST_DEFAULT_TAG);
       xok = XmStringCreate(_("Ok"), XmFONTLIST_DEFAULT_TAG);
@@ -2279,6 +2245,8 @@ snd_info *make_new_file_dialog(char *newname, int header_type, int data_format,
     return(NULL);
   else
     {
+      off_t loc;
+      char *tmpstr, *newer_name = NULL;
       newer_name = XmTextGetString(new_file_name);
       if (newer_name == NULL) return(NULL);
       tmpstr = read_file_data_choices(new_dialog_data, &srate, &chans, &header_type, &data_format, &loc, &initial_samples);
@@ -2341,21 +2309,26 @@ Widget edit_header(snd_info *sp)
    * if any are changed, need save button, cancel button, dismiss (leave unsaved but pending), reflect (change Snd display, not file)
    * this means the Snd-effective header is separate from the in-file header even across saves??
    */
-  XmString xstr1, xstr2, xstr3, xstr4, titlestr;
-  int n;
-  Arg args[20];
   file_info *hdr;
-  char *str;
+  XmString xstr4;
   if (!sp) return(NULL);
   edit_header_sp = sp;
   hdr = sp->hdr;
-  str = (char *)CALLOC(PRINT_BUFFER_SIZE, sizeof(char));
-  mus_snprintf(str, PRINT_BUFFER_SIZE, _("Edit header of %s"), sp->short_filename);
-  xstr4 = XmStringCreate(str, XmFONTLIST_DEFAULT_TAG);
-  FREE(str);
+
+  {
+    char *str;
+    str = (char *)CALLOC(PRINT_BUFFER_SIZE, sizeof(char));
+    mus_snprintf(str, PRINT_BUFFER_SIZE, _("Edit header of %s"), sp->short_filename);
+    xstr4 = XmStringCreate(str, XmFONTLIST_DEFAULT_TAG);
+    FREE(str);
+  }
 
   if (!edit_header_dialog)
     {
+      int n;
+      Arg args[20];
+      XmString xstr1, xstr2, xstr3, titlestr;
+
       n = 0;
       xstr1 = XmStringCreate(_("Cancel"), XmFONTLIST_DEFAULT_TAG); /* needed by template dialog */
       xstr2 = XmStringCreate(_("Help"), XmFONTLIST_DEFAULT_TAG);
