@@ -80,13 +80,13 @@ static void edit_find_cancel_callback(Widget w, XtPointer context, XtPointer inf
 
 static void make_edit_find_dialog(bool managed)
 {
-  Widget dl, rc;
-  Arg args[20];
-  int n;
-  XmString xmstr1, xmstr3, titlestr;
-
   if (!edit_find_dialog)
     {
+      Widget dl, rc;
+      Arg args[20];
+      int n;
+      XmString xmstr1, xmstr3, titlestr;
+
       n = 0;
       if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->basic_color); n++;}
       xmstr1 = XmStringCreate(_("Dismiss"), XmFONTLIST_DEFAULT_TAG);
@@ -179,11 +179,10 @@ static XEN g_find_dialog(XEN managed)
 {
   #define H_find_dialog "(" S_find_dialog "): create and activate the Edit:Find dialog, return the dialog widget"
   XEN_ASSERT_TYPE(XEN_BOOLEAN_IF_BOUND_P(managed), managed, XEN_ONLY_ARG, S_find_dialog, "a boolean");
-  make_edit_find_dialog(XEN_TO_C_BOOLEAN_OR_TRUE(managed));
+  make_edit_find_dialog(XEN_TO_C_BOOLEAN(managed));
   return(XEN_WRAP_WIDGET(edit_find_dialog));
 }
 
-#if DEBUGGING
 static XEN g_find_dialog_widgets(void)
 {
   if (edit_find_dialog)
@@ -195,25 +194,18 @@ static XEN g_find_dialog_widgets(void)
 			XEN_EMPTY_LIST))))));
   return(XEN_EMPTY_LIST);
 }
-#endif
 
 #ifdef XEN_ARGIFY_1
 XEN_ARGIFY_1(g_find_dialog_w, g_find_dialog)
-#if DEBUGGING
-  XEN_NARGIFY_0(g_find_dialog_widgets_w, g_find_dialog_widgets)
-#endif
+XEN_NARGIFY_0(g_find_dialog_widgets_w, g_find_dialog_widgets)
 #else
 #define g_find_dialog_w g_find_dialog
-#if DEBUGGING
-  #define g_find_dialog_widgets_w g_find_dialog_widgets
-#endif
+#define g_find_dialog_widgets_w g_find_dialog_widgets
 #endif
 
 void g_init_gxfind(void)
 {
   XEN_DEFINE_PROCEDURE(S_find_dialog, g_find_dialog_w, 0, 1, 0, H_find_dialog);
-#if DEBUGGING
-  XEN_DEFINE_PROCEDURE("find-dialog-widgets", g_find_dialog_widgets_w, 0, 0, 0, "");
-#endif
+  XEN_DEFINE_PROCEDURE("find-dialog-widgets", g_find_dialog_widgets_w, 0, 0, 0, "internal auto-test function");
 }
 
