@@ -39,12 +39,7 @@ enum {menu_menu,
 #define NUM_MENU_WIDGETS 99
 static Widget mw[NUM_MENU_WIDGETS];
 
-enum {W_pop_menu, W_pop_sep, W_pop_play, W_pop_undo, W_pop_redo, W_pop_save, W_pop_equalize_panes, W_pop_info};
-#define NUM_POPUP_CHILDREN 8
 static Widget popup_menu = NULL;
-static Widget popup_children[NUM_POPUP_CHILDREN];
-
-bool popup_menu_exists(void) {return(popup_menu != NULL);}
 
 Widget file_open_menu(void) {return(mw[f_open_menu]);}
 Widget file_close_menu(void) {return(mw[f_close_menu]);}
@@ -93,13 +88,6 @@ Widget options_focus_right_menu(void) {return(mw[o_focus_right_menu]);}
 Widget options_focus_middle_menu(void) {return(mw[o_focus_middle_menu]);}
 Widget options_focus_active_menu(void) {return(mw[o_focus_active_menu]);}
 
-Widget popup_play_menu(void) {return(popup_children[W_pop_play]);}
-Widget popup_undo_menu(void) {return(popup_children[W_pop_undo]);}
-Widget popup_redo_menu(void) {return(popup_children[W_pop_redo]);}
-Widget popup_save_menu(void) {return(popup_children[W_pop_save]);}
-Widget popup_equalize_panes_menu(void) {return(popup_children[W_pop_equalize_panes]);}
-Widget popup_info_menu(void) {return(popup_children[W_pop_info]);}
-
 void set_menu_label(Widget w, const char *label) {if (w) set_button_label(w, label);}
 
 /* -------------------------------- FILE MENU -------------------------------- */
@@ -121,9 +109,9 @@ static void file_print_callback_1(Widget w, XtPointer info, XtPointer context) {
 /* -------------------------------- EDIT MENU -------------------------------- */
 
 
-static void edit_mix_callback(Widget w, XtPointer info, XtPointer context) {add_selection_or_region(0, selected_channel(), "Edit: mix");}
+static void edit_mix_callback(Widget w, XtPointer info, XtPointer context) {add_selection_or_region(0, selected_channel());}
 static void edit_envelope_callback(Widget w, XtPointer info, XtPointer context) {create_envelope_editor();}
-static void edit_cut_callback(Widget w, XtPointer info, XtPointer context) {delete_selection("Edit: Cut", UPDATE_DISPLAY);}
+static void edit_cut_callback(Widget w, XtPointer info, XtPointer context) {delete_selection(UPDATE_DISPLAY);}
 static void edit_paste_callback(Widget w, XtPointer info, XtPointer context) {insert_selection_from_menu();}
 static void edit_save_as_callback(Widget w, XtPointer info, XtPointer context) {make_edit_save_as_dialog(true);}
 static void edit_select_all_callback(Widget w, XtPointer info, XtPointer context) {select_all(current_channel());}
@@ -141,7 +129,7 @@ static void edit_play_callback(Widget w, XtPointer info, XtPointer context)
     {
       set_menu_label(edit_play_menu(), _("Stop"));
       selection_play_stop = true;
-      play_selection(IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "play selection", 0);
+      play_selection(IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
     }
 }
 
@@ -714,7 +702,7 @@ static void GHC_callback(Widget w, XtPointer info, XtPointer context)
 Widget menu_widget(int which_menu)
 {
   unsigned int i;
-  Widget w, subw;
+  Widget w;
   CompositeWidget cw;
   int menu;
   if (which_menu == 5) return(popup_menu); /* special case -- not in main menuBar, presumably */
@@ -729,6 +717,7 @@ Widget menu_widget(int which_menu)
 	  /* fprintf(stderr,"%s: menu: %d, slot: %d (%x)\n", XtName(w), MENU_INDEX(menu), CALL_INDEX(menu), menu); */
 	  if (which_menu == MENU_INDEX(menu))
 	    {
+	      Widget subw;
 	      XtVaGetValues(w, XmNsubMenuId, &subw, NULL);
 	      return(subw);
 	    }
@@ -739,12 +728,12 @@ Widget menu_widget(int which_menu)
 
 static bool or_over_children(Widget w, bool (*func)(Widget, void *), void *userptr)
 {
-  unsigned int i;
   if (w)
     {
       if ((*func)(w, userptr)) return(true);
       if (XtIsComposite(w))
 	{
+	  unsigned int i;
 	  CompositeWidget cw = (CompositeWidget)w;
 	  for (i = 0; i < cw->composite.num_children; i++)
 	    if (or_over_children(cw->composite.children[i], func, userptr))
@@ -859,6 +848,19 @@ Widget g_add_to_menu(int which_menu, char *label, int callb, int position)
 
 /* -------------------------------- POPUP MENU -------------------------------- */
 
+enum {W_pop_menu, W_pop_sep, W_pop_play, W_pop_undo, W_pop_redo, W_pop_save, W_pop_equalize_panes, W_pop_info, W_pop_apply, W_pop_reset};
+#define NUM_POPUP_CHILDREN 10
+static Widget popup_children[NUM_POPUP_CHILDREN];
+
+bool popup_menu_exists(void) {return(popup_menu != NULL);}
+
+Widget popup_play_menu(void) {return(popup_children[W_pop_play]);}
+Widget popup_undo_menu(void) {return(popup_children[W_pop_undo]);}
+Widget popup_redo_menu(void) {return(popup_children[W_pop_redo]);}
+Widget popup_save_menu(void) {return(popup_children[W_pop_save]);}
+Widget popup_equalize_panes_menu(void) {return(popup_children[W_pop_equalize_panes]);}
+Widget popup_info_menu(void) {return(popup_children[W_pop_info]);}
+
 static bool stopping = false;
 
 static void popup_play_callback(Widget w, XtPointer info, XtPointer context) 
@@ -876,7 +878,7 @@ static void popup_play_callback(Widget w, XtPointer info, XtPointer context)
     {
       if (sp)
 	{
-	  play_sound(sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, C_TO_XEN_INT(AT_CURRENT_EDIT_POSITION), "popup play", 0);
+	  play_sound(sp, 0, NO_END_SPECIFIED, IN_BACKGROUND, AT_CURRENT_EDIT_POSITION);
 	  stopping = true;
 	  set_button_label(w, _("Stop playing"));
 	}
@@ -901,6 +903,9 @@ static void popup_info_callback(Widget w, XtPointer info, XtPointer context)
   if (sp) display_info(sp);
 }
 
+static void popup_apply_callback(Widget w, XtPointer info, XtPointer context) {menu_apply_controls(any_selected_sound());}
+static void popup_reset_callback(Widget w, XtPointer info, XtPointer context) {menu_reset_controls(any_selected_sound());}
+
 void post_popup(XButtonPressedEvent *event)
 {
   XmMenuPosition(popup_menu, event);
@@ -918,11 +923,12 @@ static void post_popup_menu(Widget w, XtPointer info, XEvent *event, Boolean *fl
 void create_popup_menu(void)
 {
   /* make it a child of the main window */
-  Widget mainp;
-  Arg args[20];
-  int n;
   if (!popup_menu)
     {
+      Widget mainp;
+      Arg args[20];
+      int n;
+
       n = 0;
       if (!(ss->using_schemes)) {XtSetArg(args[n], XmNbackground, (ss->sgx)->highlight_color); n++;}
       mainp = MAIN_PANE(ss);
@@ -955,19 +961,26 @@ void create_popup_menu(void)
       popup_children[W_pop_info] = XtCreateManagedWidget(_("Info"), xmPushButtonWidgetClass, popup_menu, args, n);
       XtVaSetValues(popup_children[W_pop_info], XmNsensitive, false, NULL);
       XtAddCallback(popup_children[W_pop_info], XmNactivateCallback, popup_info_callback, NULL);
+      popup_children[W_pop_apply] = XtCreateManagedWidget(_("Apply controls"), xmPushButtonWidgetClass, popup_menu, args, n);
+      XtVaSetValues(popup_children[W_pop_apply], XmNsensitive, true, NULL);
+      XtAddCallback(popup_children[W_pop_apply], XmNactivateCallback, popup_apply_callback, NULL);
+      popup_children[W_pop_reset] = XtCreateManagedWidget(_("Reset controls"), xmPushButtonWidgetClass, popup_menu, args, n);
+      XtVaSetValues(popup_children[W_pop_reset], XmNsensitive, true, NULL);
+      XtAddCallback(popup_children[W_pop_reset], XmNactivateCallback, popup_reset_callback, NULL);
     }
 }
 
 static XEN g_menu_widgets(void)
 {
-  #define H_menu_widgets "(" S_menu_widgets "): a list of top level menu widgets: ((0)main (1)file (2)edit (3)view (4)options (5)help)"
+  #define H_menu_widgets "(" S_menu_widgets "): a list of top level menu widgets: ((0)main (1)file (2)edit (3)view (4)options (5)help (6)popup)"
   return(XEN_CONS(XEN_WRAP_WIDGET(mw[menu_menu]),
 	  XEN_CONS(XEN_WRAP_WIDGET(mw[f_cascade_menu]),
            XEN_CONS(XEN_WRAP_WIDGET(mw[e_cascade_menu]),
             XEN_CONS(XEN_WRAP_WIDGET(mw[v_cascade_menu]),
              XEN_CONS(XEN_WRAP_WIDGET(mw[o_cascade_menu]),
               XEN_CONS(XEN_WRAP_WIDGET(mw[h_cascade_menu]),
-	       XEN_EMPTY_LIST)))))));
+	       XEN_CONS(XEN_WRAP_WIDGET(popup_menu),
+	        XEN_EMPTY_LIST))))))));
 }
 
 #ifdef XEN_ARGIFY_1
