@@ -34,7 +34,7 @@ static Float scroll_to_speed(Widget speed_number, int ival)
 static void speed_click_callback(Widget w, XtPointer context, XtPointer info) 
 {
   char speed_number_buffer[6];
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   mix_dialog_set_mix_speed(mix_dialog_id, 
 			   speed_changed(1.0,
 					 speed_number_buffer,
@@ -50,7 +50,7 @@ static void speed_drag_callback(Widget w, XtPointer context, XtPointer info)
 {
   int ival;
   ASSERT_WIDGET_TYPE(XmIsScrollBar(w), w);
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   ival = ((XmScrollBarCallbackStruct *)info)->value;
   if (!mix_dialog_slider_dragging) mix_dialog_start_drag(mix_dialog_id);
   mix_dialog_slider_dragging = true;
@@ -63,7 +63,7 @@ static void speed_valuechanged_callback(Widget w, XtPointer context, XtPointer i
 {
   XmScrollBarCallbackStruct *cb = (XmScrollBarCallbackStruct *)info;
   ASSERT_WIDGET_TYPE(XmIsScrollBar(w), w);
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   mix_dialog_slider_dragging = false;
   mix_dialog_set_mix_speed(mix_dialog_id, 
 			   scroll_to_speed(w_speed_number, cb->value),
@@ -111,7 +111,7 @@ static void change_mix_amp(int mix_id, int chan, Float val)
 static void amp_click_callback(Widget w, XtPointer context, XtPointer info) 
 {
   int chan;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   XtVaGetValues(w, XmNuserData, &chan, NULL);
   change_mix_amp(mix_dialog_id, chan, 1.0);
   XtVaSetValues(w_amps[chan], XmNvalue, amp_to_scroll(amp_control_min(ss), 1.0, amp_control_max(ss)), NULL);
@@ -121,7 +121,7 @@ static void amp_drag_callback(Widget w, XtPointer context, XtPointer info)
 {
   int ival, chan;
   ASSERT_WIDGET_TYPE(XmIsScrollBar(w), w);
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   XtVaGetValues(w, XmNuserData, &chan, NULL);
   ival = ((XmScrollBarCallbackStruct *)info)->value;
   if (!mix_dialog_slider_dragging) mix_dialog_start_drag(mix_dialog_id);
@@ -132,7 +132,7 @@ static void amp_drag_callback(Widget w, XtPointer context, XtPointer info)
 static void amp_valuechanged_callback(Widget w, XtPointer context, XtPointer info) 
 {
   int ival, chan;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   ival = ((XmScrollBarCallbackStruct *)info)->value;
   XtVaGetValues(w, XmNuserData, &chan, NULL);
   ASSERT_WIDGET_TYPE(XmIsScrollBar(w), w);
@@ -169,13 +169,12 @@ static void show_mix_background_wave(int mix_id, int chan)
 
 static void mix_amp_env_resize(Widget w, XtPointer context, XtPointer info) 
 {
-  XGCValues gv;
   int chans, chan;
   env **e;
-  env *cur_env;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   if (ax == NULL)
     {
+      XGCValues gv;
       gv.function = GXcopy;
       XtVaGetValues(w_env, XmNbackground, &gv.background, XmNforeground, &gv.foreground, NULL);
       cur_gc = XtGetGC(w_env, GCForeground | GCFunction, &gv);
@@ -189,16 +188,17 @@ static void mix_amp_env_resize(Widget w, XtPointer context, XtPointer info)
   e = mix_dialog_envs(mix_dialog_id);
   for (chan = 0; chan < chans; chan++)
     {
+      env *cur_env;
       spfs[chan]->with_dots = true;
       env_editor_display_env(spfs[chan], e[chan], ax, _("mix env"), (int)(chan * widget_width(w) / chans), 0,
-			     widget_width(w) / chans, widget_height(w), false);
+			     widget_width(w) / chans, widget_height(w), NOT_PRINTING);
       cur_env = mix_dialog_mix_amp_env(mix_dialog_id, chan);
       if (cur_env)
 	{
 	  XSetForeground(MAIN_DISPLAY(ss), ax->gc, (ss->sgx)->enved_waveform_color);
 	  spfs[chan]->with_dots = false;
 	  env_editor_display_env(spfs[chan], cur_env, ax, _("mix env"), (int)(chan * widget_width(w) / chans), 0,
-				 widget_width(w) / chans, widget_height(w), false);
+				 widget_width(w) / chans, widget_height(w), NOT_PRINTING);
 	  XSetForeground(MAIN_DISPLAY(ss), ax->gc, (ss->sgx)->black);
 	}
       if (with_mix_background_wave)
@@ -216,7 +216,7 @@ static void mix_drawer_button_motion(Widget w, XtPointer context, XEvent *event,
   int chans, chan;
   env *e;
   Float pos;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
 #ifdef MAC_OSX
   if ((press_x == ev->x) && (press_y == ev->y)) return;
 #endif
@@ -235,7 +235,7 @@ static void mix_drawer_button_press(Widget w, XtPointer context, XEvent *event,
   int chans, chan;
   env *e;
   Float pos;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
 #ifdef MAC_OSX
   press_x = ev->x;
   press_y = ev->y;
@@ -255,7 +255,7 @@ static void mix_drawer_button_release(Widget w, XtPointer context, XEvent *event
   int chans, chan;
   env *e;
   Float pos;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   chans = mix_dialog_mix_input_chans(mix_dialog_id);
   pos = (Float)(ev->x) / (Float)widget_width(w);
   chan = (int)(pos * chans);
@@ -275,7 +275,7 @@ static void track_activated(void)
 {
   char *val;
   track_changed = false;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   val = XmTextGetString(w_track);
   if (val)
     {
@@ -285,10 +285,8 @@ static void track_activated(void)
 	mix_dialog_set_mix_track(mix_dialog_id, trk);
       else
 	{
-	  char lab[16];
 	  XmTextSetString(w_beg, _("track must be >= 0"));
-	  mus_snprintf(lab, 16, "%d", mix_dialog_mix_track(mix_dialog_id));
-	  XmTextSetString(w_track, lab);
+	  widget_int_to_text(w_track, mix_dialog_mix_track(mix_dialog_id));
 	}
       XtFree(val);
     }
@@ -311,11 +309,11 @@ static bool id_changed = false;
 static void id_activated(void)
 {
   char *val;
-  int id;
   id_changed = false;
   val = XmTextGetString(w_id);
   if (val)
     {
+      int id;
       id = string_to_int(val);
       if (mix_ok_and_unlocked(id))
 	{
@@ -339,11 +337,11 @@ static void id_check_callback(Widget w, XtPointer context, XtPointer info)
 static void beg_activated(void)
 {
   char *val;
-  chan_info *cp;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   val = XmTextGetString(w_beg);
   if (val)
     {
+      chan_info *cp;
       char *up_to_colon;
       up_to_colon = string_to_colon(val);
       cp = mix_dialog_mix_channel(mix_dialog_id);
@@ -359,7 +357,7 @@ static void apply_mix_dialog_callback(Widget w, XtPointer context, XtPointer inf
   /* set all mix amp envs, last one should remix */
   int i, chans;
   env **envs;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   chans = mix_dialog_mix_input_chans(mix_dialog_id);
   envs = mix_dialog_envs(mix_dialog_id);
   for (i = 0; i < chans; i++)
@@ -496,7 +494,7 @@ static void mix_previous_callback(Widget w, XtPointer context, XtPointer info)
 static void mix_dialog_pan_callback(Widget w, XtPointer context, XtPointer info) 
 {
   bool inverted;
-  if (!(mix_ok(mix_dialog_id))) return;
+  if (!(mix_ok_and_unlocked(mix_dialog_id))) return;
   inverted = (!(mix_dialog_mix_inverted(mix_dialog_id)));
   if (!(ss->using_schemes))
     XmChangeColor(w_mix_pan, (inverted) ? (ss->sgx)->yellow : (ss->sgx)->highlight_color);
@@ -536,16 +534,17 @@ static Widget w_sep1;
 
 Widget make_mix_dialog(void) 
 {
-  Widget mainform, mix_row, track_row, last_label, last_number, mix_frame, track_frame, sep;
-  Widget w_dB_frame, w_dB, w_clip, w_wave, w_dB_row;
-  XmString xdismiss, xhelp, xtitle, s1, xapply;
-  int n, i;
-  Arg args[20];
-  XtCallbackList n1, n2;
-  XGCValues v;
-  char amplab[LABEL_BUFFER_SIZE];
   if (mix_dialog == NULL)
     {
+      Widget mainform, mix_row, track_row, last_label, last_number, mix_frame, track_frame, sep;
+      Widget w_dB_frame, w_dB, w_clip, w_wave, w_dB_row;
+      XmString xdismiss, xhelp, xtitle, s1, xapply;
+      int n, i;
+      Arg args[20];
+      XtCallbackList n1, n2;
+      XGCValues v;
+      char amplab[LABEL_BUFFER_SIZE];
+
       mix_dialog_id = any_mix_id();
       xdismiss = XmStringCreate(_("Dismiss"), XmFONTLIST_DEFAULT_TAG);
       xapply = XmStringCreate(_("Apply Env"), XmFONTLIST_DEFAULT_TAG);
@@ -959,11 +958,6 @@ static bool track_dialog_slider_dragging = false;
 
 static void update_mix_dialog(int mix_id) 
 {
-  chan_info *cp = NULL;
-  int i, chans;
-  off_t beg, len;
-  Float val;
-  char lab[LABEL_BUFFER_SIZE];
   if (!(mix_ok(mix_dialog_id)))
     {
       mix_dialog_id = mix_id; /* close-sound kills current mix, for example */
@@ -975,6 +969,8 @@ static void update_mix_dialog(int mix_id)
     }
   if ((mix_id == mix_dialog_id) || (mix_id == ANY_MIX_ID))
     {
+      Float val;
+      int chans;
       if (mix_dialog == NULL) 
 	make_mix_dialog();
       else
@@ -985,20 +981,22 @@ static void update_mix_dialog(int mix_id)
       /* now reflect current mix state in mix dialog controls */
       if (mix_ok(mix_dialog_id))
 	{
+	  chan_info *cp = NULL;
+	  off_t beg, len;
+	  char lab[LABEL_BUFFER_SIZE];
 	  cp = mix_dialog_mix_channel(mix_dialog_id);
 	  val = mix_dialog_mix_speed(mix_dialog_id);
 	  XtVaSetValues(w_speed, XmNvalue, speed_to_scroll(speed_control_min(ss), val, speed_control_max(ss)), NULL);
 	  speed_changed(val, lab, speed_control_style(ss), speed_control_tones(ss), 6);
 	  set_label(w_speed_number, lab);
-	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%d", mix_dialog_mix_track(mix_dialog_id));
-	  XmTextSetString(w_track, lab);
-	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%d", mix_dialog_id);
-	  XmTextSetString(w_id, lab);
+	  widget_int_to_text(w_track, mix_dialog_mix_track(mix_dialog_id));
+	  widget_int_to_text(w_id, mix_dialog_id);
 	  beg = mix_dialog_mix_position(mix_dialog_id);
 	  len = mix_frames(mix_dialog_id);
-	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%.3f : %.3f",
+	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%.3f : %.3f%s",
 		       (float)((double)beg / (float)SND_SRATE(cp->sound)),
-		       (float)((double)(beg + len) / (float)SND_SRATE(cp->sound)));
+		       (float)((double)(beg + len) / (float)SND_SRATE(cp->sound)),
+		       (mix_ok_and_unlocked(mix_dialog_id)) ? "" : " (locked)");
 	  XmTextSetString(w_beg, lab);
 	  chans = mix_dialog_mix_input_chans(mix_dialog_id);
 	  if (chans == 0) return;
@@ -1022,6 +1020,7 @@ static void update_mix_dialog(int mix_id)
 	}
       if ((!mix_dialog_slider_dragging) && (!track_dialog_slider_dragging))
 	{
+	  int i;
 	  for (i = 0; i < chans; i++)
 	    {
 	      XmString s1;
@@ -1032,7 +1031,7 @@ static void update_mix_dialog(int mix_id)
 	      s1 = XmStringCreate(amplab, XmFONTLIST_DEFAULT_TAG);
 	      XtVaSetValues(w_amp_labels[i], XmNlabelString, s1, NULL);
 	      XmStringFree(s1);
-	      if (mix_ok(mix_dialog_id))
+	      if (mix_ok_and_unlocked(mix_dialog_id))
 		val = mix_dialog_mix_amp(mix_dialog_id, i);
 	      else val = 1.0;
 	      XtVaSetValues(w_amps[i], XmNvalue, mix_amp_to_scroll(val, i), NULL);
@@ -1247,12 +1246,12 @@ void show_track_background_wave(int pts, bool two_sided)
 
 static void track_amp_env_resize(Widget w, XtPointer context, XtPointer info) 
 {
-  XGCValues gv;
   env *e;
   env *cur_env;
   if (!(track_p(track_dialog_id))) return;
   if (track_ax == NULL)
     {
+      XGCValues gv;
       gv.function = GXcopy;
       XtVaGetValues(w_track_env, XmNbackground, &gv.background, XmNforeground, &gv.foreground, NULL);
       track_cur_gc = XtGetGC(w_track_env, GCForeground | GCFunction, &gv);
@@ -1264,13 +1263,13 @@ static void track_amp_env_resize(Widget w, XtPointer context, XtPointer info)
   else clear_window(track_ax);
   e = track_dialog_env(track_dialog_id);
   track_spf->with_dots = true;
-  env_editor_display_env(track_spf, e, track_ax, _("track env"), 0, 0, widget_width(w), widget_height(w), false);
+  env_editor_display_env(track_spf, e, track_ax, _("track env"), 0, 0, widget_width(w), widget_height(w), NOT_PRINTING);
   cur_env = track_dialog_track_amp_env(track_dialog_id);
   if (cur_env)
     {
       XSetForeground(MAIN_DISPLAY(ss), track_ax->gc, (ss->sgx)->enved_waveform_color);
       track_spf->with_dots = false;
-      env_editor_display_env(track_spf, cur_env, track_ax, _("track env"), 0, 0, widget_width(w), widget_height(w), false);
+      env_editor_display_env(track_spf, cur_env, track_ax, _("track env"), 0, 0, widget_width(w), widget_height(w), NOT_PRINTING);
       XSetForeground(MAIN_DISPLAY(ss), track_ax->gc, (ss->sgx)->black);
     }
   if (with_track_background_wave)
@@ -1339,19 +1338,15 @@ static void track_track_activated(void)
 	  if ((id == track_dialog_id) ||
 	      (!(set_track_track(track_dialog_id, id))))
 	    {
-	      char lab[16];
 	      set_label(w_track_text, _("circular track chain"));
-	      mus_snprintf(lab, 16, "%d", track_dialog_track_track(track_dialog_id));
-	      XmTextSetString(w_track_track, lab);
+	      widget_int_to_text(w_track_track, track_dialog_track_track(track_dialog_id));
 	    }
 	  else update_track_dialog(id);
 	}
       else
 	{
-	  char lab[16];
 	  set_label(w_track_text, _("track must be >= 0"));
-	  mus_snprintf(lab, 16, "%d", track_dialog_track_track(track_dialog_id));
-	  XmTextSetString(w_track_track, lab);
+	  widget_int_to_text(w_track_track, track_dialog_track_track(track_dialog_id));
 	}
       XtFree(val);
     }
@@ -1373,11 +1368,11 @@ static bool track_id_changed = false;
 static void track_id_activated(void)
 {
   char *val;
-  int id;
   track_id_changed = false;
   val = XmTextGetString(w_track_id);
   if (val)
     {
+      int id;
       id = string_to_int(val);
       if (track_p(id))
 	{
@@ -1405,12 +1400,12 @@ static void track_id_check_callback(Widget w, XtPointer context, XtPointer info)
 
 static void redisplay_track_bounds(void)
 {
-  char lab[LABEL_BUFFER_SIZE];
   chan_info *cp;
-  off_t beg, len;
   cp = track_channel(track_dialog_id, 0); /* can be NULL */
   if (cp)
     {
+      char lab[LABEL_BUFFER_SIZE];
+      off_t beg, len;
       beg = track_position(track_dialog_id, -1);
       len = track_frames(track_dialog_id, -1);
       mus_snprintf(lab, LABEL_BUFFER_SIZE, "%.3f : %.3f",
@@ -1427,11 +1422,11 @@ static void redisplay_track_bounds(void)
 static void track_beg_activated(void)
 {
   char *val;
-  chan_info *cp;
   if (!(track_p(track_dialog_id))) return;
   val = XmTextGetString(w_track_beg);
   if (val)
     {
+      chan_info *cp;
       cp = track_channel(track_dialog_id, 0);
       if (cp)
 	{
@@ -1600,15 +1595,16 @@ static Widget w_track_frame = NULL;
 
 Widget make_track_dialog(void) 
 {
-  Widget mainform, track_row, track_track_row, track_track_frame, sep, sep2;
-  Widget w_dB_frame, w_dB, w_clip, w_wave, w_dB_row;
-  XmString xdismiss, xhelp, xtitle, s1, xapply;
-  int n;
-  Arg args[20];
-  XtCallbackList n1, n2;
-  XGCValues v;
   if (track_dialog == NULL)
     {
+      Widget mainform, track_row, track_track_row, track_track_frame, sep, sep2;
+      Widget w_dB_frame, w_dB, w_clip, w_wave, w_dB_row;
+      XmString xdismiss, xhelp, xtitle, s1, xapply;
+      int n;
+      Arg args[20];
+      XtCallbackList n1, n2;
+      XGCValues v;
+
       track_dialog_id = any_track_id();
       xdismiss = XmStringCreate(_("Dismiss"), XmFONTLIST_DEFAULT_TAG);
       xapply = XmStringCreate(_("Apply Env"), XmFONTLIST_DEFAULT_TAG);
@@ -2070,11 +2066,6 @@ Widget make_track_dialog(void)
 
 static void update_track_dialog(int track_id) 
 {
-  chan_info *cp;
-  off_t beg, len;
-  Float val;
-  char lab[LABEL_BUFFER_SIZE];
-  char *temp = NULL;
   if (!(track_p(track_dialog_id))) track_dialog_id = track_id;
   if (!(track_p(track_dialog_id))) track_dialog_id = any_track_id();
   if (track_id == track_dialog_id)
@@ -2089,14 +2080,17 @@ static void update_track_dialog(int track_id)
       /* now reflect current track state in track dialog controls */
       if (track_p(track_dialog_id))
 	{
+	  chan_info *cp;
+	  off_t beg, len;
+	  Float val;
+	  char lab[LABEL_BUFFER_SIZE];
+	  char *temp = NULL;
 	  val = track_dialog_track_speed(track_dialog_id);
 	  XtVaSetValues(w_track_speed, XmNvalue, speed_to_scroll(speed_control_min(ss), val, speed_control_max(ss)), NULL);
 	  speed_changed(val, lab, speed_control_style(ss), speed_control_tones(ss), 6);
 	  set_label(w_track_speed_number, lab);
-	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%d", track_dialog_track_track(track_dialog_id));
-	  XmTextSetString(w_track_track, lab);
-	  mus_snprintf(lab, LABEL_BUFFER_SIZE, "%d", track_dialog_id);
-	  XmTextSetString(w_track_id, lab);
+	  widget_int_to_text(w_track_track, track_dialog_track_track(track_dialog_id));
+	  widget_int_to_text(w_track_id, track_dialog_id);
 	  val = track_dialog_track_tempo(track_dialog_id);
 	  XtVaSetValues(w_track_tempo, XmNvalue, tempo_to_scroll(tempo_control_min(ss), val, tempo_control_max(ss)), NULL);
 	  mus_snprintf(lab, 5, "%.2f", val);
@@ -2153,7 +2147,7 @@ void reflect_mix_or_track_change(int mix_id, int track_id, bool forced)
   if ((mix_dialog) && 
       (XtIsManaged(mix_dialog)))
     {
-      if ((forced) && (mix_ok(mix_id))) mix_dialog_id = mix_id;
+      if ((forced) && (mix_ok_and_unlocked(mix_id))) mix_dialog_id = mix_id;
       update_mix_dialog(mix_id);
       if (mix_id != INVALID_MIX_ID)
 	{
