@@ -13,9 +13,9 @@ static regrow *region_row(int n);
 
 void reflect_regions_in_region_browser(void)
 {
-  int i;
   if (rsp)
     {
+      int i;
       rsp->active = true;
       if (rsp->chans)
 	for (i = 0; i < rsp->nchans; i++)
@@ -25,9 +25,9 @@ void reflect_regions_in_region_browser(void)
 
 void reflect_no_regions_in_region_browser(void)
 {
-  int i;
   if (rsp)
     {
+      int i;
       rsp->active = false;
       if (rsp->chans)
 	for (i = 0; i < rsp->nchans; i++)
@@ -61,9 +61,9 @@ void reflect_region_graph_style(void)
 
 static void unhighlight_region(void)
 {
-  regrow *oldr;
   if (current_region != -1)
     {
+      regrow *oldr;
       oldr = region_row(current_region);
       if (!(ss->using_schemes)) 
 	{
@@ -75,9 +75,9 @@ static void unhighlight_region(void)
 
 static void highlight_region(void)
 {
-  regrow *oldr;
   if (current_region != -1)
     {
+      regrow *oldr;
       oldr = region_row(current_region);
       if (!(ss->using_schemes)) 
 	{
@@ -243,9 +243,9 @@ static void region_focus_callback(Widget w, XtPointer context, XtPointer info)
 
 void reflect_play_region_stop(int n)
 {
-  regrow *rg;
   if (region_rows)
     {
+      regrow *rg;
       rg = region_row(region_id_to_list_position(n));
       if (rg) XmToggleButtonSetState(rg->pl, false, false);
     }
@@ -450,7 +450,7 @@ static void make_region_dialog(void)
   if (widget_width(region_dialog) < 400) set_widget_width(region_dialog, 400);
 
   id = region_list_position_to_id(0);
-  rsp = make_simple_channel_display(region_srate(id), region_len(id), WITH_ARROWS, region_graph_style(ss), region_grf, false);
+  rsp = make_simple_channel_display(region_srate(id), region_len(id), WITH_ARROWS, region_graph_style(ss), region_grf, WITHOUT_EVENTS);
   rsp->inuse = SOUND_REGION;
   current_region = 0;
   cp = rsp->chans[0];
@@ -501,10 +501,10 @@ bool region_dialog_is_active(void)
 
 void allocate_region_rows(int n)
 {
-  int i;
   if ((region_dialog) && 
       (n > region_rows_size))
     {
+      int i;
       region_rows = (regrow **)REALLOC(region_rows, n * sizeof(regrow *));
       for (i = region_rows_size; i < n; i++) region_rows[i] = NULL;
       region_rows_size = n;
@@ -513,9 +513,9 @@ void allocate_region_rows(int n)
 
 static regrow *region_row(int n)
 {
-  regrow *r;
   if (n < region_rows_size)
     {
+      regrow *r;
       if (region_rows[n] == NULL)
 	{
 	  r = make_regrow(region_ww, 
