@@ -24,7 +24,7 @@ static XmRenderTable get_xm_font(XFontStruct *ignore, char *font, char *tag)
   XtSetArg(args[n], XmNloadModel, XmLOAD_IMMEDIATE); n++;
   tmp = XmRenditionCreate(MAIN_SHELL(ss), tag, args, n);
   tabl = XmRenderTableAddRenditions(NULL, &tmp, 1, XmMERGE_NEW);
-  XmRenditionFree(tmp);
+  /* XmRenditionFree(tmp); */ /* valgrind thinks this is a bad idea */
   return(tabl);
 }
 #else
@@ -204,9 +204,9 @@ void map_over_children (Widget w, void (*func)(Widget, void *), void *userptr)
   /* apply func to each child in entire tree beneath top widget */
   /* taken from Douglas Young, "Motif Debugging and Performance Tuning" Prentice-Hall 1995 */
   /* used mostly to get colors right in non-SGI environments with "convenience" widgets */
-  unsigned int i;
   if (w)
     {
+      unsigned int i;
       (*func)(w, userptr);
       if (XtIsComposite(w))
 	{
@@ -230,9 +230,9 @@ void raise_dialog(Widget w)
   /* since we're using non-transient message dialogs, the dialog window can become completely
    * hidden behind other windows, with no easy way to raise it back to the top, so...
    */
-  Widget parent;
   if ((w) && (XtIsManaged(w)))
     {
+      Widget parent;
       parent = XtParent(w);
       if ((parent) && 
 	  (XtIsSubclass(parent, xmDialogShellWidgetClass)))
@@ -281,7 +281,6 @@ static bool complain_about_focus_policy = true;
 
 void goto_window(Widget text)
 {
-  int err;
   if ((XmIsTraversable(text)) && (1))
     /*
       there's a major memory leak here
@@ -292,6 +291,7 @@ void goto_window(Widget text)
 	{
 	  if (complain_about_focus_policy)
 	    {
+	      int err;
 	      XtVaGetValues(text, XmNkeyboardFocusPolicy, &err, NULL);
 	      if (err == XmEXPLICIT)
 		snd_error("goto_window: traverse to %s failed!", XtName(text));
@@ -523,6 +523,9 @@ void fixup_axis_context(axis_context *ax, Widget w, GC gc)
 {
   ax->dp = XtDisplay(w);
   ax->wn = XtWindow(w);
+#if DEBUGGING
+  if (!(ax->wn)) {fprintf(stderr, "fixup axis widget %p's window is null?", w); abort();}
+#endif
   if (gc) ax->gc = gc;
 }
 
@@ -561,3 +564,32 @@ int attach_all_sides(Arg *args, int n)
   XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
   return(n);
 }
+
+
+void widget_int_to_text(Widget w, int val)
+{
+  char *str;
+  str = (char *)CALLOC(8, sizeof(char));
+  mus_snprintf(str, 8, "%d", val);
+  XmTextFieldSetString(w, str);
+  FREE(str);
+}
+
+void widget_float_to_text(Widget w, Float val)
+{
+  char *str;
+  str = (char *)CALLOC(16, sizeof(char));
+  mus_snprintf(str, 16, "%.2f", val);
+  XmTextFieldSetString(w, str);
+  FREE(str);
+}
+
+void widget_off_t_to_text(Widget w, off_t val)
+{
+  char *str;
+  str = (char *)CALLOC(32, sizeof(char));
+  mus_snprintf(str, 32, OFF_TD, val);
+  XmTextFieldSetString(w, str);
+  FREE(str);
+}
+
