@@ -11,13 +11,22 @@ static XEN g_in(XEN ms, XEN code)
   #define H_in "(" S_in " msecs thunk): invoke thunk in msecs milliseconds (named call_in in Ruby)"
   XEN_ASSERT_TYPE(XEN_NUMBER_P(ms), ms, XEN_ARG_1, S_in, "a number");
   XEN_ASSERT_TYPE(XEN_PROCEDURE_P(code), code, XEN_ARG_2, S_in, "a procedure");
-  if (XEN_REQUIRED_ARGS(code) == 0)
+  if (XEN_REQUIRED_ARGS_OK(code, 0))
     {
-      XtAppAddTimeOut(MAIN_APP(ss), 
-		      (unsigned long)XEN_TO_C_INT(ms),
-		      (XtTimerCallbackProc)timed_eval, 
-		      (XtPointer)code);
-      snd_protect(code);
+      int secs;
+      secs = XEN_TO_C_INT(ms);
+      if (secs < 0) 
+	XEN_OUT_OF_RANGE_ERROR(S_in, XEN_ARG_1, ms, "a positive integer");
+      else
+	{
+	  XtAppAddTimeOut(MAIN_APP(ss), 
+			  (unsigned long)secs,
+			  (XtTimerCallbackProc)timed_eval, 
+			  (XtPointer)code);
+	  /* this "code" can still be something misleading like an applicable smob */
+	  /*   there's a way to catch that in snd-run.c line 129, but not sure it's the "right thing" here */
+	  snd_protect(code);
+	}
     }
   else XEN_BAD_ARITY_ERROR(S_in, 2, code, "should take no args");
   return(ms);
@@ -71,10 +80,10 @@ static XEN g_make_snd_color(XEN r, XEN g, XEN b)
 void color_unselected_graphs(color_t color)
 {
   int i, j;
-  chan_info *cp;
-  snd_info *sp;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      chan_info *cp;
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse != SOUND_WRAPPER))
 	for (j = 0; j < sp->allocated_chans; j++)
@@ -89,10 +98,10 @@ void color_unselected_graphs(color_t color)
 void color_chan_components(color_t color, slider_choice_t which_component)
 {
   int i, j;
-  chan_info *cp;
-  snd_info *sp;
   for (i = 0; i < ss->max_sounds; i++)
     {
+      chan_info *cp;
+      snd_info *sp;
       sp = ss->sounds[i];
       if ((sp) && (sp->inuse != SOUND_WRAPPER))
 	for (j = 0; j < sp->allocated_chans; j++)
