@@ -117,6 +117,7 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
 
 #if HAVE_SYS_FPU_H
   union fpc_csr f; f.fc_word = get_fpc_csr(); f.fc_struct.flush = 1; set_fpc_csr(f.fc_word);
+  /* what the hell? -- can this be deleted? */
 #endif
 
 #if HAVE_GSL
@@ -134,6 +135,14 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   #endif
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+  /*
+    (bindtextdomain "snd" "/usr/local/share/locale")
+    (textdomain "snd")
+    (define _ gettext)
+    (display (_ "no selection"))
+
+    but that is limited to Snd's messages
+  */
 #endif
 
   ss = (snd_state *)CALLOC(1, sizeof(snd_state));
@@ -203,6 +212,7 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   ss->With_GL = DEFAULT_WITH_GL;
   ss->With_Background_Processes = DEFAULT_WITH_BACKGROUND_PROCESSES;
   ss->Dot_Size = DEFAULT_DOT_SIZE;
+  ss->Grid_Density = DEFAULT_GRID_DENSITY;
   ss->Cursor_Size = DEFAULT_CURSOR_SIZE;
   ss->Cursor_Style = DEFAULT_CURSOR_STYLE;
   ss->cursor_proc = XEN_UNDEFINED;
@@ -228,6 +238,7 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   ss->Show_Transform_Peaks = DEFAULT_SHOW_TRANSFORM_PEAKS;
   ss->Show_Y_Zero = DEFAULT_SHOW_Y_ZERO;
   ss->Show_Grid = DEFAULT_SHOW_GRID;
+  ss->Show_Sonogram_Cursor = DEFAULT_SHOW_SONOGRAM_CURSOR;
   ss->Show_Axes = DEFAULT_SHOW_AXES;
   ss->Show_Marks = DEFAULT_SHOW_MARKS;
   ss->Show_Indices = DEFAULT_SHOW_INDICES;
@@ -252,6 +263,8 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   ss->Mix_Waveform_Height = DEFAULT_MIX_WAVEFORM_HEIGHT;
   ss->Mix_Tag_Width = DEFAULT_MIX_TAG_WIDTH;
   ss->Mix_Tag_Height = DEFAULT_MIX_TAG_HEIGHT;
+  ss->Mark_Tag_Width = DEFAULT_MARK_TAG_WIDTH;
+  ss->Mark_Tag_Height = DEFAULT_MARK_TAG_HEIGHT;
   if (DEFAULT_SAVE_STATE_FILE != (char *)NULL) ss->Save_State_File = copy_string(DEFAULT_SAVE_STATE_FILE); else ss->Save_State_File = NULL;
   ss->Enved_Base = DEFAULT_ENVED_BASE;
   ss->Enved_Power = DEFAULT_ENVED_POWER;
@@ -260,7 +273,6 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   ss->Enved_Target = DEFAULT_ENVED_TARGET;
   ss->Dac_Size = DEFAULT_DAC_SIZE;
   ss->Dac_Combines_Channels = DEFAULT_DAC_COMBINES_CHANNELS;
-  ss->Emacs_Style_Save_As = DEFAULT_EMACS_STYLE_SAVE_AS;
   ss->Auto_Update_Interval = DEFAULT_AUTO_UPDATE_INTERVAL;
   ss->Cursor_Update_Interval = DEFAULT_CURSOR_UPDATE_INTERVAL;
   ss->Cursor_Location_Offset = DEFAULT_CURSOR_LOCATION_OFFSET;
@@ -269,6 +281,7 @@ static void snd_gsl_error(const char *reason, const char *file, int line, int gs
   allocate_regions(max_regions(ss));
   ss->HTML_Dir = NULL;
   ss->HTML_Program = copy_string(DEFAULT_HTML_PROGRAM);
+  ss->Log_Freq_Start = DEFAULT_LOG_FREQ_START;
   ss->min_dB = DEFAULT_MIN_DB;
   ss->lin_dB = pow(10.0, DEFAULT_MIN_DB * 0.05);
   ss->init_window_x = DEFAULT_INIT_WINDOW_X; 
