@@ -3,99 +3,6 @@
 (use-modules (ice-9 format) (ice-9 optargs))
 (provide 'snd-snd4.scm)
 
-(if #f
-    ;; these are the Snd-4 names 
-    (begin
-      (define smooth            smooth-sound)
-      (define cut               delete-selection)
-      (define call-apply        apply-controls)
-      (define (open-alternate-sound file) (close-sound) (open-sound file))
-      (define normalize-view    equalize-panes)
-      (define save-control-panel save-controls)
-      (define restore-control-panel restore-controls)
-      (define reset-control-panel reset-controls)
-      (define mark->sound       mark-home)
-      (define (mix-sound-index m) (car (mix-home m)))
-      (define (mix-sound-channel m) (cadr (mix-home m)))
-      (define amp               amp-control)
-      (define contrast          contrast-control)
-      (define contrast-amp      contrast-control-amp)
-      (define contrast-func     #f)
-      (define contrasting       contrast-control?)
-      (define expand            expand-control)
-      (define expand-hop        expand-control-hop)
-      (define expand-length     expand-control-length)
-      (define expand-ramp       expand-control-ramp)
-      (define expanding         expand-control?)
-      (define filtering         filter-control?)
-      (define filter-order      filter-control-order)
-      (define filter-env        filter-control-env)
-      (define filter-dBing      filter-control-in-dB)
-      (define reverb-decay      reverb-control-decay)
-      (define reverb-feedback   reverb-control-feedback)
-      (define reverb-funcs      #f)
-      (define reverb-length     reverb-control-length)
-      (define reverb-lowpass    reverb-control-lowpass)
-      (define reverb-scale      reverb-control-scale)
-      (define reverbing         reverb-control?)
-      (define speed             speed-control)
-      (define speed-as-float    speed-control-as-float)
-      (define speed-as-ratio    speed-control-as-ratio)
-      (define speed-as-semitone speed-control-as-semitone)
-      (define speed-style       speed-control-style)
-      (define speed-tones       speed-control-tones)
-      (define filter-env-order  enved-filter-order)
-      (define enved-dBing       enved-in-dB)
-      (define enved-exping      enved-exp?)
-      (define enved-waving      enved-wave?)
-      (define enved-clipping    enved-clip?)
-      (define amplitude-env     enved-amplitude)
-      (define srate-env         enved-srate)
-      (define spectrum-env      enved-spectrum)
-      (define (hide-listener) (set! (show-listener) #f))
-      (define activate-listener show-listener)
-      (define dac-folding       dac-combines-channels)
-      (define focus-left        zoom-focus-left)
-      (define focus-right       zoom-focus-right)
-      (define focus-middle      zoom-focus-middle)
-      (define focus-active      zoom-focus-active)
-      (define x-to-one          x-axis-as-percentage)
-      (define x-in-seconds      x-axis-in-seconds)
-      (define x-in-samples      x-axis-in-samples)
-
-      (define graphing          graph-lisp?)
-      ;; old transform-size is shadowed by new version = transform-samples-size
-      (define waving            graph-time?)
-      (define ffting            graph-transform?)
-      (define fft-graph         transform-graph)
-      (define fft-beta          fft-window-beta)
-      (define fft-hook          transform-hook)
-      (define max-fft-peaks     max-transform-peaks)
-      (define show-fft-peaks    show-transform-peaks)
-      (define before-fft-hook   before-transform-hook)
-      (define fft-style         transform-graph-type)
-      (define normal-fft        graph-transform-once)
-      (define sonogram          graph-transform-as-sonogram)
-      (define spectrogram       graph-transform-as-spectrogram)
-      (define update-fft        update-transform)
-      (define update-graph      update-time-graph)
-      (define fft-size          transform-size)
-      (define wavo              time-graph-type) ;not quite right since wavo used booleans
-      (define yes-or-no-p       yes-or-no?)
-      (define corruption-time   auto-update-interval)
-
-      (define uniting 
-	(make-procedure-with-setter 
-	 (lambda arg 
-	   (apply channel-style arg)) 
-	 (lambda args 
-	   (if (= (length args) 1)
-	       (set! (channel-style) (car args))
-	       (set! (channel-style (car args)) (cadr args))))))
-
-      ))
-
-
 ;;; Snd-4 map/scan/temp functions
 
 (define* (scan-sound-chans proc #:optional (beg 0) end snd edpos)
