@@ -45,9 +45,7 @@
 #define S_mus_audio_read                "mus-audio-read"
 #define S_mus_audio_reclev              "mus-audio-reclev"
 #define S_mus_audio_report              "mus-audio-report"
-#define S_mus_audio_restore             "mus-audio-restore"
 #define S_mus_audio_samples_per_channel "mus-audio-samples-per-channel"
-#define S_mus_audio_save                "mus-audio-save"
 #define S_mus_audio_set_oss_buffers     "mus-audio-set-oss-buffers"
 #define S_mus_audio_spdif_in            "mus-audio-spdif-in"
 #define S_mus_audio_spdif_out           "mus-audio-spdif-out"
