@@ -2,21 +2,11 @@
 #define SNDLIB_H
 
 #define SNDLIB_VERSION 19
-#define SNDLIB_REVISION 1
-#define SNDLIB_DATE "27-Apr-04"
-
-/* try to figure out what type of machine (and in worst case, what OS) we're running on */
+#define SNDLIB_REVISION 7
+#define SNDLIB_DATE "15-Nov-04"
 
 #include <config.h>
 
-#if (!defined(WORDS_BIGENDIAN))
-   #define MUS_LITTLE_ENDIAN 1
-#endif
-
-#if defined(ALPHA) || defined(WINDOZE) || defined(__alpha__)
-  #define MUS_LITTLE_ENDIAN 1
-#endif
-
 #if (!(defined(MACOS))) && (defined(MPW_C) || defined(macintosh) || defined(__MRC__))
   #define MACOS 1
   #include <MacMemory.h>
@@ -25,42 +15,10 @@
 #endif
 
 #if HAVE_UNISTD_H && (!(defined(_MSC_VER)))
-  /* SEEK_END */
   #include <unistd.h>
 #endif
 
-#if defined(macosx) || (defined(__APPLE__) && defined(__MACH__))
-  #define MAC_OSX 1
-#endif
-
-#if (!defined(SGI)) && (!defined(LINUX)) && (!defined(MACOS)) && (!defined(SUN)) && (!defined(ALPHA)) && (!defined(WINDOZE)) && (!defined(MAC_OSX))
-  #if defined(__dest_os)
-    /* we're in Metrowerks Land */
-    #if (__dest_os == __mac_os)
-      #define MACOS 1
-    #endif
-  #else
-    #if macintosh
-      #define MACOS 1
-    #else
-      #if (__WINDOWS__) || (__NT__) || (_WIN32) || (__CYGWIN__)
-        #define WINDOZE 1
-        #define MUS_LITTLE_ENDIAN 1
-      #else
-        #ifdef __alpha__
-          #define ALPHA 1
-          #define MUS_LITTLE_ENDIAN 1
-        #endif
-      #endif
-    #endif
-  #endif
-#endif  
-
-#if MACOS
-  #define off_t long
-#else
-  #include <sys/types.h>
-#endif
+#include <sys/types.h>
 
 #ifndef __cplusplus
 #if HAVE_STDBOOL_H
@@ -81,7 +39,11 @@
 #endif
 
 #ifndef MUS_LITTLE_ENDIAN
-  #define MUS_LITTLE_ENDIAN 0
+  #if WORDS_BIGENDIAN
+    #define MUS_LITTLE_ENDIAN 0
+  #else
+    #define MUS_LITTLE_ENDIAN 1
+  #endif
 #endif
 
 #ifndef c__FUNCTION__
@@ -96,22 +58,11 @@
 #endif
 #endif
 
-/* this block needed because not everyone uses configure, and those who don't often have no clue what audio system they're using */
-/*   so, if nothing is set but we're on a system that looks linux-like and we can find the OSS headers, use OSS */
-
-#if (!(defined(ESD))) && (!(defined(HAVE_OSS))) && (!(defined(HAVE_ALSA))) && (!(defined(HAVE_JACK)))
-  #if defined(LINUX) || defined(HAVE_SOUNDCARD_H) || defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H) || defined(USR_LIB_OSS) || defined(USR_LOCAL_LIB_OSS) || defined(OPT_OSS) || defined(VAR_LIB_OSS) || defined(__FreeBSD__) || defined(__bsdi__)
-    #define HAVE_OSS 1
-  #endif
-#endif
-
 #if (!defined(M_PI))
   #define M_PI 3.14159265358979323846264338327
   #define M_PI_2 (M_PI / 2.0)
 #endif
 
-#define TWO_PI (2.0 * M_PI)
-
 #define POWER_OF_2_P(x)	((((x) - 1) & (x)) == 0)
 /* from sys/param.h */
 
@@ -175,9 +126,6 @@
   #define mus_sample_abs(Sample) fabs(Sample)
 #endif
 
-#define MUS_DAC_CHANNEL 252525
-#define MUS_DAC_REVERB 252520
-
 enum {MUS_UNSUPPORTED, MUS_NEXT, MUS_AIFC, MUS_RIFF, MUS_BICSF, MUS_NIST, MUS_INRS, MUS_ESPS, MUS_SVX, MUS_VOC, 
       MUS_SNDT, MUS_RAW, MUS_SMP, MUS_AVR, MUS_IRCAM, MUS_SD1, MUS_SPPACK, MUS_MUS10, MUS_HCOM, MUS_PSION, MUS_MAUD,
       MUS_IEEE, MUS_MATLAB, MUS_ADC, MUS_MIDI, MUS_SOUNDFONT, MUS_GRAVIS, MUS_COMDISCO, MUS_GOLDWAVE, MUS_SRFS,
@@ -268,7 +216,6 @@ enum {MUS_NO_ERROR, MUS_NO_FREQUENCY, MUS_NO_PHASE, MUS_NO_GEN, MUS_NO_LENGTH,
       MUS_NO_SUCH_FFT_WINDOW, MUS_UNSUPPORTED_DATA_FORMAT, MUS_HEADER_READ_FAILED,
       MUS_UNSUPPORTED_HEADER_TYPE,
       MUS_FILE_DESCRIPTORS_NOT_INITIALIZED, MUS_NOT_A_SOUND_FILE, MUS_FILE_CLOSED, MUS_WRITE_ERROR,
-      MUS_BOGUS_FREE, MUS_BUFFER_OVERFLOW, MUS_BUFFER_UNDERFLOW, MUS_FILE_OVERFLOW,
       MUS_HEADER_WRITE_FAILED, MUS_CANT_OPEN_TEMP_FILE, MUS_INTERRUPTED, MUS_BAD_ENVELOPE,
 
       MUS_AUDIO_CHANNELS_NOT_AVAILABLE, MUS_AUDIO_SRATE_NOT_AVAILABLE, MUS_AUDIO_FORMAT_NOT_AVAILABLE,
@@ -306,11 +253,11 @@ enum {MUS_NO_ERROR, MUS_NO_FREQUENCY, MUS_NO_PHASE, MUS_NO_GEN, MUS_NO_LENGTH,
     #define CREAT(File, Flags) creat((File), 0)
   #endif
 #else
-  #ifdef DEBUG_MEMORY
-    #define CALLOC(a, b)  mem_calloc((size_t)(a), (size_t)(b), c__FUNCTION__, __FILE__, __LINE__)
-    #define MALLOC(a)     mem_malloc((size_t)(a), c__FUNCTION__, __FILE__, __LINE__)
-    #define FREE(a)       {if ((void *)a == (void *)0x99999999) abort(); mem_free(a, c__FUNCTION__, __FILE__, __LINE__); a = (void *)0x99999999;}
-    #define REALLOC(a, b) mem_realloc(a, (size_t)(b), c__FUNCTION__, __FILE__, __LINE__)
+  #if DEBUGGING
+    #define CALLOC(a, b)  mem_calloc((a), (b), c__FUNCTION__, __FILE__, __LINE__)
+    #define MALLOC(a)     mem_malloc((a), c__FUNCTION__, __FILE__, __LINE__)
+    #define FREE(a)       mem_free(a, c__FUNCTION__, __FILE__, __LINE__)
+    #define REALLOC(a, b) mem_realloc(a, (b), c__FUNCTION__, __FILE__, __LINE__)
 
     #define OPEN(File, Flags, Mode) io_open((File), (Flags), (Mode), c__FUNCTION__, __FILE__, __LINE__)
     #define FOPEN(File, Flags) io_fopen((File), (Flags), c__FUNCTION__, __FILE__, __LINE__)
@@ -395,7 +342,7 @@ off_t mus_sound_comment_start(const char *arg);
 off_t mus_sound_comment_end(const char *arg);
 off_t mus_sound_length(const char *arg);
 int mus_sound_fact_samples(const char *arg);
-int mus_sound_write_date(const char *arg);
+time_t mus_sound_write_date(const char *arg);
 int mus_sound_type_specifier(const char *arg);
 int mus_sound_block_align(const char *arg);
 int mus_sound_bits_per_sample(const char *arg);
@@ -430,7 +377,6 @@ int mus_sound_close_output(int fd, off_t bytes_of_data);
 int mus_sound_read(int fd, int beg, int end, int chans, mus_sample_t **bufs);
 int mus_sound_write(int tfd, int beg, int end, int chans, mus_sample_t **bufs);
 off_t mus_sound_seek_frame(int tfd, off_t frame);
-off_t mus_sound_maxamp(const char *ifile, mus_sample_t *vals);
 off_t mus_sound_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t *times);
 int mus_sound_set_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t *times);
 bool mus_sound_maxamp_exists(const char *ifile);
@@ -460,8 +406,6 @@ int mus_audio_read_buffers(int line, int frames, int chans, mus_sample_t **bufs,
 
 int mus_audio_mixer_read(int dev, int field, int chan, float *val);
 int mus_audio_mixer_write(int dev, int field, int chan, float *val);
-void mus_audio_save(void);
-void mus_audio_restore(void);
 int mus_audio_initialize(void);
 #if HAVE_OSS || HAVE_ALSA
 int mus_audio_reinitialize(void); /* 29-Aug-01 for CLM/Snd bugfix? */
@@ -489,9 +433,6 @@ int mus_audio_compatible_format(int dev);
 /* -------- io.c -------- */
 
 int mus_file_open_descriptors(int tfd, const char *arg, int df, int ds, off_t dl, int dc, int dt);
-#ifndef SNDLIB_DISABLE_DEPRECATED
-  #define mus_file_set_descriptors(Tfd, Arg, Df, Ds, Dl, Dc, Dt) mus_file_open_descriptors(Tfd, Arg, Df, Ds, Dl, Dc, Dt)
-#endif
 int mus_file_open_read(const char *arg);
 bool mus_file_probe(const char *arg);
 int mus_file_open_write(const char *arg);
@@ -518,6 +459,10 @@ char *mus_file_fd_name(int tfd);
 int mus_file_set_chans(int tfd, int chans);
 float mus_file_prescaler(int tfd);
 float mus_file_set_prescaler(int tfd, float val);
+bool mus_file_data_clipped_default(void);
+bool mus_file_set_data_clipped_default(bool new_value);
+Float mus_file_prescaler_default(void);
+Float mus_file_set_prescaler_default(Float new_value);
 
 void mus_bint_to_char(unsigned char *j, int x);
 int mus_char_to_bint(const unsigned char *inp);
@@ -552,7 +497,6 @@ Float mus_fclamp(Float lo, Float val, Float hi);
 void mus_reset_io_c(void);
 void mus_reset_headers_c(void);
 void mus_reset_audio_c(void);
-void mus_set_rt_audio_p(int rt);
 
 
 
@@ -627,12 +571,12 @@ char *mus_midi_describe(void);
 #endif
 
 
-#ifdef DEBUG_MEMORY
+#if DEBUGGING
   /* snd-utils.c (only used in conjunction with Snd's memory tracking functions) */
-  void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line);
-  void *mem_malloc(size_t len, const char *func, const char *file, int line);
+  void *mem_calloc(int len, int size, const char *func, const char *file, int line);
+  void *mem_malloc(int len, const char *func, const char *file, int line);
   void mem_free(void *ptr, const char *func, const char *file, int line);
-  void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line);
+  void *mem_realloc(void *ptr, int size, const char *func, const char *file, int line);
   int io_open(const char *pathname, int flags, mode_t mode, const char *func, const char *file, int line);
   int io_creat(const char *pathname, mode_t mode, const char *func, const char *file, int line);
   int io_close(int fd, const char *func, const char *file, int line);
