@@ -218,7 +218,8 @@ static XEN g_mus_sound_comment(XEN gfilename)
 static XEN g_mus_sound_write_date(XEN filename) 
 {
   #define H_mus_sound_write_date "(" S_mus_sound_write_date " filename): write date of sound file"
-  return(gmus_sound(S_mus_sound_write_date, mus_sound_write_date, filename));
+  XEN_ASSERT_TYPE(XEN_STRING_P(filename), filename, XEN_ONLY_ARG, S_mus_sound_write_date, "a string"); 
+  return(C_TO_XEN_ULONG((unsigned long)mus_sound_write_date(local_mus_expand_filename(XEN_TO_C_STRING(filename)))));
 }
 
 static XEN g_mus_header_type_name(XEN type) 
@@ -304,10 +305,7 @@ in the sound cache; if it isn't, a call on " S_mus_sound_maxamp " has to open an
 static XEN g_mus_sound_maxamp(XEN file)
 {
   #define H_mus_sound_maxamp "(" S_mus_sound_maxamp " filename): maxamps in sound (a list of paired amps (floats) and locations (samples))"
-  int chans, i;
-  off_t rtn;
-  mus_sample_t *vals;
-  off_t *times;
+  int chans;
   char *filename;
   XEN res = XEN_EMPTY_LIST;
   XEN_ASSERT_TYPE(XEN_STRING_P(file), file, XEN_ONLY_ARG, S_mus_sound_maxamp, "a string");
@@ -315,6 +313,10 @@ static XEN g_mus_sound_maxamp(XEN file)
   chans = mus_sound_chans(filename);
   if (chans > 0)
     {
+      off_t rtn;
+      int i;
+      mus_sample_t *vals;
+      off_t *times;
       vals = (mus_sample_t *)CALLOC(chans, sizeof(mus_sample_t));
       times = (off_t *)CALLOC(chans, sizeof(off_t));
       rtn = mus_sound_maxamps(filename, chans, vals, times);
@@ -333,17 +335,18 @@ static XEN g_mus_sound_maxamp(XEN file)
 
 static XEN g_mus_sound_set_maxamp(XEN file, XEN vals)
 {
-  int i, j, chans, len;
-  mus_sample_t *mvals;
-  off_t *times;
+  int chans;
   char *filename;
-  XEN lst;
   XEN_ASSERT_TYPE(XEN_STRING_P(file), file, XEN_ARG_1, S_setB S_mus_sound_maxamp, "a string");
   XEN_ASSERT_TYPE(XEN_LIST_P(vals), vals, XEN_ARG_2, S_setB S_mus_sound_maxamp, "a list");
   filename = local_mus_expand_filename(XEN_TO_C_STRING(file));
   chans = mus_sound_chans(filename);
   if (chans > 0)
     {
+      XEN lst;
+      int i, j, len;
+      mus_sample_t *mvals;
+      off_t *times;
       len = XEN_LIST_LENGTH(vals);
       if (len < (chans * 2))
 	XEN_WRONG_TYPE_ARG_ERROR(S_setB S_mus_sound_maxamp, 2, vals, "max amp list length must = 2 * chans");
@@ -390,11 +393,11 @@ static mus_sample_t **get_sound_data(XEN arg)
 
 void sound_data_free(sound_data *v)
 {
-  int i;
   if (v)
     {
       if ((v->data) && (!(v->wrapped)))
 	{
+	  int i;
 	  for (i = 0; i < v->chans; i++) if (v->data[i]) FREE(v->data[i]);
 	  FREE(v->data);
 	}
@@ -420,12 +423,12 @@ XEN_MAKE_OBJECT_PRINT_PROCEDURE(sound_data, print_sound_data, sound_data_to_stri
 
 bool sound_data_equalp(sound_data *v1, sound_data *v2)
 {
-  int i, chn;
   if (v1 == v2) return(true);
   if ((v1) && (v2) &&
       (v1->chans == v2->chans) &&
       (v1->length == v2->length))
     {
+      int i, chn;
       for (chn = 0; chn < v1->chans; chn++)
 	for (i = 0; i < v1->length; i++)
 	  if (v1->data[chn][i] != v2->data[chn][i])
@@ -455,7 +458,7 @@ static XEN g_sound_data_chans(XEN obj)
   sound_data *v;
   XEN_ASSERT_TYPE(SOUND_DATA_P(obj), obj, XEN_ONLY_ARG, S_sound_data_chans, "a sound-data object");
   v = (sound_data *)XEN_OBJECT_REF(obj);
-  return(C_TO_SMALL_XEN_INT(v->chans));
+  return(C_TO_XEN_INT(v->chans));
 }
 
 sound_data *c_make_sound_data(int chans, int frames)
@@ -529,8 +532,6 @@ static XEN g_mus_sound_data_maxamp(XEN obj)
   #define H_sound_data_maxamp "(" S_sound_data_maxamp " sd): list of maxamps of data in sd"
   sound_data *v;
   int i, j, chans, len;
-  mus_sample_t mx;
-  mus_sample_t *buf;
   XEN lst = XEN_EMPTY_LIST;
   XEN_ASSERT_TYPE(SOUND_DATA_P(obj), obj, XEN_ARG_1, S_sound_data_maxamp, "a sound-data object");
   v = (sound_data *)XEN_OBJECT_REF(obj);
@@ -538,6 +539,8 @@ static XEN g_mus_sound_data_maxamp(XEN obj)
   len = v->length;
   for (i = chans - 1; i >= 0; i--)
     {
+      mus_sample_t mx;
+      mus_sample_t *buf;
       mx = MUS_SAMPLE_MIN;
       buf = v->data[i];
       for (j = 0; j < len; j++)
@@ -627,6 +630,55 @@ static XEN g_vct_to_sound_data(XEN vobj, XEN sdobj, XEN chan)
   return(sdobj);
 }
 
+#define S_sound_data_to_sound_data "sound-data->sound-data"
+static XEN g_sound_data_to_sound_data(XEN sd_in, XEN sd_out, XEN start, XEN frames, XEN cycle_length, XEN chan_offset)
+{
+  #define H_sound_data_to_sound_data "(" S_sound_data_to_sound_data " sd-in sd-out start frames): copies sound-data sd-in's \
+data from 0 for 'frames' frames into 'sd-out' starting at 'start', wrapping around if sd-out's end is reached."
+  sound_data *sdi, *sdo;
+  int len, beg, i, j = 0, ilen, olen, cycle, chans, offset = 0;
+  XEN_ASSERT_TYPE(SOUND_DATA_P(sd_in), sd_in, XEN_ARG_1, S_sound_data_to_sound_data, "a sound-data object");
+  XEN_ASSERT_TYPE(SOUND_DATA_P(sd_out), sd_out, XEN_ARG_2, S_sound_data_to_sound_data, "a sound-data object");
+  XEN_ASSERT_TYPE(XEN_INTEGER_P(start), start, XEN_ARG_3, S_sound_data_to_sound_data, "an integer");
+  XEN_ASSERT_TYPE(XEN_INTEGER_P(frames), frames, XEN_ARG_4, S_sound_data_to_sound_data, "an integer");
+  XEN_ASSERT_TYPE(XEN_INTEGER_P(cycle_length), cycle_length, XEN_ARG_5, S_sound_data_to_sound_data, "an integer");
+  XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(chan_offset), chan_offset, XEN_ARG_6, S_sound_data_to_sound_data, "an integer");
+  sdi = (sound_data *)XEN_OBJECT_REF(sd_in);
+  sdo = (sound_data *)XEN_OBJECT_REF(sd_out);
+  beg = XEN_TO_C_INT(start);
+  len = XEN_TO_C_INT(frames);
+  cycle = XEN_TO_C_INT(cycle_length);
+  if (XEN_INTEGER_P(chan_offset)) offset = XEN_TO_C_INT(chan_offset);
+  if (beg >= cycle) beg = 0;
+  ilen = sdi->length;
+  if (len > ilen) len = ilen;
+  olen = sdo->length;
+  if (beg >= olen) beg = 0;
+  if (cycle > olen) cycle = olen;
+  chans = sdo->chans;
+  if ((sdi->chans - offset) < chans) chans = sdi->chans - offset;
+  if ((beg + len) < cycle)
+    {
+      for (i = 0; i < chans; i++)
+	memmove((void *)(sdo->data[i] + beg), (void *)(sdi->data[i + offset]), (len * sizeof(mus_sample_t)));
+      j = beg + len;
+    }
+  else
+    {
+      int k;
+      for (i = offset; i < chans; i++)
+	{
+	  j = beg;
+	  for (k = 0; k < len; k++)
+	    {
+	      sdo->data[i][j++] = sdi->data[i][k];
+	      if (j == cycle) j = 0;
+	    }
+	}
+    }
+  return(C_TO_XEN_INT(j));
+}
+
 
 static XEN g_mus_sound_open_input(XEN file)
 {
@@ -647,10 +699,9 @@ The file size is normally set later via " S_mus_sound_close_output ". srate is a
 data-format is a sndlib format indicator such as " S_mus_bshort ", if #f if defaults to a format compatible with sndlib, \
 header-type is a sndlib type indicator such as " S_mus_aiff "; sndlib currently only writes 5 or so header types."
 
-  int fd = -1, df, ht, chns;
-  char *com = NULL;
+  int fd = -1, df;
   XEN_ASSERT_TYPE(XEN_STRING_P(file), file, XEN_ARG_1, S_mus_sound_open_output, "a string");
-  XEN_ASSERT_TYPE(XEN_NUMBER_OR_BOOLEAN_P(srate), srate, XEN_ARG_2, S_mus_sound_open_output, "a number or #f");
+  XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(srate), srate, XEN_ARG_2, S_mus_sound_open_output, "an integer or #f");
   XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(chans), chans, XEN_ARG_3, S_mus_sound_open_output, "an integer or #f");
   XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(data_format), data_format, XEN_ARG_4, S_mus_sound_open_output, "an integer (data-format id) or #f");
   XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(header_type), header_type, XEN_ARG_5, S_mus_sound_open_output, "an integer (header-type id) or #f");
@@ -660,15 +711,18 @@ header-type is a sndlib type indicator such as " S_mus_aiff "; sndlib currently
   else df = MUS_OUT_FORMAT;
   if (MUS_DATA_FORMAT_OK(df))
     {
+      int ht;
       ht = XEN_TO_C_INT_OR_ELSE(header_type, 0);
       if (MUS_HEADER_TYPE_OK(ht))
 	{
+	  int chns;
 	  chns = XEN_TO_C_INT_OR_ELSE(chans, 0);
 	  if (chns > 0)
 	    {
+	      char *com = NULL;
 	      if (XEN_STRING_P(comment)) com = XEN_TO_C_STRING(comment);
 	      fd = mus_sound_open_output(local_mus_expand_filename(XEN_TO_C_STRING(file)),
-					 XEN_TO_C_INT_OR_ELSE(srate, 0),
+					 XEN_TO_C_INT_OR_ELSE(srate, 22050),
 					 chns, df, ht, com);
 	    }
 	  else XEN_OUT_OF_RANGE_ERROR(S_mus_sound_open_output, 3, chans, "chans ~A <= 0?");
@@ -687,7 +741,7 @@ reopen (without alteration) filename for output \
 data-format and header-type are sndlib indicators such as " S_mus_bshort " or " S_mus_aiff ". \
 data-location should be retrieved from a previous call to " S_mus_sound_data_location "."
 
-  int fd = -1, df, ht, chns;
+  int fd = -1, df;
   XEN_ASSERT_TYPE(XEN_STRING_P(file), file, XEN_ARG_1, S_mus_sound_reopen_output, "a string");
   XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(chans), chans, XEN_ARG_2, S_mus_sound_reopen_output, "an integer or #f");
   XEN_ASSERT_TYPE(XEN_INTEGER_OR_BOOLEAN_P(data_format), data_format, XEN_ARG_3, S_mus_sound_reopen_output, "an integer (data-format id) or #f");
@@ -696,9 +750,11 @@ data-location should be retrieved from a previous call to " S_mus_sound_data_loc
   df = XEN_TO_C_INT_OR_ELSE(data_format, MUS_OUT_FORMAT);
   if (MUS_DATA_FORMAT_OK(df))
     {
+      int ht;
       ht = XEN_TO_C_INT_OR_ELSE(header_type, 0);
       if (MUS_HEADER_TYPE_OK(ht))
 	{
+	  int chns;
 	  chns = XEN_TO_C_INT_OR_ELSE(chans, 0);
 	  if (chns > 0)
 	    {
@@ -918,20 +974,6 @@ static XEN g_mus_audio_close(XEN line)
   return(C_TO_XEN_INT(res));
 }
 
-static XEN g_mus_audio_save (void) 
-{
-  #define H_mus_audio_save "(" S_mus_audio_save "): save the current audio state for a subsequent " S_mus_audio_restore "."
-  mus_audio_save(); 
-  return(XEN_FALSE);
-}
-
-static XEN g_mus_audio_restore (void) 
-{
-  #define H_mus_audio_restore "(" S_mus_audio_restore "): restore a previously saved audio hardware state"
-  mus_audio_restore(); 
-  return(XEN_FALSE);
-}
-
 static XEN g_mus_audio_systems (void) 
 {
   #define H_mus_audio_systems "(" S_mus_audio_systems "): number of audio systems; normally each sound card is a separate 'system'"
@@ -985,7 +1027,7 @@ from the audio line into sound-data sdata."
   inbytes = frms * sd->chans * mus_bytes_per_sample(fmt);
   inbuf = (char *)CALLOC(inbytes, sizeof(char));
   val = mus_audio_read(fd, inbuf, inbytes);
-  mus_file_read_buffer(fmt, 0, sd->chans, frms - 1, sd->data, inbuf);
+  mus_file_read_buffer(fmt, 0, sd->chans, frms, sd->data, inbuf); /* frms here because this is "nints" not "end"! */
   FREE(inbuf);
   return(xen_return_first(C_TO_XEN_INT(val), sdata));
 }
@@ -995,34 +1037,32 @@ static XEN g_mus_audio_mixer_read(XEN dev, XEN field, XEN chan, XEN vals)
   #define H_mus_audio_mixer_read "(" S_mus_audio_mixer_read " device field channel vals): read some portion of the sound card mixer state.\
 The device is the nominal audio device, normally " S_mus_audio_default ". The field describes what info we are requesting: \
 to get the devices max available chans, the field would be " S_mus_audio_channel ". The channel arg, when relevant, specifies \
-which channel we want info on, or how big the 'vals' array is (normally the latter).  The vals array is a vector in which \
-the requested info will be written by " S_mus_audio_mixer_read ".  So, \n\
-  (let ((vals (make-vector 32)))\n\
+which channel we want info on or the 'vals' vct length. \
+The requested info will be written into 'vals': \
+  (let ((vals (make-vct 32)))\n\
     (mus-audio-mixer-read mus-audio-default mus-audio-format 32 vals))\n\
-sets (vector-ref vals 0) to the default device's desired audio sample data format."
+sets (vct-ref vals 0) to the default device's desired audio sample data format."
 
   int val, i, len;
   float *fvals;
-  XEN *vdata;
+  vct *v = NULL;
   XEN_ASSERT_TYPE(XEN_INTEGER_P(dev), dev, XEN_ARG_1, S_mus_audio_mixer_read, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(field), field, XEN_ARG_2, S_mus_audio_mixer_read, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(chan), chan, XEN_ARG_3, S_mus_audio_mixer_read, "an integer");
-  XEN_ASSERT_TYPE(XEN_VECTOR_P(vals), vals, XEN_ARG_4, S_mus_audio_mixer_read, "a vector");
+  XEN_ASSERT_TYPE(VCT_P(vals), vals, XEN_ARG_4, S_mus_audio_mixer_read, "a vct");
   if (!(MUS_AUDIO_DEVICE_OK(MUS_AUDIO_DEVICE(XEN_TO_C_INT(dev)))))
     XEN_OUT_OF_RANGE_ERROR(S_mus_audio_mixer_read, 1, dev, "~A: invalid device");
   if (!(MUS_AUDIO_DEVICE_OK(MUS_AUDIO_DEVICE(XEN_TO_C_INT(field)))))
     XEN_OUT_OF_RANGE_ERROR(S_mus_audio_mixer_read, 2, field, "~A: invalid field");
-  len = XEN_VECTOR_LENGTH(vals);
-  if (len == 0)
-    fvals = (float *)CALLOC(1, sizeof(float));
-  else fvals = (float *)CALLOC(len, sizeof(float));
+  v = TO_VCT(vals);
+  len = v->length;
+  fvals = (float *)CALLOC(len, sizeof(float));
   val = mus_audio_mixer_read(XEN_TO_C_INT(dev),
 			     XEN_TO_C_INT(field),
 			     XEN_TO_C_INT(chan),
 			     fvals);
-  vdata = XEN_VECTOR_ELEMENTS(vals);
   for (i = 0; i < len; i++) 
-    vdata[i] = C_TO_XEN_DOUBLE(fvals[i]);
+    v->data[i] = fvals[i];
   FREE(fvals);
   return(xen_return_first(C_TO_XEN_INT(val), vals));
 }
@@ -1032,25 +1072,20 @@ static XEN g_mus_audio_mixer_write(XEN dev, XEN field, XEN chan, XEN vals)
   #define H_mus_audio_mixer_write "(" S_mus_audio_mixer_write " device field channel vals): change some portion of the sound card mixer state"
   int i, len, res;
   float *fvals;
-  XEN *vdata;
+  vct *v = NULL;
   XEN_ASSERT_TYPE(XEN_INTEGER_P(dev), dev, XEN_ARG_1, S_mus_audio_mixer_write, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(field), field, XEN_ARG_2, S_mus_audio_mixer_write, "an integer");
   XEN_ASSERT_TYPE(XEN_INTEGER_P(chan), chan, XEN_ARG_3, S_mus_audio_mixer_write, "an integer");
-  XEN_ASSERT_TYPE(XEN_VECTOR_P(vals), vals, XEN_ARG_4, S_mus_audio_mixer_write, "a vector");
+  XEN_ASSERT_TYPE(VCT_P(vals), vals, XEN_ARG_4, S_mus_audio_mixer_write, "a vct");
   if (!(MUS_AUDIO_DEVICE_OK(MUS_AUDIO_DEVICE(XEN_TO_C_INT(dev)))))
     XEN_OUT_OF_RANGE_ERROR(S_mus_audio_mixer_write, 1, dev, "~A: invalid device");
   if (!(MUS_AUDIO_DEVICE_OK(MUS_AUDIO_DEVICE(XEN_TO_C_INT(field)))))
     XEN_OUT_OF_RANGE_ERROR(S_mus_audio_mixer_write, 2, field, "~A: invalid field");
-  len = XEN_VECTOR_LENGTH(vals);
-  if (len == 0)
-    fvals = (float *)CALLOC(1, sizeof(float));
-  else
-    {
-      fvals = (float *)CALLOC(len, sizeof(float));
-      vdata = XEN_VECTOR_ELEMENTS(vals);
-      for (i = 0; i < len; i++) 
-	fvals[i] = XEN_TO_C_DOUBLE_OR_ELSE(vdata[i], 0.0);
-    }
+  v = TO_VCT(vals);
+  len = v->length;
+  fvals = (float *)CALLOC(len, sizeof(float));
+  for (i = 0; i < len; i++) 
+    fvals[i] = v->data[i];
   res = mus_audio_mixer_write(XEN_TO_C_INT(dev),
 			      XEN_TO_C_INT(field),
 			      XEN_TO_C_INT(chan),
@@ -1059,19 +1094,17 @@ static XEN g_mus_audio_mixer_write(XEN dev, XEN field, XEN chan, XEN vals)
   return(xen_return_first(C_TO_XEN_INT(res), vals));
 }
 
-static XEN g_mus_file_data_clipped(XEN fd)
+static XEN g_mus_file_data_clipped(void)
 {
-  #define H_mus_file_data_clipped "(" S_mus_file_data_clipped " fd): whether data associated with file fd is clipped or allowed to wrap around"
-  XEN_ASSERT_TYPE(XEN_INTEGER_P(fd), fd, XEN_ONLY_ARG, S_mus_file_data_clipped, "an integer");
-  return(C_TO_XEN_BOOLEAN(mus_file_data_clipped(XEN_TO_C_INT(fd))));
+  /* this sets the default value, not the local file value */
+  #define H_mus_file_data_clipped "(" S_mus_file_data_clipped "): whether sound data accessed from a file should be clipped"
+  return(C_TO_XEN_BOOLEAN(mus_file_data_clipped_default()));
 }
 
-static XEN g_mus_file_set_data_clipped(XEN fd, XEN clipped)
+static XEN g_mus_file_set_data_clipped(XEN clipped)
 {
-  XEN_ASSERT_TYPE(XEN_INTEGER_P(fd), fd, XEN_ARG_1, S_setB S_mus_file_data_clipped, "an integer");
-  XEN_ASSERT_TYPE(XEN_BOOLEAN_P(clipped), fd, XEN_ARG_2, S_setB S_mus_file_data_clipped, "a boolean");
-  return(C_TO_XEN_INT(mus_file_set_data_clipped(XEN_TO_C_INT(fd),
-						(!(XEN_FALSE_P(clipped))))));
+  XEN_ASSERT_TYPE(XEN_BOOLEAN_P(clipped), clipped, XEN_ONLY_ARG, S_setB S_mus_file_data_clipped, "a boolean");
+  return(C_TO_XEN_BOOLEAN(mus_file_set_data_clipped_default(XEN_TO_C_BOOLEAN(clipped))));
 }
 
 static XEN g_mus_file_prescaler(XEN fd)
@@ -1168,6 +1201,7 @@ XEN_NARGIFY_2(g_make_sound_data_w, g_make_sound_data)
 XEN_NARGIFY_1(g_mus_sound_data_p_w, g_mus_sound_data_p)
 XEN_NARGIFY_1(g_mus_sound_data_maxamp_w, g_mus_sound_data_maxamp)
 XEN_ARGIFY_3(g_sound_data_to_vct_w, g_sound_data_to_vct)
+XEN_ARGIFY_6(g_sound_data_to_sound_data_w, g_sound_data_to_sound_data)
 XEN_ARGIFY_3(g_vct_to_sound_data_w, g_vct_to_sound_data)
 XEN_NARGIFY_1(g_mus_sound_samples_w, g_mus_sound_samples)
 XEN_NARGIFY_2(g_mus_sound_set_samples_w, g_mus_sound_set_samples)
@@ -1201,13 +1235,11 @@ XEN_NARGIFY_1(g_mus_sound_open_input_w, g_mus_sound_open_input)
 XEN_NARGIFY_1(g_mus_sound_close_input_w, g_mus_sound_close_input)
 XEN_NARGIFY_1(g_mus_make_error_w, g_mus_make_error)
 XEN_NARGIFY_1(g_mus_audio_close_w, g_mus_audio_close)
-XEN_NARGIFY_0(g_mus_audio_save_w, g_mus_audio_save)
-XEN_NARGIFY_0(g_mus_audio_restore_w, g_mus_audio_restore)
 XEN_NARGIFY_0(g_mus_audio_systems_w, g_mus_audio_systems)
 XEN_NARGIFY_4(g_mus_audio_mixer_read_w, g_mus_audio_mixer_read)
 XEN_NARGIFY_4(g_mus_audio_mixer_write_w, g_mus_audio_mixer_write)
-XEN_NARGIFY_1(g_mus_file_data_clipped_w, g_mus_file_data_clipped)
-XEN_NARGIFY_2(g_mus_file_set_data_clipped_w, g_mus_file_set_data_clipped)
+XEN_NARGIFY_0(g_mus_file_data_clipped_w, g_mus_file_data_clipped)
+XEN_NARGIFY_1(g_mus_file_set_data_clipped_w, g_mus_file_set_data_clipped)
 XEN_NARGIFY_1(g_mus_file_prescaler_w, g_mus_file_prescaler)
 XEN_NARGIFY_2(g_mus_file_set_prescaler_w, g_mus_file_set_prescaler)
 XEN_NARGIFY_1(g_mus_expand_filename_w, g_mus_expand_filename)
@@ -1238,6 +1270,7 @@ XEN_NARGIFY_2(g_mus_audio_set_oss_buffers_w, g_mus_audio_set_oss_buffers)
 #define g_mus_sound_data_p_w g_mus_sound_data_p
 #define g_mus_sound_data_maxamp_w g_mus_sound_data_maxamp
 #define g_sound_data_to_vct_w g_sound_data_to_vct
+#define g_sound_data_to_sound_data_w g_sound_data_to_sound_data
 #define g_vct_to_sound_data_w g_vct_to_sound_data
 #define g_mus_sound_samples_w g_mus_sound_samples
 #define g_mus_sound_set_samples_w g_mus_sound_set_samples
@@ -1271,8 +1304,6 @@ XEN_NARGIFY_2(g_mus_audio_set_oss_buffers_w, g_mus_audio_set_oss_buffers)
 #define g_mus_sound_close_input_w g_mus_sound_close_input
 #define g_mus_make_error_w g_mus_make_error
 #define g_mus_audio_close_w g_mus_audio_close
-#define g_mus_audio_save_w g_mus_audio_save
-#define g_mus_audio_restore_w g_mus_audio_restore
 #define g_mus_audio_systems_w g_mus_audio_systems
 #define g_mus_audio_mixer_read_w g_mus_audio_mixer_read
 #define g_mus_audio_mixer_write_w g_mus_audio_mixer_write
@@ -1336,7 +1367,7 @@ static XEN sound_data_compare(XEN vr1, XEN vr2)
 	      return(C_TO_XEN_INT(1));
       len = v1->length - v2->length;
       if (len == 0) return(XEN_ZERO);
-      if (len > 0) return(C_TO_SMALL_XEN_INT(1));
+      if (len > 0) return(C_TO_XEN_INT(1));
     }
   return(C_TO_XEN_INT(-1));
 }
@@ -1453,54 +1484,53 @@ void mus_sndlib_xen_initialize(void)
   Init_Hook();
   rb_include_module(sound_data_tag, rb_mComparable);
   rb_include_module(sound_data_tag, rb_mEnumerable);
-  rb_define_method(sound_data_tag, "to_s", XEN_PROCEDURE_CAST print_sound_data, 0);
-  rb_define_method(sound_data_tag, "eql?", XEN_PROCEDURE_CAST equalp_sound_data, 1);
-  rb_define_method(sound_data_tag, "==", XEN_PROCEDURE_CAST equalp_sound_data, 1);
-  rb_define_method(sound_data_tag, "each", XEN_PROCEDURE_CAST sound_data_each, 0);
-  rb_define_method(sound_data_tag, "<=>", XEN_PROCEDURE_CAST sound_data_compare, 1);
-  rb_define_method(sound_data_tag, "[]", XEN_PROCEDURE_CAST g_sound_data_ref, 2);
-  rb_define_method(sound_data_tag, "[]=", XEN_PROCEDURE_CAST g_sound_data_set, 3);
-  rb_define_method(sound_data_tag, "length", XEN_PROCEDURE_CAST sound_data_size, 0);
-  rb_define_method(sound_data_tag, "size", XEN_PROCEDURE_CAST sound_data_size, 0);
-  rb_define_method(sound_data_tag, "new", XEN_PROCEDURE_CAST g_make_sound_data, 2);
-  rb_define_method(sound_data_tag, "fill", XEN_PROCEDURE_CAST sound_data_fill, 1);
-  rb_define_method(sound_data_tag, "dup", XEN_PROCEDURE_CAST sound_data_dup, 0);
+  rb_define_method(sound_data_tag, "to_s",   XEN_PROCEDURE_CAST print_sound_data,   0);
+  rb_define_method(sound_data_tag, "eql?",   XEN_PROCEDURE_CAST equalp_sound_data,  1);
+  rb_define_method(sound_data_tag, "==",     XEN_PROCEDURE_CAST equalp_sound_data,  1);
+  rb_define_method(sound_data_tag, "each",   XEN_PROCEDURE_CAST sound_data_each,    0);
+  rb_define_method(sound_data_tag, "<=>",    XEN_PROCEDURE_CAST sound_data_compare, 1);
+  rb_define_method(sound_data_tag, "[]",     XEN_PROCEDURE_CAST g_sound_data_ref,   2);
+  rb_define_method(sound_data_tag, "[]=",    XEN_PROCEDURE_CAST g_sound_data_set,   3);
+  rb_define_method(sound_data_tag, "length", XEN_PROCEDURE_CAST sound_data_size,    0);
+  rb_define_method(sound_data_tag, "size",   XEN_PROCEDURE_CAST sound_data_size,    0);
+  rb_define_method(sound_data_tag, "new",    XEN_PROCEDURE_CAST g_make_sound_data,  2);
+  rb_define_method(sound_data_tag, "fill",   XEN_PROCEDURE_CAST sound_data_fill,    1);
+  rb_define_method(sound_data_tag, "dup",    XEN_PROCEDURE_CAST sound_data_dup,     0);
 #endif
 
-  XEN_DEFINE_CONSTANT(S_mus_out_format, MUS_OUT_FORMAT, "sample format for fastest IO");
-
+  XEN_DEFINE_CONSTANT(S_mus_out_format,  MUS_OUT_FORMAT,  "sample format for fastest IO");
   XEN_DEFINE_CONSTANT(S_mus_unsupported, MUS_UNSUPPORTED, "unsupported header id");
-  XEN_DEFINE_CONSTANT(S_mus_next,    MUS_NEXT,    "NeXT (Sun) sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_aifc,    MUS_AIFC,    "AIFC sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_riff,    MUS_RIFF,    "RIFF (MS wave) sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_nist,    MUS_NIST,    "NIST (Sphere) sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_raw,     MUS_RAW,     "raw (headerless) sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_ircam,   MUS_IRCAM,   "IRCAM sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_aiff,    MUS_AIFF,    "AIFF (old-style) sound header id");
-  XEN_DEFINE_CONSTANT(S_mus_bicsf,   MUS_BICSF,   "BICSF header id");
-  XEN_DEFINE_CONSTANT(S_mus_voc,     MUS_VOC,     "VOC header id");
-  XEN_DEFINE_CONSTANT(S_mus_svx,     MUS_SVX,     "SVX (IFF) header id");
-  XEN_DEFINE_CONSTANT(S_mus_soundfont, MUS_SOUNDFONT, "soundfont header id");
+  XEN_DEFINE_CONSTANT(S_mus_next,        MUS_NEXT,        "NeXT (Sun) sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_aifc,        MUS_AIFC,        "AIFC sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_riff,        MUS_RIFF,        "RIFF (MS wave) sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_nist,        MUS_NIST,        "NIST (Sphere) sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_raw,         MUS_RAW,         "raw (headerless) sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_ircam,       MUS_IRCAM,       "IRCAM sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_aiff,        MUS_AIFF,        "AIFF (old-style) sound header id");
+  XEN_DEFINE_CONSTANT(S_mus_bicsf,       MUS_BICSF,       "BICSF header id");
+  XEN_DEFINE_CONSTANT(S_mus_voc,         MUS_VOC,         "VOC header id");
+  XEN_DEFINE_CONSTANT(S_mus_svx,         MUS_SVX,         "SVX (IFF) header id");
+  XEN_DEFINE_CONSTANT(S_mus_soundfont,   MUS_SOUNDFONT,   "soundfont header id");
 
-  XEN_DEFINE_CONSTANT(S_mus_unknown, MUS_UNKNOWN, "unknown data format");
-  XEN_DEFINE_CONSTANT(S_mus_bshort,  MUS_BSHORT,  "big-endian short data format id");
-  XEN_DEFINE_CONSTANT(S_mus_lshort,  MUS_LSHORT,  "little-endian short data format id");
-  XEN_DEFINE_CONSTANT(S_mus_mulaw,   MUS_MULAW,   "mulaw (8-bit) data format id");
-  XEN_DEFINE_CONSTANT(S_mus_alaw,    MUS_ALAW,    "alaw (8-bit) data format id");
-  XEN_DEFINE_CONSTANT(S_mus_byte,    MUS_BYTE,    "signed byte data format id");
-  XEN_DEFINE_CONSTANT(S_mus_ubyte,   MUS_UBYTE,   "unsigned byte data format id");
-  XEN_DEFINE_CONSTANT(S_mus_bfloat,  MUS_BFLOAT,  "big-endian float data format id");
-  XEN_DEFINE_CONSTANT(S_mus_lfloat,  MUS_LFLOAT,  "little-endian float data format id");
-  XEN_DEFINE_CONSTANT(S_mus_bint,    MUS_BINT,    "big-endian int data format id");
-  XEN_DEFINE_CONSTANT(S_mus_lint,    MUS_LINT,    "little-endian int data format id");
-  XEN_DEFINE_CONSTANT(S_mus_bintn,   MUS_BINTN,   "normalized big-endian int data format id");
-  XEN_DEFINE_CONSTANT(S_mus_lintn,   MUS_LINTN,   "normalized little-endian int data format id");
-  XEN_DEFINE_CONSTANT(S_mus_b24int,  MUS_B24INT,  "big-endian 24-bit data format id");
-  XEN_DEFINE_CONSTANT(S_mus_l24int,  MUS_L24INT,  "little-endian 24-bit data format id");
-  XEN_DEFINE_CONSTANT(S_mus_bdouble, MUS_BDOUBLE, "big-endian double data format id");
-  XEN_DEFINE_CONSTANT(S_mus_ldouble, MUS_LDOUBLE, "little-endian double data format id");
-  XEN_DEFINE_CONSTANT(S_mus_ubshort, MUS_UBSHORT, "unsigned big-endian short data format id");
-  XEN_DEFINE_CONSTANT(S_mus_ulshort, MUS_ULSHORT, "unsigned little-endian short data format id");
+  XEN_DEFINE_CONSTANT(S_mus_unknown,          MUS_UNKNOWN,          "unknown data format");
+  XEN_DEFINE_CONSTANT(S_mus_bshort,           MUS_BSHORT,           "big-endian short data format id");
+  XEN_DEFINE_CONSTANT(S_mus_lshort,           MUS_LSHORT,           "little-endian short data format id");
+  XEN_DEFINE_CONSTANT(S_mus_mulaw,            MUS_MULAW,            "mulaw (8-bit) data format id");
+  XEN_DEFINE_CONSTANT(S_mus_alaw,             MUS_ALAW,             "alaw (8-bit) data format id");
+  XEN_DEFINE_CONSTANT(S_mus_byte,             MUS_BYTE,             "signed byte data format id");
+  XEN_DEFINE_CONSTANT(S_mus_ubyte,            MUS_UBYTE,            "unsigned byte data format id");
+  XEN_DEFINE_CONSTANT(S_mus_bfloat,           MUS_BFLOAT,           "big-endian float data format id");
+  XEN_DEFINE_CONSTANT(S_mus_lfloat,           MUS_LFLOAT,           "little-endian float data format id");
+  XEN_DEFINE_CONSTANT(S_mus_bint,             MUS_BINT,             "big-endian int data format id");
+  XEN_DEFINE_CONSTANT(S_mus_lint,             MUS_LINT,             "little-endian int data format id");
+  XEN_DEFINE_CONSTANT(S_mus_bintn,            MUS_BINTN,            "normalized big-endian int data format id");
+  XEN_DEFINE_CONSTANT(S_mus_lintn,            MUS_LINTN,            "normalized little-endian int data format id");
+  XEN_DEFINE_CONSTANT(S_mus_b24int,           MUS_B24INT,           "big-endian 24-bit data format id");
+  XEN_DEFINE_CONSTANT(S_mus_l24int,           MUS_L24INT,           "little-endian 24-bit data format id");
+  XEN_DEFINE_CONSTANT(S_mus_bdouble,          MUS_BDOUBLE,          "big-endian double data format id");
+  XEN_DEFINE_CONSTANT(S_mus_ldouble,          MUS_LDOUBLE,          "little-endian double data format id");
+  XEN_DEFINE_CONSTANT(S_mus_ubshort,          MUS_UBSHORT,          "unsigned big-endian short data format id");
+  XEN_DEFINE_CONSTANT(S_mus_ulshort,          MUS_ULSHORT,          "unsigned little-endian short data format id");
   XEN_DEFINE_CONSTANT(S_mus_bdouble_unscaled, MUS_BDOUBLE_UNSCALED, "unscaled big-endian double data format id");
   XEN_DEFINE_CONSTANT(S_mus_ldouble_unscaled, MUS_LDOUBLE_UNSCALED, "unscaled little-endian double data format id");
   XEN_DEFINE_CONSTANT(S_mus_bfloat_unscaled,  MUS_BFLOAT_UNSCALED,  "unscaled big-endian float data format id");
@@ -1568,6 +1598,7 @@ void mus_sndlib_xen_initialize(void)
   XEN_DEFINE_PROCEDURE(S_sound_data_maxamp,        g_mus_sound_data_maxamp_w,      1, 0, 0, H_sound_data_maxamp);
   XEN_DEFINE_PROCEDURE(S_sound_data_setB,          g_sound_data_set_w,             4, 0, 0, H_sound_data_setB);
   XEN_DEFINE_PROCEDURE(S_sound_data_to_vct,        g_sound_data_to_vct_w,          1, 2, 0, H_sound_data_to_vct);
+  XEN_DEFINE_PROCEDURE(S_sound_data_to_sound_data, g_sound_data_to_sound_data_w,   5, 1, 0, H_sound_data_to_sound_data);
   XEN_DEFINE_PROCEDURE(S_vct_to_sound_data,        g_vct_to_sound_data_w,          1, 2, 0, H_vct_to_sound_data);
   XEN_DEFINE_PROCEDURE(S_mus_sound_frames,         g_mus_sound_frames_w,           1, 0, 0, H_mus_sound_frames);
   XEN_DEFINE_PROCEDURE(S_mus_sound_duration,       g_mus_sound_duration_w,         1, 0, 0, H_mus_sound_duration);
@@ -1593,8 +1624,6 @@ void mus_sndlib_xen_initialize(void)
   XEN_DEFINE_PROCEDURE(S_mus_sound_open_input,     g_mus_sound_open_input_w,       1, 0, 0, H_mus_sound_open_input);
   XEN_DEFINE_PROCEDURE(S_mus_sound_close_input,    g_mus_sound_close_input_w,      1, 0, 0, H_mus_sound_close_input);
   XEN_DEFINE_PROCEDURE(S_mus_audio_close,          g_mus_audio_close_w,            1, 0, 0, H_mus_audio_close);
-  XEN_DEFINE_PROCEDURE(S_mus_audio_save,           g_mus_audio_save_w,             0, 0, 0, H_mus_audio_save);
-  XEN_DEFINE_PROCEDURE(S_mus_audio_restore,        g_mus_audio_restore_w,          0, 0, 0, H_mus_audio_restore);
   XEN_DEFINE_PROCEDURE(S_mus_audio_systems,        g_mus_audio_systems_w,          0, 0, 0, H_mus_audio_systems);
   XEN_DEFINE_PROCEDURE(S_mus_audio_mixer_read,     g_mus_audio_mixer_read_w,       4, 0, 0, H_mus_audio_mixer_read);
   XEN_DEFINE_PROCEDURE(S_mus_audio_mixer_write,    g_mus_audio_mixer_write_w,      4, 0, 0, H_mus_audio_mixer_write);
@@ -1617,7 +1646,7 @@ void mus_sndlib_xen_initialize(void)
 				   S_setB S_mus_file_prescaler, g_mus_file_set_prescaler_w, 1, 0, 2, 0);
 
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_mus_file_data_clipped, g_mus_file_data_clipped_w, H_mus_file_data_clipped,
-				   S_setB S_mus_file_data_clipped, g_mus_file_set_data_clipped_w, 1, 0, 2, 0);
+				   S_setB S_mus_file_data_clipped, g_mus_file_set_data_clipped_w, 0, 0, 1, 0);
 
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_sound_data_ref, g_sound_data_ref_w, H_sound_data_ref,
 				   S_setB S_sound_data_ref, g_sound_data_set_w,  3, 0, 4, 0);
@@ -1688,9 +1717,7 @@ void mus_sndlib_xen_initialize(void)
 	       S_mus_audio_reinitialize,
 #endif
 	       S_mus_audio_report,
-	       S_mus_audio_restore,
 	       S_mus_audio_samples_per_channel,
-	       S_mus_audio_save,
 	       S_mus_audio_set_oss_buffers,
 	       S_mus_audio_spdif_in,
 	       S_mus_audio_spdif_out,
@@ -1776,6 +1803,7 @@ void mus_sndlib_xen_initialize(void)
 	       S_sound_data_p,
 	       S_sound_data_ref,
 	       S_sound_data_setB,
+	       S_sound_data_to_sound_data,
 	       S_vct_to_sound_data,
 	       NULL);
 #endif
