@@ -55,7 +55,7 @@ static char x_string[2] = {'x','\0'};
 
 /* special case multicard quad code split out for clarity (it could be folded into the main branches) */
 
-#ifdef DEBUG_MEMORY
+#if DEBUGGING
 void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line) {return(calloc(len, size));}
 void *mem_malloc(size_t len, const char *func, const char *file, int line) {return(malloc(len));}
 void mem_free(void *ptr, const char *func, const char *file, int line) {free(ptr);}
@@ -331,8 +331,7 @@ static int main_alsa(int argc, char *argv[])
   off_t start = 0, end = 0;
   double begin_time = 0.0, end_time = 0.0;
 
-  /* for clisp-based clm use, we need a couple added switches
-   * -describe => call mus_audio_describe and exit
+  /* -describe => call mus_audio_describe and exit
    * -buffers axb => set OSS fragment numbers 
    */
   for (i = 1; i < argc; i++)
