@@ -133,7 +133,6 @@ static const char *mus_initial_error_names[] = {
   "no such channel", "no file name provided", "no location method", "no channel method",
   "no such fft window", "unsupported data format", "header read failed",
   "unsupported header type", "file descriptors not initialized", "not a sound file", "file closed", "write error",
-  "bogus free", "buffer overflow", "buffer underflow", "file overflow",
   "header write failed", "can't open temp file", "interrupted", "bad envelope",
 
   "audio channels not available", "audio srate not available", "audio format not available",
@@ -217,7 +216,7 @@ static time_t local_file_write_date(const char *filename)
   return((time_t)(statbuf.st_mtime));
 }
 #else
-static int local_file_write_date(const char *filename) {return(1);}
+static time_t local_file_write_date(const char *filename) {return(1);}
 #endif
 
 static bool sndlib_initialized = false;
@@ -366,7 +365,7 @@ int mus_sound_forget(const char *name)
 	  free_sound_file(sound_table[i]);
 	  sound_table[i] = NULL;
 	}
-#if DEBUG_MEMORY || (defined(MPW_C))
+#if DEBUGGING || (defined(MPW_C))
   if (free_name) FREE(short_name); /* copy_string=MALLOC(snd-utils.c) in io.c mus_expand_filename or strdup=MALLOC in MPW */
 #else
   if (free_name) free(short_name); /* strdup=malloc, presumably */
@@ -376,11 +375,9 @@ int mus_sound_forget(const char *name)
 
 static sound_file *check_write_date(const char *name, sound_file *sf)
 {
-  int chan;
-  off_t data_size;
-  time_t date;
   if (sf)
     {
+      time_t date;
       date = local_file_write_date(name);
       if (date == sf->write_date)
 	return(sf);
@@ -388,6 +385,8 @@ static sound_file *check_write_date(const char *name, sound_file *sf)
 	{
 	  if ((sf->header_type == MUS_RAW) && (mus_header_no_header(name)))
 	    {
+	      int chan;
+	      off_t data_size;
 	      /* sound has changed since we last read it, but it has no header, so
 	       * the only sensible thing to check is the new length (i.e. caller
 	       * has set other fields by hand)
@@ -609,7 +608,7 @@ off_t mus_sound_comment_start (const char *arg) {MUS_SF(arg, sf->comment_start);
 off_t mus_sound_comment_end (const char *arg)   {MUS_SF(arg, sf->comment_end);}
 off_t mus_sound_length (const char *arg)        {MUS_SF(arg, sf->true_file_length);}
 int mus_sound_fact_samples (const char *arg)    {MUS_SF(arg, sf->fact_samples);}
-int mus_sound_write_date (const char *arg)      {MUS_SF(arg, (int)(sf->write_date));}
+time_t mus_sound_write_date (const char *arg)   {MUS_SF(arg, sf->write_date);}
 int mus_sound_type_specifier (const char *arg)  {MUS_SF(arg, sf->type_specifier);}
 int mus_sound_block_align (const char *arg)     {MUS_SF(arg, sf->block_align);}
 int mus_sound_bits_per_sample (const char *arg) {MUS_SF(arg, sf->bits_per_sample);}
@@ -753,11 +752,11 @@ char *mus_sound_comment(const char *name)
 int mus_sound_open_input (const char *arg) 
 {
   int fd = -1;
-  sound_file *sf = NULL;
   if (!(mus_file_probe(arg)))
     mus_error(MUS_CANT_OPEN_FILE, S_mus_sound_open_input " can't open %s: %s", arg, strerror(errno));
   else
     {
+      sound_file *sf = NULL;
       mus_sound_initialize();
       sf = find_sound_file(arg);
       if (!sf)
@@ -812,10 +811,10 @@ int mus_sound_close_input(int fd)
 int mus_sound_close_output(int fd, off_t bytes_of_data) 
 {
   char *name;
-  int err = MUS_ERROR, old_type;
   name = mus_file_fd_name(fd);
   if (name)
     {
+      int err = MUS_ERROR, old_type;
       char *fname;
       fname = strdup(name); /* strdup defined, if necessary, in io.c */
       old_type = mus_file_header_type(fd);
@@ -911,27 +910,6 @@ int mus_sound_override_header(const char *arg, int srate, int chans, int format,
   return(MUS_ERROR);
 }
 
-off_t mus_sound_maxamp(const char *ifile, mus_sample_t *vals)
-{
-  /* for CLM (ffi.lisp) */
-  mus_sample_t *mvals;
-  off_t *mtimes;
-  int chans, i, j;
-  off_t frames;
-  chans = mus_sound_chans(ifile);
-  mvals = (mus_sample_t *)CALLOC(chans, sizeof(mus_sample_t));
-  mtimes = (off_t *)CALLOC(chans, sizeof(off_t));
-  frames = mus_sound_maxamps(ifile, chans, mvals, mtimes);
-  for (i = 0, j = 0; i < chans; i++, j += 2)
-    {
-      vals[j] = (mus_sample_t)(mtimes[i]);
-      vals[j + 1] = mvals[i];
-    }
-  FREE(mvals);
-  FREE(mtimes);
-  return(frames);
-}
-
 bool mus_sound_maxamp_exists(const char *ifile)
 {
   sound_file *sf; 
@@ -1017,11 +995,11 @@ off_t mus_sound_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t
 
 int mus_sound_set_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t *times)
 {
-  int i, ichans = 0;
   sound_file *sf; 
   sf = getsf(ifile); 
   if (sf)
     {
+      int i, ichans = 0;
       if (sf->maxamps)
 	{
 	  if (chans > sf->chans) ichans = sf->chans; else ichans = chans;
