@@ -34,6 +34,8 @@
  *   (vct->vector v)                  return vector of vct contents
  *   (vct-move! v new old)            v[new++] = v[old++] -> v
  *   (vct-subseq v start end vnew)    vnew = v[start..end]
+ *   (vct-reverse v (len #f))         reverse contents (using len as end point if given)
+ *   (vct->string v)                  scheme-readable description of vct
  *
  * The intended use is a sort of latter-day array-processing system that handles huge
  * one-dimensional vectors -- fft's, etc.  Some of these functions can be found in
@@ -115,16 +117,17 @@ XEN_MAKE_OBJECT_FREE_PROCEDURE(vct, free_vct, vct_free)
 
 char *vct_to_string(vct *v)
 {
-  int len, i;
+  int len;
   char *buf;
   char flt[16];
-  if (v == NULL) return(NULL); /* not copy_string! -- it's not a sndlib function */
+  if (v == NULL) return(NULL);
   len = vct_print_length;
   if (len > v->length) len = v->length;
   buf = (char *)CALLOC(64 + len * 8, sizeof(char));
   sprintf(buf, "#<vct[len=%d]:", v->length);
   if (len > 0)
     {
+      int i;
       for (i = 0; i < len; i++)
 	{
 	  mus_snprintf(flt, 16, " %.3f", v->data[i]);
@@ -137,6 +140,38 @@ char *vct_to_string(vct *v)
   return(buf);
 }
 
+char *vct_to_readable_string(vct *v)
+{
+  int i, len;
+  char *buf;
+  char flt[16];
+  if (v == NULL) return(NULL);
+  len = v->length;
+  buf = (char *)CALLOC(64 + len * 8, sizeof(char));
+  sprintf(buf, "(vct");
+  for (i = 0; i < len; i++)
+    {
+      mus_snprintf(flt, 16, " %.3f", v->data[i]);
+      strcat(buf, flt);
+    }
+  strcat(buf, ")");
+  return(buf);
+}
+
+#if HAVE_GUILE
+static XEN g_vct_to_readable_string(XEN obj)
+{
+  char *vstr;
+  XEN result;
+  #define H_vct_to_string "(" S_vct_to_string " v) -> scheme readable description of v"
+  XEN_ASSERT_TYPE(VCT_P(obj), obj, XEN_ONLY_ARG, S_vct_to_string, "a vct");
+  vstr = vct_to_readable_string(TO_VCT(obj));
+  result = C_TO_XEN_STRING(vstr);
+  FREE(vstr);
+  return(result);
+}
+#endif
+
 bool vct_equalp(vct *v1, vct *v2)
 {
   int i;
@@ -428,14 +463,14 @@ v. (" S_vct_mapB " v (lambda () 3.0)) is the same as (" S_vct_fillB " v 3.0)"
 	  }
       }
     proc = XEN_CADR(proc);
-    XEN_ASSERT_TYPE(XEN_PROCEDURE_P(proc) && (XEN_REQUIRED_ARGS(proc) == 0), proc, XEN_ARG_2, S_vct_mapB, "a thunk");
+    XEN_ASSERT_TYPE(XEN_PROCEDURE_P(proc) && (XEN_REQUIRED_ARGS_OK(proc, 0)), proc, XEN_ARG_2, S_vct_mapB, "a thunk");
     for (i = 0; i < v->length; i++) 
-      v->data[i] = XEN_TO_C_DOUBLE(XEN_CALL_0_NO_CATCH(proc, S_vct_mapB));
+      v->data[i] = XEN_TO_C_DOUBLE(XEN_CALL_0_NO_CATCH(proc));
   }
 #else
-  XEN_ASSERT_TYPE(XEN_PROCEDURE_P(proc) && (XEN_REQUIRED_ARGS(proc) == 0), proc, XEN_ARG_2, S_vct_mapB, "a thunk");
+  XEN_ASSERT_TYPE(XEN_PROCEDURE_P(proc) && (XEN_REQUIRED_ARGS_OK(proc, 0)), proc, XEN_ARG_2, S_vct_mapB, "a thunk");
   for (i = 0; i < v->length; i++) 
-    v->data[i] = XEN_TO_C_DOUBLE(XEN_CALL_0_NO_CATCH(proc, S_vct_mapB));
+    v->data[i] = XEN_TO_C_DOUBLE(XEN_CALL_0_NO_CATCH(proc));
 #endif
   return(xen_return_first(obj, proc));
 }
@@ -492,7 +527,7 @@ static XEN vct_subseq(XEN vobj, XEN start, XEN end, XEN newv)
   return(xen_return_first(res, vobj, vnew));
 }
 
-static XEN list_to_vct(XEN lst)
+XEN list_to_vct(XEN lst)
 {
   #define H_list_to_vct "(" S_list_to_vct " lst): returns a new vct filled with elements of list lst"
   int len = 0, i;
@@ -504,7 +539,7 @@ static XEN list_to_vct(XEN lst)
   scv = make_vct(len, (Float *)CALLOC(len, sizeof(Float)));
   v = TO_VCT(scv);
   for (i = 0, lst1 = XEN_COPY_ARG(lst); i < len; i++, lst1 = XEN_CDR(lst1)) 
-    v->data[i] = (Float)XEN_TO_C_DOUBLE_WITH_CALLER(XEN_CAR(lst1), S_list_to_vct);
+    v->data[i] = (Float)XEN_TO_C_DOUBLE(XEN_CAR(lst1));
   return(xen_return_first(scv, lst));
 }
 
@@ -578,6 +613,28 @@ static XEN vct_to_vector(XEN vobj)
   return(xen_return_first(new_vect, vobj));
 }
 
+static XEN vct_reverse(XEN vobj, XEN size)
+{
+  #define H_vct_reverse "(" S_vct_reverse " vct len): in-place reversal of vct contents"
+  vct *v;
+  int i, j, len = -1;
+  XEN_ASSERT_TYPE(VCT_P(vobj), vobj, XEN_ARG_1, S_vct_to_vector, "a vct");
+  XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(size), size, XEN_ARG_2, S_vct_to_vector, "an integer");
+  v = TO_VCT(vobj);
+  if (XEN_INTEGER_P(size))
+    len = XEN_TO_C_INT(size);
+  if ((len <= 0) || (len > v->length))
+    len = v->length;
+  if (len == 1) return(vobj);
+  for (i = 0, j = len - 1; i < j; i++, j--)
+    {
+      Float temp;
+      temp = v->data[i];
+      v->data[i] = v->data[j];
+      v->data[j] = temp;
+    }
+  return(vobj);
+}
 
 #ifdef XEN_ARGIFY_1
 XEN_ARGIFY_2(g_make_vct_w, g_make_vct)
@@ -601,6 +658,7 @@ XEN_NARGIFY_1(vct_peak_w, vct_peak)
 XEN_ARGIFY_4(vct_move_w, vct_move)
 XEN_ARGIFY_4(vct_subseq_w, vct_subseq)
 XEN_VARGIFY(g_vct_w, g_vct)
+XEN_ARGIFY_2(vct_reverse_w, vct_reverse)
 #else
 #define g_make_vct_w g_make_vct
 #define copy_vct_w copy_vct
@@ -624,6 +682,7 @@ XEN_VARGIFY(g_vct_w, g_vct)
 #define vct_subseq_w vct_subseq
 #define g_vct_w g_vct
 #define vct_ref_w vct_ref
+#define vct_reverse_w vct_reverse
 #endif
 
 #if HAVE_RUBY
@@ -708,6 +767,7 @@ void vct_init(void)
   XEN_DEFINE_PROCEDURE(S_vct_moveB,     vct_move_w,     3, 1, 0, H_vct_moveB);
   XEN_DEFINE_PROCEDURE(S_vct_subseq,    vct_subseq_w,   2, 2, 0, H_vct_subseq);
   XEN_DEFINE_PROCEDURE(S_vct,           g_vct_w,        0, 0, 1, H_vct);
+  XEN_DEFINE_PROCEDURE(S_vct_reverse,   vct_reverse_w,  1, 1, 0, H_vct_reverse);
 #if WITH_RUN && USE_SND
   XEN_DEFINE_PROCEDURE("vct-map-1",     vct_mapB_w,     2, 0, 0, H_vct_mapB);
   XEN_EVAL_C_STRING("(defmacro vct-map! (v form) `(vct-map-1 ,v (list ',form ,form)))");
@@ -718,6 +778,7 @@ void vct_init(void)
 
 #if HAVE_GUILE
   XEN_DEFINE_PROCEDURE_WITH_SETTER(S_vct_ref, vct_ref_w, H_vct_ref, "set-" S_vct_ref, vct_set_w,  2, 0, 3, 0);
+  XEN_DEFINE_PROCEDURE(S_vct_to_string, g_vct_to_readable_string, 1, 0, 0, H_vct_to_string);
 #else
   XEN_DEFINE_PROCEDURE(S_vct_ref,       vct_ref_w,      2, 0, 0, H_vct_ref);
 #endif
@@ -745,6 +806,8 @@ void vct_init(void)
 	       S_vct_mapB,
 	       S_vct_ref,
 	       S_vct_setB,
+	       S_vct_reverse,
+	       S_vct_to_string,
 #if WITH_RUN && USE_SND
 	       "vct-map-1",
 #endif
