@@ -22,6 +22,8 @@
 #define S_vct_moveB      "vct-move!"
 #define S_vct_subseq     "vct-subseq"
 #define S_vct            "vct"
+#define S_vct_reverse    "vct-reverse!"
+#define S_vct_to_string  "vct->string"
 
 #ifndef Float
   #define Float float
@@ -45,6 +47,8 @@ vct *c_make_vct(int len);
 vct *c_free_vct(vct *v);
 vct *c_vct_copy(vct *vc);
 bool vct_equalp(vct *v1, vct *v2);
+char *vct_to_readable_string(vct *v);
+XEN list_to_vct(XEN lst);
 
 #define TO_VCT(arg) ((vct *)XEN_OBJECT_REF(arg))
 #define VCT_P(arg) vct_p(arg)
