@@ -172,6 +172,12 @@ returning you to the true top-level."
 (define *clm-reverb* #f)
 (define *clm-reverb-channels* 1)
 (define *clm-reverb-data* '())
+(define *clm-table-size* 512)
+(define *clm-file-buffer-size* 65536)
+(define *clm-locsig-type* mus-interp-linear)
+(define *clm-clipped* #t)
+(define *clm-array-print-length* 8)
+(define *clm-player* #f) ; default is play-and-wait (takes index of newly created sound, not the sound's file name)
 
 (define *to-snd* #t)
 
@@ -192,7 +198,7 @@ returning you to the true top-level."
 				  (header-type *clm-header-type*)
 				  (data-format *clm-data-format*)
 				  (comment #f)
-				  ;(verbose *clm-verbose*)
+				  (verbose *clm-verbose*)
 				  (reverb *clm-reverb*)
 				  (revfile "test.rev")
 				  (reverb-data *clm-reverb-data*)
@@ -202,19 +208,33 @@ returning you to the true top-level."
 				  (scaled-to #f)
 				  (play *clm-play*)
 				  (to-snd *to-snd*)
+				  (clipped 'unset)
 				  (scaled-by #f))
   (let ((old-srate (mus-srate))
 	(old-*output* *output*)
 	(old-*reverb* *reverb*)
 	(in-debugger #f)
+	(old-verbose *clm-verbose*)
+	(old-auto-update-interval (auto-update-interval))
 	(output-1 output)) ; protect during nesting
     (dynamic-wind 
 
      (lambda () 
+       (set! *clm-verbose* verbose)
+       (set! (clm-table-size) *clm-table-size*)
+       (set! (mus-file-buffer-size) *clm-file-buffer-size*)
+       (set! (locsig-type) *clm-locsig-type*)
+       (set! (mus-array-print-length) *clm-array-print-length*)
+       (set! (auto-update-interval) 0.0) ; turn it off
+       (if (equal? clipped 'unset)
+	   (if (and (or scaled-by scaled-to)
+		    (member data-format (list mus-bfloat mus-lfloat mus-bdouble mus-ldouble)))
+	       (set! (mus-file-data-clipped) #f)
+	       (set! (mus-file-data-clipped) *clm-clipped*))
+	   (set! (mus-file-data-clipped) clipped))
        (set! (mus-srate) srate))
 
      (lambda ()
-
        (if continue-old-file
 	   (begin
 	     (set! *output* (continue-sample->file output-1))
@@ -229,69 +249,103 @@ returning you to the true top-level."
 	     (if reverb (set! *reverb* (make-sample->file revfile reverb-channels data-format header-type)))))
        (let ((start (if statistics (get-internal-real-time)))
 	     (flush-reverb #f)
-	     (cycles 0))
-	 (catch 'with-sound-interrupt
-		thunk
-		(lambda args 
-		  ;; if from ws-interrupt? we have (continue stack message) as args
-		  (begin
-		    (if (and (not (null? (cdr args)))
-			     (continuation? (cadr args)))
-			;; instrument passed us a way to continue, so drop into the with-sound debugger
-			(let ((val (call-with-current-continuation
-				    (lambda (finish)
-				      (push-ws (cadr args)              ; continue in notelist from ws-interrupt
-					       finish                   ; stop notelist, go to output close section below
-					       (if (> (length args) 2)
-						   (list-ref args 2) 
-						   (make-stack #t)))    ; stack at point of ws-interrupt or stack right here
-				      (set! in-debugger #t)             ; gad -- turn off dynamic-wind output fixup below
-				      (throw 'snd-top-level             ; return to "top level" (can be nested)
-					     (format #f ";~A~A" 
-						     (if (> (length args) 3)    ; optional message to ws-interrupt
-							 (list-ref args 3)
-							 "")
-						     (let ((loc (ws-location)))
-						       (if (string? loc)
-							   (format #f ", interrupted at: ~A" loc)
-							   ""))))))))
-			  (set! in-debugger #f)
-			  (set! flush-reverb (eq? val #f)))
-			(begin
-			  (snd-print (format #f "with-sound interrupted: ~{~A~^ ~}" (cdr args)))
-			  (set! flush-reverb #t)))
-		    args)))
+	     (cycles 0)
+	     (revmax #f))
+	 (catch 'mus-error
+		(lambda ()
+		  (catch 'with-sound-interrupt
+			 thunk
+			 (lambda args 
+			   ;; if from ws-interrupt? we have (continue stack message) as args
+			   (begin
+			     (if (and (not (null? (cdr args)))
+				      (continuation? (cadr args)))
+				 ;; instrument passed us a way to continue, so drop into the with-sound debugger
+				 (let ((val (call-with-current-continuation
+					     (lambda (finish)
+					       (push-ws (cadr args)              ; continue in notelist from ws-interrupt
+							finish                   ; stop notelist, go to output close section below
+							(if (> (length args) 2)
+							    (list-ref args 2) 
+							    (make-stack #t)))    ; stack at point of ws-interrupt or stack right here
+					       (set! in-debugger #t)             ; gad -- turn off dynamic-wind output fixup below
+					       (throw 'snd-top-level             ; return to "top level" (can be nested)
+						      (format #f ";~A~A" 
+							      (if (> (length args) 3)    ; optional message to ws-interrupt
+								  (list-ref args 3)
+								  "")
+							      (let ((loc (ws-location)))
+								(if (string? loc)
+								    (format #f ", interrupted at: ~A" loc)
+								    ""))))))))
+				   (set! in-debugger #f)
+				   (set! flush-reverb (eq? val #f)))
+				 (begin
+				   (snd-print (format #f "with-sound interrupted: ~{~A~^ ~}" (cdr args)))
+				   (set! flush-reverb #t)))
+			     args))))
+		(lambda args
+		  ;; hit mus-error -- not usually continuable -- can we send a stack to the debugger here?
+		  (display "hit mus-error")
+		  (snd-print (format #f "with-sound mus-error: ~{~A~^ ~}~%" (cdr args)))
+		  (set! flush-reverb #t)))
+		  
 	 (if (and reverb (not flush-reverb))
 	     (begin
 	       (mus-close *reverb*)
+	       (if statistics (set! revmax (mus-sound-maxamp revfile)))
 	       (set! *reverb* (make-file->sample revfile))
 	       (apply reverb reverb-data)
 	       (mus-close *reverb*)
 	       (if *clm-delete-reverb* (delete-file revfile))))
 	 (mus-close *output*)
-	 (if to-snd
-	     (begin
-	       (if statistics
-		   (set! cycles (exact->inexact (/ (- (get-internal-real-time) start) internal-time-units-per-second))))
-	       (let ((cur (find-sound output-1)))
-		 (if cur (close-sound cur)))
-	       (let ((snd-output (open-sound output-1)))
-		 (set! (sync snd-output) #t)
-		 (if statistics
-		     (snd-print 
-		      (format #f "~A:~%  maxamp:~{ ~,4F~},~%  compute time: ~,3F~%"
+	 (let ((snd-output #f)
+	       (cur-sync #f))
+	   (if statistics
+	       (set! cycles (exact->inexact (/ (- (get-internal-real-time) start) internal-time-units-per-second))))
+	   (if to-snd
+	       (let* ((cur (find-sound output-1)))
+		 (set! cur-sync (and cur (sync cur)))
+		 (if cur 
+		     (set! snd-output (update-sound cur))
+		     (set! snd-output (open-sound output-1)))
+		 (set! (sync snd-output) #t)))
+	   (if statistics
+	       ((if to-snd snd-print display)
+		(format #f "~A:~%  maxamp~A:~{ ~,4F~}~%~A  compute time: ~,3F~%"
 			      output-1
-			      (maxamp snd-output #t)
+			      (if (or scaled-to scaled-by) " (before scaling)" "")
+			      (if to-snd (maxamp snd-output #t) (mus-sound-maxamp output-1))
+			      (if revmax 
+				  (format #f "  rev max: ~,4F~%" (if (list? revmax) 
+								     (cadr revmax) 
+								     revmax)) 
+				  "")
 			      cycles)))
+	   (if (or scaled-to scaled-by)
+	       (let ((scale-output (or snd-output (open-sound output-1))))
 		 (if scaled-to
-		     (scale-to scaled-to snd-output)
+		     (scale-to scaled-to scale-output)
 		     (if scaled-by
-			 (scale-by scaled-by snd-output)))
-		 (if play (play-and-wait snd-output))
-		 (update-time-graph snd-output))))
+			 (scale-by scaled-by scale-output)))
+		 (save-sound scale-output)
+		 (if (not to-snd) 
+		     (close-sound scale-output))))
+	   (if play 
+	       (if to-snd
+		   (if *clm-player*
+		       (*clm-player* snd-output)
+		       (play-and-wait 0 snd-output))
+		   (play output-1)))
+	   (if to-snd
+	       (begin
+		(update-time-graph snd-output)
+		(if (number? cur-sync) (set! (sync snd-output) cur-sync)))))
 	 output-1))
 
      (lambda () 
+       (set! *clm-verbose* old-verbose)
+       (set! (auto-update-interval) old-auto-update-interval)
        (if (not in-debugger)
 	   (begin
 	     (if *reverb*
@@ -474,11 +528,14 @@ returning you to the true top-level."
 			   output
 			   (maxamp snd-output #t)
 			   cycles)))
-	      (if scaled-to
-		  (scale-to scaled-to snd-output)
-		  (if scaled-by
-		      (scale-by scaled-by snd-output)))
-	      (if play (play-and-wait snd-output))
+		 (if (or scaled-to scaled-by)
+		     (begin
+		       (if scaled-to
+			   (scale-to scaled-to snd-output)
+			   (if scaled-by
+			       (scale-by scaled-by snd-output)))
+		       (save-sound snd-output)))
+	      (if play (play-and-wait 0 snd-output))
 	      (update-time-graph snd-output)))
 	(set! (mus-srate) old-srate)
 	output)
@@ -708,15 +765,6 @@ returning you to the true top-level."
 ;;; a test case for ws-debug
 ;;; (with-sound () (fm-violin 0 1 440 .1) (sleep 1) (fm-violin 1 1 440 .1) (sleep 1) (fm-violin 2 1 440 .1) (sleep 1))
 
-;;; SOMEDAY: mix/track output through outn (perhaps via with-mix fixed to use mix?)
-;;;   need a pre-edit/post-edit sequence to fixup reverb each time a mix/track is changed
-;;;   edit-hook after-edit-hook with flag to protect from "normal" editing 
-;;;     need edit-categories?
-;;;   so with-tagged-mix: write data, add mix to postprocess list
-;;;      with-sound mixes before reverb pass
-;;;   or add reverb to mixed data (so reverb moves with it -- affected incorrectly by speed change)
-
-
 
 ;;; -------- def-optkey-fun
 ;;;
@@ -771,4 +819,3 @@ returning you to the true top-level."
 							     (set! pa (cdr pa))
 							     (set! na (cdr na))))))))))
 				   (apply ,key-name allargs)))))))))))))
-    
