@@ -10,27 +10,18 @@
 #else
   #include <sys/types.h>
 #endif
+#if HAVE_STDINT_H
+  #include <stdint.h>
+#endif
 #include <math.h>
 #include "xen.h"
 
-
 XEN xen_return_first(XEN a, ...)
 {
   return(a);
 }
 
-int xen_to_c_int_or_else(XEN obj, int fallback, const char *origin)
-{
-  /* don't want errors about floats with non-zero fractions etc */
-  if (XEN_INTEGER_P(obj))
-    return(XEN_TO_C_INT(obj));
-  else
-    if (XEN_NUMBER_P(obj))
-      return((int)XEN_TO_C_DOUBLE_WITH_CALLER(obj, origin));
-  return(fallback);
-}
-
-off_t xen_to_c_off_t_or_else(XEN obj, off_t fallback, const char *origin)
+off_t xen_to_c_off_t_or_else(XEN obj, off_t fallback)
 {
 #if HAVE_GUILE
   if ((XEN_NOT_FALSE_P(scm_integer_p(obj))) && XEN_EXACT_P(obj))
@@ -44,7 +35,7 @@ off_t xen_to_c_off_t_or_else(XEN obj, off_t fallback, const char *origin)
 #endif
   else
     if (XEN_NUMBER_P(obj))
-      return((off_t)XEN_TO_C_DOUBLE_WITH_CALLER(obj, origin));
+      return((off_t)XEN_TO_C_DOUBLE(obj));
   return(fallback);
 }
 
@@ -99,11 +90,34 @@ void xen_gc_mark(XEN val)
   scm_gc_mark(val);
 }
 
-double xen_to_c_double_or_else(XEN a, double b) {return((XEN_NUMBER_P(a)) ? (scm_num2dbl(a, c__FUNCTION__)) : (b));}
-bool xen_to_c_boolean_or_true(XEN a) {return((!((XEN_FALSE_P(a) || ((SCM_INUMP(a)) && (SCM_INUM(a) == 0))))));}
+
+double xen_to_c_double(XEN a) 
+{
+  double num;
+#if HAVE_SCM_TO_SIGNED_INTEGER
+  num = scm_to_double(a);
+#else
+  num = scm_num2dbl(a, c__FUNCTION__);
+#endif
+#if HAVE_DECL_ISNAN
+  if (isnan(num)) return(0.0);
+#endif
+#if HAVE_DECL_ISINF
+  if (isinf(num)) return(0.0);
+#endif
+  return(num);
+}
+
+double xen_to_c_double_or_else(XEN a, double b) 
+{
+  if (XEN_NUMBER_P(a))
+    return(xen_to_c_double(a));
+  return(b);
+}
+
 bool xen_integer_p(XEN a) 
 {
-#ifndef SCM_BOOLP
+#if (!defined(SCM_PACK)) || HAVE_SCM_TO_SIGNED_INTEGER
   return(XEN_NOT_FALSE_P(scm_integer_p(a)));
 #else
   if (SCM_INUMP(a)) return(true);
@@ -122,16 +136,35 @@ bool xen_integer_p(XEN a)
 #endif
 }
 
-#if HAVE_SCM_NUM2INT
 int xen_to_c_int(XEN a)
 {
-  if (SCM_INUMP(a)) return(SCM_INUM(a));
-  if (SCM_REALP(a)) return((int)(SCM_REAL_VALUE(a)));
-  return((int)scm_num2dbl(a, c__FUNCTION__));
-}
+  /* Scheme integer (possible inexact) to C int without errors */
+#if HAVE_SCM_TO_SIGNED_INTEGER
+    if ((SCM_INEXACTP(a)) || (SCM_FRACTIONP(a))) /* avoid error if inexact integer! SCM_INUMP deprecated in 1.7 */
+      {
+	if ((scm_is_true(scm_inf_p(a))) || (scm_is_true(scm_nan_p(a)))) return(0);
+	if (SCM_REALP(a)) return((int)(SCM_REAL_VALUE(a)));
+	return((int)scm_to_double(a)); 
+      }
+    return(scm_to_int32(a)); 
 #else
-int xen_to_c_int(XEN a) {return((int)gh_scm2int(a));}
+  #if HAVE_SCM_NUM2INT
+    if (SCM_INUMP(a)) return(SCM_INUM(a));
+    if (SCM_REALP(a)) return((int)(SCM_REAL_VALUE(a)));
+    return((int)scm_num2dbl(a, c__FUNCTION__));
+  #else
+    return((int)gh_scm2int(a));  /* ah for the good old days... */
+  #endif
 #endif
+}
+
+int xen_to_c_int_or_else(XEN obj, int fallback)
+{
+  /* don't want errors about floats with non-zero fractions etc */
+  if (XEN_NUMBER_P(obj))
+    return(xen_to_c_int(obj));
+  return(fallback);
+}
 
 void xen_initialize(void)
 {
@@ -149,6 +182,7 @@ void xen_guile_define_procedure_with_setter(char *get_name, XEN (*get_func)(), c
   XEN str = XEN_FALSE;
 #if XEN_DEBUGGING
   if (XEN_DEFINED_P(get_name)) fprintf(stderr, "%s is defined\n", get_name);
+  /* if (!(snd_url(get_name))) fprintf(stderr, "%s not documented\n", get_name); */
 #endif
   if (get_help) str = C_TO_XEN_STRING(get_help);
   scm_permanent_object(
@@ -183,6 +217,7 @@ void xen_guile_define_procedure_with_reversed_setter(char *get_name, XEN (*get_f
   XEN str = XEN_FALSE;
 #if XEN_DEBUGGING
   if (XEN_DEFINED_P(get_name)) fprintf(stderr, "%s is defined\n", get_name);
+  /* if (!(snd_url(get_name))) fprintf(stderr, "%s not documented\n", get_name); */
 #endif
   if (get_help) str = C_TO_XEN_STRING(get_help);
   scm_permanent_object(
@@ -215,11 +250,12 @@ XEN xen_guile_create_hook(const char *name, int args, const char *help, XEN loca
 #if HAVE_GUILE
   if ((name) && (help))
     {
-      hook = scm_permanent_object(scm_make_hook(C_TO_SMALL_XEN_INT(args)));
+      hook = scm_permanent_object(scm_make_hook(C_TO_XEN_INT(args)));
       scm_set_object_property_x(hook, local_doc, C_TO_XEN_STRING(help));
 #if HAVE_SCM_C_DEFINE
 #if XEN_DEBUGGING
       if (XEN_DEFINED_P(name)) fprintf(stderr, "%s is defined\n", name);
+      /* if (!(snd_url(name))) fprintf(stderr, "%s not documented\n", name); */
 #endif
       scm_c_define(name, hook);
 #else
@@ -230,7 +266,7 @@ XEN xen_guile_create_hook(const char *name, int args, const char *help, XEN loca
   #endif
 #endif
     }
-  else hook = scm_make_hook(C_TO_SMALL_XEN_INT(args));
+  else hook = scm_make_hook(C_TO_XEN_INT(args));
 #endif
   return(hook);
 }
@@ -238,11 +274,54 @@ XEN xen_guile_create_hook(const char *name, int args, const char *help, XEN loca
 #if XEN_DEBUGGING
 XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int opt, int rst)
 {
+  /* look for name collisions */
   if ((name) && (strlen(name) > 0) && XEN_DEFINED_P(name)) fprintf(stderr, "%s is defined\n", name);
+  /* if ((name) && (strlen(name) > 0) && (!(snd_url(name)))) fprintf(stderr, "%s not documented\n", name); */
   return(scm_c_define_gsubr(name, req, opt, rst, func));
 }
 #endif
 
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+static char **xen_temp_strings = NULL;
+static int xen_temp_strings_ctr = 0;
+#define XEN_TEMP_STRINGS_SIZE 512
+
+#if DEBUGGING
+static char **stored_strings = NULL;
+static int stored_strings_ctr = 0;
+#endif
+
+char *xen_guile_to_c_string_with_eventual_free(XEN str)
+{
+  char *result;
+  if (!xen_temp_strings)
+    xen_temp_strings = (char **)calloc(XEN_TEMP_STRINGS_SIZE, sizeof(char *));
+  else
+    {
+      if (xen_temp_strings[xen_temp_strings_ctr]) 
+#if DEBUGGING
+	{
+	  int i, len;
+	  char *str;
+	  str = xen_temp_strings[xen_temp_strings_ctr];
+	  len = strlen(str);
+	  for (i = 0; i < len; i++) str[i] = 'X';
+	  if (!stored_strings) stored_strings = (char **)calloc(1024, sizeof(char **));
+	  if (stored_strings[stored_strings_ctr]) free(stored_strings[stored_strings_ctr]);
+	  stored_strings[stored_strings_ctr++] = str;
+	  if (stored_strings_ctr >= 1024) stored_strings_ctr = 0;
+	}
+#else
+	free(xen_temp_strings[xen_temp_strings_ctr]);
+#endif
+    }
+  result = scm_to_locale_string(str); /* not XEN_TO_C_STRING here -- infinite recursion */
+  xen_temp_strings[xen_temp_strings_ctr++] = result;
+  if (xen_temp_strings_ctr >= XEN_TEMP_STRINGS_SIZE) xen_temp_strings_ctr = 0;
+  return(result);
+}
+#endif
+
 #endif
 
 
@@ -292,6 +371,17 @@ void xen_initialize(void)
   Init_Hook();
 }
 
+int xen_to_c_int_or_else(XEN obj, int fallback)
+{
+  /* don't want errors about floats with non-zero fractions etc */
+  if (XEN_INTEGER_P(obj))
+    return(XEN_TO_C_INT(obj));
+  else
+    if (XEN_NUMBER_P(obj))
+      return((int)XEN_TO_C_DOUBLE(obj));
+  return(fallback);
+}
+
 void xen_gc_mark(XEN val)
 {
   rb_gc_mark(val);
@@ -501,9 +591,9 @@ static XEN xen_rb_rescue(XEN val)
 
 void xen_repl(int argc, char **argv)
 {
-  int status = 0;
   while (true)
     {
+      int status = 0;
       rb_protect(XEN_VALUE_ARG_PROCEDURE_CAST xen_rb_rescue,
 		 XEN_FALSE,
 		 &status);
@@ -567,7 +657,7 @@ void xen_rb_raise(XEN type, XEN info)
 int xen_rb_required_args(XEN val)
 {
   int args;
-  args = XEN_TO_SMALL_C_INT(val);
+  args = XEN_TO_C_INT(val);
   if (args == -1) return(1); 
   if (args < 0) return(abs(args + 1));
   return(args);
@@ -625,9 +715,10 @@ XEN xen_rb_funcall_0(XEN func)
   return(val);
 }
 
+#if (!HAVE_RB_ARY_DUP)
 XEN xen_rb_copy_list(XEN val)
 {
-  /* if this is considered bad form, we could fall back on flatten */
+  /* if this is considered bad form, we could fall back on flatten (rb_ary_dup?) */
   long len, i;
   VALUE collect;
   len = RARRAY(val)->len;
@@ -637,6 +728,7 @@ XEN xen_rb_copy_list(XEN val)
   RARRAY(collect)->len = len;
   return(collect);
 }
+#endif
 
 XEN xen_rb_str_new2(char *arg)
 {
@@ -734,7 +826,6 @@ VALUE xen_rb_hook_reset_hook(VALUE hook)
 static VALUE xen_rb_hook_names(VALUE hook)
 {
   VALUE ary = rb_obj_dup(rb_iv_get(hook, "@procs"));
-  
   if (RARRAY(ary)->len) {
     VALUE ret = rb_ary_new();
     while (RARRAY(ary)->len)
@@ -953,6 +1044,11 @@ char *xen_version(void)
 #endif
 }
 
+int xen_to_c_int_or_else(XEN obj, int fallback)
+{
+  return(fallback);
+}
+
 void xen_repl(int argc, char **argv)
 {
 }
