@@ -14,18 +14,30 @@
  *   Python doesn't strike me as very different from Ruby.
  *   SCM, like MIT_Scheme, isn't really oriented toward extending C programs.
  *   I've also looked briefly at Haskell (FFI too primitive), Lua (looks ok, though minimal, but lots of
- *     work to fit into the Xen mode), ocaml (FFI in flux, apparently, and I think it
- *     insists on being the main program).
+ *     work to fit into the Xen model), and ocaml (FFI in flux, apparently).
  *
  * "xen" from Greek xenos (guest, foreigner)
  */
 
 #define XEN_MAJOR_VERSION 1
-#define XEN_MINOR_VERSION 13
-#define XEN_VERSION "1.13"
+#define XEN_MINOR_VERSION 24
+#define XEN_VERSION "1.24"
 
 /* HISTORY:
  *
+ *  31-Dec-04: removed "caller" arg from *_NO_CATCH.
+ *  10-Nov-04: scm_c_vector* (new Guile functions)
+ *  21-Oct-04: XEN_LIST_REVERSE, (using rb_ary_dup available in 1.8)
+ *  7-Oct-04:  keyword changes for new Guile.
+ *  28-Sep-04: deprecated *_WITH_CALLER -- these no longer do anything useful in Guile.
+ *             NaNs and Infs -> 0 or 0.0 in XEN_TO_C_INT|DOUBLE -- perhaps I should add another set of macros?
+ *  23-Aug-04: more Guile name changes.
+ *  12-Aug-04: more Guile name changes, C_TO_XEN_STRINGN (Guile)
+ *  3-Aug-04:  xen_to_c_int bugfix thanks to Kjetil S. Matheussen.
+ *  29-Jul-04: deprecated XEN_TO_C_BOOLEAN_OR_TRUE.
+ *  21-Jul-04: deprecated XEN_TO_SMALL_C_INT and C_TO_SMALL_XEN_INT.
+ *             use new Guile 1.7 numerical function names (under flag HAVE_SCM_TO_SIGNED_INTEGER).
+ *  28-Jun-04: XEN_REQUIRED_ARGS_OK to make it easier to turn off this check.
  *  9-June-04: complex number conversions (Guile) -- I don't think Ruby has complex numbers.
  *  21-May-04: plug some memory leaks in Ruby cases.
  *  23-Feb-04: changed DEBUGGING to XEN_DEBUGGING, added redefinition checks under that switch.
@@ -67,16 +79,6 @@
 #endif
 #endif
 
-/* TODO: property lists, dynamic-wind?, procedure-source? */
-/*   currently using scm_internal_dynamic_wind, scm_current_module,
- *   scm_procedure_source, scm_set_smob*, scm_object_property, scm_set_object_property,
- *   scm_c_define, scm_hook_to_list, scm_mkstrport, scm_make_string, scm_current_error_port, scm_fluid_ref, scm_frame_source,
- *   scm_stack_ref, scm_source_property, scm_internal_stack_catch,
- *   scm_display_backtrace, scm_the_last_fluid_var, scm_prin1, scm_c_use_module, scm_c_export,
- *   SCM_OPB, SCM_WRTNG, SCM_HASHTABLE_BUCKETS, SCM_MODULE_USES, SCM_MODULE_OBARRAY, SCM_HASHTABLE_N_BUCKETS, 
- *   and all the snd-run cases.  
- */
-
 #ifndef c__FUNCTION__
 #if defined(__STDC__) && defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
   #define c__FUNCTION__ __func__
@@ -93,10 +95,10 @@
 /* ------------------------------ GUILE ------------------------------ */
 
 #if HAVE_GUILE
-#if (!HAVE_SCM_NUM2INT)
-  #include <guile/gh.h>
-#else
+#if (HAVE_SCM_NUM2INT || HAVE_SCM_C_MAKE_RECTANGULAR)
   #include <libguile.h>
+#else
+  #include <guile/gh.h>
 #endif
 
 #define XEN_OK 1
@@ -117,33 +119,37 @@
 
 #define XEN_UNDEFINED       SCM_UNDEFINED
 
-#ifndef SCM_EQ_P
-  #define SCM_EQ_P(a, b)   ((a) == (b))
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+  #define C_STRING_TO_XEN_SYMBOL(a)   scm_from_locale_symbol(a)
+#else
+  #if HAVE_SCM_STR2SYMBOL
+    #define C_STRING_TO_XEN_SYMBOL(a) scm_str2symbol(a)
+  #else
+    #define C_STRING_TO_XEN_SYMBOL(a) gh_symbol2scm(a)
+  #endif
 #endif
 
 #if HAVE_SCM_C_DEFINE
   #define XEN_VARIABLE_REF(Var)               SCM_VARIABLE_REF(Var)
   #define XEN_VARIABLE_SET(Var, Val)          SCM_VARIABLE_SET(Var, Val)
-  #define XEN_NAME_AS_C_STRING_TO_VALUE(a)    XEN_VARIABLE_REF(scm_sym2var(scm_str2symbol(a), scm_current_module_lookup_closure (), XEN_TRUE))
+  #define XEN_NAME_AS_C_STRING_TO_VALUE(a)    XEN_VARIABLE_REF(scm_sym2var(C_STRING_TO_XEN_SYMBOL(a), scm_current_module_lookup_closure (), XEN_TRUE))
       /* this is probably not the right thing -- the 3rd arg should be XEN_FALSE, else we're defining a new variable in the current module */
-  #define XEN_NAME_AS_C_STRING_TO_VARIABLE(a) scm_sym2var(scm_str2symbol(a), scm_current_module_lookup_closure(), XEN_FALSE)
+  #define XEN_NAME_AS_C_STRING_TO_VARIABLE(a) scm_sym2var(C_STRING_TO_XEN_SYMBOL(a), scm_current_module_lookup_closure(), XEN_FALSE)
   #define XEN_SYMBOL_TO_VARIABLE(a)           scm_sym2var(a, scm_current_module_lookup_closure(), XEN_FALSE)
+
+  #if HAVE_SCM_DEFINED_P
+    #define XEN_DEFINED_P(Name)               XEN_TRUE_P(scm_defined_p(C_STRING_TO_XEN_SYMBOL(Name), XEN_UNDEFINED))
+  #else
+    #define XEN_DEFINED_P(Name)               XEN_TRUE_P(scm_definedp(C_STRING_TO_XEN_SYMBOL(Name), XEN_UNDEFINED))
+  #endif
+
 #else
   #define XEN_VARIABLE_REF(Var)               SCM_CDR(Var)
   #define XEN_VARIABLE_SET(Var, Val)          SCM_SETCDR(Var, Val)
   #define XEN_NAME_AS_C_STRING_TO_VALUE(a)    scm_symbol_value0(a)
   #define XEN_NAME_AS_C_STRING_TO_VARIABLE(a) XEN_FALSE
   #define XEN_SYMBOL_TO_VARIABLE(a)           XEN_FALSE
-#endif
-
-#if HAVE_SCM_C_DEFINE
-#if HAVE_SCM_DEFINED_P
-  #define XEN_DEFINED_P(Name) XEN_TRUE_P(scm_defined_p(C_STRING_TO_XEN_SYMBOL(Name), XEN_UNDEFINED))
-#else
-  #define XEN_DEFINED_P(Name) XEN_TRUE_P(scm_definedp(C_STRING_TO_XEN_SYMBOL(Name), XEN_UNDEFINED))
-#endif
-#else
-  #define XEN_DEFINED_P(Name) false
+  #define XEN_DEFINED_P(Name)                 false
 #endif
 
 #if (SCM_DEBUG_TYPING_STRICTNESS == 2)
@@ -161,6 +167,7 @@
   #else
     #define XEN_OBJECT_TYPE            scm_bits_t
   #endif
+  /* SCM_SMOB_PREDICATE -> scm_assert_smob_type? -- don't want an error here */
   #define XEN_OBJECT_TYPE_P(Obj, Type) ((SCM_NIMP(Obj)) && (SCM_SMOB_PREDICATE(Type, Obj)))
 #else
   #define XEN_OBJECT_TYPE              long
@@ -186,13 +193,27 @@
     return(0); \
   }
 
-#define XEN_TRUE_P(a)                SCM_EQ_P(a, XEN_TRUE)
-#define XEN_FALSE_P(a)               SCM_EQ_P(a, XEN_FALSE)
-#define XEN_NULL_P(a)                SCM_NULLP(a)
+#if HAVE_SCM_TO_SIGNED_INTEGER
+  #define XEN_EQ_P(a, b)             scm_is_eq(a, b)
+#else
+  #ifndef SCM_EQ_P
+    #define SCM_EQ_P(a, b)           ((a) == (b))
+  #endif
+  #define XEN_EQ_P(a, b)             SCM_EQ_P(a, b)
+#endif
+
+#define XEN_TRUE_P(a)                XEN_EQ_P(a, XEN_TRUE)
+#define XEN_FALSE_P(a)               XEN_EQ_P(a, XEN_FALSE)
+#if HAVE_SCM_CAR
+  #define XEN_NULL_P(a)              scm_is_null(a)
+#else
+  #define XEN_NULL_P(a)              SCM_NULLP(a)
+#endif
 #define XEN_BOUND_P(Arg)             (!(SCM_UNBNDP(Arg)))
 #define XEN_NOT_BOUND_P(Arg)         SCM_UNBNDP(Arg)
 #define XEN_ZERO                     SCM_INUM0
 
+/* should SCM_CAR -> scm_car? (snd-run also) -- this appears to be a pointless change */
 #define XEN_CAR(a)                   SCM_CAR(a)
 #define XEN_CADR(a)                  SCM_CADR(a)
 #define XEN_CADDR(a)                 SCM_CADDR(a)
@@ -200,72 +221,89 @@
 #define XEN_CDR(a)                   SCM_CDR(a)
 #define XEN_CDDR(a)                  SCM_CDDR(a)
 #define XEN_COPY_ARG(Lst)            Lst
-#define XEN_EQ_P(a, b)               SCM_EQ_P(a, b)
 #define XEN_EQV_P(A, B)              XEN_TO_C_BOOLEAN(scm_eqv_p(A, B))
 #define XEN_EQUAL_P(A, B)            XEN_TO_C_BOOLEAN(scm_equal_p(A, B))
 
-#define XEN_ONLY_ARG SCM_ARGn
-#define XEN_ARG_1    SCM_ARG1
-#define XEN_ARG_2    SCM_ARG2
-#define XEN_ARG_3    SCM_ARG3
-#define XEN_ARG_4    SCM_ARG4
-#define XEN_ARG_5    SCM_ARG5
-#define XEN_ARG_6    SCM_ARG6
-#define XEN_ARG_7    SCM_ARG7
-#define XEN_ARG_8    8
-#define XEN_ARG_9    9
+#define XEN_TO_C_INT(a)              xen_to_c_int(a)
+#define XEN_TO_C_INT_OR_ELSE(a, b)   xen_to_c_int_or_else(a, b)
+#define XEN_INTEGER_P(Arg)           xen_integer_p(Arg)
+#define XEN_TO_C_DOUBLE(a)           xen_to_c_double(a)  
+#define XEN_TO_C_DOUBLE_OR_ELSE(a, b) xen_to_c_double_or_else(a, b)  
 
-#define XEN_TO_C_DOUBLE(a)            scm_num2dbl(a,  c__FUNCTION__)
-#if defined(__GNUC__) && (!(defined(__cplusplus)))
-  #define XEN_TO_C_DOUBLE_OR_ELSE(a, b) ({ XEN _xen_h_0_ = a; ((XEN_NUMBER_P(_xen_h_0_)) ? (scm_num2dbl(_xen_h_0_,  c__FUNCTION__)) : (b)); })
+/* all the number handlers changed (names...) in 1.7 */
+#if HAVE_SCM_TO_SIGNED_INTEGER
+  #define C_TO_XEN_DOUBLE(a)            scm_from_double(a)
+  #define C_TO_XEN_INT(a)               scm_from_int(a)
+  #define XEN_TO_C_ULONG(a)             scm_to_ulong(a)
+  #define C_TO_XEN_ULONG(a)             scm_from_ulong((unsigned long)a)
+  #define C_TO_XEN_LONG_LONG(a)         scm_from_long_long(a)
+  #define XEN_TO_C_LONG_LONG(a)         scm_to_long_long(a)
+  #define XEN_EXACT_P(Arg)              XEN_TRUE_P(scm_exact_p(Arg))
+  #define XEN_BOOLEAN_P(Arg)            ((bool)scm_is_bool(Arg))
+  #define XEN_NUMBER_P(Arg)             ((bool)scm_is_real(Arg))
+  #define XEN_DOUBLE_P(Arg)             ((bool)scm_is_real(Arg))
+  #define XEN_OFF_T_P(Arg)              ((bool)scm_is_integer(Arg))
+  #define XEN_ULONG_P(Arg1)             (XEN_NOT_FALSE_P(scm_number_p(Arg1)))
 #else
-  #define XEN_TO_C_DOUBLE_OR_ELSE(a, b) xen_to_c_double_or_else(a, b)
-#endif
-#define XEN_TO_C_DOUBLE_WITH_CALLER(a, b) scm_num2dbl(a, b)
-#if HAVE_SCM_MAKE_REAL
-  #define C_TO_XEN_DOUBLE(a)          scm_make_real(a)
-#else
-  #define C_TO_XEN_DOUBLE(a)          scm_makdbl(a, 0.0)
-#endif
-#define XEN_TO_C_INT(a)               xen_to_c_int(a)
-#define XEN_TO_C_INT_OR_ELSE(a, b)    xen_to_c_int_or_else(a, b, c__FUNCTION__)
-#define XEN_TO_C_INT_OR_ELSE_WITH_CALLER(a, b, c) xen_to_c_int_or_else(a, b, c)
-#define C_TO_XEN_INT(a)               scm_long2num((long)a)
-#define C_TO_SMALL_XEN_INT(a)         SCM_MAKINUM(a)
-#define XEN_TO_SMALL_C_INT(a)         ((int)(SCM_INUM(a)))
-#define XEN_TO_C_ULONG(a)             scm_num2ulong(a, 0, c__FUNCTION__)
-#define C_TO_XEN_ULONG(a)             scm_ulong2num((unsigned long)a)
-#define XEN_ULONG_P(Arg1)             (XEN_NOT_FALSE_P(scm_number_p(Arg1)))
-#define XEN_EXACT_P(Arg)              XEN_TRUE_P(scm_exact_p(Arg))
-#if HAVE_SCM_NUM2LONG_LONG
-  #define C_TO_XEN_LONG_LONG(a)       scm_long_long2num(a)
-  #define XEN_TO_C_LONG_LONG(a)       scm_num2long_long(a, 0, c__FUNCTION__)
-#else
-  #define C_TO_XEN_LONG_LONG(a)       scm_long2num(a)
-  #define XEN_TO_C_LONG_LONG(a)       scm_num2long(a, 0, c__FUNCTION__)
+  #if HAVE_SCM_MAKE_REAL
+    #define C_TO_XEN_DOUBLE(a)          scm_make_real(a)
+  #else
+    #define C_TO_XEN_DOUBLE(a)          scm_makdbl(a, 0.0)
+  #endif
+  #define C_TO_XEN_INT(a)               scm_long2num((long)a)
+  #define XEN_TO_C_ULONG(a)             scm_num2ulong(a, 0, c__FUNCTION__)
+  #define C_TO_XEN_ULONG(a)             scm_ulong2num((unsigned long)a)
+  #define XEN_ULONG_P(Arg1)             (XEN_NOT_FALSE_P(scm_number_p(Arg1)))
+  #define XEN_EXACT_P(Arg)              XEN_TRUE_P(scm_exact_p(Arg))
+  #if HAVE_SCM_NUM2LONG_LONG
+    #define C_TO_XEN_LONG_LONG(a)       scm_long_long2num(a)
+    #define XEN_TO_C_LONG_LONG(a)       scm_num2long_long(a, 0, c__FUNCTION__)
+  #else
+    #define C_TO_XEN_LONG_LONG(a)       scm_long2num(a)
+    #define XEN_TO_C_LONG_LONG(a)       scm_num2long(a, 0, c__FUNCTION__)
+  #endif
+  #define XEN_BOOLEAN_P(Arg)            (SCM_BOOLP(Arg))
+  #define XEN_NUMBER_P(Arg)             (XEN_NOT_FALSE_P(scm_real_p(Arg)))
+  #define XEN_DOUBLE_P(Arg)             (XEN_NOT_FALSE_P(scm_real_p(Arg)))
+  #define XEN_OFF_T_P(Arg)              (XEN_NOT_FALSE_P(scm_integer_p(Arg)))
 #endif
+
 #if HAVE_COMPLEX_TRIG
-  #define XEN_TO_C_COMPLEX(a)         XEN_TO_C_DOUBLE(scm_real_part(a)) + (XEN_TO_C_DOUBLE(scm_imag_part(a)) * 1.0fi)
-  #define C_TO_XEN_COMPLEX(a)         scm_make_complex(__real__ a, __imag__ a)
-  #define XEN_COMPLEX_P(Arg)          (XEN_NOT_FALSE_P(scm_number_p(Arg)))
+  #if HAVE_SCM_C_MAKE_RECTANGULAR
+    #if defined(__GNUC__) && (!(defined(__cplusplus)))
+      #define XEN_TO_C_COMPLEX(a)       ({ XEN _xen_h_23_ = a; (scm_c_real_part(_xen_h_23_) + scm_c_imag_part(_xen_h_23_) * 1.0fi); })    
+      #define C_TO_XEN_COMPLEX(a)       ({ complex _xen_h_24_ = a; scm_c_make_rectangular(__real__ _xen_h_24_, __imag__ _xen_h_24_); })
+      #define XEN_COMPLEX_P(Arg)        scm_is_complex(Arg)
+    #else
+      #define XEN_TO_C_COMPLEX(a)       (scm_c_real_part(a) + scm_c_imag_part(a) * 1.0fi)
+      #define C_TO_XEN_COMPLEX(a)       scm_c_make_rectangular(__real__ a, __imag__ a)
+      #define XEN_COMPLEX_P(Arg)        scm_is_complex(Arg)
+    #endif
+  #else
+    #define XEN_TO_C_COMPLEX(a)         XEN_TO_C_DOUBLE(scm_real_part(a)) + (XEN_TO_C_DOUBLE(scm_imag_part(a)) * 1.0fi)
+    #define C_TO_XEN_COMPLEX(a)         scm_make_complex(__real__ a, __imag__ a)
+    #define XEN_COMPLEX_P(Arg)          (XEN_NOT_FALSE_P(scm_number_p(Arg)))
+  #endif
 #endif
 
 /* there is SCM_CONTINUATIONP -- why doesn't scheme have continuation? */
 
-#ifndef SCM_STRING_CHARS
-  #define XEN_TO_C_STRING(STR)        SCM_CHARS(STR)
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+  #define XEN_TO_C_STRING(Str)        xen_guile_to_c_string_with_eventual_free(Str)
+  #define C_TO_XEN_STRING(a)          ((a) ? scm_from_locale_string(a) : XEN_FALSE)
+  #define C_TO_XEN_STRINGN(Str, Len)  scm_from_locale_stringn(Str, Len)
 #else
-  #define XEN_TO_C_STRING(STR)        SCM_STRING_CHARS(STR)
-  /* this assumes its argument is a XEN string and does not allocate new space */
+  #ifndef SCM_STRING_CHARS
+    #define XEN_TO_C_STRING(STR)      SCM_CHARS(STR)
+  #else
+    #define XEN_TO_C_STRING(STR)      SCM_STRING_CHARS(STR)
+    /* this assumes its argument is a XEN string and does not allocate new space */
+  #endif
+  #define C_TO_XEN_STRING(a)          scm_makfrom0str((const char *)(a))
+  #define C_TO_XEN_STRINGN(Str, Len)  scm_mem2string(Str, Len)
 #endif
-#define C_TO_XEN_STRING(a)            scm_makfrom0str((const char *)(a))
 
 #define C_TO_XEN_BOOLEAN(a)           ((a) ? XEN_TRUE : XEN_FALSE)
-#if defined(__GNUC__) && (!(defined(__cplusplus)))
-  #define XEN_TO_C_BOOLEAN_OR_TRUE(a) ({ XEN _xen_h_2_ = a; (!((XEN_FALSE_P(_xen_h_2_) || ((SCM_INUMP(_xen_h_2_)) && (SCM_INUM(_xen_h_2_) == 0))))); })
-#else
-  #define XEN_TO_C_BOOLEAN_OR_TRUE(a) xen_to_c_boolean_or_true(a)
-#endif
 #define XEN_TO_C_BOOLEAN(a)           (!(XEN_FALSE_P(a)))
 
 #if HAVE_SCM_C_EVAL_STRING
@@ -274,20 +312,20 @@
   #define C_STRING_TO_XEN_FORM(Str)   scm_read_0str(Str)
 #endif
 
-#if HAVE_SCM_STR2SYMBOL
-  #define C_STRING_TO_XEN_SYMBOL(a)   scm_str2symbol(a)
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+  #define XEN_EVAL_FORM(Form)         scm_eval((XEN)(Form), scm_interaction_environment())
+  #define XEN_SYMBOL_TO_C_STRING(a)   XEN_TO_C_STRING(scm_symbol_to_string(a))
 #else
-  #define C_STRING_TO_XEN_SYMBOL(a)   gh_symbol2scm(a)
+  #ifdef SCM_SYMBOL_CHARS
+    #define XEN_EVAL_FORM(Form)       scm_eval((XEN)(Form), scm_interaction_environment())
+    /* was scm_eval_x but I'm not sure that's safe */
+    #define XEN_SYMBOL_TO_C_STRING(a) SCM_SYMBOL_CHARS(a)
+  #else
+    #define XEN_EVAL_FORM(Form)       scm_eval((XEN)(Form))
+    #define XEN_SYMBOL_TO_C_STRING(a) gh_symbol2newstr(a, NULL)
+  #endif
 #endif
 
-#ifdef SCM_SYMBOL_CHARS
-  #define XEN_EVAL_FORM(Form)         scm_eval((XEN)(Form), scm_interaction_environment())
-  /* was scm_eval_x but I'm not sure that's safe */
-  #define XEN_SYMBOL_TO_C_STRING(a)   SCM_SYMBOL_CHARS(a)
-#else
-  #define XEN_EVAL_FORM(Form)         scm_eval((XEN)(Form))
-  #define XEN_SYMBOL_TO_C_STRING(a)   gh_symbol2newstr(a, NULL)
-#endif
 #if HAVE_SCM_OBJECT_TO_STRING
   #define XEN_TO_STRING(Obj)          scm_object_to_string(Obj, XEN_UNDEFINED)
 #else
@@ -342,20 +380,20 @@
 #define XEN_DEFINE_CONSTANT(Name, Value, Help) \
   { \
     if (XEN_DEFINED_P(Name)) fprintf(stderr, "%s is defined\n", Name); \
-    scm_c_define(Name, C_TO_SMALL_XEN_INT(Value)); \
+    scm_c_define(Name, C_TO_XEN_INT(Value)); \
     XEN_SET_DOCUMENTATION(Name, Help); \
   }
 #else
 #define XEN_DEFINE_CONSTANT(Name, Value, Help) \
   { \
-    scm_c_define(Name, C_TO_SMALL_XEN_INT(Value)); \
+    scm_c_define(Name, C_TO_XEN_INT(Value)); \
     XEN_SET_DOCUMENTATION(Name, Help); \
   }
 #endif
 #else
 #define XEN_DEFINE_CONSTANT(Name, Value, Help) \
   { \
-    gh_define(Name, C_TO_SMALL_XEN_INT(Value)); \
+    gh_define(Name, C_TO_XEN_INT(Value)); \
     XEN_SET_DOCUMENTATION(Name, Help); \
   }
 #endif
@@ -364,6 +402,7 @@
 #define XEN_OBJECT_HELP(Name)           scm_object_property(Name, XEN_DOCUMENTATION_SYMBOL)
 #define XEN_PROCEDURE_HELP(Name)        scm_procedure_property(Name, XEN_DOCUMENTATION_SYMBOL)
 #define XEN_PROCEDURE_SOURCE_HELP(Name) scm_procedure_documentation(Name)
+#define XEN_PROCEDURE_SOURCE(Func)      scm_procedure_source(Func)
 
 #if HAVE_SCM_C_DEFINE
   #if XEN_DEBUGGING
@@ -387,7 +426,8 @@
 #endif
 #define XEN_THROW(Tag, Arg)           scm_throw(Tag, Arg)
 
-#ifndef SCM_BOOLP
+/* these are only needed in 1.3.4, but it's hard to find the right way to distinguish it */
+#ifndef SCM_PACK
   #define SCM_BOOLP(Arg)              gh_boolean_p(Arg)
   /* the next exist in 1.3.4 but are not usable in this context (need SCM_NIMP check) */
   #undef SCM_STRINGP
@@ -398,24 +438,34 @@
   #define SCM_SYMBOLP(Arg)            gh_symbol_p(Arg)
 #endif
 
-#define XEN_INTEGER_P(Arg)            xen_integer_p(Arg)
-#define XEN_BOOLEAN_P(Arg)            (SCM_BOOLP(Arg))
-#define XEN_NUMBER_P(Arg)             (XEN_NOT_FALSE_P(scm_real_p(Arg)))
-#define XEN_DOUBLE_P(Arg)             (XEN_NOT_FALSE_P(scm_real_p(Arg)))
-#define XEN_OFF_T_P(Arg)              (XEN_NOT_FALSE_P(scm_integer_p(Arg)))
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+  #define XEN_SYMBOL_P(Arg)           scm_is_symbol(Arg)
+#else
+  #define XEN_SYMBOL_P(Arg)           (SCM_SYMBOLP(Arg))
+#endif
 
-#define XEN_SYMBOL_P(Arg)             (SCM_SYMBOLP(Arg))
 #define XEN_PROCEDURE_P(Arg)          (XEN_NOT_FALSE_P(scm_procedure_p(Arg)))
-#define XEN_STRING_P(Arg)             (SCM_STRINGP(Arg))
-#define XEN_VECTOR_P(Arg)             (SCM_VECTORP(Arg))
+
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+  #define XEN_STRING_P(Arg)           scm_is_string(Arg)
+#else
+  #define XEN_STRING_P(Arg)           (SCM_STRINGP(Arg))
+#endif
+
+#if HAVE_SCM_IS_VECTOR
+  #define XEN_VECTOR_P(Arg)           (scm_is_vector(Arg))
+#else
+  #define XEN_VECTOR_P(Arg)           (SCM_VECTORP(Arg))
+#endif
+
 #define XEN_LIST_P(Arg)               (scm_ilength(Arg) >= 0)
 #define XEN_LIST_P_WITH_LENGTH(Arg, Len) ((Len = ((int)scm_ilength(Arg))) >= 0)
-#define XEN_ULONG_P(Arg1)             (XEN_NOT_FALSE_P(scm_number_p(Arg1)))
 #define XEN_HOOK_P(Arg)               (SCM_HOOKP(Arg))
 
 #define XEN_LIST_LENGTH(Arg)          ((int)(scm_ilength(Arg)))
-#define XEN_LIST_REF(Lst, Num)        scm_list_ref(Lst, C_TO_SMALL_XEN_INT(Num))
+#define XEN_LIST_REF(Lst, Num)        scm_list_ref(Lst, C_TO_XEN_INT(Num))
 #define XEN_LIST_SET(Lst, Loc, Val)   scm_list_set_x(Lst, C_TO_XEN_INT(Loc), Val)
+#define XEN_LIST_REVERSE(Lst)         scm_reverse(Lst)
 #define XEN_CONS(Arg1, Arg2)          scm_cons(Arg1, Arg2)
 #define XEN_CONS_2(Arg1, Arg2, Arg3)  scm_cons2(Arg1, Arg2, Arg3)
 #if HAVE_SCM_LIST_N
@@ -441,17 +491,25 @@
 #endif
 #define XEN_APPEND(a, b)                  scm_append(XEN_LIST_2(a, b))
 
+#if HAVE_SCM_IS_VECTOR
+  #define XEN_VECTOR_LENGTH(Arg)      (scm_c_vector_length(Arg))
+#else
 #ifndef SCM_VECTOR_LENGTH
   #define XEN_VECTOR_LENGTH(Arg)      ((int)(gh_vector_length(Arg)))
 #else
   #define XEN_VECTOR_LENGTH(Arg)      ((int)(SCM_VECTOR_LENGTH(Arg)))
 #endif
+#endif
 #ifdef SCM_WRITABLE_VELTS
   #define XEN_VECTOR_ELEMENTS(a)      SCM_WRITABLE_VELTS(a)
 #else
   #define XEN_VECTOR_ELEMENTS(a)      SCM_VELTS(a)
 #endif
 
+#if HAVE_SCM_IS_VECTOR
+  #define XEN_VECTOR_REF(Vect, Num)      scm_c_vector_ref(Vect, Num)
+  #define XEN_VECTOR_SET(Vect, Num, Val) scm_c_vector_set_x(Vect, Num, Val)
+#else
 #ifdef SCM_VECTOR_REF
   #define XEN_VECTOR_REF(Vect, Num)      SCM_VECTOR_REF(Vect, Num)
   #define XEN_VECTOR_SET(Vect, Num, Val) SCM_VECTOR_SET(Vect, Num, Val)
@@ -459,6 +517,7 @@
   #define XEN_VECTOR_REF(Vect, Num)      scm_vector_ref(Vect, C_TO_XEN_INT(Num))
   #define XEN_VECTOR_SET(Vect, Num, Val) scm_vector_set_x(Vect, C_TO_XEN_INT(Num), Val)
 #endif
+#endif
 
 #define XEN_VECTOR_TO_LIST(Vect)      scm_vector_to_list(Vect)
 #if HAVE_SCM_C_MAKE_VECTOR
@@ -483,37 +542,69 @@
   #define C_TO_XEN_CHAR(c)            SCM_MAKICHR(c)
 #endif
 
-#if HAVE_SCM_MAKE_RATIO
+#if HAVE_SCM_MAKE_RATIO || HAVE_SCM_C_MAKE_RECTANGULAR
   #define XEN_NUMERATOR(Arg)          scm_numerator(Arg)
   #define XEN_DENOMINATOR(Arg)        scm_denominator(Arg)
   #define XEN_RATIONALIZE(Arg1, Arg2) scm_rationalize(scm_inexact_to_exact(Arg1), scm_inexact_to_exact(Arg2))
   #define XEN_RATIO_P(Arg)            SCM_FRACTIONP(Arg)
-  #define XEN_MAKE_RATIO(Num, Den)    scm_make_ratio(Num, Den)
+  #if HAVE_SCM_C_MAKE_RECTANGULAR
+    #define XEN_MAKE_RATIO(Num, Den)  scm_divide(Num, Den)
+  #else
+    #define XEN_MAKE_RATIO(Num, Den)  scm_make_ratio(Num, Den)
+  #endif
 #endif
 
-#define XEN_CONS_P(Arg)               SCM_CONSP(Arg)
-#define XEN_PAIR_P(a)                 XEN_TRUE_P(scm_pair_p(a))
+#if HAVE_SCM_CAR
+  #define XEN_CONS_P(Arg)             scm_is_pair(Arg)
+  #define XEN_PAIR_P(Arg)             scm_is_pair(Arg)
+#else
+  #define XEN_CONS_P(Arg)             SCM_CONSP(Arg)
+  #define XEN_PAIR_P(Arg)             XEN_TRUE_P(scm_pair_p(Arg))
+#endif
 
 #define XEN_ARITY(Func)               scm_i_procedure_arity(Func)
-#define XEN_REQUIRED_ARGS(Func)       XEN_TO_SMALL_C_INT(XEN_CAR(XEN_ARITY(Func)))
-#ifdef SCM_CHARP
-  #define XEN_KEYWORD_P(Obj)          (SCM_KEYWORDP(Obj))
+
+#if (!WITH_HOBBIT)
+#define XEN_REQUIRED_ARGS(Func)       XEN_TO_C_INT(XEN_CAR(XEN_ARITY(Func)))
 #else
-  #define XEN_KEYWORD_P(Obj)          XEN_TRUE_P(scm_keyword_p(Obj))
+#define XEN_REQUIRED_ARGS(Func) \
+  XEN_TO_C_INT(((!(SCM_CLOSUREP(Func))) && \
+                (XEN_NOT_FALSE_P(scm_procedure_property(Func, C_STRING_TO_XEN_SYMBOL("hobbit-numargs"))))) ? \
+		 scm_procedure_property(Func, C_STRING_TO_XEN_SYMBOL("hobbit-numargs")) : XEN_CAR(XEN_ARITY(Func)))
+#endif
+
+#define XEN_REQUIRED_ARGS_OK(Func, Args) (XEN_TO_C_INT(XEN_CAR(XEN_ARITY(Func))) == Args)
+
+#if HAVE_SCM_FROM_LOCALE_KEYWORD
+  #define XEN_KEYWORD_P(Obj)          scm_is_keyword(Obj)
+  #define XEN_MAKE_KEYWORD(Arg)       scm_from_locale_keyword(Arg)
+#else
+  #ifdef SCM_CHARP
+    #define XEN_KEYWORD_P(Obj)        (SCM_KEYWORDP(Obj))
+  #else
+    #define XEN_KEYWORD_P(Obj)        XEN_TRUE_P(scm_keyword_p(Obj))
+  #endif
+  #define XEN_MAKE_KEYWORD(Arg)       scm_c_make_keyword(Arg)
 #endif
 #define XEN_KEYWORD_EQ_P(k1, k2)      XEN_EQ_P(k1, k2)
-#define XEN_MAKE_KEYWORD(Arg)         scm_c_make_keyword(Arg)
 #define XEN_YES_WE_HAVE(Feature)      scm_add_feature(Feature)
 #define XEN_PROTECT_FROM_GC(Obj)      scm_permanent_object(Obj)
 #define XEN_LOAD_FILE(File)           scm_primitive_load(C_TO_XEN_STRING(File))
 
 #define XEN_DEFINE_HOOK(Var, Name, Arity, Help) Var = xen_guile_create_hook(Name, Arity, Help, XEN_DOCUMENTATION_SYMBOL)
-#define XEN_DEFINE_SIMPLE_HOOK(Var, Arity) Var = scm_make_hook(C_TO_SMALL_XEN_INT(Arity))
+#define XEN_DEFINE_SIMPLE_HOOK(Var, Arity) Var = scm_make_hook(C_TO_XEN_INT(Arity))
 #define XEN_CLEAR_HOOK(Arg)           scm_reset_hook_x(Arg)
 #define XEN_HOOKED(a)                 (XEN_NOT_NULL_P(SCM_HOOK_PROCEDURES(a)))
 #define XEN_HOOK_PROCEDURES(a)        SCM_HOOK_PROCEDURES(a)
 
 /* disabling type checks saves almost no space (200k out of 12M) and no time (5% or so) */
+#if HAVE_SCM_FROM_LOCALE_KEYWORD
+  #define XEN_ASSERT_TYPE(Assertion, Arg, Position, Caller, Correct_Type) \
+    SCM_ASSERT_TYPE(Assertion, Arg, Position, Caller, Correct_Type)
+
+  #define XEN_WRONG_TYPE_ARG_ERROR(Caller, ArgN, Arg, Descr) \
+    scm_wrong_type_arg_msg(Caller, ArgN, Arg, Descr)
+#else
 #ifdef SCM_ASSERT_TYPE
   #define XEN_ASSERT_TYPE(Assertion, Arg, Position, Caller, Correct_Type) \
     do {SCM_ASSERT_TYPE(Assertion, Arg, Position, Caller, Correct_Type);} while (0) /* actual macro is unprotected if..then */
@@ -527,6 +618,7 @@
   #define XEN_WRONG_TYPE_ARG_ERROR(Caller, ArgN, Arg, Descr) \
     scm_wrong_type_arg(Caller, ArgN, Arg)
 #endif
+#endif
 
 #define XEN_OUT_OF_RANGE_ERROR(Caller, ArgN, Arg, Descr) \
   XEN_ERROR(XEN_ERROR_TYPE("out-of-range"), \
@@ -576,11 +668,11 @@
   #define XEN_CALL_6(Func, Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Caller) \
      scm_apply(Func, Arg1, scm_cons2(Arg2, Arg3, scm_cons2(Arg4, Arg5, scm_cons(Arg6, XEN_APPLY_ARG_LIST_END))))
 #endif
-#define XEN_APPLY_NO_CATCH(Func, Args, Caller)              scm_apply(Func, Args, XEN_EMPTY_LIST)
-#define XEN_CALL_0_NO_CATCH(Func, Caller)                   scm_apply(Func, XEN_EMPTY_LIST, XEN_EMPTY_LIST)
-#define XEN_CALL_1_NO_CATCH(Func, Arg1, Caller)             scm_apply(Func, Arg1, XEN_APPLY_ARG_LIST_END)
-#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2, Caller)       scm_apply(Func, Arg1, scm_cons(Arg2, XEN_APPLY_ARG_LIST_END))
-#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3, Caller) scm_apply(Func, Arg1, scm_cons2(Arg2, Arg3, XEN_APPLY_ARG_LIST_END))
+#define XEN_APPLY_NO_CATCH(Func, Args)              scm_apply(Func, Args, XEN_EMPTY_LIST)
+#define XEN_CALL_0_NO_CATCH(Func)                   scm_apply(Func, XEN_EMPTY_LIST, XEN_EMPTY_LIST)
+#define XEN_CALL_1_NO_CATCH(Func, Arg1)             scm_apply(Func, Arg1, XEN_APPLY_ARG_LIST_END)
+#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2)       scm_apply(Func, Arg1, scm_cons(Arg2, XEN_APPLY_ARG_LIST_END))
+#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3) scm_apply(Func, Arg1, scm_cons2(Arg2, Arg3, XEN_APPLY_ARG_LIST_END))
 
 #define XEN_PUTS(Str, Port)      scm_puts(Str, Port)
 #define XEN_DISPLAY(Val, Port)   scm_display(Val, Port)
@@ -598,9 +690,9 @@ void xen_guile_define_procedure_with_reversed_setter(char *get_name, XEN (*get_f
 						     char *set_name, XEN (*set_func)(), XEN (*reversed_set_func)(), 
 						     XEN local_doc,
 						     int get_req, int get_opt, int set_req, int set_opt);
+double xen_to_c_double(XEN a);
 double xen_to_c_double_or_else(XEN a, double b);
 int xen_to_c_int(XEN a);
-bool xen_to_c_boolean_or_true(XEN a);
 bool xen_integer_p(XEN a);
 #if XEN_DEBUGGING
 XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int opt, int rst);
@@ -612,6 +704,10 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
   #define XEN_CATCH_BODY_TYPE scm_catch_body_t
 #endif
 
+#if HAVE_SCM_C_MAKE_RECTANGULAR
+char *xen_guile_to_c_string_with_eventual_free(XEN str);
+#endif
+
 #endif
 /* end HAVE_GUILE */
 
@@ -633,17 +729,6 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #define XEN_FILE_EXTENSION  "rb"
 #define XEN_COMMENT_STRING  "#"
 
-#define XEN_ONLY_ARG 1
-#define XEN_ARG_1 1
-#define XEN_ARG_2 2
-#define XEN_ARG_3 3
-#define XEN_ARG_4 4
-#define XEN_ARG_5 5
-#define XEN_ARG_6 6
-#define XEN_ARG_7 7
-#define XEN_ARG_8 8
-#define XEN_ARG_9 9
-
 #ifdef __cplusplus
   #ifdef ANYARGS
     #define XEN_PROCEDURE_CAST (XEN (*)(ANYARGS))
@@ -696,21 +781,16 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #else
   #define XEN_TO_C_DOUBLE_OR_ELSE(a, b) xen_rb_to_c_double_or_else(a, b)
 #endif
-#define XEN_TO_C_DOUBLE_WITH_CALLER(a, b) NUM2DBL(a)
 #define C_TO_XEN_DOUBLE(a)                rb_float_new(a)
 
 #define XEN_TO_C_INT(a)                   rb_num2long(a)
 #if defined(__GNUC__) && (!(defined(__cplusplus)))
   #define XEN_TO_C_INT_OR_ELSE(a, b) ({ XEN _xen_h_5_ = a; (XEN_INTEGER_P(_xen_h_5_) ? FIX2INT(_xen_h_5_) : b); })
-  #define XEN_TO_C_INT_OR_ELSE_WITH_CALLER(a, b, c) ({ XEN _xen_h_6_ = a; (XEN_INTEGER_P(_xen_h_6_) ? FIX2INT(_xen_h_6_) : b); })
 #else
   #define XEN_TO_C_INT_OR_ELSE(a, b) xen_rb_to_c_int_or_else(a, b)
-  #define XEN_TO_C_INT_OR_ELSE_WITH_CALLER(a, b, c) xen_rb_to_c_int_or_else(a, b)
 #endif
 
 #define C_TO_XEN_INT(a)                   INT2NUM(a)
-#define C_TO_SMALL_XEN_INT(a)             INT2FIX(a)
-#define XEN_TO_SMALL_C_INT(a)             FIX2INT(a)
 #define XEN_TO_C_ULONG(a)                 NUM2ULONG(a)
 #ifdef ULONG2NUM
   #define C_TO_XEN_ULONG(a)               ULONG2NUM((unsigned long)a)
@@ -730,7 +810,6 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #define XEN_TO_C_STRING(Str)              RSTRING(Str)->ptr
 
 #define C_TO_XEN_BOOLEAN(a)               ((a) ? Qtrue : Qfalse)
-#define XEN_TO_C_BOOLEAN_OR_TRUE(a)       (!(XEN_FALSE_P(a)))
 #define XEN_TO_C_BOOLEAN(a)               (!(XEN_FALSE_P(a)))
 
 #define XEN_NAME_AS_C_STRING_TO_VALUE(a)  rb_gv_get(xen_scheme_global_variable_to_ruby(a))
@@ -746,6 +825,8 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #define XEN_UNWRAP_C_POINTER(a)           DATA_PTR(a)
 #define XEN_WRAPPED_C_POINTER_P(a)        (TYPE(a) == T_DATA)
 
+#define XEN_PROCEDURE_SOURCE(Func)        Func
+
 #if XEN_DEBUGGING
 /* the otiose casts to int here are required by g++ */
 #define XEN_DEFINE_PROCEDURE(Name, Func, ReqArg, OptArg, RstArg, Doc) \
@@ -857,12 +938,17 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #define XEN_CADDDR(a)                   rb_ary_entry(a, 3)
 #define XEN_CDR(a)                      xen_rb_cdr(a)
 #define XEN_CDDR(a)                     XEN_CDR(XEN_CDR(a))
-#define XEN_COPY_ARG(Lst)               xen_rb_copy_list(Lst)
+#if HAVE_RB_ARY_DUP
+  #define XEN_COPY_ARG(Lst)             rb_ary_dup(Lst)
+#else
+  #define XEN_COPY_ARG(Lst)             xen_rb_copy_list(Lst)
+#endif
 #define XEN_CONS(Arg1, Arg2)            xen_rb_cons(Arg1, Arg2)
 #define XEN_CONS_2(Arg1, Arg2, Arg3)    xen_rb_cons2(Arg1, Arg2, Arg3)
 #define XEN_LIST_REF(Lst, Num)          rb_ary_entry(Lst, Num)
 #define XEN_LIST_SET(Lst, Num, Val)     rb_ary_store(Lst, Num, Val)
 #define XEN_APPEND(X, Y)                rb_ary_concat(X, Y)
+#define XEN_LIST_REVERSE(Lst)           rb_ary_reverse(XEN_COPY_ARG(Lst))
 
 #define XEN_HOOK_PROCEDURES(a)          ((xen_rb_is_hook_p(a)) ? xen_rb_hook_to_a(a) : ((XEN_NULL_P(a)) ? Qnil : XEN_LIST_1(a)))
 #define XEN_CLEAR_HOOK(a)               ((xen_rb_is_hook_p(a)) ? xen_rb_hook_reset_hook(a) : (a = Qnil))
@@ -890,14 +976,15 @@ XEN xen_guile_dbg_new_procedure(const char *name, XEN (*func)(), int req, int op
 #define XEN_CALL_6(Func, Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Caller) rb_funcall(Func, rb_intern("call"), 6, Arg1, Arg2, Arg3, Arg4, Arg5, Arg6)
 #define XEN_APPLY(Func, Args, Caller)              xen_rb_apply(Func, Args)
 #define XEN_APPLY_ARG_LIST_END          Qnil
-#define XEN_CALL_0_NO_CATCH(Func, Caller)                   xen_rb_funcall_0(Func)
-#define XEN_CALL_1_NO_CATCH(Func, Arg1, Caller)             rb_funcall(Func, rb_intern("call"), 1, Arg1)
-#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2, Caller)       rb_funcall(Func, rb_intern("call"), 2, Arg1, Arg2)
-#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3, Caller) rb_funcall(Func, rb_intern("call"), 3, Arg1, Arg2, Arg3)
-#define XEN_APPLY_NO_CATCH(Func, Args, Caller)              xen_rb_apply(Func, Args)
+#define XEN_CALL_0_NO_CATCH(Func)                   xen_rb_funcall_0(Func)
+#define XEN_CALL_1_NO_CATCH(Func, Arg1)             rb_funcall(Func, rb_intern("call"), 1, Arg1)
+#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2)       rb_funcall(Func, rb_intern("call"), 2, Arg1, Arg2)
+#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3) rb_funcall(Func, rb_intern("call"), 3, Arg1, Arg2, Arg3)
+#define XEN_APPLY_NO_CATCH(Func, Args)              xen_rb_apply(Func, Args)
 
 #define XEN_ARITY(Func)                 rb_funcall(Func, rb_intern("arity"), 0)
 #define XEN_REQUIRED_ARGS(Func)         xen_rb_required_args(XEN_ARITY(Func))
+#define XEN_REQUIRED_ARGS_OK(Func, Args) (xen_rb_required_args(XEN_ARITY(Func)) == Args)
 #define XEN_KEYWORD_EQ_P(k1, k2)        ((k1) == (k2))
 #define XEN_MAKE_KEYWORD(Arg)           C_STRING_TO_XEN_SYMBOL(Arg)
 #define XEN_YES_WE_HAVE(a)              rb_provide(a)
@@ -1271,7 +1358,9 @@ XEN xen_rb_ary_new_with_initial_element(long num, XEN element);
 XEN xen_rb_apply(XEN func, XEN args);
 XEN xen_rb_funcall_0(XEN func);
 int xen_rb_required_args(XEN val);
-XEN xen_rb_copy_list(XEN val); /* Ruby arrays (lists) are passed by reference */
+#if (!HAVE_RB_ARY_DUP)
+XEN xen_rb_copy_list(XEN val);
+#endif
 XEN xen_rb_str_new2(char *arg);
 void xen_add_help(char *name, const char *help);
 char *xen_help(char *name);
@@ -1330,6 +1419,7 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 #define XEN_HOOK_PROCEDURES(a) 0
 #define XEN_VARIABLE_SET(a, b)
 #define XEN_VARIABLE_REF(a) 0
+#define XEN_PROCEDURE_SOURCE(Func) 0
 
 #define XEN_MAKE_AND_RETURN_OBJECT(Tag, Val, ig1, ig2) return(0)
 #define XEN_OBJECT_REF(a) 0
@@ -1346,21 +1436,16 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 
 #define XEN_TO_C_DOUBLE(a) 0.0
 #define XEN_TO_C_DOUBLE_OR_ELSE(a, b) b
-#define XEN_TO_C_DOUBLE_WITH_CALLER(a, b) 0.0
 #define XEN_TO_C_INT(a) 0
 #define XEN_TO_C_INT_OR_ELSE(a, b) b
-#define XEN_TO_C_INT_OR_ELSE_WITH_CALLER(a, b, c) b
 #define XEN_TO_C_STRING(STR) NULL
 #define C_TO_XEN_DOUBLE(a) 0
 #define C_TO_XEN_INT(a) a
-#define C_TO_SMALL_XEN_INT(a) a
-#define XEN_TO_SMALL_C_INT(a) a
 #define C_TO_XEN_LONG_LONG(a) a
 #define XEN_TO_C_LONG_LONG(a) a
 #define C_TO_XEN_STRING(a) 0
 #define C_TO_XEN_BOOLEAN(a) 0
 #define C_STRING_TO_XEN_SYMBOL(a) 0
-#define XEN_TO_C_BOOLEAN_OR_TRUE(a) 0
 #define XEN_TO_C_BOOLEAN(a) 0
 #define C_STRING_TO_XEN_FORM(Str) 0
 #define XEN_EVAL_FORM(Form) 0
@@ -1397,6 +1482,7 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 #define XEN_CONS_2(Arg1, Arg2, Arg3) 0
 #define XEN_LIST_REF(Lst, Num) 0
 #define XEN_LIST_SET(Lst, Num, Val)
+#define XEN_LIST_REVERSE(Lst) 0
 #define XEN_VECTOR_REF(Vect, Num) 0
 #define XEN_VECTOR_SET(a, b, c)
 #define XEN_EVAL_C_STRING(Arg) 0
@@ -1417,13 +1503,14 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 #define XEN_CALL_6(Func, Arg1, Arg2, Arg3, Arg4, Arg5, Arg6, Caller) 0
 #define XEN_APPLY(Func, Args, Caller) 0
 #define XEN_APPLY_ARG_LIST_END 0
-#define XEN_CALL_0_NO_CATCH(Func, Caller) 0
-#define XEN_CALL_1_NO_CATCH(Func, Arg1, Caller) 0
-#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2, Caller) 0
-#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3, Caller) 0
-#define XEN_APPLY_NO_CATCH(Func, Args, Caller) 0
+#define XEN_CALL_0_NO_CATCH(Func) 0
+#define XEN_CALL_1_NO_CATCH(Func, Arg1) 0
+#define XEN_CALL_2_NO_CATCH(Func, Arg1, Arg2) 0
+#define XEN_CALL_3_NO_CATCH(Func, Arg1, Arg2, Arg3) 0
+#define XEN_APPLY_NO_CATCH(Func, Args) 0
 #define XEN_ARITY(Func) 0
 #define XEN_REQUIRED_ARGS(Func) 0
+#define XEN_REQUIRED_ARGS_OK(Func, Args) false
 #define XEN_KEYWORD_P(Obj) 0
 #define XEN_KEYWORD_EQ_P(k1, k2) 0
 #define XEN_MAKE_KEYWORD(Arg) 0
@@ -1439,16 +1526,6 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 #define XEN_WRONG_TYPE_ARG_ERROR(Caller, ArgN, Arg, Descr)
 #define XEN_OUT_OF_RANGE_ERROR(Caller, ArgN, Arg, Descr)
 #define XEN_APPEND(X, Y) 0
-#define XEN_ONLY_ARG 0
-#define XEN_ARG_1    0
-#define XEN_ARG_2    0
-#define XEN_ARG_3    0
-#define XEN_ARG_4    0
-#define XEN_ARG_5    0
-#define XEN_ARG_6    0
-#define XEN_ARG_7    0
-#define XEN_ARG_8    0
-#define XEN_ARG_9    0
 typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 
 #endif
@@ -1480,7 +1557,18 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
   #define XEN_ULONG_IF_BOUND_P(Arg)              ((XEN_NOT_BOUND_P(Arg)) || (XEN_ULONG_P(Arg)))
 #endif
 
-#define XEN_TO_C_OFF_T_OR_ELSE(a, b)  xen_to_c_off_t_or_else(a, b, c__FUNCTION__)
+#define XEN_ONLY_ARG 0
+#define XEN_ARG_1    1
+#define XEN_ARG_2    2
+#define XEN_ARG_3    3
+#define XEN_ARG_4    4
+#define XEN_ARG_5    5
+#define XEN_ARG_6    6
+#define XEN_ARG_7    7
+#define XEN_ARG_8    8
+#define XEN_ARG_9    9
+
+#define XEN_TO_C_OFF_T_OR_ELSE(a, b)  xen_to_c_off_t_or_else(a, b)
 #define C_TO_XEN_OFF_T(a)             c_to_xen_off_t(a)
 #define XEN_TO_C_OFF_T(a)             xen_to_c_off_t(a)
 #define XEN_AS_STRING(form)           XEN_TO_C_STRING(XEN_TO_STRING(form))
@@ -1503,12 +1591,20 @@ typedef XEN (*XEN_CATCH_BODY_TYPE) (void *data);
 #endif
 
 XEN xen_return_first(XEN a, ...);
-int xen_to_c_int_or_else(XEN obj, int fallback, const char *origin);
-off_t xen_to_c_off_t_or_else(XEN obj, off_t fallback, const char *origin);
+int xen_to_c_int_or_else(XEN obj, int fallback);
+off_t xen_to_c_off_t_or_else(XEN obj, off_t fallback);
 off_t xen_to_c_off_t(XEN obj);
 XEN c_to_xen_off_t(off_t val);
 char *xen_version(void);
 void xen_repl(int argc, char **argv);
 void xen_initialize(void);
 void xen_gc_mark(XEN val);
+
+#ifndef XEN_DISABLE_DEPRECATED
+  #define C_TO_SMALL_XEN_INT(a) C_TO_XEN_INT(a)
+  #define XEN_TO_SMALL_C_INT(a) XEN_TO_C_INT(a)
+  #define XEN_TO_C_BOOLEAN_OR_TRUE(a) (!(XEN_FALSE_P(a)))
+  #define XEN_TO_C_INT_OR_ELSE_WITH_CALLER(a, b, c) XEN_TO_C_INT_OR_ELSE(a, b)
+  #define XEN_TO_C_DOUBLE_WITH_CALLER(a, b) XEN_TO_C_DOUBLE(a)
+#endif
 #endif
